#ifndef VECTOR3
#define VECTOR3

#include <math.h>

class vector3 {
public:
   double x;
   double y;
   double z;

   vector3(double in_x, double in_y, double in_z) : x(in_x), y(in_y), z(in_z){};
   vector3() : x(0.0), y(0.0), z(0.0) {};
   vector3(const vector3 &v) : x(v.x), y(v.y), z(v.z) {};
   vector3(vector3 *v) : x(v->x), y(v->y), z(v->z) {};

   inline double dot(vector3 v) { return x*v.x + y*v.y + z*v.z; }
   inline vector3 cross(vector3 v) { return vector3(y*v.z-z*v.y,z*v.x-x*v.z,x*v.y-y*v.x); }
   inline double length() { return sqrt(x*x + y*y + z*z); }
   inline double magnitude() { return sqrt(x*x + y*y + z*z); }
   inline vector3& normalize() { 
       double invlength = 1.0/length(); 
       x *= invlength; 
       y *= invlength; 
       z *= invlength; 
       return *this;
   }
   inline void invert() { x = -x; y = -y; z = -z; }


   // Overloaded operators for vector math
   vector3& operator+= (const vector3 &v)
      { x += v.x; y += v.y; z += v.z; return *this; }
   vector3& operator-= (const vector3 &v)
      { x -= v.x; y -= v.y; z -= v.z; return *this; }
   vector3& operator*= (double f)
      { x*=f; y*=f; z*=f; return *this; }
   vector3& operator/= (double f)
      { x/=f; y/=f; z/=f; return *this; }
   friend vector3 operator- (const vector3 &v) 
      { return vector3(-v.x, -v.y, -v.z); }
   friend vector3 operator+ (const vector3 &v1, const vector3 &v2) 
      { return vector3(v1.x + v2.x, v1.y + v2.y, v1.z + v2.z); }
   friend vector3 operator- (const vector3& v1, const vector3& v2) 
      { return vector3(v1.x - v2.x, v1.y - v2.y, v1.z - v2.z); }
   friend vector3 operator* (const vector3 &v, double scale) // Right side scale multiplication
      { return vector3(scale*v.x, scale*v.y, scale*v.z); }
   friend vector3 operator* (double scale, const vector3& v) // Left side scale multiplication
      { return vector3(scale*v.x,scale*v.y,scale*v.z); }
   friend vector3 operator/ (const vector3 &v, double scale) 
      { return vector3(v.x/scale, v.y/scale, v.z/scale); }
   friend vector3 operator* (vector3& v1, vector3& v2) // Vector multiplication
      { return vector3(v1.x*v2.x, v1.y*v2.y, v1.z*v2.z); }
};

#endif
