/* PACKAGES */
// The following are default packages from BuildScene.cpp
#ifdef _WIN32
#include <windows.h>
#endif
#include <stdio.h>
#include <math.h>
#include <vector>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include "camera.h"

#include <iostream>
#include <fstream>


using namespace std;

UserCamera camera;
enum ClickLocation { TOP_RIGHT, BOTTOM_RIGHT, BOTTOM_LEFT, TOP_LEFT, MIDDLE };
ClickLocation clickLocation;
int width = 500;
int height = 500;
int lastx = 0;
int lasty = 0;

/* FUNCTION PROTOTYPES */
void display(void);
void timer(int v);
void initializeCoord(void);
void keyCallback(unsigned char key, int x, int y);
void keyUpCallback(unsigned char key, int x, int y);
void normCrossProd(GLfloat[3], GLfloat[3], GLfloat[3]);
void mouseCallback(int button, int state, int x, int y);
void mouseMotionCallback(int x, int y);
void reshapeCallback(int w, int h);

void getShading(GLfloat[3], GLfloat[3]);
void getDistancePathShading(GLfloat[3], GLfloat[3]);
void getEnergyPathShading(GLfloat[3], GLfloat[3]);
void getDirectPathShading(GLfloat[3], GLfloat[3]);

void runDijkstra(void);
void processVertex(int x, int y);
void printData(void);

GLfloat calcDistance(GLfloat startX, GLfloat endX, GLfloat startY, GLfloat endY, GLfloat startZ, GLfloat endZ);
int checkGrade(GLfloat base, GLfloat height);

/* CONSTANT VALUES */
const int MATRIX_MAX = 365; // For atascadero or santa maria
const float ELEV_DIVISOR = 1000.0;
const float WATER_ALT = -0.1;       // Walter Altitude

const GLfloat myINFINITY = 1.0e35;

const int X = 0;
const int Y = 1;
const int Z = 2;

const int PREV_X = 0;
const int PREV_Y = 1;
const int VISITED = 2;

const int YES = 1;
const int NO = 0;
const int myUNDEFINED = (int)(1.0e35);
const double GRADE_PERCENT_MAX = 7.0;

const float GRAVITY = 9.80665;
const float PATH_WIDTH = 0.02;
const float PATH_HEIGHT_UP = -0.01; // Negative values goes toward the screen

const int ENERGY_MODE = 1;
const int DISTANCE_MODE = 2;
static int mode = DISTANCE_MODE;

/* STATIC VALUES */
static GLfloat modelTrans[] = {-0.5, 2.25, -2.5};
static GLfloat translate[3] = {-10.0, 0.0, 0.0};
static GLfloat theta[] = {180.0, 0.0, 20.0};
static double thetaIncr = 5.0;

static int start_x = 10;
static int start_y = 360;
static int end_x = 350;
static int end_y = 340;

static GLfloat coord[MATRIX_MAX][MATRIX_MAX][3];
static GLfloat dist[MATRIX_MAX][MATRIX_MAX];
static GLfloat energy[MATRIX_MAX][MATRIX_MAX];

static int dist_info[MATRIX_MAX][MATRIX_MAX][3];
static int energy_info[MATRIX_MAX][MATRIX_MAX][3];

static float distRings = 15.0;
static float distInc = 1000.0;

static float speed = 55.0;
static float carWeight = 1325.0; // in kilogram

static float energyRings = 10.0;
static float energyInc = 50000.0;













//SANTA MARIA starts here
static int zValues[]={
54,54,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,56,57,57,56,56,56,55,55,56,56,56,55,53,52,52,49,46,40,34,30,30,30,30,36,36,30,30,30,34,34,30,30,30,39,46,49,51,51,51,51,50,49,49,50,52,53,54,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,56,56,56,56,55,55,55,55,54,53,52,52,51,51,52,52,52,54,55,55,54,54,53,53,52,52,52,52,52,52,52,52,52,51,51,51,51,51,51,51,51,51,51,50,50,50,50,50,50,50,50,49,49,49,49,49,49,49,49,49,48,48,47,46,46,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,46,45,45,46,46,47,47,47,47,48,48,48,48,48,48,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,48,48,48,48,48,48,48,47,47,47,46,46,45,45,45,46,46,46,46,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,54,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,57,58,58,57,57,56,56,56,56,56,55,54,52,51,49,45,43,38,30,30,30,35,39,41,41,40,38,39,39,36,30,30,30,37,44,49,51,51,52,51,51,50,49,51,52,53,54,55,55,55,55,55,55,55,55,55,55,56,56,56,55,56,56,56,56,56,56,56,56,55,55,55,54,54,53,52,51,52,54,54,54,55,55,55,55,54,54,53,53,52,52,52,52,52,52,52,51,51,51,51,51,51,51,51,51,51,51,50,50,50,50,50,50,50,50,50,50,50,49,49,49,49,49,49,48,48,47,46,46,45,45,46,45,45,45,45,45,45,45,45,45,45,45,45,45,45,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,46,45,45,45,45,46,47,47,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,48,48,48,48,48,48,47,47,46,46,45,45,46,46,46,46,46,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,45,45,45,45,45,44,44,45,44,44,44,44,44,54,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,58,59,59,58,58,57,56,56,57,57,55,54,51,51,49,45,42,42,39,34,34,37,40,43,44,44,44,43,43,42,38,30,30,30,30,42,49,51,52,52,52,52,51,51,52,53,54,55,55,55,55,55,55,55,55,55,55,56,56,56,56,56,56,56,56,56,56,56,56,56,55,55,55,55,54,52,52,53,54,55,54,55,55,55,55,55,55,55,54,53,53,52,53,52,52,52,51,51,51,51,51,51,51,51,51,51,51,50,50,50,50,50,50,50,50,50,50,50,50,50,49,49,49,49,49,48,47,47,46,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,46,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,46,46,45,45,46,48,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,48,48,48,48,48,47,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,45,45,45,45,45,45,45,45,45,45,44,44,44,44,44,54,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,61,61,60,58,58,58,57,56,56,55,55,51,49,50,46,42,42,41,42,42,42,43,45,45,45,46,46,46,45,44,40,35,30,30,30,42,49,51,52,52,53,52,52,52,53,54,55,55,55,55,55,55,55,55,55,55,56,56,56,56,56,56,56,56,56,57,56,56,56,56,55,56,55,55,54,53,53,54,55,55,55,55,55,55,55,55,55,55,54,54,53,53,52,52,52,52,51,51,51,51,51,51,51,51,51,51,51,50,50,50,50,50,50,50,50,50,50,50,50,50,49,49,49,49,49,48,47,46,46,45,45,45,45,46,46,46,45,46,45,45,45,45,46,45,45,46,46,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,46,46,46,45,48,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,50,50,50,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,48,48,47,47,47,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,44,44,44,44,44,44,44,44,44,45,44,45,44,45,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,45,45,45,45,45,45,45,45,45,45,45,45,45,44,44,44,44,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,57,59,61,61,59,59,58,56,55,53,52,51,50,49,49,47,43,41,41,41,44,46,47,46,46,46,46,47,47,47,46,45,42,37,30,30,30,43,49,52,52,53,53,53,52,53,54,55,55,55,55,55,55,55,55,55,55,56,56,56,56,56,56,56,56,57,57,57,56,56,56,56,56,56,56,55,54,54,54,55,55,55,55,56,56,55,55,55,55,55,54,54,53,53,52,52,52,52,52,52,52,51,51,51,51,51,51,51,51,50,50,50,50,50,50,50,50,50,50,50,50,50,50,49,49,49,49,48,47,46,46,45,45,45,45,45,45,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,46,46,46,47,49,49,49,49,49,49,49,49,49,49,49,49,50,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,50,50,50,50,50,50,50,50,50,50,50,50,50,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,48,48,47,47,47,47,47,47,47,47,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,45,44,45,45,45,45,45,45,45,45,45,45,45,45,45,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,44,44,44,44,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,57,58,60,61,60,60,59,57,55,52,48,47,47,47,45,43,43,41,41,41,45,49,50,49,48,47,47,47,48,48,48,48,46,43,38,30,30,37,45,50,52,53,53,53,54,53,54,55,55,55,55,55,55,55,55,55,55,55,56,56,56,56,56,56,56,57,57,57,57,57,57,56,57,56,56,56,55,54,55,55,55,56,55,56,56,56,56,56,56,55,55,54,54,53,53,52,52,52,52,52,52,52,52,51,51,51,51,51,51,51,51,51,50,50,50,50,50,50,50,50,50,50,50,50,49,49,49,49,48,47,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,46,46,47,48,49,49,49,49,49,49,49,49,49,49,50,50,50,50,50,50,50,50,49,49,49,49,49,49,49,49,49,49,49,49,49,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,48,48,48,48,47,47,47,47,47,47,47,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,44,44,44,44,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,57,59,60,61,60,59,61,56,54,49,44,44,43,43,42,42,41,41,41,44,49,52,51,50,48,47,47,48,49,49,49,48,46,43,38,30,30,39,46,50,52,53,53,54,54,54,55,55,55,55,55,55,56,56,56,55,55,55,56,56,56,56,56,57,57,57,57,57,57,57,57,57,57,57,56,55,55,55,55,56,56,56,57,57,57,57,57,57,56,55,55,54,54,54,53,52,52,52,53,52,52,52,52,51,51,51,51,51,51,51,51,51,51,50,50,50,50,50,50,50,50,50,50,50,49,49,49,49,48,48,47,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,46,46,47,49,49,49,49,49,50,49,49,49,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,48,48,48,48,47,47,47,47,47,47,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,46,45,46,45,45,45,45,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,57,60,61,61,61,55,53,51,47,43,42,42,42,41,41,41,42,44,48,53,55,52,50,48,48,48,48,49,50,50,49,47,44,38,30,30,40,47,50,52,53,53,54,54,54,55,55,55,55,55,56,56,56,55,55,55,56,56,56,56,57,57,57,57,57,57,57,57,57,57,57,57,56,56,55,55,55,56,56,57,57,57,57,57,57,57,57,56,55,55,55,54,54,53,53,53,53,53,52,52,52,52,51,51,51,51,51,51,51,51,51,51,50,51,50,50,50,50,50,50,50,50,49,49,49,49,48,48,48,47,47,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,46,47,48,49,49,49,49,49,49,49,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,48,48,48,48,47,47,47,47,47,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,44,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,46,46,46,45,45,45,45,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,56,58,61,61,59,53,52,50,46,42,41,41,41,41,43,45,46,49,53,55,55,52,49,48,48,48,49,50,51,51,50,48,44,38,30,30,39,46,50,52,53,53,54,54,55,55,55,55,55,56,56,56,56,56,56,56,56,56,56,57,57,57,57,57,57,58,58,58,57,57,57,57,57,57,56,56,56,56,57,57,57,57,58,58,58,58,58,57,56,55,55,55,54,54,54,55,54,52,52,52,52,52,51,51,51,51,51,51,51,51,51,51,51,51,50,50,50,50,50,50,50,50,49,49,49,49,48,48,48,47,47,47,47,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,46,47,48,49,49,49,49,49,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,48,48,48,47,47,47,47,46,46,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,44,44,44,44,44,44,44,44,44,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,46,46,46,46,45,45,45,45,54,54,54,54,54,55,55,55,55,55,55,55,55,55,55,55,55,56,59,56,54,50,50,49,45,42,41,41,41,44,46,49,51,53,55,55,54,52,50,49,49,49,51,52,53,53,51,48,45,39,30,30,37,44,49,51,53,53,54,54,55,55,55,55,55,56,56,56,56,56,56,56,56,56,56,57,57,57,57,57,58,58,58,58,58,57,57,57,57,57,57,57,57,57,57,57,58,58,58,58,58,58,58,58,56,55,55,54,55,55,55,55,54,52,52,52,52,51,51,51,51,51,51,51,51,51,51,51,51,51,50,50,50,50,50,50,50,50,49,49,49,49,48,48,48,48,47,47,47,47,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,47,48,49,49,49,49,49,49,49,49,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,48,48,48,48,47,47,47,46,46,46,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,46,46,46,46,45,45,45,45,54,53,54,54,54,54,54,55,55,55,55,55,55,55,55,55,55,56,57,54,52,49,48,47,43,42,41,41,43,46,49,52,54,55,55,55,54,52,51,50,49,50,51,53,55,55,52,48,44,38,30,30,30,43,48,50,52,54,55,55,55,55,55,55,56,56,56,56,56,56,56,56,57,56,57,57,57,57,57,58,58,58,58,58,58,58,58,58,58,58,58,57,57,57,58,58,58,58,58,58,58,58,58,58,56,55,55,55,55,55,55,54,53,52,52,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,51,50,50,50,50,50,50,49,49,49,49,49,48,48,48,47,47,47,47,47,47,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,47,48,49,49,49,49,49,49,49,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,48,48,48,47,47,47,47,47,46,46,46,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,46,46,46,46,46,45,45,45,45,53,53,53,53,53,53,53,54,54,54,54,54,54,54,55,55,55,56,55,54,52,50,49,45,42,41,42,41,44,48,51,54,55,55,55,54,54,53,52,51,51,50,51,54,55,54,51,47,42,36,30,30,38,44,48,50,53,54,55,55,55,55,55,56,56,56,56,56,56,56,56,57,57,57,57,57,57,57,57,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,57,56,55,55,56,56,55,54,53,52,52,52,51,51,52,51,51,51,51,51,51,51,51,51,51,51,51,51,51,50,50,50,50,50,49,49,49,49,49,48,48,48,47,47,47,47,47,47,47,47,47,47,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,47,48,49,49,49,49,49,49,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,50,49,49,49,49,49,49,49,49,49,49,49,49,49,49,49,48,48,48,48,47,47,47,47,47,46,46,46,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,46,46,46,46,46,46,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,45,46,46,46,46,46,46,46,45,45,45,45,53,53,53,53,53,53,54,54,54,54,54,54,55,55,55,55,56,55,54,53,51,49,48,44,42,41,41,41,45,48,52,55,55,55,55,55,54,53,53,52,52,52,52,53,55,53,50,46,40,30,30,34,40,45,48,50,53,55,55,55,55,55,56,56,56,56,56,56,56,56,56,57,57,57,57,57,57,57,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,58,57,55,56,57,56,55,54,53,52,52,52,52,52,52,51,51,51,51,52,52,51,51,51,51,51,51,51,51,50,50,50,50,50,49,49,49,49,49,48,48,48,48,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,46,46,46,46,46,46,46,46,46,45,45,45,45,45,45,45,45,47,48,49,49,49,49,49,49,50,50,50,50,50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
49,
49,
49,
48,
47,
47,
47,
47,
47,
47,
47,
47,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
46,
46,
46,
46,
46,
46,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
45,
45,
45,
45,
53,
53,
54,
54,
55,
55,
55,
54,
54,
54,
55,
55,
55,
55,
55,
56,
55,
55,
55,
52,
51,
50,
46,
43,
41,
41,
41,
42,
46,
50,
54,
56,
56,
56,
55,
55,
54,
54,
53,
53,
53,
52,
52,
52,
51,
52,
50,
46,
41,
30,
30,
30,
41,
45,
48,
50,
53,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
55,
55,
53,
52,
52,
52,
52,
52,
52,
52,
51,
51,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
49,
49,
49,
49,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
46,
46,
46,
46,
46,
46,
45,
45,
45,
45,
45,
45,
45,
47,
48,
49,
49,
49,
49,
49,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
46,
46,
46,
46,
46,
46,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
46,
46,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
45,
45,
45,
45,
53,
53,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
55,
55,
54,
50,
49,
47,
44,
41,
41,
41,
41,
42,
48,
52,
55,
56,
56,
56,
55,
55,
54,
54,
54,
53,
53,
53,
52,
52,
51,
51,
50,
47,
43,
39,
38,
39,
42,
45,
48,
50,
52,
54,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
56,
54,
52,
52,
52,
52,
52,
52,
52,
51,
51,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
45,
45,
45,
45,
45,
45,
47,
48,
49,
49,
49,
49,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
46,
46,
46,
46,
46,
46,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
46,
46,
46,
46,
46,
45,
45,
45,
45,
45,
45,
45,
45,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
45,
45,
45,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
56,
55,
56,
55,
55,
53,
50,
48,
47,
45,
43,
41,
41,
41,
41,
43,
49,
53,
55,
56,
56,
56,
55,
55,
55,
54,
54,
54,
54,
53,
53,
52,
51,
50,
50,
48,
46,
44,
42,
42,
43,
44,
46,
48,
52,
54,
55,
55,
56,
56,
56,
56,
56,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
55,
54,
52,
52,
54,
53,
52,
52,
52,
51,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
48,
49,
49,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
45,
45,
45,
45,
45,
45,
45,
45,
45,
46,
46,
46,
46,
46,
46,
46,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
46,
46,
46,
46,
46,
46,
45,
45,
45,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
45,
45,
45,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
56,
55,
55,
55,
55,
54,
52,
48,
48,
47,
45,
43,
41,
41,
41,
42,
45,
50,
54,
56,
56,
56,
56,
56,
55,
55,
55,
55,
54,
54,
54,
53,
53,
52,
51,
50,
48,
48,
46,
45,
43,
43,
43,
45,
48,
53,
55,
55,
55,
56,
56,
56,
56,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
57,
55,
54,
54,
54,
55,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
49,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
45,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
45,
45,
45,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
55,
55,
53,
52,
49,
48,
45,
43,
41,
41,
41,
42,
45,
50,
54,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
54,
54,
53,
52,
51,
50,
50,
50,
49,
46,
43,
43,
43,
45,
48,
52,
55,
55,
55,
56,
56,
56,
56,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
58,
58,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
57,
55,
54,
55,
55,
55,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
50,
50,
50,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
51,
51,
50,
50,
50,
51,
51,
50,
50,
50,
51,
51,
51,
50,
50,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
45,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
45,
45,
45,
54,
54,
55,
55,
55,
55,
55,
55,
55,
56,
55,
55,
55,
56,
56,
55,
55,
55,
53,
51,
49,
45,
43,
42,
41,
42,
42,
45,
50,
55,
57,
57,
56,
57,
56,
56,
55,
55,
55,
55,
55,
55,
54,
54,
53,
52,
51,
51,
50,
49,
47,
43,
43,
43,
43,
47,
52,
55,
55,
55,
56,
56,
56,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
57,
55,
55,
55,
55,
54,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
50,
50,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
50,
50,
50,
50,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
45,
45,
45,
45,
45,
45,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
45,
45,
54,
55,
55,
55,
55,
55,
55,
55,
56,
56,
55,
55,
56,
56,
55,
55,
55,
55,
54,
52,
49,
47,
45,
42,
41,
42,
42,
44,
48,
55,
57,
56,
57,
56,
57,
56,
56,
55,
55,
55,
55,
55,
55,
54,
53,
53,
52,
51,
50,
49,
46,
43,
43,
43,
44,
48,
53,
55,
55,
56,
56,
56,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
56,
56,
55,
55,
54,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
50,
50,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
50,
50,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
47,
46,
46,
45,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
55,
55,
55,
54,
55,
55,
54,
52,
50,
50,
46,
43,
41,
41,
43,
45,
51,
55,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
55,
55,
55,
54,
54,
54,
53,
52,
50,
48,
45,
43,
43,
43,
46,
51,
54,
55,
55,
56,
56,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
59,
59,
58,
58,
57,
56,
55,
55,
54,
54,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
50,
50,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
48,
48,
48,
48,
48,
48,
49,
49,
49,
49,
49,
50,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
51,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
47,
47,
47,
46,
46,
45,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
55,
54,
54,
53,
55,
55,
54,
53,
51,
50,
47,
43,
41,
42,
43,
54,
53,
55,
56,
56,
57,
58,
57,
56,
56,
56,
56,
56,
56,
55,
55,
54,
55,
55,
54,
53,
50,
48,
44,
43,
43,
43,
47,
52,
55,
55,
56,
56,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
59,
59,
59,
59,
58,
58,
57,
56,
55,
55,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
50,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
48,
48,
48,
47,
47,
47,
48,
48,
48,
48,
48,
48,
48,
49,
49,
49,
49,
49,
50,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
47,
46,
46,
47,
47,
47,
46,
46,
45,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
55,
54,
53,
53,
54,
55,
55,
52,
51,
49,
46,
43,
42,
42,
50,
53,
56,
56,
56,
56,
57,
57,
57,
57,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
53,
50,
47,
43,
43,
43,
43,
48,
52,
55,
55,
56,
56,
56,
56,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
57,
56,
55,
55,
55,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
50,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
49,
49,
49,
50,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
47,
47,
47,
47,
47,
46,
46,
47,
47,
47,
47,
46,
45,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
55,
54,
53,
52,
51,
51,
52,
52,
50,
48,
45,
43,
43,
42,
49,
55,
56,
55,
56,
56,
57,
56,
57,
57,
57,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
53,
50,
46,
43,
43,
43,
43,
49,
52,
55,
55,
56,
56,
56,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
57,
56,
55,
55,
55,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
50,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
50,
50,
50,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
47,
47,
47,
47,
46,
46,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
55,
53,
51,
50,
49,
47,
47,
45,
44,
43,
42,
42,
42,
46,
55,
55,
56,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
55,
55,
56,
56,
55,
55,
53,
50,
45,
43,
43,
43,
43,
49,
52,
54,
55,
56,
56,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
59,
59,
59,
58,
57,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
51,
51,
50,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
49,
50,
50,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
47,
47,
47,
47,
46,
46,
55,
55,
56,
56,
55,
56,
56,
56,
56,
56,
56,
52,
50,
50,
48,
45,
43,
42,
42,
42,
42,
42,
42,
45,
53,
54,
55,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
55,
55,
52,
49,
45,
43,
43,
43,
44,
49,
52,
54,
55,
56,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
59,
59,
59,
59,
60,
60,
59,
59,
59,
59,
60,
60,
59,
59,
58,
58,
56,
55,
56,
57,
57,
55,
55,
55,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
51,
51,
50,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
49,
49,
50,
50,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
46,
55,
56,
55,
55,
55,
55,
56,
56,
56,
55,
53,
48,
47,
46,
44,
42,
42,
42,
42,
42,
42,
44,
46,
52,
54,
55,
56,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
55,
55,
52,
49,
45,
43,
43,
43,
43,
48,
52,
55,
56,
56,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
59,
59,
59,
60,
60,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
57,
57,
58,
59,
58,
55,
55,
55,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
49,
50,
50,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
46,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
46,
56,
55,
55,
55,
55,
55,
55,
55,
56,
55,
50,
47,
44,
42,
42,
42,
42,
42,
42,
43,
47,
51,
53,
55,
55,
56,
56,
56,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
55,
55,
52,
49,
44,
43,
43,
43,
44,
49,
52,
55,
56,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
59,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
57,
58,
58,
58,
58,
58,
55,
55,
55,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
49,
49,
50,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
46,
55,
55,
55,
55,
55,
55,
55,
55,
54,
52,
48,
45,
42,
43,
45,
43,
42,
42,
42,
45,
52,
53,
55,
55,
56,
56,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
55,
55,
55,
51,
48,
43,
43,
43,
43,
45,
48,
52,
55,
55,
56,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
59,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
60,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
57,
57,
56,
55,
55,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
50,
50,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
48,
48,
48,
48,
48,
48,
49,
49,
49,
50,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
48,
48,
48,
47,
55,
55,
55,
55,
55,
54,
54,
54,
53,
50,
49,
46,
45,
46,
49,
45,
42,
42,
48,
48,
52,
54,
55,
55,
55,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
55,
55,
54,
53,
51,
49,
43,
43,
43,
43,
43,
49,
50,
54,
55,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
60,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
56,
55,
55,
55,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
49,
49,
49,
50,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
48,
48,
48,
48,
48,
47,
55,
55,
55,
55,
55,
55,
53,
50,
49,
49,
49,
49,
49,
49,
49,
47,
44,
42,
42,
42,
48,
55,
55,
55,
55,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
55,
55,
55,
53,
50,
45,
43,
44,
43,
44,
44,
45,
50,
55,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
60,
60,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
56,
55,
55,
55,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
49,
49,
50,
50,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
48,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
48,
48,
48,
48,
48,
47,
55,
55,
55,
55,
55,
55,
51,
49,
49,
49,
49,
49,
52,
52,
51,
50,
47,
43,
42,
42,
45,
51,
54,
55,
56,
56,
57,
57,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
55,
52,
49,
43,
43,
45,
49,
43,
44,
45,
51,
55,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
60,
60,
60,
59,
59,
59,
59,
58,
58,
58,
59,
59,
58,
58,
57,
56,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
49,
49,
50,
50,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
55,
55,
55,
55,
55,
51,
49,
49,
49,
49,
49,
51,
55,
55,
53,
51,
48,
45,
42,
42,
42,
46,
52,
55,
56,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
56,
56,
54,
51,
47,
43,
43,
50,
53,
45,
44,
45,
50,
55,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
58,
58,
59,
59,
58,
58,
57,
56,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
49,
49,
49,
50,
50,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
55,
55,
54,
54,
52,
49,
49,
49,
49,
49,
51,
55,
56,
55,
54,
52,
49,
46,
43,
42,
43,
45,
49,
53,
55,
56,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
56,
56,
55,
53,
49,
45,
43,
46,
54,
55,
46,
44,
45,
50,
54,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
58,
57,
56,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
50,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
55,
55,
53,
50,
49,
49,
49,
49,
51,
51,
54,
56,
56,
56,
54,
52,
49,
47,
44,
43,
44,
45,
47,
51,
55,
56,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
56,
56,
55,
51,
47,
44,
43,
48,
55,
55,
49,
44,
45,
50,
55,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
58,
58,
57,
56,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
50,
50,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
47,
48,
47,
47,
47,
47,
47,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
47,
55,
55,
51,
49,
49,
49,
49,
51,
52,
53,
55,
57,
56,
57,
55,
53,
50,
47,
45,
44,
44,
46,
48,
52,
55,
56,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
56,
56,
54,
49,
46,
43,
43,
52,
57,
55,
50,
44,
45,
50,
56,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
58,
58,
56,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
50,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
55,
54,
50,
49,
49,
49,
51,
52,
53,
54,
56,
57,
56,
57,
56,
53,
50,
48,
46,
45,
46,
46,
48,
52,
55,
56,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
56,
55,
51,
47,
45,
43,
47,
54,
55,
55,
50,
44,
45,
51,
57,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
60,
59,
59,
59,
58,
57,
56,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
54,
53,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
50,
50,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
55,
52,
49,
49,
49,
51,
52,
54,
55,
55,
56,
57,
57,
57,
55,
54,
50,
48,
46,
46,
47,
47,
49,
53,
55,
56,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
56,
55,
52,
48,
47,
46,
45,
51,
55,
56,
55,
50,
44,
47,
54,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
58,
58,
57,
56,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
54,
53,
53,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
50,
50,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
55,
51,
49,
49,
50,
52,
54,
55,
56,
56,
57,
58,
58,
57,
55,
51,
49,
47,
47,
47,
47,
48,
49,
54,
55,
56,
57,
58,
58,
58,
58,
58,
59,
58,
58,
58,
58,
58,
58,
58,
56,
55,
52,
49,
47,
46,
47,
49,
54,
56,
56,
55,
50,
48,
49,
56,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
60,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
58,
58,
57,
56,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
54,
53,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
50,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
49,
49,
49,
52,
49,
49,
49,
51,
53,
55,
56,
57,
58,
57,
57,
57,
55,
52,
48,
47,
47,
47,
47,
48,
50,
52,
55,
56,
56,
57,
58,
58,
58,
58,
58,
58,
59,
58,
58,
58,
58,
58,
57,
56,
54,
50,
49,
47,
46,
48,
50,
56,
56,
56,
54,
50,
48,
50,
57,
58,
58,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
60,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
60,
60,
59,
59,
58,
57,
56,
55,
55,
55,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
54,
54,
53,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
50,
50,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
49,
49,
49,
49,
49,
50,
49,
49,
49,
52,
54,
56,
57,
57,
57,
57,
57,
56,
53,
50,
47,
47,
47,
47,
47,
49,
51,
54,
55,
56,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
56,
55,
50,
48,
47,
47,
49,
53,
56,
57,
56,
54,
51,
48,
49,
57,
58,
58,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
60,
58,
58,
57,
56,
55,
55,
55,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
54,
54,
53,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
51,
53,
54,
56,
57,
57,
57,
57,
57,
55,
52,
49,
47,
47,
47,
47,
48,
50,
53,
55,
56,
56,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
53,
49,
47,
47,
48,
51,
55,
56,
58,
56,
54,
51,
49,
49,
56,
58,
58,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
60,
59,
58,
57,
56,
55,
55,
55,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
54,
53,
53,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
48,
49,
49,
49,
49,
49,
49,
49,
48,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
51,
53,
54,
56,
57,
58,
58,
58,
57,
56,
55,
51,
49,
47,
47,
47,
47,
49,
51,
53,
55,
56,
56,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
58,
55,
51,
48,
47,
48,
52,
56,
57,
58,
56,
54,
52,
49,
49,
54,
58,
58,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
60,
59,
59,
58,
57,
56,
55,
55,
55,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
53,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
50,
53,
54,
55,
57,
58,
57,
58,
57,
57,
56,
53,
51,
49,
47,
47,
47,
48,
50,
52,
54,
55,
56,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
54,
50,
48,
48,
53,
56,
58,
59,
57,
54,
52,
49,
49,
53,
57,
58,
59,
60,
60,
60,
60,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
60,
59,
58,
58,
56,
55,
55,
55,
55,
54,
54,
55,
55,
55,
55,
55,
55,
55,
54,
54,
53,
53,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
51,
51,
51,
51,
50,
51,
51,
50,
51,
50,
50,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
55,
55,
55,
56,
57,
57,
57,
57,
58,
56,
55,
52,
49,
48,
47,
47,
47,
49,
51,
53,
55,
56,
56,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
58,
51,
49,
49,
50,
54,
57,
58,
59,
57,
54,
52,
50,
49,
53,
57,
59,
60,
60,
60,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
60,
60,
59,
58,
57,
56,
55,
55,
55,
55,
54,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
53,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
51,
51,
51,
51,
51,
51,
51,
51,
50,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
48,
48,
48,
48,
48,
48,
48,
48,
48,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
55,
57,
57,
57,
58,
57,
57,
57,
57,
56,
54,
51,
48,
47,
47,
47,
47,
49,
52,
54,
55,
56,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
60,
58,
56,
49,
49,
49,
51,
55,
58,
59,
59,
58,
55,
53,
50,
49,
55,
58,
59,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
60,
60,
59,
59,
58,
57,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
53,
53,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
53,
56,
57,
57,
57,
58,
58,
58,
57,
56,
55,
52,
49,
47,
47,
47,
47,
48,
50,
53,
55,
56,
56,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
60,
59,
54,
49,
49,
49,
51,
54,
58,
59,
60,
58,
56,
53,
51,
49,
55,
58,
59,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
61,
61,
61,
60,
60,
60,
59,
59,
58,
57,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
55,
55,
54,
54,
54,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
55,
56,
57,
58,
58,
58,
58,
58,
57,
56,
55,
51,
48,
47,
47,
47,
47,
48,
51,
54,
55,
56,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
58,
50,
49,
49,
49,
56,
58,
57,
58,
59,
59,
56,
54,
51,
49,
54,
57,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
58,
57,
56,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
55,
55,
55,
55,
54,
54,
54,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
50,
55,
57,
58,
58,
58,
57,
57,
57,
56,
55,
53,
50,
48,
47,
47,
47,
47,
48,
51,
54,
55,
56,
57,
58,
58,
58,
58,
58,
58,
58,
57,
58,
58,
57,
57,
58,
58,
59,
60,
58,
56,
49,
49,
49,
54,
57,
58,
58,
58,
58,
59,
57,
54,
52,
52,
54,
57,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
61,
61,
60,
60,
60,
60,
60,
60,
60,
59,
59,
58,
58,
57,
56,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
55,
55,
55,
55,
54,
54,
54,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
50,
50,
50,
56,
58,
58,
58,
57,
57,
57,
57,
56,
54,
52,
49,
48,
47,
47,
47,
47,
48,
51,
54,
55,
56,
57,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
58,
58,
59,
58,
52,
49,
49,
49,
55,
58,
59,
58,
58,
58,
58,
58,
54,
53,
53,
55,
57,
59,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
59,
58,
58,
57,
56,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
55,
55,
55,
55,
54,
54,
54,
54,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
50,
50,
50,
50,
57,
58,
58,
58,
58,
57,
56,
55,
54,
51,
50,
48,
47,
47,
47,
47,
47,
48,
51,
54,
55,
56,
57,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
58,
59,
57,
50,
49,
49,
49,
55,
58,
59,
58,
58,
58,
58,
57,
55,
54,
53,
55,
56,
58,
61,
62,
61,
61,
60,
60,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
59,
58,
57,
56,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
55,
55,
55,
55,
54,
54,
54,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
50,
50,
50,
50,
50,
50,
58,
58,
58,
58,
58,
58,
56,
53,
50,
48,
48,
47,
47,
47,
47,
47,
47,
48,
50,
55,
56,
56,
57,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
56,
56,
57,
58,
58,
51,
49,
49,
49,
56,
58,
59,
58,
58,
58,
58,
57,
56,
55,
54,
54,
56,
57,
61,
62,
61,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
59,
58,
57,
56,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
55,
55,
55,
54,
54,
54,
54,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
50,
50,
50,
50,
50,
50,
58,
58,
58,
58,
58,
57,
56,
53,
48,
47,
47,
47,
47,
47,
47,
47,
47,
47,
48,
54,
55,
56,
57,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
55,
55,
56,
50,
49,
49,
54,
57,
58,
59,
59,
58,
58,
58,
58,
58,
56,
55,
54,
55,
56,
59,
61,
60,
59,
59,
59,
60,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
59,
58,
58,
57,
56,
56,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
54,
54,
54,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
58,
58,
58,
57,
57,
56,
56,
53,
49,
47,
48,
48,
49,
49,
47,
47,
47,
47,
48,
52,
55,
56,
57,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
55,
53,
49,
49,
49,
49,
55,
58,
58,
59,
58,
58,
58,
58,
58,
58,
58,
56,
55,
54,
55,
56,
57,
58,
58,
58,
58,
60,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
59,
58,
57,
57,
56,
56,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
54,
54,
54,
54,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
49,
49,
50,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
49,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
58,
58,
57,
57,
57,
56,
55,
52,
48,
48,
49,
50,
51,
50,
48,
47,
48,
47,
49,
51,
54,
56,
57,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
56,
56,
55,
55,
51,
49,
49,
49,
49,
57,
58,
58,
58,
59,
58,
58,
58,
58,
58,
58,
58,
56,
55,
55,
55,
55,
55,
56,
57,
58,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
59,
58,
57,
56,
56,
55,
55,
55,
55,
55,
56,
56,
56,
56,
57,
57,
57,
56,
56,
56,
55,
55,
55,
54,
54,
54,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
58,
57,
57,
57,
57,
55,
54,
51,
48,
48,
50,
53,
53,
52,
50,
48,
47,
48,
49,
52,
54,
56,
57,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
56,
56,
55,
55,
52,
49,
49,
49,
55,
58,
58,
59,
58,
58,
58,
58,
58,
59,
58,
58,
58,
58,
57,
57,
57,
55,
55,
55,
57,
58,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
59,
58,
57,
56,
56,
55,
55,
55,
55,
56,
56,
56,
56,
57,
57,
57,
57,
57,
56,
56,
56,
55,
55,
54,
54,
54,
54,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
57,
57,
57,
57,
56,
56,
54,
51,
49,
49,
50,
55,
55,
54,
53,
49,
48,
48,
49,
52,
54,
56,
57,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
56,
56,
55,
55,
53,
49,
49,
49,
56,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
55,
55,
57,
59,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
61,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
59,
58,
57,
57,
56,
55,
55,
55,
55,
56,
56,
56,
57,
57,
57,
57,
57,
57,
58,
57,
57,
56,
55,
55,
55,
54,
54,
54,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
51,
50,
50,
57,
57,
57,
56,
57,
56,
56,
53,
49,
49,
49,
53,
55,
55,
55,
51,
49,
48,
49,
51,
54,
56,
57,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
56,
56,
56,
55,
55,
52,
49,
49,
52,
55,
57,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
55,
55,
57,
59,
61,
61,
61,
61,
61,
61,
61,
61,
60,
61,
61,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
59,
58,
57,
56,
56,
55,
55,
55,
56,
56,
56,
57,
57,
57,
57,
57,
57,
58,
58,
58,
57,
56,
55,
55,
55,
54,
54,
54,
54,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
51,
51,
50,
51,
51,
50,
57,
57,
57,
57,
56,
55,
56,
55,
49,
49,
50,
52,
55,
56,
56,
52,
48,
48,
48,
50,
54,
56,
57,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
56,
56,
55,
55,
50,
49,
49,
52,
55,
56,
57,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
55,
56,
57,
59,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
61,
61,
61,
60,
58,
57,
57,
56,
55,
55,
55,
55,
56,
56,
57,
57,
57,
57,
58,
58,
57,
58,
58,
58,
57,
56,
55,
55,
55,
54,
54,
54,
54,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
51,
51,
51,
50,
50,
51,
51,
51,
51,
51,
50,
56,
57,
57,
56,
56,
55,
54,
54,
49,
49,
50,
52,
54,
56,
56,
52,
48,
48,
48,
50,
54,
56,
58,
58,
58,
58,
57,
57,
58,
57,
57,
57,
57,
57,
56,
56,
55,
53,
49,
49,
49,
54,
56,
56,
57,
58,
59,
59,
59,
59,
60,
60,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
57,
59,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
61,
61,
61,
59,
58,
57,
56,
56,
55,
55,
55,
56,
56,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
57,
56,
55,
55,
55,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
51,
50,
50,
50,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
56,
57,
57,
56,
55,
54,
53,
50,
49,
49,
51,
53,
55,
56,
55,
52,
48,
48,
48,
52,
56,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
56,
56,
55,
53,
49,
49,
49,
55,
56,
57,
58,
59,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
58,
58,
58,
58,
58,
59,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
61,
61,
61,
59,
58,
57,
56,
55,
55,
55,
56,
56,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
56,
55,
55,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
56,
57,
56,
56,
54,
53,
50,
49,
49,
50,
52,
54,
55,
56,
55,
51,
48,
48,
48,
54,
56,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
56,
56,
56,
54,
49,
49,
50,
56,
57,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
58,
58,
58,
59,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
61,
61,
61,
61,
61,
62,
62,
62,
62,
61,
61,
61,
61,
61,
58,
57,
57,
56,
55,
56,
56,
56,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
56,
55,
55,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
50,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
56,
56,
56,
55,
54,
49,
49,
49,
49,
51,
53,
54,
56,
56,
55,
51,
48,
48,
48,
55,
57,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
58,
58,
57,
57,
56,
55,
54,
50,
50,
52,
56,
59,
60,
60,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
59,
59,
59,
58,
58,
58,
58,
59,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
61,
61,
61,
61,
62,
62,
62,
62,
62,
61,
61,
61,
61,
59,
58,
57,
57,
56,
56,
56,
56,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
56,
56,
55,
55,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
50,
50,
50,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
50,
55,
56,
55,
54,
50,
49,
49,
49,
51,
53,
54,
55,
55,
55,
53,
50,
48,
48,
49,
54,
57,
58,
58,
58,
58,
58,
58,
59,
60,
60,
60,
60,
59,
58,
57,
56,
55,
53,
50,
50,
53,
57,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
59,
59,
58,
58,
58,
58,
59,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
61,
62,
61,
61,
62,
62,
62,
62,
62,
61,
61,
61,
61,
59,
57,
57,
57,
57,
56,
57,
57,
57,
58,
58,
58,
58,
59,
58,
58,
58,
59,
58,
58,
58,
57,
56,
56,
55,
55,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
53,
54,
53,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
55,
55,
55,
52,
49,
49,
50,
52,
54,
55,
55,
56,
55,
54,
52,
49,
49,
49,
49,
51,
56,
58,
58,
58,
58,
58,
59,
60,
60,
60,
61,
60,
60,
59,
58,
56,
55,
52,
50,
50,
54,
59,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
59,
59,
58,
58,
58,
59,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
61,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
60,
60,
58,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
59,
59,
59,
58,
59,
58,
58,
58,
57,
56,
56,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
55,
54,
53,
50,
49,
49,
51,
54,
55,
56,
56,
55,
55,
53,
52,
51,
50,
49,
49,
49,
55,
55,
56,
57,
57,
58,
59,
60,
60,
61,
61,
61,
60,
60,
58,
57,
55,
52,
50,
50,
55,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
59,
59,
58,
58,
59,
61,
61,
61,
61,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
59,
59,
58,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
58,
58,
58,
57,
56,
56,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
50,
49,
49,
50,
52,
54,
56,
56,
56,
55,
54,
53,
52,
53,
52,
50,
49,
49,
53,
55,
56,
56,
56,
58,
59,
60,
60,
61,
61,
61,
61,
60,
59,
57,
55,
51,
50,
51,
56,
60,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
59,
59,
59,
59,
60,
61,
61,
61,
61,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
60,
59,
58,
58,
58,
57,
57,
57,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
58,
58,
58,
57,
57,
56,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
49,
49,
49,
49,
51,
54,
55,
55,
56,
56,
55,
54,
53,
54,
55,
53,
50,
49,
49,
50,
54,
55,
55,
56,
57,
59,
59,
60,
61,
61,
62,
62,
61,
59,
57,
55,
50,
50,
53,
57,
60,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
59,
59,
59,
60,
61,
61,
61,
61,
61,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
60,
60,
59,
58,
58,
58,
58,
57,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
57,
57,
56,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
55,
54,
53,
49,
49,
51,
55,
55,
56,
55,
55,
55,
54,
55,
55,
55,
52,
49,
49,
49,
49,
52,
54,
55,
56,
57,
58,
59,
60,
61,
61,
62,
62,
61,
60,
58,
55,
50,
50,
55,
58,
60,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
59,
59,
60,
61,
61,
61,
61,
61,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
60,
60,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
57,
56,
56,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
55,
55,
51,
49,
49,
53,
55,
55,
55,
55,
55,
55,
54,
55,
55,
55,
51,
49,
49,
50,
53,
54,
54,
55,
55,
56,
58,
59,
60,
61,
61,
62,
62,
62,
61,
59,
55,
50,
52,
55,
57,
60,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
60,
60,
59,
59,
60,
61,
61,
61,
61,
61,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
63,
62,
62,
62,
61,
61,
61,
60,
60,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
57,
56,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
55,
54,
50,
49,
54,
55,
55,
56,
55,
55,
55,
55,
55,
55,
55,
54,
50,
49,
50,
54,
55,
55,
55,
55,
55,
56,
57,
59,
60,
61,
62,
62,
62,
62,
62,
59,
55,
53,
53,
54,
56,
59,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
60,
60,
60,
59,
60,
61,
61,
61,
61,
61,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
63,
63,
62,
62,
62,
61,
61,
60,
60,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
57,
56,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
55,
53,
49,
52,
54,
55,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
49,
49,
51,
55,
56,
57,
56,
55,
55,
55,
57,
59,
60,
61,
61,
62,
62,
62,
62,
60,
56,
54,
53,
53,
55,
57,
62,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
60,
60,
60,
59,
59,
61,
61,
61,
61,
61,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
63,
62,
63,
63,
62,
62,
62,
61,
61,
60,
60,
59,
59,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
60,
59,
59,
59,
59,
58,
58,
58,
57,
56,
55,
55,
55,
54,
54,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
55,
53,
52,
54,
55,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
52,
51,
49,
53,
56,
57,
58,
57,
56,
55,
56,
57,
59,
60,
60,
61,
62,
62,
62,
62,
61,
55,
54,
54,
53,
54,
56,
62,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
60,
60,
59,
59,
60,
61,
61,
61,
61,
62,
62,
62,
62,
62,
61,
61,
61,
62,
61,
61,
61,
62,
63,
63,
63,
63,
63,
63,
62,
61,
61,
60,
60,
59,
59,
59,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
59,
59,
58,
58,
58,
58,
56,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
51,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
55,
54,
53,
55,
55,
56,
56,
56,
55,
55,
55,
55,
55,
55,
50,
50,
51,
51,
53,
57,
59,
60,
59,
57,
55,
56,
58,
59,
60,
61,
61,
62,
62,
62,
62,
61,
57,
56,
55,
54,
54,
56,
62,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
60,
60,
59,
60,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
61,
62,
62,
62,
62,
62,
62,
63,
63,
63,
63,
63,
62,
62,
61,
61,
60,
60,
60,
59,
59,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
59,
59,
58,
58,
58,
57,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
51,
51,
51,
51,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
55,
53,
54,
55,
55,
56,
56,
57,
55,
55,
55,
55,
55,
54,
50,
50,
50,
52,
55,
58,
60,
61,
60,
57,
56,
57,
59,
60,
61,
61,
62,
62,
62,
62,
62,
61,
59,
57,
56,
54,
54,
56,
61,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
60,
60,
59,
60,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
64,
64,
63,
63,
63,
62,
62,
61,
61,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
58,
57,
56,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
52,
55,
54,
55,
55,
56,
57,
56,
56,
55,
55,
55,
55,
55,
50,
50,
50,
50,
50,
58,
60,
61,
61,
60,
59,
58,
59,
59,
60,
61,
61,
61,
62,
62,
62,
62,
61,
60,
58,
56,
55,
55,
56,
61,
62,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
63,
63,
63,
63,
63,
63,
64,
64,
64,
63,
63,
62,
61,
61,
61,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
61,
61,
61,
60,
60,
59,
58,
58,
57,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
52,
54,
55,
55,
56,
56,
57,
56,
56,
56,
56,
56,
56,
55,
50,
50,
50,
50,
56,
61,
61,
62,
61,
60,
59,
59,
60,
60,
61,
61,
61,
62,
62,
62,
62,
62,
61,
60,
58,
56,
55,
55,
56,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
63,
63,
63,
63,
63,
64,
64,
64,
63,
63,
63,
62,
61,
61,
61,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
60,
60,
59,
58,
58,
57,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
52,
54,
55,
55,
56,
57,
57,
56,
57,
56,
56,
56,
56,
55,
50,
50,
50,
55,
59,
61,
61,
62,
61,
60,
60,
60,
61,
61,
61,
61,
61,
62,
62,
62,
61,
61,
60,
59,
58,
56,
55,
55,
58,
60,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
61,
60,
61,
61,
61,
61,
61,
62,
62,
63,
63,
63,
63,
64,
64,
64,
64,
63,
63,
62,
62,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
60,
60,
59,
58,
58,
57,
57,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
52,
53,
52,
55,
55,
55,
56,
57,
57,
58,
57,
57,
57,
56,
55,
55,
50,
50,
50,
57,
60,
61,
61,
61,
61,
61,
60,
60,
61,
61,
61,
61,
62,
62,
62,
62,
61,
61,
59,
59,
61,
59,
55,
55,
58,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
62,
62,
63,
63,
63,
63,
64,
64,
64,
64,
63,
63,
62,
62,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
60,
59,
58,
58,
57,
57,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
52,
55,
55,
56,
56,
57,
58,
58,
58,
57,
56,
56,
55,
55,
50,
50,
50,
59,
61,
61,
61,
61,
61,
60,
60,
61,
61,
61,
61,
62,
62,
62,
62,
61,
61,
61,
60,
61,
61,
61,
56,
55,
56,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
62,
62,
63,
63,
63,
63,
64,
64,
64,
64,
63,
63,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
60,
59,
58,
58,
57,
57,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
55,
56,
56,
57,
57,
58,
58,
58,
57,
57,
55,
55,
55,
50,
50,
50,
61,
61,
61,
61,
61,
61,
60,
60,
61,
61,
61,
62,
62,
62,
62,
61,
61,
61,
60,
61,
61,
61,
60,
58,
55,
55,
60,
61,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
62,
62,
63,
63,
63,
63,
64,
64,
64,
64,
64,
63,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
60,
58,
58,
57,
57,
57,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
55,
56,
57,
57,
58,
58,
58,
58,
57,
56,
55,
55,
50,
50,
50,
56,
60,
61,
61,
61,
61,
61,
60,
61,
61,
61,
62,
62,
62,
62,
62,
61,
61,
60,
60,
61,
61,
61,
60,
58,
55,
55,
55,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
62,
62,
63,
63,
63,
64,
64,
64,
64,
64,
64,
63,
63,
62,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
58,
58,
57,
57,
57,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
52,
52,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
54,
53,
55,
56,
57,
58,
58,
59,
59,
58,
56,
55,
55,
55,
50,
50,
56,
58,
60,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
61,
61,
60,
61,
61,
62,
62,
60,
58,
55,
55,
55,
59,
62,
62,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
62,
62,
63,
63,
63,
64,
64,
64,
64,
64,
64,
63,
63,
62,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
58,
58,
57,
57,
57,
57,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
52,
53,
52,
52,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
56,
57,
57,
58,
59,
59,
59,
59,
56,
55,
55,
54,
50,
55,
56,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
62,
62,
61,
56,
55,
55,
55,
59,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
62,
62,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
63,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
58,
58,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
52,
52,
53,
52,
52,
52,
52,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
57,
58,
58,
59,
59,
60,
60,
58,
56,
55,
55,
54,
55,
56,
59,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
63,
62,
61,
61,
61,
61,
61,
62,
62,
61,
59,
55,
55,
55,
55,
58,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
60,
60,
60,
61,
60,
60,
60,
61,
61,
61,
61,
61,
62,
62,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
63,
63,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
58,
58,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
53,
54,
53,
53,
53,
54,
54,
53,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
58,
58,
59,
60,
60,
60,
60,
60,
59,
58,
56,
54,
54,
56,
59,
60,
61,
62,
61,
61,
61,
61,
61,
61,
62,
62,
62,
63,
63,
62,
61,
61,
61,
61,
61,
62,
62,
61,
58,
55,
55,
55,
55,
58,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
60,
60,
60,
61,
60,
61,
61,
61,
61,
61,
61,
61,
62,
62,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
63,
63,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
59,
58,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
53,
54,
54,
59,
59,
60,
60,
60,
61,
61,
61,
61,
60,
59,
56,
54,
55,
55,
61,
61,
62,
61,
61,
61,
61,
62,
62,
62,
62,
62,
63,
63,
62,
61,
61,
61,
61,
61,
62,
61,
61,
58,
56,
55,
55,
55,
59,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
61,
61,
61,
60,
59,
58,
58,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
59,
56,
55,
55,
59,
61,
62,
61,
61,
61,
62,
62,
62,
62,
63,
63,
63,
63,
62,
61,
61,
61,
61,
61,
62,
62,
61,
59,
57,
55,
55,
56,
60,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
61,
61,
61,
60,
58,
58,
58,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
60,
60,
61,
61,
61,
61,
61,
61,
62,
61,
61,
61,
58,
55,
55,
60,
61,
62,
62,
62,
62,
62,
62,
63,
63,
63,
63,
63,
62,
62,
61,
61,
61,
62,
62,
63,
62,
61,
60,
57,
55,
55,
58,
60,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
60,
59,
58,
58,
58,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
60,
61,
61,
61,
61,
61,
62,
62,
62,
61,
61,
61,
58,
56,
55,
60,
62,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
63,
63,
63,
62,
60,
57,
55,
56,
60,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
60,
59,
58,
58,
58,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
53,
54,
54,
54,
54,
54,
54,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
61,
61,
58,
57,
58,
61,
62,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
63,
61,
58,
55,
59,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
62,
62,
62,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
60,
59,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
54,
54,
54,
53,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
54,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
61,
61,
58,
59,
58,
60,
61,
62,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
61,
58,
58,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
62,
62,
62,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
60,
59,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
54,
53,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
54,
61,
61,
61,
62,
62,
62,
62,
62,
62,
61,
61,
60,
59,
59,
59,
60,
61,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
64,
64,
64,
64,
64,
64,
64,
62,
60,
60,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
62,
62,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
60,
59,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
57,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
55,
54,
61,
61,
61,
62,
62,
62,
62,
62,
61,
61,
61,
61,
60,
59,
59,
60,
62,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
64,
64,
63,
61,
60,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
62,
61,
61,
61,
61,
61,
62,
62,
62,
62,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
62,
62,
62,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
60,
59,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
55,
55,
56,
61,
61,
61,
62,
62,
62,
62,
62,
61,
61,
61,
61,
60,
60,
59,
61,
62,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
64,
64,
63,
61,
60,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
60,
59,
59,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
55,
55,
59,
61,
61,
61,
62,
62,
62,
62,
61,
61,
61,
61,
61,
60,
59,
60,
61,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
64,
62,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
62,
63,
62,
62,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
60,
59,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
55,
58,
62,
61,
61,
61,
62,
62,
62,
61,
61,
61,
61,
61,
61,
60,
60,
60,
61,
63,
64,
64,
64,
64,
64,
64,
64,
64,
65,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
64,
63,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
62,
62,
63,
63,
63,
63,
62,
63,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
60,
59,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
55,
59,
62,
61,
61,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
61,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
63,
61,
61,
61,
61,
61,
61,
62,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
63,
62,
62,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
62,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
60,
59,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
54,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
58,
61,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
60,
61,
62,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
64,
61,
61,
61,
61,
61,
61,
62,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
60,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
53,
53,
53,
53,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
56,
63,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
63,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
63,
61,
61,
61,
61,
61,
61,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
59,
58,
57,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
53,
53,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
61,
62,
62,
62,
62,
61,
61,
61,
61,
61,
62,
61,
61,
61,
62,
62,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
65,
65,
65,
65,
64,
63,
61,
61,
61,
61,
62,
62,
63,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
59,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
54,
61,
62,
62,
62,
62,
61,
61,
61,
61,
62,
64,
62,
61,
61,
62,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
65,
65,
65,
65,
64,
63,
61,
61,
61,
61,
63,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
59,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
56,
64,
63,
62,
62,
62,
61,
61,
61,
61,
63,
64,
64,
62,
62,
62,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
65,
65,
65,
63,
62,
62,
61,
61,
61,
62,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
61,
61,
61,
60,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
56,
59,
71,
63,
63,
62,
62,
61,
61,
61,
61,
63,
64,
64,
64,
64,
63,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
65,
64,
64,
63,
61,
61,
61,
61,
61,
61,
62,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
64,
64,
64,
64,
64,
64,
64,
64,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
64,
64,
63,
63,
63,
63,
63,
62,
62,
62,
62,
61,
61,
61,
61,
61,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
63,
79,
63,
63,
62,
62,
62,
61,
61,
61,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
65,
64,
64,
64,
61,
61,
61,
61,
61,
61,
61,
61,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
64,
64,
64,
64,
64,
64,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
62,
62,
62,
62,
61,
61,
61,
61,
61,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
56,
57,
70,
85,
63,
63,
62,
62,
62,
61,
61,
62,
64,
64,
65,
65,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
65,
64,
64,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
64,
64,
64,
64,
64,
64,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
62,
62,
62,
62,
61,
61,
61,
61,
60,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
57,
61,
77,
89,
63,
63,
63,
62,
62,
61,
61,
63,
64,
65,
65,
65,
65,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
63,
61,
61,
62,
61,
61,
61,
62,
61,
61,
62,
63,
64,
64,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
62,
62,
62,
62,
62,
61,
61,
61,
61,
60,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
68,
81,
90,
64,
63,
63,
63,
62,
62,
62,
64,
64,
65,
65,
65,
65,
65,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
63,
62,
61,
63,
64,
64,
63,
62,
62,
61,
63,
64,
64,
64,
64,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
60,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
56,
58,
73,
85,
92,
64,
63,
63,
63,
63,
63,
64,
64,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
65,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
64,
64,
63,
62,
63,
64,
64,
64,
64,
63,
63,
64,
64,
64,
64,
64,
64,
65,
65,
64,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
55,
64,
75,
86,
91,
64,
63,
63,
64,
64,
64,
64,
65,
65,
65,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
66,
66,
66,
65,
66,
65,
65,
65,
65,
64,
64,
64,
63,
63,
64,
64,
64,
64,
64,
63,
63,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
62,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
57,
68,
85,
87,
91,
64,
63,
63,
64,
64,
64,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
64,
64,
64,
65,
64,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
62,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
57,
70,
89,
91,
94,
64,
63,
64,
64,
64,
65,
65,
65,
66,
66,
66,
66,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
65,
66,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
62,
62,
62,
62,
62,
61,
61,
61,
61,
60,
60,
60,
60,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
56,
57,
72,
86,
97,
97,
64,
64,
64,
64,
64,
65,
65,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
62,
62,
62,
62,
61,
61,
61,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
56,
58,
78,
91,
96,
98,
64,
64,
64,
64,
65,
65,
65,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
67,
67,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
64,
64,
64,
64,
64,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
62,
62,
61,
61,
61,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
54,
54,
54,
54,
55,
55,
55,
59,
82,
93,
94,
96,
65,
64,
64,
65,
65,
65,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
64,
65,
64,
64,
64,
64,
64,
64,
64,
64,
66,
67,
67,
67,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
64,
65,
64,
64,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
62,
61,
61,
61,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
54,
54,
54,
54,
54,
55,
55,
55,
55,
60,
84,
92,
93,
91,
65,
65,
65,
65,
65,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
65,
67,
67,
67,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
62,
61,
61,
61,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
64,
87,
90,
92,
91,
65,
65,
65,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
65,
66,
67,
67,
67,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
65,
65,
64,
65,
64,
64,
64,
64,
62,
61,
61,
61,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
57,
69,
90,
91,
91,
91,
65,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
65,
66,
67,
67,
67,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
62,
61,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
57,
57,
57,
57,
56,
56,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
60,
76,
85,
89,
90,
90,
66,
65,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
65,
66,
67,
67,
67,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
63,
61,
60,
61,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
57,
57,
57,
57,
56,
56,
57,
57,
57,
56,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
58,
68,
83,
87,
89,
90,
90,
65,
65,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
68,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
65,
66,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
65,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
62,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
55,
56,
55,
55,
55,
55,
55,
55,
55,
55,
56,
56,
56,
57,
59,
61,
76,
87,
89,
90,
90,
90,
64,
65,
66,
66,
67,
67,
67,
67,
67,
67,
67,
68,
68,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
63,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
57,
59,
59,
60,
69,
86,
90,
91,
91,
91,
91,
64,
65,
66,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
66,
66,
66,
65,
65,
65,
65,
64,
64,
62,
60,
60,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
57,
57,
58,
59,
62,
65,
84,
88,
91,
92,
92,
92,
90,
65,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
66,
65,
66,
65,
66,
66,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
66,
65,
65,
65,
65,
65,
64,
64,
63,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
60,
60,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
57,
57,
58,
58,
59,
60,
61,
76,
91,
92,
92,
91,
91,
92,
90,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
64,
64,
62,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
56,
57,
57,
57,
57,
58,
58,
59,
60,
61,
64,
85,
89,
92,
92,
92,
92,
92,
91,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
64,
64,
62,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
56,
56,
56,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
59,
59,
60,
61,
71,
87,
90,
92,
91,
91,
92,
91,
90,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
66,
65,
66,
66,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
64,
63,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
59,
59,
60,
60,
61,
76,
84,
91,
91,
91,
90,
90,
90,
89,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
66,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
65,
65,
65,
64,
64,
62,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
59,
59,
60,
60,
60,
63,
81,
86,
90,
91,
90,
90,
89,
88,
88,
68,
68,
68,
68,
68,
67,
67,
67,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
65,
65,
65,
64,
63,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
61,
60,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
59,
59,
60,
60,
61,
63,
66,
86,
87,
88,
89,
89,
89,
88,
88,
88,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
65,
64,
64,
63,
62,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
59,
59,
59,
60,
60,
60,
64,
69,
87,
86,
88,
87,
88,
89,
88,
88,
88,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
67,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
65,
64,
64,
63,
62,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
59,
59,
59,
60,
60,
60,
60,
61,
72,
81,
85,
87,
87,
88,
88,
88,
88,
88,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
65,
64,
64,
63,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
59,
59,
59,
60,
60,
60,
61,
61,
75,
81,
84,
86,
88,
88,
88,
88,
88,
88,
69,
69,
68,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
65,
65,
64,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
59,
59,
59,
60,
60,
60,
61,
61,
77,
82,
84,
85,
87,
88,
88,
88,
88,
88,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
65,
64,
64,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
61,
61,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
59,
59,
59,
60,
60,
60,
61,
63,
77,
83,
84,
86,
87,
88,
88,
88,
88,
88,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
66,
65,
64,
63,
63,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
57,
57,
57,
57,
57,
58,
58,
58,
58,
58,
58,
59,
59,
59,
60,
60,
62,
64,
79,
84,
85,
86,
87,
88,
88,
89,
89,
88,
70,
70,
70,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
66,
65,
64,
63,
63,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
57,
58,
58,
58,
58,
58,
58,
59,
59,
59,
60,
61,
63,
67,
84,
85,
85,
86,
88,
88,
88,
89,
89,
88,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
66,
65,
64,
63,
63,
63,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
60,
61,
63,
66,
84,
86,
86,
87,
87,
88,
88,
89,
89,
89,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
65,
65,
64,
64,
63,
63,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
60,
61,
63,
65,
82,
86,
86,
87,
87,
88,
88,
89,
89,
89,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
66,
65,
64,
64,
64,
64,
63,
63,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
61,
63,
65,
82,
86,
86,
87,
88,
88,
88,
89,
89,
89,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
66,
65,
64,
64,
64,
64,
64,
63,
63,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
63,
67,
84,
86,
86,
87,
88,
88,
88,
89,
90,
90,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
64,
64,
64,
64,
64,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
58,
69,
88,
86,
87,
87,
88,
88,
88,
89,
90,
90,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
64,
64,
64,
64,
64,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
72,
86,
86,
87,
87,
88,
88,
88,
89,
90,
90,
71,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
69,
69,
69,
69,
69,
69,
69,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
69,
68,
68,
68,
69,
69,
69,
68,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
64,
64,
64,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
73,
82,
86,
87,
87,
88,
88,
88,
90,
90,
90,
71,
71,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
69,
69,
68,
68,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
64,
64,
64,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
73,
88,
85,
86,
87,
88,
87,
88,
90,
90,
91,
71,
71,
71,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
67,
64,
64,
64,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
60,
61,
76,
79,
80,
86,
87,
87,
87,
88,
89,
90,
91,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
70,
70,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
67,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
63,
63,
63,
63,
62,
62,
62,
63,
63,
63,
62,
62,
63,
62,
62,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
60,
65,
73,
78,
86,
86,
87,
86,
86,
86,
87,
90,
91,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
70,
70,
70,
73,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
67,
67,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
59,
60,
61,
74,
79,
88,
86,
86,
86,
86,
86,
86,
86,
90,
91,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
70,
69,
69,
69,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
70,
70,
71,
73,
71,
70,
70,
70,
70,
71,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
67,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
63,
62,
63,
63,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
58,
58,
58,
58,
58,
58,
59,
58,
59,
59,
60,
60,
61,
75,
82,
85,
86,
87,
86,
86,
86,
86,
89,
91,
92,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
70,
70,
70,
70,
70,
70,
70,
71,
72,
72,
70,
70,
70,
71,
71,
71,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
67,
66,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
58,
58,
58,
59,
59,
58,
59,
59,
59,
59,
60,
61,
67,
68,
78,
86,
86,
87,
88,
86,
86,
87,
91,
91,
91,
72,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
72,
73,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
67,
65,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
62,
66,
82,
86,
86,
86,
87,
88,
88,
88,
87,
89,
91,
91,
91,
72,
72,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
72,
73,
73,
73,
72,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
68,
68,
67,
65,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
61,
74,
86,
87,
88,
87,
87,
88,
88,
89,
89,
90,
91,
91,
91,
72,
72,
72,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
70,
70,
70,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
71,
72,
73,
73,
73,
72,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
68,
68,
67,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
63,
62,
63,
62,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
80,
84,
87,
87,
87,
88,
88,
88,
89,
90,
90,
91,
91,
91,
72,
72,
72,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
70,
70,
70,
70,
71,
72,
73,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
72,
72,
73,
73,
73,
73,
72,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
68,
68,
67,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
61,
84,
86,
86,
87,
87,
85,
86,
87,
90,
91,
91,
91,
91,
91,
73,
72,
72,
72,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
70,
69,
69,
70,
70,
71,
72,
73,
73,
73,
72,
72,
72,
72,
72,
71,
71,
71,
72,
73,
72,
72,
72,
73,
73,
72,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
68,
68,
67,
65,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
60,
60,
60,
61,
60,
60,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
61,
63,
85,
86,
87,
86,
86,
81,
86,
88,
90,
91,
92,
91,
91,
91,
73,
73,
72,
72,
72,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
72,
72,
72,
72,
72,
73,
73,
72,
71,
72,
72,
73,
73,
73,
72,
72,
71,
71,
72,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
68,
67,
65,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
64,
85,
86,
86,
86,
80,
84,
88,
88,
90,
91,
92,
91,
91,
91,
73,
73,
73,
72,
72,
72,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
72,
72,
72,
72,
72,
72,
73,
73,
72,
72,
72,
72,
72,
73,
73,
72,
71,
70,
70,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
68,
66,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
61,
82,
86,
82,
78,
81,
85,
88,
89,
91,
92,
92,
91,
91,
91,
73,
73,
73,
72,
72,
72,
72,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
72,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
73,
73,
73,
73,
72,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
72,
72,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
68,
66,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
60,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
63,
69,
80,
74,
74,
81,
86,
87,
89,
91,
91,
92,
91,
91,
90,
73,
73,
73,
72,
72,
72,
72,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
72,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
73,
73,
73,
72,
72,
72,
70,
71,
70,
71,
70,
70,
70,
70,
70,
70,
70,
70,
71,
72,
73,
73,
71,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
69,
68,
66,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
59,
60,
60,
60,
60,
61,
61,
60,
63,
73,
78,
78,
85,
86,
88,
90,
91,
91,
91,
90,
90,
73,
73,
73,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
72,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
73,
72,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
72,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
69,
67,
66,
65,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
61,
65,
68,
79,
86,
86,
85,
89,
90,
91,
91,
91,
90,
90,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
72,
72,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
69,
68,
67,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
59,
59,
59,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
63,
64,
69,
73,
86,
86,
87,
88,
89,
90,
90,
90,
90,
89,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
72,
73,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
69,
68,
67,
67,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
66,
71,
82,
82,
87,
88,
89,
89,
89,
90,
90,
90,
90,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
73,
73,
72,
71,
71,
71,
70,
70,
70,
70,
70,
71,
71,
72,
73,
75,
76,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
70,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
69,
69,
68,
68,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
4,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
62,
62,
61,
79,
85,
86,
87,
88,
89,
89,
89,
90,
90,
90,
89,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
73,
73,
73,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
72,
73,
74,
76,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
71,
71,
71,
71,
72,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
69,
69,
68,
68,
66,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
62,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
63,
66,
84,
84,
87,
87,
88,
88,
88,
89,
89,
89,
89,
89,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
73,
73,
73,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
73,
75,
73,
73,
72,
72,
72,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
73,
73,
73,
73,
71,
71,
71,
72,
72,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
69,
68,
68,
66,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
72,
75,
84,
85,
87,
88,
87,
88,
89,
89,
89,
89,
89,
74,
73,
73,
73,
72,
72,
72,
72,
72,
72,
73,
73,
73,
72,
72,
72,
71,
71,
71,
71,
71,
72,
71,
72,
72,
73,
73,
72,
72,
72,
72,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
73,
72,
72,
73,
71,
71,
73,
73,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
69,
68,
68,
67,
66,
66,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
62,
62,
62,
61,
61,
62,
62,
61,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
63,
66,
79,
80,
84,
85,
86,
87,
87,
88,
88,
89,
88,
88,
88,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
73,
73,
73,
73,
72,
71,
71,
71,
72,
72,
72,
72,
72,
72,
72,
73,
73,
72,
72,
72,
72,
72,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
73,
73,
73,
72,
72,
72,
72,
73,
73,
73,
73,
72,
72,
73,
73,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
69,
68,
68,
67,
67,
66,
66,
66,
66,
66,
66,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
61,
61,
77,
80,
84,
86,
86,
86,
87,
87,
87,
88,
88,
88,
88,
88,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
73,
73,
72,
72,
72,
72,
72,
72,
72,
73,
73,
73,
72,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
73,
72,
73,
73,
73,
73,
73,
72,
73,
73,
73,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
69,
69,
68,
68,
67,
67,
67,
67,
67,
67,
67,
66,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
62,
62,
65,
85,
84,
84,
86,
86,
86,
86,
86,
86,
87,
88,
88,
88,
87,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
73,
73,
73,
73,
73,
73,
72,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
73,
73,
73,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
69,
68,
68,
68,
67,
67,
67,
67,
67,
67,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
62,
61,
61,
61,
61,
61,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
62,
64,
68,
85,
86,
87,
87,
86,
85,
85,
85,
85,
87,
87,
87,
88,
87,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
73,
73,
73,
73,
73,
73,
73,
72,
72,
71,
71,
72,
72,
72,
72,
72,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
69,
69,
69,
68,
67,
67,
67,
67,
67,
67,
66,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
62,
62,
62,
62,
61,
61,
61,
61,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
60,
60,
60,
61,
63,
64,
69,
80,
86,
87,
86,
86,
85,
85,
85,
85,
87,
88,
88,
87,
87,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
73,
73,
73,
73,
73,
73,
73,
73,
72,
71,
71,
71,
71,
72,
72,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
74,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
69,
69,
69,
68,
67,
67,
67,
67,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
63,
62,
62,
62,
63,
63,
62,
62,
62,
63,
62,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
62,
63,
63,
63,
63,
63,
63,
63,
63,
62,
61,
61,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
60,
60,
60,
60,
60,
60,
61,
61,
64,
66,
75,
81,
88,
86,
86,
85,
84,
84,
85,
86,
87,
87,
87,
87,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
72,
72,
72,
72,
72,
73,
73,
73,
73,
73,
73,
74,
73,
73,
72,
72,
71,
71,
72,
71,
71,
73,
72,
73,
73,
73,
72,
72,
73,
73,
73,
74,
74,
75,
75,
73,
74,
74,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
69,
69,
68,
68,
67,
67,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
62,
63,
63,
63,
63,
63,
63,
63,
63,
64,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
60,
61,
61,
61,
61,
74,
79,
77,
85,
86,
86,
85,
85,
84,
85,
86,
86,
87,
86,
86,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
72,
72,
72,
73,
73,
73,
73,
73,
73,
73,
74,
73,
73,
73,
72,
72,
71,
71,
71,
72,
72,
72,
72,
72,
72,
73,
73,
73,
73,
74,
74,
75,
76,
75,
74,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
69,
69,
69,
68,
67,
67,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
63,
62,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
76,
87,
89,
85,
87,
86,
86,
84,
83,
83,
84,
85,
86,
86,
86,
76,
75,
75,
75,
75,
75,
75,
75,
75,
74,
73,
73,
73,
73,
73,
73,
72,
73,
73,
73,
73,
73,
74,
74,
74,
74,
73,
73,
72,
71,
70,
70,
71,
71,
72,
72,
72,
72,
72,
73,
73,
73,
73,
74,
75,
75,
76,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
70,
70,
69,
69,
68,
68,
67,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
63,
63,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
62,
64,
64,
64,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
73,
83,
85,
87,
87,
87,
85,
81,
82,
82,
83,
84,
85,
85,
84,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
74,
74,
74,
74,
74,
74,
73,
73,
72,
71,
70,
70,
71,
71,
71,
72,
72,
73,
73,
73,
73,
73,
73,
74,
75,
76,
76,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
70,
70,
69,
69,
68,
68,
67,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
65,
65,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
63,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
63,
63,
64,
63,
61,
60,
63,
64,
64,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
63,
68,
78,
84,
84,
86,
86,
85,
80,
79,
81,
82,
83,
84,
84,
84,
76,
76,
76,
76,
75,
75,
75,
76,
76,
76,
75,
73,
73,
73,
73,
73,
73,
73,
73,
74,
74,
74,
74,
74,
74,
74,
74,
73,
71,
70,
70,
70,
71,
71,
72,
72,
73,
73,
73,
73,
73,
73,
74,
75,
75,
76,
76,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
70,
70,
69,
69,
69,
68,
67,
67,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
63,
62,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
63,
64,
64,
61,
59,
59,
64,
64,
64,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
62,
61,
68,
77,
81,
85,
86,
85,
80,
79,
80,
82,
83,
83,
83,
82,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
76,
74,
73,
73,
73,
73,
73,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
71,
70,
70,
70,
71,
71,
72,
72,
73,
73,
73,
73,
74,
74,
74,
75,
75,
75,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
70,
70,
69,
69,
69,
68,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
63,
62,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
62,
64,
64,
64,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
66,
71,
73,
76,
79,
75,
77,
79,
81,
82,
82,
82,
81,
77,
77,
76,
76,
76,
76,
76,
77,
79,
78,
76,
75,
74,
74,
74,
74,
74,
74,
75,
75,
75,
75,
75,
74,
74,
74,
73,
72,
71,
70,
70,
70,
71,
71,
72,
72,
73,
73,
73,
74,
74,
74,
74,
75,
75,
75,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
70,
70,
70,
69,
69,
68,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
63,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
63,
67,
70,
69,
75,
78,
80,
81,
81,
81,
81,
77,
77,
77,
77,
77,
77,
77,
79,
80,
79,
77,
76,
75,
75,
75,
75,
75,
75,
75,
76,
75,
75,
75,
74,
74,
73,
72,
71,
70,
70,
70,
70,
71,
71,
72,
72,
73,
73,
73,
74,
74,
75,
75,
75,
75,
75,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
70,
70,
69,
69,
68,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
61,
61,
62,
61,
62,
62,
62,
62,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
64,
64,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
61,
63,
64,
67,
74,
79,
78,
79,
80,
80,
78,
77,
77,
77,
77,
78,
78,
79,
80,
79,
78,
76,
76,
75,
75,
76,
76,
76,
76,
76,
76,
75,
75,
74,
73,
72,
71,
70,
70,
70,
70,
71,
71,
71,
72,
72,
73,
73,
73,
73,
74,
75,
75,
75,
76,
76,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
73,
72,
73,
73,
72,
73,
72,
73,
72,
72,
72,
72,
72,
71,
71,
70,
69,
69,
68,
68,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
62,
61,
57,
55,
61,
60,
58,
61,
62,
62,
62,
62,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
64,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
61,
61,
62,
61,
61,
61,
61,
61,
63,
65,
68,
69,
76,
80,
78,
78,
78,
78,
78,
78,
79,
79,
80,
79,
78,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
74,
73,
73,
71,
70,
70,
70,
70,
70,
71,
71,
71,
72,
72,
73,
73,
73,
74,
74,
75,
75,
75,
76,
76,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
71,
71,
70,
69,
69,
69,
68,
68,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
62,
61,
59,
54,
62,
58,
56,
60,
60,
61,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
64,
62,
61,
61,
62,
62,
62,
62,
67,
75,
79,
78,
78,
79,
79,
79,
79,
79,
80,
79,
78,
77,
76,
76,
76,
77,
77,
77,
77,
76,
76,
75,
74,
73,
72,
71,
70,
70,
70,
71,
71,
71,
71,
71,
72,
72,
73,
73,
73,
74,
74,
75,
75,
75,
76,
76,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
71,
71,
70,
69,
69,
69,
68,
68,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
66,
65,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
62,
62,
60,
55,
62,
56,
55,
55,
57,
62,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
63,
64,
64,
63,
64,
63,
62,
62,
62,
62,
65,
79,
79,
79,
79,
79,
79,
79,
80,
80,
79,
79,
77,
76,
76,
77,
77,
77,
77,
77,
77,
76,
75,
74,
73,
73,
71,
71,
71,
71,
71,
71,
71,
71,
71,
72,
72,
73,
73,
73,
73,
74,
75,
75,
75,
76,
76,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
71,
71,
70,
70,
69,
69,
68,
68,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
62,
60,
57,
62,
54,
54,
54,
55,
59,
64,
64,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
62,
62,
63,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
62,
63,
63,
64,
64,
64,
64,
64,
64,
64,
63,
62,
62,
63,
79,
79,
79,
79,
79,
80,
80,
80,
79,
79,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
75,
74,
73,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
72,
72,
73,
73,
73,
74,
74,
74,
75,
75,
75,
75,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
71,
70,
70,
69,
69,
68,
68,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
61,
60,
61,
60,
59,
57,
57,
59,
63,
64,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
62,
63,
63,
62,
62,
62,
62,
62,
62,
62,
63,
63,
63,
63,
64,
64,
64,
65,
65,
64,
64,
64,
64,
64,
64,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
78,
78,
78,
78,
78,
78,
77,
77,
77,
76,
75,
74,
73,
72,
72,
72,
71,
71,
71,
72,
72,
72,
71,
72,
72,
73,
73,
73,
74,
74,
74,
75,
75,
75,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
71,
70,
70,
69,
69,
68,
68,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
60,
62,
61,
62,
64,
64,
64,
63,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
65,
65,
65,
65,
64,
64,
64,
64,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
78,
78,
78,
77,
77,
77,
76,
75,
74,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
73,
73,
73,
74,
74,
74,
75,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
71,
70,
70,
69,
69,
68,
68,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
64,
63,
63,
63,
64,
64,
64,
64,
64,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
63,
63,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
81,
81,
81,
80,
81,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
78,
78,
77,
77,
76,
76,
75,
74,
73,
73,
73,
73,
72,
72,
72,
72,
73,
73,
72,
72,
72,
72,
73,
73,
73,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
74,
73,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
71,
70,
70,
69,
69,
68,
68,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
63,
64,
64,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
79,
79,
79,
79,
78,
77,
77,
77,
76,
76,
75,
74,
74,
73,
73,
73,
73,
72,
72,
72,
73,
73,
73,
73,
72,
72,
72,
73,
73,
73,
73,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
72,
72,
71,
71,
70,
69,
69,
68,
68,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
64,
64,
64,
65,
64,
64,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
61,
62,
64,
65,
64,
64,
64,
64,
64,
64,
63,
63,
63,
64,
64,
64,
64,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
63,
64,
64,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
64,
65,
64,
64,
64,
63,
63,
63,
63,
63,
63,
63,
63,
63,
64,
64,
64,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
79,
79,
78,
77,
77,
76,
76,
75,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
72,
72,
71,
71,
70,
70,
69,
69,
68,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
66,
66,
66,
67,
66,
66,
66,
66,
66,
66,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
62,
60,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
66,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
64,
63,
64,
63,
63,
63,
63,
64,
64,
64,
64,
64,
64,
65,
65,
65,
66,
66,
66,
66,
68,
67,
67,
82,
82,
82,
82,
81,
81,
81,
81,
81,
80,
80,
80,
79,
79,
79,
78,
77,
77,
76,
76,
75,
75,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
72,
72,
72,
71,
70,
70,
69,
69,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
62,
61,
61,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
66,
66,
66,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
66,
66,
66,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
66,
67,
67,
68,
73,
74,
68,
82,
82,
82,
82,
82,
81,
81,
81,
80,
80,
80,
80,
79,
79,
78,
78,
77,
76,
76,
76,
75,
75,
75,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
72,
72,
71,
71,
70,
69,
69,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
63,
61,
60,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
66,
67,
66,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
66,
66,
66,
66,
67,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
66,
66,
67,
69,
75,
79,
80,
80,
76,
83,
83,
83,
83,
83,
82,
81,
81,
80,
80,
80,
79,
79,
79,
78,
78,
77,
76,
76,
76,
75,
75,
75,
75,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
72,
72,
71,
71,
70,
70,
69,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
63,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
67,
67,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
66,
66,
66,
67,
67,
71,
71,
68,
67,
67,
66,
66,
66,
65,
65,
65,
65,
65,
65,
66,
68,
73,
78,
80,
81,
81,
80,
84,
84,
84,
83,
82,
82,
81,
81,
81,
80,
79,
79,
79,
79,
78,
78,
77,
77,
76,
76,
76,
75,
75,
75,
74,
74,
74,
74,
74,
74,
73,
73,
73,
74,
74,
73,
73,
73,
73,
73,
73,
73,
74,
74,
73,
73,
73,
73,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
72,
72,
71,
70,
70,
69,
69,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
66,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
66,
66,
66,
66,
68,
70,
84,
86,
80,
79,
77,
73,
73,
72,
67,
67,
66,
66,
66,
65,
66,
68,
73,
77,
80,
82,
83,
81,
84,
84,
84,
84,
83,
82,
82,
81,
80,
80,
80,
79,
79,
79,
79,
79,
78,
77,
77,
76,
76,
76,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
74,
74,
74,
74,
74,
74,
74,
74,
74,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
72,
72,
71,
70,
70,
69,
69,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
64,
64,
64,
65,
65,
65,
66,
66,
66,
67,
67,
81,
91,
93,
90,
86,
84,
79,
81,
79,
72,
75,
73,
67,
66,
65,
67,
68,
71,
77,
80,
82,
84,
84,
85,
85,
85,
84,
84,
83,
82,
81,
81,
80,
80,
79,
79,
79,
79,
79,
79,
78,
77,
77,
76,
76,
76,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
75,
75,
75,
75,
75,
75,
75,
75,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
72,
72,
71,
71,
70,
70,
69,
69,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
66,
66,
66,
69,
73,
91,
98,
95,
93,
92,
87,
85,
86,
85,
79,
82,
80,
74,
66,
65,
66,
66,
68,
75,
79,
81,
84,
85,
85,
85,
85,
85,
83,
82,
82,
82,
81,
80,
80,
80,
80,
80,
80,
80,
80,
79,
78,
77,
77,
76,
76,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
75,
75,
75,
75,
75,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
73,
73,
72,
72,
71,
71,
70,
70,
69,
69,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
66,
66,
67,
67,
81,
99,
98,
96,
94,
92,
89,
87,
88,
87,
86,
86,
85,
80,
67,
67,
65,
65,
67,
74,
78,
80,
83,
86,
85,
85,
85,
85,
83,
83,
82,
82,
82,
81,
81,
81,
81,
81,
81,
80,
80,
79,
79,
78,
77,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
75,
74,
75,
75,
75,
75,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
73,
73,
72,
72,
72,
71,
71,
70,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
66,
66,
66,
68,
71,
89,
96,
98,
96,
94,
92,
90,
90,
89,
89,
88,
88,
87,
85,
75,
68,
65,
65,
65,
73,
76,
79,
83,
86,
85,
85,
84,
83,
84,
84,
83,
83,
82,
83,
82,
82,
81,
81,
81,
81,
81,
80,
80,
78,
77,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
73,
73,
73,
74,
75,
75,
75,
75,
75,
75,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
73,
73,
72,
72,
71,
71,
71,
70,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
66,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
64,
64,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
66,
67,
68,
67,
79,
98,
98,
98,
97,
95,
93,
92,
91,
91,
90,
90,
89,
88,
86,
78,
70,
67,
65,
66,
70,
75,
78,
81,
85,
83,
83,
83,
83,
83,
83,
84,
84,
84,
84,
83,
83,
82,
82,
82,
81,
81,
80,
80,
79,
78,
77,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
73,
73,
73,
73,
74,
74,
75,
75,
75,
75,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
73,
73,
73,
72,
72,
71,
70,
70,
70,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
66,
66,
66,
66,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
64,
65,
65,
64,
65,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
66,
67,
67,
70,
73,
91,
101,
101,
99,
97,
96,
95,
93,
93,
92,
92,
91,
90,
88,
86,
82,
72,
67,
65,
65,
66,
73,
76,
78,
82,
83,
83,
83,
83,
83,
83,
84,
84,
84,
85,
84,
84,
83,
83,
82,
82,
81,
81,
80,
80,
79,
77,
76,
76,
76,
76,
75,
75,
76,
76,
76,
75,
75,
75,
74,
74,
74,
73,
73,
73,
74,
74,
75,
75,
75,
75,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
73,
73,
73,
72,
73,
71,
70,
70,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
66,
66,
69,
69,
85,
100,
104,
103,
100,
98,
97,
96,
95,
95,
94,
93,
92,
90,
89,
86,
85,
73,
67,
65,
65,
66,
67,
72,
76,
80,
83,
83,
83,
83,
83,
83,
83,
84,
85,
85,
85,
85,
84,
84,
83,
82,
82,
81,
81,
80,
79,
78,
77,
76,
76,
76,
75,
76,
76,
76,
76,
76,
75,
75,
74,
74,
73,
73,
73,
73,
74,
75,
75,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
73,
73,
73,
73,
73,
71,
70,
70,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
66,
66,
69,
67,
82,
97,
101,
103,
103,
102,
99,
98,
98,
97,
96,
95,
94,
93,
91,
89,
87,
82,
72,
67,
65,
66,
65,
66,
69,
75,
79,
84,
83,
83,
83,
83,
83,
83,
84,
85,
85,
86,
85,
85,
84,
84,
83,
82,
82,
81,
80,
79,
78,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
74,
73,
74,
74,
75,
75,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
73,
72,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
66,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
65,
64,
64,
64,
64,
64,
64,
64,
65,
65,
67,
68,
67,
81,
96,
105,
104,
104,
103,
102,
101,
100,
99,
98,
97,
96,
95,
93,
91,
90,
87,
82,
77,
70,
66,
65,
65,
65,
66,
72,
77,
84,
84,
84,
83,
84,
84,
84,
84,
85,
86,
86,
86,
85,
84,
84,
84,
83,
83,
82,
80,
79,
78,
77,
76,
76,
76,
76,
76,
76,
77,
76,
76,
76,
75,
75,
74,
74,
74,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
73,
71,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
64,
64,
65,
65,
64,
64,
64,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
65,
65,
66,
66,
69,
82,
94,
101,
104,
104,
103,
103,
102,
101,
100,
100,
99,
98,
97,
96,
93,
92,
90,
88,
86,
80,
75,
65,
65,
65,
65,
66,
70,
75,
84,
84,
84,
84,
84,
84,
84,
85,
85,
86,
86,
86,
85,
85,
84,
84,
84,
84,
83,
81,
79,
78,
77,
76,
76,
76,
76,
77,
77,
77,
77,
76,
76,
76,
75,
74,
74,
75,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
74,
74,
74,
74,
73,
73,
70,
70,
71,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
65,
65,
66,
71,
86,
93,
97,
100,
102,
103,
102,
102,
101,
100,
101,
100,
99,
99,
98,
96,
94,
92,
91,
89,
87,
85,
77,
67,
65,
65,
65,
66,
67,
71,
84,
84,
84,
84,
84,
84,
84,
85,
85,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
82,
80,
79,
77,
77,
76,
76,
76,
77,
77,
77,
77,
76,
76,
76,
75,
74,
75,
76,
76,
76,
76,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
74,
74,
74,
73,
73,
72,
71,
71,
71,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
65,
66,
66,
73,
87,
92,
94,
97,
99,
100,
101,
100,
100,
100,
101,
100,
99,
100,
99,
98,
97,
95,
93,
92,
91,
88,
84,
76,
68,
65,
65,
65,
66,
67,
67,
85,
85,
84,
84,
84,
84,
85,
85,
85,
85,
86,
86,
86,
86,
86,
86,
86,
86,
85,
83,
81,
79,
78,
77,
76,
76,
76,
78,
78,
77,
77,
77,
76,
76,
75,
75,
75,
76,
76,
76,
76,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
74,
74,
73,
73,
72,
72,
71,
71,
71,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
66,
66,
66,
66,
65,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
65,
66,
75,
87,
92,
93,
94,
96,
97,
98,
99,
99,
99,
99,
99,
100,
100,
100,
99,
98,
97,
95,
94,
92,
91,
87,
82,
75,
67,
65,
65,
65,
66,
69,
72,
85,
85,
85,
85,
84,
84,
85,
85,
85,
86,
87,
86,
86,
86,
86,
86,
86,
86,
85,
84,
81,
79,
78,
77,
76,
76,
77,
79,
78,
78,
77,
76,
76,
76,
75,
75,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
76,
76,
76,
77,
76,
76,
76,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
74,
74,
73,
73,
72,
72,
71,
72,
72,
71,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
65,
72,
84,
89,
92,
93,
94,
95,
96,
97,
97,
97,
97,
97,
98,
99,
100,
99,
99,
98,
97,
94,
92,
91,
88,
84,
80,
71,
66,
65,
66,
66,
72,
75,
77,
85,
85,
85,
85,
84,
84,
85,
85,
85,
86,
87,
87,
87,
87,
86,
87,
87,
86,
86,
84,
82,
80,
79,
77,
77,
78,
79,
79,
78,
77,
77,
76,
76,
76,
75,
75,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
74,
73,
73,
73,
72,
72,
72,
73,
73,
73,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
66,
65,
65,
65,
65,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
64,
67,
73,
81,
88,
92,
93,
94,
94,
95,
96,
96,
96,
96,
97,
98,
98,
100,
99,
99,
98,
96,
94,
91,
88,
85,
78,
72,
68,
66,
65,
65,
71,
76,
80,
82,
86,
86,
86,
85,
85,
85,
85,
85,
86,
86,
87,
87,
87,
87,
87,
87,
87,
86,
86,
84,
82,
81,
79,
79,
79,
79,
79,
79,
78,
77,
77,
76,
76,
75,
75,
75,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
76,
76,
76,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
75,
75,
74,
73,
73,
73,
72,
72,
73,
73,
74,
73,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
66,
65,
65,
65,
66,
65,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
66,
66,
65,
65,
65,
65,
65,
66,
66,
67,
77,
85,
91,
92,
93,
94,
94,
94,
95,
95,
96,
96,
97,
99,
99,
100,
99,
98,
96,
94,
91,
84,
76,
72,
70,
67,
65,
66,
67,
74,
80,
84,
87,
86,
86,
86,
85,
85,
85,
85,
86,
86,
87,
87,
87,
88,
88,
88,
87,
87,
86,
85,
84,
83,
81,
80,
80,
79,
79,
79,
78,
78,
77,
77,
76,
76,
75,
75,
75,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
76,
76,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
75,
74,
73,
73,
73,
73,
73,
73,
74,
74,
73,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
72,
81,
88,
91,
92,
93,
93,
93,
93,
94,
95,
96,
97,
99,
100,
101,
101,
99,
97,
94,
91,
84,
74,
71,
69,
67,
67,
67,
70,
77,
82,
86,
87,
87,
86,
86,
85,
85,
85,
85,
86,
86,
87,
87,
88,
88,
89,
89,
88,
87,
86,
85,
84,
83,
81,
80,
79,
79,
79,
78,
78,
78,
77,
77,
76,
76,
75,
75,
75,
75,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
76,
76,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
75,
74,
73,
73,
73,
73,
73,
74,
75,
75,
73,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
67,
67,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
66,
65,
64,
64,
64,
64,
64,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
66,
66,
66,
67,
77,
85,
90,
92,
93,
91,
91,
92,
93,
95,
96,
97,
99,
100,
102,
102,
101,
98,
95,
93,
84,
73,
71,
69,
68,
68,
68,
72,
77,
82,
86,
87,
87,
87,
86,
85,
85,
85,
86,
86,
87,
87,
88,
88,
89,
89,
90,
89,
89,
87,
86,
85,
84,
82,
79,
79,
79,
79,
79,
79,
79,
77,
76,
76,
76,
75,
75,
75,
75,
75,
76,
76,
76,
76,
77,
77,
78,
77,
77,
77,
77,
77,
76,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
75,
73,
73,
73,
73,
73,
74,
76,
76,
75,
74,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
65,
64,
64,
65,
65,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
68,
68,
67,
66,
66,
67,
72,
80,
88,
92,
92,
91,
91,
91,
93,
94,
96,
97,
99,
100,
102,
104,
103,
98,
95,
92,
84,
73,
71,
70,
69,
69,
70,
72,
76,
80,
84,
87,
88,
87,
87,
86,
86,
85,
86,
86,
87,
88,
88,
88,
89,
90,
90,
91,
90,
88,
86,
85,
85,
84,
80,
79,
79,
79,
80,
80,
79,
77,
76,
76,
76,
75,
75,
75,
75,
76,
76,
76,
76,
77,
77,
77,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
75,
73,
73,
73,
73,
74,
75,
76,
76,
75,
74,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
65,
65,
64,
62,
62,
64,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
68,
68,
80,
75,
66,
66,
66,
67,
75,
85,
91,
91,
90,
90,
91,
92,
94,
96,
97,
98,
100,
102,
104,
102,
98,
96,
92,
85,
74,
72,
71,
70,
70,
71,
71,
73,
76,
79,
83,
89,
88,
87,
87,
86,
86,
86,
87,
87,
88,
88,
89,
89,
90,
91,
91,
91,
88,
85,
85,
86,
83,
80,
79,
79,
79,
80,
80,
79,
77,
76,
75,
75,
75,
75,
75,
75,
75,
76,
76,
76,
77,
77,
78,
78,
78,
78,
78,
78,
77,
78,
78,
77,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
74,
73,
73,
73,
74,
74,
75,
76,
77,
76,
74,
72,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
65,
65,
64,
60,
64,
64,
65,
65,
65,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
68,
67,
77,
88,
86,
69,
67,
66,
67,
73,
85,
89,
90,
89,
90,
91,
92,
94,
95,
97,
98,
100,
102,
101,
100,
97,
95,
94,
86,
74,
72,
72,
71,
71,
72,
72,
72,
73,
73,
76,
90,
89,
88,
88,
87,
87,
87,
87,
88,
88,
88,
89,
89,
90,
90,
92,
87,
86,
85,
85,
85,
83,
80,
79,
79,
80,
80,
80,
78,
76,
76,
75,
75,
75,
75,
75,
75,
75,
76,
76,
77,
77,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
73,
73,
73,
74,
74,
75,
76,
76,
77,
76,
75,
72,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
65,
65,
64,
64,
64,
65,
65,
64,
64,
64,
64,
64,
64,
64,
64,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
68,
67,
77,
86,
91,
89,
79,
66,
66,
66,
72,
84,
87,
88,
89,
90,
91,
92,
94,
95,
96,
97,
99,
100,
100,
99,
97,
97,
98,
88,
74,
72,
72,
71,
72,
73,
73,
72,
72,
72,
74,
91,
90,
90,
89,
88,
88,
88,
88,
88,
88,
88,
88,
89,
88,
91,
86,
85,
85,
85,
84,
84,
82,
80,
80,
81,
80,
80,
79,
78,
76,
76,
75,
75,
75,
75,
75,
75,
75,
76,
76,
76,
77,
78,
78,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
75,
73,
73,
73,
74,
75,
75,
76,
76,
77,
77,
75,
73,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
65,
65,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
69,
67,
75,
90,
93,
92,
91,
85,
69,
66,
66,
71,
81,
86,
88,
89,
90,
92,
93,
94,
95,
96,
97,
98,
99,
100,
98,
97,
98,
97,
91,
76,
74,
71,
73,
72,
76,
78,
76,
72,
72,
72,
92,
91,
91,
90,
89,
88,
88,
88,
89,
89,
89,
89,
88,
87,
85,
85,
86,
85,
85,
85,
83,
83,
82,
81,
80,
79,
79,
78,
77,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
76,
76,
77,
78,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
74,
73,
73,
74,
74,
75,
76,
76,
76,
76,
76,
76,
74,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
65,
65,
65,
65,
65,
64,
64,
64,
64,
64,
64,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
69,
68,
81,
94,
97,
96,
94,
92,
87,
73,
66,
67,
72,
82,
86,
88,
90,
91,
92,
93,
94,
94,
94,
95,
96,
98,
98,
98,
98,
99,
102,
97,
84,
73,
72,
71,
73,
77,
78,
80,
73,
72,
72,
92,
91,
91,
90,
90,
89,
89,
89,
89,
90,
90,
90,
90,
89,
86,
88,
87,
86,
86,
86,
85,
84,
82,
80,
80,
79,
78,
78,
77,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
76,
76,
77,
78,
79,
82,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
75,
74,
73,
74,
74,
75,
75,
76,
76,
76,
76,
76,
76,
75,
73,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
65,
65,
65,
65,
65,
65,
64,
64,
64,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
68,
67,
82,
94,
101,
100,
97,
95,
93,
88,
76,
69,
68,
73,
84,
86,
88,
90,
91,
92,
93,
93,
93,
93,
93,
93,
94,
95,
95,
97,
100,
105,
102,
92,
83,
73,
72,
73,
77,
79,
79,
75,
73,
72,
92,
91,
90,
90,
89,
89,
89,
89,
89,
90,
90,
90,
91,
91,
91,
90,
89,
88,
87,
86,
86,
85,
82,
80,
79,
79,
78,
77,
77,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
76,
76,
77,
78,
79,
82,
81,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
75,
74,
74,
75,
75,
76,
75,
76,
76,
76,
76,
77,
76,
75,
73,
72,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
65,
65,
65,
65,
64,
64,
65,
65,
65,
66,
66,
66,
66,
66,
66,
67,
67,
67,
66,
66,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
69,
74,
88,
98,
105,
103,
99,
96,
93,
88,
79,
71,
71,
74,
82,
86,
88,
89,
91,
92,
92,
92,
92,
91,
91,
90,
91,
92,
94,
97,
100,
103,
99,
89,
80,
72,
72,
73,
77,
81,
79,
80,
74,
74,
91,
91,
90,
89,
89,
89,
89,
89,
89,
89,
90,
91,
91,
92,
92,
91,
90,
89,
88,
87,
86,
85,
81,
79,
79,
78,
78,
77,
77,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
76,
77,
78,
79,
81,
81,
79,
79,
80,
80,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
75,
75,
75,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
76,
74,
73,
72,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
65,
65,
65,
65,
65,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
68,
77,
89,
101,
104,
104,
100,
97,
94,
91,
84,
74,
72,
72,
79,
86,
87,
88,
89,
90,
91,
91,
90,
89,
89,
89,
90,
92,
94,
96,
99,
99,
96,
85,
73,
73,
72,
72,
74,
76,
78,
79,
77,
76,
91,
90,
90,
89,
89,
89,
89,
89,
89,
90,
90,
91,
91,
93,
94,
92,
91,
90,
89,
88,
86,
84,
81,
79,
79,
79,
78,
77,
77,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
76,
77,
78,
79,
80,
82,
82,
80,
79,
82,
81,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
75,
75,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
74,
73,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
72,
80,
91,
103,
106,
105,
102,
99,
96,
93,
89,
80,
72,
73,
76,
82,
84,
85,
85,
86,
87,
88,
88,
88,
87,
88,
89,
91,
93,
95,
97,
97,
96,
91,
81,
72,
72,
72,
72,
73,
73,
73,
73,
78,
91,
90,
90,
89,
89,
89,
89,
89,
89,
89,
90,
91,
92,
94,
95,
94,
92,
91,
89,
87,
85,
84,
81,
80,
79,
79,
78,
78,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
78,
80,
81,
82,
82,
82,
80,
79,
82,
82,
80,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
76,
74,
73,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
67,
67,
68,
68,
69,
79,
89,
101,
110,
109,
107,
104,
101,
98,
95,
92,
84,
72,
72,
72,
75,
78,
78,
78,
78,
80,
85,
86,
86,
86,
86,
88,
90,
92,
95,
97,
97,
98,
99,
86,
75,
73,
74,
73,
73,
73,
73,
73,
73,
90,
90,
89,
89,
89,
89,
89,
89,
89,
90,
90,
91,
91,
95,
97,
95,
93,
91,
88,
86,
85,
83,
82,
81,
79,
79,
78,
78,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
77,
78,
79,
81,
83,
83,
83,
82,
80,
79,
82,
85,
81,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
77,
77,
77,
75,
73,
73,
73,
73,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
67,
67,
67,
67,
70,
75,
87,
98,
110,
111,
110,
108,
106,
104,
101,
97,
94,
88,
75,
73,
72,
73,
79,
81,
82,
83,
85,
85,
86,
86,
86,
86,
87,
90,
92,
95,
97,
99,
100,
100,
88,
84,
84,
77,
76,
74,
77,
73,
73,
73,
90,
90,
90,
90,
89,
89,
89,
89,
89,
90,
90,
91,
92,
95,
98,
96,
93,
91,
88,
86,
84,
83,
82,
81,
80,
79,
78,
78,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
77,
78,
79,
81,
82,
82,
83,
83,
80,
80,
82,
85,
82,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
74,
73,
74,
73,
73,
73,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
67,
67,
68,
69,
67,
81,
93,
103,
112,
115,
113,
110,
108,
106,
103,
99,
95,
90,
77,
73,
73,
78,
83,
86,
85,
86,
86,
86,
86,
87,
87,
87,
88,
91,
93,
95,
98,
101,
102,
102,
95,
97,
98,
91,
86,
81,
85,
81,
73,
73,
90,
90,
90,
90,
89,
89,
89,
90,
89,
90,
90,
90,
91,
94,
97,
96,
91,
91,
88,
86,
84,
84,
83,
81,
80,
79,
79,
78,
78,
77,
77,
77,
77,
76,
76,
76,
76,
76,
77,
77,
78,
79,
80,
81,
82,
82,
80,
79,
81,
83,
82,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
74,
75,
75,
74,
73,
73,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
67,
66,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
67,
67,
68,
68,
71,
77,
90,
102,
111,
114,
115,
116,
115,
111,
108,
105,
101,
96,
92,
79,
74,
73,
76,
84,
85,
87,
86,
87,
87,
87,
87,
88,
88,
89,
91,
93,
96,
98,
101,
104,
105,
104,
105,
104,
99,
90,
85,
91,
88,
78,
77,
90,
90,
90,
90,
89,
90,
90,
90,
90,
90,
90,
90,
91,
93,
96,
96,
93,
92,
89,
86,
85,
85,
84,
81,
80,
79,
79,
78,
78,
78,
77,
77,
77,
77,
76,
76,
77,
77,
77,
77,
78,
79,
79,
79,
79,
79,
79,
79,
80,
82,
81,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
74,
73,
73,
73,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
66,
67,
67,
70,
71,
83,
96,
109,
112,
115,
116,
116,
115,
114,
109,
106,
102,
97,
93,
80,
73,
73,
73,
78,
84,
86,
87,
87,
87,
88,
88,
88,
89,
90,
91,
94,
97,
98,
102,
104,
105,
105,
104,
104,
98,
91,
88,
98,
96,
87,
85,
91,
90,
90,
89,
89,
89,
90,
90,
91,
91,
90,
90,
91,
92,
96,
97,
95,
91,
89,
87,
86,
85,
84,
81,
80,
79,
79,
79,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
78,
78,
78,
78,
78,
78,
79,
80,
81,
80,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
77,
76,
74,
73,
73,
73,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
68,
68,
68,
67,
86,
98,
103,
113,
113,
114,
115,
115,
114,
111,
109,
106,
103,
98,
93,
87,
82,
76,
73,
73,
77,
84,
86,
86,
87,
88,
88,
89,
89,
90,
91,
95,
98,
99,
101,
103,
104,
105,
103,
101,
97,
90,
91,
98,
100,
97,
92,
91,
90,
89,
89,
89,
89,
90,
91,
91,
91,
90,
91,
91,
93,
96,
98,
93,
90,
88,
86,
85,
84,
84,
82,
81,
80,
79,
79,
79,
78,
78,
78,
78,
78,
77,
78,
78,
78,
78,
78,
77,
78,
78,
78,
78,
78,
79,
79,
80,
80,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
77,
76,
74,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
66,
66,
67,
68,
69,
71,
94,
110,
113,
114,
113,
113,
112,
112,
111,
109,
108,
106,
103,
100,
98,
94,
91,
86,
79,
73,
73,
78,
85,
86,
87,
87,
89,
89,
90,
91,
91,
96,
98,
99,
101,
102,
102,
97,
97,
97,
96,
94,
97,
101,
104,
104,
98,
91,
90,
89,
90,
89,
90,
90,
91,
91,
91,
91,
91,
91,
93,
97,
96,
93,
91,
88,
86,
85,
84,
83,
82,
81,
80,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
78,
78,
78,
77,
77,
77,
77,
77,
76,
77,
77,
76,
74,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
67,
67,
67,
67,
71,
78,
99,
116,
117,
115,
113,
111,
109,
109,
109,
109,
108,
107,
104,
103,
101,
98,
95,
92,
86,
79,
73,
73,
79,
85,
86,
88,
89,
90,
91,
92,
93,
96,
98,
99,
101,
102,
101,
98,
97,
97,
97,
97,
99,
103,
104,
103,
101,
91,
90,
90,
90,
90,
90,
91,
91,
91,
91,
91,
91,
91,
93,
93,
94,
93,
91,
87,
86,
85,
84,
83,
82,
82,
81,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
76,
76,
76,
76,
77,
77,
77,
77,
77,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
76,
74,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
67,
67,
68,
68,
67,
85,
107,
118,
117,
115,
112,
110,
107,
105,
106,
108,
107,
107,
106,
105,
104,
101,
98,
95,
90,
84,
74,
75,
76,
84,
86,
88,
90,
91,
92,
93,
94,
96,
97,
99,
100,
101,
102,
101,
100,
100,
100,
100,
102,
104,
105,
105,
99,
91,
90,
90,
90,
90,
90,
91,
91,
92,
91,
91,
91,
91,
92,
93,
93,
92,
90,
86,
85,
85,
85,
84,
83,
82,
81,
81,
80,
79,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
76,
76,
76,
77,
77,
77,
77,
77,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
76,
74,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
66,
67,
67,
67,
70,
74,
96,
107,
110,
111,
111,
110,
107,
105,
104,
104,
106,
107,
106,
106,
105,
105,
104,
100,
97,
93,
87,
78,
75,
77,
85,
87,
88,
90,
91,
93,
93,
95,
96,
98,
99,
100,
102,
103,
103,
103,
103,
103,
103,
103,
104,
105,
104,
97,
91,
90,
90,
90,
90,
91,
91,
92,
92,
91,
91,
91,
92,
92,
92,
91,
90,
88,
86,
86,
86,
85,
84,
83,
83,
82,
81,
81,
81,
80,
80,
79,
79,
79,
79,
79,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
78,
79,
79,
79,
79,
79,
78,
78,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
76,
76,
77,
77,
77,
77,
77,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
76,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
67,
67,
67,
68,
68,
86,
103,
105,
106,
107,
107,
106,
105,
104,
103,
104,
106,
106,
105,
105,
105,
105,
103,
102,
98,
94,
89,
80,
75,
75,
85,
87,
88,
90,
91,
93,
94,
95,
97,
98,
100,
101,
103,
104,
105,
105,
105,
105,
105,
105,
106,
105,
105,
98,
91,
91,
90,
90,
90,
91,
91,
92,
92,
92,
92,
92,
92,
93,
92,
91,
90,
88,
88,
88,
87,
86,
85,
84,
83,
82,
82,
82,
81,
81,
80,
80,
79,
79,
79,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
78,
78,
78,
78,
78,
78,
78,
78,
78,
79,
79,
78,
79,
78,
78,
77,
77,
77,
76,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
66,
66,
67,
68,
68,
67,
77,
95,
99,
100,
101,
102,
102,
102,
101,
101,
102,
104,
106,
105,
105,
104,
103,
103,
104,
101,
98,
94,
89,
80,
75,
77,
86,
87,
89,
90,
91,
92,
94,
95,
96,
97,
100,
101,
103,
105,
106,
107,
107,
108,
108,
108,
107,
105,
103,
92,
91,
91,
90,
91,
91,
91,
92,
93,
92,
93,
93,
93,
93,
92,
92,
91,
90,
89,
89,
90,
91,
86,
85,
84,
83,
83,
82,
82,
82,
81,
81,
80,
79,
79,
79,
80,
81,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
77,
77,
78,
78,
78,
78,
78,
78,
78,
78,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
77,
77,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
68,
67,
67,
67,
67,
67,
67,
67,
66,
67,
67,
67,
70,
72,
91,
93,
96,
97,
97,
98,
98,
97,
97,
99,
103,
105,
105,
106,
105,
104,
103,
102,
101,
100,
97,
94,
88,
80,
75,
78,
86,
89,
91,
91,
91,
92,
94,
96,
97,
98,
100,
102,
103,
106,
108,
108,
109,
110,
110,
110,
109,
106,
95,
85,
91,
91,
91,
91,
91,
91,
92,
94,
94,
95,
95,
94,
94,
92,
91,
90,
90,
90,
91,
92,
91,
86,
85,
84,
84,
83,
83,
83,
83,
82,
81,
80,
79,
79,
79,
81,
81,
81,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
77,
77,
75,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
68,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
69,
88,
93,
93,
94,
94,
95,
96,
96,
96,
97,
100,
103,
105,
105,
105,
105,
104,
103,
102,
100,
99,
97,
93,
88,
81,
78,
77,
85,
91,
92,
92,
92,
92,
94,
96,
98,
99,
101,
102,
103,
108,
110,
109,
110,
110,
111,
110,
110,
105,
94,
82,
91,
91,
91,
91,
91,
92,
94,
95,
96,
96,
95,
95,
93,
92,
91,
91,
91,
92,
92,
92,
90,
86,
85,
84,
84,
84,
84,
85,
84,
83,
82,
80,
79,
80,
80,
81,
81,
81,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
79,
78,
78,
79,
78,
78,
79,
78,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
77,
77,
76,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
71,
70,
70,
70,
70,
70,
70,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
69,
69,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
67,
69,
75,
86,
91,
92,
92,
92,
93,
95,
96,
97,
98,
100,
103,
105,
105,
105,
104,
103,
102,
102,
100,
98,
97,
95,
91,
84,
78,
79,
82,
90,
91,
93,
93,
93,
95,
97,
98,
100,
102,
103,
106,
110,
111,
110,
111,
111,
111,
112,
109,
103,
90,
81,
91,
91,
91,
91,
93,
94,
95,
96,
97,
98,
96,
95,
93,
91,
91,
91,
92,
93,
93,
92,
91,
88,
85,
85,
84,
85,
85,
86,
85,
84,
83,
82,
81,
81,
81,
81,
81,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
77,
76,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
68,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
67,
67,
68,
68,
67,
69,
68,
72,
72,
82,
88,
89,
91,
94,
97,
98,
98,
98,
97,
101,
104,
104,
104,
104,
103,
102,
102,
101,
99,
99,
98,
95,
88,
80,
79,
79,
86,
92,
93,
93,
94,
96,
97,
99,
101,
103,
105,
108,
110,
112,
111,
111,
112,
112,
112,
111,
103,
91,
82,
91,
91,
91,
93,
95,
97,
97,
97,
99,
98,
97,
95,
93,
91,
91,
92,
93,
94,
94,
94,
92,
90,
87,
85,
85,
86,
86,
87,
86,
85,
84,
83,
83,
82,
82,
82,
81,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
79,
79,
79,
78,
78,
78,
77,
76,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
69,
69,
68,
68,
68,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
67,
67,
67,
67,
68,
69,
69,
67,
81,
91,
81,
74,
83,
88,
92,
95,
98,
98,
98,
98,
96,
98,
101,
103,
103,
103,
103,
103,
102,
102,
101,
101,
100,
98,
91,
84,
79,
80,
83,
91,
92,
93,
95,
96,
97,
99,
101,
104,
106,
108,
110,
112,
112,
112,
112,
112,
111,
109,
105,
92,
84,
91,
92,
93,
95,
97,
99,
99,
100,
100,
98,
96,
95,
93,
92,
92,
92,
94,
95,
96,
96,
94,
92,
89,
87,
87,
88,
88,
88,
86,
85,
84,
84,
84,
84,
84,
82,
81,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
80,
80,
80,
79,
79,
79,
78,
78,
78,
77,
77,
75,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
67,
67,
68,
67,
69,
69,
72,
76,
92,
89,
93,
84,
79,
85,
91,
93,
96,
98,
98,
96,
95,
97,
98,
100,
102,
103,
104,
104,
103,
103,
103,
103,
103,
100,
96,
87,
83,
81,
83,
86,
92,
93,
94,
96,
97,
99,
101,
103,
106,
108,
110,
112,
112,
112,
112,
112,
111,
109,
105,
96,
88,
93,
93,
94,
97,
99,
100,
101,
103,
104,
99,
97,
95,
93,
93,
92,
93,
95,
96,
97,
98,
96,
93,
91,
90,
90,
91,
91,
90,
87,
85,
85,
85,
85,
85,
85,
83,
81,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
78,
78,
78,
77,
77,
76,
77,
75,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
68,
68,
68,
68,
68,
67,
67,
69,
69,
72,
73,
89,
90,
93,
92,
92,
92,
85,
86,
91,
93,
94,
94,
93,
95,
96,
98,
101,
103,
105,
107,
107,
106,
105,
105,
105,
105,
103,
98,
92,
84,
82,
82,
83,
91,
93,
95,
96,
97,
98,
100,
103,
105,
108,
110,
111,
112,
112,
112,
111,
111,
109,
106,
103,
95,
95,
95,
96,
97,
101,
102,
102,
104,
104,
102,
97,
96,
93,
92,
92,
94,
95,
96,
98,
98,
97,
94,
91,
91,
92,
92,
92,
91,
89,
86,
85,
86,
87,
86,
85,
83,
81,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
79,
79,
79,
79,
79,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
78,
78,
78,
78,
77,
79,
80,
75,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
68,
68,
68,
68,
67,
70,
73,
73,
89,
90,
93,
92,
94,
93,
92,
91,
85,
88,
91,
91,
91,
93,
94,
96,
98,
101,
104,
108,
110,
111,
109,
108,
107,
107,
106,
104,
99,
94,
85,
84,
83,
84,
86,
92,
95,
97,
98,
97,
98,
102,
104,
108,
110,
112,
112,
112,
112,
112,
110,
109,
108,
105,
101,
96,
96,
97,
100,
104,
104,
102,
101,
102,
102,
99,
97,
93,
92,
92,
94,
96,
97,
98,
98,
97,
95,
93,
93,
93,
93,
93,
92,
90,
87,
87,
88,
89,
86,
85,
83,
82,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
78,
78,
78,
78,
79,
79,
80,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
70,
69,
77,
92,
95,
95,
95,
94,
94,
94,
93,
92,
89,
89,
90,
91,
91,
92,
93,
96,
98,
100,
104,
108,
109,
111,
111,
109,
109,
108,
107,
104,
99,
94,
87,
85,
84,
84,
84,
91,
95,
98,
99,
99,
99,
102,
103,
107,
110,
111,
112,
112,
112,
112,
111,
111,
110,
107,
104,
99,
97,
100,
104,
104,
103,
99,
97,
97,
98,
98,
97,
95,
94,
93,
94,
96,
98,
98,
98,
97,
95,
94,
94,
94,
93,
92,
91,
90,
90,
90,
91,
91,
88,
86,
84,
82,
81,
80,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
78,
78,
79,
79,
79,
78,
75,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
70,
70,
70,
70,
70,
70,
70,
69,
69,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
74,
77,
93,
99,
98,
97,
96,
96,
95,
94,
93,
91,
90,
91,
92,
92,
91,
91,
92,
95,
97,
99,
101,
105,
109,
110,
112,
114,
114,
111,
108,
104,
98,
93,
89,
88,
86,
84,
85,
85,
93,
98,
99,
100,
101,
103,
104,
107,
110,
111,
112,
112,
112,
112,
112,
111,
110,
107,
104,
102,
103,
104,
105,
104,
102,
98,
97,
97,
97,
96,
96,
98,
96,
95,
95,
96,
98,
98,
97,
96,
95,
95,
95,
95,
94,
92,
91,
91,
91,
91,
92,
92,
89,
86,
84,
82,
81,
80,
80,
80,
80,
80,
79,
79,
79,
79,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
76,
75,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
72,
74,
74,
93,
104,
103,
99,
98,
97,
96,
95,
94,
92,
91,
91,
92,
93,
94,
94,
94,
94,
95,
96,
96,
97,
100,
103,
107,
112,
116,
115,
113,
108,
104,
98,
93,
90,
92,
91,
87,
85,
86,
88,
96,
98,
99,
102,
104,
106,
107,
109,
111,
112,
112,
113,
112,
112,
110,
108,
105,
100,
105,
105,
106,
105,
103,
101,
99,
98,
98,
97,
96,
97,
98,
97,
96,
96,
96,
97,
98,
97,
97,
96,
95,
96,
96,
96,
92,
92,
92,
93,
92,
93,
92,
89,
86,
84,
83,
82,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
78,
76,
75,
75,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
69,
69,
69,
69,
69,
69,
69,
69,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
71,
72,
72,
73,
73,
90,
103,
102,
103,
101,
99,
97,
96,
95,
93,
92,
91,
91,
93,
95,
96,
97,
97,
97,
97,
97,
97,
97,
97,
100,
105,
110,
114,
117,
114,
108,
104,
98,
95,
92,
93,
92,
91,
85,
86,
86,
91,
97,
99,
101,
103,
105,
107,
109,
110,
112,
112,
112,
112,
111,
109,
107,
102,
97,
107,
108,
108,
105,
103,
102,
101,
99,
99,
98,
97,
97,
97,
96,
96,
96,
97,
97,
98,
98,
97,
96,
95,
97,
98,
98,
95,
93,
94,
94,
94,
93,
92,
89,
85,
83,
82,
82,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
80,
79,
77,
76,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
70,
69,
69,
69,
69,
69,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
71,
72,
72,
72,
73,
73,
83,
97,
101,
103,
103,
101,
99,
98,
97,
96,
93,
92,
91,
91,
93,
96,
98,
99,
99,
100,
100,
100,
99,
97,
98,
99,
102,
105,
110,
114,
112,
108,
104,
99,
96,
94,
94,
93,
90,
85,
86,
90,
92,
94,
97,
100,
102,
104,
106,
108,
110,
111,
113,
112,
112,
112,
110,
108,
103,
102,
110,
110,
109,
107,
105,
104,
102,
101,
103,
99,
97,
97,
97,
96,
96,
96,
96,
97,
98,
97,
97,
96,
96,
97,
99,
100,
97,
96,
96,
95,
94,
92,
91,
87,
85,
83,
82,
82,
81,
81,
81,
82,
82,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
80,
80,
79,
79,
77,
76,
76,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
69,
69,
70,
70,
70,
70,
70,
70,
71,
71,
71,
71,
71,
71,
71,
72,
72,
72,
72,
72,
74,
76,
92,
104,
103,
104,
103,
102,
101,
99,
98,
96,
94,
92,
91,
92,
95,
96,
98,
100,
102,
103,
104,
104,
104,
102,
100,
99,
101,
103,
108,
110,
109,
106,
103,
99,
97,
95,
95,
93,
89,
86,
86,
93,
97,
98,
99,
100,
102,
103,
105,
108,
109,
110,
113,
113,
113,
111,
110,
107,
103,
104,
114,
113,
110,
107,
105,
104,
102,
103,
103,
101,
97,
97,
97,
96,
96,
96,
96,
97,
98,
98,
97,
97,
97,
98,
101,
102,
99,
98,
96,
95,
93,
92,
90,
87,
85,
83,
82,
82,
81,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
77,
76,
76,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
71,
71,
71,
71,
72,
72,
72,
72,
72,
73,
73,
85,
101,
103,
105,
105,
104,
104,
102,
101,
99,
98,
95,
92,
91,
95,
97,
98,
98,
100,
103,
104,
106,
106,
107,
106,
104,
103,
102,
103,
104,
104,
104,
103,
99,
97,
96,
96,
97,
96,
91,
86,
86,
94,
99,
101,
102,
102,
103,
105,
107,
109,
110,
112,
113,
113,
112,
110,
108,
106,
103,
108,
116,
116,
110,
106,
104,
103,
102,
104,
103,
103,
100,
98,
97,
97,
96,
96,
97,
97,
98,
98,
97,
97,
97,
99,
105,
104,
103,
98,
96,
95,
93,
92,
90,
87,
85,
83,
82,
82,
82,
82,
82,
83,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
78,
77,
76,
76,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
71,
71,
72,
72,
72,
72,
72,
72,
73,
75,
79,
93,
105,
105,
105,
105,
105,
104,
104,
103,
101,
99,
96,
92,
91,
98,
99,
100,
100,
101,
102,
104,
106,
108,
110,
110,
108,
106,
105,
104,
103,
102,
102,
101,
98,
97,
97,
98,
98,
98,
91,
86,
86,
93,
99,
103,
105,
106,
107,
108,
110,
110,
111,
113,
115,
115,
113,
110,
109,
107,
108,
111,
116,
114,
108,
105,
103,
102,
103,
104,
105,
104,
104,
100,
98,
97,
97,
96,
97,
97,
98,
99,
98,
97,
98,
101,
104,
104,
103,
98,
96,
95,
93,
92,
90,
88,
86,
84,
83,
82,
82,
83,
83,
83,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
81,
81,
81,
80,
80,
81,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
77,
76,
76,
76,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
71,
71,
71,
72,
72,
72,
72,
72,
73,
74,
73,
88,
103,
104,
105,
105,
106,
106,
105,
104,
104,
103,
100,
97,
93,
93,
98,
99,
101,
102,
102,
103,
103,
106,
108,
110,
114,
114,
110,
108,
106,
104,
103,
102,
103,
103,
103,
102,
100,
99,
96,
89,
86,
86,
91,
99,
104,
106,
109,
110,
110,
110,
112,
113,
115,
116,
116,
115,
115,
112,
109,
110,
114,
116,
111,
107,
104,
103,
103,
103,
104,
105,
105,
104,
103,
99,
98,
97,
97,
97,
98,
100,
99,
98,
97,
98,
102,
104,
105,
100,
97,
96,
95,
93,
92,
91,
88,
87,
85,
83,
82,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
81,
80,
80,
80,
80,
79,
77,
76,
76,
76,
76,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
71,
71,
72,
72,
72,
72,
72,
72,
73,
74,
73,
83,
97,
102,
105,
105,
106,
106,
106,
105,
105,
104,
104,
102,
98,
95,
95,
97,
99,
101,
104,
105,
106,
107,
108,
108,
110,
114,
115,
116,
111,
108,
105,
103,
104,
106,
107,
107,
106,
103,
99,
93,
86,
86,
87,
94,
100,
104,
107,
110,
111,
111,
111,
112,
114,
115,
116,
117,
117,
116,
115,
111,
113,
116,
114,
109,
107,
106,
104,
104,
104,
104,
105,
105,
105,
103,
99,
97,
97,
97,
98,
101,
101,
100,
98,
97,
97,
101,
104,
101,
97,
97,
96,
95,
93,
92,
90,
89,
87,
84,
82,
83,
84,
84,
84,
83,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
78,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
71,
71,
72,
72,
72,
72,
72,
72,
73,
74,
76,
77,
92,
104,
104,
105,
105,
106,
106,
106,
106,
105,
105,
105,
104,
99,
97,
95,
96,
97,
99,
104,
107,
110,
110,
110,
108,
109,
112,
116,
118,
113,
108,
106,
106,
108,
110,
111,
112,
110,
105,
98,
90,
86,
86,
88,
96,
101,
104,
107,
110,
111,
112,
112,
113,
114,
115,
116,
117,
116,
117,
116,
114,
115,
116,
113,
109,
108,
107,
106,
105,
105,
105,
106,
105,
104,
102,
99,
98,
99,
100,
102,
104,
103,
101,
98,
97,
97,
98,
100,
98,
97,
96,
96,
95,
93,
92,
92,
90,
87,
84,
82,
84,
85,
85,
84,
83,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
78,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
71,
72,
72,
72,
72,
72,
73,
73,
75,
73,
79,
90,
104,
104,
105,
106,
106,
106,
106,
106,
106,
106,
106,
106,
104,
102,
97,
96,
95,
95,
97,
101,
105,
110,
109,
110,
107,
107,
108,
110,
112,
111,
108,
108,
109,
111,
113,
116,
117,
114,
108,
102,
92,
86,
86,
88,
96,
101,
104,
106,
108,
110,
111,
111,
112,
113,
115,
116,
117,
117,
117,
116,
116,
116,
117,
114,
111,
111,
109,
108,
107,
106,
106,
106,
105,
103,
103,
102,
102,
102,
103,
104,
104,
103,
100,
98,
97,
97,
98,
98,
98,
97,
96,
95,
95,
94,
93,
92,
91,
87,
85,
84,
86,
87,
85,
84,
83,
83,
83,
83,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
82,
82,
82,
81,
79,
77,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
72,
72,
72,
72,
72,
73,
73,
75,
76,
87,
96,
104,
104,
106,
106,
106,
106,
106,
106,
106,
107,
107,
107,
106,
105,
104,
99,
97,
96,
95,
96,
97,
101,
104,
106,
106,
105,
104,
105,
106,
108,
108,
108,
108,
110,
112,
115,
116,
117,
115,
109,
105,
100,
89,
87,
86,
93,
99,
102,
105,
106,
108,
108,
110,
111,
113,
114,
115,
116,
117,
117,
116,
117,
117,
117,
115,
116,
114,
110,
109,
109,
108,
107,
107,
106,
105,
105,
105,
104,
104,
105,
105,
104,
102,
101,
98,
97,
97,
97,
98,
98,
97,
96,
96,
95,
95,
94,
93,
91,
87,
85,
85,
87,
87,
85,
85,
84,
84,
83,
83,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
82,
82,
81,
81,
82,
82,
82,
82,
82,
82,
80,
77,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
72,
72,
72,
72,
73,
73,
73,
77,
87,
98,
108,
109,
107,
107,
107,
107,
107,
106,
107,
107,
107,
107,
107,
106,
105,
105,
102,
99,
97,
95,
95,
96,
97,
99,
101,
102,
103,
103,
104,
104,
106,
107,
109,
108,
110,
113,
116,
117,
117,
112,
108,
106,
102,
98,
86,
86,
88,
95,
99,
102,
104,
104,
104,
107,
110,
112,
114,
115,
116,
116,
117,
117,
118,
116,
115,
116,
117,
116,
111,
110,
109,
108,
108,
108,
107,
107,
106,
106,
106,
106,
106,
105,
104,
104,
102,
99,
98,
97,
98,
98,
98,
97,
96,
96,
95,
96,
96,
94,
91,
89,
87,
88,
88,
87,
85,
85,
85,
85,
85,
83,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
81,
77,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
72,
72,
73,
73,
73,
73,
77,
83,
94,
108,
109,
110,
109,
107,
107,
106,
107,
107,
107,
107,
107,
107,
107,
107,
106,
105,
104,
101,
98,
96,
95,
95,
96,
97,
98,
100,
102,
102,
104,
105,
107,
109,
109,
109,
110,
113,
113,
114,
112,
109,
107,
104,
102,
98,
89,
86,
86,
90,
97,
99,
98,
97,
103,
108,
110,
112,
114,
115,
115,
116,
117,
118,
118,
116,
115,
119,
118,
117,
114,
110,
110,
109,
108,
109,
109,
109,
108,
108,
108,
108,
107,
106,
106,
105,
104,
102,
99,
98,
98,
98,
98,
97,
96,
96,
97,
98,
98,
96,
91,
90,
91,
90,
90,
88,
86,
85,
85,
86,
85,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
82,
77,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
72,
73,
73,
73,
74,
79,
85,
94,
109,
108,
110,
109,
108,
108,
107,
107,
106,
106,
107,
107,
108,
108,
108,
108,
107,
106,
104,
102,
99,
97,
96,
95,
96,
98,
99,
100,
102,
103,
104,
106,
109,
110,
111,
110,
111,
112,
111,
110,
109,
108,
106,
104,
103,
99,
95,
90,
88,
90,
91,
92,
97,
98,
105,
110,
112,
114,
115,
116,
116,
116,
117,
118,
117,
117,
116,
121,
120,
117,
114,
111,
111,
109,
109,
110,
111,
110,
109,
109,
109,
109,
108,
107,
106,
105,
105,
104,
100,
98,
98,
99,
98,
97,
96,
96,
97,
98,
98,
96,
93,
93,
93,
91,
91,
91,
87,
85,
86,
87,
85,
84,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
82,
78,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
72,
74,
73,
79,
85,
93,
100,
111,
108,
111,
110,
109,
108,
107,
106,
104,
104,
104,
106,
107,
108,
108,
109,
109,
108,
107,
104,
103,
102,
100,
98,
97,
96,
98,
100,
102,
103,
104,
104,
107,
110,
112,
113,
114,
113,
113,
110,
109,
108,
109,
108,
107,
106,
104,
99,
91,
90,
90,
92,
98,
100,
98,
105,
110,
112,
116,
116,
117,
116,
117,
118,
118,
118,
117,
116,
122,
122,
117,
114,
112,
111,
109,
110,
112,
112,
112,
111,
111,
110,
110,
109,
107,
104,
104,
103,
102,
99,
99,
99,
98,
98,
97,
96,
97,
98,
98,
98,
96,
96,
95,
94,
93,
93,
92,
90,
86,
87,
87,
86,
84,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
79,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
70,
72,
78,
83,
91,
100,
110,
107,
109,
110,
110,
109,
108,
107,
106,
105,
104,
103,
104,
104,
105,
108,
109,
109,
109,
109,
109,
108,
106,
105,
103,
101,
98,
98,
99,
102,
104,
105,
105,
104,
107,
108,
111,
114,
116,
116,
113,
110,
109,
110,
111,
111,
110,
108,
106,
100,
90,
91,
94,
99,
104,
109,
108,
104,
109,
112,
116,
117,
118,
118,
118,
118,
118,
117,
116,
114,
124,
122,
116,
114,
112,
110,
110,
112,
115,
114,
114,
114,
113,
112,
111,
110,
107,
104,
102,
102,
102,
101,
100,
99,
98,
97,
97,
97,
97,
98,
99,
98,
98,
98,
97,
96,
96,
95,
93,
91,
89,
89,
87,
86,
85,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
84,
84,
83,
83,
79,
77,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
70,
72,
77,
83,
91,
100,
107,
106,
110,
110,
110,
109,
109,
108,
107,
106,
105,
104,
103,
103,
103,
103,
104,
107,
109,
110,
111,
112,
112,
112,
110,
109,
107,
104,
101,
98,
100,
104,
106,
108,
108,
108,
107,
108,
111,
115,
118,
118,
115,
112,
111,
113,
115,
115,
114,
110,
107,
101,
91,
91,
97,
103,
107,
110,
110,
110,
110,
111,
115,
117,
118,
118,
118,
118,
118,
116,
112,
110,
124,
122,
116,
114,
113,
110,
110,
115,
116,
116,
116,
116,
115,
113,
112,
111,
106,
103,
103,
102,
102,
103,
102,
101,
99,
98,
97,
97,
97,
98,
100,
100,
99,
98,
98,
97,
98,
96,
94,
92,
90,
90,
88,
86,
85,
84,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
84,
84,
84,
84,
83,
79,
77,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
70,
71,
71,
75,
79,
85,
91,
99,
105,
103,
105,
107,
107,
107,
107,
107,
107,
107,
106,
105,
104,
103,
101,
102,
103,
104,
105,
108,
110,
111,
112,
113,
115,
116,
115,
112,
109,
106,
104,
99,
101,
104,
107,
110,
111,
110,
109,
109,
112,
116,
119,
121,
119,
115,
114,
116,
119,
119,
118,
113,
107,
101,
91,
91,
98,
107,
110,
111,
112,
112,
110,
110,
114,
116,
118,
118,
118,
119,
117,
115,
110,
107,
123,
118,
116,
116,
115,
111,
111,
115,
116,
117,
117,
117,
116,
114,
112,
110,
106,
103,
104,
103,
105,
104,
104,
103,
100,
99,
97,
97,
98,
99,
101,
100,
99,
97,
97,
98,
98,
97,
94,
92,
91,
92,
89,
85,
85,
84,
84,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
84,
84,
85,
85,
85,
83,
79,
77,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
75,
75,
74,
75,
75,
74,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
70,
71,
70,
70,
70,
70,
70,
70,
70,
70,
70,
75,
80,
86,
91,
98,
101,
103,
104,
104,
104,
104,
104,
104,
104,
105,
105,
105,
105,
104,
102,
99,
100,
103,
104,
105,
106,
108,
110,
111,
112,
113,
116,
117,
116,
115,
110,
107,
105,
103,
101,
103,
107,
110,
112,
114,
115,
114,
114,
115,
120,
122,
123,
117,
117,
117,
121,
123,
121,
116,
107,
98,
91,
91,
97,
109,
111,
113,
114,
115,
113,
112,
114,
116,
117,
118,
119,
117,
116,
114,
110,
105,
122,
117,
116,
116,
116,
114,
115,
116,
116,
116,
116,
117,
116,
114,
111,
109,
106,
103,
105,
106,
109,
108,
106,
104,
101,
100,
98,
97,
100,
103,
102,
101,
99,
97,
97,
98,
98,
97,
95,
93,
93,
92,
88,
85,
85,
84,
84,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
84,
84,
84,
85,
85,
85,
85,
84,
79,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
75,
82,
89,
95,
98,
98,
99,
99,
99,
99,
99,
100,
100,
101,
101,
101,
101,
100,
99,
97,
97,
101,
104,
105,
107,
108,
108,
109,
110,
110,
111,
112,
114,
114,
113,
110,
107,
105,
103,
102,
103,
106,
109,
112,
116,
117,
118,
118,
118,
121,
123,
123,
119,
117,
119,
123,
123,
121,
116,
106,
95,
93,
93,
101,
109,
112,
114,
115,
117,
116,
115,
116,
117,
118,
118,
119,
117,
115,
113,
110,
104,
121,
117,
116,
116,
115,
115,
116,
116,
116,
116,
116,
116,
116,
114,
112,
110,
106,
105,
106,
109,
111,
111,
108,
104,
102,
102,
101,
101,
103,
104,
103,
101,
99,
98,
98,
100,
98,
97,
96,
95,
93,
91,
88,
86,
85,
85,
84,
84,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
84,
84,
84,
84,
85,
85,
86,
86,
86,
85,
79,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
72,
72,
72,
73,
73,
71,
74,
80,
87,
90,
91,
91,
91,
92,
92,
93,
94,
94,
95,
96,
96,
96,
96,
97,
97,
99,
102,
105,
107,
109,
109,
109,
109,
109,
109,
109,
109,
110,
110,
110,
108,
107,
105,
104,
103,
103,
106,
108,
110,
115,
117,
119,
121,
122,
122,
124,
125,
121,
121,
123,
123,
122,
120,
116,
104,
93,
93,
94,
100,
107,
111,
115,
116,
118,
117,
117,
118,
119,
119,
118,
118,
116,
114,
111,
110,
107,
121,
119,
117,
116,
115,
115,
116,
116,
116,
116,
116,
116,
115,
113,
111,
110,
108,
108,
109,
112,
116,
113,
110,
105,
104,
104,
103,
103,
104,
105,
104,
102,
101,
101,
101,
99,
97,
96,
96,
97,
92,
90,
87,
85,
85,
85,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
84,
84,
84,
84,
84,
84,
85,
85,
85,
85,
86,
86,
86,
85,
80,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
72,
72,
72,
73,
81,
85,
83,
79,
75,
79,
79,
80,
81,
81,
81,
82,
83,
84,
86,
87,
89,
91,
92,
92,
94,
96,
98,
101,
104,
106,
108,
110,
110,
111,
110,
110,
109,
109,
109,
109,
109,
108,
107,
107,
105,
105,
104,
103,
106,
110,
111,
115,
117,
119,
121,
123,
124,
125,
129,
126,
124,
124,
123,
120,
118,
113,
101,
93,
93,
98,
102,
106,
109,
115,
117,
119,
119,
119,
120,
120,
121,
120,
118,
116,
112,
110,
108,
109,
120,
118,
117,
116,
115,
116,
117,
116,
115,
115,
116,
115,
114,
112,
111,
110,
109,
109,
113,
116,
116,
114,
110,
108,
106,
105,
103,
104,
105,
105,
104,
103,
103,
103,
103,
101,
98,
96,
95,
94,
92,
89,
86,
85,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
84,
84,
84,
84,
84,
84,
84,
85,
85,
85,
85,
85,
85,
86,
87,
87,
86,
84,
80,
78,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
71,
72,
72,
72,
72,
72,
73,
82,
95,
98,
94,
89,
84,
80,
79,
79,
79,
79,
79,
79,
79,
79,
80,
84,
85,
88,
90,
90,
93,
97,
99,
102,
104,
107,
109,
111,
111,
111,
112,
112,
111,
110,
109,
109,
110,
109,
108,
107,
106,
106,
105,
104,
106,
110,
112,
114,
116,
117,
118,
120,
123,
124,
128,
128,
126,
125,
122,
118,
114,
107,
97,
93,
93,
99,
104,
109,
111,
115,
118,
120,
120,
121,
121,
122,
121,
119,
116,
114,
111,
109,
109,
110,
120,
118,
116,
116,
116,
117,
117,
116,
116,
115,
114,
113,
113,
112,
111,
109,
110,
111,
116,
117,
117,
115,
116,
109,
108,
105,
104,
105,
106,
105,
104,
104,
104,
104,
104,
101,
98,
96,
94,
93,
91,
89,
85,
85,
85,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
84,
84,
84,
84,
84,
84,
84,
84,
84,
85,
85,
85,
85,
85,
86,
86,
87,
88,
88,
85,
83,
80,
78,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
71,
71,
71,
71,
71,
72,
72,
72,
72,
72,
72,
72,
72,
75,
77,
95,
106,
105,
102,
96,
89,
85,
84,
85,
85,
86,
87,
87,
87,
86,
84,
83,
84,
88,
91,
92,
95,
98,
100,
102,
104,
107,
110,
110,
111,
112,
114,
114,
114,
112,
110,
110,
110,
110,
109,
108,
107,
108,
107,
106,
106,
109,
110,
112,
113,
114,
115,
117,
121,
122,
123,
126,
128,
127,
121,
115,
110,
100,
93,
93,
96,
100,
103,
108,
112,
116,
120,
122,
122,
122,
123,
123,
122,
117,
115,
113,
112,
112,
110,
111,
119,
119,
119,
118,
118,
119,
118,
117,
116,
115,
114,
114,
114,
112,
110,
109,
110,
113,
116,
117,
117,
118,
115,
111,
109,
107,
106,
106,
107,
107,
106,
105,
104,
103,
103,
101,
97,
95,
93,
92,
91,
89,
87,
85,
85,
85,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
84,
84,
84,
84,
84,
84,
84,
84,
84,
85,
85,
85,
85,
85,
86,
85,
86,
87,
88,
89,
88,
85,
83,
81,
79,
78,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
71,
71,
72,
72,
72,
72,
72,
72,
72,
72,
73,
74,
73,
90,
105,
111,
110,
106,
100,
95,
93,
95,
95,
96,
97,
96,
95,
93,
92,
87,
84,
85,
91,
96,
98,
98,
98,
100,
102,
103,
106,
109,
110,
111,
112,
114,
115,
116,
115,
113,
112,
111,
111,
110,
109,
109,
110,
110,
109,
107,
108,
109,
110,
111,
113,
114,
115,
117,
120,
122,
126,
129,
130,
124,
116,
108,
99,
93,
93,
100,
105,
109,
111,
114,
117,
122,
124,
124,
125,
125,
124,
122,
118,
115,
114,
115,
116,
115,
113,
120,
120,
121,
120,
120,
119,
118,
117,
116,
115,
115,
116,
116,
113,
110,
110,
110,
113,
116,
117,
117,
118,
116,
115,
109,
108,
107,
108,
108,
108,
108,
106,
105,
104,
103,
100,
98,
96,
92,
91,
91,
90,
88,
86,
85,
85,
85,
84,
84,
84,
84,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
84,
84,
84,
84,
84,
84,
84,
84,
84,
85,
85,
85,
85,
85,
86,
86,
86,
87,
88,
88,
89,
87,
84,
83,
81,
80,
79,
78,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
73,
74,
73,
84,
100,
113,
112,
111,
109,
103,
100,
101,
102,
102,
102,
101,
100,
99,
96,
93,
87,
86,
86,
95,
99,
100,
101,
102,
102,
102,
102,
104,
108,
110,
111,
112,
113,
115,
116,
116,
115,
113,
112,
112,
111,
110,
110,
111,
111,
110,
109,
108,
109,
109,
110,
113,
114,
115,
117,
120,
124,
127,
129,
130,
129,
119,
110,
98,
93,
97,
102,
108,
111,
115,
119,
122,
123,
125,
127,
128,
127,
125,
122,
118,
115,
115,
117,
117,
116,
115,
119,
121,
122,
121,
121,
120,
119,
118,
116,
116,
115,
116,
116,
115,
111,
110,
110,
114,
117,
117,
118,
118,
117,
112,
110,
109,
109,
109,
109,
110,
108,
106,
105,
105,
103,
98,
96,
92,
91,
91,
92,
92,
89,
87,
85,
85,
85,
84,
84,
84,
84,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
83,
83,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
85,
85,
85,
85,
85,
86,
86,
87,
87,
88,
89,
89,
88,
86,
84,
82,
81,
80,
79,
78,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
74,
74,
73,
84,
100,
117,
113,
115,
113,
111,
106,
105,
106,
106,
105,
105,
104,
103,
100,
97,
94,
89,
86,
86,
96,
100,
103,
105,
106,
105,
104,
103,
103,
105,
108,
110,
111,
112,
113,
114,
114,
114,
114,
113,
113,
112,
111,
111,
113,
112,
111,
109,
108,
110,
111,
112,
114,
114,
116,
118,
121,
125,
128,
130,
130,
130,
121,
112,
99,
93,
96,
103,
108,
112,
116,
122,
124,
125,
127,
128,
129,
128,
126,
123,
119,
116,
117,
119,
119,
117,
116,
119,
121,
122,
122,
122,
121,
121,
119,
116,
116,
116,
117,
117,
116,
113,
110,
110,
115,
117,
118,
118,
117,
115,
111,
109,
109,
110,
110,
110,
110,
108,
107,
106,
104,
100,
95,
91,
91,
92,
92,
93,
92,
90,
87,
85,
85,
85,
84,
84,
84,
84,
84,
83,
83,
83,
83,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
85,
85,
85,
85,
85,
85,
86,
86,
87,
88,
88,
89,
88,
88,
86,
84,
82,
81,
80,
79,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
73,
73,
76,
81,
100,
116,
114,
116,
116,
114,
112,
110,
109,
109,
109,
108,
107,
106,
105,
102,
100,
98,
92,
86,
86,
96,
100,
103,
107,
109,
110,
109,
107,
105,
104,
107,
109,
110,
111,
112,
112,
112,
113,
114,
114,
115,
114,
114,
113,
114,
115,
113,
110,
109,
110,
112,
114,
116,
116,
117,
119,
122,
126,
129,
129,
128,
128,
121,
112,
102,
93,
93,
101,
107,
111,
115,
121,
123,
126,
128,
129,
130,
129,
127,
123,
120,
119,
120,
121,
121,
120,
116,
119,
121,
122,
122,
122,
122,
122,
120,
118,
117,
117,
117,
116,
115,
113,
112,
114,
117,
117,
117,
116,
116,
115,
110,
109,
110,
113,
114,
112,
109,
108,
107,
105,
104,
99,
95,
92,
92,
93,
93,
93,
92,
92,
89,
85,
85,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
86,
85,
86,
87,
87,
88,
89,
88,
87,
86,
85,
83,
81,
79,
79,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
73,
73,
74,
73,
90,
110,
114,
116,
117,
116,
114,
112,
111,
110,
110,
110,
110,
108,
107,
105,
103,
103,
100,
96,
87,
86,
94,
100,
103,
108,
110,
111,
111,
110,
109,
107,
107,
108,
109,
110,
110,
110,
110,
111,
114,
115,
115,
115,
115,
115,
115,
116,
115,
109,
109,
110,
112,
114,
116,
117,
117,
118,
121,
124,
126,
128,
127,
122,
119,
112,
102,
93,
93,
98,
104,
108,
112,
116,
122,
125,
128,
130,
130,
130,
129,
125,
122,
121,
122,
123,
123,
122,
116,
118,
119,
122,
123,
122,
123,
122,
121,
119,
118,
117,
116,
115,
111,
111,
112,
114,
116,
118,
114,
112,
112,
111,
109,
110,
111,
116,
117,
111,
109,
108,
106,
104,
101,
97,
96,
94,
94,
95,
94,
94,
93,
92,
89,
85,
85,
85,
85,
85,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
84,
84,
84,
84,
84,
84,
84,
84,
84,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
86,
85,
85,
86,
87,
87,
88,
88,
88,
88,
87,
86,
85,
82,
79,
79,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
73,
73,
73,
85,
103,
117,
116,
116,
116,
116,
115,
113,
112,
112,
111,
111,
110,
109,
108,
106,
106,
105,
103,
98,
90,
86,
91,
98,
102,
106,
110,
111,
112,
111,
110,
110,
108,
108,
108,
108,
109,
109,
109,
110,
111,
114,
115,
116,
116,
116,
115,
116,
116,
111,
109,
110,
111,
113,
115,
115,
115,
115,
116,
117,
120,
120,
119,
118,
115,
109,
101,
93,
93,
100,
106,
110,
114,
117,
121,
125,
128,
130,
130,
131,
129,
127,
124,
122,
124,
125,
125,
123,
117,
115,
117,
122,
123,
123,
122,
121,
120,
118,
117,
115,
114,
112,
109,
110,
113,
112,
111,
109,
108,
108,
109,
109,
109,
109,
112,
116,
113,
110,
109,
109,
106,
104,
98,
96,
96,
95,
96,
96,
95,
94,
93,
92,
89,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
86,
87,
88,
88,
89,
88,
88,
87,
86,
85,
82,
79,
79,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
73,
73,
73,
94,
112,
114,
116,
116,
116,
115,
114,
113,
113,
112,
112,
111,
111,
110,
109,
108,
108,
106,
105,
100,
93,
88,
89,
95,
100,
104,
108,
110,
111,
112,
111,
110,
109,
108,
108,
109,
108,
108,
108,
110,
110,
113,
115,
116,
117,
117,
116,
117,
115,
111,
110,
109,
110,
113,
114,
115,
115,
115,
115,
115,
116,
114,
113,
112,
109,
104,
98,
93,
93,
98,
105,
112,
116,
120,
123,
126,
129,
131,
132,
132,
130,
128,
126,
125,
127,
127,
129,
124,
122,
109,
114,
119,
122,
122,
121,
120,
119,
118,
116,
114,
111,
109,
109,
111,
112,
111,
109,
105,
103,
105,
106,
106,
107,
109,
111,
112,
110,
110,
110,
109,
106,
104,
97,
96,
96,
97,
97,
97,
96,
94,
93,
92,
89,
87,
86,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
85,
85,
85,
85,
85,
85,
85,
85,
85,
86,
85,
85,
85,
85,
85,
85,
85,
86,
87,
88,
88,
88,
89,
88,
88,
88,
87,
86,
85,
81,
79,
79,
79,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
73,
73,
73,
92,
110,
113,
116,
115,
115,
115,
114,
113,
113,
112,
112,
111,
111,
110,
110,
110,
110,
108,
106,
102,
97,
92,
91,
92,
98,
102,
105,
108,
110,
111,
111,
112,
111,
110,
109,
108,
109,
109,
110,
110,
111,
114,
115,
116,
118,
117,
117,
116,
117,
111,
110,
110,
110,
114,
116,
117,
116,
115,
115,
115,
115,
111,
111,
111,
107,
104,
103,
101,
98,
95,
99,
109,
116,
121,
124,
127,
130,
132,
134,
133,
131,
129,
127,
127,
128,
128,
129,
127,
124,
110,
111,
116,
118,
119,
119,
118,
117,
116,
113,
110,
109,
109,
110,
112,
113,
111,
102,
102,
102,
102,
104,
106,
108,
110,
110,
109,
110,
110,
111,
109,
106,
102,
97,
97,
98,
99,
98,
98,
96,
94,
93,
91,
89,
88,
86,
86,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
87,
88,
88,
88,
88,
88,
88,
88,
88,
87,
87,
86,
85,
81,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
72,
73,
73,
73,
73,
86,
105,
111,
116,
115,
115,
115,
114,
114,
113,
113,
112,
112,
112,
112,
113,
114,
113,
111,
109,
105,
101,
97,
91,
91,
95,
100,
103,
105,
108,
110,
111,
113,
113,
111,
110,
110,
110,
111,
112,
113,
115,
116,
116,
116,
118,
117,
117,
116,
115,
112,
111,
110,
110,
115,
118,
119,
119,
117,
116,
117,
117,
117,
116,
113,
111,
109,
109,
107,
104,
98,
96,
104,
113,
120,
124,
127,
129,
132,
135,
134,
133,
130,
128,
128,
129,
129,
129,
129,
123,
110,
110,
112,
115,
117,
117,
117,
115,
112,
109,
110,
109,
112,
115,
116,
116,
112,
102,
102,
102,
102,
102,
105,
107,
109,
109,
110,
110,
110,
110,
107,
104,
101,
98,
98,
100,
103,
102,
98,
96,
94,
91,
91,
89,
88,
87,
86,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
87,
87,
88,
89,
88,
89,
89,
88,
88,
88,
88,
87,
87,
86,
85,
81,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
73,
73,
73,
77,
79,
101,
111,
116,
115,
114,
114,
114,
113,
113,
113,
113,
112,
112,
112,
115,
116,
116,
115,
112,
110,
106,
101,
95,
91,
93,
99,
102,
104,
107,
109,
110,
112,
114,
115,
114,
113,
114,
114,
114,
115,
117,
117,
117,
117,
118,
116,
116,
115,
114,
113,
114,
112,
110,
114,
119,
122,
122,
120,
119,
119,
120,
121,
121,
119,
115,
115,
114,
112,
107,
100,
100,
99,
108,
115,
120,
123,
127,
130,
134,
134,
134,
132,
130,
130,
131,
130,
130,
129,
126,
109,
110,
109,
111,
114,
115,
113,
110,
109,
109,
109,
113,
117,
116,
116,
113,
110,
102,
102,
102,
102,
102,
105,
106,
108,
109,
110,
111,
110,
109,
105,
104,
101,
101,
102,
103,
104,
103,
99,
96,
94,
92,
90,
88,
87,
87,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
86,
86,
86,
86,
86,
86,
86,
87,
87,
87,
88,
88,
88,
89,
89,
89,
89,
88,
88,
88,
87,
87,
87,
86,
85,
82,
80,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
73,
73,
73,
78,
88,
106,
112,
116,
115,
114,
114,
114,
114,
114,
114,
113,
113,
113,
114,
115,
116,
116,
116,
115,
113,
110,
105,
99,
92,
91,
99,
104,
108,
109,
109,
109,
112,
115,
116,
116,
116,
116,
116,
116,
116,
117,
118,
117,
117,
117,
116,
115,
114,
116,
116,
116,
116,
114,
114,
117,
122,
122,
122,
121,
121,
122,
123,
124,
123,
121,
119,
118,
116,
110,
105,
99,
100,
102,
108,
112,
121,
125,
128,
131,
133,
132,
130,
129,
129,
129,
130,
129,
129,
128,
113,
112,
110,
110,
113,
111,
109,
109,
109,
109,
113,
117,
117,
116,
116,
111,
108,
102,
102,
102,
102,
102,
103,
106,
108,
110,
111,
112,
110,
106,
104,
103,
103,
103,
104,
104,
104,
103,
98,
96,
94,
92,
90,
88,
87,
87,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
86,
86,
86,
86,
87,
87,
87,
87,
87,
88,
88,
88,
89,
89,
89,
88,
89,
88,
88,
87,
87,
87,
87,
86,
86,
84,
82,
81,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
73,
73,
73,
73,
91,
108,
116,
115,
115,
114,
114,
114,
114,
114,
114,
114,
114,
114,
115,
115,
116,
116,
116,
117,
116,
112,
108,
102,
94,
91,
99,
105,
110,
110,
111,
112,
114,
116,
117,
118,
118,
118,
118,
117,
116,
116,
117,
118,
117,
116,
115,
114,
115,
115,
116,
117,
118,
117,
116,
115,
119,
123,
123,
122,
123,
125,
126,
127,
126,
124,
123,
121,
118,
113,
108,
104,
99,
99,
101,
106,
114,
122,
125,
127,
129,
129,
128,
126,
127,
129,
129,
130,
130,
129,
116,
115,
114,
112,
112,
111,
110,
109,
109,
113,
116,
116,
117,
116,
114,
111,
108,
104,
102,
102,
102,
102,
102,
107,
109,
112,
115,
113,
110,
105,
104,
104,
104,
105,
105,
104,
104,
102,
98,
97,
95,
92,
90,
89,
88,
87,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
86,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
86,
86,
86,
86,
87,
87,
87,
87,
88,
88,
88,
88,
88,
89,
89,
88,
88,
88,
88,
88,
88,
87,
87,
87,
87,
86,
86,
84,
82,
80,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
73,
73,
73,
73,
90,
107,
116,
115,
115,
115,
114,
114,
114,
114,
114,
114,
114,
114,
114,
115,
115,
116,
116,
117,
116,
114,
109,
104,
97,
91,
96,
103,
110,
111,
112,
113,
116,
117,
117,
118,
117,
117,
118,
117,
116,
115,
116,
116,
116,
115,
114,
115,
116,
117,
118,
119,
119,
119,
119,
117,
117,
121,
123,
125,
126,
127,
128,
129,
129,
128,
128,
125,
121,
116,
112,
109,
104,
99,
100,
99,
106,
114,
120,
123,
125,
125,
124,
123,
122,
125,
128,
129,
129,
130,
117,
116,
116,
114,
113,
111,
110,
109,
109,
113,
115,
115,
116,
115,
113,
111,
108,
105,
102,
102,
102,
102,
105,
108,
110,
112,
116,
111,
109,
107,
106,
106,
106,
106,
106,
105,
104,
102,
102,
97,
98,
95,
92,
90,
88,
88,
87,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
87,
87,
87,
88,
88,
88,
88,
88,
88,
89,
89,
88,
88,
88,
88,
88,
88,
88,
87,
87,
88,
88,
88,
86,
85,
84,
81,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
77,
76,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
72,
73,
74,
79,
87,
107,
112,
116,
115,
115,
114,
114,
113,
114,
114,
114,
114,
114,
114,
114,
115,
115,
116,
117,
116,
114,
109,
104,
98,
93,
91,
99,
105,
110,
112,
114,
115,
116,
116,
117,
117,
117,
117,
115,
114,
114,
114,
114,
113,
112,
113,
114,
116,
118,
119,
120,
120,
121,
122,
121,
119,
120,
122,
125,
127,
129,
129,
130,
130,
130,
128,
127,
123,
120,
118,
114,
109,
106,
99,
100,
99,
106,
111,
113,
115,
116,
116,
115,
117,
120,
121,
126,
128,
130,
118,
117,
116,
115,
114,
112,
111,
110,
110,
109,
113,
114,
116,
115,
115,
112,
108,
105,
102,
102,
102,
102,
105,
109,
110,
109,
109,
109,
108,
108,
108,
108,
107,
106,
105,
104,
104,
102,
99,
98,
98,
97,
93,
91,
90,
89,
88,
87,
87,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
87,
87,
87,
87,
88,
88,
89,
90,
89,
89,
89,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
87,
86,
85,
82,
80,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
72,
74,
77,
80,
86,
108,
113,
115,
116,
113,
112,
113,
113,
113,
113,
113,
114,
115,
115,
115,
115,
116,
116,
116,
115,
112,
108,
104,
100,
96,
92,
91,
99,
105,
110,
111,
113,
114,
114,
115,
116,
115,
114,
113,
112,
112,
112,
111,
110,
110,
112,
115,
118,
120,
121,
121,
122,
122,
123,
123,
122,
121,
121,
123,
127,
128,
130,
130,
130,
130,
130,
128,
126,
124,
121,
118,
115,
112,
108,
99,
99,
101,
104,
106,
108,
109,
111,
114,
115,
120,
120,
125,
129,
130,
118,
117,
117,
116,
115,
113,
112,
111,
110,
110,
111,
113,
114,
116,
116,
111,
107,
102,
102,
102,
102,
102,
105,
107,
108,
107,
105,
106,
108,
110,
109,
109,
107,
106,
105,
105,
104,
101,
98,
98,
99,
97,
94,
92,
91,
89,
88,
87,
87,
87,
87,
86,
86,
86,
86,
86,
86,
86,
85,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
87,
87,
87,
87,
88,
88,
89,
91,
92,
91,
89,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
89,
88,
88,
87,
86,
84,
80,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
72,
72,
74,
74,
75,
80,
88,
111,
116,
116,
114,
112,
111,
112,
112,
112,
112,
113,
114,
115,
115,
115,
116,
116,
116,
117,
116,
110,
108,
106,
104,
101,
98,
96,
98,
97,
103,
109,
111,
112,
113,
113,
113,
112,
111,
109,
109,
110,
110,
109,
108,
110,
113,
117,
119,
122,
122,
122,
122,
123,
123,
123,
122,
122,
122,
122,
125,
128,
128,
130,
130,
131,
131,
130,
129,
129,
126,
123,
120,
117,
112,
106,
99,
99,
102,
107,
111,
115,
122,
123,
123,
122,
121,
124,
128,
130,
118,
118,
117,
116,
115,
114,
113,
111,
110,
110,
111,
114,
115,
116,
114,
111,
106,
102,
102,
102,
102,
102,
105,
106,
107,
105,
103,
104,
107,
110,
109,
109,
105,
104,
104,
104,
103,
100,
98,
99,
99,
98,
96,
94,
92,
90,
88,
88,
87,
87,
87,
87,
87,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
88,
88,
91,
92,
92,
91,
90,
88,
88,
88,
88,
88,
89,
89,
88,
88,
88,
88,
89,
89,
89,
88,
88,
86,
85,
82,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
72,
73,
73,
74,
75,
75,
91,
111,
113,
113,
111,
110,
110,
111,
111,
112,
113,
114,
115,
116,
116,
116,
116,
116,
117,
116,
114,
111,
109,
108,
107,
106,
104,
101,
97,
99,
97,
102,
107,
110,
111,
111,
110,
109,
107,
105,
104,
104,
104,
104,
106,
109,
113,
116,
118,
121,
123,
122,
122,
123,
123,
124,
123,
123,
122,
122,
122,
125,
128,
129,
129,
130,
132,
132,
132,
133,
131,
129,
125,
121,
116,
111,
104,
99,
103,
110,
115,
122,
125,
127,
128,
128,
127,
126,
127,
130,
119,
119,
118,
116,
115,
115,
114,
112,
110,
111,
112,
116,
117,
115,
112,
109,
104,
102,
102,
102,
102,
102,
102,
105,
106,
104,
103,
103,
105,
107,
108,
106,
104,
103,
103,
103,
102,
100,
98,
100,
100,
98,
97,
96,
93,
92,
89,
88,
88,
88,
87,
87,
87,
87,
87,
87,
86,
86,
86,
86,
86,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
88,
88,
88,
90,
91,
92,
91,
89,
88,
88,
88,
88,
88,
89,
89,
88,
88,
88,
89,
89,
89,
89,
88,
88,
86,
86,
83,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
72,
72,
72,
72,
73,
73,
73,
73,
76,
80,
99,
110,
111,
110,
110,
109,
110,
110,
111,
112,
113,
115,
116,
116,
116,
116,
116,
117,
117,
115,
114,
113,
111,
111,
110,
109,
108,
105,
102,
100,
97,
97,
101,
104,
106,
107,
106,
105,
103,
104,
104,
104,
104,
104,
104,
107,
110,
111,
115,
118,
122,
123,
123,
123,
124,
124,
124,
125,
124,
122,
122,
122,
125,
127,
128,
128,
130,
133,
134,
134,
135,
132,
129,
123,
118,
114,
109,
103,
103,
110,
117,
122,
126,
128,
129,
128,
129,
129,
129,
130,
121,
121,
120,
117,
116,
116,
116,
113,
109,
113,
118,
118,
117,
116,
111,
108,
104,
103,
103,
102,
102,
102,
102,
106,
106,
105,
104,
103,
103,
103,
103,
103,
103,
103,
103,
102,
102,
101,
100,
100,
102,
102,
100,
97,
94,
92,
90,
88,
88,
88,
88,
88,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
88,
88,
88,
88,
88,
89,
91,
91,
89,
88,
88,
89,
89,
88,
88,
89,
89,
89,
89,
89,
89,
89,
88,
88,
88,
87,
87,
86,
84,
80,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
78,
77,
77,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
74,
74,
73,
89,
104,
106,
107,
108,
108,
108,
109,
110,
111,
112,
113,
115,
116,
116,
116,
116,
116,
116,
116,
115,
114,
114,
113,
113,
113,
113,
111,
110,
107,
105,
103,
101,
101,
102,
103,
103,
102,
103,
103,
103,
105,
108,
109,
108,
107,
108,
108,
110,
110,
114,
119,
122,
123,
124,
124,
123,
125,
124,
125,
126,
124,
124,
125,
126,
127,
127,
128,
132,
134,
135,
135,
135,
131,
127,
121,
118,
113,
107,
103,
105,
113,
119,
123,
126,
127,
128,
130,
131,
131,
131,
122,
122,
121,
119,
118,
118,
117,
115,
114,
117,
118,
119,
117,
115,
111,
107,
104,
103,
103,
103,
103,
103,
106,
108,
108,
107,
104,
103,
103,
103,
103,
103,
103,
103,
104,
104,
103,
102,
101,
102,
103,
104,
101,
97,
95,
93,
91,
89,
88,
88,
88,
88,
88,
88,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
89,
89,
90,
90,
89,
89,
89,
90,
90,
89,
89,
90,
89,
89,
89,
89,
89,
89,
88,
88,
87,
86,
86,
86,
85,
82,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
74,
75,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
74,
74,
76,
78,
97,
102,
102,
103,
104,
105,
106,
108,
109,
110,
111,
112,
114,
115,
115,
115,
113,
112,
112,
113,
112,
113,
115,
114,
115,
116,
116,
114,
112,
110,
109,
106,
104,
103,
102,
102,
102,
102,
102,
103,
105,
110,
113,
114,
114,
113,
111,
109,
109,
109,
110,
114,
118,
122,
123,
123,
123,
124,
124,
125,
127,
127,
127,
127,
127,
127,
128,
130,
132,
134,
135,
136,
136,
134,
130,
124,
120,
116,
112,
105,
103,
111,
118,
122,
125,
127,
128,
132,
133,
133,
133,
123,
123,
121,
121,
120,
119,
118,
116,
117,
120,
121,
119,
117,
115,
110,
107,
104,
103,
105,
104,
104,
105,
108,
111,
111,
109,
108,
105,
104,
103,
103,
103,
103,
104,
105,
104,
103,
103,
102,
103,
103,
104,
103,
98,
95,
93,
91,
90,
90,
90,
89,
88,
88,
88,
88,
88,
87,
87,
87,
87,
87,
87,
87,
87,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
89,
90,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
90,
90,
89,
89,
89,
88,
88,
88,
86,
85,
85,
84,
82,
81,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
76,
76,
76,
76,
76,
75,
75,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
76,
78,
90,
98,
100,
100,
100,
101,
102,
104,
106,
108,
110,
110,
111,
112,
113,
113,
112,
110,
109,
109,
109,
112,
113,
115,
116,
116,
117,
117,
116,
114,
112,
110,
108,
105,
104,
103,
102,
102,
102,
102,
104,
106,
110,
114,
116,
116,
117,
116,
113,
109,
109,
109,
109,
112,
115,
117,
121,
122,
122,
123,
125,
127,
128,
129,
128,
128,
128,
129,
130,
133,
134,
135,
136,
136,
136,
133,
128,
122,
119,
114,
108,
105,
113,
121,
124,
127,
129,
131,
134,
135,
134,
133,
125,
124,
122,
122,
121,
120,
119,
118,
119,
120,
122,
118,
117,
114,
109,
106,
104,
104,
106,
106,
107,
108,
112,
116,
116,
112,
110,
107,
105,
104,
103,
104,
104,
104,
105,
105,
104,
104,
104,
104,
104,
105,
104,
99,
96,
93,
91,
92,
92,
91,
90,
89,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
89,
91,
92,
93,
92,
92,
94,
95,
95,
93,
92,
92,
91,
91,
90,
90,
89,
89,
89,
88,
88,
88,
86,
86,
85,
85,
86,
83,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
73,
89,
98,
98,
98,
97,
97,
98,
100,
102,
104,
105,
107,
108,
109,
111,
111,
111,
109,
108,
108,
108,
109,
113,
116,
117,
117,
118,
117,
118,
116,
114,
112,
110,
108,
107,
105,
104,
104,
104,
103,
103,
103,
105,
109,
113,
116,
117,
117,
117,
117,
113,
109,
109,
109,
110,
110,
115,
115,
117,
120,
122,
123,
126,
128,
129,
129,
129,
129,
130,
131,
133,
134,
134,
135,
136,
136,
136,
131,
126,
120,
115,
107,
109,
113,
120,
124,
128,
130,
133,
135,
135,
135,
134,
129,
126,
125,
124,
122,
121,
120,
119,
119,
120,
119,
118,
116,
110,
108,
106,
103,
104,
109,
109,
109,
112,
117,
117,
116,
114,
111,
110,
108,
106,
105,
105,
105,
105,
106,
105,
105,
105,
105,
105,
105,
105,
103,
98,
96,
93,
93,
93,
92,
92,
91,
90,
89,
89,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
89,
89,
89,
89,
89,
89,
89,
89,
90,
92,
94,
95,
93,
92,
94,
93,
93,
92,
92,
91,
91,
90,
90,
89,
89,
89,
88,
88,
88,
87,
86,
86,
86,
86,
86,
84,
81,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
75,
75,
75,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
73,
73,
74,
78,
95,
98,
97,
97,
96,
96,
97,
97,
99,
100,
102,
103,
105,
107,
109,
109,
107,
105,
106,
106,
108,
110,
114,
116,
118,
118,
118,
118,
117,
115,
114,
112,
110,
108,
108,
107,
106,
106,
106,
105,
104,
104,
104,
107,
110,
113,
116,
118,
119,
118,
117,
112,
110,
109,
111,
115,
116,
116,
116,
118,
121,
122,
123,
126,
127,
128,
129,
130,
130,
131,
132,
133,
134,
134,
135,
136,
135,
133,
127,
121,
117,
108,
109,
113,
117,
122,
126,
129,
133,
135,
136,
136,
135,
129,
128,
128,
125,
122,
121,
120,
119,
119,
118,
117,
116,
111,
108,
106,
105,
104,
107,
112,
114,
114,
116,
116,
117,
115,
114,
113,
111,
110,
109,
108,
108,
107,
106,
106,
106,
107,
108,
108,
107,
106,
105,
102,
97,
96,
95,
94,
93,
93,
92,
91,
90,
89,
89,
89,
88,
88,
88,
88,
88,
88,
88,
88,
88,
88,
89,
89,
89,
89,
89,
89,
89,
89,
89,
90,
91,
92,
94,
94,
92,
91,
92,
92,
91,
91,
91,
91,
91,
90,
89,
89,
88,
88,
88,
88,
88,
87,
87,
87,
87,
86,
86,
84,
82,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
74,
74,
74,
74,
74,
73,
73,
73,
73,
73,
73,
74,
75,
77,
87,
92,
95,
95,
95,
95,
95,
96,
96,
97,
98,
99,
101,
103,
104,
105,
104,
103,
103,
104,
106,
109,
111,
114,
116,
118,
118,
118,
117,
116,
115,
113,
112,
110,
109,
108,
108,
109,
109,
108,
107,
106,
106,
106,
107,
108,
111,
114,
117,
120,
121,
120,
117,
110,
110,
111,
117,
120,
120,
119,
119,
120,
120,
121,
123,
125,
126,
128,
129,
130,
131,
132,
132,
132,
132,
134,
135,
135,
133,
128,
123,
117,
111,
110,
112,
115,
118,
122,
126,
129,
134,
135,
136,
136,
130,
129,
128,
125,
122,
121,
120,
120,
119,
117,
116,
113,
109,
107,
107,
106,
107,
111,
116,
117,
118,
118,
117,
116,
113,
113,
112,
111,
111,
110,
110,
110,
109,
107,
106,
107,
108,
110,
110,
108,
106,
104,
100,
97,
96,
95,
94,
94,
93,
92,
92,
91,
90,
89,
89,
89,
89,
88,
88,
88,
88,
88,
88,
88,
89,
89,
89,
89,
89,
89,
90,
90,
90,
90,
90,
91,
91,
93,
92,
91,
91,
91,
91,
91,
91,
90,
90,
90,
89,
89,
88,
88,
88,
88,
88,
87,
87,
87,
87,
87,
86,
86,
85,
82,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
74,
74,
74,
74,
74,
74,
74,
74,
73,
73,
74,
74,
79,
87,
92,
92,
93,
93,
94,
94,
94,
95,
95,
96,
96,
97,
99,
101,
102,
103,
102,
102,
103,
104,
107,
109,
111,
113,
115,
117,
118,
117,
116,
116,
115,
114,
113,
112,
111,
111,
111,
111,
112,
111,
110,
108,
108,
108,
109,
109,
109,
112,
116,
119,
122,
123,
120,
116,
110,
110,
115,
122,
121,
122,
121,
121,
121,
121,
122,
122,
126,
128,
129,
130,
131,
132,
132,
130,
128,
129,
135,
136,
132,
127,
122,
117,
113,
112,
112,
114,
116,
120,
122,
127,
129,
133,
135,
136,
130,
129,
128,
125,
122,
121,
121,
120,
118,
117,
116,
110,
108,
107,
107,
106,
109,
114,
118,
119,
118,
118,
115,
115,
110,
110,
111,
111,
111,
111,
110,
110,
109,
108,
107,
107,
107,
110,
110,
107,
104,
102,
98,
97,
96,
96,
95,
94,
94,
93,
92,
91,
90,
90,
89,
89,
89,
88,
88,
88,
89,
89,
89,
89,
89,
89,
90,
90,
90,
90,
90,
90,
90,
90,
90,
91,
91,
91,
92,
91,
91,
91,
91,
91,
91,
90,
90,
89,
89,
89,
88,
88,
88,
88,
87,
87,
87,
87,
87,
88,
86,
85,
84,
82,
81,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
74,
74,
74,
74,
74,
74,
74,
74,
75,
80,
85,
88,
90,
90,
91,
92,
92,
93,
93,
93,
94,
94,
95,
96,
97,
99,
100,
101,
102,
103,
103,
104,
107,
109,
110,
112,
114,
115,
117,
117,
116,
115,
114,
115,
115,
116,
116,
116,
115,
115,
115,
114,
112,
110,
110,
111,
112,
112,
111,
110,
115,
118,
121,
122,
122,
118,
114,
110,
110,
116,
122,
122,
124,
125,
122,
123,
121,
122,
126,
128,
129,
130,
131,
133,
132,
130,
128,
127,
128,
130,
129,
124,
119,
116,
113,
112,
114,
116,
120,
124,
126,
128,
128,
131,
134,
135,
129,
128,
127,
126,
123,
122,
122,
120,
118,
117,
113,
109,
107,
107,
107,
108,
109,
118,
119,
119,
118,
117,
112,
108,
108,
109,
109,
110,
110,
110,
110,
110,
109,
108,
108,
107,
107,
108,
107,
106,
104,
101,
99,
97,
96,
96,
95,
95,
94,
93,
92,
91,
91,
90,
90,
89,
89,
89,
89,
89,
89,
89,
89,
89,
90,
90,
90,
91,
91,
91,
91,
90,
90,
91,
90,
91,
91,
91,
92,
91,
91,
91,
91,
91,
90,
90,
90,
89,
89,
89,
88,
88,
88,
88,
88,
87,
87,
88,
88,
88,
87,
86,
85,
82,
81,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
74,
75,
75,
76,
77,
79,
81,
83,
85,
86,
86,
87,
89,
90,
91,
92,
92,
93,
94,
95,
96,
97,
99,
102,
103,
104,
105,
106,
107,
108,
109,
110,
111,
113,
115,
116,
116,
115,
116,
116,
118,
119,
120,
119,
118,
118,
118,
117,
117,
114,
114,
114,
115,
116,
116,
115,
115,
116,
119,
122,
122,
119,
118,
116,
112,
115,
116,
120,
123,
129,
129,
126,
124,
121,
124,
127,
128,
129,
131,
133,
134,
132,
129,
125,
121,
121,
121,
118,
116,
115,
114,
114,
115,
119,
123,
127,
129,
129,
130,
131,
134,
134,
128,
127,
126,
126,
124,
123,
122,
120,
118,
116,
111,
108,
107,
108,
107,
108,
113,
117,
119,
118,
117,
112,
108,
107,
107,
107,
108,
109,
110,
109,
109,
109,
108,
108,
109,
108,
108,
107,
106,
105,
104,
103,
100,
97,
97,
96,
96,
96,
95,
94,
93,
92,
91,
91,
90,
90,
89,
89,
89,
89,
89,
89,
90,
90,
90,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
92,
91,
91,
91,
91,
90,
90,
90,
90,
89,
89,
89,
88,
88,
88,
88,
88,
88,
88,
88,
88,
89,
88,
86,
84,
82,
81,
81,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
75,
75,
75,
75,
75,
76,
77,
77,
78,
79,
79,
80,
78,
80,
84,
85,
85,
85,
85,
91,
93,
94,
94,
95,
98,
99,
102,
104,
106,
108,
109,
109,
109,
109,
110,
110,
111,
113,
115,
115,
115,
116,
118,
120,
122,
123,
122,
121,
120,
119,
121,
122,
120,
119,
118,
118,
119,
119,
118,
116,
115,
117,
119,
119,
121,
122,
120,
117,
115,
117,
116,
119,
122,
128,
129,
128,
125,
125,
126,
127,
128,
130,
133,
134,
132,
130,
126,
121,
121,
121,
116,
116,
116,
115,
115,
116,
120,
125,
128,
129,
130,
131,
132,
133,
134,
126,
126,
125,
125,
123,
122,
122,
119,
117,
114,
109,
108,
107,
107,
108,
109,
115,
117,
118,
116,
114,
109,
108,
107,
107,
107,
108,
108,
108,
108,
109,
108,
109,
108,
109,
109,
108,
108,
107,
106,
105,
104,
102,
98,
97,
97,
97,
97,
96,
95,
93,
92,
92,
91,
90,
90,
90,
89,
89,
89,
89,
90,
90,
91,
91,
91,
92,
93,
94,
93,
92,
91,
91,
92,
91,
91,
92,
92,
91,
91,
91,
91,
91,
90,
90,
90,
89,
89,
89,
89,
88,
88,
88,
88,
88,
88,
88,
88,
89,
89,
88,
85,
82,
82,
81,
81,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
75,
75,
76,
76,
76,
76,
77,
77,
77,
78,
78,
78,
78,
78,
80,
81,
83,
83,
84,
84,
86,
92,
93,
93,
94,
96,
98,
102,
104,
107,
110,
111,
113,
113,
111,
110,
110,
110,
112,
113,
114,
116,
117,
119,
120,
122,
122,
123,
122,
121,
121,
122,
123,
125,
122,
121,
122,
123,
123,
123,
120,
117,
118,
119,
120,
121,
122,
122,
122,
121,
119,
117,
116,
121,
122,
127,
128,
129,
129,
129,
129,
128,
131,
133,
132,
131,
130,
127,
122,
124,
125,
125,
124,
122,
116,
115,
116,
121,
126,
129,
130,
131,
132,
133,
134,
137,
123,
124,
124,
124,
123,
122,
120,
118,
116,
113,
109,
108,
108,
107,
109,
111,
117,
116,
115,
114,
111,
109,
107,
107,
107,
107,
107,
107,
108,
107,
108,
108,
108,
109,
110,
110,
109,
108,
108,
107,
106,
105,
104,
100,
99,
98,
99,
98,
98,
97,
95,
93,
92,
91,
90,
90,
90,
90,
90,
89,
90,
90,
90,
91,
91,
92,
94,
95,
95,
94,
92,
91,
91,
93,
94,
92,
93,
92,
91,
91,
91,
91,
91,
90,
90,
90,
89,
89,
89,
89,
88,
88,
88,
88,
88,
88,
89,
89,
89,
89,
88,
84,
82,
82,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
78,
78,
78,
78,
79,
79,
80,
82,
82,
83,
83,
84,
87,
90,
90,
92,
93,
97,
100,
103,
106,
109,
111,
114,
117,
117,
115,
113,
112,
112,
113,
114,
115,
116,
117,
120,
122,
122,
122,
122,
121,
120,
120,
123,
125,
123,
122,
123,
123,
123,
123,
120,
119,
119,
120,
120,
120,
121,
122,
125,
128,
127,
123,
121,
120,
121,
123,
129,
130,
131,
132,
132,
132,
132,
133,
133,
132,
129,
128,
129,
132,
131,
130,
129,
124,
116,
116,
116,
122,
126,
128,
130,
131,
132,
134,
136,
139,
122,
123,
122,
122,
122,
121,
120,
118,
116,
112,
109,
108,
108,
109,
109,
114,
115,
116,
112,
110,
109,
107,
107,
106,
107,
107,
107,
107,
107,
108,
107,
108,
109,
109,
110,
111,
110,
109,
109,
108,
108,
106,
104,
103,
103,
103,
101,
98,
98,
97,
95,
93,
92,
91,
91,
90,
90,
90,
90,
90,
90,
90,
90,
91,
92,
93,
94,
95,
94,
93,
92,
91,
91,
93,
95,
94,
94,
92,
91,
91,
91,
91,
91,
91,
90,
90,
90,
89,
89,
89,
89,
89,
89,
89,
89,
89,
89,
89,
89,
89,
88,
84,
82,
82,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
77,
77,
78,
78,
78,
79,
79,
80,
81,
81,
81,
81,
82,
83,
84,
85,
85,
86,
88,
92,
99,
102,
104,
106,
109,
113,
116,
117,
116,
115,
114,
115,
114,
113,
114,
115,
116,
117,
120,
121,
121,
120,
117,
116,
118,
120,
121,
121,
121,
121,
122,
121,
122,
120,
120,
121,
120,
121,
121,
121,
122,
122,
129,
128,
127,
124,
121,
121,
121,
128,
130,
131,
134,
134,
134,
135,
135,
134,
132,
129,
129,
136,
134,
134,
133,
130,
124,
116,
116,
116,
121,
125,
128,
129,
130,
132,
134,
137,
140,
120,
121,
121,
120,
121,
121,
119,
117,
115,
112,
109,
108,
109,
111,
112,
115,
116,
113,
109,
108,
108,
107,
107,
108,
107,
107,
107,
108,
108,
108,
108,
109,
109,
109,
111,
111,
111,
110,
110,
109,
109,
109,
107,
106,
106,
105,
104,
101,
99,
97,
95,
93,
92,
91,
91,
91,
90,
90,
90,
90,
90,
90,
90,
91,
92,
93,
94,
94,
93,
92,
91,
91,
91,
93,
94,
95,
93,
92,
91,
91,
91,
91,
91,
90,
90,
90,
90,
89,
89,
89,
89,
89,
89,
89,
89,
89,
89,
89,
89,
89,
88,
84,
82,
82,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
78,
78,
78,
78,
77,
77,
78,
78,
78,
79,
79,
80,
80,
80,
80,
80,
81,
81,
82,
83,
83,
84,
85,
89,
98,
100,
101,
102,
106,
111,
115,
117,
115,
114,
115,
116,
116,
113,
113,
114,
114,
115,
118,
119,
118,
117,
115,
115,
116,
117,
118,
118,
118,
118,
120,
120,
120,
121,
122,
122,
122,
122,
121,
123,
123,
125,
127,
129,
128,
127,
124,
121,
121,
124,
129,
131,
134,
135,
136,
138,
139,
137,
133,
131,
132,
134,
135,
135,
133,
130,
126,
120,
116,
116,
118,
121,
125,
128,
129,
131,
134,
136,
139,
121,
119,
118,
118,
120,
119,
119,
117,
114,
112,
109,
109,
110,
112,
114,
116,
116,
110,
108,
108,
107,
107,
107,
107,
107,
108,
108,
108,
108,
108,
109,
109,
110,
111,
112,
113,
113,
112,
112,
112,
112,
111,
109,
108,
108,
107,
105,
103,
99,
97,
95,
93,
92,
91,
91,
90,
90,
90,
90,
90,
90,
90,
90,
91,
91,
92,
93,
93,
92,
91,
91,
91,
92,
93,
94,
94,
93,
91,
91,
91,
91,
91,
91,
90,
90,
90,
90,
89,
89,
89,
89,
89,
89,
89,
89,
89,
89,
89,
89,
89,
88,
83,
82,
82,
82,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
78,
78,
79,
78,
78,
78,
78,
78,
78,
78,
79,
79,
79,
79,
79,
79,
79,
79,
80,
81,
81,
83,
83,
84,
86,
93,
98,
99,
99,
102,
108,
112,
112,
112,
111,
113,
115,
114,
111,
110,
113,
115,
116,
118,
118,
116,
115,
114,
114,
115,
115,
116,
116,
116,
117,
120,
120,
121,
123,
124,
125,
126,
126,
126,
126,
125,
126,
128,
129,
129,
129,
128,
123,
121,
121,
127,
130,
132,
134,
136,
138,
141,
139,
134,
133,
134,
135,
136,
136,
134,
131,
129,
125,
117,
116,
116,
117,
119,
123,
127,
129,
132,
134,
136,
122,
119,
116,
116,
118,
118,
118,
117,
114,
111,
109,
109,
110,
114,
116,
116,
115,
111,
109,
108,
107,
107,
107,
108,
107,
109,
109,
108,
108,
109,
110,
110,
111,
112,
113,
115,
115,
115,
114,
114,
115,
114,
111,
112,
111,
110,
108,
104,
98,
96,
94,
93,
92,
91,
91,
90,
91,
90,
90,
90,
90,
90,
90,
91,
91,
92,
93,
93,
92,
91,
91,
91,
92,
94,
95,
95,
92,
91,
91,
91,
91,
91,
91,
90,
90,
90,
90,
90,
89,
89,
89,
89,
89,
89,
89,
89,
89,
89,
89,
89,
88,
84,
82,
82,
82,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
76,
77,
77,
77,
77,
78,
78,
79,
79,
79,
79,
79,
78,
78,
78,
79,
79,
80,
79,
79,
79,
80,
80,
80,
79,
80,
81,
82,
83,
83,
85,
88,
91,
91,
91,
96,
101,
105,
107,
106,
106,
108,
111,
111,
109,
109,
112,
116,
117,
118,
116,
115,
114,
114,
114,
114,
114,
114,
115,
116,
118,
119,
121,
122,
125,
127,
128,
129,
129,
129,
129,
127,
126,
128,
129,
130,
130,
129,
126,
123,
121,
122,
127,
129,
132,
134,
136,
138,
138,
135,
133,
134,
136,
138,
138,
136,
133,
131,
128,
122,
117,
116,
116,
116,
118,
122,
125,
127,
130,
133,
122,
119,
116,
116,
117,
118,
118,
117,
114,
111,
110,
109,
112,
115,
117,
116,
114,
112,
110,
108,
109,
108,
108,
109,
109,
109,
109,
108,
109,
109,
110,
111,
112,
113,
114,
116,
116,
117,
115,
116,
117,
116,
115,
116,
116,
113,
110,
104,
98,
94,
94,
93,
92,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
92,
92,
93,
92,
92,
92,
93,
93,
94,
94,
93,
91,
91,
91,
91,
91,
91,
91,
90,
90,
90,
90,
90,
90,
90,
90,
90,
90,
89,
89,
89,
89,
89,
89,
89,
88,
84,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
78,
78,
78,
79,
79,
82,
82,
80,
79,
78,
78,
78,
79,
81,
81,
81,
82,
83,
83,
84,
84,
81,
82,
82,
83,
83,
83,
85,
85,
85,
85,
89,
94,
98,
99,
98,
98,
101,
105,
105,
104,
107,
110,
115,
117,
117,
116,
114,
112,
113,
113,
114,
114,
115,
116,
117,
118,
120,
121,
124,
127,
129,
131,
132,
132,
131,
130,
128,
127,
128,
129,
131,
130,
130,
129,
128,
126,
124,
126,
127,
129,
132,
133,
135,
135,
134,
133,
134,
136,
138,
140,
141,
136,
133,
130,
127,
121,
116,
116,
116,
116,
117,
121,
126,
131,
135,
122,
119,
116,
116,
116,
117,
117,
116,
112,
110,
110,
111,
115,
117,
118,
116,
116,
113,
110,
110,
109,
108,
109,
110,
111,
110,
109,
109,
109,
109,
111,
112,
113,
114,
115,
116,
117,
117,
116,
117,
118,
118,
118,
119,
118,
115,
111,
104,
99,
95,
94,
93,
92,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
92,
92,
93,
93,
93,
94,
94,
94,
95,
92,
91,
91,
91,
91,
91,
91,
91,
91,
91,
90,
90,
90,
90,
90,
90,
90,
90,
90,
90,
90,
90,
89,
89,
89,
88,
87,
83,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
77,
77,
78,
78,
78,
79,
79,
88,
91,
86,
83,
81,
80,
79,
81,
83,
84,
85,
87,
89,
94,
96,
96,
90,
84,
84,
84,
83,
83,
83,
83,
83,
84,
85,
88,
90,
91,
91,
93,
96,
97,
98,
100,
104,
107,
112,
115,
116,
115,
111,
111,
113,
115,
116,
116,
115,
117,
118,
119,
120,
122,
125,
128,
131,
134,
135,
134,
134,
132,
129,
128,
129,
130,
131,
132,
132,
131,
130,
129,
128,
128,
128,
128,
130,
132,
134,
134,
133,
133,
134,
136,
138,
140,
141,
140,
134,
131,
129,
123,
116,
119,
120,
117,
117,
119,
126,
133,
135,
121,
119,
116,
116,
116,
118,
117,
114,
110,
110,
111,
116,
116,
117,
117,
117,
116,
116,
111,
110,
108,
109,
112,
115,
115,
111,
110,
109,
109,
110,
112,
113,
115,
115,
116,
117,
118,
118,
118,
118,
119,
120,
121,
122,
121,
115,
111,
103,
101,
98,
95,
94,
93,
92,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
92,
93,
93,
94,
94,
95,
94,
93,
92,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
90,
90,
90,
90,
90,
90,
90,
90,
90,
90,
90,
89,
89,
89,
89,
88,
87,
84,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
76,
77,
77,
77,
77,
77,
78,
78,
78,
79,
82,
89,
99,
99,
94,
88,
85,
84,
85,
86,
90,
97,
102,
106,
109,
110,
109,
101,
95,
92,
91,
85,
83,
83,
83,
83,
83,
84,
85,
87,
89,
90,
91,
93,
95,
97,
97,
100,
103,
108,
112,
112,
112,
109,
110,
114,
117,
119,
119,
118,
118,
119,
120,
121,
124,
127,
130,
132,
134,
135,
135,
135,
133,
132,
130,
131,
132,
132,
134,
134,
133,
131,
131,
130,
130,
129,
129,
129,
131,
132,
132,
133,
133,
134,
135,
137,
139,
142,
141,
135,
131,
128,
122,
122,
127,
126,
123,
117,
117,
123,
130,
135,
121,
120,
118,
118,
118,
117,
116,
111,
110,
110,
114,
116,
117,
117,
117,
118,
116,
114,
110,
109,
109,
111,
115,
116,
116,
112,
109,
109,
109,
110,
112,
116,
116,
115,
116,
117,
118,
119,
119,
120,
120,
121,
123,
124,
122,
115,
110,
105,
104,
101,
98,
95,
94,
93,
92,
91,
91,
91,
91,
91,
91,
91,
91,
92,
92,
93,
93,
94,
94,
95,
95,
93,
92,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
90,
90,
90,
90,
90,
90,
90,
90,
90,
90,
90,
90,
89,
89,
89,
89,
88,
86,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
78,
78,
78,
78,
80,
81,
86,
101,
111,
110,
106,
101,
97,
97,
100,
106,
110,
112,
110,
110,
110,
109,
110,
101,
96,
93,
87,
84,
83,
83,
83,
84,
85,
85,
86,
88,
89,
91,
93,
94,
97,
100,
101,
103,
105,
106,
109,
109,
110,
110,
115,
121,
123,
122,
121,
121,
121,
121,
122,
125,
128,
130,
132,
134,
135,
135,
135,
134,
133,
133,
133,
134,
133,
134,
135,
134,
133,
132,
132,
131,
130,
130,
131,
131,
132,
133,
132,
132,
133,
134,
134,
136,
137,
136,
134,
130,
125,
122,
128,
131,
131,
128,
123,
119,
122,
122,
130,
122,
120,
120,
119,
118,
116,
113,
110,
110,
111,
114,
115,
117,
117,
118,
118,
117,
114,
109,
109,
110,
113,
117,
116,
116,
113,
110,
110,
110,
110,
112,
114,
115,
115,
117,
118,
119,
120,
121,
121,
121,
124,
128,
127,
121,
115,
110,
108,
106,
104,
100,
97,
96,
94,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
93,
94,
94,
94,
95,
95,
93,
92,
92,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
90,
90,
90,
90,
90,
90,
90,
90,
90,
90,
90,
89,
89,
89,
89,
88,
87,
84,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
78,
78,
79,
80,
81,
81,
98,
109,
111,
109,
113,
112,
112,
113,
110,
110,
111,
110,
110,
110,
111,
109,
107,
99,
94,
90,
86,
84,
83,
83,
84,
85,
86,
87,
87,
89,
90,
92,
94,
98,
104,
104,
104,
104,
104,
106,
108,
109,
110,
114,
119,
122,
123,
122,
122,
122,
122,
123,
126,
128,
130,
131,
133,
134,
134,
134,
133,
133,
134,
135,
135,
134,
133,
134,
135,
136,
134,
134,
133,
132,
131,
131,
132,
132,
132,
132,
132,
133,
133,
133,
133,
133,
132,
130,
128,
122,
122,
131,
135,
134,
131,
128,
124,
121,
123,
122,
123,
121,
120,
119,
118,
116,
111,
110,
110,
112,
114,
116,
117,
117,
116,
117,
115,
113,
111,
109,
109,
116,
116,
117,
116,
113,
112,
112,
112,
112,
112,
114,
114,
115,
116,
118,
120,
121,
122,
122,
122,
128,
127,
126,
120,
115,
112,
110,
108,
104,
101,
98,
98,
94,
93,
93,
93,
92,
92,
92,
92,
92,
92,
93,
94,
94,
95,
95,
95,
94,
93,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
90,
90,
90,
90,
90,
90,
90,
90,
90,
89,
89,
89,
88,
87,
86,
83,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
78,
78,
78,
80,
80,
82,
95,
104,
107,
110,
110,
112,
112,
112,
112,
110,
110,
108,
108,
109,
110,
110,
112,
107,
99,
95,
91,
88,
85,
84,
84,
84,
85,
86,
86,
87,
89,
90,
91,
97,
103,
105,
105,
106,
107,
107,
108,
108,
109,
111,
114,
116,
118,
121,
122,
123,
123,
125,
126,
127,
128,
129,
131,
132,
133,
133,
133,
134,
135,
135,
135,
135,
135,
134,
134,
139,
138,
137,
135,
134,
132,
132,
132,
132,
132,
132,
133,
133,
133,
133,
133,
132,
130,
128,
124,
122,
123,
133,
136,
136,
134,
132,
129,
127,
123,
122,
123,
121,
120,
118,
117,
114,
110,
110,
111,
113,
115,
116,
116,
115,
109,
111,
114,
113,
110,
110,
112,
115,
115,
114,
115,
114,
113,
115,
115,
114,
113,
114,
114,
115,
116,
117,
120,
122,
124,
125,
125,
126,
127,
124,
120,
115,
113,
112,
110,
105,
103,
102,
99,
95,
94,
93,
93,
93,
93,
93,
93,
93,
93,
94,
94,
95,
95,
95,
94,
93,
92,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
90,
90,
91,
91,
91,
90,
90,
90,
90,
89,
89,
89,
88,
87,
85,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
77,
78,
78,
78,
78,
79,
80,
85,
94,
99,
102,
104,
108,
110,
111,
111,
110,
108,
106,
105,
105,
107,
110,
111,
111,
112,
107,
100,
95,
93,
90,
86,
85,
84,
85,
85,
85,
86,
88,
89,
90,
95,
101,
104,
105,
106,
108,
110,
110,
109,
110,
110,
111,
114,
115,
116,
119,
121,
123,
123,
123,
124,
127,
129,
130,
131,
131,
132,
134,
135,
135,
135,
135,
137,
138,
139,
140,
140,
141,
139,
140,
136,
135,
133,
132,
132,
132,
132,
133,
134,
134,
134,
134,
132,
130,
128,
128,
128,
127,
134,
137,
138,
137,
135,
133,
131,
128,
123,
123,
121,
120,
118,
116,
110,
110,
110,
111,
114,
115,
117,
116,
115,
108,
110,
111,
112,
113,
115,
114,
112,
110,
110,
113,
115,
114,
115,
116,
116,
113,
114,
115,
115,
116,
118,
120,
121,
125,
128,
127,
126,
125,
122,
119,
115,
115,
113,
110,
107,
106,
104,
101,
97,
94,
94,
94,
94,
93,
93,
93,
93,
93,
94,
94,
95,
95,
94,
94,
93,
92,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
90,
91,
91,
91,
91,
90,
90,
90,
90,
89,
89,
89,
87,
85,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
77,
78,
78,
78,
79,
80,
85,
93,
97,
99,
101,
102,
104,
105,
105,
104,
103,
103,
102,
103,
105,
108,
110,
111,
111,
111,
105,
98,
96,
94,
92,
85,
86,
87,
85,
85,
86,
87,
87,
89,
92,
98,
102,
104,
105,
109,
110,
111,
111,
112,
112,
112,
114,
114,
115,
116,
117,
119,
120,
120,
122,
126,
128,
129,
130,
130,
131,
133,
134,
133,
133,
134,
136,
139,
140,
141,
142,
142,
141,
140,
142,
140,
134,
134,
133,
133,
133,
133,
134,
135,
135,
134,
133,
131,
130,
131,
132,
131,
133,
137,
139,
139,
138,
137,
134,
130,
126,
123,
121,
120,
117,
115,
110,
110,
110,
112,
114,
116,
118,
117,
113,
109,
109,
109,
112,
116,
115,
115,
112,
109,
109,
114,
116,
115,
115,
116,
114,
113,
113,
115,
116,
117,
118,
120,
122,
127,
128,
127,
126,
124,
122,
119,
116,
116,
114,
110,
109,
108,
105,
101,
97,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
95,
94,
93,
93,
92,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
90,
90,
90,
89,
89,
89,
88,
85,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
77,
77,
78,
78,
78,
79,
79,
79,
91,
93,
97,
98,
99,
100,
101,
102,
101,
101,
102,
102,
102,
103,
105,
107,
109,
110,
109,
108,
102,
97,
95,
92,
89,
91,
91,
87,
85,
85,
86,
86,
86,
89,
93,
97,
100,
103,
107,
110,
111,
113,
114,
115,
117,
116,
116,
114,
114,
115,
115,
116,
119,
121,
123,
126,
127,
128,
128,
129,
130,
130,
129,
130,
134,
135,
137,
140,
141,
142,
140,
140,
139,
141,
142,
142,
137,
134,
134,
134,
133,
134,
134,
135,
135,
133,
133,
133,
135,
136,
134,
134,
136,
141,
141,
141,
139,
136,
132,
128,
123,
122,
120,
117,
115,
112,
111,
113,
114,
115,
117,
119,
118,
117,
116,
112,
111,
113,
116,
117,
116,
114,
111,
111,
111,
114,
114,
113,
112,
111,
112,
114,
116,
116,
116,
118,
121,
124,
128,
128,
127,
127,
123,
121,
119,
118,
116,
114,
114,
111,
110,
106,
101,
97,
96,
95,
95,
95,
94,
94,
94,
94,
94,
94,
95,
94,
94,
94,
93,
93,
92,
92,
91,
91,
91,
91,
91,
91,
91,
92,
92,
92,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
90,
90,
90,
89,
89,
88,
85,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
76,
76,
77,
77,
77,
77,
78,
78,
78,
78,
79,
79,
89,
93,
95,
96,
97,
98,
100,
101,
102,
102,
102,
102,
102,
102,
103,
105,
107,
109,
110,
109,
104,
99,
96,
93,
93,
95,
91,
87,
85,
85,
85,
86,
86,
86,
88,
91,
95,
100,
104,
109,
111,
111,
113,
116,
116,
117,
116,
116,
115,
115,
115,
115,
116,
119,
121,
122,
122,
123,
123,
125,
126,
127,
127,
128,
131,
133,
135,
137,
139,
140,
139,
135,
133,
136,
135,
135,
136,
133,
133,
133,
134,
134,
135,
135,
136,
135,
134,
138,
140,
141,
137,
134,
136,
140,
141,
142,
140,
138,
133,
132,
123,
123,
121,
117,
115,
113,
114,
115,
116,
115,
118,
120,
121,
120,
118,
115,
114,
115,
116,
117,
116,
116,
116,
112,
110,
110,
110,
111,
110,
109,
111,
113,
114,
116,
116,
119,
122,
127,
128,
128,
128,
128,
123,
121,
120,
118,
116,
116,
116,
113,
110,
106,
101,
97,
96,
96,
95,
95,
94,
94,
94,
94,
94,
95,
95,
94,
94,
94,
93,
93,
92,
92,
91,
91,
91,
91,
91,
91,
92,
92,
92,
92,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
91,
90,
90,
89,
89,
88,
85,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
76,
76,
77,
77,
77,
77,
78,
78,
78,
78,
79,
79,
92,
94,
96,
97,
97,
98,
100,
101,
102,
102,
102,
102,
103,
102,
102,
103,
104,
106,
108,
107,
105,
100,
96,
96,
99,
98,
91,
89,
87,
87,
87,
87,
86,
86,
86,
86,
89,
94,
98,
100,
103,
108,
111,
114,
116,
116,
116,
117,
116,
116,
116,
116,
116,
118,
120,
120,
121,
121,
121,
121,
122,
123,
125,
127,
130,
131,
133,
134,
134,
134,
132,
129,
128,
129,
128,
127,
127,
128,
130,
132,
133,
134,
136,
137,
139,
137,
140,
143,
145,
144,
141,
134,
135,
139,
141,
141,
141,
139,
134,
135,
122,
123,
121,
117,
115,
113,
114,
116,
117,
118,
120,
122,
124,
122,
118,
115,
116,
116,
116,
116,
116,
116,
116,
113,
110,
109,
109,
109,
109,
110,
112,
114,
115,
117,
118,
121,
125,
129,
130,
129,
128,
128,
122,
121,
120,
118,
117,
117,
116,
110,
108,
105,
100,
97,
96,
96,
95,
95,
94,
94,
94,
94,
94,
95,
95,
94,
94,
94,
93,
93,
92,
92,
91,
91,
91,
91,
92,
92,
92,
92,
92,
92,
92,
91,
91,
91,
91,
91,
91,
91,
91,
91,
92,
91,
91,
91,
91,
91,
91,
90,
90,
89,
88,
85,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
76,
76,
76,
77,
77,
78,
78,
78,
78,
79,
80,
80,
93,
98,
99,
100,
101,
101,
102,
103,
103,
102,
102,
102,
103,
102,
102,
102,
103,
104,
105,
106,
104,
100,
96,
97,
103,
102,
94,
91,
90,
89,
88,
88,
87,
87,
86,
86,
86,
86,
88,
91,
93,
96,
102,
108,
112,
113,
114,
117,
118,
118,
119,
119,
119,
118,
119,
119,
119,
120,
120,
120,
121,
122,
124,
127,
128,
130,
130,
128,
127,
126,
124,
119,
116,
116,
118,
120,
121,
124,
129,
131,
132,
133,
135,
138,
141,
141,
143,
146,
147,
147,
142,
136,
135,
138,
141,
142,
142,
140,
138,
138,
123,
123,
120,
117,
115,
114,
114,
115,
117,
120,
124,
129,
128,
123,
117,
116,
116,
117,
117,
117,
116,
116,
115,
113,
111,
109,
108,
108,
109,
112,
114,
116,
117,
118,
121,
124,
128,
129,
131,
131,
130,
128,
122,
121,
120,
119,
118,
117,
115,
110,
106,
103,
99,
97,
96,
96,
95,
95,
94,
94,
94,
94,
94,
95,
95,
94,
94,
94,
93,
93,
92,
92,
92,
91,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
91,
91,
91,
91,
91,
91,
91,
91,
92,
92,
92,
91,
91,
91,
91,
91,
90,
89,
88,
85,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
76,
76,
76,
77,
77,
77,
78,
78,
78,
79,
79,
90,
97,
101,
103,
103,
103,
104,
105,
105,
105,
104,
104,
104,
104,
104,
103,
102,
102,
103,
104,
105,
104,
101,
97,
99,
105,
105,
98,
95,
92,
91,
90,
89,
89,
88,
87,
86,
86,
89,
88,
91,
92,
93,
92,
96,
103,
108,
111,
116,
117,
122,
122,
122,
122,
118,
119,
120,
120,
119,
119,
120,
121,
123,
123,
124,
125,
126,
124,
120,
115,
111,
112,
113,
116,
121,
125,
128,
129,
130,
135,
135,
134,
133,
134,
135,
139,
142,
144,
147,
148,
148,
142,
139,
138,
139,
140,
142,
142,
140,
140,
142,
125,
123,
120,
115,
114,
114,
114,
116,
118,
121,
128,
128,
126,
123,
116,
116,
117,
117,
116,
116,
116,
115,
115,
115,
113,
111,
110,
109,
110,
114,
116,
117,
118,
120,
122,
127,
129,
130,
130,
130,
129,
127,
122,
121,
120,
119,
118,
117,
114,
110,
105,
102,
99,
97,
97,
96,
95,
94,
94,
94,
94,
95,
95,
95,
95,
94,
94,
94,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
91,
91,
91,
91,
91,
91,
91,
92,
92,
92,
92,
92,
92,
91,
91,
91,
90,
88,
85,
84,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
76,
76,
76,
76,
77,
77,
77,
78,
78,
79,
80,
82,
93,
98,
103,
104,
105,
105,
106,
108,
109,
108,
107,
107,
107,
107,
107,
106,
104,
103,
103,
103,
104,
104,
102,
98,
100,
103,
104,
104,
101,
96,
91,
91,
90,
90,
90,
89,
86,
86,
90,
91,
92,
92,
92,
92,
93,
92,
94,
103,
109,
115,
118,
120,
121,
120,
117,
118,
121,
122,
119,
118,
118,
120,
122,
121,
122,
119,
118,
112,
107,
109,
113,
118,
123,
127,
130,
133,
136,
136,
135,
138,
139,
137,
135,
133,
135,
135,
138,
142,
146,
147,
146,
142,
141,
140,
142,
142,
143,
143,
143,
144,
145,
127,
127,
120,
116,
114,
114,
115,
116,
120,
124,
129,
129,
124,
122,
116,
116,
117,
117,
116,
116,
116,
115,
116,
117,
116,
114,
112,
112,
113,
115,
117,
118,
118,
120,
122,
128,
130,
130,
131,
130,
129,
126,
122,
121,
120,
119,
117,
116,
111,
108,
104,
102,
99,
98,
97,
95,
94,
94,
94,
94,
94,
95,
95,
95,
95,
94,
94,
94,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
91,
91,
91,
91,
91,
91,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
91,
88,
86,
85,
84,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
76,
76,
77,
77,
76,
77,
76,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
76,
76,
76,
76,
76,
77,
77,
77,
78,
78,
79,
81,
85,
93,
95,
99,
104,
104,
106,
108,
109,
112,
112,
110,
111,
112,
113,
112,
110,
108,
106,
104,
103,
104,
105,
104,
101,
101,
104,
105,
105,
103,
102,
96,
93,
91,
90,
91,
92,
89,
86,
91,
92,
92,
93,
93,
92,
92,
92,
92,
92,
97,
103,
107,
110,
111,
110,
113,
116,
119,
122,
119,
117,
116,
117,
119,
120,
112,
107,
105,
104,
110,
117,
124,
128,
131,
134,
138,
140,
141,
140,
140,
142,
143,
142,
141,
136,
134,
134,
134,
139,
144,
146,
146,
146,
146,
145,
146,
146,
145,
145,
145,
147,
148,
129,
129,
120,
116,
114,
115,
116,
118,
121,
124,
127,
126,
122,
120,
116,
117,
118,
117,
117,
117,
116,
116,
116,
117,
117,
116,
114,
115,
116,
116,
117,
118,
119,
120,
123,
128,
130,
131,
131,
131,
128,
126,
122,
121,
119,
118,
117,
114,
109,
105,
103,
102,
100,
98,
98,
96,
94,
94,
94,
94,
94,
95,
95,
95,
95,
94,
94,
94,
94,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
91,
91,
91,
91,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
88,
86,
85,
84,
84,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
78,
78,
80,
79,
89,
92,
95,
96,
100,
104,
105,
107,
109,
113,
117,
117,
117,
119,
121,
120,
117,
111,
108,
106,
105,
105,
105,
104,
102,
103,
103,
104,
106,
106,
104,
104,
103,
92,
91,
92,
93,
92,
91,
91,
91,
91,
91,
92,
94,
94,
94,
94,
92,
92,
92,
92,
95,
96,
99,
106,
110,
112,
114,
114,
111,
107,
109,
109,
106,
102,
102,
102,
110,
119,
126,
130,
134,
137,
141,
144,
145,
145,
145,
145,
145,
145,
145,
143,
139,
134,
136,
136,
134,
141,
146,
147,
147,
148,
148,
148,
148,
146,
146,
148,
149,
151,
130,
129,
120,
116,
115,
115,
117,
119,
121,
123,
124,
123,
121,
119,
117,
117,
118,
118,
119,
118,
117,
117,
118,
118,
117,
117,
116,
116,
117,
117,
118,
118,
119,
120,
123,
128,
130,
132,
131,
132,
130,
127,
122,
121,
119,
118,
117,
113,
107,
104,
103,
102,
100,
99,
98,
97,
94,
94,
94,
94,
94,
95,
95,
95,
95,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
91,
91,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
88,
87,
86,
85,
85,
84,
84,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
78,
79,
79,
92,
96,
97,
96,
96,
97,
101,
104,
106,
108,
112,
117,
119,
121,
123,
124,
124,
121,
117,
110,
108,
107,
107,
106,
104,
103,
104,
105,
105,
106,
107,
106,
105,
104,
100,
94,
93,
94,
93,
93,
92,
92,
91,
91,
91,
92,
94,
96,
97,
97,
96,
95,
94,
92,
92,
92,
94,
102,
105,
105,
103,
97,
97,
101,
101,
102,
104,
108,
109,
114,
124,
132,
135,
138,
141,
146,
146,
147,
147,
147,
147,
148,
148,
146,
145,
142,
140,
141,
141,
140,
140,
144,
147,
148,
149,
150,
151,
150,
146,
150,
152,
152,
154,
129,
127,
120,
116,
115,
116,
118,
120,
121,
122,
122,
122,
120,
120,
119,
118,
119,
119,
120,
120,
120,
120,
119,
119,
118,
118,
117,
117,
117,
117,
117,
119,
120,
121,
124,
129,
130,
132,
133,
132,
130,
127,
122,
120,
118,
117,
116,
111,
104,
103,
102,
101,
101,
100,
99,
97,
94,
94,
94,
95,
95,
95,
95,
95,
95,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
88,
86,
85,
85,
85,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
79,
86,
97,
100,
101,
99,
97,
98,
97,
102,
105,
107,
110,
115,
119,
123,
126,
129,
127,
123,
119,
113,
109,
109,
108,
108,
105,
104,
104,
107,
108,
108,
108,
107,
107,
106,
105,
100,
96,
96,
95,
95,
94,
93,
92,
92,
91,
91,
94,
97,
100,
103,
104,
104,
104,
101,
98,
94,
92,
96,
98,
97,
97,
100,
104,
106,
109,
111,
114,
116,
116,
119,
127,
135,
137,
140,
143,
146,
146,
147,
148,
147,
148,
148,
148,
147,
146,
144,
144,
144,
144,
142,
141,
143,
146,
147,
149,
151,
152,
153,
152,
153,
154,
156,
157,
128,
125,
119,
115,
116,
117,
119,
121,
122,
122,
122,
121,
120,
120,
120,
120,
120,
119,
121,
121,
121,
121,
121,
121,
119,
117,
116,
117,
116,
116,
117,
119,
121,
123,
125,
128,
130,
132,
133,
134,
131,
126,
122,
119,
117,
116,
112,
109,
105,
103,
102,
101,
101,
100,
99,
98,
94,
94,
94,
95,
95,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
89,
86,
85,
85,
85,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
76,
76,
77,
76,
76,
76,
76,
76,
76,
77,
77,
92,
99,
104,
106,
104,
102,
99,
97,
99,
104,
106,
109,
113,
117,
122,
126,
129,
127,
124,
119,
115,
112,
112,
111,
110,
108,
105,
107,
110,
110,
110,
109,
108,
108,
107,
106,
104,
97,
100,
98,
97,
96,
95,
94,
92,
91,
91,
94,
97,
103,
105,
106,
107,
109,
110,
104,
100,
97,
96,
96,
96,
99,
104,
109,
113,
116,
120,
122,
125,
128,
125,
132,
136,
138,
140,
142,
144,
146,
146,
147,
147,
147,
147,
147,
145,
144,
145,
146,
147,
147,
145,
143,
144,
145,
147,
148,
150,
152,
153,
154,
155,
156,
158,
161,
128,
123,
119,
116,
116,
118,
120,
121,
122,
122,
121,
121,
121,
121,
121,
121,
121,
120,
121,
122,
122,
122,
121,
122,
120,
116,
116,
116,
116,
116,
117,
119,
122,
124,
127,
128,
130,
132,
134,
133,
130,
125,
121,
117,
115,
112,
110,
107,
104,
103,
102,
101,
101,
100,
99,
98,
94,
94,
94,
95,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
89,
87,
85,
85,
85,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
76,
76,
76,
76,
76,
78,
79,
95,
102,
109,
111,
109,
106,
103,
100,
98,
102,
105,
108,
111,
115,
119,
123,
126,
125,
124,
120,
116,
116,
116,
116,
115,
111,
109,
110,
112,
114,
114,
111,
109,
108,
108,
107,
105,
103,
108,
110,
107,
98,
96,
95,
93,
91,
91,
94,
98,
104,
106,
107,
109,
110,
110,
110,
108,
105,
101,
96,
97,
97,
97,
104,
112,
120,
124,
126,
129,
128,
128,
134,
137,
138,
140,
141,
142,
143,
144,
145,
145,
145,
144,
142,
139,
141,
145,
148,
151,
150,
147,
145,
145,
145,
146,
147,
148,
148,
152,
154,
156,
158,
162,
164,
125,
122,
119,
117,
117,
119,
121,
122,
122,
122,
121,
121,
121,
122,
122,
122,
121,
120,
121,
123,
122,
122,
122,
122,
121,
116,
117,
117,
116,
116,
118,
120,
122,
125,
128,
129,
131,
132,
134,
133,
130,
125,
121,
118,
116,
112,
108,
105,
103,
102,
102,
101,
102,
101,
99,
97,
94,
94,
94,
95,
95,
96,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
89,
87,
86,
85,
85,
85,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
77,
77,
77,
88,
98,
104,
111,
114,
114,
111,
107,
104,
102,
102,
104,
107,
110,
113,
116,
120,
123,
124,
123,
121,
120,
119,
119,
119,
118,
116,
110,
112,
115,
117,
117,
116,
113,
109,
109,
108,
107,
107,
108,
111,
110,
106,
97,
95,
93,
92,
92,
93,
98,
104,
105,
107,
108,
110,
112,
112,
111,
111,
107,
101,
102,
104,
103,
102,
102,
108,
117,
122,
126,
129,
130,
134,
136,
137,
138,
137,
138,
140,
142,
143,
143,
142,
140,
138,
136,
139,
145,
150,
153,
152,
148,
147,
145,
144,
144,
145,
146,
147,
153,
155,
157,
162,
166,
169,
122,
121,
119,
119,
118,
119,
121,
122,
123,
122,
121,
121,
122,
122,
121,
120,
119,
118,
120,
122,
122,
123,
122,
122,
120,
119,
118,
118,
118,
118,
119,
120,
122,
125,
129,
130,
131,
132,
133,
132,
130,
126,
121,
120,
119,
113,
108,
104,
102,
101,
101,
101,
104,
103,
98,
95,
94,
94,
94,
95,
95,
96,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
89,
87,
85,
85,
85,
85,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
76,
76,
77,
77,
77,
77,
77,
77,
78,
78,
79,
87,
96,
101,
105,
111,
116,
117,
115,
111,
107,
104,
103,
103,
106,
108,
111,
114,
117,
120,
122,
123,
122,
122,
122,
122,
121,
120,
117,
115,
114,
117,
119,
119,
119,
117,
114,
110,
110,
109,
108,
110,
113,
116,
108,
97,
95,
94,
92,
92,
92,
95,
98,
103,
105,
107,
110,
112,
112,
113,
112,
111,
105,
107,
110,
111,
108,
104,
102,
103,
110,
117,
123,
128,
132,
135,
134,
135,
133,
134,
136,
140,
141,
141,
139,
136,
134,
133,
137,
142,
147,
148,
147,
147,
146,
144,
142,
141,
142,
146,
150,
154,
157,
160,
167,
171,
175,
121,
120,
119,
119,
118,
119,
121,
122,
123,
122,
121,
122,
122,
122,
121,
118,
115,
117,
119,
120,
121,
122,
122,
122,
121,
121,
120,
119,
120,
121,
121,
121,
122,
124,
129,
130,
129,
129,
129,
130,
130,
129,
126,
124,
121,
113,
107,
103,
102,
100,
100,
101,
103,
100,
97,
94,
94,
95,
95,
95,
95,
96,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
89,
87,
85,
85,
85,
85,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
76,
76,
76,
77,
77,
77,
77,
77,
77,
78,
78,
79,
87,
96,
100,
103,
107,
112,
117,
118,
117,
115,
110,
107,
104,
104,
105,
107,
109,
112,
115,
118,
121,
124,
125,
125,
126,
126,
125,
122,
118,
115,
116,
118,
121,
122,
121,
120,
117,
116,
112,
109,
109,
110,
115,
117,
108,
98,
96,
94,
93,
92,
92,
93,
95,
97,
99,
105,
110,
112,
112,
113,
114,
112,
110,
111,
115,
117,
117,
113,
107,
102,
102,
107,
114,
120,
126,
130,
132,
131,
130,
131,
133,
135,
138,
138,
135,
131,
128,
131,
136,
141,
144,
144,
144,
142,
142,
141,
139,
137,
142,
147,
152,
156,
159,
165,
172,
176,
183,
120,
120,
119,
119,
120,
120,
122,
124,
123,
122,
122,
122,
122,
122,
122,
116,
116,
116,
117,
119,
120,
122,
121,
121,
122,
122,
121,
120,
121,
122,
123,
124,
123,
126,
128,
128,
128,
127,
128,
129,
130,
129,
129,
125,
118,
111,
107,
104,
100,
97,
99,
100,
99,
98,
95,
94,
94,
95,
95,
95,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
89,
88,
86,
85,
85,
85,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
78,
78,
78,
79,
87,
97,
99,
102,
105,
108,
112,
116,
119,
119,
118,
114,
110,
106,
104,
104,
107,
109,
112,
114,
118,
122,
125,
127,
128,
129,
130,
129,
125,
121,
117,
117,
120,
123,
124,
124,
121,
119,
117,
112,
110,
109,
114,
115,
117,
109,
99,
96,
95,
93,
92,
92,
92,
94,
95,
96,
101,
108,
111,
112,
113,
113,
114,
115,
115,
116,
117,
118,
121,
119,
109,
103,
102,
106,
113,
117,
122,
125,
126,
126,
128,
131,
133,
134,
135,
132,
127,
127,
131,
136,
139,
140,
140,
140,
138,
136,
135,
134,
133,
142,
148,
153,
158,
164,
171,
177,
181,
184,
120,
120,
119,
120,
120,
122,
124,
128,
123,
122,
122,
122,
122,
121,
119,
115,
115,
116,
117,
119,
120,
121,
122,
122,
122,
122,
121,
120,
122,
123,
125,
127,
126,
127,
128,
127,
127,
126,
127,
128,
129,
128,
128,
123,
115,
111,
107,
104,
100,
97,
98,
98,
98,
96,
94,
94,
94,
95,
95,
96,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
90,
89,
86,
85,
85,
85,
85,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
78,
78,
78,
78,
79,
87,
94,
98,
100,
103,
106,
109,
112,
116,
119,
122,
122,
119,
114,
110,
108,
106,
107,
109,
111,
114,
118,
122,
126,
129,
130,
132,
132,
132,
128,
122,
119,
120,
122,
126,
128,
128,
123,
120,
117,
110,
109,
114,
117,
117,
117,
110,
105,
98,
96,
94,
92,
92,
92,
93,
94,
96,
98,
104,
110,
112,
113,
114,
115,
116,
117,
117,
118,
119,
121,
122,
118,
111,
105,
102,
102,
106,
113,
118,
121,
121,
124,
129,
131,
132,
132,
129,
123,
124,
131,
135,
135,
135,
135,
135,
133,
131,
131,
132,
133,
141,
148,
154,
160,
166,
175,
181,
188,
187,
120,
119,
119,
119,
121,
124,
128,
128,
123,
122,
122,
122,
122,
120,
115,
115,
115,
116,
117,
119,
120,
121,
122,
123,
122,
120,
118,
119,
121,
123,
125,
128,
128,
127,
127,
126,
126,
125,
124,
126,
128,
128,
125,
120,
114,
110,
107,
104,
100,
97,
97,
97,
96,
95,
94,
94,
94,
95,
95,
96,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
89,
87,
85,
85,
85,
85,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
78,
78,
78,
81,
85,
93,
95,
98,
100,
103,
106,
110,
112,
116,
120,
123,
126,
123,
119,
115,
110,
108,
108,
109,
111,
114,
118,
122,
126,
130,
133,
134,
135,
135,
131,
124,
122,
121,
123,
128,
129,
128,
126,
121,
116,
109,
114,
117,
119,
119,
117,
111,
107,
104,
97,
96,
93,
92,
92,
93,
94,
95,
97,
103,
110,
111,
112,
114,
115,
116,
117,
118,
119,
120,
120,
121,
122,
122,
116,
109,
106,
104,
105,
110,
115,
115,
122,
126,
129,
130,
129,
125,
121,
123,
127,
129,
128,
127,
127,
127,
126,
127,
129,
131,
133,
137,
147,
154,
161,
167,
176,
184,
190,
186,
119,
120,
120,
120,
121,
128,
128,
128,
123,
121,
121,
120,
121,
121,
116,
116,
116,
118,
119,
120,
120,
121,
122,
122,
120,
117,
116,
116,
118,
121,
124,
125,
125,
125,
125,
125,
125,
124,
121,
122,
124,
124,
122,
115,
111,
108,
105,
103,
100,
98,
95,
95,
95,
94,
94,
94,
94,
95,
95,
96,
95,
96,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
89,
87,
85,
85,
85,
85,
84,
84,
84,
84,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
78,
78,
79,
81,
91,
93,
93,
96,
98,
102,
105,
109,
112,
116,
120,
125,
129,
128,
123,
119,
115,
110,
109,
108,
110,
113,
117,
120,
125,
130,
134,
137,
139,
138,
134,
127,
121,
122,
125,
129,
130,
129,
127,
122,
116,
115,
117,
119,
121,
121,
119,
116,
112,
108,
105,
101,
97,
92,
92,
93,
94,
95,
97,
101,
108,
111,
112,
114,
115,
116,
117,
118,
120,
120,
121,
121,
122,
122,
121,
121,
116,
111,
107,
109,
109,
111,
118,
122,
125,
126,
124,
121,
119,
120,
122,
123,
123,
122,
122,
126,
128,
130,
132,
133,
131,
133,
142,
152,
160,
166,
175,
184,
188,
185,
119,
121,
121,
121,
122,
129,
128,
127,
122,
121,
121,
120,
119,
118,
118,
117,
118,
120,
122,
122,
121,
120,
120,
119,
117,
115,
115,
116,
115,
118,
120,
122,
122,
123,
124,
125,
124,
123,
120,
117,
119,
119,
116,
113,
109,
106,
104,
102,
100,
98,
96,
94,
94,
94,
94,
94,
94,
95,
95,
96,
97,
96,
96,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
89,
86,
85,
85,
85,
85,
84,
84,
84,
84,
83,
83,
83,
83,
83,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
78,
78,
78,
78,
78,
78,
78,
78,
83,
85,
89,
93,
95,
98,
103,
108,
111,
115,
120,
125,
129,
128,
128,
121,
118,
114,
110,
109,
109,
113,
116,
119,
123,
129,
134,
138,
141,
142,
137,
128,
123,
123,
127,
130,
130,
129,
127,
122,
115,
116,
118,
121,
126,
125,
121,
120,
117,
113,
110,
108,
104,
93,
92,
93,
94,
95,
97,
100,
105,
110,
112,
113,
115,
116,
117,
119,
120,
121,
122,
122,
122,
123,
124,
123,
121,
122,
116,
109,
110,
109,
110,
115,
119,
119,
118,
116,
115,
117,
119,
121,
121,
123,
126,
127,
130,
132,
137,
141,
142,
140,
141,
150,
158,
164,
172,
182,
186,
185,
119,
122,
122,
122,
125,
129,
129,
127,
123,
122,
121,
120,
119,
119,
119,
120,
121,
124,
125,
122,
121,
121,
121,
120,
118,
116,
117,
116,
116,
116,
118,
120,
122,
123,
124,
123,
124,
122,
118,
114,
113,
113,
112,
110,
107,
103,
102,
101,
100,
99,
96,
94,
94,
94,
94,
94,
94,
94,
95,
96,
97,
97,
97,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
89,
86,
85,
85,
85,
85,
84,
84,
84,
84,
83,
83,
83,
83,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
78,
78,
78,
78,
78,
78,
79,
79,
78,
79,
82,
86,
92,
95,
100,
105,
110,
114,
119,
123,
127,
128,
127,
125,
120,
116,
112,
110,
109,
112,
115,
118,
122,
127,
132,
137,
141,
141,
139,
132,
128,
127,
129,
130,
130,
130,
126,
121,
116,
118,
121,
126,
130,
130,
128,
127,
122,
119,
117,
113,
106,
95,
92,
93,
94,
95,
97,
100,
104,
110,
111,
113,
115,
116,
118,
119,
120,
122,
122,
123,
124,
124,
123,
123,
124,
124,
124,
118,
109,
110,
110,
110,
110,
112,
115,
115,
115,
115,
119,
123,
126,
129,
130,
132,
133,
136,
142,
146,
148,
148,
143,
148,
156,
163,
171,
182,
186,
184,
122,
122,
123,
124,
125,
128,
127,
125,
124,
123,
122,
120,
120,
119,
119,
121,
125,
129,
128,
122,
121,
122,
122,
122,
120,
119,
119,
118,
118,
118,
119,
120,
122,
123,
123,
124,
123,
121,
116,
111,
109,
109,
109,
108,
106,
103,
101,
100,
99,
98,
96,
94,
94,
94,
94,
94,
94,
94,
94,
94,
96,
97,
97,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
88,
86,
85,
85,
85,
85,
84,
84,
84,
84,
83,
83,
83,
83,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
78,
78,
78,
78,
78,
79,
79,
80,
80,
83,
82,
79,
79,
85,
92,
97,
103,
108,
112,
116,
121,
124,
127,
127,
128,
121,
118,
114,
112,
111,
112,
114,
117,
120,
125,
130,
134,
139,
141,
140,
135,
128,
129,
131,
131,
131,
129,
125,
120,
119,
119,
124,
130,
134,
134,
135,
134,
128,
125,
121,
116,
107,
97,
94,
94,
94,
95,
97,
99,
103,
108,
111,
112,
114,
116,
117,
119,
120,
122,
123,
123,
124,
124,
124,
124,
123,
124,
122,
122,
123,
123,
123,
120,
114,
112,
115,
117,
117,
117,
119,
121,
126,
130,
134,
138,
140,
140,
144,
148,
152,
151,
145,
150,
157,
164,
175,
188,
190,
183,
123,
123,
122,
122,
125,
125,
122,
122,
123,
123,
122,
120,
120,
119,
121,
123,
128,
128,
126,
122,
122,
123,
122,
122,
122,
121,
121,
121,
121,
121,
121,
121,
122,
123,
124,
123,
121,
117,
114,
111,
107,
105,
107,
106,
105,
102,
100,
98,
97,
96,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
96,
98,
97,
96,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
91,
90,
88,
86,
85,
85,
85,
85,
84,
84,
84,
84,
83,
83,
83,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
77,
78,
78,
78,
78,
78,
79,
79,
80,
81,
90,
92,
92,
89,
85,
86,
85,
92,
98,
104,
109,
113,
116,
120,
122,
125,
127,
122,
119,
116,
114,
114,
114,
114,
116,
119,
123,
127,
131,
135,
139,
140,
135,
133,
132,
132,
133,
132,
129,
125,
120,
120,
121,
127,
132,
134,
135,
136,
135,
135,
130,
124,
119,
110,
97,
96,
95,
94,
95,
96,
97,
100,
104,
107,
110,
112,
115,
117,
117,
118,
122,
123,
124,
124,
124,
125,
124,
123,
121,
121,
121,
123,
127,
127,
127,
123,
121,
121,
122,
121,
122,
122,
122,
121,
127,
134,
141,
140,
143,
145,
150,
152,
153,
148,
153,
157,
166,
177,
189,
190,
181,
128,
123,
122,
122,
122,
122,
122,
122,
122,
122,
122,
120,
120,
119,
121,
125,
129,
128,
126,
123,
123,
123,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
123,
123,
123,
121,
118,
115,
112,
109,
105,
103,
105,
106,
104,
101,
99,
97,
96,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
97,
98,
97,
96,
96,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
91,
90,
89,
86,
85,
85,
85,
85,
84,
84,
84,
84,
83,
83,
83,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
77,
77,
77,
78,
78,
77,
77,
77,
77,
78,
77,
77,
78,
78,
78,
78,
78,
79,
79,
79,
88,
93,
96,
98,
97,
93,
87,
87,
85,
91,
98,
103,
110,
113,
116,
118,
120,
123,
122,
120,
118,
117,
116,
116,
115,
115,
118,
121,
125,
128,
133,
136,
137,
135,
134,
133,
134,
134,
133,
130,
125,
121,
120,
122,
128,
132,
135,
135,
135,
135,
135,
131,
127,
123,
114,
101,
98,
95,
95,
96,
96,
96,
97,
99,
102,
103,
104,
107,
110,
112,
113,
117,
122,
122,
124,
124,
123,
123,
120,
116,
113,
117,
121,
126,
131,
128,
127,
130,
129,
129,
128,
128,
128,
125,
121,
123,
130,
136,
139,
143,
147,
152,
152,
152,
151,
154,
158,
167,
177,
190,
187,
179,
129,
124,
124,
122,
123,
122,
121,
122,
121,
122,
121,
120,
120,
120,
122,
126,
129,
128,
125,
123,
123,
123,
122,
122,
122,
122,
122,
122,
123,
123,
123,
123,
123,
123,
122,
119,
117,
114,
111,
106,
103,
102,
104,
105,
103,
98,
97,
97,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
96,
97,
96,
96,
96,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
91,
91,
89,
87,
85,
85,
85,
85,
85,
84,
84,
84,
83,
83,
83,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
77,
77,
78,
78,
78,
78,
78,
78,
78,
78,
78,
79,
79,
82,
90,
94,
98,
99,
99,
99,
94,
87,
86,
85,
88,
96,
103,
109,
112,
114,
116,
121,
123,
123,
122,
120,
119,
118,
117,
116,
117,
120,
123,
126,
130,
134,
136,
135,
134,
134,
135,
135,
134,
130,
126,
122,
121,
121,
127,
131,
134,
134,
135,
136,
134,
132,
130,
128,
119,
107,
100,
98,
97,
95,
96,
95,
96,
97,
99,
101,
102,
103,
103,
104,
105,
108,
116,
122,
121,
123,
120,
114,
109,
111,
111,
108,
112,
116,
121,
126,
130,
128,
130,
131,
131,
131,
130,
129,
126,
122,
123,
129,
134,
143,
148,
151,
149,
145,
148,
154,
158,
166,
176,
189,
184,
177,
129,
128,
124,
126,
124,
122,
121,
121,
121,
121,
120,
121,
121,
121,
122,
127,
129,
128,
127,
124,
123,
123,
122,
122,
122,
122,
122,
122,
123,
123,
123,
122,
122,
121,
120,
117,
115,
113,
110,
105,
102,
101,
102,
102,
99,
97,
97,
96,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
96,
96,
95,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
87,
85,
85,
85,
85,
85,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
79,
79,
81,
83,
91,
95,
98,
100,
103,
102,
99,
94,
85,
86,
88,
97,
105,
108,
110,
114,
117,
121,
124,
126,
126,
124,
123,
121,
120,
119,
119,
120,
122,
125,
128,
132,
135,
136,
135,
136,
136,
137,
135,
132,
129,
125,
123,
122,
127,
129,
133,
134,
134,
136,
135,
133,
131,
129,
122,
116,
113,
109,
99,
97,
95,
95,
95,
97,
98,
99,
100,
101,
102,
102,
102,
103,
104,
114,
117,
114,
110,
104,
104,
107,
108,
109,
110,
109,
112,
116,
121,
126,
129,
133,
134,
133,
132,
132,
130,
128,
125,
126,
130,
138,
146,
147,
142,
144,
147,
153,
157,
165,
174,
185,
181,
174,
132,
131,
129,
130,
124,
121,
120,
121,
121,
120,
120,
120,
120,
121,
123,
128,
129,
129,
128,
123,
123,
123,
123,
122,
122,
122,
122,
122,
123,
123,
122,
121,
121,
119,
118,
116,
113,
110,
107,
103,
100,
99,
99,
99,
98,
97,
96,
96,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
94,
95,
94,
94,
94,
94,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
86,
85,
85,
85,
85,
85,
84,
84,
84,
84,
83,
83,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
79,
79,
80,
89,
93,
95,
97,
100,
103,
105,
102,
98,
86,
86,
90,
100,
109,
113,
115,
116,
121,
123,
126,
128,
129,
129,
128,
126,
123,
121,
120,
120,
121,
123,
126,
130,
134,
136,
137,
138,
138,
137,
136,
133,
131,
129,
122,
126,
130,
134,
135,
138,
139,
140,
136,
133,
133,
130,
127,
125,
120,
114,
105,
95,
96,
97,
96,
97,
98,
99,
99,
100,
100,
101,
101,
101,
103,
103,
105,
104,
104,
104,
106,
108,
110,
110,
109,
110,
110,
110,
112,
117,
123,
132,
134,
135,
135,
135,
134,
132,
130,
129,
128,
132,
135,
135,
139,
143,
147,
153,
157,
166,
174,
185,
178,
172,
135,
134,
132,
130,
122,
120,
121,
120,
120,
120,
121,
121,
122,
122,
124,
127,
128,
129,
128,
126,
126,
124,
123,
122,
122,
122,
122,
122,
123,
122,
122,
121,
120,
117,
116,
114,
111,
107,
103,
99,
98,
97,
97,
97,
97,
97,
96,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
92,
91,
86,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
83,
84,
83,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
79,
79,
79,
85,
92,
94,
95,
97,
101,
103,
104,
104,
99,
87,
87,
90,
102,
112,
118,
120,
120,
123,
126,
128,
131,
132,
133,
132,
130,
128,
126,
123,
121,
121,
122,
124,
127,
131,
134,
137,
139,
139,
139,
137,
134,
132,
131,
128,
127,
131,
135,
139,
141,
143,
142,
140,
136,
135,
133,
131,
130,
127,
120,
115,
110,
105,
98,
97,
97,
97,
98,
99,
99,
99,
99,
99,
100,
101,
102,
102,
104,
104,
107,
109,
109,
118,
120,
118,
115,
110,
110,
111,
112,
120,
127,
135,
136,
139,
138,
139,
140,
137,
133,
132,
133,
132,
136,
140,
143,
149,
154,
158,
166,
175,
182,
176,
171,
135,
134,
134,
128,
121,
121,
120,
121,
120,
121,
122,
122,
122,
122,
124,
125,
125,
127,
127,
126,
128,
127,
122,
122,
122,
122,
122,
122,
122,
123,
123,
122,
119,
117,
114,
112,
109,
105,
100,
97,
97,
97,
97,
97,
97,
96,
96,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
92,
89,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
83,
84,
83,
83,
83,
83,
83,
83,
83,
83,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
79,
79,
79,
80,
83,
90,
93,
95,
96,
99,
101,
104,
104,
104,
96,
87,
87,
90,
103,
114,
123,
125,
125,
126,
128,
132,
134,
136,
136,
135,
134,
132,
130,
127,
124,
122,
122,
123,
126,
130,
133,
136,
139,
140,
140,
139,
136,
134,
135,
132,
130,
132,
135,
141,
144,
146,
146,
144,
140,
138,
135,
133,
130,
128,
124,
119,
115,
110,
104,
98,
97,
97,
97,
98,
98,
98,
98,
98,
99,
100,
101,
103,
104,
110,
114,
117,
121,
123,
124,
127,
126,
122,
116,
110,
110,
117,
125,
130,
134,
140,
139,
141,
140,
141,
141,
140,
140,
138,
134,
139,
144,
150,
155,
158,
166,
176,
178,
176,
172,
133,
135,
131,
125,
122,
121,
120,
121,
121,
122,
122,
123,
123,
123,
123,
122,
122,
125,
125,
126,
128,
127,
122,
122,
122,
122,
122,
122,
122,
122,
122,
121,
118,
117,
113,
109,
106,
103,
98,
97,
97,
97,
97,
96,
96,
96,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
92,
91,
88,
85,
85,
85,
85,
85,
85,
85,
84,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
83,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
79,
79,
79,
80,
81,
89,
92,
94,
96,
98,
100,
103,
104,
103,
99,
91,
87,
87,
92,
104,
115,
124,
128,
130,
131,
133,
135,
138,
140,
140,
139,
137,
135,
133,
130,
127,
125,
123,
122,
125,
129,
133,
136,
139,
141,
141,
141,
140,
139,
139,
137,
133,
133,
134,
140,
145,
147,
147,
148,
144,
140,
136,
133,
130,
127,
122,
115,
109,
104,
102,
98,
97,
97,
98,
97,
97,
97,
97,
97,
98,
99,
101,
102,
103,
111,
118,
122,
123,
125,
127,
129,
129,
126,
126,
119,
111,
113,
121,
128,
131,
134,
136,
139,
141,
142,
143,
143,
143,
142,
140,
140,
145,
152,
155,
159,
166,
176,
178,
177,
172,
131,
131,
127,
125,
122,
121,
120,
120,
120,
121,
122,
123,
123,
123,
122,
122,
122,
122,
124,
124,
125,
123,
122,
122,
122,
122,
122,
122,
122,
121,
121,
119,
117,
116,
112,
108,
104,
100,
97,
97,
97,
97,
96,
96,
96,
95,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
91,
90,
87,
86,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
78,
78,
78,
79,
78,
78,
78,
78,
78,
78,
79,
79,
79,
80,
86,
92,
93,
95,
96,
99,
102,
104,
106,
105,
98,
88,
87,
87,
95,
105,
114,
124,
128,
131,
134,
136,
138,
141,
141,
142,
141,
141,
139,
136,
134,
131,
128,
126,
125,
125,
128,
133,
136,
138,
141,
142,
142,
142,
141,
141,
141,
136,
135,
137,
140,
145,
148,
149,
147,
144,
140,
136,
132,
129,
124,
120,
117,
116,
111,
106,
101,
97,
102,
99,
97,
97,
97,
98,
97,
98,
99,
100,
101,
102,
105,
116,
123,
124,
127,
128,
130,
130,
129,
128,
126,
121,
112,
115,
121,
126,
128,
132,
135,
140,
141,
143,
144,
145,
146,
144,
143,
147,
153,
157,
162,
168,
176,
183,
179,
173,
129,
126,
122,
122,
123,
122,
121,
119,
120,
120,
122,
123,
122,
122,
122,
122,
122,
122,
123,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
121,
120,
117,
115,
113,
110,
107,
102,
98,
97,
97,
97,
96,
96,
96,
96,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
92,
91,
90,
89,
87,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
78,
79,
79,
79,
79,
79,
79,
78,
78,
78,
79,
79,
79,
79,
85,
91,
93,
94,
96,
97,
100,
103,
107,
109,
106,
99,
89,
87,
87,
96,
105,
113,
121,
127,
130,
133,
135,
137,
140,
142,
142,
143,
142,
141,
140,
137,
135,
131,
129,
127,
127,
127,
131,
134,
137,
141,
142,
141,
142,
142,
143,
142,
140,
139,
142,
143,
146,
151,
152,
147,
143,
140,
135,
131,
128,
122,
123,
125,
123,
116,
108,
100,
100,
107,
107,
102,
101,
102,
100,
98,
98,
99,
99,
100,
101,
102,
106,
118,
123,
125,
128,
130,
130,
129,
130,
128,
126,
119,
116,
115,
118,
126,
133,
138,
141,
142,
143,
144,
146,
147,
146,
145,
149,
154,
158,
164,
171,
179,
185,
180,
173,
124,
122,
121,
121,
122,
122,
121,
119,
117,
118,
121,
122,
123,
122,
122,
122,
122,
123,
123,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
120,
118,
115,
111,
108,
105,
100,
97,
97,
97,
97,
96,
96,
96,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
91,
90,
90,
88,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
78,
78,
78,
78,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
80,
83,
90,
93,
95,
96,
97,
99,
102,
105,
109,
109,
108,
101,
92,
87,
87,
97,
104,
111,
117,
123,
127,
129,
131,
134,
137,
140,
142,
143,
143,
143,
142,
140,
137,
134,
131,
129,
127,
128,
129,
132,
135,
139,
140,
141,
142,
143,
142,
143,
143,
144,
146,
147,
149,
153,
152,
145,
142,
138,
134,
130,
127,
129,
131,
131,
128,
119,
109,
101,
102,
110,
114,
110,
106,
106,
104,
98,
98,
98,
99,
99,
100,
102,
105,
105,
114,
119,
123,
128,
129,
130,
130,
130,
129,
126,
116,
115,
116,
124,
135,
140,
141,
142,
142,
144,
145,
146,
147,
150,
152,
155,
160,
166,
173,
182,
189,
181,
174,
122,
121,
121,
121,
121,
121,
120,
118,
116,
117,
120,
122,
123,
123,
122,
122,
122,
123,
123,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
119,
116,
112,
108,
103,
98,
97,
97,
97,
96,
96,
96,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
92,
92,
91,
90,
89,
88,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
78,
78,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
89,
93,
96,
97,
98,
99,
101,
104,
107,
110,
111,
111,
104,
95,
87,
87,
95,
102,
108,
113,
119,
123,
126,
127,
131,
135,
138,
141,
142,
142,
142,
143,
142,
140,
137,
134,
131,
129,
128,
128,
130,
132,
135,
139,
140,
140,
141,
141,
144,
145,
146,
148,
150,
153,
152,
149,
144,
140,
136,
133,
132,
133,
135,
136,
136,
131,
120,
112,
104,
103,
110,
121,
119,
110,
108,
107,
101,
98,
98,
98,
99,
99,
102,
102,
104,
103,
108,
117,
125,
129,
130,
130,
130,
130,
129,
122,
118,
115,
119,
126,
135,
140,
140,
142,
142,
146,
146,
147,
152,
154,
156,
160,
167,
175,
185,
191,
181,
175,
121,
121,
121,
121,
121,
121,
121,
118,
116,
115,
118,
121,
122,
123,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
123,
122,
122,
122,
122,
121,
117,
111,
107,
103,
98,
97,
97,
97,
96,
96,
96,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
91,
88,
88,
88,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
81,
85,
93,
96,
98,
98,
99,
101,
102,
105,
108,
110,
112,
112,
108,
99,
87,
87,
92,
99,
105,
110,
115,
121,
125,
128,
130,
133,
137,
140,
141,
142,
142,
142,
142,
141,
139,
135,
133,
131,
130,
129,
129,
131,
133,
135,
137,
139,
140,
141,
144,
146,
148,
150,
152,
153,
152,
149,
144,
140,
136,
133,
135,
138,
140,
141,
139,
134,
124,
117,
110,
103,
108,
118,
127,
124,
116,
110,
106,
102,
99,
98,
98,
100,
101,
102,
103,
104,
105,
109,
119,
127,
130,
131,
131,
131,
130,
128,
120,
118,
115,
119,
127,
133,
137,
137,
139,
146,
147,
149,
153,
154,
156,
160,
168,
176,
187,
191,
181,
175,
121,
121,
122,
121,
121,
123,
123,
120,
116,
116,
119,
121,
123,
122,
123,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
121,
116,
110,
106,
102,
98,
97,
97,
97,
96,
96,
96,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
91,
88,
87,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
79,
79,
79,
79,
79,
80,
81,
92,
97,
98,
99,
99,
100,
102,
103,
106,
109,
111,
113,
115,
112,
106,
91,
87,
87,
96,
102,
109,
117,
124,
128,
131,
134,
135,
138,
139,
141,
142,
142,
142,
142,
142,
140,
137,
134,
133,
132,
131,
132,
132,
132,
134,
137,
139,
141,
142,
144,
146,
148,
151,
153,
155,
154,
150,
145,
141,
138,
138,
139,
143,
144,
143,
141,
136,
129,
122,
116,
109,
107,
114,
125,
135,
128,
119,
112,
108,
107,
106,
103,
102,
103,
103,
103,
103,
104,
107,
113,
121,
129,
131,
130,
130,
132,
130,
127,
121,
119,
116,
120,
125,
128,
131,
135,
141,
146,
149,
152,
154,
156,
160,
168,
177,
188,
185,
179,
174,
121,
121,
121,
121,
122,
126,
126,
121,
118,
118,
121,
122,
123,
123,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
120,
114,
110,
105,
101,
98,
97,
97,
96,
96,
96,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
92,
92,
91,
88,
87,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
80,
80,
79,
79,
79,
79,
79,
87,
97,
98,
99,
100,
101,
101,
102,
104,
106,
109,
111,
114,
117,
118,
112,
99,
87,
87,
94,
102,
110,
120,
125,
129,
132,
135,
136,
137,
139,
140,
140,
140,
140,
140,
142,
141,
139,
137,
135,
134,
133,
132,
132,
132,
134,
137,
140,
142,
143,
145,
146,
150,
153,
155,
155,
154,
151,
145,
141,
140,
140,
144,
148,
148,
146,
143,
139,
133,
127,
120,
115,
107,
110,
121,
135,
139,
131,
119,
117,
117,
116,
112,
110,
110,
104,
106,
103,
104,
104,
109,
118,
127,
128,
130,
132,
132,
134,
134,
130,
124,
120,
122,
122,
122,
125,
129,
135,
141,
147,
150,
153,
156,
159,
168,
178,
182,
178,
175,
171,
122,
121,
121,
122,
125,
129,
128,
122,
119,
121,
123,
123,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
121,
118,
113,
108,
105,
100,
98,
97,
97,
96,
96,
96,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
92,
91,
88,
87,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
80,
80,
81,
80,
80,
80,
79,
79,
81,
83,
96,
99,
100,
101,
101,
102,
102,
103,
105,
107,
109,
112,
114,
118,
122,
119,
106,
87,
87,
94,
102,
111,
118,
124,
128,
131,
134,
135,
136,
137,
137,
136,
136,
136,
139,
142,
143,
142,
141,
139,
137,
136,
134,
132,
132,
133,
136,
140,
142,
144,
147,
150,
153,
155,
157,
158,
155,
150,
145,
142,
140,
142,
147,
152,
153,
150,
145,
142,
138,
131,
124,
118,
111,
109,
118,
131,
141,
142,
132,
127,
131,
128,
124,
123,
121,
117,
115,
114,
103,
103,
105,
115,
121,
126,
128,
130,
132,
134,
135,
135,
132,
129,
124,
122,
122,
123,
123,
127,
132,
142,
149,
153,
156,
160,
169,
176,
175,
173,
170,
168,
123,
122,
122,
123,
127,
127,
125,
122,
122,
123,
127,
123,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
121,
118,
114,
108,
104,
99,
97,
97,
97,
96,
96,
96,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
93,
94,
94,
94,
94,
93,
91,
88,
87,
87,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
84,
83,
83,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
79,
80,
81,
81,
81,
81,
80,
80,
79,
81,
92,
99,
100,
102,
102,
103,
103,
103,
104,
105,
107,
110,
112,
115,
120,
121,
123,
113,
93,
87,
92,
99,
108,
114,
120,
124,
127,
131,
133,
134,
134,
134,
133,
133,
134,
140,
143,
144,
144,
143,
142,
141,
140,
136,
133,
131,
132,
135,
138,
140,
143,
147,
151,
154,
156,
159,
159,
157,
151,
145,
142,
143,
144,
151,
154,
155,
154,
148,
144,
141,
136,
129,
121,
115,
109,
115,
127,
135,
141,
143,
142,
142,
137,
132,
130,
128,
122,
115,
112,
105,
104,
105,
104,
114,
120,
123,
128,
131,
134,
136,
137,
138,
136,
134,
131,
129,
124,
123,
126,
126,
137,
148,
152,
156,
162,
171,
172,
171,
168,
166,
163,
127,
124,
122,
124,
127,
126,
122,
122,
122,
126,
129,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
121,
117,
112,
107,
103,
98,
97,
97,
97,
96,
96,
96,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
93,
92,
88,
88,
87,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
79,
79,
79,
80,
81,
81,
81,
82,
82,
82,
84,
86,
98,
100,
102,
103,
104,
104,
105,
105,
106,
106,
108,
110,
113,
116,
120,
122,
123,
120,
100,
87,
89,
95,
103,
109,
114,
118,
123,
127,
128,
128,
128,
127,
129,
133,
136,
141,
145,
147,
147,
146,
145,
144,
141,
138,
133,
130,
129,
134,
136,
138,
140,
147,
150,
153,
156,
159,
159,
159,
153,
147,
144,
144,
146,
153,
155,
157,
156,
153,
146,
144,
141,
135,
127,
118,
111,
112,
121,
129,
134,
139,
141,
143,
146,
141,
136,
129,
121,
109,
109,
107,
105,
104,
104,
115,
120,
122,
128,
130,
134,
136,
138,
141,
140,
142,
140,
137,
133,
130,
126,
129,
131,
145,
151,
155,
162,
171,
170,
167,
163,
159,
157,
128,
128,
124,
124,
127,
129,
122,
122,
123,
128,
131,
124,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
120,
116,
110,
105,
101,
98,
97,
97,
97,
96,
96,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
94,
92,
89,
88,
87,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
79,
79,
80,
80,
81,
82,
82,
83,
83,
84,
85,
95,
99,
102,
104,
105,
106,
106,
107,
107,
108,
108,
108,
110,
114,
117,
121,
124,
125,
124,
110,
94,
89,
91,
97,
104,
111,
114,
119,
122,
123,
122,
121,
125,
129,
133,
138,
142,
146,
147,
147,
146,
147,
143,
141,
136,
131,
129,
129,
130,
133,
137,
141,
146,
150,
153,
156,
159,
160,
160,
154,
149,
145,
145,
149,
154,
157,
159,
160,
156,
151,
146,
143,
139,
131,
121,
114,
116,
115,
122,
128,
134,
139,
143,
150,
148,
139,
128,
120,
110,
109,
107,
106,
105,
104,
110,
121,
127,
130,
129,
133,
135,
137,
139,
141,
141,
141,
140,
141,
138,
132,
128,
128,
137,
147,
154,
160,
168,
167,
164,
158,
156,
154,
127,
126,
124,
122,
125,
127,
126,
122,
125,
128,
127,
123,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
118,
113,
108,
103,
100,
98,
97,
97,
96,
96,
96,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
95,
94,
93,
89,
88,
87,
87,
86,
86,
85,
85,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
79,
79,
79,
80,
81,
82,
83,
83,
83,
84,
85,
91,
98,
101,
103,
105,
106,
107,
108,
109,
110,
110,
109,
109,
111,
115,
118,
121,
124,
127,
129,
119,
104,
95,
89,
92,
100,
109,
112,
114,
116,
118,
120,
122,
126,
130,
133,
138,
142,
146,
148,
145,
143,
143,
142,
138,
133,
128,
126,
126,
129,
133,
137,
142,
146,
151,
154,
157,
159,
161,
160,
156,
151,
147,
147,
150,
155,
159,
164,
164,
159,
155,
150,
146,
142,
134,
122,
117,
123,
122,
119,
128,
135,
140,
145,
153,
151,
140,
132,
126,
119,
111,
109,
108,
106,
104,
104,
115,
121,
124,
124,
126,
132,
135,
137,
139,
140,
141,
141,
142,
140,
140,
133,
130,
128,
140,
151,
159,
165,
164,
159,
153,
153,
153,
125,
123,
122,
122,
122,
124,
125,
125,
126,
126,
123,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
123,
121,
115,
110,
107,
104,
99,
97,
97,
97,
96,
96,
96,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
95,
95,
93,
90,
88,
87,
87,
86,
86,
85,
85,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
80,
80,
80,
81,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
80,
81,
82,
83,
83,
84,
84,
85,
89,
98,
100,
102,
105,
107,
108,
109,
110,
110,
111,
112,
112,
112,
113,
116,
118,
121,
125,
128,
128,
126,
113,
103,
95,
89,
96,
104,
110,
112,
114,
115,
120,
124,
127,
131,
133,
137,
140,
140,
138,
137,
136,
136,
138,
135,
129,
125,
123,
125,
128,
132,
136,
141,
146,
152,
156,
159,
163,
164,
162,
157,
154,
152,
151,
152,
156,
160,
164,
165,
161,
157,
153,
149,
143,
134,
121,
122,
129,
130,
122,
128,
138,
144,
150,
155,
153,
142,
135,
131,
125,
119,
115,
109,
107,
105,
104,
106,
110,
112,
114,
124,
129,
132,
135,
137,
139,
140,
141,
142,
142,
141,
140,
133,
128,
134,
147,
157,
161,
159,
155,
151,
153,
156,
123,
122,
121,
121,
122,
123,
123,
124,
125,
125,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
120,
115,
111,
107,
104,
98,
97,
97,
97,
96,
96,
96,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
95,
95,
95,
93,
91,
88,
88,
87,
86,
86,
85,
85,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
82,
83,
84,
84,
85,
85,
90,
97,
98,
100,
103,
106,
109,
110,
110,
111,
112,
113,
114,
114,
115,
115,
116,
119,
122,
125,
129,
133,
130,
121,
110,
102,
94,
90,
98,
104,
109,
112,
115,
120,
124,
127,
130,
132,
134,
134,
134,
133,
131,
129,
130,
132,
129,
125,
122,
122,
126,
128,
130,
134,
138,
146,
152,
156,
162,
165,
165,
165,
159,
158,
156,
155,
156,
157,
159,
162,
162,
160,
158,
154,
149,
142,
131,
125,
127,
132,
137,
133,
127,
139,
147,
155,
159,
155,
146,
139,
134,
128,
121,
117,
110,
108,
105,
104,
104,
105,
109,
109,
116,
126,
130,
132,
134,
136,
138,
140,
141,
143,
143,
141,
138,
132,
130,
140,
152,
156,
154,
151,
148,
151,
159,
122,
122,
121,
121,
121,
122,
122,
123,
124,
123,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
121,
120,
118,
116,
112,
106,
100,
97,
97,
97,
96,
96,
96,
96,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
95,
95,
95,
94,
92,
88,
88,
87,
86,
86,
85,
85,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
81,
82,
82,
82,
83,
84,
85,
85,
86,
91,
93,
96,
98,
100,
102,
107,
110,
111,
111,
112,
113,
114,
115,
117,
118,
118,
119,
120,
122,
125,
129,
134,
134,
128,
119,
109,
102,
93,
92,
97,
105,
111,
115,
119,
123,
126,
127,
127,
127,
128,
128,
127,
125,
124,
124,
123,
121,
120,
122,
124,
129,
130,
131,
133,
136,
143,
150,
154,
159,
165,
165,
165,
164,
163,
161,
159,
160,
161,
161,
162,
160,
157,
154,
149,
144,
137,
132,
135,
136,
138,
142,
140,
133,
139,
149,
158,
159,
157,
149,
142,
136,
129,
121,
117,
114,
109,
107,
106,
107,
109,
109,
109,
109,
121,
127,
130,
132,
134,
136,
138,
141,
142,
143,
143,
142,
137,
132,
134,
141,
144,
145,
145,
144,
152,
158,
122,
121,
121,
121,
121,
121,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
123,
123,
123,
122,
121,
118,
117,
118,
117,
116,
110,
104,
98,
97,
97,
97,
96,
96,
96,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
95,
95,
95,
95,
95,
94,
92,
89,
88,
87,
86,
86,
85,
85,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
81,
81,
81,
81,
81,
81,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
83,
84,
84,
85,
86,
87,
89,
90,
91,
95,
98,
101,
107,
109,
111,
112,
113,
114,
115,
117,
119,
120,
121,
121,
122,
123,
126,
128,
132,
135,
133,
126,
118,
110,
102,
92,
94,
102,
110,
114,
118,
122,
123,
122,
121,
121,
122,
122,
121,
119,
117,
118,
119,
119,
121,
122,
128,
134,
135,
132,
133,
133,
138,
147,
151,
155,
159,
164,
164,
166,
167,
166,
165,
165,
165,
164,
162,
157,
152,
146,
146,
140,
138,
139,
142,
142,
142,
146,
145,
139,
139,
149,
159,
164,
160,
153,
145,
138,
130,
122,
120,
117,
111,
109,
107,
108,
110,
110,
110,
111,
115,
123,
127,
128,
131,
135,
137,
139,
141,
142,
143,
142,
142,
138,
135,
134,
138,
140,
141,
142,
149,
155,
122,
122,
121,
121,
122,
121,
122,
122,
122,
121,
122,
122,
122,
122,
122,
122,
122,
122,
123,
124,
125,
125,
124,
123,
122,
120,
115,
115,
113,
115,
113,
108,
104,
98,
97,
97,
96,
96,
96,
95,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
95,
95,
95,
95,
95,
95,
95,
94,
93,
89,
88,
87,
86,
86,
85,
85,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
83,
83,
84,
84,
84,
85,
85,
86,
85,
86,
88,
90,
93,
96,
100,
104,
110,
111,
112,
114,
115,
117,
119,
121,
122,
123,
123,
123,
124,
126,
127,
131,
134,
133,
132,
125,
118,
111,
101,
92,
96,
107,
112,
115,
117,
117,
116,
115,
115,
115,
115,
115,
116,
116,
116,
117,
120,
121,
125,
131,
138,
141,
140,
134,
133,
134,
141,
148,
152,
155,
159,
163,
166,
167,
170,
169,
170,
169,
166,
162,
157,
153,
150,
147,
146,
144,
145,
146,
147,
147,
152,
150,
145,
140,
149,
160,
164,
166,
158,
149,
141,
136,
128,
124,
120,
118,
113,
109,
109,
110,
111,
110,
110,
110,
116,
121,
124,
130,
134,
136,
138,
140,
141,
141,
141,
142,
140,
139,
139,
139,
140,
140,
140,
141,
149,
122,
121,
122,
122,
122,
121,
122,
122,
122,
121,
121,
122,
122,
122,
122,
122,
122,
123,
125,
126,
127,
126,
125,
123,
122,
120,
117,
116,
109,
110,
110,
106,
102,
98,
97,
97,
97,
96,
96,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
94,
89,
88,
87,
86,
86,
85,
85,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
84,
84,
84,
84,
84,
85,
85,
89,
85,
85,
86,
85,
86,
89,
92,
95,
99,
102,
110,
111,
113,
114,
116,
119,
121,
122,
123,
124,
124,
125,
125,
126,
127,
129,
133,
134,
133,
132,
127,
119,
110,
98,
98,
98,
106,
110,
111,
111,
111,
110,
110,
110,
111,
114,
114,
116,
118,
120,
122,
125,
129,
134,
141,
146,
148,
145,
141,
138,
138,
143,
148,
152,
155,
159,
163,
167,
172,
174,
177,
175,
169,
165,
159,
156,
154,
151,
149,
148,
149,
151,
153,
154,
154,
154,
153,
146,
148,
159,
164,
165,
163,
157,
152,
145,
138,
132,
127,
124,
119,
115,
111,
109,
111,
111,
110,
111,
111,
115,
123,
129,
132,
134,
136,
137,
139,
139,
140,
140,
142,
141,
141,
146,
148,
149,
148,
146,
144,
121,
121,
122,
123,
122,
121,
122,
122,
122,
121,
121,
122,
122,
122,
122,
122,
123,
125,
126,
128,
128,
126,
124,
122,
121,
118,
116,
116,
111,
108,
106,
103,
100,
97,
97,
97,
97,
96,
95,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
95,
94,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
94,
92,
88,
88,
87,
86,
86,
85,
85,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
83,
83,
83,
83,
83,
84,
84,
84,
84,
84,
85,
85,
92,
97,
98,
98,
102,
90,
85,
87,
91,
94,
98,
102,
109,
111,
113,
115,
118,
121,
122,
123,
124,
125,
125,
126,
126,
127,
127,
128,
132,
134,
135,
135,
131,
125,
117,
108,
98,
99,
98,
101,
103,
104,
109,
110,
110,
110,
111,
113,
115,
118,
121,
124,
127,
129,
132,
138,
145,
150,
153,
153,
149,
145,
143,
142,
145,
148,
152,
155,
158,
162,
169,
174,
178,
176,
175,
169,
164,
158,
156,
153,
151,
151,
152,
154,
156,
158,
158,
158,
158,
154,
151,
158,
164,
167,
170,
167,
160,
152,
144,
138,
135,
132,
125,
123,
119,
114,
113,
112,
111,
110,
111,
111,
120,
124,
127,
132,
134,
135,
134,
136,
139,
140,
142,
145,
147,
150,
153,
154,
154,
153,
150,
120,
121,
124,
124,
122,
121,
122,
122,
122,
121,
121,
122,
122,
122,
122,
122,
123,
125,
126,
126,
126,
125,
123,
121,
120,
119,
116,
114,
110,
108,
106,
104,
101,
99,
98,
98,
97,
96,
95,
95,
95,
95,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
92,
89,
88,
87,
87,
86,
86,
85,
85,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
83,
84,
84,
84,
84,
84,
84,
84,
85,
85,
85,
91,
98,
104,
107,
112,
112,
99,
85,
85,
88,
93,
98,
103,
110,
112,
113,
115,
119,
122,
123,
124,
125,
125,
126,
127,
128,
128,
128,
129,
132,
134,
135,
135,
134,
129,
123,
114,
106,
98,
98,
98,
104,
107,
110,
110,
111,
112,
112,
114,
116,
120,
123,
128,
130,
132,
135,
140,
146,
152,
156,
158,
156,
152,
148,
148,
149,
149,
151,
152,
157,
160,
167,
173,
177,
177,
179,
174,
168,
161,
157,
155,
153,
154,
155,
157,
160,
164,
164,
165,
165,
161,
157,
157,
163,
167,
171,
172,
167,
158,
151,
147,
143,
136,
132,
131,
126,
118,
115,
114,
112,
111,
110,
110,
112,
119,
124,
128,
129,
130,
133,
137,
140,
141,
143,
145,
148,
152,
154,
156,
157,
158,
156,
116,
121,
123,
124,
122,
121,
122,
122,
122,
121,
121,
122,
122,
122,
122,
122,
123,
124,
125,
124,
123,
123,
122,
121,
121,
122,
121,
118,
115,
109,
108,
105,
101,
98,
97,
97,
96,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
93,
89,
88,
87,
87,
86,
86,
85,
85,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
84,
84,
84,
84,
84,
84,
84,
84,
85,
85,
85,
91,
97,
103,
107,
113,
117,
119,
108,
95,
86,
85,
92,
98,
105,
111,
112,
114,
116,
121,
123,
123,
124,
125,
126,
127,
129,
129,
130,
131,
131,
132,
134,
134,
135,
135,
134,
129,
121,
112,
104,
98,
98,
104,
110,
109,
109,
111,
111,
110,
115,
118,
121,
125,
129,
131,
134,
137,
141,
147,
153,
158,
161,
161,
158,
155,
155,
155,
156,
156,
156,
158,
158,
164,
170,
172,
174,
176,
173,
170,
164,
159,
157,
156,
157,
158,
161,
164,
167,
170,
172,
171,
166,
163,
160,
162,
166,
170,
176,
172,
164,
160,
156,
150,
144,
141,
139,
132,
126,
118,
115,
114,
113,
112,
112,
113,
115,
121,
122,
125,
129,
135,
139,
141,
142,
144,
146,
148,
152,
154,
157,
159,
160,
160,
120,
123,
124,
123,
122,
121,
121,
122,
122,
122,
121,
121,
122,
122,
122,
122,
123,
124,
123,
122,
122,
122,
122,
122,
122,
122,
122,
119,
116,
111,
108,
104,
99,
97,
97,
96,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
90,
88,
88,
87,
87,
86,
86,
85,
85,
86,
86,
86,
86,
87,
87,
87,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
84,
84,
84,
84,
84,
84,
84,
84,
85,
85,
85,
90,
95,
101,
105,
110,
117,
121,
125,
120,
106,
95,
85,
85,
97,
104,
110,
112,
114,
117,
122,
123,
124,
125,
126,
127,
128,
130,
131,
132,
133,
133,
132,
133,
134,
135,
136,
135,
135,
126,
119,
112,
103,
98,
99,
103,
104,
106,
108,
108,
109,
115,
119,
123,
127,
130,
133,
136,
139,
143,
151,
156,
160,
164,
165,
163,
161,
162,
163,
164,
163,
162,
162,
161,
163,
164,
165,
167,
168,
168,
166,
164,
161,
158,
158,
160,
162,
164,
167,
170,
175,
177,
175,
170,
168,
165,
165,
166,
170,
175,
177,
169,
167,
165,
160,
153,
149,
145,
141,
133,
121,
116,
115,
115,
115,
114,
114,
116,
116,
120,
123,
129,
133,
136,
139,
141,
142,
145,
148,
151,
153,
156,
159,
161,
165,
122,
123,
123,
122,
121,
121,
122,
122,
122,
120,
120,
121,
121,
122,
122,
123,
123,
123,
122,
122,
122,
122,
122,
122,
122,
122,
121,
118,
114,
110,
107,
103,
98,
97,
96,
96,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
93,
88,
88,
87,
87,
86,
86,
86,
85,
86,
86,
86,
86,
87,
87,
87,
87,
87,
87,
87,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
83,
84,
84,
84,
84,
84,
84,
85,
85,
85,
86,
89,
94,
99,
103,
108,
114,
119,
124,
130,
126,
115,
101,
90,
85,
93,
100,
106,
111,
113,
117,
122,
123,
124,
125,
126,
127,
129,
131,
132,
134,
135,
134,
134,
133,
134,
134,
135,
135,
135,
135,
127,
120,
113,
104,
98,
98,
102,
103,
103,
105,
108,
114,
119,
124,
128,
131,
134,
138,
142,
146,
153,
157,
161,
164,
164,
164,
166,
168,
171,
171,
166,
165,
165,
164,
164,
160,
158,
159,
161,
162,
162,
160,
159,
158,
161,
165,
167,
168,
170,
174,
180,
183,
181,
176,
171,
169,
170,
171,
170,
173,
175,
176,
172,
171,
167,
165,
159,
152,
147,
137,
126,
120,
116,
117,
117,
115,
115,
116,
117,
119,
120,
121,
123,
128,
134,
136,
140,
143,
147,
150,
153,
155,
158,
161,
164,
123,
124,
124,
123,
122,
123,
125,
126,
122,
121,
122,
122,
122,
122,
122,
123,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
119,
117,
114,
110,
106,
102,
98,
97,
96,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
93,
90,
88,
88,
88,
87,
86,
86,
86,
86,
86,
86,
86,
86,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
84,
84,
84,
84,
84,
84,
85,
85,
85,
85,
86,
92,
97,
100,
105,
109,
115,
120,
125,
129,
129,
119,
103,
93,
85,
86,
95,
99,
104,
111,
115,
121,
122,
124,
125,
126,
128,
129,
131,
133,
135,
136,
136,
136,
135,
134,
134,
134,
135,
136,
134,
133,
128,
121,
112,
105,
102,
104,
103,
103,
103,
106,
111,
117,
122,
127,
131,
135,
139,
142,
147,
153,
156,
158,
160,
163,
165,
164,
166,
168,
168,
167,
165,
165,
166,
162,
159,
154,
155,
156,
156,
157,
156,
157,
160,
166,
171,
173,
174,
176,
180,
185,
188,
187,
182,
178,
175,
176,
175,
174,
175,
175,
173,
169,
166,
164,
163,
160,
159,
153,
144,
134,
128,
123,
120,
119,
117,
116,
115,
116,
118,
119,
120,
122,
123,
122,
126,
134,
141,
145,
149,
152,
154,
157,
160,
163,
124,
124,
125,
125,
125,
126,
128,
128,
127,
125,
125,
123,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
121,
118,
114,
112,
109,
105,
102,
98,
97,
96,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
95,
95,
95,
96,
95,
95,
95,
95,
95,
95,
95,
95,
94,
88,
88,
88,
87,
87,
87,
87,
86,
86,
86,
86,
86,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
84,
84,
84,
84,
84,
85,
85,
85,
85,
85,
92,
96,
99,
102,
106,
110,
115,
120,
125,
128,
126,
121,
105,
96,
89,
85,
85,
93,
97,
102,
109,
116,
122,
122,
124,
126,
128,
130,
132,
134,
135,
136,
136,
138,
137,
135,
134,
134,
134,
135,
135,
134,
131,
125,
115,
108,
108,
110,
110,
108,
103,
104,
104,
114,
120,
125,
129,
133,
137,
141,
145,
148,
151,
153,
156,
158,
159,
158,
159,
161,
162,
162,
164,
164,
159,
157,
151,
149,
150,
151,
151,
151,
152,
156,
162,
169,
175,
180,
183,
184,
187,
189,
190,
190,
186,
183,
182,
181,
180,
180,
178,
176,
170,
163,
158,
153,
152,
152,
151,
150,
148,
143,
136,
132,
124,
120,
118,
117,
115,
116,
117,
119,
121,
122,
122,
122,
122,
125,
137,
140,
148,
151,
153,
156,
158,
161,
124,
125,
127,
127,
127,
128,
129,
130,
129,
129,
127,
124,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
122,
121,
121,
118,
116,
112,
109,
107,
104,
102,
98,
97,
96,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
95,
95,
95,
96,
95,
95,
95,
95,
95,
95,
95,
95,
92,
88,
88,
88,
88,
87,
88,
86,
87,
86,
86,
86,
86,
86,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
84,
84,
84,
84,
85,
85,
85,
85,
86,
89,
96,
99,
102,
104,
107,
110,
115,
120,
124,
127,
127,
125,
110,
100,
95,
92,
85,
85,
86,
95,
102,
110,
115,
121,
123,
124,
127,
129,
131,
133,
134,
136,
138,
138,
139,
137,
134,
133,
134,
135,
134,
132,
129,
124,
116,
109,
111,
112,
112,
110,
106,
105,
104,
111,
118,
123,
126,
130,
135,
137,
139,
142,
145,
149,
151,
152,
151,
153,
154,
155,
155,
156,
157,
156,
151,
148,
145,
144,
144,
144,
145,
146,
152,
156,
164,
173,
182,
186,
189,
190,
190,
190,
190,
190,
185,
185,
186,
186,
185,
185,
183,
179,
171,
165,
157,
152,
145,
140,
140,
140,
141,
140,
138,
134,
126,
121,
121,
118,
116,
116,
117,
119,
122,
123,
128,
128,
125,
123,
133,
138,
147,
150,
153,
155,
157,
160,
125,
126,
128,
128,
129,
129,
130,
129,
130,
128,
128,
126,
124,
122,
122,
122,
122,
122,
122,
122,
122,
122,
121,
120,
118,
116,
111,
109,
108,
104,
102,
101,
99,
97,
96,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
93,
89,
88,
88,
88,
88,
88,
87,
87,
88,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
83,
84,
84,
84,
84,
85,
85,
85,
85,
85,
94,
99,
103,
106,
108,
109,
110,
115,
120,
125,
128,
129,
129,
121,
111,
101,
95,
93,
92,
88,
92,
98,
104,
109,
112,
115,
118,
122,
127,
129,
130,
132,
135,
137,
139,
141,
139,
136,
134,
132,
131,
130,
129,
125,
120,
115,
114,
115,
115,
113,
110,
108,
106,
104,
107,
116,
120,
124,
127,
131,
134,
135,
136,
139,
144,
145,
143,
143,
148,
148,
149,
150,
151,
152,
151,
148,
144,
141,
140,
140,
140,
141,
146,
151,
158,
167,
175,
182,
187,
191,
189,
189,
192,
188,
183,
181,
184,
189,
190,
190,
190,
187,
183,
176,
169,
160,
154,
146,
144,
140,
134,
132,
132,
129,
127,
128,
126,
126,
121,
118,
116,
116,
118,
120,
123,
127,
129,
130,
126,
128,
136,
144,
148,
152,
154,
156,
158,
124,
126,
128,
129,
130,
130,
130,
129,
128,
127,
126,
125,
124,
123,
122,
122,
122,
122,
122,
122,
122,
122,
120,
119,
117,
115,
112,
110,
108,
105,
102,
99,
99,
97,
96,
95,
95,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
94,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
95,
96,
94,
91,
88,
88,
88,
88,
88,
88,
87,
88,
88,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
87,
86,
87,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
86,
85,
85,
85,
85,
85,
85,
85,
85,
85,
84,
84,
84,
84,
84,
84,
83,
83,
83,
83,
83,
83,
83,
83,
83,
83,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
82,
83,
83,
83,
83,
84,
84,
84,
84,
85,
85,
85,
86,
90,
97,
102,
107,
111,
112,
112,
113,
115,
120,
125,
128,
130,
130,
127,
122,
111,
99,
96,
95,
93,
90,
93,
98,
102,
104,
106,
110,
114,
119,
122,
124,
127,
131,
135,
136,
138,
137,
136,
135,
132,
128,
127,
123,
120,
118,
115,
116,
117,
116,
116,
114,
110,
108,
104,
104,
111,
116,
119,
123,
125,
127,
129,
130,
132,
135,
134,
132,
133,
137,
139,
141,
146,
146,
145,
144,
141,
139,
136,
135,
136,
138,
141,
146,
151,
159,
166,
173,
177,
181,
184,
186,
184,
182,
179,
176,
175,
180,
187,
191,
194,
194,
192,
187,
180,
173,
166,
159,
156,
152,
146,
139,
138,
140,
137,
134,
133,
138,
134,
125,
122,
118,
116,
118,
120,
120,
123,
129,
134,
131,
126,
134,
138,
145,
149,
152,
154,
155
}; // Santa maria ends here































// ATASCADERO starts here
/*static int zValues[]={
285,
287,
289,
291,
294,
297,
299,
302,
306,
314,
320,
326,
333,
341,
347,
354,
363,
383,
395,
394,
391,
391,
391,
402,
408,
407,
395,
374,
376,
392,
389,
383,
394,
403,
407,
412,
427,
438,
444,
445,
449,
456,
463,
469,
475,
484,
495,
506,
518,
527,
525,
524,
517,
512,
510,
514,
523,
531,
540,
542,
533,
537,
544,
550,
543,
536,
534,
524,
519,
515,
504,
490,
478,
464,
441,
429,
430,
419,
400,
379,
360,
353,
355,
350,
332,
320,
329,
324,
315,
304,
298,
292,
288,
287,
289,
292,
294,
296,
300,
303,
305,
316,
316,
299,
299,
302,
307,
318,
329,
339,
360,
381,
395,
390,
374,
358,
340,
330,
316,
331,
348,
360,
372,
386,
383,
370,
350,
335,
314,
318,
324,
329,
337,
346,
360,
369,
363,
365,
369,
366,
356,
363,
357,
342,
329,
323,
328,
334,
335,
334,
328,
338,
344,
341,
338,
344,
353,
366,
369,
361,
356,
367,
367,
375,
389,
405,
416,
418,
426,
430,
414,
396,
379,
359,
332,
315,
314,
312,
316,
320,
324,
323,
324,
324,
329,
338,
345,
356,
368,
379,
390,
396,
406,
416,
421,
416,
425,
431,
430,
419,
403,
396,
391,
383,
374,
366,
357,
349,
343,
338,
334,
329,
324,
320,
321,
321,
322,
324,
325,
327,
328,
330,
333,
336,
338,
346,
357,
367,
379,
388,
394,
404,
406,
413,
423,
435,
446,
460,
478,
496,
518,
536,
545,
544,
537,
533,
530,
537,
541,
531,
537,
552,
564,
570,
570,
571,
582,
592,
599,
598,
590,
581,
578,
576,
569,
559,
558,
554,
548,
545,
548,
551,
558,
562,
565,
568,
570,
573,
579,
586,
594,
600,
605,
612,
604,
619,
612,
601,
588,
581,
576,
572,
565,
556,
546,
540,
534,
524,
520,
512,
509,
507,
507,
507,
508,
506,
505,
502,
494,
482,
473,
464,
460,
464,
471,
480,
489,
499,
511,
519,
519,
521,
520,
517,
505,
498,
491,
476,
460,
464,
457,
454,
451,
446,
444,
434,
434,
450,
461,
468,
474,
481,
487,
487,
487,
489,
496,
498,
497,
497,
499,
501,
504,
504,
506,
504,
500,
497,
491,
484,
477,
467,
457,
446,
435,
288,
289,
291,
294,
297,
301,
304,
304,
305,
313,
320,
328,
334,
339,
346,
354,
362,
378,
402,
406,
407,
409,
410,
416,
419,
414,
399,
384,
379,
398,
403,
395,
401,
415,
420,
420,
422,
428,
434,
438,
446,
454,
464,
471,
478,
489,
500,
512,
524,
524,
524,
521,
514,
515,
520,
527,
534,
544,
551,
553,
546,
545,
551,
561,
551,
549,
540,
530,
520,
508,
501,
488,
473,
455,
437,
418,
411,
414,
401,
382,
362,
340,
339,
340,
330,
314,
313,
307,
300,
295,
291,
290,
293,
295,
296,
296,
299,
299,
308,
310,
311,
317,
326,
309,
301,
307,
313,
322,
338,
354,
368,
385,
396,
386,
372,
360,
351,
340,
328,
342,
352,
362,
376,
386,
392,
385,
368,
348,
325,
316,
322,
328,
340,
351,
361,
366,
354,
356,
358,
357,
348,
347,
342,
329,
331,
340,
345,
350,
349,
348,
342,
343,
354,
351,
343,
346,
358,
371,
379,
376,
366,
379,
383,
382,
398,
413,
421,
431,
440,
431,
412,
389,
368,
346,
321,
315,
312,
313,
316,
325,
333,
336,
334,
329,
330,
339,
346,
352,
363,
377,
386,
396,
407,
421,
430,
429,
436,
445,
442,
428,
416,
409,
402,
393,
382,
374,
368,
359,
351,
347,
343,
338,
333,
327,
321,
323,
322,
323,
325,
327,
330,
334,
337,
343,
351,
354,
352,
362,
371,
378,
388,
398,
403,
413,
424,
436,
450,
466,
483,
501,
521,
530,
529,
525,
518,
513,
514,
530,
535,
523,
531,
548,
559,
560,
559,
568,
577,
585,
588,
585,
576,
569,
569,
565,
556,
548,
546,
544,
543,
544,
547,
548,
554,
560,
560,
564,
568,
571,
576,
582,
587,
593,
597,
601,
611,
622,
620,
603,
589,
580,
571,
564,
559,
548,
547,
540,
538,
531,
526,
524,
519,
518,
517,
518,
520,
517,
515,
508,
496,
484,
474,
465,
460,
464,
473,
482,
492,
504,
516,
525,
528,
530,
536,
528,
518,
508,
494,
478,
470,
476,
481,
480,
472,
461,
442,
432,
431,
444,
458,
466,
478,
489,
494,
496,
495,
497,
497,
497,
496,
497,
499,
501,
503,
501,
500,
498,
494,
488,
484,
476,
470,
464,
456,
448,
437,
290,
292,
295,
299,
302,
306,
308,
306,
305,
314,
321,
329,
335,
340,
346,
352,
358,
368,
399,
411,
416,
419,
424,
427,
428,
419,
408,
398,
379,
394,
406,
413,
411,
417,
428,
429,
432,
432,
435,
437,
445,
456,
465,
473,
484,
496,
508,
515,
524,
524,
522,
521,
521,
522,
529,
539,
548,
558,
566,
566,
558,
557,
563,
564,
562,
554,
547,
534,
520,
503,
492,
484,
467,
451,
437,
418,
396,
393,
393,
383,
364,
344,
326,
322,
324,
310,
300,
294,
291,
291,
293,
297,
307,
305,
305,
303,
304,
314,
316,
321,
321,
326,
335,
322,
303,
315,
321,
326,
335,
352,
364,
387,
394,
387,
375,
364,
357,
341,
335,
341,
354,
363,
376,
381,
387,
386,
373,
356,
330,
318,
324,
332,
343,
352,
359,
362,
354,
346,
345,
344,
340,
336,
337,
336,
338,
347,
356,
364,
363,
361,
352,
351,
359,
360,
356,
352,
356,
372,
387,
391,
386,
392,
397,
389,
409,
425,
433,
436,
438,
430,
409,
387,
362,
337,
316,
313,
312,
315,
319,
328,
337,
347,
346,
340,
343,
346,
354,
363,
371,
376,
387,
400,
412,
424,
437,
442,
449,
460,
455,
442,
434,
424,
413,
403,
394,
389,
381,
370,
361,
358,
353,
350,
346,
340,
331,
324,
324,
323,
325,
328,
333,
339,
342,
349,
358,
367,
367,
369,
379,
387,
394,
401,
407,
415,
424,
434,
447,
462,
483,
500,
511,
513,
509,
504,
500,
498,
511,
526,
527,
520,
525,
540,
549,
547,
553,
563,
572,
576,
575,
571,
564,
557,
558,
552,
545,
540,
537,
536,
538,
546,
552,
556,
560,
566,
572,
575,
577,
580,
584,
585,
585,
590,
594,
599,
610,
620,
618,
603,
590,
581,
574,
568,
562,
558,
554,
551,
546,
542,
540,
536,
534,
531,
527,
530,
534,
529,
524,
509,
496,
484,
475,
464,
463,
470,
476,
484,
492,
502,
513,
520,
524,
531,
538,
538,
525,
511,
500,
487,
478,
486,
490,
480,
464,
446,
431,
429,
430,
440,
449,
462,
478,
494,
502,
504,
502,
502,
499,
496,
497,
497,
498,
500,
500,
498,
493,
491,
488,
481,
473,
467,
461,
453,
448,
442,
435,
293,
300,
301,
306,
310,
313,
313,
312,
312,
316,
321,
330,
335,
343,
348,
353,
359,
365,
394,
413,
420,
425,
429,
431,
431,
426,
419,
409,
393,
392,
409,
423,
427,
425,
427,
438,
441,
443,
442,
443,
450,
460,
469,
476,
489,
501,
512,
516,
518,
522,
523,
522,
523,
530,
542,
552,
561,
571,
580,
578,
575,
573,
570,
569,
563,
556,
548,
536,
521,
502,
484,
468,
454,
440,
425,
412,
395,
376,
379,
379,
368,
349,
331,
313,
312,
302,
295,
292,
294,
296,
297,
306,
317,
318,
314,
310,
310,
312,
321,
329,
338,
343,
348,
331,
310,
329,
334,
342,
341,
340,
354,
373,
383,
384,
381,
373,
363,
354,
347,
338,
349,
359,
363,
365,
379,
380,
370,
352,
328,
329,
326,
331,
341,
351,
361,
368,
358,
345,
340,
340,
340,
339,
341,
343,
340,
349,
360,
370,
377,
370,
359,
359,
367,
371,
368,
361,
365,
369,
386,
399,
402,
407,
408,
400,
415,
432,
444,
452,
439,
424,
402,
381,
357,
334,
313,
312,
313,
318,
324,
332,
342,
353,
353,
349,
354,
356,
365,
375,
387,
392,
397,
410,
426,
435,
442,
453,
460,
470,
470,
459,
452,
438,
425,
413,
411,
406,
395,
384,
374,
371,
365,
364,
360,
353,
345,
333,
323,
324,
325,
328,
333,
341,
353,
355,
361,
372,
379,
376,
381,
392,
396,
403,
406,
412,
421,
430,
440,
453,
468,
482,
492,
500,
487,
484,
482,
491,
506,
516,
517,
510,
517,
530,
536,
535,
544,
553,
560,
563,
560,
560,
552,
546,
546,
539,
534,
532,
529,
532,
541,
550,
557,
563,
568,
573,
580,
585,
588,
591,
596,
599,
598,
594,
594,
597,
606,
610,
610,
598,
585,
577,
573,
566,
563,
562,
558,
556,
552,
551,
549,
545,
540,
537,
530,
537,
536,
529,
521,
508,
494,
482,
472,
464,
466,
473,
484,
492,
499,
499,
504,
512,
518,
523,
534,
538,
528,
516,
504,
491,
486,
491,
490,
476,
456,
433,
427,
426,
432,
441,
452,
466,
482,
499,
512,
514,
510,
510,
503,
499,
497,
497,
497,
497,
496,
492,
487,
482,
479,
474,
464,
455,
451,
446,
439,
434,
428,
302,
306,
311,
314,
315,
318,
318,
317,
319,
316,
318,
328,
334,
340,
346,
356,
361,
371,
395,
412,
420,
427,
431,
434,
437,
435,
429,
418,
405,
397,
407,
424,
433,
437,
435,
436,
443,
451,
454,
455,
460,
466,
475,
483,
494,
505,
513,
516,
519,
521,
524,
531,
538,
542,
553,
564,
575,
586,
592,
594,
590,
586,
580,
573,
565,
555,
545,
537,
526,
509,
490,
469,
451,
433,
418,
401,
389,
375,
364,
362,
361,
353,
336,
319,
306,
296,
292,
294,
300,
304,
304,
300,
308,
312,
314,
317,
321,
322,
323,
328,
342,
357,
359,
340,
318,
335,
349,
356,
357,
359,
361,
364,
366,
380,
388,
381,
374,
370,
359,
343,
342,
350,
356,
367,
383,
379,
366,
346,
328,
339,
334,
333,
342,
353,
365,
372,
360,
349,
352,
352,
350,
343,
348,
352,
349,
348,
356,
369,
381,
379,
370,
365,
379,
384,
382,
373,
377,
382,
386,
401,
416,
424,
426,
414,
422,
437,
451,
451,
433,
413,
393,
371,
348,
326,
316,
313,
315,
316,
326,
333,
338,
342,
349,
353,
361,
368,
376,
388,
397,
406,
415,
420,
433,
444,
450,
460,
471,
482,
485,
474,
463,
449,
434,
423,
421,
417,
407,
394,
387,
385,
380,
381,
376,
368,
358,
348,
332,
322,
326,
326,
332,
341,
353,
366,
367,
373,
387,
389,
385,
388,
390,
396,
401,
412,
419,
426,
432,
440,
450,
462,
469,
471,
469,
466,
472,
486,
494,
500,
505,
501,
507,
516,
522,
526,
536,
542,
547,
550,
547,
547,
541,
534,
532,
525,
523,
524,
523,
536,
545,
553,
561,
569,
574,
583,
586,
594,
598,
600,
605,
610,
610,
601,
595,
596,
604,
611,
605,
589,
578,
571,
566,
561,
556,
555,
549,
546,
549,
548,
546,
536,
528,
526,
526,
528,
523,
516,
512,
504,
492,
482,
474,
468,
471,
478,
488,
500,
513,
514,
510,
510,
514,
520,
526,
531,
524,
513,
500,
493,
493,
493,
487,
468,
446,
432,
421,
421,
429,
440,
451,
466,
484,
501,
519,
523,
519,
515,
509,
502,
499,
497,
496,
494,
492,
488,
482,
473,
467,
463,
457,
449,
440,
438,
431,
424,
420,
306,
310,
316,
320,
322,
324,
324,
324,
324,
323,
323,
330,
336,
339,
342,
352,
361,
374,
391,
408,
419,
427,
432,
436,
440,
442,
439,
428,
415,
400,
412,
427,
437,
444,
445,
441,
447,
455,
463,
465,
468,
473,
480,
489,
502,
510,
514,
518,
521,
523,
526,
534,
547,
553,
562,
574,
587,
598,
602,
605,
603,
595,
587,
578,
567,
555,
543,
530,
519,
505,
490,
476,
456,
438,
420,
403,
392,
380,
369,
354,
340,
340,
335,
320,
304,
294,
292,
296,
303,
314,
316,
310,
302,
307,
312,
316,
330,
335,
336,
333,
343,
362,
364,
350,
329,
336,
354,
363,
371,
378,
379,
382,
377,
375,
389,
391,
388,
382,
368,
346,
342,
353,
366,
380,
382,
377,
364,
342,
336,
350,
346,
339,
347,
358,
376,
381,
368,
357,
360,
366,
362,
352,
358,
364,
358,
351,
355,
370,
384,
391,
385,
380,
383,
398,
395,
385,
389,
399,
397,
401,
418,
432,
436,
429,
432,
445,
454,
438,
424,
405,
387,
364,
340,
324,
315,
314,
317,
318,
323,
328,
328,
338,
345,
353,
364,
369,
380,
393,
402,
412,
424,
434,
443,
453,
460,
467,
477,
488,
488,
475,
464,
453,
437,
433,
429,
422,
415,
403,
399,
397,
394,
391,
386,
380,
371,
361,
347,
334,
328,
326,
331,
339,
351,
366,
379,
384,
391,
392,
383,
378,
387,
395,
403,
408,
414,
419,
424,
431,
436,
440,
445,
450,
451,
451,
460,
466,
475,
486,
493,
494,
499,
503,
511,
522,
529,
535,
543,
541,
538,
536,
531,
522,
519,
514,
513,
514,
526,
537,
546,
553,
562,
570,
577,
585,
590,
596,
600,
606,
612,
620,
616,
608,
601,
598,
600,
601,
599,
587,
574,
564,
558,
552,
546,
544,
541,
532,
535,
534,
539,
533,
522,
516,
516,
518,
514,
507,
502,
498,
490,
480,
474,
470,
474,
485,
496,
506,
518,
525,
521,
515,
516,
520,
521,
524,
514,
508,
499,
498,
500,
496,
483,
463,
444,
429,
417,
417,
425,
433,
448,
462,
483,
502,
520,
528,
525,
518,
511,
504,
500,
498,
495,
492,
491,
489,
482,
471,
459,
451,
447,
440,
432,
426,
423,
414,
412,
307,
313,
318,
326,
329,
329,
329,
329,
330,
334,
334,
337,
343,
344,
349,
354,
365,
379,
391,
405,
416,
425,
432,
437,
442,
448,
450,
435,
421,
403,
417,
432,
444,
454,
457,
453,
450,
460,
468,
472,
476,
480,
486,
498,
509,
514,
517,
520,
523,
526,
530,
535,
545,
557,
571,
582,
598,
606,
610,
612,
610,
604,
595,
586,
576,
560,
547,
533,
519,
504,
491,
475,
462,
443,
437,
426,
406,
393,
375,
364,
349,
329,
325,
318,
302,
294,
295,
300,
302,
308,
322,
323,
314,
309,
319,
328,
337,
345,
349,
353,
352,
364,
369,
359,
341,
338,
356,
370,
384,
391,
395,
395,
383,
375,
383,
393,
399,
390,
371,
357,
350,
353,
366,
379,
379,
373,
358,
339,
344,
358,
356,
346,
352,
366,
382,
392,
382,
368,
365,
378,
374,
364,
368,
375,
369,
357,
364,
368,
384,
395,
401,
397,
392,
408,
408,
398,
398,
411,
412,
407,
426,
439,
444,
444,
445,
453,
442,
428,
414,
398,
380,
358,
335,
321,
313,
314,
317,
319,
323,
330,
332,
342,
345,
354,
362,
366,
376,
387,
396,
405,
414,
424,
436,
447,
454,
460,
470,
482,
484,
477,
468,
455,
448,
442,
434,
423,
417,
411,
406,
405,
402,
396,
390,
382,
375,
366,
355,
342,
331,
327,
331,
339,
351,
364,
380,
393,
390,
384,
374,
376,
383,
390,
397,
403,
407,
410,
414,
420,
423,
424,
428,
433,
437,
441,
447,
455,
463,
476,
483,
487,
495,
505,
514,
524,
532,
538,
539,
534,
526,
524,
520,
512,
509,
507,
506,
512,
523,
533,
540,
549,
555,
563,
571,
577,
582,
586,
592,
601,
614,
624,
621,
612,
609,
607,
604,
602,
598,
586,
573,
563,
555,
547,
540,
534,
528,
522,
518,
523,
527,
528,
519,
510,
508,
510,
506,
500,
492,
487,
480,
474,
470,
473,
482,
493,
503,
514,
522,
530,
525,
522,
521,
522,
525,
523,
516,
508,
501,
501,
502,
494,
481,
462,
443,
426,
415,
414,
424,
436,
452,
467,
490,
506,
518,
526,
525,
517,
512,
505,
500,
497,
494,
492,
491,
489,
486,
476,
463,
447,
436,
428,
423,
417,
414,
408,
409,
307,
313,
319,
326,
330,
331,
332,
334,
338,
343,
344,
343,
342,
348,
351,
355,
369,
381,
388,
399,
415,
427,
433,
439,
444,
450,
453,
452,
432,
418,
414,
431,
446,
460,
469,
468,
460,
465,
472,
478,
484,
489,
493,
504,
514,
517,
520,
522,
524,
528,
532,
538,
551,
564,
576,
588,
602,
611,
613,
616,
610,
608,
600,
591,
582,
570,
556,
543,
530,
515,
503,
487,
472,
460,
451,
436,
418,
401,
385,
369,
352,
333,
316,
306,
299,
295,
300,
312,
308,
314,
326,
330,
323,
318,
317,
330,
344,
355,
362,
369,
370,
377,
377,
364,
346,
340,
358,
375,
391,
403,
406,
405,
385,
380,
387,
399,
405,
393,
376,
369,
356,
366,
369,
374,
372,
367,
350,
337,
355,
367,
367,
357,
354,
371,
388,
402,
396,
385,
375,
387,
385,
375,
376,
384,
380,
370,
374,
376,
379,
392,
407,
414,
413,
420,
418,
414,
413,
420,
426,
419,
433,
447,
457,
460,
456,
452,
436,
418,
405,
390,
371,
349,
331,
318,
313,
315,
318,
321,
324,
328,
333,
340,
344,
352,
355,
366,
373,
381,
390,
398,
405,
413,
421,
430,
437,
446,
461,
473,
475,
478,
476,
466,
457,
449,
437,
424,
414,
408,
405,
406,
403,
395,
387,
379,
370,
361,
351,
340,
332,
326,
332,
340,
353,
369,
381,
377,
374,
371,
365,
368,
377,
385,
390,
395,
398,
400,
403,
407,
409,
410,
414,
422,
434,
446,
458,
470,
475,
475,
480,
489,
501,
512,
522,
532,
539,
538,
533,
525,
518,
514,
510,
504,
502,
499,
507,
514,
522,
530,
536,
547,
550,
558,
561,
568,
573,
578,
586,
598,
610,
619,
622,
614,
619,
614,
609,
607,
599,
589,
578,
568,
560,
550,
548,
541,
536,
524,
511,
511,
515,
521,
517,
510,
500,
501,
499,
494,
487,
476,
472,
472,
476,
482,
488,
501,
512,
520,
530,
536,
528,
525,
527,
526,
528,
526,
519,
510,
506,
502,
495,
486,
468,
453,
435,
421,
412,
414,
427,
443,
459,
472,
492,
506,
515,
523,
521,
515,
510,
503,
498,
496,
493,
490,
490,
488,
483,
476,
464,
450,
437,
427,
419,
412,
407,
406,
408,
306,
313,
316,
320,
324,
326,
328,
329,
336,
342,
346,
347,
348,
352,
353,
354,
366,
374,
386,
400,
416,
428,
434,
440,
445,
450,
455,
458,
450,
425,
419,
427,
439,
456,
471,
478,
473,
472,
478,
483,
489,
494,
498,
511,
520,
522,
523,
524,
528,
532,
534,
537,
549,
561,
576,
591,
603,
611,
615,
614,
612,
608,
601,
594,
584,
572,
559,
546,
535,
524,
510,
497,
489,
480,
466,
450,
428,
410,
393,
374,
356,
337,
320,
302,
297,
297,
307,
323,
324,
322,
334,
341,
336,
332,
324,
326,
342,
359,
373,
384,
390,
389,
378,
366,
353,
338,
346,
363,
383,
402,
406,
408,
389,
383,
394,
404,
405,
395,
389,
381,
364,
370,
382,
381,
365,
352,
343,
338,
358,
376,
381,
370,
362,
371,
389,
406,
411,
400,
391,
394,
396,
386,
380,
393,
390,
379,
382,
390,
388,
392,
407,
428,
435,
435,
434,
431,
431,
432,
439,
433,
443,
456,
466,
465,
456,
445,
430,
410,
395,
379,
359,
340,
324,
316,
315,
318,
322,
325,
327,
328,
331,
334,
339,
345,
350,
358,
366,
374,
384,
392,
398,
402,
408,
413,
425,
441,
450,
459,
476,
470,
470,
463,
455,
445,
436,
425,
414,
404,
400,
398,
396,
390,
382,
372,
363,
354,
341,
335,
328,
328,
331,
339,
351,
367,
366,
361,
361,
360,
358,
364,
374,
379,
381,
386,
389,
391,
394,
398,
403,
412,
422,
434,
448,
460,
476,
488,
491,
490,
490,
500,
509,
519,
530,
538,
537,
530,
523,
515,
508,
505,
500,
493,
491,
498,
510,
517,
527,
537,
544,
551,
558,
559,
561,
567,
572,
575,
583,
589,
599,
605,
610,
619,
623,
618,
614,
610,
601,
590,
580,
569,
560,
552,
548,
538,
534,
526,
514,
506,
505,
510,
510,
506,
500,
493,
491,
488,
479,
472,
470,
475,
483,
492,
501,
508,
520,
530,
538,
534,
527,
524,
528,
531,
532,
529,
522,
515,
502,
492,
479,
468,
455,
440,
426,
415,
411,
419,
435,
452,
467,
484,
499,
509,
514,
520,
518,
513,
508,
502,
498,
497,
490,
488,
488,
485,
477,
472,
464,
453,
443,
434,
426,
417,
406,
405,
408,
316,
317,
316,
317,
320,
323,
326,
328,
334,
342,
346,
351,
354,
358,
364,
366,
363,
370,
389,
402,
415,
428,
436,
442,
446,
450,
458,
464,
456,
441,
434,
437,
436,
449,
467,
477,
478,
481,
485,
489,
494,
499,
503,
517,
526,
524,
526,
528,
533,
539,
543,
543,
546,
557,
573,
590,
603,
611,
616,
615,
614,
611,
608,
600,
589,
573,
556,
543,
532,
521,
510,
502,
492,
488,
477,
455,
434,
416,
395,
375,
356,
336,
320,
300,
297,
300,
312,
331,
342,
339,
340,
350,
348,
348,
344,
335,
333,
346,
363,
378,
396,
389,
376,
364,
352,
350,
360,
361,
369,
379,
399,
410,
396,
388,
391,
397,
404,
406,
404,
394,
376,
375,
391,
387,
368,
353,
364,
355,
366,
389,
395,
385,
372,
370,
388,
407,
420,
416,
406,
400,
408,
402,
391,
400,
403,
391,
390,
404,
401,
400,
410,
428,
445,
453,
448,
446,
443,
445,
450,
449,
456,
459,
457,
451,
444,
435,
421,
399,
381,
363,
345,
330,
321,
318,
320,
324,
332,
336,
332,
333,
328,
335,
337,
340,
342,
352,
362,
371,
380,
386,
391,
394,
397,
406,
420,
430,
438,
456,
468,
464,
456,
450,
442,
434,
427,
419,
414,
407,
398,
388,
387,
382,
378,
366,
356,
350,
343,
335,
329,
328,
331,
336,
345,
351,
351,
353,
354,
356,
358,
363,
364,
370,
374,
379,
385,
389,
396,
403,
412,
419,
436,
452,
466,
478,
495,
504,
506,
505,
503,
509,
516,
525,
535,
532,
526,
519,
513,
505,
496,
493,
486,
474,
490,
503,
512,
520,
526,
538,
548,
553,
561,
564,
566,
573,
578,
585,
589,
590,
595,
598,
609,
622,
619,
609,
607,
601,
593,
584,
567,
558,
550,
544,
541,
534,
526,
523,
516,
507,
499,
500,
500,
498,
495,
490,
484,
480,
475,
476,
473,
480,
490,
499,
510,
518,
525,
535,
532,
525,
521,
521,
528,
535,
536,
532,
525,
513,
502,
481,
460,
449,
438,
427,
417,
410,
413,
427,
445,
463,
479,
494,
507,
515,
520,
517,
515,
512,
509,
508,
503,
499,
490,
483,
479,
478,
471,
466,
459,
452,
442,
433,
424,
412,
406,
404,
410,
318,
319,
317,
316,
318,
323,
326,
329,
335,
341,
346,
353,
356,
363,
368,
375,
372,
372,
385,
399,
413,
427,
436,
442,
447,
452,
460,
468,
467,
457,
446,
454,
455,
451,
460,
476,
485,
488,
490,
494,
499,
505,
511,
524,
527,
528,
530,
533,
536,
549,
555,
557,
555,
559,
574,
592,
604,
611,
617,
617,
617,
613,
610,
600,
590,
573,
554,
540,
528,
515,
502,
495,
486,
475,
462,
444,
429,
413,
394,
378,
359,
334,
313,
303,
297,
301,
317,
325,
333,
339,
349,
354,
349,
357,
359,
356,
350,
352,
372,
392,
404,
397,
387,
372,
353,
360,
375,
374,
382,
389,
394,
412,
397,
396,
390,
403,
401,
414,
417,
406,
390,
377,
397,
400,
379,
366,
381,
370,
378,
394,
411,
399,
385,
377,
386,
405,
426,
430,
424,
414,
420,
424,
412,
403,
417,
409,
400,
415,
418,
414,
412,
432,
451,
463,
462,
460,
457,
461,
465,
466,
463,
449,
438,
433,
430,
426,
406,
392,
369,
348,
334,
323,
321,
325,
329,
330,
341,
348,
345,
347,
346,
343,
339,
338,
340,
351,
361,
369,
374,
379,
383,
388,
389,
399,
413,
421,
439,
455,
462,
457,
446,
435,
427,
419,
412,
404,
404,
405,
399,
390,
379,
370,
365,
358,
350,
344,
339,
333,
329,
328,
331,
335,
340,
345,
348,
350,
354,
357,
360,
364,
369,
377,
383,
386,
392,
400,
406,
414,
422,
433,
448,
468,
486,
501,
510,
516,
517,
515,
511,
513,
518,
523,
523,
518,
514,
508,
501,
493,
484,
475,
471,
475,
491,
505,
516,
526,
532,
538,
546,
552,
559,
562,
566,
572,
580,
589,
598,
601,
603,
605,
609,
621,
611,
600,
593,
585,
575,
564,
554,
545,
532,
523,
530,
531,
523,
514,
511,
505,
499,
494,
492,
490,
488,
486,
484,
481,
482,
482,
477,
483,
492,
504,
515,
526,
527,
525,
521,
517,
514,
522,
529,
537,
537,
524,
511,
500,
490,
476,
455,
437,
424,
415,
409,
408,
418,
436,
458,
476,
491,
504,
516,
522,
518,
517,
519,
518,
516,
512,
504,
498,
488,
480,
473,
473,
465,
458,
454,
444,
435,
428,
418,
409,
403,
405,
412,
319,
320,
319,
319,
321,
325,
328,
329,
336,
345,
346,
354,
360,
366,
375,
379,
379,
379,
380,
389,
406,
422,
433,
441,
447,
453,
461,
472,
478,
470,
460,
467,
475,
474,
473,
472,
481,
490,
493,
498,
504,
512,
521,
527,
531,
531,
533,
535,
539,
551,
563,
573,
572,
565,
572,
584,
604,
615,
620,
619,
616,
611,
605,
596,
586,
564,
547,
538,
527,
514,
502,
490,
478,
464,
452,
438,
422,
407,
392,
378,
361,
342,
317,
304,
297,
301,
307,
315,
327,
340,
347,
345,
339,
353,
366,
370,
369,
369,
368,
386,
406,
404,
394,
376,
354,
367,
380,
388,
391,
403,
409,
416,
402,
404,
400,
412,
413,
411,
428,
413,
402,
388,
402,
410,
392,
381,
401,
389,
383,
403,
420,
418,
401,
389,
388,
405,
427,
443,
442,
432,
428,
436,
431,
421,
429,
425,
413,
423,
435,
431,
418,
430,
450,
464,
476,
474,
474,
476,
476,
467,
454,
437,
420,
410,
410,
404,
393,
377,
357,
340,
326,
321,
324,
334,
342,
347,
352,
361,
358,
359,
361,
360,
356,
348,
346,
348,
356,
362,
365,
376,
384,
391,
399,
408,
414,
422,
437,
452,
455,
450,
440,
430,
415,
405,
400,
395,
392,
395,
392,
384,
373,
357,
351,
348,
343,
339,
333,
330,
328,
329,
332,
337,
342,
347,
351,
355,
359,
364,
366,
375,
381,
387,
394,
394,
398,
406,
415,
425,
436,
448,
466,
482,
503,
514,
525,
526,
519,
512,
505,
505,
510,
512,
509,
504,
499,
493,
486,
478,
470,
465,
477,
489,
498,
507,
516,
525,
536,
546,
552,
557,
558,
559,
563,
570,
576,
585,
596,
602,
611,
612,
620,
620,
607,
596,
586,
578,
568,
557,
544,
530,
517,
509,
512,
525,
524,
514,
507,
500,
498,
490,
487,
486,
486,
487,
486,
488,
489,
489,
487,
486,
487,
502,
512,
518,
519,
517,
513,
510,
513,
520,
526,
533,
530,
515,
491,
477,
471,
464,
448,
431,
416,
407,
404,
409,
424,
444,
466,
489,
501,
510,
524,
520,
514,
514,
516,
524,
523,
513,
504,
495,
485,
476,
467,
466,
464,
450,
445,
439,
426,
419,
411,
404,
402,
406,
419,
320,
320,
321,
322,
325,
326,
329,
334,
336,
348,
352,
357,
363,
369,
375,
378,
382,
383,
389,
394,
401,
414,
427,
439,
446,
453,
463,
474,
481,
480,
471,
476,
481,
485,
485,
482,
483,
489,
497,
502,
509,
518,
528,
535,
537,
534,
536,
541,
545,
548,
559,
571,
579,
586,
583,
596,
611,
624,
623,
619,
613,
608,
599,
588,
573,
559,
541,
527,
515,
509,
501,
489,
478,
463,
450,
434,
416,
400,
386,
374,
361,
345,
324,
307,
298,
301,
308,
317,
324,
332,
334,
329,
334,
345,
353,
362,
374,
387,
391,
394,
409,
411,
396,
376,
361,
363,
373,
388,
396,
408,
422,
422,
405,
411,
414,
415,
420,
413,
423,
423,
415,
403,
419,
422,
409,
389,
412,
404,
390,
406,
427,
430,
418,
406,
398,
411,
431,
450,
455,
447,
438,
446,
447,
433,
440,
440,
429,
433,
450,
447,
432,
432,
451,
463,
481,
489,
480,
476,
466,
458,
446,
428,
409,
393,
389,
390,
379,
358,
345,
332,
324,
323,
332,
335,
345,
353,
362,
370,
372,
370,
375,
375,
370,
362,
361,
356,
354,
363,
374,
384,
394,
402,
410,
418,
425,
433,
437,
447,
449,
442,
434,
424,
414,
400,
389,
385,
382,
381,
378,
371,
361,
351,
341,
339,
335,
332,
330,
328,
328,
330,
335,
342,
349,
356,
360,
363,
372,
375,
375,
384,
394,
402,
406,
405,
406,
412,
421,
435,
451,
465,
480,
496,
512,
510,
523,
515,
506,
499,
493,
494,
499,
498,
494,
489,
484,
478,
472,
464,
460,
472,
485,
500,
510,
517,
525,
531,
538,
550,
560,
564,
565,
566,
565,
570,
574,
583,
591,
597,
603,
609,
616,
622,
610,
597,
587,
576,
564,
552,
540,
527,
510,
499,
499,
514,
515,
514,
508,
500,
497,
495,
491,
490,
490,
492,
493,
496,
499,
498,
496,
490,
487,
496,
503,
508,
509,
508,
507,
510,
514,
521,
527,
535,
524,
511,
486,
466,
451,
448,
440,
426,
412,
403,
402,
411,
428,
451,
467,
491,
503,
514,
515,
512,
510,
512,
514,
519,
513,
503,
499,
491,
484,
475,
465,
457,
453,
443,
433,
430,
423,
412,
402,
399,
402,
413,
429,
319,
319,
320,
324,
327,
328,
334,
341,
344,
348,
355,
360,
366,
371,
376,
380,
381,
387,
389,
402,
406,
412,
425,
437,
445,
452,
461,
473,
484,
487,
481,
476,
487,
490,
490,
489,
486,
488,
497,
504,
512,
522,
534,
540,
541,
538,
541,
548,
553,
555,
558,
564,
577,
591,
597,
605,
618,
622,
622,
616,
611,
603,
593,
578,
563,
550,
534,
521,
507,
495,
490,
482,
473,
458,
444,
430,
412,
397,
382,
370,
355,
339,
322,
308,
300,
301,
305,
306,
314,
317,
326,
332,
337,
341,
348,
358,
374,
394,
409,
416,
418,
412,
396,
376,
365,
373,
377,
378,
391,
403,
424,
426,
414,
420,
423,
416,
430,
427,
427,
430,
425,
418,
431,
436,
425,
406,
419,
423,
414,
409,
425,
441,
437,
425,
419,
414,
435,
451,
465,
459,
450,
450,
455,
444,
444,
451,
438,
441,
459,
459,
446,
443,
460,
473,
480,
472,
464,
459,
451,
439,
434,
423,
407,
389,
376,
376,
367,
345,
336,
329,
328,
328,
330,
341,
342,
351,
353,
365,
380,
386,
389,
389,
381,
374,
374,
367,
364,
370,
378,
390,
398,
406,
413,
424,
435,
443,
442,
441,
442,
437,
426,
419,
410,
399,
387,
377,
377,
373,
369,
365,
355,
342,
341,
335,
332,
330,
328,
328,
330,
332,
338,
348,
358,
369,
373,
374,
379,
390,
388,
391,
406,
416,
419,
418,
414,
419,
428,
441,
461,
480,
492,
504,
507,
503,
499,
497,
491,
484,
478,
481,
484,
481,
479,
475,
468,
460,
457,
459,
465,
473,
486,
500,
510,
525,
535,
542,
549,
557,
567,
575,
578,
580,
577,
572,
571,
576,
583,
586,
592,
600,
609,
613,
603,
591,
582,
573,
557,
542,
527,
516,
506,
493,
489,
500,
500,
509,
507,
502,
497,
493,
493,
496,
498,
499,
499,
503,
506,
506,
503,
495,
490,
491,
498,
500,
500,
500,
506,
511,
519,
525,
533,
532,
517,
497,
477,
459,
439,
433,
431,
419,
409,
401,
401,
412,
430,
448,
466,
489,
502,
512,
512,
508,
503,
504,
506,
502,
493,
487,
485,
483,
479,
476,
466,
451,
442,
433,
424,
419,
412,
404,
396,
396,
403,
422,
440,
317,
317,
319,
325,
328,
332,
338,
344,
349,
353,
358,
364,
368,
373,
378,
382,
386,
390,
398,
400,
409,
418,
427,
437,
444,
449,
459,
470,
483,
490,
487,
486,
489,
492,
498,
496,
488,
492,
497,
504,
515,
527,
539,
548,
547,
545,
547,
553,
562,
572,
573,
572,
581,
595,
598,
606,
612,
613,
614,
612,
605,
597,
586,
570,
555,
540,
525,
511,
498,
484,
477,
472,
461,
450,
436,
422,
406,
393,
379,
364,
350,
334,
317,
306,
302,
305,
302,
308,
318,
320,
327,
342,
352,
356,
362,
364,
376,
396,
413,
429,
427,
415,
398,
380,
379,
384,
386,
391,
395,
403,
424,
433,
428,
425,
432,
428,
438,
440,
437,
430,
441,
437,
443,
448,
439,
422,
426,
439,
435,
425,
423,
443,
451,
440,
434,
428,
436,
450,
466,
469,
462,
458,
466,
456,
451,
456,
446,
448,
466,
468,
454,
458,
470,
479,
478,
462,
445,
440,
436,
422,
416,
413,
404,
391,
368,
359,
352,
339,
331,
332,
342,
342,
331,
339,
354,
361,
367,
370,
381,
397,
402,
402,
393,
386,
386,
378,
375,
383,
381,
390,
402,
411,
420,
430,
441,
437,
432,
433,
434,
429,
417,
410,
405,
395,
384,
378,
372,
374,
371,
367,
360,
352,
344,
337,
333,
330,
329,
329,
333,
337,
340,
355,
368,
381,
387,
385,
393,
402,
403,
400,
415,
425,
434,
435,
432,
434,
442,
454,
473,
486,
488,
488,
489,
486,
482,
477,
472,
467,
463,
465,
466,
465,
462,
460,
453,
449,
456,
468,
480,
487,
492,
498,
507,
518,
536,
550,
558,
564,
573,
584,
589,
594,
589,
578,
571,
573,
581,
585,
593,
602,
610,
609,
596,
582,
574,
563,
551,
534,
512,
500,
496,
486,
479,
485,
496,
508,
510,
503,
497,
490,
493,
499,
502,
503,
505,
509,
513,
513,
504,
497,
494,
495,
496,
495,
498,
502,
508,
514,
522,
528,
528,
522,
512,
493,
471,
449,
434,
422,
417,
411,
404,
398,
401,
414,
428,
444,
462,
483,
499,
505,
506,
504,
499,
496,
499,
492,
482,
475,
472,
472,
472,
470,
466,
456,
444,
433,
420,
410,
406,
397,
392,
396,
408,
429,
450,
316,
318,
321,
327,
330,
337,
342,
348,
353,
358,
363,
367,
371,
375,
379,
384,
389,
394,
397,
406,
411,
423,
432,
439,
444,
448,
458,
470,
481,
488,
487,
487,
489,
496,
501,
498,
494,
499,
501,
509,
521,
534,
548,
553,
552,
552,
548,
554,
564,
574,
582,
590,
594,
592,
597,
602,
606,
602,
610,
605,
597,
588,
576,
563,
550,
536,
524,
503,
486,
474,
464,
459,
450,
438,
427,
414,
398,
384,
370,
358,
341,
326,
312,
302,
304,
315,
319,
309,
317,
320,
326,
337,
350,
354,
369,
377,
380,
393,
412,
428,
424,
414,
403,
391,
393,
389,
400,
400,
405,
408,
414,
435,
441,
439,
440,
440,
442,
444,
440,
438,
443,
450,
456,
463,
452,
438,
436,
452,
452,
446,
434,
444,
463,
458,
450,
439,
439,
448,
463,
477,
476,
472,
474,
464,
461,
465,
454,
455,
473,
476,
467,
473,
478,
475,
466,
454,
435,
422,
419,
406,
395,
393,
390,
380,
368,
343,
335,
333,
331,
338,
355,
355,
344,
339,
348,
361,
374,
382,
391,
403,
411,
415,
409,
403,
399,
389,
378,
392,
396,
395,
401,
413,
424,
436,
436,
426,
422,
423,
426,
422,
414,
401,
396,
387,
381,
378,
380,
382,
382,
379,
368,
357,
350,
343,
336,
332,
328,
329,
333,
340,
347,
362,
377,
393,
402,
402,
405,
415,
418,
414,
420,
436,
448,
450,
446,
446,
456,
466,
472,
473,
469,
471,
471,
468,
464,
460,
455,
451,
447,
449,
449,
449,
448,
446,
448,
451,
462,
478,
492,
502,
512,
513,
516,
522,
530,
547,
558,
564,
574,
585,
596,
599,
590,
582,
581,
581,
587,
591,
597,
601,
606,
598,
586,
575,
565,
550,
539,
524,
511,
498,
484,
478,
472,
477,
489,
502,
499,
498,
492,
487,
489,
493,
499,
502,
502,
501,
504,
507,
506,
503,
500,
497,
496,
496,
499,
503,
511,
519,
525,
524,
511,
504,
498,
489,
472,
451,
434,
418,
412,
404,
398,
397,
404,
418,
429,
441,
462,
479,
490,
499,
501,
501,
495,
487,
488,
485,
475,
467,
463,
462,
462,
464,
463,
456,
449,
440,
429,
416,
404,
395,
390,
395,
412,
433,
454,
316,
318,
323,
329,
334,
340,
347,
352,
358,
363,
368,
371,
374,
378,
381,
385,
391,
395,
399,
409,
414,
426,
434,
440,
445,
446,
453,
468,
477,
481,
487,
493,
494,
497,
502,
500,
497,
501,
508,
516,
530,
542,
552,
559,
559,
560,
557,
557,
560,
570,
582,
594,
598,
594,
598,
599,
596,
588,
598,
595,
588,
576,
562,
550,
542,
529,
516,
501,
487,
474,
460,
449,
438,
425,
416,
404,
390,
375,
360,
347,
332,
319,
307,
302,
305,
314,
332,
323,
316,
315,
319,
334,
342,
346,
366,
385,
400,
405,
417,
418,
413,
408,
404,
403,
403,
400,
404,
413,
413,
425,
428,
440,
439,
439,
440,
437,
434,
434,
432,
426,
428,
444,
463,
465,
462,
460,
458,
467,
471,
465,
450,
448,
473,
472,
462,
449,
453,
450,
459,
476,
488,
485,
482,
474,
469,
474,
462,
466,
481,
485,
480,
479,
472,
460,
448,
438,
426,
408,
400,
392,
384,
378,
375,
370,
360,
338,
332,
331,
335,
346,
363,
370,
360,
350,
344,
353,
365,
378,
391,
400,
404,
413,
417,
417,
409,
396,
386,
394,
404,
407,
410,
413,
419,
428,
427,
421,
414,
413,
414,
410,
405,
395,
389,
384,
381,
382,
389,
394,
393,
385,
374,
362,
364,
346,
341,
335,
331,
328,
331,
341,
351,
368,
384,
401,
413,
417,
418,
427,
434,
432,
434,
440,
454,
455,
454,
453,
455,
456,
457,
456,
450,
452,
453,
451,
448,
444,
440,
437,
434,
436,
436,
438,
436,
446,
454,
460,
465,
482,
497,
513,
525,
532,
532,
535,
540,
544,
552,
559,
571,
581,
591,
594,
587,
583,
582,
586,
588,
592,
594,
596,
598,
595,
587,
576,
560,
547,
530,
514,
497,
490,
478,
465,
464,
472,
483,
490,
485,
488,
489,
480,
478,
485,
491,
495,
496,
494,
495,
499,
507,
512,
501,
495,
498,
498,
500,
504,
512,
521,
527,
516,
498,
484,
479,
473,
461,
451,
442,
426,
410,
400,
394,
397,
410,
430,
442,
452,
462,
476,
487,
492,
492,
492,
490,
483,
476,
475,
467,
460,
455,
450,
452,
453,
454,
449,
443,
437,
429,
418,
404,
394,
390,
396,
414,
435,
455,
316,
317,
325,
331,
338,
343,
350,
355,
364,
370,
374,
376,
378,
381,
382,
388,
391,
396,
401,
405,
412,
425,
437,
444,
450,
445,
450,
462,
474,
480,
488,
496,
498,
498,
501,
504,
501,
507,
514,
525,
539,
550,
559,
565,
569,
570,
566,
563,
566,
572,
585,
597,
595,
590,
590,
589,
582,
576,
582,
581,
575,
561,
545,
534,
532,
523,
507,
492,
480,
469,
458,
448,
437,
419,
406,
392,
377,
364,
351,
335,
322,
309,
304,
301,
305,
314,
321,
324,
320,
331,
328,
327,
327,
346,
368,
386,
395,
403,
406,
403,
400,
394,
398,
401,
408,
408,
414,
427,
430,
430,
434,
431,
427,
426,
427,
424,
420,
417,
415,
408,
416,
433,
441,
448,
465,
476,
480,
485,
486,
479,
459,
458,
478,
481,
468,
461,
468,
466,
460,
471,
487,
498,
497,
487,
483,
480,
475,
481,
492,
493,
489,
480,
462,
444,
432,
421,
407,
393,
383,
378,
372,
366,
360,
356,
343,
336,
334,
337,
337,
349,
367,
381,
376,
364,
353,
350,
359,
371,
382,
388,
399,
408,
417,
418,
417,
404,
393,
397,
404,
414,
423,
426,
426,
430,
431,
424,
414,
409,
402,
400,
397,
391,
380,
387,
385,
383,
397,
404,
402,
393,
381,
370,
376,
366,
344,
340,
335,
331,
328,
342,
352,
368,
385,
402,
417,
428,
434,
441,
451,
453,
453,
454,
452,
443,
440,
441,
441,
441,
442,
441,
437,
438,
439,
436,
433,
429,
427,
426,
425,
426,
431,
434,
437,
447,
460,
471,
480,
485,
497,
512,
528,
543,
552,
549,
556,
556,
554,
559,
565,
573,
581,
586,
584,
582,
582,
585,
586,
588,
586,
588,
595,
598,
592,
582,
566,
546,
530,
512,
498,
486,
472,
461,
456,
462,
464,
471,
469,
474,
481,
475,
469,
476,
483,
488,
488,
484,
485,
493,
503,
510,
500,
499,
500,
502,
503,
507,
512,
517,
520,
509,
489,
471,
461,
453,
448,
442,
434,
424,
406,
396,
392,
399,
415,
438,
456,
466,
475,
481,
484,
484,
481,
484,
485,
478,
469,
465,
460,
451,
448,
445,
442,
444,
444,
442,
437,
429,
423,
415,
401,
391,
388,
397,
414,
434,
452,
319,
320,
328,
336,
343,
349,
356,
363,
372,
378,
382,
383,
384,
384,
388,
388,
390,
397,
404,
408,
412,
424,
437,
449,
456,
452,
451,
464,
473,
481,
490,
500,
502,
502,
502,
507,
510,
513,
523,
535,
548,
560,
567,
573,
572,
573,
577,
575,
575,
585,
590,
588,
589,
590,
582,
573,
568,
559,
565,
566,
563,
551,
532,
516,
516,
513,
502,
484,
467,
458,
449,
436,
421,
408,
396,
382,
367,
352,
337,
323,
313,
310,
304,
302,
304,
311,
318,
322,
331,
342,
346,
338,
327,
341,
356,
364,
379,
386,
387,
385,
383,
390,
400,
410,
416,
421,
424,
438,
439,
431,
426,
419,
413,
406,
415,
414,
405,
403,
394,
396,
408,
417,
425,
441,
463,
480,
494,
502,
504,
488,
467,
470,
488,
488,
471,
467,
481,
478,
470,
471,
486,
504,
510,
503,
501,
492,
492,
496,
502,
496,
490,
484,
471,
452,
434,
412,
400,
388,
378,
374,
364,
353,
346,
343,
337,
336,
341,
348,
350,
350,
367,
386,
392,
380,
364,
354,
353,
363,
374,
384,
393,
400,
411,
419,
423,
415,
400,
405,
409,
414,
426,
436,
438,
444,
439,
431,
422,
413,
408,
402,
393,
389,
393,
400,
398,
389,
402,
411,
410,
403,
389,
384,
382,
370,
353,
347,
342,
336,
334,
339,
352,
364,
376,
396,
411,
423,
433,
440,
445,
443,
441,
438,
433,
428,
427,
428,
425,
427,
428,
426,
423,
424,
423,
420,
416,
414,
419,
423,
424,
429,
439,
442,
450,
455,
462,
474,
488,
497,
505,
509,
524,
540,
555,
565,
570,
570,
561,
560,
565,
571,
573,
578,
583,
583,
580,
579,
580,
581,
582,
585,
591,
588,
585,
576,
562,
546,
531,
518,
506,
495,
482,
465,
453,
447,
449,
454,
457,
466,
472,
467,
461,
466,
469,
475,
476,
473,
479,
488,
496,
504,
499,
500,
502,
507,
506,
508,
510,
512,
513,
498,
481,
464,
447,
438,
431,
430,
421,
408,
397,
391,
392,
403,
418,
439,
460,
473,
477,
478,
475,
472,
471,
474,
478,
473,
465,
455,
451,
444,
438,
438,
435,
435,
435,
435,
432,
427,
419,
409,
394,
387,
386,
398,
420,
433,
435,
323,
325,
328,
341,
349,
354,
359,
369,
380,
392,
390,
393,
390,
387,
389,
386,
389,
396,
404,
410,
414,
425,
438,
452,
465,
461,
460,
464,
474,
485,
490,
499,
504,
507,
510,
511,
517,
524,
534,
547,
558,
566,
573,
576,
577,
575,
577,
586,
587,
594,
594,
585,
581,
586,
583,
571,
556,
546,
546,
552,
550,
542,
524,
501,
500,
500,
494,
480,
464,
446,
436,
425,
412,
396,
382,
367,
354,
337,
326,
318,
316,
313,
308,
305,
304,
308,
316,
324,
337,
344,
351,
354,
346,
339,
342,
352,
361,
365,
364,
366,
375,
386,
397,
405,
422,
433,
440,
438,
428,
419,
413,
404,
403,
396,
404,
401,
396,
387,
400,
411,
417,
423,
429,
439,
457,
475,
492,
510,
513,
501,
480,
478,
492,
491,
477,
473,
488,
492,
483,
475,
488,
506,
522,
521,
518,
514,
512,
504,
494,
481,
473,
476,
477,
463,
445,
430,
418,
404,
389,
377,
368,
358,
350,
342,
337,
340,
351,
367,
370,
366,
371,
384,
400,
395,
381,
370,
361,
364,
371,
378,
389,
401,
412,
422,
430,
422,
411,
412,
419,
420,
425,
437,
446,
452,
442,
432,
423,
417,
415,
405,
396,
399,
404,
411,
410,
401,
404,
417,
419,
413,
399,
400,
392,
375,
368,
361,
350,
347,
341,
341,
346,
352,
357,
373,
390,
412,
423,
428,
425,
424,
423,
420,
416,
413,
412,
412,
410,
409,
410,
410,
406,
407,
411,
412,
413,
417,
422,
427,
431,
435,
439,
450,
456,
464,
472,
476,
485,
500,
513,
522,
524,
533,
546,
563,
574,
578,
568,
561,
570,
576,
580,
584,
586,
586,
578,
573,
575,
574,
576,
584,
587,
581,
572,
565,
555,
541,
523,
510,
501,
493,
486,
477,
464,
449,
441,
442,
448,
456,
459,
457,
450,
454,
455,
462,
463,
467,
474,
480,
488,
498,
498,
498,
506,
512,
506,
506,
506,
512,
515,
505,
490,
472,
447,
426,
415,
414,
407,
395,
389,
388,
396,
411,
429,
446,
462,
468,
471,
471,
465,
462,
461,
467,
471,
469,
463,
453,
441,
437,
430,
429,
429,
426,
427,
426,
427,
425,
418,
409,
394,
386,
384,
395,
416,
418,
415,
325,
326,
329,
342,
354,
360,
367,
376,
388,
397,
400,
397,
396,
396,
391,
388,
387,
393,
405,
415,
425,
435,
446,
461,
470,
474,
473,
469,
474,
482,
492,
499,
510,
513,
516,
520,
527,
535,
547,
558,
566,
572,
580,
584,
579,
574,
573,
582,
588,
596,
597,
590,
578,
568,
568,
564,
554,
539,
526,
535,
538,
532,
516,
496,
480,
477,
477,
470,
456,
439,
426,
415,
401,
386,
370,
353,
338,
338,
341,
339,
332,
319,
312,
308,
305,
306,
312,
320,
328,
336,
338,
345,
354,
364,
363,
368,
366,
372,
378,
377,
381,
396,
404,
414,
429,
430,
427,
423,
415,
406,
403,
396,
391,
385,
384,
391,
385,
401,
417,
429,
430,
438,
448,
458,
465,
471,
483,
503,
516,
510,
494,
483,
494,
500,
486,
479,
490,
502,
498,
488,
487,
509,
526,
537,
534,
525,
512,
499,
488,
477,
461,
458,
466,
470,
460,
447,
433,
417,
400,
384,
373,
367,
353,
343,
338,
342,
359,
371,
380,
382,
388,
393,
402,
406,
397,
384,
371,
368,
378,
381,
388,
400,
410,
419,
423,
426,
421,
418,
428,
432,
435,
437,
438,
446,
442,
431,
423,
414,
412,
405,
403,
403,
412,
419,
422,
412,
410,
418,
426,
425,
415,
417,
406,
392,
383,
378,
371,
356,
351,
345,
346,
347,
350,
363,
380,
400,
412,
415,
412,
410,
407,
402,
403,
402,
400,
399,
397,
398,
399,
400,
402,
406,
410,
414,
416,
421,
426,
432,
439,
444,
448,
449,
460,
470,
479,
489,
496,
498,
514,
528,
538,
536,
546,
560,
574,
586,
574,
568,
574,
580,
585,
590,
590,
587,
578,
570,
568,
569,
576,
580,
579,
570,
561,
553,
545,
532,
518,
508,
496,
485,
479,
477,
465,
452,
438,
435,
440,
445,
447,
446,
441,
445,
447,
451,
454,
462,
466,
475,
484,
489,
491,
496,
502,
506,
505,
500,
503,
508,
512,
507,
494,
473,
447,
430,
412,
400,
391,
387,
384,
388,
401,
419,
436,
451,
456,
458,
463,
464,
456,
450,
454,
461,
465,
464,
458,
450,
440,
428,
425,
424,
424,
421,
419,
419,
420,
418,
415,
407,
395,
384,
380,
388,
400,
400,
401,
327,
328,
333,
349,
358,
364,
371,
378,
393,
404,
403,
401,
400,
401,
400,
395,
390,
398,
408,
429,
440,
449,
460,
471,
478,
484,
485,
484,
482,
486,
498,
511,
522,
522,
524,
532,
539,
549,
560,
569,
571,
575,
584,
584,
578,
575,
573,
576,
583,
586,
593,
594,
586,
572,
558,
547,
540,
533,
522,
519,
522,
520,
508,
493,
478,
462,
460,
456,
447,
435,
420,
405,
388,
370,
353,
352,
354,
358,
363,
354,
337,
329,
321,
311,
308,
305,
308,
315,
325,
327,
327,
333,
350,
366,
373,
376,
381,
384,
389,
392,
392,
402,
411,
418,
420,
420,
410,
408,
408,
399,
388,
388,
379,
380,
384,
392,
400,
403,
421,
435,
438,
441,
456,
470,
482,
489,
488,
500,
517,
516,
507,
490,
497,
503,
498,
490,
494,
504,
512,
503,
495,
508,
528,
542,
537,
518,
499,
485,
475,
467,
456,
443,
452,
464,
470,
460,
444,
424,
408,
394,
383,
370,
355,
343,
338,
342,
352,
361,
368,
377,
376,
384,
396,
404,
412,
400,
384,
375,
386,
394,
396,
399,
405,
414,
424,
436,
436,
429,
435,
444,
448,
451,
447,
443,
439,
429,
423,
416,
414,
413,
412,
407,
412,
419,
427,
424,
416,
418,
427,
432,
432,
432,
424,
412,
395,
388,
386,
377,
359,
356,
350,
350,
350,
359,
381,
395,
407,
406,
401,
396,
394,
393,
393,
393,
393,
388,
390,
396,
399,
403,
405,
409,
413,
417,
421,
427,
432,
438,
444,
452,
458,
459,
461,
471,
483,
493,
505,
510,
511,
525,
538,
549,
554,
564,
580,
592,
580,
570,
577,
585,
589,
596,
597,
588,
578,
569,
560,
568,
575,
575,
570,
560,
552,
542,
531,
514,
508,
501,
489,
480,
473,
465,
461,
451,
439,
429,
432,
435,
436,
434,
433,
436,
440,
441,
450,
457,
463,
470,
478,
487,
489,
487,
495,
500,
501,
498,
498,
502,
503,
496,
487,
474,
454,
438,
420,
403,
386,
382,
381,
389,
409,
424,
437,
443,
444,
449,
456,
456,
449,
440,
448,
454,
456,
456,
455,
448,
438,
427,
420,
417,
418,
415,
413,
413,
414,
412,
409,
404,
392,
381,
377,
381,
385,
391,
403,
330,
330,
343,
355,
366,
370,
376,
384,
400,
410,
410,
407,
407,
408,
409,
406,
400,
409,
422,
429,
437,
453,
466,
478,
485,
490,
494,
495,
495,
492,
502,
519,
530,
536,
540,
548,
554,
561,
571,
570,
573,
577,
577,
578,
578,
575,
573,
571,
572,
582,
588,
591,
590,
582,
567,
554,
539,
524,
515,
510,
505,
504,
497,
485,
471,
455,
437,
436,
436,
426,
411,
397,
376,
361,
368,
376,
374,
377,
370,
353,
345,
344,
330,
316,
315,
307,
305,
311,
317,
320,
326,
337,
350,
358,
358,
358,
365,
370,
378,
388,
398,
404,
405,
404,
401,
406,
398,
393,
401,
392,
375,
377,
374,
376,
386,
398,
417,
425,
431,
440,
442,
447,
463,
480,
494,
503,
506,
511,
522,
528,
520,
505,
502,
509,
509,
499,
504,
509,
518,
515,
509,
514,
533,
542,
530,
513,
498,
475,
460,
452,
444,
437,
439,
450,
460,
464,
453,
437,
418,
403,
386,
369,
348,
341,
338,
341,
346,
351,
356,
361,
367,
369,
377,
391,
407,
417,
398,
385,
392,
404,
408,
409,
409,
413,
422,
436,
443,
442,
440,
454,
453,
449,
443,
439,
435,
429,
424,
420,
418,
416,
414,
411,
410,
412,
417,
419,
422,
426,
426,
433,
439,
443,
440,
426,
413,
410,
404,
398,
386,
367,
357,
354,
355,
363,
381,
398,
402,
393,
389,
386,
386,
388,
389,
391,
389,
395,
402,
400,
402,
402,
406,
409,
413,
417,
425,
431,
439,
446,
450,
456,
465,
470,
470,
473,
483,
495,
510,
521,
527,
531,
536,
553,
563,
574,
585,
598,
586,
572,
578,
587,
594,
599,
598,
585,
576,
567,
558,
563,
565,
568,
562,
553,
544,
534,
522,
509,
496,
487,
482,
475,
470,
464,
456,
448,
437,
424,
424,
426,
425,
423,
424,
427,
428,
439,
450,
454,
459,
465,
476,
481,
482,
484,
491,
502,
500,
496,
490,
494,
492,
482,
472,
460,
449,
438,
426,
405,
387,
377,
378,
391,
405,
419,
430,
432,
435,
444,
451,
448,
442,
437,
442,
448,
449,
449,
450,
446,
438,
428,
415,
413,
414,
412,
409,
409,
408,
408,
405,
398,
384,
377,
375,
380,
388,
398,
409,
340,
342,
350,
361,
372,
378,
384,
394,
408,
417,
417,
414,
414,
416,
418,
414,
406,
425,
437,
442,
441,
450,
464,
473,
485,
494,
500,
505,
507,
510,
504,
518,
534,
546,
553,
560,
568,
573,
575,
573,
573,
573,
575,
575,
574,
573,
573,
571,
572,
577,
582,
584,
588,
585,
572,
559,
544,
534,
527,
518,
507,
495,
483,
471,
457,
444,
431,
416,
418,
412,
400,
377,
370,
380,
389,
397,
396,
385,
373,
364,
360,
354,
341,
326,
325,
311,
305,
308,
313,
317,
323,
330,
334,
338,
343,
356,
365,
372,
379,
388,
394,
391,
389,
390,
384,
393,
386,
381,
390,
383,
373,
370,
375,
382,
388,
401,
420,
436,
446,
452,
458,
452,
466,
484,
501,
508,
516,
525,
535,
538,
534,
522,
512,
518,
520,
514,
514,
516,
523,
529,
528,
527,
538,
536,
518,
506,
494,
474,
458,
448,
442,
440,
434,
435,
442,
448,
445,
439,
424,
403,
381,
361,
346,
339,
340,
344,
348,
353,
353,
355,
362,
365,
367,
380,
398,
415,
414,
401,
396,
407,
417,
420,
420,
415,
424,
437,
449,
453,
457,
461,
450,
440,
435,
430,
428,
424,
420,
418,
416,
415,
413,
412,
412,
413,
415,
424,
427,
429,
428,
429,
435,
442,
452,
444,
439,
434,
426,
411,
398,
379,
363,
356,
362,
373,
390,
392,
386,
383,
381,
381,
385,
390,
393,
394,
400,
402,
403,
402,
402,
405,
406,
410,
413,
415,
426,
433,
443,
453,
460,
466,
473,
480,
480,
476,
483,
494,
508,
522,
534,
542,
549,
556,
557,
565,
582,
601,
593,
581,
582,
590,
597,
601,
598,
584,
575,
566,
553,
550,
548,
558,
554,
545,
538,
528,
518,
510,
501,
490,
479,
474,
464,
461,
456,
450,
447,
438,
428,
419,
414,
413,
415,
417,
425,
436,
444,
448,
454,
461,
473,
478,
474,
486,
498,
500,
495,
491,
486,
486,
485,
473,
458,
446,
438,
427,
413,
396,
381,
372,
375,
385,
400,
412,
417,
418,
430,
442,
442,
438,
434,
429,
437,
446,
448,
449,
446,
442,
438,
427,
416,
408,
405,
406,
404,
404,
405,
405,
403,
390,
377,
373,
375,
384,
396,
407,
418,
348,
351,
356,
365,
376,
384,
392,
404,
416,
425,
427,
423,
422,
424,
428,
420,
412,
431,
446,
457,
451,
447,
452,
470,
488,
497,
505,
512,
521,
525,
525,
521,
532,
547,
559,
566,
574,
576,
577,
575,
573,
573,
573,
573,
573,
572,
571,
570,
568,
572,
572,
575,
577,
573,
566,
558,
554,
551,
542,
529,
514,
499,
485,
471,
454,
437,
422,
410,
401,
394,
388,
392,
397,
404,
411,
414,
412,
398,
385,
376,
370,
362,
347,
332,
330,
314,
309,
308,
311,
315,
320,
328,
330,
342,
356,
369,
380,
382,
380,
377,
383,
375,
376,
381,
368,
384,
377,
369,
378,
371,
366,
368,
379,
398,
408,
416,
424,
435,
452,
462,
471,
474,
471,
474,
485,
497,
508,
522,
532,
541,
546,
541,
530,
530,
530,
524,
528,
530,
535,
540,
540,
540,
540,
537,
512,
493,
479,
466,
456,
455,
456,
452,
442,
425,
426,
430,
427,
421,
411,
394,
374,
350,
343,
341,
346,
352,
352,
362,
369,
374,
378,
382,
377,
379,
398,
413,
423,
413,
408,
406,
418,
429,
432,
426,
424,
436,
448,
461,
469,
466,
450,
438,
432,
426,
426,
422,
414,
412,
413,
413,
412,
412,
412,
416,
420,
427,
429,
428,
424,
424,
429,
440,
449,
452,
454,
447,
436,
419,
400,
383,
366,
357,
362,
379,
379,
377,
376,
376,
377,
380,
387,
395,
404,
404,
404,
404,
403,
399,
403,
404,
409,
411,
414,
419,
425,
433,
441,
453,
462,
473,
482,
490,
492,
488,
486,
490,
502,
515,
526,
537,
542,
537,
546,
559,
574,
590,
591,
588,
588,
594,
598,
602,
598,
586,
574,
563,
551,
542,
546,
552,
547,
539,
528,
514,
506,
503,
496,
489,
482,
478,
472,
466,
462,
456,
454,
446,
437,
425,
414,
409,
405,
410,
421,
433,
440,
442,
449,
458,
466,
468,
474,
486,
495,
494,
488,
486,
482,
476,
476,
466,
448,
432,
424,
415,
402,
387,
374,
369,
373,
384,
396,
404,
410,
419,
432,
438,
432,
427,
424,
430,
440,
449,
452,
451,
442,
440,
431,
422,
413,
404,
399,
401,
399,
398,
400,
399,
393,
384,
371,
371,
376,
390,
401,
414,
426,
355,
358,
363,
369,
378,
386,
394,
405,
419,
429,
432,
430,
428,
430,
430,
424,
412,
435,
452,
466,
467,
460,
456,
479,
492,
502,
513,
521,
529,
536,
538,
534,
536,
548,
561,
569,
575,
577,
578,
576,
574,
574,
574,
574,
573,
572,
572,
569,
570,
570,
567,
564,
565,
565,
566,
567,
567,
561,
551,
537,
522,
506,
490,
477,
462,
448,
428,
417,
420,
417,
411,
406,
416,
424,
426,
419,
417,
408,
397,
387,
373,
362,
351,
338,
332,
320,
309,
310,
312,
314,
320,
329,
344,
356,
366,
368,
369,
368,
362,
366,
368,
360,
361,
369,
358,
371,
370,
363,
362,
363,
362,
369,
386,
408,
422,
432,
442,
446,
452,
467,
481,
490,
489,
486,
482,
483,
494,
507,
520,
533,
550,
551,
549,
542,
538,
532,
538,
545,
550,
546,
536,
525,
531,
535,
525,
500,
480,
466,
468,
472,
473,
468,
456,
435,
413,
406,
400,
399,
395,
384,
364,
348,
342,
345,
355,
363,
369,
367,
377,
387,
394,
394,
386,
383,
397,
414,
430,
429,
426,
420,
416,
432,
443,
441,
437,
438,
446,
460,
475,
474,
458,
444,
437,
436,
438,
432,
425,
417,
414,
412,
411,
412,
414,
417,
423,
426,
426,
421,
415,
417,
423,
430,
434,
434,
433,
435,
429,
416,
400,
381,
366,
358,
360,
367,
369,
370,
371,
373,
376,
381,
389,
402,
415,
415,
413,
407,
403,
402,
402,
404,
405,
406,
413,
418,
427,
435,
444,
455,
466,
475,
486,
502,
504,
499,
493,
490,
497,
504,
513,
522,
526,
529,
544,
555,
565,
580,
592,
597,
595,
596,
600,
602,
596,
586,
575,
563,
546,
535,
537,
541,
540,
533,
521,
506,
492,
488,
485,
481,
476,
472,
467,
465,
461,
452,
448,
446,
441,
435,
422,
415,
402,
402,
416,
427,
430,
434,
439,
447,
453,
466,
481,
491,
491,
489,
484,
482,
479,
472,
460,
453,
438,
422,
411,
404,
391,
378,
369,
367,
373,
384,
391,
399,
410,
419,
428,
429,
424,
425,
427,
434,
439,
442,
447,
444,
439,
430,
423,
416,
407,
400,
394,
391,
390,
390,
393,
392,
386,
377,
366,
370,
378,
392,
406,
422,
433,
359,
364,
367,
374,
380,
386,
394,
402,
420,
432,
440,
437,
435,
435,
431,
426,
419,
435,
451,
465,
477,
478,
469,
472,
491,
502,
514,
524,
532,
541,
550,
550,
549,
553,
562,
568,
574,
579,
582,
581,
579,
576,
577,
575,
575,
574,
572,
576,
577,
575,
572,
571,
561,
558,
564,
574,
570,
557,
549,
536,
522,
509,
494,
480,
466,
448,
435,
441,
441,
437,
430,
424,
427,
430,
422,
412,
414,
414,
403,
388,
371,
356,
343,
345,
340,
320,
311,
316,
314,
318,
320,
328,
341,
356,
355,
350,
350,
350,
344,
350,
350,
345,
346,
351,
354,
356,
358,
356,
354,
358,
361,
370,
387,
404,
420,
434,
447,
457,
464,
477,
490,
502,
509,
508,
506,
507,
510,
514,
522,
529,
540,
559,
553,
538,
528,
526,
528,
536,
545,
540,
532,
519,
515,
524,
530,
509,
492,
486,
488,
491,
491,
488,
468,
448,
424,
405,
390,
383,
381,
373,
355,
344,
342,
351,
367,
377,
384,
385,
386,
395,
405,
401,
389,
391,
393,
414,
430,
441,
440,
437,
429,
431,
451,
454,
453,
451,
456,
470,
479,
474,
462,
453,
448,
444,
443,
442,
436,
425,
414,
411,
410,
413,
416,
418,
421,
422,
414,
413,
409,
410,
414,
418,
419,
419,
414,
416,
412,
402,
394,
379,
365,
358,
359,
363,
366,
367,
369,
373,
377,
383,
389,
400,
413,
417,
415,
411,
403,
400,
400,
402,
402,
402,
411,
413,
420,
431,
442,
455,
467,
481,
494,
508,
514,
510,
504,
500,
497,
498,
505,
512,
519,
531,
549,
564,
576,
588,
600,
609,
604,
600,
600,
599,
593,
583,
576,
564,
546,
536,
528,
529,
529,
527,
516,
502,
486,
475,
473,
472,
467,
461,
459,
457,
452,
444,
437,
438,
441,
440,
429,
419,
409,
398,
408,
417,
420,
423,
425,
440,
460,
477,
490,
492,
489,
486,
480,
477,
476,
468,
454,
434,
425,
412,
400,
393,
382,
372,
366,
366,
373,
382,
389,
400,
410,
417,
422,
423,
422,
424,
427,
431,
436,
437,
438,
437,
431,
423,
415,
409,
402,
394,
388,
384,
382,
382,
384,
381,
377,
370,
363,
372,
382,
396,
411,
426,
439,
364,
368,
371,
376,
381,
386,
391,
402,
420,
434,
442,
442,
440,
440,
435,
426,
423,
432,
444,
459,
473,
488,
486,
479,
481,
495,
507,
523,
530,
539,
549,
552,
554,
557,
562,
568,
572,
578,
584,
585,
585,
585,
587,
585,
585,
584,
584,
587,
589,
589,
588,
586,
582,
570,
560,
558,
553,
550,
540,
529,
517,
504,
492,
477,
465,
457,
458,
460,
461,
456,
448,
444,
438,
430,
415,
401,
406,
403,
391,
378,
366,
352,
353,
354,
338,
318,
315,
324,
330,
321,
323,
325,
333,
339,
339,
338,
337,
337,
338,
339,
341,
341,
342,
343,
345,
347,
350,
351,
353,
357,
361,
373,
384,
396,
410,
422,
437,
454,
468,
484,
497,
508,
517,
520,
522,
525,
532,
538,
542,
546,
550,
561,
556,
542,
529,
520,
521,
525,
530,
528,
524,
515,
509,
516,
526,
525,
512,
507,
510,
504,
500,
486,
468,
450,
426,
409,
392,
373,
364,
354,
349,
343,
345,
356,
369,
381,
393,
400,
403,
405,
412,
401,
398,
403,
401,
411,
427,
439,
449,
451,
449,
444,
458,
471,
473,
470,
472,
478,
473,
462,
455,
446,
439,
441,
439,
436,
437,
427,
416,
409,
409,
410,
415,
418,
424,
426,
421,
413,
402,
399,
402,
405,
405,
403,
399,
399,
392,
386,
381,
368,
363,
360,
361,
365,
366,
367,
370,
374,
378,
382,
388,
394,
401,
408,
409,
403,
393,
393,
395,
396,
400,
406,
415,
419,
421,
426,
436,
449,
463,
477,
488,
505,
520,
521,
512,
507,
506,
504,
509,
515,
521,
533,
546,
555,
567,
580,
594,
602,
596,
588,
582,
593,
590,
581,
574,
563,
546,
537,
523,
522,
520,
521,
511,
496,
484,
473,
462,
459,
457,
453,
448,
445,
443,
437,
430,
428,
432,
432,
427,
422,
409,
397,
402,
406,
411,
415,
429,
451,
466,
483,
488,
484,
480,
477,
476,
470,
466,
460,
450,
424,
412,
402,
391,
382,
374,
367,
366,
370,
374,
380,
387,
396,
405,
413,
416,
423,
424,
424,
425,
427,
432,
434,
434,
431,
426,
418,
408,
399,
393,
387,
381,
376,
376,
374,
376,
373,
368,
364,
363,
374,
389,
403,
419,
429,
431,
370,
373,
375,
377,
382,
386,
392,
401,
419,
435,
449,
449,
446,
444,
439,
428,
422,
426,
438,
453,
471,
485,
490,
492,
489,
491,
499,
509,
521,
534,
544,
550,
553,
558,
564,
569,
573,
582,
588,
592,
595,
599,
600,
598,
596,
598,
602,
605,
605,
603,
601,
597,
590,
581,
564,
548,
543,
534,
527,
520,
509,
497,
487,
486,
482,
481,
483,
483,
479,
475,
468,
458,
446,
430,
413,
402,
393,
390,
377,
365,
360,
358,
356,
352,
330,
316,
321,
333,
344,
337,
328,
326,
326,
331,
332,
332,
333,
335,
337,
339,
342,
344,
345,
345,
344,
343,
347,
353,
359,
363,
365,
370,
377,
387,
396,
413,
430,
446,
462,
477,
483,
492,
503,
509,
512,
520,
530,
541,
550,
552,
557,
561,
559,
548,
535,
523,
514,
516,
521,
520,
514,
510,
500,
507,
524,
536,
530,
524,
507,
496,
491,
481,
468,
452,
424,
403,
389,
374,
362,
355,
348,
346,
351,
356,
367,
380,
393,
406,
415,
417,
416,
402,
407,
417,
420,
420,
425,
436,
449,
461,
467,
466,
474,
487,
484,
476,
469,
468,
460,
445,
442,
440,
433,
439,
433,
426,
429,
429,
419,
407,
402,
408,
412,
417,
426,
426,
426,
418,
404,
391,
390,
391,
393,
386,
384,
383,
382,
378,
370,
365,
362,
362,
363,
367,
368,
369,
371,
374,
377,
380,
385,
389,
392,
396,
398,
394,
393,
390,
390,
396,
402,
410,
420,
428,
432,
433,
433,
443,
458,
474,
488,
502,
518,
527,
521,
514,
516,
514,
515,
523,
534,
538,
534,
542,
556,
568,
579,
584,
577,
565,
568,
583,
586,
576,
568,
559,
546,
536,
528,
514,
516,
524,
508,
492,
481,
471,
462,
450,
445,
443,
441,
435,
432,
429,
422,
417,
419,
419,
419,
412,
400,
389,
393,
402,
412,
422,
437,
453,
469,
483,
478,
469,
465,
469,
470,
464,
458,
452,
438,
418,
400,
390,
383,
375,
368,
366,
370,
377,
384,
390,
394,
398,
405,
417,
428,
429,
430,
429,
427,
429,
430,
431,
431,
427,
420,
412,
402,
391,
383,
377,
374,
369,
369,
365,
368,
366,
362,
361,
363,
376,
392,
406,
416,
417,
420,
379,
378,
379,
381,
386,
389,
395,
403,
420,
440,
453,
451,
451,
450,
442,
432,
425,
436,
445,
467,
483,
490,
493,
499,
503,
505,
506,
508,
513,
526,
535,
543,
549,
557,
564,
572,
580,
590,
596,
602,
608,
614,
616,
615,
613,
614,
618,
620,
619,
616,
612,
606,
596,
588,
577,
562,
548,
536,
524,
510,
501,
500,
506,
504,
502,
501,
500,
499,
496,
489,
479,
468,
451,
437,
421,
408,
398,
391,
384,
375,
365,
354,
348,
338,
325,
314,
325,
340,
356,
352,
340,
326,
326,
329,
333,
329,
335,
341,
344,
348,
350,
356,
358,
355,
349,
346,
348,
355,
364,
376,
377,
372,
372,
382,
397,
412,
426,
440,
454,
462,
469,
477,
483,
486,
494,
507,
523,
535,
545,
552,
560,
554,
546,
547,
538,
523,
513,
510,
513,
511,
504,
503,
499,
503,
519,
528,
532,
518,
495,
477,
470,
467,
464,
453,
431,
413,
397,
383,
368,
357,
350,
351,
364,
369,
372,
377,
388,
398,
410,
418,
413,
409,
416,
424,
430,
436,
441,
448,
455,
467,
478,
481,
486,
490,
478,
465,
453,
455,
454,
443,
427,
428,
425,
430,
427,
424,
427,
428,
418,
404,
401,
406,
411,
417,
425,
424,
420,
415,
402,
390,
381,
381,
378,
376,
375,
374,
372,
368,
365,
363,
362,
363,
366,
367,
368,
369,
370,
373,
376,
378,
381,
384,
386,
388,
390,
389,
388,
390,
395,
403,
408,
416,
424,
430,
440,
445,
442,
440,
456,
473,
484,
496,
511,
520,
518,
522,
525,
525,
522,
532,
546,
555,
550,
540,
549,
559,
566,
571,
566,
558,
559,
567,
578,
573,
566,
559,
547,
544,
539,
518,
505,
510,
507,
494,
482,
471,
462,
450,
439,
434,
431,
428,
419,
413,
408,
406,
405,
404,
407,
402,
392,
387,
394,
405,
415,
425,
437,
453,
471,
476,
471,
456,
450,
459,
465,
465,
456,
447,
434,
420,
403,
387,
377,
369,
365,
368,
376,
386,
397,
404,
407,
411,
416,
428,
435,
437,
437,
436,
427,
430,
431,
430,
428,
422,
412,
402,
395,
386,
376,
368,
365,
364,
364,
361,
360,
359,
357,
358,
365,
378,
392,
401,
403,
403,
416,
386,
388,
385,
387,
389,
393,
403,
417,
426,
449,
456,
458,
458,
456,
448,
436,
431,
448,
455,
468,
486,
496,
502,
505,
511,
516,
522,
525,
524,
522,
530,
538,
548,
557,
568,
576,
583,
600,
612,
618,
623,
626,
627,
625,
626,
628,
631,
634,
636,
634,
631,
622,
611,
602,
591,
577,
559,
545,
526,
510,
516,
525,
526,
522,
522,
518,
514,
512,
508,
500,
485,
472,
456,
442,
430,
416,
405,
398,
390,
380,
366,
357,
346,
336,
326,
316,
328,
341,
359,
357,
343,
326,
326,
337,
346,
343,
343,
352,
357,
363,
369,
373,
374,
367,
358,
351,
350,
356,
364,
373,
387,
393,
393,
390,
389,
404,
418,
431,
445,
455,
463,
473,
477,
491,
504,
521,
535,
544,
554,
561,
563,
551,
535,
540,
543,
533,
522,
512,
503,
500,
497,
495,
496,
501,
515,
514,
518,
512,
493,
470,
447,
446,
447,
447,
437,
420,
405,
390,
372,
358,
351,
354,
368,
378,
381,
381,
381,
389,
401,
416,
417,
412,
416,
424,
432,
441,
447,
456,
465,
476,
478,
474,
479,
482,
467,
452,
432,
441,
439,
438,
425,
417,
417,
424,
416,
420,
431,
423,
410,
398,
400,
409,
417,
424,
426,
423,
418,
414,
398,
385,
377,
376,
375,
374,
373,
369,
367,
365,
363,
363,
364,
364,
364,
367,
365,
366,
369,
373,
374,
377,
379,
380,
379,
380,
383,
388,
388,
390,
398,
405,
414,
421,
430,
438,
444,
454,
457,
453,
453,
463,
474,
485,
499,
506,
511,
518,
526,
530,
531,
539,
553,
562,
555,
546,
547,
555,
558,
561,
557,
550,
548,
559,
571,
568,
560,
559,
553,
550,
540,
521,
498,
496,
500,
497,
483,
473,
463,
456,
448,
437,
425,
414,
408,
401,
397,
393,
394,
393,
396,
393,
384,
386,
396,
406,
416,
423,
436,
452,
465,
461,
459,
447,
438,
444,
454,
458,
454,
446,
432,
416,
401,
386,
373,
365,
366,
369,
383,
397,
408,
416,
426,
429,
433,
439,
445,
446,
446,
441,
436,
430,
428,
426,
421,
412,
402,
393,
387,
379,
371,
364,
360,
358,
358,
354,
353,
353,
353,
358,
365,
375,
384,
387,
388,
398,
411,
392,
393,
390,
393,
397,
401,
416,
427,
445,
457,
462,
464,
465,
461,
452,
439,
436,
450,
467,
465,
479,
495,
504,
512,
517,
523,
534,
539,
539,
538,
535,
546,
556,
563,
573,
581,
593,
611,
622,
630,
635,
637,
638,
636,
638,
639,
643,
649,
656,
656,
651,
640,
628,
616,
601,
584,
565,
545,
524,
523,
532,
539,
539,
536,
536,
535,
530,
526,
519,
506,
486,
474,
460,
446,
434,
423,
411,
402,
395,
387,
376,
367,
356,
343,
329,
317,
326,
338,
349,
350,
345,
327,
329,
340,
354,
360,
360,
364,
374,
380,
384,
388,
390,
376,
366,
357,
354,
359,
365,
372,
381,
390,
404,
413,
411,
413,
416,
437,
454,
469,
483,
496,
502,
510,
514,
528,
541,
552,
563,
564,
561,
548,
534,
523,
539,
540,
534,
525,
518,
508,
502,
497,
489,
495,
503,
499,
504,
506,
491,
472,
443,
426,
427,
428,
428,
420,
408,
391,
372,
358,
351,
352,
359,
367,
371,
372,
380,
389,
401,
415,
425,
422,
424,
430,
435,
439,
448,
458,
470,
478,
468,
460,
468,
473,
459,
444,
431,
426,
425,
423,
418,
410,
413,
415,
411,
421,
428,
416,
404,
398,
402,
414,
423,
426,
426,
422,
417,
408,
394,
382,
376,
374,
373,
375,
370,
366,
365,
364,
364,
365,
365,
365,
367,
369,
369,
368,
373,
374,
375,
375,
376,
376,
376,
378,
381,
385,
389,
394,
401,
405,
412,
419,
430,
445,
455,
463,
469,
470,
467,
473,
478,
479,
486,
494,
505,
516,
525,
527,
535,
541,
549,
554,
555,
550,
552,
553,
553,
551,
548,
545,
544,
556,
560,
559,
554,
553,
550,
542,
531,
516,
498,
483,
489,
494,
487,
476,
469,
464,
456,
446,
435,
422,
409,
399,
389,
388,
386,
385,
384,
379,
376,
385,
397,
406,
412,
421,
434,
449,
454,
440,
441,
440,
428,
431,
441,
446,
444,
439,
422,
406,
391,
380,
369,
365,
369,
379,
391,
406,
418,
429,
440,
445,
449,
455,
458,
457,
452,
441,
432,
427,
423,
417,
409,
403,
394,
386,
379,
373,
366,
358,
353,
353,
354,
352,
351,
356,
362,
363,
364,
369,
375,
379,
388,
400,
411,
397,
399,
400,
403,
410,
413,
421,
440,
455,
466,
467,
472,
472,
465,
455,
442,
437,
450,
472,
484,
479,
490,
504,
512,
523,
534,
544,
552,
555,
555,
553,
555,
563,
571,
577,
584,
602,
618,
630,
638,
646,
645,
645,
646,
648,
654,
661,
665,
678,
674,
665,
652,
637,
619,
601,
580,
561,
541,
542,
542,
546,
551,
551,
551,
549,
548,
545,
539,
529,
519,
504,
485,
474,
458,
444,
426,
415,
408,
399,
394,
386,
376,
364,
349,
333,
322,
327,
336,
339,
348,
353,
336,
327,
343,
359,
371,
378,
378,
383,
390,
397,
403,
396,
383,
371,
363,
360,
366,
374,
378,
384,
390,
400,
413,
427,
433,
436,
439,
460,
478,
493,
508,
521,
527,
530,
535,
548,
559,
562,
560,
558,
550,
539,
523,
523,
533,
540,
538,
535,
527,
519,
508,
494,
486,
489,
487,
491,
495,
490,
472,
449,
424,
404,
405,
406,
405,
401,
387,
369,
356,
351,
352,
357,
362,
364,
368,
379,
386,
396,
408,
426,
429,
427,
434,
442,
447,
455,
465,
477,
472,
456,
450,
459,
466,
456,
436,
428,
421,
418,
414,
408,
402,
405,
402,
407,
418,
422,
409,
399,
395,
405,
415,
419,
420,
423,
426,
415,
400,
392,
387,
379,
375,
372,
373,
368,
365,
364,
364,
365,
367,
368,
368,
372,
376,
378,
379,
376,
375,
375,
375,
376,
378,
378,
379,
383,
388,
392,
400,
407,
413,
417,
424,
427,
442,
456,
468,
477,
484,
484,
489,
501,
496,
487,
489,
503,
510,
514,
516,
523,
531,
540,
545,
546,
549,
550,
549,
546,
542,
539,
536,
542,
549,
548,
549,
547,
549,
540,
526,
512,
505,
495,
475,
475,
484,
489,
481,
476,
470,
463,
453,
443,
430,
419,
410,
400,
393,
388,
379,
379,
374,
373,
383,
394,
401,
408,
418,
430,
441,
438,
423,
423,
428,
418,
417,
426,
430,
425,
419,
411,
398,
385,
374,
366,
365,
373,
388,
401,
416,
430,
445,
455,
462,
464,
469,
470,
462,
449,
437,
427,
419,
416,
408,
398,
391,
386,
379,
370,
364,
358,
352,
351,
351,
350,
349,
353,
363,
371,
373,
367,
370,
377,
388,
399,
409,
419,
408,
412,
416,
420,
428,
428,
424,
442,
455,
466,
474,
477,
478,
469,
458,
448,
446,
450,
471,
490,
498,
488,
501,
512,
525,
539,
549,
560,
568,
572,
568,
564,
571,
577,
582,
587,
607,
623,
635,
646,
650,
652,
652,
657,
666,
673,
681,
689,
685,
676,
662,
650,
634,
613,
597,
574,
560,
562,
562,
558,
556,
560,
562,
563,
562,
562,
561,
552,
542,
530,
514,
499,
483,
470,
452,
433,
428,
413,
405,
402,
394,
383,
370,
354,
336,
324,
330,
335,
347,
356,
357,
348,
340,
338,
354,
368,
374,
376,
383,
389,
398,
405,
400,
389,
376,
368,
364,
373,
391,
395,
396,
398,
392,
406,
423,
441,
454,
458,
461,
476,
492,
505,
517,
527,
540,
549,
556,
565,
558,
549,
554,
551,
541,
530,
518,
521,
526,
533,
537,
540,
533,
520,
502,
488,
473,
469,
475,
479,
484,
474,
455,
430,
405,
388,
387,
388,
387,
374,
362,
354,
351,
355,
362,
367,
372,
379,
381,
386,
390,
405,
420,
430,
437,
437,
444,
454,
462,
474,
479,
464,
446,
441,
452,
456,
456,
442,
429,
418,
419,
414,
407,
402,
398,
399,
406,
415,
416,
403,
395,
392,
399,
402,
406,
412,
419,
429,
424,
410,
402,
393,
382,
375,
371,
370,
366,
365,
364,
364,
366,
368,
370,
374,
377,
381,
388,
391,
384,
377,
379,
378,
377,
379,
382,
382,
386,
388,
390,
400,
407,
417,
424,
431,
437,
440,
451,
465,
477,
488,
494,
501,
514,
509,
497,
492,
498,
499,
499,
506,
510,
516,
523,
532,
538,
546,
547,
545,
538,
536,
534,
534,
540,
540,
540,
539,
538,
542,
536,
516,
497,
489,
487,
471,
465,
474,
481,
486,
486,
478,
469,
460,
451,
441,
432,
421,
412,
404,
396,
386,
376,
372,
367,
380,
389,
394,
405,
413,
421,
428,
421,
410,
409,
411,
405,
405,
412,
412,
408,
402,
397,
388,
377,
370,
365,
365,
376,
392,
407,
425,
445,
455,
465,
469,
476,
479,
476,
460,
444,
432,
419,
410,
405,
399,
391,
383,
377,
371,
363,
354,
352,
349,
348,
346,
350,
358,
361,
369,
380,
383,
378,
379,
388,
397,
411,
422,
428,
429,
430,
434,
439,
441,
443,
429,
442,
454,
465,
473,
478,
480,
471,
461,
454,
454,
451,
470,
491,
502,
496,
499,
511,
523,
539,
552,
562,
574,
576,
575,
576,
575,
584,
590,
597,
613,
627,
639,
648,
654,
658,
661,
672,
682,
692,
698,
698,
682,
667,
654,
640,
623,
605,
589,
584,
584,
584,
581,
572,
564,
572,
573,
573,
573,
573,
571,
563,
555,
541,
526,
511,
493,
478,
458,
451,
441,
429,
419,
410,
402,
390,
375,
356,
337,
325,
328,
333,
343,
356,
366,
362,
354,
347,
350,
361,
361,
369,
378,
388,
395,
402,
406,
395,
382,
372,
368,
375,
394,
407,
415,
414,
412,
411,
418,
433,
455,
473,
477,
478,
486,
497,
507,
512,
526,
536,
546,
559,
554,
546,
549,
549,
544,
539,
524,
510,
511,
516,
519,
530,
540,
530,
514,
496,
480,
462,
451,
461,
471,
473,
457,
436,
412,
393,
377,
374,
371,
363,
357,
353,
354,
358,
367,
378,
380,
386,
390,
393,
397,
402,
416,
431,
442,
450,
450,
452,
463,
476,
477,
459,
440,
432,
441,
446,
447,
453,
441,
434,
428,
419,
411,
402,
396,
395,
402,
404,
406,
395,
390,
389,
390,
400,
408,
418,
426,
430,
422,
409,
401,
391,
381,
374,
369,
367,
366,
365,
365,
365,
365,
369,
371,
376,
381,
388,
397,
400,
390,
385,
387,
389,
382,
380,
384,
388,
390,
392,
394,
399,
403,
413,
423,
433,
443,
452,
458,
462,
473,
486,
500,
513,
524,
517,
504,
496,
496,
497,
500,
505,
510,
515,
523,
531,
538,
545,
545,
539,
535,
532,
531,
531,
537,
535,
532,
530,
531,
538,
527,
512,
492,
480,
477,
464,
456,
463,
477,
489,
489,
479,
472,
463,
454,
452,
441,
431,
421,
412,
404,
396,
385,
368,
368,
375,
381,
388,
398,
404,
408,
411,
405,
397,
396,
398,
394,
391,
397,
395,
391,
386,
381,
374,
369,
365,
368,
374,
383,
395,
410,
429,
446,
456,
462,
467,
474,
480,
473,
454,
436,
424,
413,
401,
394,
390,
382,
376,
367,
360,
355,
350,
349,
347,
351,
354,
360,
370,
375,
376,
390,
392,
388,
389,
399,
407,
418,
430,
428,
441,
442,
446,
451,
454,
450,
438,
437,
450,
463,
473,
478,
481,
475,
465,
464,
462,
458,
469,
489,
505,
506,
498,
505,
518,
534,
551,
560,
567,
574,
579,
583,
586,
589,
597,
610,
620,
631,
642,
650,
657,
669,
676,
688,
700,
706,
704,
690,
674,
660,
645,
630,
618,
610,
606,
603,
603,
601,
597,
584,
574,
582,
582,
580,
580,
579,
577,
573,
565,
554,
538,
521,
502,
484,
471,
466,
456,
443,
429,
413,
402,
392,
377,
363,
345,
326,
328,
331,
338,
344,
361,
373,
371,
361,
350,
352,
357,
366,
376,
388,
395,
405,
413,
401,
390,
378,
371,
377,
386,
401,
418,
429,
429,
428,
433,
433,
448,
464,
480,
492,
501,
501,
502,
520,
537,
549,
560,
562,
551,
547,
542,
540,
540,
538,
534,
518,
511,
506,
509,
518,
528,
535,
524,
505,
485,
468,
448,
442,
451,
461,
452,
431,
412,
394,
378,
364,
363,
359,
357,
358,
357,
367,
374,
380,
390,
394,
399,
401,
407,
408,
419,
435,
448,
459,
463,
463,
467,
476,
471,
455,
438,
425,
431,
434,
429,
444,
445,
441,
435,
428,
416,
401,
394,
391,
391,
392,
393,
386,
382,
384,
389,
402,
411,
416,
415,
415,
410,
401,
388,
380,
375,
370,
367,
366,
366,
367,
366,
364,
366,
368,
373,
376,
381,
389,
400,
402,
393,
391,
394,
394,
388,
386,
386,
389,
394,
398,
402,
405,
404,
411,
418,
428,
440,
451,
460,
469,
479,
488,
504,
517,
529,
524,
510,
499,
499,
502,
503,
509,
514,
520,
526,
533,
542,
544,
542,
538,
536,
532,
526,
528,
529,
528,
525,
522,
525,
527,
520,
501,
487,
478,
466,
456,
449,
462,
474,
490,
486,
478,
468,
458,
449,
442,
439,
432,
425,
417,
408,
403,
388,
375,
362,
368,
373,
382,
389,
393,
395,
398,
394,
386,
385,
386,
382,
378,
383,
380,
373,
370,
368,
365,
365,
368,
375,
385,
396,
405,
413,
428,
438,
450,
460,
465,
475,
478,
462,
446,
428,
414,
404,
394,
385,
380,
372,
365,
360,
353,
350,
350,
347,
352,
359,
366,
368,
379,
390,
386,
396,
402,
397,
399,
409,
419,
428,
427,
421,
443,
446,
449,
454,
459,
464,
442,
435,
450,
461,
473,
482,
485,
477,
474,
474,
465,
461,
467,
486,
502,
515,
513,
512,
513,
526,
542,
551,
563,
573,
583,
587,
590,
595,
605,
618,
629,
640,
649,
655,
665,
678,
690,
704,
710,
710,
697,
682,
669,
656,
648,
644,
637,
636,
626,
621,
624,
620,
608,
596,
586,
592,
592,
589,
587,
587,
584,
578,
572,
565,
547,
527,
507,
495,
490,
482,
469,
452,
437,
425,
414,
401,
390,
373,
353,
336,
332,
331,
341,
350,
362,
374,
381,
372,
359,
356,
369,
381,
391,
403,
412,
416,
419,
410,
397,
377,
375,
379,
394,
404,
420,
432,
443,
450,
449,
437,
445,
460,
478,
492,
498,
508,
518,
535,
547,
558,
564,
558,
549,
545,
539,
536,
538,
536,
539,
533,
527,
516,
500,
503,
514,
524,
524,
507,
488,
470,
449,
426,
434,
444,
444,
426,
407,
390,
376,
367,
362,
361,
362,
365,
369,
376,
382,
387,
391,
402,
407,
411,
417,
420,
423,
437,
453,
464,
468,
465,
465,
466,
462,
450,
436,
420,
419,
420,
420,
430,
433,
432,
424,
414,
405,
395,
388,
386,
385,
384,
383,
381,
380,
381,
384,
391,
397,
399,
399,
397,
395,
388,
379,
373,
370,
367,
366,
367,
368,
368,
366,
365,
368,
370,
372,
376,
378,
387,
390,
396,
398,
400,
403,
398,
388,
390,
389,
389,
396,
402,
409,
415,
416,
415,
417,
424,
429,
441,
452,
465,
478,
490,
501,
516,
533,
530,
512,
502,
502,
508,
511,
510,
516,
523,
534,
541,
549,
548,
543,
540,
538,
532,
524,
524,
524,
523,
521,
520,
513,
511,
508,
502,
492,
476,
462,
449,
447,
461,
479,
488,
482,
475,
465,
456,
447,
436,
425,
421,
417,
411,
407,
400,
391,
376,
367,
363,
366,
374,
378,
380,
384,
387,
383,
374,
373,
373,
366,
365,
366,
364,
362,
362,
364,
365,
371,
375,
383,
396,
411,
422,
426,
438,
449,
459,
467,
473,
478,
468,
451,
437,
422,
407,
396,
388,
379,
371,
365,
359,
353,
351,
350,
347,
350,
356,
368,
378,
383,
388,
399,
399,
404,
410,
407,
410,
416,
421,
423,
417,
410,
442,
446,
449,
453,
459,
464,
443,
439,
451,
463,
474,
486,
488,
484,
480,
478,
472,
468,
466,
486,
501,
519,
525,
524,
521,
520,
532,
544,
558,
572,
584,
589,
594,
604,
616,
625,
637,
648,
657,
665,
674,
687,
707,
717,
717,
705,
693,
680,
667,
667,
664,
658,
659,
655,
644,
642,
640,
633,
623,
605,
596,
606,
607,
600,
596,
594,
591,
585,
580,
573,
554,
535,
522,
513,
508,
500,
479,
468,
457,
440,
423,
409,
395,
380,
364,
348,
338,
334,
340,
359,
374,
385,
391,
383,
368,
357,
366,
378,
394,
406,
415,
417,
419,
416,
404,
385,
379,
380,
397,
412,
423,
433,
449,
459,
462,
461,
464,
480,
495,
508,
518,
517,
521,
532,
540,
546,
560,
558,
548,
545,
541,
540,
536,
535,
537,
543,
541,
530,
514,
498,
500,
509,
515,
506,
489,
470,
447,
423,
416,
427,
434,
421,
401,
383,
367,
366,
366,
367,
370,
374,
373,
378,
389,
395,
400,
404,
413,
420,
427,
432,
434,
438,
453,
466,
464,
454,
452,
454,
452,
442,
429,
418,
411,
408,
408,
412,
414,
413,
406,
400,
393,
389,
386,
384,
383,
382,
381,
380,
380,
381,
381,
382,
383,
384,
384,
383,
381,
378,
374,
370,
367,
367,
367,
368,
369,
369,
368,
365,
368,
371,
374,
377,
381,
387,
393,
401,
405,
407,
406,
398,
395,
396,
394,
392,
397,
405,
414,
423,
429,
428,
429,
430,
429,
436,
442,
455,
468,
476,
488,
509,
526,
527,
512,
507,
507,
512,
517,
522,
524,
529,
537,
549,
548,
546,
542,
540,
537,
531,
524,
522,
521,
520,
519,
516,
512,
499,
492,
491,
487,
477,
463,
447,
444,
463,
480,
489,
483,
473,
465,
456,
448,
440,
427,
418,
407,
400,
396,
393,
389,
379,
367,
360,
360,
366,
369,
370,
374,
375,
372,
364,
360,
358,
359,
358,
358,
358,
359,
362,
367,
373,
378,
387,
397,
411,
425,
435,
443,
450,
456,
464,
469,
472,
466,
453,
438,
427,
413,
401,
389,
381,
374,
366,
359,
354,
351,
349,
347,
354,
358,
365,
373,
383,
396,
401,
410,
411,
415,
417,
412,
408,
410,
411,
410,
403,
397,
442,
445,
448,
451,
455,
454,
446,
443,
454,
465,
476,
488,
490,
487,
488,
484,
479,
478,
474,
486,
506,
521,
528,
534,
533,
530,
533,
545,
557,
570,
581,
588,
594,
610,
626,
634,
645,
656,
667,
676,
685,
698,
721,
726,
721,
708,
695,
681,
686,
686,
680,
677,
679,
673,
664,
663,
657,
646,
629,
611,
611,
623,
625,
618,
610,
606,
601,
596,
588,
581,
566,
552,
540,
532,
522,
511,
502,
488,
475,
453,
431,
414,
399,
385,
370,
354,
342,
336,
338,
350,
363,
380,
394,
391,
376,
362,
370,
383,
391,
397,
401,
404,
413,
418,
410,
394,
384,
383,
389,
401,
414,
429,
450,
467,
476,
482,
476,
492,
510,
523,
529,
532,
533,
533,
535,
540,
550,
555,
553,
542,
537,
545,
550,
525,
531,
545,
552,
545,
526,
508,
488,
488,
501,
503,
487,
468,
447,
421,
402,
408,
418,
416,
398,
377,
369,
367,
371,
375,
378,
383,
382,
385,
392,
403,
410,
414,
414,
426,
434,
443,
449,
451,
459,
467,
463,
447,
436,
440,
439,
430,
417,
409,
402,
398,
394,
391,
397,
396,
394,
391,
388,
386,
386,
386,
386,
386,
385,
385,
386,
386,
384,
382,
381,
380,
379,
377,
375,
373,
371,
369,
368,
368,
370,
372,
372,
372,
370,
367,
367,
372,
375,
379,
385,
392,
399,
407,
415,
415,
407,
402,
401,
404,
402,
399,
399,
405,
417,
429,
439,
443,
444,
443,
441,
438,
439,
448,
460,
475,
494,
513,
526,
524,
512,
508,
512,
516,
522,
528,
532,
534,
541,
545,
549,
543,
538,
540,
541,
532,
521,
519,
518,
519,
524,
525,
514,
500,
487,
478,
477,
470,
457,
442,
438,
458,
475,
492,
494,
483,
472,
461,
451,
440,
428,
418,
407,
395,
388,
382,
379,
374,
366,
358,
355,
358,
361,
362,
365,
364,
360,
356,
353,
353,
355,
357,
359,
361,
364,
368,
374,
383,
392,
404,
418,
427,
435,
444,
454,
461,
465,
469,
470,
462,
448,
438,
426,
416,
404,
395,
384,
375,
368,
361,
353,
351,
347,
347,
356,
362,
368,
375,
383,
391,
404,
414,
419,
418,
416,
410,
401,
394,
394,
394,
392,
389,
383,
442,
445,
447,
449,
451,
451,
450,
451,
462,
471,
479,
487,
491,
491,
492,
488,
488,
489,
486,
484,
500,
516,
529,
540,
545,
545,
544,
547,
556,
566,
579,
588,
596,
615,
633,
648,
654,
666,
676,
685,
693,
708,
723,
733,
725,
714,
703,
702,
706,
705,
696,
698,
699,
692,
686,
681,
671,
655,
636,
619,
626,
638,
641,
636,
630,
623,
614,
609,
603,
593,
581,
573,
561,
551,
541,
526,
516,
504,
485,
465,
440,
419,
402,
387,
372,
358,
346,
339,
336,
343,
352,
373,
390,
395,
381,
368,
368,
380,
380,
387,
397,
405,
412,
417,
417,
401,
391,
387,
401,
414,
427,
438,
455,
467,
479,
490,
497,
505,
519,
531,
539,
546,
548,
550,
548,
547,
556,
563,
558,
547,
535,
538,
542,
530,
522,
538,
548,
550,
535,
516,
500,
481,
485,
496,
485,
466,
446,
420,
402,
394,
402,
407,
396,
379,
368,
367,
375,
382,
388,
393,
393,
390,
400,
408,
418,
424,
420,
426,
437,
454,
463,
467,
470,
468,
461,
443,
426,
425,
420,
409,
407,
404,
401,
398,
395,
394,
392,
390,
389,
389,
389,
388,
389,
391,
393,
393,
394,
392,
398,
398,
392,
389,
388,
385,
381,
378,
376,
373,
372,
371,
371,
372,
374,
376,
376,
375,
373,
370,
371,
371,
375,
379,
387,
395,
404,
410,
412,
407,
404,
407,
409,
412,
410,
407,
401,
408,
420,
431,
444,
454,
459,
458,
454,
448,
446,
449,
466,
483,
502,
518,
531,
527,
516,
513,
513,
520,
526,
535,
540,
539,
544,
560,
554,
540,
536,
540,
549,
541,
526,
517,
515,
521,
524,
516,
511,
498,
487,
477,
465,
458,
449,
439,
434,
449,
469,
485,
501,
488,
476,
468,
456,
446,
435,
421,
407,
398,
389,
384,
376,
365,
362,
354,
352,
353,
354,
355,
355,
354,
352,
350,
349,
351,
355,
361,
366,
367,
373,
375,
382,
392,
404,
419,
434,
444,
451,
454,
460,
465,
467,
470,
463,
448,
434,
423,
414,
404,
395,
388,
380,
369,
361,
356,
352,
347,
347,
353,
361,
369,
378,
385,
393,
401,
405,
412,
411,
409,
405,
397,
389,
380,
376,
381,
382,
376,
379,
442,
445,
447,
449,
450,
452,
450,
453,
466,
477,
481,
487,
490,
495,
496,
495,
496,
496,
496,
495,
502,
518,
533,
548,
555,
558,
558,
556,
558,
567,
580,
589,
598,
621,
641,
654,
662,
675,
685,
694,
703,
713,
730,
739,
731,
720,
709,
715,
721,
723,
717,
717,
717,
712,
705,
693,
676,
658,
641,
632,
644,
655,
659,
657,
649,
637,
631,
624,
618,
608,
598,
587,
576,
564,
549,
528,
514,
500,
485,
466,
443,
420,
401,
387,
373,
361,
349,
340,
337,
341,
357,
367,
383,
400,
389,
374,
374,
376,
379,
385,
391,
398,
403,
410,
416,
413,
403,
387,
394,
408,
420,
431,
444,
456,
468,
484,
502,
520,
534,
546,
554,
561,
565,
564,
562,
562,
567,
562,
561,
549,
534,
528,
538,
534,
521,
536,
540,
538,
539,
523,
504,
486,
475,
486,
478,
460,
440,
420,
407,
392,
386,
392,
390,
376,
368,
369,
378,
390,
394,
401,
405,
400,
405,
415,
424,
432,
426,
434,
440,
457,
470,
476,
467,
464,
457,
444,
427,
412,
409,
405,
404,
404,
405,
404,
402,
399,
396,
394,
392,
394,
395,
393,
394,
399,
402,
404,
404,
402,
410,
413,
410,
405,
401,
394,
388,
385,
379,
377,
375,
374,
374,
375,
377,
379,
380,
378,
377,
374,
374,
374,
375,
376,
383,
392,
400,
403,
401,
400,
404,
410,
416,
420,
419,
415,
404,
409,
417,
429,
441,
454,
465,
471,
469,
463,
460,
455,
462,
483,
500,
516,
530,
535,
522,
519,
521,
522,
527,
536,
538,
541,
550,
558,
556,
540,
532,
536,
546,
538,
526,
516,
510,
510,
510,
500,
504,
497,
486,
478,
467,
455,
441,
432,
431,
444,
463,
474,
482,
480,
476,
470,
461,
450,
438,
427,
416,
403,
397,
394,
385,
373,
364,
353,
349,
350,
349,
348,
348,
347,
346,
346,
347,
351,
358,
366,
376,
379,
382,
386,
390,
400,
416,
433,
449,
460,
464,
457,
462,
463,
466,
465,
452,
436,
421,
409,
399,
390,
385,
380,
372,
363,
354,
352,
349,
346,
350,
353,
361,
369,
378,
383,
387,
390,
397,
402,
400,
396,
392,
384,
376,
369,
372,
375,
380,
387,
392,
442,
444,
448,
450,
453,
454,
452,
455,
468,
478,
482,
487,
495,
500,
501,
501,
500,
504,
507,
509,
510,
520,
538,
552,
563,
569,
569,
567,
564,
570,
579,
589,
601,
625,
643,
659,
668,
681,
692,
702,
712,
725,
745,
748,
738,
726,
721,
728,
737,
738,
737,
740,
737,
729,
715,
699,
681,
661,
646,
647,
661,
672,
676,
674,
666,
658,
650,
640,
628,
612,
596,
583,
569,
552,
533,
518,
504,
491,
476,
460,
439,
416,
398,
378,
373,
362,
348,
344,
339,
343,
352,
364,
383,
397,
395,
378,
378,
387,
394,
397,
400,
407,
416,
420,
425,
427,
417,
407,
398,
401,
412,
422,
431,
439,
458,
477,
500,
518,
536,
553,
565,
570,
577,
576,
563,
557,
562,
556,
553,
552,
542,
523,
529,
534,
521,
524,
524,
532,
542,
529,
507,
488,
472,
477,
474,
453,
436,
421,
410,
401,
391,
377,
379,
373,
370,
374,
379,
390,
400,
406,
413,
412,
411,
421,
432,
440,
437,
442,
452,
462,
476,
473,
464,
459,
454,
441,
427,
410,
406,
404,
407,
411,
415,
417,
413,
409,
406,
400,
405,
408,
408,
404,
400,
410,
416,
418,
415,
413,
422,
428,
427,
422,
413,
405,
399,
395,
388,
381,
378,
376,
376,
377,
380,
382,
382,
382,
380,
377,
376,
375,
376,
381,
385,
386,
389,
389,
395,
402,
411,
416,
423,
429,
429,
418,
411,
415,
419,
425,
434,
450,
461,
473,
481,
480,
476,
470,
464,
474,
491,
508,
520,
524,
521,
519,
528,
528,
526,
533,
535,
540,
548,
548,
546,
534,
526,
531,
534,
528,
522,
518,
513,
499,
498,
489,
496,
498,
485,
475,
466,
456,
443,
427,
426,
438,
452,
461,
463,
476,
474,
466,
460,
452,
448,
439,
425,
412,
407,
401,
391,
381,
369,
359,
353,
348,
345,
343,
343,
342,
343,
344,
346,
354,
361,
370,
380,
390,
393,
399,
405,
414,
432,
449,
464,
470,
461,
452,
454,
460,
463,
455,
441,
425,
410,
397,
388,
382,
375,
366,
362,
356,
352,
350,
349,
344,
347,
351,
357,
364,
370,
375,
374,
378,
391,
393,
391,
385,
380,
374,
366,
365,
368,
376,
384,
393,
405,
440,
442,
449,
453,
457,
463,
462,
462,
469,
477,
484,
489,
498,
502,
506,
507,
508,
511,
516,
519,
520,
522,
540,
554,
570,
577,
580,
578,
575,
576,
584,
593,
605,
622,
644,
660,
676,
686,
699,
712,
724,
743,
758,
757,
742,
734,
731,
739,
748,
754,
757,
756,
750,
738,
720,
704,
686,
668,
657,
663,
676,
686,
692,
694,
685,
673,
659,
641,
624,
604,
585,
572,
555,
536,
520,
507,
494,
481,
467,
449,
428,
410,
396,
393,
383,
372,
360,
349,
342,
343,
358,
378,
394,
404,
402,
387,
377,
388,
398,
408,
416,
419,
425,
429,
431,
436,
436,
428,
414,
408,
419,
425,
430,
438,
453,
474,
495,
512,
532,
548,
573,
582,
589,
582,
567,
552,
541,
541,
544,
556,
550,
535,
522,
527,
525,
514,
516,
534,
544,
534,
510,
489,
466,
463,
465,
458,
446,
434,
423,
410,
397,
387,
380,
374,
373,
383,
392,
389,
401,
412,
419,
424,
419,
429,
439,
447,
449,
450,
461,
467,
473,
472,
462,
458,
453,
440,
426,
413,
407,
407,
415,
422,
429,
429,
424,
422,
419,
412,
418,
422,
423,
417,
412,
420,
430,
431,
428,
424,
434,
440,
440,
437,
428,
416,
412,
406,
395,
387,
382,
378,
378,
381,
384,
386,
387,
386,
385,
382,
377,
375,
376,
379,
384,
390,
391,
393,
401,
410,
417,
425,
429,
438,
439,
421,
417,
427,
432,
435,
436,
440,
450,
465,
480,
486,
488,
484,
479,
477,
488,
499,
509,
514,
513,
523,
537,
538,
529,
530,
533,
537,
543,
544,
538,
524,
522,
523,
521,
519,
518,
515,
512,
501,
486,
481,
483,
489,
489,
475,
462,
453,
441,
430,
423,
430,
440,
444,
456,
465,
468,
463,
459,
453,
452,
443,
431,
423,
414,
405,
394,
386,
376,
365,
355,
344,
340,
340,
339,
339,
341,
345,
352,
359,
364,
374,
385,
396,
404,
412,
420,
433,
449,
464,
475,
469,
457,
449,
452,
456,
456,
444,
430,
414,
401,
390,
381,
372,
364,
360,
355,
352,
350,
347,
345,
344,
345,
349,
354,
359,
364,
364,
364,
375,
381,
382,
380,
375,
370,
364,
360,
360,
366,
374,
390,
396,
402,
435,
440,
446,
453,
457,
463,
466,
463,
469,
477,
483,
490,
498,
503,
509,
513,
515,
517,
522,
526,
528,
532,
541,
556,
572,
583,
587,
588,
587,
587,
592,
599,
616,
636,
654,
672,
687,
700,
712,
730,
740,
754,
763,
758,
745,
737,
741,
749,
760,
768,
774,
771,
761,
746,
727,
711,
697,
681,
673,
680,
688,
699,
708,
706,
688,
669,
654,
634,
613,
591,
575,
557,
538,
522,
510,
496,
483,
471,
456,
438,
420,
417,
415,
404,
392,
380,
366,
353,
345,
345,
357,
377,
392,
400,
403,
397,
383,
386,
393,
401,
406,
410,
415,
420,
427,
438,
444,
440,
425,
416,
433,
448,
452,
455,
465,
488,
504,
525,
541,
560,
580,
596,
598,
586,
575,
560,
541,
524,
533,
550,
552,
541,
521,
526,
531,
520,
515,
527,
538,
535,
514,
490,
468,
453,
450,
451,
447,
442,
435,
421,
407,
395,
386,
378,
376,
381,
390,
397,
402,
412,
423,
432,
432,
439,
449,
457,
462,
454,
460,
464,
472,
473,
467,
462,
454,
442,
429,
413,
409,
411,
423,
435,
440,
438,
435,
436,
433,
426,
431,
437,
437,
433,
425,
428,
441,
443,
439,
434,
446,
455,
454,
450,
442,
434,
428,
417,
402,
394,
388,
385,
384,
386,
388,
391,
392,
391,
390,
390,
384,
377,
376,
377,
381,
385,
388,
390,
400,
411,
424,
433,
440,
441,
443,
426,
427,
434,
442,
449,
449,
450,
454,
460,
467,
477,
488,
494,
497,
498,
494,
495,
499,
506,
512,
522,
536,
535,
530,
527,
530,
534,
535,
534,
529,
524,
520,
517,
512,
511,
514,
512,
502,
493,
483,
473,
472,
476,
482,
480,
467,
453,
440,
431,
425,
422,
427,
433,
444,
453,
462,
463,
456,
448,
443,
441,
434,
424,
412,
403,
393,
384,
375,
365,
354,
344,
339,
339,
338,
342,
346,
352,
360,
366,
371,
376,
386,
398,
410,
422,
433,
445,
461,
469,
469,
461,
452,
443,
447,
453,
448,
432,
418,
404,
392,
383,
376,
367,
359,
353,
351,
348,
346,
345,
343,
342,
343,
345,
350,
354,
355,
356,
358,
362,
364,
366,
365,
365,
362,
358,
357,
358,
364,
368,
371,
380,
387,
432,
436,
442,
448,
452,
458,
464,
467,
469,
475,
480,
487,
493,
499,
508,
514,
519,
523,
526,
529,
533,
538,
545,
556,
569,
585,
589,
592,
594,
597,
610,
620,
630,
653,
672,
691,
706,
718,
736,
748,
753,
755,
758,
753,
743,
742,
749,
759,
769,
780,
790,
780,
768,
753,
736,
720,
705,
690,
689,
695,
704,
714,
721,
707,
686,
664,
643,
618,
599,
582,
563,
545,
526,
510,
497,
484,
472,
461,
448,
434,
431,
424,
413,
404,
394,
382,
371,
358,
349,
346,
355,
375,
382,
385,
397,
408,
400,
389,
387,
389,
394,
399,
403,
416,
427,
440,
450,
452,
438,
432,
440,
458,
467,
474,
482,
494,
508,
533,
552,
570,
586,
586,
592,
587,
581,
570,
549,
528,
523,
532,
542,
537,
518,
522,
536,
524,
510,
519,
529,
531,
516,
490,
471,
454,
442,
439,
438,
436,
432,
427,
416,
400,
391,
384,
381,
382,
388,
400,
410,
415,
425,
439,
448,
450,
462,
466,
464,
460,
460,
462,
469,
474,
468,
464,
458,
453,
437,
415,
412,
418,
431,
443,
448,
447,
446,
450,
449,
439,
442,
453,
454,
449,
440,
436,
451,
456,
452,
440,
457,
469,
472,
467,
459,
453,
440,
424,
414,
405,
394,
389,
391,
392,
394,
396,
397,
396,
395,
394,
391,
382,
377,
375,
378,
381,
386,
389,
399,
408,
421,
434,
445,
452,
447,
432,
434,
444,
454,
464,
466,
462,
472,
475,
480,
489,
492,
499,
508,
512,
509,
500,
499,
504,
512,
519,
526,
528,
523,
524,
529,
531,
532,
526,
523,
521,
522,
520,
516,
513,
508,
500,
493,
489,
478,
470,
462,
465,
470,
476,
473,
459,
447,
438,
427,
414,
416,
422,
432,
440,
451,
454,
449,
441,
432,
428,
423,
416,
406,
397,
387,
378,
368,
360,
351,
344,
339,
338,
338,
343,
350,
357,
366,
375,
382,
386,
389,
398,
405,
416,
427,
446,
461,
464,
459,
452,
444,
437,
443,
445,
438,
425,
408,
391,
379,
372,
367,
361,
353,
347,
342,
342,
344,
344,
343,
340,
341,
344,
347,
350,
352,
353,
355,
356,
356,
357,
358,
359,
359,
357,
358,
362,
367,
371,
372,
372,
374,
434,
437,
439,
446,
449,
453,
461,
461,
468,
472,
476,
481,
487,
496,
509,
516,
522,
526,
529,
531,
535,
540,
546,
553,
563,
581,
590,
594,
602,
616,
628,
641,
650,
663,
682,
698,
717,
734,
749,
764,
761,
751,
746,
741,
733,
740,
753,
765,
777,
791,
800,
790,
775,
757,
741,
726,
711,
698,
703,
710,
720,
728,
724,
707,
683,
663,
642,
613,
590,
572,
548,
535,
521,
508,
495,
480,
468,
458,
450,
439,
427,
415,
406,
399,
389,
384,
375,
361,
351,
346,
352,
359,
362,
375,
394,
408,
416,
410,
404,
404,
406,
414,
417,
422,
430,
441,
450,
463,
459,
450,
443,
461,
477,
490,
503,
514,
524,
535,
556,
575,
572,
572,
578,
575,
579,
575,
558,
533,
522,
520,
525,
528,
512,
515,
526,
527,
515,
512,
518,
523,
517,
491,
475,
466,
455,
436,
436,
430,
424,
418,
410,
403,
399,
395,
391,
387,
388,
393,
404,
420,
432,
442,
454,
464,
475,
473,
462,
460,
459,
460,
463,
464,
468,
468,
464,
455,
440,
418,
417,
428,
438,
449,
455,
456,
458,
464,
465,
453,
453,
467,
472,
467,
459,
448,
463,
469,
466,
455,
461,
478,
487,
484,
479,
466,
449,
440,
429,
414,
402,
400,
403,
402,
402,
403,
403,
401,
400,
397,
393,
386,
379,
375,
377,
381,
388,
400,
409,
413,
428,
439,
446,
453,
451,
440,
441,
451,
463,
475,
478,
480,
488,
492,
498,
500,
504,
505,
512,
523,
523,
514,
504,
505,
510,
514,
521,
522,
521,
524,
528,
531,
531,
525,
524,
528,
530,
529,
524,
516,
506,
496,
485,
478,
470,
461,
452,
453,
458,
464,
468,
464,
452,
439,
428,
416,
411,
413,
418,
426,
442,
450,
440,
432,
422,
415,
411,
405,
399,
391,
382,
373,
364,
354,
348,
343,
338,
338,
338,
346,
356,
363,
369,
380,
392,
399,
400,
409,
416,
422,
437,
452,
454,
452,
449,
442,
433,
427,
441,
440,
427,
412,
399,
384,
370,
362,
356,
352,
346,
341,
339,
341,
345,
345,
343,
340,
338,
345,
350,
353,
355,
355,
357,
357,
357,
357,
360,
361,
362,
360,
363,
372,
376,
381,
378,
374,
374,
442,
442,
444,
448,
451,
459,
466,
472,
477,
479,
477,
477,
486,
490,
503,
516,
523,
527,
528,
533,
537,
543,
547,
553,
560,
577,
593,
597,
616,
633,
651,
664,
668,
677,
692,
705,
725,
744,
759,
773,
760,
746,
733,
725,
723,
736,
749,
765,
779,
793,
801,
794,
776,
759,
744,
729,
713,
708,
715,
723,
731,
730,
720,
702,
676,
650,
628,
610,
592,
586,
570,
554,
537,
517,
501,
483,
470,
460,
448,
435,
423,
414,
407,
400,
394,
386,
378,
364,
354,
347,
349,
355,
367,
381,
391,
398,
410,
416,
422,
426,
428,
428,
434,
436,
438,
443,
453,
465,
476,
472,
468,
464,
482,
502,
518,
531,
539,
550,
570,
572,
559,
563,
566,
561,
573,
569,
558,
542,
533,
523,
520,
515,
511,
513,
521,
526,
526,
514,
508,
511,
510,
501,
490,
476,
465,
454,
443,
434,
427,
421,
415,
412,
412,
409,
404,
396,
391,
406,
417,
424,
435,
449,
461,
471,
479,
475,
464,
461,
458,
454,
456,
458,
461,
464,
464,
456,
443,
420,
423,
429,
439,
453,
464,
465,
471,
480,
480,
469,
463,
478,
491,
485,
473,
460,
473,
484,
481,
472,
467,
481,
498,
502,
496,
479,
468,
455,
438,
424,
415,
410,
412,
412,
411,
410,
408,
406,
404,
400,
394,
388,
379,
376,
377,
383,
394,
407,
419,
428,
432,
442,
452,
457,
454,
446,
446,
453,
463,
467,
474,
485,
491,
499,
506,
513,
515,
515,
517,
528,
528,
523,
512,
509,
508,
510,
512,
517,
520,
523,
526,
530,
532,
534,
536,
538,
539,
537,
528,
517,
507,
495,
486,
478,
468,
458,
448,
445,
447,
452,
458,
462,
455,
445,
434,
420,
404,
405,
412,
422,
433,
439,
430,
424,
415,
406,
400,
395,
391,
385,
376,
366,
357,
350,
344,
340,
339,
338,
341,
351,
363,
371,
379,
386,
396,
407,
413,
416,
428,
434,
445,
454,
445,
440,
435,
427,
422,
432,
435,
425,
413,
398,
385,
375,
365,
354,
347,
342,
340,
338,
339,
340,
342,
344,
342,
340,
341,
349,
358,
364,
360,
363,
365,
365,
361,
365,
366,
374,
370,
366,
365,
381,
390,
395,
391,
381,
380,
449,
454,
457,
461,
459,
470,
478,
486,
491,
492,
493,
490,
486,
487,
494,
500,
513,
524,
529,
534,
540,
545,
551,
557,
564,
579,
598,
609,
639,
660,
676,
685,
688,
696,
707,
720,
734,
751,
769,
771,
753,
740,
726,
714,
715,
728,
742,
757,
767,
778,
790,
781,
768,
756,
743,
728,
719,
720,
726,
727,
721,
712,
704,
697,
683,
660,
639,
624,
612,
601,
581,
563,
541,
519,
503,
488,
476,
466,
453,
438,
428,
419,
411,
404,
394,
386,
378,
366,
356,
350,
350,
354,
360,
368,
376,
387,
395,
402,
416,
424,
426,
428,
435,
441,
447,
458,
468,
477,
486,
490,
486,
483,
485,
509,
526,
543,
559,
570,
575,
559,
547,
559,
556,
550,
563,
556,
546,
548,
546,
535,
530,
524,
516,
511,
512,
520,
527,
526,
515,
508,
502,
512,
510,
491,
478,
465,
456,
443,
438,
434,
419,
426,
432,
431,
417,
405,
397,
402,
417,
434,
443,
452,
463,
472,
479,
478,
472,
465,
458,
452,
450,
450,
450,
452,
451,
446,
437,
426,
430,
437,
447,
461,
469,
474,
482,
496,
498,
482,
478,
492,
506,
499,
484,
472,
483,
497,
496,
486,
481,
493,
507,
515,
510,
500,
482,
464,
451,
439,
425,
423,
422,
423,
421,
417,
414,
411,
405,
400,
394,
386,
379,
375,
377,
385,
398,
411,
423,
434,
442,
448,
453,
456,
450,
441,
439,
445,
449,
454,
464,
475,
486,
495,
501,
510,
520,
526,
523,
533,
538,
529,
520,
514,
511,
511,
514,
519,
522,
523,
528,
532,
535,
537,
539,
539,
538,
534,
526,
516,
506,
496,
485,
474,
464,
455,
447,
440,
438,
441,
446,
451,
454,
447,
436,
425,
407,
400,
404,
416,
427,
428,
422,
416,
409,
402,
393,
386,
381,
377,
371,
363,
354,
347,
342,
339,
336,
343,
353,
360,
366,
378,
390,
397,
400,
413,
424,
426,
431,
442,
454,
442,
424,
422,
418,
414,
426,
430,
422,
411,
398,
385,
374,
364,
357,
350,
341,
338,
337,
336,
336,
338,
341,
342,
341,
340,
345,
355,
369,
376,
372,
372,
378,
378,
374,
378,
380,
388,
378,
375,
373,
380,
397,
405,
406,
398,
395,
458,
464,
474,
477,
468,
478,
489,
496,
500,
503,
507,
506,
505,
502,
498,
503,
509,
519,
528,
536,
543,
548,
555,
560,
571,
588,
607,
636,
663,
685,
695,
702,
706,
714,
726,
736,
748,
763,
773,
764,
747,
733,
719,
706,
706,
718,
732,
743,
755,
762,
769,
768,
757,
745,
734,
727,
724,
732,
734,
722,
707,
693,
687,
684,
678,
667,
653,
637,
619,
597,
579,
562,
544,
524,
507,
492,
479,
467,
457,
445,
433,
423,
412,
404,
394,
385,
376,
367,
357,
353,
355,
357,
358,
364,
370,
369,
381,
392,
399,
406,
406,
409,
415,
434,
447,
459,
476,
487,
495,
501,
504,
506,
506,
514,
532,
551,
572,
584,
569,
549,
535,
549,
550,
539,
549,
549,
533,
542,
550,
546,
542,
536,
529,
519,
509,
512,
520,
523,
521,
513,
500,
503,
514,
509,
493,
486,
471,
458,
455,
447,
432,
443,
448,
447,
430,
414,
403,
403,
416,
431,
443,
453,
459,
468,
476,
476,
473,
467,
461,
451,
451,
452,
452,
452,
453,
454,
453,
438,
431,
442,
454,
467,
476,
481,
489,
506,
513,
499,
488,
504,
515,
507,
489,
480,
494,
508,
511,
501,
494,
508,
522,
528,
526,
514,
496,
476,
459,
447,
438,
435,
432,
429,
429,
426,
419,
412,
406,
399,
393,
383,
379,
376,
379,
385,
397,
410,
422,
434,
444,
454,
457,
450,
437,
432,
437,
444,
452,
466,
478,
489,
495,
506,
515,
522,
525,
529,
536,
539,
544,
535,
524,
520,
516,
514,
518,
521,
524,
527,
532,
536,
537,
539,
536,
532,
525,
522,
519,
515,
509,
502,
493,
483,
473,
464,
453,
442,
434,
428,
435,
439,
441,
441,
437,
428,
415,
399,
397,
408,
416,
418,
415,
409,
404,
396,
389,
383,
375,
369,
364,
357,
351,
344,
340,
339,
338,
348,
364,
372,
378,
386,
397,
406,
409,
412,
425,
430,
437,
448,
445,
425,
410,
404,
414,
425,
431,
427,
414,
400,
386,
373,
364,
355,
346,
341,
339,
336,
335,
334,
334,
335,
337,
339,
338,
340,
349,
361,
376,
386,
388,
387,
390,
392,
390,
392,
395,
398,
384,
382,
386,
385,
396,
411,
416,
413,
411,
469,
477,
489,
496,
480,
476,
490,
498,
501,
508,
511,
519,
522,
526,
517,
523,
518,
521,
525,
538,
549,
556,
560,
571,
586,
604,
619,
657,
683,
701,
712,
720,
726,
734,
744,
755,
765,
772,
766,
754,
739,
723,
709,
694,
693,
706,
717,
727,
734,
746,
758,
754,
742,
729,
721,
715,
722,
724,
723,
710,
693,
673,
664,
663,
661,
654,
643,
624,
608,
589,
574,
556,
539,
523,
506,
493,
481,
470,
460,
451,
441,
429,
417,
406,
396,
386,
378,
367,
358,
356,
362,
364,
363,
367,
371,
373,
374,
375,
380,
387,
397,
413,
429,
439,
443,
449,
462,
476,
489,
510,
523,
526,
526,
527,
536,
551,
574,
587,
568,
548,
533,
540,
538,
534,
546,
541,
523,
534,
542,
547,
551,
545,
539,
531,
519,
509,
507,
508,
516,
513,
503,
498,
505,
513,
507,
501,
489,
474,
469,
458,
450,
459,
466,
457,
436,
419,
411,
409,
417,
422,
433,
441,
453,
461,
467,
468,
467,
464,
460,
453,
458,
462,
463,
466,
470,
471,
460,
441,
442,
449,
460,
469,
478,
485,
498,
515,
525,
511,
499,
504,
521,
517,
499,
487,
501,
517,
527,
516,
510,
520,
536,
544,
534,
518,
499,
480,
464,
453,
446,
441,
439,
435,
431,
427,
418,
411,
404,
398,
391,
384,
380,
378,
383,
390,
401,
416,
429,
439,
449,
454,
449,
439,
425,
432,
441,
453,
461,
465,
477,
490,
501,
513,
524,
527,
532,
537,
543,
549,
551,
542,
533,
527,
522,
520,
522,
525,
528,
533,
537,
539,
540,
534,
526,
517,
512,
508,
511,
506,
501,
502,
500,
490,
479,
470,
459,
446,
435,
424,
424,
427,
430,
429,
427,
425,
416,
402,
389,
396,
404,
409,
406,
402,
396,
389,
384,
378,
372,
364,
355,
348,
342,
340,
340,
336,
339,
353,
371,
382,
390,
399,
404,
411,
418,
422,
425,
433,
439,
443,
432,
414,
398,
405,
421,
429,
427,
417,
404,
389,
376,
364,
353,
346,
340,
338,
336,
335,
333,
332,
332,
333,
335,
337,
336,
341,
350,
361,
374,
387,
394,
402,
403,
404,
407,
408,
410,
405,
389,
394,
402,
395,
401,
412,
423,
424,
424,
473,
485,
499,
513,
501,
486,
500,
511,
516,
522,
525,
534,
542,
551,
541,
545,
550,
547,
548,
549,
554,
564,
572,
583,
601,
619,
639,
672,
695,
716,
730,
741,
747,
754,
762,
770,
774,
767,
756,
742,
728,
714,
700,
684,
681,
690,
700,
708,
719,
736,
748,
742,
730,
717,
703,
701,
706,
706,
707,
698,
684,
665,
647,
637,
638,
635,
621,
609,
594,
582,
569,
555,
538,
522,
504,
492,
483,
474,
463,
454,
443,
432,
419,
408,
397,
387,
379,
365,
359,
360,
369,
373,
371,
375,
381,
378,
376,
376,
381,
386,
396,
410,
421,
427,
432,
442,
459,
477,
494,
510,
529,
540,
546,
548,
552,
565,
581,
588,
572,
557,
538,
530,
527,
528,
539,
537,
520,
525,
529,
541,
550,
545,
543,
540,
534,
520,
503,
496,
505,
508,
502,
496,
497,
502,
508,
509,
501,
493,
486,
471,
470,
474,
479,
467,
447,
429,
418,
416,
428,
425,
435,
438,
447,
455,
464,
462,
460,
460,
460,
459,
466,
473,
480,
487,
491,
485,
467,
445,
450,
449,
460,
474,
480,
488,
500,
518,
536,
524,
515,
513,
526,
526,
509,
498,
508,
521,
537,
533,
527,
533,
551,
556,
535,
517,
495,
475,
463,
455,
449,
444,
441,
436,
431,
425,
416,
409,
402,
395,
389,
383,
380,
382,
390,
403,
412,
420,
432,
440,
449,
445,
438,
425,
426,
439,
448,
461,
475,
485,
486,
486,
496,
503,
515,
524,
528,
536,
548,
560,
562,
553,
548,
538,
529,
524,
525,
530,
534,
537,
539,
539,
534,
523,
515,
507,
501,
496,
500,
496,
488,
490,
494,
488,
480,
471,
462,
451,
439,
428,
414,
414,
412,
408,
408,
407,
406,
400,
387,
386,
392,
398,
395,
391,
387,
383,
378,
372,
367,
359,
350,
341,
337,
338,
339,
338,
334,
348,
366,
378,
390,
403,
412,
418,
426,
432,
437,
441,
441,
438,
419,
400,
388,
400,
414,
419,
415,
402,
390,
378,
365,
354,
343,
337,
334,
335,
335,
334,
333,
331,
332,
333,
334,
336,
340,
343,
351,
358,
366,
378,
389,
399,
406,
405,
413,
417,
419,
409,
400,
406,
414,
411,
410,
413,
424,
432,
432,
470,
483,
501,
522,
517,
501,
497,
522,
532,
538,
545,
553,
561,
566,
558,
554,
566,
563,
567,
566,
563,
576,
588,
598,
615,
630,
656,
677,
697,
720,
744,
757,
764,
772,
779,
780,
768,
755,
742,
729,
716,
705,
690,
674,
662,
675,
684,
696,
714,
728,
741,
732,
719,
704,
690,
684,
686,
684,
683,
684,
671,
653,
633,
612,
614,
610,
602,
592,
582,
574,
563,
549,
535,
520,
504,
493,
485,
476,
464,
452,
444,
435,
423,
410,
400,
389,
376,
364,
359,
364,
380,
386,
379,
375,
378,
385,
386,
385,
387,
394,
402,
406,
413,
420,
429,
445,
462,
479,
491,
506,
524,
547,
561,
563,
570,
580,
588,
578,
571,
564,
550,
531,
520,
525,
533,
531,
520,
524,
534,
549,
552,
536,
536,
547,
544,
524,
509,
498,
499,
499,
497,
492,
491,
496,
499,
500,
500,
504,
497,
491,
490,
491,
488,
477,
458,
439,
427,
425,
434,
439,
441,
451,
452,
456,
464,
469,
464,
460,
460,
463,
472,
482,
492,
500,
504,
493,
474,
451,
460,
458,
460,
465,
478,
492,
506,
520,
535,
542,
536,
525,
534,
533,
516,
513,
524,
530,
540,
548,
547,
552,
562,
555,
532,
512,
491,
474,
463,
455,
448,
444,
439,
434,
428,
421,
414,
406,
399,
392,
387,
383,
381,
386,
399,
414,
422,
430,
438,
441,
436,
428,
424,
420,
429,
440,
452,
466,
482,
490,
494,
497,
496,
499,
508,
512,
521,
534,
544,
558,
564,
559,
553,
543,
536,
529,
530,
537,
538,
536,
534,
532,
525,
516,
506,
502,
492,
486,
488,
488,
478,
476,
484,
486,
476,
469,
463,
454,
443,
433,
421,
412,
406,
400,
395,
388,
388,
390,
386,
378,
382,
385,
383,
380,
378,
375,
372,
367,
360,
352,
344,
340,
343,
350,
352,
349,
339,
331,
342,
359,
375,
386,
401,
421,
432,
439,
442,
436,
428,
422,
404,
390,
380,
394,
406,
406,
398,
387,
376,
364,
354,
343,
337,
336,
336,
335,
335,
334,
333,
332,
332,
333,
335,
337,
340,
347,
355,
364,
369,
375,
376,
382,
387,
394,
402,
413,
414,
411,
410,
416,
419,
420,
420,
422,
427,
429,
429,
472,
486,
506,
526,
532,
516,
508,
514,
527,
536,
552,
563,
569,
577,
575,
563,
575,
576,
584,
584,
581,
591,
605,
617,
628,
645,
666,
683,
703,
724,
746,
770,
782,
780,
774,
766,
751,
741,
727,
716,
704,
692,
679,
664,
651,
658,
669,
691,
706,
721,
734,
732,
720,
704,
683,
665,
664,
658,
660,
667,
662,
646,
629,
605,
590,
590,
582,
574,
568,
563,
553,
538,
525,
514,
502,
493,
484,
474,
460,
449,
439,
432,
422,
410,
400,
388,
374,
363,
360,
366,
378,
394,
397,
391,
392,
392,
391,
403,
392,
396,
413,
427,
432,
433,
433,
438,
456,
470,
481,
502,
516,
539,
549,
559,
571,
582,
580,
563,
561,
565,
557,
538,
526,
522,
523,
524,
522,
536,
545,
556,
555,
534,
529,
544,
543,
524,
511,
499,
497,
493,
491,
488,
486,
489,
492,
494,
497,
501,
510,
508,
508,
504,
491,
474,
460,
443,
434,
427,
438,
443,
444,
449,
454,
463,
471,
476,
462,
461,
459,
463,
475,
487,
499,
513,
513,
499,
479,
462,
466,
472,
472,
473,
474,
486,
500,
514,
531,
550,
549,
542,
544,
539,
524,
530,
538,
546,
556,
565,
567,
574,
572,
550,
531,
510,
492,
475,
464,
456,
449,
444,
438,
432,
425,
417,
410,
402,
396,
390,
385,
382,
382,
389,
404,
417,
429,
438,
434,
428,
425,
415,
414,
421,
425,
436,
448,
461,
474,
481,
489,
500,
506,
505,
507,
511,
517,
526,
536,
550,
556,
552,
545,
537,
532,
526,
531,
535,
531,
524,
525,
526,
519,
510,
498,
492,
490,
482,
476,
479,
473,
465,
471,
477,
478,
469,
461,
452,
442,
435,
429,
424,
417,
411,
403,
395,
388,
382,
377,
374,
374,
374,
371,
369,
369,
368,
364,
361,
355,
351,
346,
349,
356,
365,
369,
360,
347,
331,
331,
335,
352,
370,
395,
417,
431,
440,
437,
420,
411,
407,
397,
381,
375,
391,
395,
389,
380,
370,
360,
351,
342,
339,
338,
338,
336,
337,
336,
334,
331,
331,
333,
334,
337,
338,
344,
350,
358,
370,
380,
383,
385,
389,
394,
399,
402,
412,
416,
415,
417,
416,
417,
422,
423,
426,
427,
427,
424,
476,
487,
508,
531,
543,
537,
529,
515,
528,
544,
556,
564,
570,
576,
576,
576,
589,
594,
594,
599,
595,
607,
623,
636,
644,
664,
682,
698,
716,
740,
766,
776,
765,
760,
755,
747,
735,
724,
714,
703,
691,
680,
666,
653,
638,
647,
664,
680,
698,
713,
724,
712,
705,
696,
679,
654,
641,
638,
639,
647,
650,
639,
622,
602,
576,
563,
561,
557,
551,
548,
541,
530,
518,
507,
493,
486,
480,
471,
457,
445,
436,
430,
420,
409,
397,
385,
373,
364,
365,
372,
377,
394,
407,
412,
412,
414,
412,
414,
415,
404,
410,
428,
444,
452,
452,
452,
457,
463,
476,
491,
512,
527,
536,
543,
557,
574,
575,
561,
551,
556,
552,
548,
541,
536,
528,
521,
524,
538,
548,
558,
556,
538,
522,
534,
539,
524,
514,
499,
495,
494,
490,
490,
487,
485,
487,
488,
493,
502,
513,
516,
513,
501,
485,
466,
449,
447,
443,
447,
449,
453,
456,
455,
458,
463,
470,
465,
458,
461,
462,
462,
476,
492,
505,
519,
521,
504,
486,
470,
473,
485,
490,
491,
492,
495,
504,
513,
527,
542,
558,
557,
557,
552,
538,
545,
554,
562,
568,
564,
568,
569,
562,
548,
530,
509,
491,
480,
469,
460,
452,
446,
440,
432,
425,
416,
406,
399,
391,
387,
384,
382,
384,
391,
405,
421,
432,
439,
428,
420,
416,
409,
416,
425,
433,
442,
453,
458,
464,
477,
486,
497,
508,
515,
519,
523,
523,
523,
532,
542,
546,
545,
536,
526,
522,
523,
526,
522,
519,
515,
516,
518,
514,
502,
494,
486,
481,
478,
472,
470,
467,
460,
461,
467,
468,
468,
464,
455,
450,
444,
438,
434,
428,
422,
412,
404,
395,
385,
377,
374,
370,
367,
364,
362,
362,
360,
354,
352,
351,
351,
356,
361,
369,
380,
382,
376,
364,
353,
339,
331,
334,
358,
381,
405,
427,
433,
432,
418,
402,
393,
391,
374,
365,
380,
380,
372,
362,
353,
346,
341,
339,
336,
338,
339,
346,
341,
339,
336,
334,
331,
334,
336,
339,
346,
350,
357,
366,
374,
382,
391,
396,
401,
405,
414,
418,
416,
415,
418,
418,
416,
414,
415,
420,
421,
420,
418,
414,
488,
489,
506,
523,
539,
545,
546,
539,
534,
552,
565,
576,
581,
582,
583,
588,
596,
613,
607,
616,
612,
621,
636,
646,
667,
680,
702,
713,
730,
759,
766,
753,
748,
744,
737,
728,
718,
708,
698,
687,
676,
665,
654,
639,
623,
639,
659,
677,
695,
710,
710,
693,
685,
683,
670,
650,
628,
612,
621,
628,
634,
628,
612,
597,
574,
552,
542,
538,
533,
529,
526,
522,
514,
504,
489,
475,
471,
466,
457,
447,
437,
427,
418,
406,
395,
383,
372,
367,
374,
382,
382,
390,
408,
424,
428,
430,
431,
428,
428,
420,
413,
426,
441,
455,
468,
478,
480,
486,
495,
504,
508,
517,
527,
541,
555,
564,
570,
566,
549,
547,
559,
561,
559,
552,
543,
522,
516,
525,
545,
561,
551,
539,
524,
515,
528,
526,
517,
506,
495,
495,
497,
502,
496,
488,
483,
483,
488,
500,
510,
511,
500,
493,
481,
461,
447,
453,
456,
463,
462,
466,
468,
466,
458,
464,
463,
459,
461,
462,
464,
463,
477,
492,
505,
518,
526,
510,
496,
482,
473,
488,
501,
506,
513,
513,
520,
533,
543,
547,
558,
571,
574,
568,
556,
559,
570,
576,
572,
559,
559,
557,
553,
545,
531,
516,
503,
489,
477,
467,
457,
449,
440,
431,
422,
414,
406,
399,
392,
387,
384,
383,
388,
398,
414,
427,
432,
427,
418,
413,
405,
412,
419,
428,
436,
447,
459,
469,
473,
475,
484,
494,
502,
513,
525,
533,
532,
530,
533,
537,
538,
537,
529,
521,
516,
518,
516,
511,
509,
504,
505,
514,
514,
508,
497,
486,
476,
471,
466,
461,
460,
454,
451,
456,
456,
460,
460,
455,
453,
454,
452,
446,
440,
431,
420,
409,
399,
392,
386,
380,
376,
371,
368,
367,
365,
365,
360,
358,
357,
364,
370,
375,
382,
389,
394,
388,
382,
369,
353,
331,
337,
344,
372,
401,
418,
423,
424,
417,
402,
387,
380,
370,
357,
367,
362,
354,
346,
341,
340,
338,
337,
335,
337,
345,
357,
350,
341,
335,
331,
331,
335,
338,
342,
354,
357,
364,
372,
381,
389,
394,
402,
406,
410,
414,
413,
410,
410,
415,
412,
410,
410,
412,
416,
414,
410,
406,
403,
488,
492,
497,
507,
519,
528,
552,
562,
553,
557,
570,
582,
588,
588,
591,
588,
610,
620,
626,
632,
628,
631,
647,
666,
684,
699,
713,
724,
745,
752,
741,
736,
732,
726,
721,
713,
702,
692,
681,
670,
660,
648,
638,
623,
615,
636,
654,
671,
690,
703,
696,
674,
661,
662,
660,
646,
624,
600,
601,
609,
612,
610,
601,
583,
570,
554,
535,
522,
516,
514,
511,
513,
508,
501,
485,
471,
461,
460,
455,
451,
440,
429,
416,
404,
393,
381,
372,
374,
385,
398,
385,
390,
411,
424,
432,
440,
446,
446,
440,
434,
430,
427,
437,
450,
462,
478,
492,
504,
513,
525,
526,
529,
541,
553,
555,
560,
565,
564,
555,
540,
551,
563,
566,
558,
544,
517,
511,
530,
546,
551,
539,
528,
523,
508,
514,
520,
520,
512,
499,
497,
500,
509,
512,
504,
490,
482,
487,
494,
499,
499,
486,
480,
469,
456,
457,
462,
472,
478,
479,
479,
477,
468,
462,
459,
462,
462,
462,
462,
466,
464,
475,
485,
496,
510,
524,
521,
511,
498,
479,
483,
487,
494,
500,
511,
526,
543,
558,
566,
572,
579,
588,
585,
576,
574,
584,
584,
566,
551,
546,
547,
545,
540,
533,
521,
509,
497,
484,
472,
461,
450,
439,
427,
420,
411,
404,
397,
391,
387,
384,
385,
393,
404,
416,
423,
417,
409,
404,
400,
402,
414,
425,
431,
441,
451,
461,
474,
480,
484,
490,
503,
514,
519,
527,
537,
535,
536,
537,
540,
533,
525,
521,
513,
510,
514,
512,
505,
499,
495,
498,
503,
506,
506,
498,
488,
480,
472,
462,
453,
450,
450,
444,
444,
444,
445,
450,
446,
445,
449,
450,
448,
440,
434,
423,
413,
407,
400,
392,
384,
377,
376,
376,
375,
376,
375,
374,
375,
373,
378,
386,
393,
397,
400,
408,
406,
397,
380,
363,
341,
331,
349,
376,
400,
412,
409,
410,
408,
400,
385,
368,
364,
352,
351,
346,
341,
340,
337,
338,
336,
337,
339,
345,
354,
363,
361,
348,
339,
331,
335,
336,
340,
345,
352,
356,
362,
370,
380,
389,
393,
396,
402,
403,
404,
401,
397,
401,
404,
401,
403,
404,
406,
405,
402,
398,
395,
395,
502,
511,
510,
509,
512,
524,
549,
565,
570,
570,
570,
575,
590,
598,
603,
601,
596,
610,
624,
635,
641,
646,
657,
676,
695,
709,
717,
740,
743,
734,
727,
721,
716,
711,
706,
697,
682,
672,
662,
651,
640,
631,
620,
608,
609,
631,
650,
667,
680,
692,
680,
662,
640,
634,
640,
637,
617,
595,
580,
590,
596,
592,
585,
575,
565,
553,
538,
521,
508,
500,
500,
502,
502,
492,
479,
464,
454,
452,
451,
448,
439,
427,
412,
398,
388,
378,
372,
378,
403,
414,
403,
392,
402,
411,
423,
435,
446,
454,
452,
450,
446,
445,
442,
442,
453,
471,
493,
512,
529,
537,
540,
541,
553,
562,
562,
556,
561,
561,
557,
536,
546,
562,
563,
552,
532,
512,
508,
526,
541,
542,
526,
511,
513,
512,
499,
510,
515,
515,
508,
501,
506,
512,
516,
514,
499,
488,
484,
486,
488,
486,
478,
468,
458,
455,
466,
473,
486,
490,
485,
477,
471,
464,
462,
461,
462,
464,
463,
463,
471,
475,
474,
481,
492,
505,
520,
528,
516,
499,
486,
496,
497,
501,
501,
511,
524,
538,
555,
573,
585,
589,
597,
599,
592,
589,
587,
575,
561,
546,
536,
537,
538,
536,
530,
522,
511,
501,
488,
475,
464,
452,
440,
430,
421,
412,
402,
394,
388,
386,
385,
386,
392,
402,
408,
408,
403,
398,
399,
402,
406,
409,
414,
423,
432,
444,
457,
470,
478,
488,
498,
507,
513,
514,
521,
528,
531,
534,
534,
536,
528,
520,
514,
510,
508,
510,
506,
500,
492,
486,
488,
490,
495,
502,
497,
485,
479,
472,
463,
452,
446,
440,
440,
435,
433,
433,
438,
435,
432,
437,
437,
440,
440,
435,
428,
422,
414,
406,
396,
388,
384,
385,
387,
387,
388,
386,
387,
388,
388,
394,
399,
408,
415,
417,
419,
416,
398,
376,
358,
340,
331,
352,
378,
400,
401,
389,
392,
395,
391,
380,
366,
351,
345,
340,
339,
337,
336,
336,
333,
335,
337,
341,
349,
358,
366,
365,
351,
340,
331,
336,
336,
338,
342,
346,
350,
355,
363,
371,
380,
381,
384,
388,
390,
390,
385,
383,
388,
388,
386,
391,
395,
395,
393,
389,
387,
387,
391,
525,
516,
530,
531,
534,
535,
545,
563,
584,
588,
580,
584,
587,
602,
612,
616,
617,
607,
622,
635,
652,
658,
673,
684,
702,
712,
721,
743,
734,
728,
718,
706,
697,
692,
689,
682,
668,
658,
648,
634,
621,
613,
607,
596,
612,
627,
640,
655,
670,
678,
665,
648,
627,
614,
619,
622,
609,
592,
570,
567,
574,
574,
573,
566,
556,
546,
533,
521,
507,
494,
487,
489,
491,
482,
471,
459,
448,
441,
443,
442,
431,
416,
402,
390,
381,
374,
376,
384,
398,
419,
421,
409,
403,
418,
429,
440,
444,
450,
456,
464,
468,
463,
458,
455,
455,
471,
492,
513,
531,
542,
552,
559,
564,
573,
562,
550,
560,
565,
553,
532,
540,
551,
555,
544,
521,
505,
504,
518,
532,
531,
517,
499,
500,
511,
503,
499,
503,
506,
505,
506,
509,
511,
516,
524,
512,
501,
489,
485,
481,
478,
473,
469,
477,
473,
481,
488,
492,
488,
476,
464,
460,
458,
460,
463,
463,
467,
468,
471,
478,
485,
487,
489,
502,
517,
533,
537,
516,
498,
497,
509,
512,
517,
516,
510,
521,
536,
552,
565,
574,
584,
598,
601,
601,
596,
582,
566,
551,
540,
530,
528,
528,
528,
525,
519,
511,
502,
490,
478,
465,
453,
442,
432,
425,
413,
404,
395,
388,
387,
386,
387,
389,
390,
395,
396,
397,
400,
402,
410,
415,
416,
420,
426,
427,
438,
452,
461,
472,
482,
492,
498,
503,
504,
513,
525,
526,
531,
535,
535,
527,
517,
510,
504,
503,
503,
501,
495,
486,
476,
475,
479,
486,
497,
499,
490,
484,
475,
466,
454,
445,
437,
433,
430,
424,
423,
426,
423,
422,
425,
426,
430,
435,
436,
432,
427,
418,
409,
400,
396,
392,
396,
400,
402,
400,
396,
395,
397,
402,
410,
415,
418,
418,
417,
415,
410,
392,
371,
351,
330,
332,
363,
384,
392,
386,
375,
374,
378,
373,
366,
355,
343,
339,
336,
335,
333,
330,
331,
331,
332,
337,
341,
348,
355,
362,
358,
347,
338,
331,
336,
337,
338,
340,
342,
344,
348,
354,
361,
366,
368,
370,
373,
375,
375,
373,
372,
373,
370,
374,
381,
383,
382,
381,
388,
393,
397,
400,
536,
525,
538,
550,
550,
552,
541,
559,
572,
586,
590,
598,
600,
604,
614,
626,
630,
626,
626,
642,
656,
671,
685,
698,
711,
730,
734,
727,
731,
723,
710,
693,
679,
677,
672,
664,
656,
646,
642,
632,
613,
596,
590,
592,
607,
618,
630,
645,
661,
663,
647,
634,
613,
596,
596,
600,
598,
586,
568,
548,
550,
556,
558,
556,
546,
538,
525,
514,
503,
492,
480,
474,
477,
471,
466,
456,
440,
432,
434,
434,
424,
406,
392,
381,
375,
374,
381,
389,
397,
412,
427,
427,
411,
412,
423,
432,
438,
447,
457,
469,
481,
479,
473,
474,
475,
482,
491,
509,
522,
536,
554,
566,
573,
572,
561,
546,
559,
564,
548,
520,
527,
536,
545,
536,
515,
497,
496,
507,
513,
513,
506,
489,
490,
504,
503,
497,
498,
499,
500,
510,
503,
505,
509,
516,
514,
506,
495,
486,
482,
476,
479,
486,
496,
499,
497,
501,
495,
479,
466,
460,
463,
462,
462,
464,
468,
473,
476,
475,
486,
496,
501,
502,
516,
529,
541,
536,
512,
498,
506,
520,
529,
533,
534,
524,
520,
527,
537,
551,
564,
576,
584,
589,
596,
594,
578,
563,
547,
530,
521,
516,
516,
518,
517,
514,
509,
501,
491,
478,
465,
452,
441,
432,
424,
412,
404,
395,
389,
387,
388,
391,
390,
387,
389,
392,
397,
402,
408,
417,
427,
431,
432,
438,
440,
437,
437,
452,
462,
475,
484,
486,
487,
495,
503,
512,
522,
530,
535,
529,
522,
514,
505,
499,
498,
500,
496,
491,
481,
472,
465,
471,
483,
491,
490,
493,
487,
478,
468,
459,
447,
436,
426,
428,
424,
414,
413,
412,
414,
418,
420,
423,
428,
431,
433,
429,
423,
414,
409,
404,
403,
407,
415,
415,
412,
404,
402,
401,
406,
415,
416,
412,
404,
401,
400,
395,
378,
363,
343,
326,
340,
370,
382,
379,
373,
365,
360,
360,
354,
349,
343,
334,
331,
333,
332,
330,
328,
329,
330,
333,
336,
339,
344,
353,
355,
348,
339,
334,
331,
338,
339,
340,
341,
342,
343,
345,
348,
352,
357,
358,
359,
362,
363,
363,
362,
362,
365,
369,
373,
377,
380,
377,
388,
397,
404,
410,
413,
542,
546,
552,
562,
565,
564,
551,
551,
571,
587,
600,
607,
617,
619,
619,
626,
640,
642,
636,
647,
660,
673,
691,
706,
721,
733,
735,
728,
729,
718,
704,
687,
664,
654,
650,
647,
644,
639,
636,
627,
612,
592,
580,
591,
602,
608,
621,
639,
654,
645,
632,
618,
604,
588,
572,
577,
577,
573,
562,
543,
533,
538,
549,
545,
538,
530,
519,
507,
496,
485,
477,
463,
461,
460,
459,
451,
441,
426,
426,
420,
408,
394,
385,
380,
377,
377,
394,
406,
415,
411,
422,
432,
427,
416,
420,
429,
438,
448,
464,
475,
490,
491,
487,
490,
494,
500,
507,
517,
531,
545,
562,
574,
568,
564,
558,
539,
549,
558,
542,
512,
511,
521,
532,
524,
509,
488,
484,
494,
498,
497,
494,
484,
490,
503,
501,
498,
496,
500,
502,
511,
499,
499,
500,
502,
503,
499,
491,
485,
480,
482,
485,
493,
501,
505,
508,
505,
490,
472,
462,
464,
466,
466,
470,
476,
479,
484,
486,
485,
492,
506,
514,
519,
530,
540,
551,
534,
512,
500,
510,
525,
540,
549,
550,
541,
537,
540,
550,
559,
560,
567,
571,
579,
586,
589,
576,
561,
545,
529,
515,
509,
507,
509,
510,
509,
504,
499,
488,
476,
464,
451,
440,
432,
424,
413,
403,
394,
389,
387,
388,
398,
395,
389,
394,
397,
402,
409,
414,
419,
426,
440,
445,
449,
454,
452,
441,
444,
458,
466,
466,
473,
478,
488,
499,
513,
525,
535,
536,
526,
521,
513,
504,
497,
494,
492,
491,
484,
473,
466,
461,
464,
475,
480,
487,
491,
488,
476,
465,
454,
442,
431,
425,
419,
416,
414,
405,
404,
407,
411,
412,
415,
421,
426,
430,
433,
429,
424,
418,
413,
415,
415,
421,
417,
412,
403,
396,
393,
397,
401,
403,
399,
390,
383,
384,
381,
369,
352,
332,
325,
341,
362,
369,
368,
361,
355,
348,
343,
341,
336,
329,
327,
329,
329,
329,
328,
328,
329,
330,
332,
332,
336,
341,
345,
341,
336,
337,
331,
338,
339,
343,
345,
346,
346,
348,
349,
349,
349,
350,
350,
350,
354,
354,
355,
356,
359,
365,
371,
375,
381,
381,
391,
396,
405,
412,
418,
423,
547,
558,
568,
576,
582,
577,
566,
560,
576,
583,
602,
618,
627,
635,
634,
632,
648,
659,
650,
653,
664,
679,
697,
712,
726,
736,
733,
731,
719,
708,
694,
679,
660,
644,
629,
630,
629,
628,
625,
618,
605,
588,
571,
586,
599,
606,
620,
637,
642,
630,
619,
605,
593,
578,
563,
556,
561,
558,
552,
538,
522,
527,
530,
532,
527,
521,
510,
498,
486,
476,
468,
459,
454,
447,
446,
441,
434,
424,
412,
406,
398,
391,
379,
392,
386,
383,
390,
407,
425,
427,
419,
427,
439,
434,
425,
432,
437,
447,
463,
481,
500,
501,
500,
505,
511,
515,
527,
538,
548,
557,
571,
574,
559,
554,
550,
531,
537,
550,
538,
508,
495,
506,
517,
519,
508,
483,
477,
481,
484,
484,
482,
485,
496,
504,
500,
495,
496,
500,
498,
508,
500,
497,
496,
494,
494,
489,
487,
484,
484,
487,
490,
495,
498,
508,
514,
499,
484,
469,
463,
467,
474,
477,
480,
485,
491,
496,
499,
499,
498,
513,
525,
535,
544,
555,
557,
537,
517,
511,
518,
524,
542,
551,
557,
558,
554,
553,
562,
571,
574,
571,
569,
576,
585,
587,
574,
557,
540,
526,
517,
508,
499,
498,
504,
504,
500,
493,
484,
471,
459,
448,
437,
429,
422,
410,
400,
391,
389,
388,
390,
404,
403,
394,
398,
399,
406,
417,
426,
430,
434,
442,
454,
463,
466,
462,
451,
447,
450,
459,
463,
473,
482,
492,
502,
514,
524,
534,
535,
530,
525,
517,
508,
497,
488,
485,
481,
473,
465,
458,
451,
459,
465,
471,
477,
486,
485,
477,
467,
455,
440,
426,
420,
415,
413,
411,
406,
400,
400,
403,
404,
407,
413,
420,
428,
428,
429,
429,
427,
423,
421,
418,
418,
415,
406,
398,
390,
386,
386,
386,
388,
386,
378,
369,
369,
368,
357,
342,
328,
325,
336,
348,
356,
354,
348,
344,
338,
332,
329,
327,
326,
325,
327,
328,
327,
328,
328,
328,
329,
330,
329,
332,
336,
334,
329,
335,
331,
338,
340,
343,
350,
355,
351,
353,
358,
358,
356,
353,
352,
356,
353,
350,
350,
351,
354,
361,
368,
376,
379,
378,
379,
382,
390,
396,
403,
413,
422,
556,
565,
578,
592,
598,
593,
583,
573,
573,
594,
608,
622,
639,
651,
653,
645,
656,
670,
662,
662,
673,
686,
701,
717,
732,
740,
732,
721,
708,
695,
681,
667,
653,
636,
612,
613,
614,
614,
611,
605,
596,
580,
563,
578,
589,
599,
613,
631,
631,
620,
606,
593,
578,
566,
554,
543,
538,
540,
541,
533,
514,
512,
512,
519,
513,
509,
501,
488,
475,
463,
453,
447,
443,
435,
431,
426,
419,
413,
407,
403,
400,
396,
406,
408,
396,
389,
390,
411,
426,
431,
432,
430,
438,
444,
443,
442,
449,
457,
464,
480,
495,
510,
514,
518,
527,
536,
544,
554,
560,
569,
577,
569,
549,
541,
540,
518,
520,
538,
532,
510,
485,
493,
506,
514,
509,
489,
474,
473,
474,
476,
474,
480,
498,
502,
498,
487,
491,
492,
497,
506,
511,
499,
498,
502,
506,
505,
491,
484,
485,
488,
494,
501,
506,
512,
513,
494,
478,
464,
466,
469,
474,
482,
491,
497,
505,
510,
516,
515,
510,
519,
536,
547,
557,
569,
563,
543,
524,
526,
535,
535,
537,
550,
560,
570,
571,
567,
573,
578,
583,
578,
572,
574,
582,
578,
565,
553,
540,
528,
519,
510,
503,
497,
493,
497,
492,
487,
478,
467,
454,
444,
435,
425,
416,
407,
398,
389,
388,
390,
397,
410,
409,
398,
404,
406,
411,
418,
431,
440,
445,
453,
463,
469,
474,
471,
457,
450,
459,
465,
471,
481,
490,
496,
500,
511,
521,
530,
530,
526,
525,
518,
509,
495,
481,
473,
467,
463,
456,
449,
447,
450,
456,
464,
470,
477,
479,
475,
464,
454,
440,
426,
424,
424,
423,
419,
415,
405,
399,
393,
394,
398,
405,
414,
420,
423,
423,
426,
428,
422,
416,
414,
416,
416,
406,
395,
386,
378,
376,
374,
375,
376,
370,
362,
355,
352,
345,
332,
325,
324,
330,
338,
342,
341,
334,
331,
329,
326,
324,
325,
325,
326,
327,
327,
327,
328,
329,
328,
327,
328,
328,
329,
330,
331,
331,
331,
336,
339,
341,
349,
357,
367,
361,
359,
368,
371,
370,
367,
366,
356,
359,
356,
352,
351,
353,
356,
361,
365,
365,
367,
376,
382,
389,
394,
401,
410,
419,
566,
568,
583,
602,
614,
607,
599,
589,
581,
598,
616,
634,
650,
665,
670,
656,
664,
678,
674,
671,
679,
692,
705,
720,
733,
738,
729,
716,
700,
680,
666,
655,
642,
625,
606,
594,
596,
596,
596,
592,
582,
567,
556,
569,
577,
589,
604,
620,
623,
608,
595,
581,
568,
554,
544,
535,
522,
525,
527,
525,
507,
494,
497,
504,
500,
493,
489,
480,
466,
455,
446,
437,
432,
426,
414,
414,
414,
413,
410,
401,
415,
422,
428,
420,
407,
398,
393,
412,
423,
432,
441,
445,
448,
455,
464,
461,
458,
468,
476,
482,
491,
509,
525,
533,
543,
551,
558,
564,
566,
574,
575,
563,
542,
524,
530,
511,
510,
525,
525,
507,
489,
484,
493,
505,
505,
497,
480,
473,
474,
472,
470,
476,
485,
496,
494,
480,
486,
486,
501,
510,
511,
508,
503,
513,
516,
508,
489,
484,
485,
488,
495,
503,
512,
513,
502,
486,
475,
468,
470,
472,
473,
478,
492,
507,
521,
530,
534,
535,
529,
527,
543,
557,
571,
576,
565,
548,
530,
537,
542,
550,
549,
553,
563,
571,
576,
578,
577,
583,
587,
586,
579,
575,
576,
571,
559,
546,
535,
530,
524,
514,
506,
497,
486,
480,
480,
474,
466,
459,
450,
440,
430,
422,
413,
404,
396,
391,
389,
394,
405,
416,
413,
404,
407,
416,
414,
416,
427,
441,
451,
462,
470,
476,
481,
476,
460,
452,
467,
476,
481,
485,
494,
502,
508,
511,
520,
525,
524,
520,
519,
515,
507,
496,
484,
467,
458,
452,
447,
440,
437,
443,
450,
457,
464,
469,
473,
473,
464,
454,
442,
435,
434,
437,
432,
426,
418,
412,
405,
397,
388,
390,
398,
407,
414,
414,
414,
419,
424,
422,
415,
411,
410,
413,
404,
395,
384,
375,
366,
366,
363,
366,
364,
357,
350,
344,
339,
329,
325,
324,
328,
329,
329,
329,
327,
326,
325,
324,
323,
324,
325,
326,
327,
328,
327,
328,
330,
333,
334,
337,
343,
339,
333,
332,
334,
335,
337,
338,
344,
352,
362,
375,
372,
365,
374,
382,
383,
379,
376,
370,
373,
366,
359,
355,
356,
357,
357,
357,
359,
365,
374,
379,
386,
389,
393,
404,
415,
580,
578,
584,
602,
622,
624,
611,
606,
596,
597,
616,
635,
652,
673,
676,
668,
672,
684,
681,
680,
681,
696,
712,
729,
743,
734,
722,
710,
693,
672,
653,
636,
627,
614,
600,
582,
583,
583,
580,
574,
566,
551,
548,
558,
563,
580,
598,
612,
613,
599,
586,
572,
558,
544,
534,
526,
516,
509,
518,
521,
504,
482,
483,
491,
487,
478,
477,
474,
464,
449,
440,
427,
414,
414,
419,
418,
422,
428,
424,
414,
425,
440,
443,
430,
416,
409,
414,
412,
417,
427,
442,
456,
464,
471,
478,
475,
470,
472,
486,
497,
497,
512,
532,
548,
557,
567,
574,
565,
555,
554,
556,
552,
534,
514,
512,
508,
499,
512,
516,
503,
487,
480,
490,
503,
500,
488,
481,
478,
485,
478,
468,
468,
473,
480,
487,
474,
481,
494,
498,
501,
500,
500,
503,
501,
503,
501,
488,
484,
484,
486,
492,
497,
499,
498,
491,
477,
470,
471,
474,
476,
479,
483,
487,
504,
524,
541,
553,
554,
550,
546,
549,
564,
580,
579,
561,
541,
536,
540,
548,
550,
559,
564,
570,
579,
583,
588,
587,
584,
589,
590,
585,
581,
575,
569,
562,
549,
536,
525,
524,
515,
512,
505,
496,
478,
464,
459,
452,
446,
442,
438,
427,
419,
410,
402,
394,
391,
391,
396,
410,
424,
427,
415,
412,
420,
428,
426,
426,
438,
452,
464,
475,
484,
488,
481,
464,
459,
469,
480,
490,
493,
499,
504,
513,
519,
521,
523,
517,
512,
514,
512,
505,
496,
486,
469,
454,
445,
436,
428,
428,
436,
444,
450,
455,
459,
462,
467,
467,
460,
449,
446,
448,
446,
439,
431,
425,
418,
410,
402,
391,
386,
391,
400,
405,
406,
408,
413,
418,
418,
413,
404,
405,
407,
404,
394,
385,
375,
366,
358,
354,
354,
354,
352,
346,
339,
330,
325,
321,
323,
326,
328,
328,
325,
325,
324,
323,
322,
322,
323,
325,
327,
329,
329,
329,
331,
334,
337,
342,
345,
351,
356,
350,
342,
336,
336,
337,
340,
346,
353,
367,
379,
381,
380,
378,
390,
396,
393,
390,
387,
388,
378,
363,
363,
364,
364,
361,
358,
359,
361,
366,
370,
371,
375,
385,
398,
406,
583,
591,
584,
602,
619,
635,
630,
623,
615,
606,
613,
622,
644,
662,
672,
678,
682,
687,
682,
681,
691,
703,
719,
733,
743,
728,
712,
700,
683,
665,
648,
630,
610,
600,
586,
572,
569,
568,
564,
561,
555,
541,
550,
548,
559,
570,
585,
599,
601,
590,
577,
565,
552,
535,
523,
516,
508,
498,
507,
513,
506,
486,
468,
477,
480,
466,
461,
462,
455,
442,
425,
414,
417,
421,
426,
427,
431,
446,
437,
430,
435,
446,
452,
437,
417,
419,
427,
427,
421,
425,
439,
456,
472,
484,
491,
490,
486,
486,
487,
502,
510,
512,
532,
552,
565,
578,
575,
563,
541,
533,
537,
537,
528,
511,
499,
499,
496,
505,
506,
495,
486,
475,
485,
497,
496,
484,
485,
490,
497,
490,
476,
460,
463,
471,
478,
476,
482,
499,
495,
500,
500,
502,
500,
492,
490,
487,
484,
483,
480,
482,
485,
485,
481,
481,
479,
473,
472,
473,
478,
482,
488,
492,
488,
503,
521,
540,
561,
573,
567,
566,
568,
576,
587,
573,
554,
545,
548,
555,
560,
560,
560,
574,
583,
588,
594,
594,
598,
594,
594,
598,
588,
575,
566,
558,
552,
546,
535,
525,
514,
513,
514,
514,
502,
483,
466,
452,
443,
434,
425,
427,
425,
417,
408,
400,
393,
390,
389,
399,
412,
425,
433,
427,
414,
420,
432,
439,
439,
441,
449,
461,
475,
489,
495,
486,
470,
463,
472,
481,
490,
500,
504,
512,
517,
525,
525,
519,
512,
509,
508,
507,
500,
492,
482,
469,
455,
444,
432,
422,
425,
430,
438,
442,
442,
447,
454,
462,
466,
463,
457,
455,
456,
453,
446,
438,
431,
423,
415,
406,
397,
388,
388,
392,
397,
398,
401,
407,
413,
416,
412,
402,
398,
401,
403,
395,
387,
380,
367,
356,
350,
345,
345,
344,
341,
335,
326,
317,
319,
320,
324,
326,
325,
324,
324,
323,
322,
322,
322,
324,
326,
328,
329,
332,
332,
334,
339,
341,
342,
350,
357,
364,
360,
350,
339,
337,
337,
342,
350,
358,
368,
380,
390,
393,
393,
394,
404,
406,
404,
403,
399,
387,
371,
373,
379,
372,
369,
366,
366,
365,
366,
367,
367,
369,
376,
386,
393,
588,
603,
600,
597,
616,
636,
647,
641,
633,
620,
628,
640,
650,
656,
673,
682,
697,
694,
688,
689,
694,
708,
723,
730,
726,
718,
702,
687,
671,
657,
643,
629,
613,
600,
584,
568,
559,
554,
550,
550,
545,
533,
542,
538,
548,
562,
573,
583,
585,
580,
567,
554,
546,
536,
522,
508,
500,
488,
492,
505,
506,
492,
469,
459,
467,
463,
441,
446,
439,
432,
424,
421,
420,
424,
439,
439,
440,
454,
458,
454,
455,
454,
460,
443,
423,
419,
432,
435,
426,
427,
440,
456,
472,
488,
502,
506,
505,
506,
505,
509,
522,
522,
528,
546,
561,
575,
567,
558,
541,
522,
517,
520,
519,
508,
497,
488,
491,
500,
495,
486,
480,
475,
479,
491,
489,
484,
491,
501,
510,
497,
484,
471,
461,
466,
469,
477,
487,
492,
496,
498,
501,
499,
496,
487,
485,
483,
485,
482,
480,
474,
477,
475,
471,
470,
471,
472,
475,
476,
482,
488,
499,
508,
504,
507,
521,
540,
560,
580,
586,
586,
589,
593,
586,
568,
548,
553,
561,
567,
573,
574,
569,
572,
584,
594,
601,
600,
602,
601,
602,
601,
588,
575,
563,
550,
537,
535,
528,
522,
512,
501,
505,
512,
500,
482,
467,
456,
447,
438,
430,
416,
418,
416,
407,
400,
394,
390,
392,
401,
412,
423,
437,
440,
426,
424,
430,
444,
451,
453,
460,
470,
476,
491,
501,
492,
476,
476,
473,
483,
490,
497,
506,
514,
523,
528,
526,
518,
510,
504,
503,
503,
501,
491,
481,
468,
455,
444,
432,
423,
419,
424,
429,
431,
429,
438,
450,
459,
465,
465,
463,
459,
462,
462,
452,
442,
432,
425,
417,
409,
400,
390,
380,
380,
388,
390,
395,
402,
405,
409,
410,
402,
390,
391,
394,
391,
384,
381,
374,
361,
349,
339,
337,
337,
334,
328,
322,
317,
318,
320,
322,
323,
324,
323,
323,
323,
322,
322,
323,
324,
326,
328,
329,
332,
334,
337,
341,
347,
353,
353,
359,
368,
365,
354,
341,
338,
338,
343,
354,
365,
374,
377,
388,
396,
403,
406,
408,
413,
415,
414,
403,
391,
380,
382,
384,
375,
378,
379,
374,
374,
372,
368,
368,
368,
370,
375,
378,
595,
611,
618,
617,
618,
634,
653,
656,
646,
636,
639,
654,
671,
674,
683,
693,
700,
696,
689,
690,
699,
712,
720,
728,
712,
708,
694,
678,
661,
647,
636,
625,
613,
600,
586,
570,
557,
546,
538,
535,
530,
522,
537,
526,
539,
548,
559,
565,
572,
571,
556,
541,
532,
528,
524,
510,
499,
486,
485,
495,
500,
487,
471,
451,
455,
456,
442,
437,
433,
429,
426,
426,
427,
423,
425,
433,
439,
450,
466,
476,
475,
471,
463,
445,
427,
425,
434,
442,
431,
437,
446,
458,
471,
484,
497,
512,
520,
525,
524,
522,
526,
535,
534,
539,
559,
561,
544,
548,
541,
523,
511,
503,
503,
500,
494,
487,
486,
486,
485,
480,
478,
475,
476,
486,
484,
486,
496,
504,
506,
497,
490,
477,
462,
461,
465,
478,
490,
500,
496,
498,
499,
500,
490,
484,
486,
486,
486,
486,
490,
488,
480,
474,
474,
473,
474,
474,
478,
484,
486,
496,
509,
521,
523,
518,
522,
542,
562,
585,
598,
603,
606,
602,
585,
570,
557,
562,
569,
576,
584,
588,
585,
582,
589,
596,
603,
606,
608,
611,
609,
593,
580,
570,
561,
548,
535,
528,
521,
515,
506,
495,
488,
493,
494,
485,
471,
459,
450,
442,
430,
415,
411,
415,
406,
399,
393,
392,
397,
406,
417,
426,
436,
445,
440,
434,
435,
440,
454,
464,
472,
482,
490,
496,
504,
498,
484,
487,
479,
487,
496,
502,
510,
517,
525,
529,
526,
516,
508,
501,
498,
499,
498,
490,
482,
471,
458,
447,
436,
426,
417,
414,
419,
423,
427,
438,
448,
455,
461,
461,
457,
456,
459,
462,
452,
443,
434,
426,
420,
413,
404,
396,
386,
376,
379,
382,
386,
391,
394,
400,
403,
399,
389,
380,
383,
381,
376,
376,
372,
363,
353,
340,
329,
330,
327,
323,
316,
317,
318,
319,
321,
322,
322,
322,
322,
322,
323,
323,
324,
325,
326,
327,
328,
329,
333,
337,
340,
349,
361,
366,
365,
369,
368,
355,
344,
338,
337,
344,
358,
370,
380,
387,
391,
397,
402,
406,
405,
413,
415,
412,
403,
397,
393,
390,
386,
388,
393,
388,
389,
385,
382,
380,
372,
369,
371,
370,
373,
599,
619,
635,
636,
638,
637,
653,
671,
665,
656,
656,
663,
680,
691,
695,
699,
697,
690,
682,
688,
699,
708,
714,
710,
701,
696,
685,
671,
650,
637,
626,
616,
607,
596,
583,
571,
557,
545,
535,
525,
518,
518,
528,
522,
527,
537,
542,
548,
563,
561,
547,
531,
515,
515,
514,
511,
501,
489,
473,
481,
487,
483,
466,
449,
445,
446,
437,
435,
433,
430,
425,
438,
440,
437,
436,
444,
454,
460,
467,
480,
486,
480,
467,
448,
436,
438,
438,
451,
444,
441,
453,
459,
468,
479,
488,
500,
515,
525,
535,
537,
537,
543,
543,
550,
555,
551,
535,
530,
532,
525,
512,
502,
495,
488,
487,
483,
484,
479,
477,
474,
474,
477,
473,
478,
480,
487,
499,
506,
497,
482,
480,
477,
461,
459,
462,
475,
489,
500,
499,
498,
499,
500,
492,
487,
488,
490,
490,
489,
494,
499,
491,
482,
474,
475,
478,
479,
484,
490,
496,
500,
515,
530,
540,
538,
531,
541,
560,
580,
598,
612,
615,
607,
592,
579,
561,
560,
570,
583,
592,
599,
597,
595,
600,
606,
612,
615,
617,
613,
602,
586,
572,
560,
550,
545,
538,
528,
517,
507,
500,
490,
480,
473,
482,
479,
470,
460,
452,
441,
426,
414,
402,
406,
402,
397,
395,
394,
399,
407,
419,
431,
441,
448,
450,
447,
444,
450,
457,
466,
476,
484,
492,
502,
512,
502,
492,
491,
485,
487,
497,
505,
514,
520,
526,
528,
522,
512,
505,
499,
495,
494,
493,
490,
481,
471,
460,
449,
437,
425,
417,
409,
408,
415,
428,
437,
442,
448,
454,
454,
450,
448,
449,
448,
443,
438,
434,
430,
425,
416,
408,
401,
392,
380,
372,
371,
376,
381,
384,
390,
393,
393,
386,
376,
372,
370,
366,
362,
363,
357,
349,
339,
328,
325,
322,
319,
317,
318,
319,
320,
320,
321,
322,
322,
322,
323,
323,
324,
324,
325,
326,
327,
328,
329,
334,
338,
342,
350,
359,
372,
379,
379,
369,
355,
346,
341,
341,
344,
356,
368,
376,
384,
391,
394,
396,
401,
402,
403,
404,
401,
397,
401,
405,
398,
390,
400,
404,
400,
399,
397,
394,
390,
382,
383,
378,
370,
370,
610,
625,
644,
658,
659,
658,
662,
679,
682,
676,
674,
678,
691,
700,
701,
698,
695,
683,
680,
684,
697,
704,
699,
690,
687,
685,
675,
661,
646,
633,
620,
607,
597,
588,
577,
569,
557,
545,
533,
524,
512,
512,
520,
511,
519,
523,
530,
550,
563,
557,
545,
530,
513,
497,
499,
500,
504,
494,
478,
460,
471,
475,
463,
449,
439,
437,
436,
436,
439,
430,
425,
434,
442,
451,
452,
452,
462,
475,
478,
487,
490,
486,
473,
453,
453,
457,
454,
460,
457,
450,
459,
465,
473,
480,
491,
502,
509,
516,
526,
534,
538,
540,
543,
548,
549,
544,
531,
516,
517,
518,
512,
500,
492,
487,
485,
483,
483,
484,
482,
480,
478,
474,
472,
475,
474,
489,
500,
504,
498,
481,
471,
473,
463,
460,
470,
479,
489,
501,
500,
495,
501,
499,
494,
491,
492,
498,
498,
495,
499,
502,
497,
481,
475,
479,
482,
485,
492,
503,
509,
509,
514,
534,
551,
555,
549,
546,
558,
577,
596,
613,
622,
613,
600,
587,
570,
558,
568,
585,
598,
607,
611,
610,
609,
614,
622,
624,
624,
612,
596,
582,
569,
556,
544,
534,
529,
523,
514,
502,
492,
486,
475,
462,
470,
471,
461,
450,
442,
437,
425,
412,
402,
398,
400,
398,
396,
396,
400,
406,
415,
426,
438,
450,
453,
454,
453,
454,
456,
462,
463,
476,
490,
504,
514,
508,
501,
494,
487,
492,
497,
503,
509,
517,
524,
528,
522,
512,
502,
496,
491,
489,
489,
488,
478,
467,
457,
447,
438,
429,
420,
408,
401,
414,
424,
429,
435,
444,
452,
455,
447,
438,
438,
437,
434,
428,
426,
427,
426,
417,
411,
402,
392,
386,
378,
366,
363,
367,
372,
377,
382,
385,
380,
371,
362,
357,
353,
350,
352,
352,
345,
337,
330,
322,
319,
317,
317,
318,
318,
320,
320,
321,
321,
321,
322,
323,
323,
324,
326,
327,
327,
327,
328,
330,
339,
343,
342,
352,
361,
372,
382,
379,
367,
353,
348,
345,
342,
341,
349,
358,
365,
373,
380,
384,
388,
389,
388,
388,
390,
390,
388,
399,
405,
398,
400,
404,
410,
410,
410,
407,
405,
400,
395,
395,
391,
385,
380,
606,
625,
643,
658,
668,
674,
678,
687,
697,
694,
693,
698,
707,
702,
692,
690,
695,
685,
678,
680,
687,
690,
685,
677,
671,
669,
663,
653,
642,
632,
621,
608,
596,
580,
566,
557,
548,
540,
532,
520,
505,
501,
512,
506,
510,
516,
533,
551,
563,
554,
544,
532,
518,
499,
485,
484,
490,
492,
481,
463,
455,
466,
469,
461,
450,
442,
440,
442,
440,
428,
422,
428,
439,
449,
460,
465,
472,
478,
480,
483,
488,
490,
479,
464,
471,
478,
477,
472,
470,
462,
464,
476,
481,
490,
506,
516,
523,
523,
523,
526,
533,
535,
536,
541,
537,
532,
525,
512,
508,
510,
500,
495,
491,
488,
485,
483,
480,
488,
489,
491,
494,
496,
489,
471,
475,
489,
492,
502,
501,
486,
467,
461,
456,
462,
471,
480,
486,
490,
487,
495,
500,
499,
501,
497,
495,
501,
501,
497,
502,
503,
491,
480,
480,
484,
487,
491,
506,
517,
526,
527,
527,
536,
561,
573,
568,
558,
566,
582,
599,
616,
627,
620,
606,
590,
575,
566,
573,
578,
590,
602,
615,
624,
625,
624,
632,
631,
618,
601,
588,
576,
565,
555,
546,
534,
522,
511,
503,
495,
485,
475,
468,
458,
459,
465,
461,
449,
436,
425,
416,
409,
401,
398,
397,
397,
397,
399,
404,
411,
419,
427,
435,
447,
454,
453,
449,
445,
448,
454,
462,
476,
491,
505,
514,
512,
505,
496,
496,
502,
504,
508,
511,
518,
526,
528,
522,
512,
502,
494,
487,
482,
481,
479,
472,
461,
453,
444,
437,
429,
419,
406,
400,
408,
416,
422,
430,
437,
446,
453,
446,
434,
425,
426,
425,
422,
418,
418,
417,
417,
415,
406,
402,
396,
389,
379,
366,
356,
357,
362,
367,
374,
379,
365,
355,
349,
342,
340,
341,
344,
336,
329,
323,
320,
317,
317,
317,
318,
318,
319,
320,
320,
320,
321,
322,
323,
324,
325,
328,
329,
330,
328,
329,
328,
335,
340,
345,
352,
361,
372,
381,
377,
364,
352,
348,
346,
344,
345,
348,
352,
358,
364,
367,
372,
376,
375,
374,
375,
376,
376,
384,
396,
403,
400,
400,
407,
413,
416,
417,
416,
416,
412,
408,
407,
401,
398,
390,
599,
617,
634,
643,
651,
670,
684,
699,
712,
714,
710,
710,
709,
697,
682,
680,
690,
690,
681,
673,
675,
676,
674,
667,
658,
654,
654,
648,
639,
631,
625,
615,
600,
583,
567,
551,
535,
530,
526,
520,
505,
495,
502,
501,
504,
516,
531,
548,
558,
548,
540,
531,
521,
506,
489,
474,
474,
479,
482,
472,
457,
447,
462,
468,
460,
449,
445,
437,
433,
424,
425,
427,
434,
445,
458,
465,
468,
471,
473,
470,
475,
488,
490,
488,
492,
499,
496,
491,
484,
478,
475,
480,
490,
501,
512,
525,
535,
536,
532,
531,
528,
528,
532,
534,
526,
519,
518,
512,
503,
502,
498,
492,
488,
486,
482,
486,
481,
472,
483,
495,
506,
510,
494,
478,
471,
477,
482,
495,
502,
490,
474,
459,
457,
462,
470,
474,
476,
477,
488,
499,
506,
508,
500,
497,
497,
501,
500,
499,
504,
508,
492,
485,
487,
493,
501,
508,
518,
533,
544,
549,
550,
551,
567,
586,
584,
574,
576,
587,
603,
619,
633,
624,
609,
593,
578,
577,
588,
594,
599,
600,
612,
625,
634,
637,
642,
628,
611,
596,
580,
568,
558,
549,
540,
531,
521,
509,
498,
487,
479,
472,
461,
450,
450,
457,
459,
451,
442,
433,
422,
412,
402,
397,
397,
397,
398,
399,
404,
413,
423,
434,
442,
447,
452,
446,
438,
443,
454,
465,
473,
484,
494,
505,
514,
517,
510,
501,
503,
507,
512,
514,
516,
523,
526,
525,
518,
509,
500,
492,
484,
475,
472,
469,
464,
454,
446,
439,
430,
423,
414,
401,
392,
400,
407,
417,
422,
431,
440,
450,
446,
434,
425,
415,
416,
416,
411,
410,
410,
413,
414,
412,
409,
404,
393,
382,
373,
363,
352,
346,
349,
358,
369,
368,
364,
353,
342,
330,
331,
333,
326,
321,
319,
317,
317,
317,
318,
318,
319,
319,
319,
319,
319,
320,
321,
323,
324,
326,
327,
328,
331,
331,
329,
330,
341,
342,
345,
352,
360,
370,
380,
374,
361,
352,
349,
351,
353,
354,
353,
352,
352,
354,
357,
360,
362,
362,
362,
364,
365,
372,
382,
392,
397,
398,
402,
405,
407,
412,
415,
416,
419,
420,
419,
415,
408,
405,
396,
589,
610,
619,
631,
649,
670,
682,
696,
715,
721,
716,
711,
704,
693,
681,
669,
676,
687,
684,
678,
664,
658,
660,
655,
649,
644,
647,
644,
640,
636,
630,
620,
605,
586,
571,
557,
539,
522,
513,
513,
503,
490,
496,
498,
504,
513,
526,
538,
547,
540,
532,
524,
515,
505,
494,
478,
462,
465,
474,
477,
463,
448,
457,
466,
462,
458,
456,
452,
440,
424,
440,
436,
426,
432,
438,
444,
450,
460,
462,
473,
481,
489,
494,
498,
502,
511,
514,
507,
497,
493,
487,
486,
497,
510,
521,
533,
543,
546,
539,
530,
523,
524,
528,
527,
521,
511,
508,
507,
498,
498,
498,
492,
488,
490,
496,
501,
501,
488,
474,
489,
502,
502,
496,
485,
471,
466,
467,
481,
493,
495,
480,
461,
453,
458,
464,
462,
468,
478,
487,
499,
504,
511,
500,
498,
501,
506,
507,
507,
512,
513,
497,
486,
496,
505,
516,
526,
534,
548,
560,
568,
569,
569,
577,
595,
596,
587,
584,
593,
606,
621,
637,
630,
614,
600,
585,
591,
603,
612,
613,
612,
611,
620,
631,
641,
638,
624,
609,
598,
581,
566,
557,
547,
539,
535,
527,
519,
511,
501,
488,
477,
466,
455,
442,
447,
451,
453,
453,
441,
427,
414,
404,
399,
398,
398,
399,
399,
403,
410,
420,
430,
437,
439,
442,
438,
436,
444,
452,
465,
476,
487,
501,
509,
518,
524,
516,
510,
513,
512,
518,
524,
523,
526,
530,
524,
513,
506,
498,
490,
481,
472,
464,
460,
452,
442,
433,
426,
421,
415,
404,
394,
388,
391,
403,
410,
414,
425,
435,
443,
443,
437,
425,
417,
412,
407,
405,
401,
401,
405,
407,
414,
415,
404,
390,
381,
371,
364,
356,
345,
338,
346,
359,
366,
366,
357,
347,
335,
319,
322,
320,
318,
317,
317,
318,
318,
318,
318,
318,
318,
318,
318,
318,
318,
321,
323,
325,
326,
327,
328,
329,
333,
339,
337,
341,
350,
357,
360,
363,
371,
378,
368,
358,
351,
351,
356,
363,
369,
368,
366,
363,
353,
354,
356,
361,
363,
363,
363,
364,
368,
377,
384,
391,
394,
397,
399,
400,
404,
408,
412,
417,
419,
417,
413,
408,
403,
394,
608,
611,
617,
626,
640,
657,
674,
691,
706,
711,
711,
706,
696,
688,
679,
663,
670,
686,
689,
687,
669,
656,
646,
642,
637,
635,
636,
634,
637,
638,
635,
623,
611,
588,
572,
557,
540,
523,
511,
498,
493,
486,
496,
503,
504,
514,
526,
534,
539,
531,
520,
507,
499,
494,
488,
478,
461,
455,
463,
465,
462,
450,
451,
458,
459,
463,
462,
461,
444,
432,
447,
452,
448,
438,
438,
440,
452,
459,
468,
478,
483,
488,
489,
486,
495,
506,
516,
516,
512,
506,
501,
498,
503,
518,
535,
544,
550,
549,
539,
524,
524,
532,
525,
524,
518,
509,
501,
500,
505,
502,
500,
501,
496,
496,
505,
512,
506,
496,
472,
477,
491,
498,
493,
484,
472,
463,
462,
471,
488,
493,
484,
464,
448,
450,
451,
463,
470,
478,
488,
493,
498,
504,
504,
502,
508,
512,
512,
511,
514,
517,
505,
492,
495,
511,
526,
536,
552,
564,
576,
586,
590,
588,
597,
606,
607,
598,
591,
595,
609,
624,
637,
635,
620,
608,
596,
601,
616,
625,
626,
622,
618,
621,
633,
645,
635,
619,
604,
587,
575,
562,
563,
558,
551,
542,
532,
522,
512,
501,
491,
480,
469,
456,
441,
437,
441,
446,
451,
441,
427,
412,
402,
398,
399,
400,
402,
400,
400,
403,
411,
418,
426,
434,
430,
431,
438,
446,
458,
474,
490,
498,
505,
514,
524,
528,
524,
521,
521,
520,
523,
529,
530,
532,
529,
522,
512,
502,
495,
489,
478,
470,
462,
452,
441,
435,
427,
422,
414,
409,
401,
390,
386,
390,
397,
401,
412,
421,
428,
436,
440,
439,
429,
421,
412,
403,
398,
392,
389,
394,
399,
407,
415,
403,
391,
379,
368,
361,
353,
346,
338,
338,
349,
354,
356,
352,
344,
333,
322,
318,
317,
317,
317,
318,
318,
318,
318,
318,
318,
318,
317,
317,
317,
318,
322,
324,
326,
326,
327,
328,
331,
336,
341,
345,
344,
350,
358,
369,
369,
373,
370,
362,
354,
348,
354,
363,
370,
377,
383,
382,
377,
366,
370,
370,
373,
372,
375,
376,
369,
365,
371,
377,
384,
389,
390,
391,
395,
399,
402,
410,
415,
416,
415,
410,
406,
401,
394,
612,
626,
632,
641,
653,
666,
676,
686,
699,
710,
707,
694,
685,
684,
676,
661,
660,
675,
689,
692,
677,
662,
650,
640,
631,
621,
623,
620,
628,
633,
632,
624,
610,
588,
572,
556,
541,
528,
516,
502,
487,
479,
491,
498,
495,
502,
519,
525,
524,
522,
510,
493,
480,
479,
476,
469,
458,
448,
450,
452,
458,
448,
445,
451,
446,
454,
463,
462,
444,
438,
451,
462,
460,
457,
457,
456,
454,
460,
462,
464,
469,
475,
476,
475,
490,
502,
511,
512,
512,
514,
515,
514,
517,
528,
542,
550,
546,
538,
523,
523,
530,
537,
525,
521,
517,
515,
512,
505,
500,
503,
509,
508,
495,
489,
502,
509,
504,
501,
479,
476,
488,
488,
491,
488,
473,
457,
460,
475,
481,
486,
484,
463,
446,
446,
451,
463,
472,
477,
485,
486,
494,
504,
506,
508,
513,
515,
516,
521,
525,
523,
511,
502,
504,
509,
525,
539,
557,
575,
589,
601,
609,
612,
620,
621,
614,
603,
596,
600,
612,
628,
643,
644,
634,
618,
614,
613,
619,
633,
636,
632,
622,
620,
631,
644,
633,
616,
600,
583,
566,
551,
546,
550,
549,
539,
529,
518,
502,
490,
481,
473,
464,
451,
438,
426,
428,
432,
436,
430,
418,
404,
401,
400,
401,
401,
403,
402,
402,
402,
411,
420,
429,
430,
424,
432,
448,
460,
468,
477,
491,
502,
510,
516,
524,
527,
527,
527,
525,
525,
528,
534,
537,
535,
527,
518,
509,
501,
493,
485,
476,
468,
460,
451,
445,
438,
429,
422,
416,
410,
405,
396,
386,
377,
388,
401,
408,
415,
422,
429,
433,
436,
432,
425,
418,
412,
403,
390,
378,
379,
388,
398,
405,
405,
394,
381,
374,
366,
358,
347,
336,
330,
338,
341,
345,
342,
333,
325,
319,
317,
317,
317,
318,
318,
318,
318,
318,
318,
318,
317,
316,
317,
320,
322,
324,
326,
327,
327,
328,
330,
334,
340,
339,
343,
354,
353,
353,
366,
370,
369,
365,
355,
351,
350,
356,
365,
375,
385,
394,
394,
386,
381,
387,
389,
387,
385,
390,
384,
376,
370,
372,
375,
378,
382,
385,
387,
390,
396,
400,
406,
410,
412,
410,
406,
403,
404,
402,
606,
621,
642,
657,
667,
681,
691,
700,
709,
716,
705,
692,
678,
671,
670,
658,
654,
668,
682,
684,
682,
667,
656,
648,
638,
625,
610,
605,
613,
616,
616,
610,
600,
585,
575,
560,
547,
535,
524,
508,
486,
474,
482,
486,
487,
502,
515,
512,
512,
512,
502,
486,
471,
461,
457,
456,
452,
445,
440,
446,
454,
452,
439,
441,
440,
453,
461,
456,
440,
437,
444,
459,
465,
473,
476,
472,
468,
461,
468,
472,
474,
478,
479,
478,
485,
492,
500,
501,
510,
515,
523,
530,
533,
539,
543,
533,
525,
518,
522,
528,
529,
526,
529,
518,
514,
515,
512,
506,
500,
499,
506,
510,
499,
486,
495,
494,
491,
489,
482,
470,
474,
472,
474,
479,
476,
453,
453,
462,
465,
475,
476,
459,
447,
450,
459,
465,
474,
475,
480,
485,
490,
499,
504,
510,
513,
510,
511,
516,
522,
517,
518,
514,
517,
515,
522,
536,
558,
577,
595,
613,
625,
631,
635,
630,
618,
610,
606,
607,
620,
633,
649,
656,
648,
633,
634,
629,
632,
632,
640,
639,
631,
623,
632,
638,
623,
608,
593,
575,
561,
549,
535,
528,
530,
529,
522,
510,
498,
486,
474,
462,
453,
443,
432,
424,
416,
417,
418,
415,
406,
399,
402,
402,
402,
401,
404,
404,
404,
402,
409,
417,
427,
426,
422,
431,
447,
464,
477,
489,
496,
504,
511,
516,
523,
528,
527,
526,
525,
525,
528,
532,
535,
533,
524,
516,
508,
500,
491,
486,
481,
476,
468,
460,
450,
439,
429,
420,
413,
408,
403,
395,
385,
376,
381,
394,
402,
409,
416,
422,
426,
429,
432,
429,
425,
419,
410,
398,
384,
372,
377,
388,
394,
403,
401,
393,
385,
379,
367,
352,
338,
326,
329,
331,
334,
330,
324,
319,
317,
317,
317,
317,
318,
318,
318,
318,
318,
317,
317,
317,
319,
320,
323,
326,
329,
328,
328,
329,
331,
332,
338,
344,
349,
341,
349,
356,
358,
364,
366,
366,
356,
349,
349,
353,
362,
372,
382,
391,
403,
404,
400,
400,
403,
402,
398,
394,
392,
385,
377,
380,
381,
381,
379,
383,
386,
388,
389,
393,
399,
404,
405,
406,
405,
401,
400,
402,
407,
610,
623,
640,
654,
669,
679,
688,
698,
712,
719,
705,
690,
675,
666,
657,
652,
646,
659,
668,
669,
668,
665,
659,
651,
644,
632,
616,
596,
591,
595,
596,
594,
584,
576,
571,
560,
549,
538,
524,
508,
488,
472,
476,
483,
493,
502,
503,
502,
504,
503,
500,
490,
480,
466,
454,
446,
439,
440,
438,
441,
450,
452,
438,
438,
440,
448,
451,
448,
438,
442,
449,
453,
461,
473,
478,
488,
485,
470,
467,
478,
485,
490,
493,
493,
490,
496,
498,
502,
514,
520,
529,
541,
550,
546,
530,
522,
520,
523,
531,
536,
526,
525,
526,
521,
510,
512,
499,
499,
496,
494,
497,
494,
488,
484,
483,
479,
476,
474,
470,
475,
475,
474,
473,
467,
466,
454,
450,
452,
461,
476,
464,
451,
449,
458,
468,
476,
478,
480,
480,
486,
489,
497,
502,
502,
506,
509,
511,
514,
522,
523,
527,
527,
530,
529,
525,
538,
556,
576,
598,
619,
634,
647,
646,
636,
627,
620,
614,
619,
627,
641,
656,
665,
659,
652,
646,
642,
644,
643,
642,
648,
638,
631,
638,
635,
618,
598,
585,
574,
560,
544,
527,
518,
512,
512,
512,
505,
495,
484,
473,
462,
451,
443,
435,
427,
416,
411,
409,
406,
402,
400,
404,
407,
407,
402,
403,
406,
405,
403,
410,
416,
418,
415,
420,
432,
446,
455,
468,
478,
490,
505,
512,
514,
519,
523,
520,
519,
516,
519,
523,
524,
526,
525,
518,
512,
504,
500,
492,
490,
485,
477,
466,
449,
438,
431,
426,
415,
409,
404,
400,
390,
382,
375,
374,
388,
394,
402,
409,
415,
420,
424,
427,
430,
430,
426,
414,
401,
387,
371,
364,
374,
386,
401,
406,
402,
393,
382,
370,
354,
341,
328,
318,
319,
323,
321,
318,
315,
316,
317,
317,
318,
318,
318,
318,
318,
317,
317,
317,
318,
320,
323,
327,
330,
336,
329,
329,
329,
333,
336,
339,
345,
358,
356,
352,
353,
359,
365,
368,
360,
350,
346,
347,
354,
365,
376,
385,
396,
405,
403,
404,
402,
401,
403,
400,
399,
394,
392,
392,
392,
392,
392,
390,
388,
387,
389,
390,
391,
396,
400,
400,
400,
398,
398,
401,
407,
414,
604,
616,
628,
640,
654,
667,
675,
689,
708,
713,
706,
693,
685,
674,
662,
652,
641,
643,
650,
651,
652,
651,
649,
645,
639,
632,
613,
592,
572,
572,
575,
573,
571,
566,
562,
555,
545,
534,
519,
506,
490,
475,
466,
477,
485,
488,
489,
493,
501,
497,
492,
490,
485,
477,
464,
453,
444,
435,
437,
442,
448,
451,
437,
437,
439,
444,
444,
447,
448,
449,
458,
466,
468,
472,
476,
479,
486,
482,
470,
472,
480,
490,
500,
507,
504,
502,
503,
506,
519,
523,
537,
547,
549,
536,
514,
514,
527,
537,
541,
537,
522,
519,
524,
526,
512,
507,
510,
514,
506,
494,
485,
484,
484,
488,
487,
481,
478,
478,
475,
489,
490,
489,
478,
473,
463,
451,
446,
450,
464,
476,
461,
447,
451,
460,
474,
478,
477,
477,
475,
488,
494,
501,
500,
502,
510,
514,
520,
526,
529,
532,
537,
541,
545,
546,
544,
541,
547,
563,
586,
605,
617,
628,
638,
639,
635,
630,
627,
634,
638,
648,
662,
674,
673,
666,
656,
656,
656,
651,
645,
647,
642,
635,
643,
637,
621,
608,
595,
578,
562,
544,
534,
526,
511,
498,
496,
496,
492,
488,
480,
471,
462,
452,
440,
426,
416,
411,
408,
404,
403,
402,
410,
416,
414,
406,
403,
405,
407,
407,
410,
412,
412,
413,
418,
429,
435,
444,
450,
463,
486,
500,
500,
504,
513,
516,
510,
510,
508,
512,
515,
516,
518,
520,
512,
503,
496,
487,
486,
484,
478,
470,
460,
447,
435,
423,
413,
406,
402,
397,
394,
390,
382,
374,
367,
375,
387,
397,
404,
411,
415,
419,
424,
428,
430,
427,
413,
400,
386,
369,
360,
366,
381,
395,
407,
404,
397,
388,
378,
367,
352,
339,
328,
320,
317,
316,
314,
314,
316,
317,
317,
317,
318,
318,
318,
317,
317,
317,
316,
318,
322,
325,
329,
342,
340,
331,
329,
332,
335,
339,
341,
341,
356,
368,
366,
363,
361,
364,
364,
355,
344,
344,
346,
352,
362,
371,
379,
394,
401,
394,
395,
393,
388,
391,
390,
394,
402,
402,
404,
402,
402,
401,
397,
393,
392,
391,
392,
394,
395,
395,
392,
391,
390,
396,
402,
410,
418,
610,
616,
622,
633,
644,
656,
673,
688,
708,
711,
703,
692,
686,
678,
667,
658,
645,
627,
629,
631,
634,
634,
633,
625,
620,
615,
602,
586,
567,
552,
555,
556,
558,
557,
552,
547,
539,
527,
511,
498,
484,
471,
460,
468,
477,
483,
487,
497,
502,
490,
480,
482,
478,
471,
467,
471,
453,
438,
435,
440,
448,
452,
439,
437,
441,
441,
438,
446,
452,
456,
463,
464,
468,
476,
467,
467,
480,
477,
477,
474,
475,
483,
494,
503,
505,
505,
507,
511,
519,
531,
542,
550,
540,
521,
510,
522,
534,
538,
532,
521,
512,
516,
522,
528,
524,
518,
520,
527,
519,
510,
505,
501,
496,
501,
501,
495,
483,
479,
485,
494,
502,
498,
485,
488,
479,
463,
451,
449,
461,
467,
453,
445,
450,
460,
468,
472,
471,
473,
478,
486,
497,
498,
501,
506,
514,
522,
530,
537,
539,
542,
548,
553,
558,
564,
564,
559,
560,
560,
571,
586,
596,
611,
626,
634,
641,
640,
635,
643,
649,
659,
671,
682,
685,
678,
668,
672,
670,
659,
651,
648,
646,
645,
648,
638,
625,
612,
596,
579,
562,
556,
549,
535,
518,
502,
486,
481,
479,
476,
475,
470,
463,
452,
440,
426,
414,
411,
408,
407,
405,
409,
414,
420,
418,
411,
411,
408,
409,
412,
414,
414,
413,
415,
418,
424,
427,
436,
446,
463,
481,
488,
490,
491,
505,
507,
500,
501,
498,
504,
507,
508,
512,
515,
512,
501,
490,
476,
473,
472,
469,
462,
454,
445,
437,
426,
414,
402,
395,
387,
383,
382,
379,
371,
367,
368,
385,
395,
403,
408,
411,
414,
420,
426,
428,
423,
412,
400,
384,
367,
353,
357,
371,
386,
395,
394,
393,
387,
377,
367,
356,
344,
330,
321,
315,
313,
313,
314,
315,
317,
317,
317,
317,
317,
317,
317,
317,
317,
318,
322,
326,
326,
329,
334,
335,
335,
332,
334,
338,
342,
351,
354,
351,
361,
368,
367,
363,
358,
355,
348,
340,
343,
347,
358,
368,
377,
385,
393,
392,
386,
384,
382,
375,
379,
387,
397,
405,
406,
405,
405,
406,
403,
400,
397,
392,
394,
393,
392,
392,
389,
387,
389,
392,
398,
402,
410,
418,
615,
632,
640,
648,
656,
664,
675,
692,
711,
716,
702,
684,
678,
674,
669,
661,
649,
631,
614,
612,
614,
615,
615,
609,
600,
595,
589,
577,
560,
542,
536,
538,
540,
545,
546,
537,
531,
522,
509,
495,
479,
463,
462,
477,
484,
491,
500,
504,
499,
482,
463,
465,
460,
461,
465,
471,
458,
440,
432,
437,
448,
454,
446,
436,
439,
436,
435,
443,
452,
454,
451,
457,
464,
458,
453,
460,
469,
475,
478,
483,
480,
477,
486,
486,
494,
502,
509,
513,
525,
529,
547,
543,
528,
510,
512,
523,
537,
530,
513,
505,
512,
524,
528,
531,
534,
531,
530,
535,
537,
527,
525,
516,
511,
514,
516,
508,
486,
487,
494,
502,
511,
505,
492,
500,
491,
477,
459,
446,
454,
454,
446,
445,
449,
452,
460,
463,
462,
473,
479,
485,
492,
498,
499,
508,
516,
527,
537,
544,
550,
551,
558,
567,
572,
582,
586,
578,
578,
578,
576,
580,
587,
599,
616,
632,
646,
648,
645,
646,
655,
667,
680,
692,
695,
686,
684,
684,
675,
664,
656,
653,
648,
650,
647,
636,
623,
610,
594,
581,
575,
567,
551,
535,
516,
501,
485,
474,
466,
463,
462,
459,
455,
449,
439,
426,
414,
410,
410,
409,
409,
412,
419,
426,
422,
415,
413,
411,
411,
412,
416,
417,
420,
425,
434,
436,
438,
447,
451,
455,
470,
476,
477,
486,
496,
496,
493,
491,
490,
496,
500,
499,
506,
512,
507,
498,
486,
472,
463,
461,
459,
454,
447,
439,
434,
430,
418,
408,
398,
390,
377,
373,
369,
367,
363,
366,
382,
392,
401,
405,
404,
412,
418,
419,
418,
418,
409,
398,
384,
367,
354,
346,
357,
372,
380,
379,
382,
376,
368,
358,
346,
336,
327,
319,
313,
312,
312,
313,
315,
316,
317,
317,
317,
317,
317,
317,
318,
318,
322,
327,
327,
326,
328,
328,
331,
328,
329,
332,
340,
341,
356,
368,
363,
356,
365,
366,
361,
355,
349,
343,
341,
342,
348,
358,
368,
378,
385,
388,
386,
382,
372,
367,
370,
374,
386,
398,
404,
402,
399,
398,
401,
399,
393,
389,
389,
390,
387,
388,
387,
383,
381,
391,
395,
400,
404,
411,
417,
621,
630,
647,
661,
671,
681,
691,
702,
719,
718,
704,
686,
672,
666,
658,
651,
642,
631,
617,
596,
584,
589,
590,
586,
580,
576,
573,
565,
551,
536,
522,
515,
518,
526,
533,
523,
521,
515,
504,
492,
478,
460,
461,
478,
490,
491,
499,
501,
494,
478,
455,
449,
451,
460,
464,
464,
457,
440,
431,
441,
449,
455,
452,
436,
428,
431,
435,
440,
442,
447,
451,
451,
448,
450,
454,
457,
460,
467,
477,
489,
492,
492,
491,
496,
504,
511,
515,
517,
522,
524,
534,
533,
514,
500,
509,
521,
530,
521,
504,
499,
512,
525,
536,
534,
536,
538,
536,
525,
524,
525,
531,
529,
528,
526,
526,
514,
498,
495,
500,
514,
523,
512,
509,
507,
499,
483,
464,
449,
449,
448,
446,
450,
460,
462,
460,
464,
469,
477,
485,
494,
502,
502,
503,
510,
518,
523,
534,
543,
552,
558,
568,
577,
588,
600,
604,
601,
596,
597,
596,
600,
598,
596,
610,
628,
642,
649,
656,
658,
658,
672,
686,
700,
706,
700,
697,
688,
677,
666,
657,
655,
650,
645,
636,
628,
619,
607,
596,
589,
580,
566,
551,
536,
520,
506,
494,
482,
476,
461,
450,
446,
442,
439,
430,
420,
412,
410,
410,
411,
412,
417,
424,
430,
426,
419,
415,
413,
412,
412,
416,
422,
427,
433,
446,
455,
457,
466,
474,
470,
465,
470,
471,
475,
482,
484,
485,
485,
486,
488,
490,
498,
505,
506,
500,
492,
481,
469,
460,
452,
450,
445,
439,
432,
425,
423,
418,
408,
400,
391,
380,
369,
362,
360,
360,
366,
380,
390,
396,
401,
402,
408,
414,
409,
406,
410,
405,
394,
382,
368,
353,
339,
341,
353,
361,
364,
370,
362,
355,
348,
339,
329,
321,
316,
312,
311,
311,
313,
315,
316,
317,
317,
317,
317,
317,
317,
317,
320,
324,
329,
328,
328,
329,
329,
327,
329,
327,
331,
340,
346,
352,
367,
381,
367,
362,
362,
357,
352,
343,
338,
338,
340,
345,
352,
367,
374,
378,
379,
377,
374,
367,
362,
365,
373,
386,
396,
398,
392,
387,
386,
391,
392,
387,
382,
382,
380,
376,
376,
378,
376,
383,
389,
397,
404,
408,
412,
420,
635,
628,
643,
664,
682,
696,
707,
719,
725,
717,
703,
687,
677,
668,
661,
652,
639,
625,
610,
595,
583,
572,
568,
568,
562,
560,
555,
548,
540,
532,
522,
508,
500,
510,
513,
506,
506,
515,
506,
496,
480,
458,
454,
475,
480,
481,
493,
500,
492,
482,
468,
448,
449,
455,
451,
453,
454,
440,
426,
441,
449,
454,
451,
437,
426,
426,
427,
434,
438,
442,
440,
437,
451,
461,
467,
472,
468,
467,
474,
486,
498,
504,
508,
511,
514,
516,
522,
527,
525,
516,
518,
521,
506,
496,
507,
519,
527,
512,
499,
499,
508,
521,
527,
510,
521,
532,
530,
513,
510,
511,
526,
538,
538,
532,
526,
519,
511,
508,
509,
519,
526,
526,
523,
504,
484,
476,
462,
449,
446,
446,
448,
458,
469,
469,
465,
471,
478,
488,
498,
507,
514,
518,
521,
528,
536,
538,
536,
538,
546,
552,
560,
573,
592,
610,
620,
618,
618,
617,
615,
615,
614,
610,
607,
620,
631,
640,
653,
666,
673,
679,
691,
706,
717,
712,
702,
692,
679,
669,
662,
656,
646,
632,
620,
613,
610,
606,
600,
590,
577,
560,
545,
531,
518,
505,
492,
483,
478,
468,
458,
446,
436,
427,
423,
418,
414,
414,
414,
412,
414,
421,
428,
434,
429,
424,
418,
415,
412,
414,
419,
426,
437,
445,
454,
466,
476,
484,
493,
490,
480,
472,
470,
470,
473,
474,
478,
482,
484,
487,
494,
502,
503,
504,
499,
490,
478,
467,
459,
450,
441,
436,
432,
426,
421,
415,
410,
401,
391,
385,
377,
369,
360,
356,
357,
366,
378,
386,
392,
397,
399,
403,
402,
396,
395,
400,
403,
394,
384,
372,
357,
340,
332,
342,
345,
356,
357,
349,
343,
338,
331,
323,
317,
313,
311,
310,
311,
313,
315,
316,
317,
317,
317,
317,
317,
317,
319,
327,
327,
328,
329,
329,
330,
327,
330,
329,
332,
341,
341,
344,
355,
364,
376,
369,
364,
358,
353,
346,
338,
335,
335,
337,
342,
349,
359,
364,
369,
367,
368,
360,
357,
351,
362,
372,
384,
391,
388,
383,
376,
375,
382,
381,
377,
373,
373,
369,
364,
366,
368,
378,
392,
396,
398,
404,
413,
415,
424,
651,
634,
644,
664,
682,
699,
714,
724,
720,
708,
697,
687,
680,
674,
669,
661,
649,
635,
619,
606,
591,
576,
557,
544,
539,
536,
532,
529,
525,
522,
518,
510,
498,
491,
495,
487,
500,
513,
504,
493,
478,
458,
451,
465,
464,
469,
482,
489,
486,
486,
476,
454,
437,
445,
438,
445,
451,
440,
424,
430,
438,
450,
438,
427,
425,
427,
429,
432,
436,
438,
436,
438,
446,
460,
469,
474,
476,
467,
477,
488,
497,
503,
508,
505,
500,
515,
528,
538,
526,
510,
510,
513,
497,
490,
505,
513,
518,
504,
496,
505,
514,
525,
516,
501,
515,
526,
524,
510,
511,
521,
531,
540,
539,
525,
516,
516,
512,
508,
513,
513,
508,
514,
520,
508,
486,
462,
451,
445,
445,
445,
452,
464,
476,
469,
469,
474,
482,
492,
503,
514,
524,
528,
534,
541,
550,
556,
557,
553,
558,
561,
561,
570,
588,
611,
623,
636,
637,
637,
634,
632,
631,
622,
612,
622,
632,
642,
654,
666,
679,
688,
697,
708,
715,
710,
700,
691,
683,
668,
659,
648,
634,
623,
612,
603,
596,
595,
593,
586,
574,
558,
543,
529,
518,
506,
493,
479,
471,
468,
462,
453,
441,
430,
424,
418,
415,
416,
415,
409,
414,
426,
435,
440,
434,
426,
420,
414,
413,
414,
421,
429,
438,
453,
465,
480,
493,
501,
511,
508,
496,
483,
472,
475,
479,
483,
485,
487,
491,
493,
497,
501,
502,
501,
495,
487,
477,
467,
459,
450,
439,
428,
425,
421,
417,
411,
406,
399,
392,
382,
374,
366,
355,
354,
354,
360,
371,
380,
391,
398,
397,
388,
387,
381,
384,
387,
390,
391,
381,
366,
352,
340,
327,
328,
339,
346,
345,
339,
332,
329,
326,
319,
314,
311,
310,
310,
312,
314,
316,
317,
317,
317,
317,
317,
317,
317,
321,
329,
328,
328,
329,
329,
330,
331,
329,
336,
341,
350,
356,
353,
351,
358,
363,
365,
363,
354,
345,
337,
329,
332,
333,
335,
339,
343,
347,
350,
353,
354,
353,
351,
348,
353,
363,
370,
377,
379,
376,
372,
364,
365,
370,
370,
367,
364,
363,
362,
360,
363,
373,
384,
395,
404,
406,
408,
415,
422,
427,
669,
651,
648,
672,
686,
699,
716,
725,
714,
698,
684,
677,
669,
666,
661,
657,
650,
638,
624,
610,
597,
580,
563,
541,
524,
514,
511,
511,
509,
500,
506,
511,
504,
488,
479,
480,
496,
505,
497,
488,
476,
458,
442,
452,
448,
456,
471,
478,
483,
482,
472,
450,
438,
437,
434,
444,
442,
433,
426,
436,
439,
443,
430,
425,
427,
429,
432,
439,
442,
446,
447,
455,
451,
451,
454,
456,
462,
463,
472,
481,
491,
495,
493,
493,
503,
519,
536,
538,
519,
499,
506,
506,
482,
487,
498,
507,
508,
495,
493,
504,
516,
516,
501,
497,
507,
514,
518,
509,
512,
523,
530,
537,
537,
527,
506,
498,
490,
488,
498,
499,
494,
500,
511,
511,
498,
479,
463,
442,
440,
446,
452,
465,
466,
464,
472,
478,
490,
500,
508,
515,
526,
536,
546,
554,
561,
567,
569,
570,
575,
579,
580,
577,
584,
604,
621,
634,
648,
654,
655,
652,
645,
635,
621,
632,
647,
656,
666,
675,
682,
694,
706,
713,
700,
691,
687,
681,
671,
661,
653,
644,
632,
622,
608,
595,
585,
580,
577,
574,
571,
562,
547,
536,
524,
512,
495,
481,
469,
460,
453,
446,
437,
426,
424,
420,
417,
417,
414,
413,
420,
430,
440,
443,
438,
428,
422,
417,
414,
415,
422,
431,
438,
452,
468,
487,
501,
514,
525,
524,
508,
494,
477,
476,
486,
491,
494,
497,
502,
502,
501,
504,
504,
498,
489,
484,
476,
469,
462,
454,
444,
434,
425,
414,
413,
407,
402,
396,
390,
382,
375,
363,
360,
355,
351,
354,
363,
374,
386,
399,
403,
390,
375,
368,
370,
372,
376,
382,
379,
365,
350,
336,
325,
317,
331,
336,
332,
330,
328,
326,
323,
317,
312,
311,
310,
310,
312,
314,
318,
317,
317,
317,
317,
317,
318,
319,
321,
324,
329,
329,
329,
332,
334,
329,
336,
344,
351,
352,
365,
362,
353,
354,
356,
358,
355,
346,
334,
332,
331,
333,
335,
337,
339,
340,
341,
344,
344,
344,
346,
345,
344,
348,
354,
363,
366,
367,
363,
360,
353,
356,
358,
358,
357,
356,
356,
357,
361,
371,
381,
391,
396,
403,
411,
417,
419,
426,
432,
677,
658,
660,
671,
684,
701,
720,
726,
710,
694,
681,
669,
657,
649,
646,
641,
637,
634,
626,
614,
601,
586,
570,
549,
534,
521,
504,
493,
495,
490,
494,
500,
505,
499,
484,
473,
481,
492,
489,
482,
473,
451,
437,
442,
450,
469,
480,
486,
486,
487,
475,
460,
449,
432,
431,
442,
434,
423,
430,
438,
436,
437,
425,
426,
428,
427,
432,
437,
448,
458,
462,
465,
461,
454,
455,
457,
465,
470,
475,
480,
485,
485,
485,
491,
506,
523,
538,
536,
515,
497,
501,
493,
484,
490,
498,
503,
496,
486,
492,
507,
519,
514,
497,
488,
496,
502,
508,
503,
507,
516,
520,
523,
528,
533,
518,
500,
484,
479,
480,
483,
489,
489,
494,
497,
499,
477,
456,
438,
442,
447,
451,
455,
460,
463,
472,
483,
494,
506,
515,
523,
534,
543,
554,
564,
570,
576,
580,
584,
590,
596,
600,
599,
594,
596,
609,
621,
637,
654,
667,
668,
661,
647,
634,
634,
654,
667,
679,
691,
697,
702,
714,
713,
697,
680,
668,
663,
656,
649,
644,
635,
625,
617,
608,
595,
583,
571,
561,
557,
558,
553,
543,
534,
525,
510,
491,
481,
469,
462,
450,
440,
434,
427,
423,
422,
419,
416,
414,
418,
424,
435,
445,
449,
440,
431,
426,
422,
419,
419,
426,
433,
438,
453,
465,
481,
494,
498,
511,
521,
513,
500,
483,
475,
489,
499,
502,
506,
509,
510,
509,
509,
505,
501,
494,
487,
482,
475,
467,
458,
449,
440,
430,
420,
414,
404,
400,
391,
387,
378,
377,
370,
365,
358,
353,
350,
353,
366,
378,
393,
405,
394,
375,
361,
358,
358,
363,
372,
372,
363,
351,
336,
323,
315,
321,
325,
324,
324,
325,
324,
321,
317,
313,
311,
310,
310,
312,
313,
315,
316,
316,
317,
317,
317,
319,
321,
323,
325,
327,
330,
332,
334,
336,
338,
342,
349,
358,
365,
366,
367,
356,
351,
349,
351,
345,
337,
332,
331,
335,
339,
343,
345,
345,
345,
345,
344,
345,
348,
344,
344,
345,
344,
347,
350,
353,
354,
354,
353,
353,
353,
354,
355,
355,
356,
358,
359,
360,
370,
378,
383,
388,
397,
409,
418,
426,
428,
436,
680,
671,
676,
684,
695,
704,
720,
730,
717,
701,
686,
674,
660,
647,
633,
627,
621,
619,
617,
613,
605,
590,
574,
556,
540,
524,
506,
491,
476,
481,
480,
489,
500,
504,
493,
478,
463,
473,
478,
472,
464,
444,
438,
451,
462,
476,
489,
492,
489,
481,
474,
466,
453,
437,
426,
434,
426,
427,
435,
438,
431,
433,
429,
426,
427,
430,
435,
441,
448,
456,
455,
454,
452,
450,
458,
466,
476,
479,
478,
478,
477,
484,
486,
491,
506,
522,
538,
534,
517,
494,
490,
487,
487,
495,
502,
495,
486,
486,
498,
514,
525,
505,
491,
481,
485,
490,
498,
498,
501,
504,
510,
511,
528,
539,
530,
514,
502,
490,
470,
471,
473,
479,
479,
485,
487,
471,
443,
439,
442,
447,
450,
454,
462,
468,
475,
487,
498,
510,
520,
536,
545,
551,
561,
570,
578,
584,
589,
594,
603,
612,
617,
618,
612,
605,
620,
627,
632,
652,
668,
679,
674,
659,
646,
638,
650,
667,
681,
695,
708,
713,
721,
711,
697,
680,
663,
648,
641,
636,
635,
632,
621,
609,
599,
591,
582,
572,
564,
548,
539,
534,
526,
522,
517,
509,
499,
489,
480,
473,
465,
454,
441,
428,
424,
423,
420,
418,
419,
426,
431,
438,
451,
452,
444,
433,
425,
421,
421,
423,
425,
426,
435,
447,
460,
468,
474,
481,
498,
508,
514,
504,
490,
479,
489,
500,
507,
514,
515,
516,
513,
510,
505,
501,
495,
490,
488,
479,
474,
462,
452,
442,
433,
424,
417,
412,
406,
400,
391,
383,
379,
370,
368,
361,
353,
348,
346,
354,
366,
383,
394,
386,
372,
359,
349,
349,
352,
361,
364,
356,
347,
334,
322,
317,
316,
317,
314,
319,
321,
321,
320,
317,
312,
310,
310,
311,
312,
313,
314,
315,
316,
318,
320,
321,
320,
322,
324,
326,
326,
327,
336,
337,
340,
343,
349,
357,
363,
370,
367,
362,
353,
347,
343,
340,
335,
331,
330,
332,
342,
348,
352,
359,
354,
350,
354,
356,
355,
354,
352,
351,
350,
345,
345,
347,
350,
351,
352,
354,
356,
358,
358,
358,
360,
364,
366,
367,
362,
364,
370,
379,
388,
393,
407,
420,
430,
434,
440,
691,
688,
692,
698,
708,
719,
725,
732,
720,
706,
690,
676,
666,
655,
648,
635,
620,
609,
608,
606,
599,
587,
570,
547,
530,
518,
504,
491,
475,
473,
473,
488,
500,
502,
493,
479,
456,
454,
462,
456,
452,
440,
438,
449,
461,
476,
488,
492,
483,
463,
456,
457,
451,
436,
424,
426,
424,
429,
428,
432,
430,
428,
429,
426,
435,
427,
427,
427,
436,
442,
438,
445,
448,
453,
457,
460,
466,
465,
461,
465,
476,
481,
485,
487,
501,
521,
536,
534,
520,
497,
484,
485,
488,
496,
500,
486,
485,
493,
503,
517,
514,
500,
483,
475,
486,
492,
499,
505,
505,
507,
515,
522,
535,
546,
542,
522,
508,
493,
472,
461,
463,
468,
469,
474,
478,
464,
438,
440,
444,
447,
452,
457,
464,
471,
480,
488,
501,
510,
524,
539,
551,
559,
567,
575,
584,
590,
596,
602,
611,
623,
634,
633,
622,
617,
623,
640,
647,
655,
670,
686,
684,
669,
652,
645,
652,
665,
677,
693,
707,
717,
714,
706,
696,
681,
664,
649,
631,
621,
621,
625,
624,
613,
604,
596,
587,
579,
570,
556,
540,
521,
511,
508,
506,
501,
492,
485,
479,
476,
466,
455,
439,
430,
428,
424,
421,
421,
423,
429,
437,
444,
452,
456,
445,
433,
425,
424,
425,
426,
427,
426,
434,
441,
448,
451,
459,
472,
489,
503,
514,
511,
499,
486,
487,
496,
504,
511,
520,
521,
517,
510,
503,
497,
489,
488,
487,
481,
475,
463,
451,
442,
437,
431,
426,
418,
410,
402,
392,
381,
375,
365,
363,
357,
353,
345,
343,
344,
355,
372,
387,
378,
370,
360,
350,
340,
344,
349,
354,
346,
337,
328,
319,
315,
313,
313,
314,
316,
314,
313,
313,
312,
311,
310,
310,
312,
312,
313,
315,
317,
319,
322,
326,
327,
324,
324,
325,
327,
328,
330,
331,
340,
341,
350,
357,
367,
369,
368,
362,
354,
347,
340,
333,
331,
329,
326,
329,
339,
348,
360,
368,
371,
361,
357,
370,
373,
371,
366,
371,
367,
358,
350,
349,
352,
355,
356,
355,
360,
366,
369,
368,
365,
370,
378,
380,
376,
365,
368,
370,
377,
383,
390,
402,
417,
428,
435,
439,
702,
706,
710,
714,
725,
732,
728,
719,
712,
705,
696,
683,
674,
666,
656,
642,
626,
614,
608,
605,
601,
591,
571,
550,
531,
514,
495,
481,
474,
465,
470,
482,
490,
492,
486,
474,
458,
447,
446,
443,
439,
435,
430,
438,
451,
467,
480,
488,
482,
462,
439,
439,
439,
425,
418,
423,
422,
423,
416,
426,
427,
427,
425,
428,
435,
426,
427,
429,
430,
436,
441,
442,
450,
451,
447,
450,
452,
457,
462,
474,
481,
487,
489,
493,
502,
520,
532,
536,
522,
501,
484,
485,
488,
493,
491,
484,
488,
503,
515,
518,
511,
495,
478,
478,
491,
503,
513,
520,
522,
522,
530,
540,
547,
556,
548,
528,
508,
486,
470,
458,
453,
457,
461,
471,
476,
456,
438,
441,
446,
448,
451,
456,
463,
470,
479,
490,
499,
510,
522,
530,
544,
560,
568,
577,
585,
592,
599,
608,
620,
632,
648,
642,
630,
630,
636,
647,
662,
669,
674,
694,
691,
674,
660,
658,
658,
673,
689,
701,
711,
718,
703,
694,
684,
672,
659,
643,
632,
616,
607,
616,
618,
612,
602,
595,
587,
577,
570,
560,
544,
526,
507,
497,
494,
492,
485,
474,
466,
462,
456,
444,
436,
437,
429,
425,
423,
424,
424,
429,
439,
448,
454,
455,
443,
432,
425,
427,
428,
434,
440,
438,
438,
437,
447,
458,
467,
478,
496,
509,
518,
518,
507,
492,
484,
492,
501,
512,
522,
526,
520,
510,
501,
492,
485,
479,
479,
479,
474,
466,
456,
448,
442,
434,
426,
417,
409,
400,
390,
386,
380,
374,
366,
360,
352,
345,
340,
337,
344,
361,
376,
382,
377,
366,
356,
340,
334,
338,
338,
336,
329,
321,
314,
312,
311,
311,
311,
311,
311,
310,
310,
310,
310,
311,
312,
314,
314,
312,
316,
320,
326,
328,
331,
329,
326,
326,
326,
328,
331,
341,
341,
342,
350,
354,
367,
369,
364,
358,
352,
346,
339,
332,
329,
327,
325,
331,
339,
346,
356,
366,
367,
365,
364,
364,
377,
380,
382,
381,
381,
378,
369,
362,
364,
367,
368,
367,
367,
373,
378,
379,
380,
379,
385,
390,
390,
381,
373,
378,
379,
378,
387,
393,
404,
418,
429,
432,
431,
718,
722,
729,
734,
740,
742,
732,
716,
702,
694,
687,
682,
674,
666,
657,
644,
631,
620,
614,
611,
609,
594,
575,
555,
537,
519,
500,
475,
467,
464,
463,
469,
474,
477,
474,
466,
456,
444,
437,
436,
431,
430,
426,
434,
448,
460,
462,
476,
481,
471,
448,
426,
424,
414,
416,
416,
419,
415,
417,
423,
427,
426,
426,
426,
428,
427,
421,
429,
425,
427,
436,
439,
444,
448,
452,
459,
463,
471,
475,
478,
486,
492,
498,
506,
518,
527,
524,
524,
523,
504,
484,
481,
484,
487,
485,
486,
493,
507,
516,
512,
500,
486,
473,
481,
494,
506,
520,
527,
529,
531,
536,
542,
549,
560,
552,
535,
516,
495,
479,
463,
450,
449,
458,
467,
466,
451,
438,
441,
449,
453,
453,
454,
461,
464,
473,
481,
490,
499,
505,
516,
535,
550,
562,
572,
582,
590,
600,
610,
620,
632,
639,
641,
642,
645,
653,
659,
672,
682,
689,
696,
693,
680,
670,
674,
676,
681,
700,
712,
712,
707,
694,
680,
668,
656,
643,
632,
621,
609,
595,
600,
600,
594,
588,
582,
579,
574,
563,
549,
537,
523,
508,
493,
484,
480,
475,
468,
459,
450,
447,
442,
445,
443,
435,
427,
426,
426,
428,
433,
441,
451,
456,
453,
440,
431,
427,
432,
436,
444,
451,
456,
456,
451,
453,
468,
481,
490,
499,
514,
524,
524,
514,
499,
489,
498,
506,
514,
522,
526,
519,
509,
498,
489,
479,
472,
469,
472,
474,
465,
459,
449,
442,
434,
426,
417,
410,
401,
396,
393,
390,
382,
375,
367,
357,
345,
332,
334,
336,
347,
363,
372,
374,
368,
356,
344,
334,
328,
327,
326,
321,
314,
312,
310,
309,
310,
310,
310,
310,
310,
311,
312,
313,
313,
317,
318,
316,
318,
317,
324,
332,
341,
332,
327,
328,
328,
328,
328,
335,
349,
353,
352,
356,
364,
369,
366,
356,
347,
343,
337,
332,
328,
326,
326,
327,
331,
338,
343,
350,
355,
353,
352,
358,
358,
357,
364,
373,
377,
376,
370,
367,
367,
369,
365,
366,
366,
369,
374,
378,
381,
384,
388,
391,
395,
393,
384,
377,
384,
388,
387,
388,
397,
410,
422,
426,
426,
424,
738,
738,
744,
747,
748,
744,
733,
717,
702,
684,
674,
667,
659,
651,
642,
632,
627,
623,
613,
610,
603,
592,
575,
555,
539,
522,
503,
482,
463,
458,
452,
453,
459,
462,
460,
456,
450,
447,
449,
442,
430,
424,
424,
434,
442,
444,
446,
459,
472,
473,
452,
433,
418,
414,
414,
419,
417,
414,
415,
421,
423,
425,
425,
428,
426,
426,
429,
437,
434,
440,
448,
452,
447,
450,
456,
464,
467,
473,
477,
477,
478,
491,
506,
518,
527,
524,
511,
517,
524,
503,
483,
476,
480,
483,
487,
495,
503,
514,
509,
501,
488,
475,
475,
490,
504,
514,
526,
527,
514,
519,
521,
528,
534,
543,
551,
541,
525,
504,
487,
468,
452,
447,
453,
464,
465,
442,
436,
438,
446,
450,
454,
464,
470,
472,
474,
477,
487,
499,
506,
513,
521,
536,
552,
564,
576,
588,
596,
604,
614,
622,
629,
636,
644,
653,
664,
672,
684,
694,
701,
699,
696,
692,
684,
691,
692,
696,
706,
711,
709,
700,
685,
668,
656,
643,
631,
620,
609,
596,
587,
584,
581,
577,
572,
569,
565,
570,
562,
547,
530,
515,
502,
491,
480,
471,
464,
460,
450,
450,
450,
456,
458,
450,
439,
433,
428,
432,
438,
440,
447,
453,
454,
452,
438,
432,
432,
437,
444,
454,
464,
472,
474,
468,
463,
471,
482,
498,
509,
520,
528,
529,
520,
507,
496,
501,
511,
516,
517,
517,
514,
507,
498,
490,
478,
467,
460,
461,
465,
468,
461,
453,
445,
436,
426,
418,
412,
406,
403,
398,
392,
388,
378,
370,
362,
352,
340,
333,
331,
337,
348,
358,
364,
363,
355,
342,
332,
323,
319,
317,
315,
312,
310,
308,
308,
309,
310,
310,
310,
311,
313,
318,
319,
317,
316,
317,
320,
331,
327,
322,
331,
332,
337,
337,
335,
331,
336,
335,
339,
347,
355,
357,
361,
364,
362,
356,
347,
338,
333,
329,
327,
325,
325,
326,
327,
329,
332,
335,
338,
340,
340,
341,
343,
345,
347,
351,
356,
360,
361,
362,
361,
356,
359,
362,
363,
365,
367,
369,
374,
378,
385,
388,
396,
400,
395,
389,
389,
394,
396,
396,
396,
401,
414,
422,
420,
418,
416,
751,
749,
749,
748,
742,
732,
722,
712,
703,
687,
667,
647,
637,
633,
624,
616,
615,
611,
603,
596,
587,
580,
568,
551,
538,
522,
504,
483,
462,
465,
461,
451,
444,
442,
445,
444,
438,
444,
452,
453,
443,
432,
424,
426,
432,
429,
426,
446,
462,
466,
452,
436,
418,
415,
414,
417,
420,
414,
417,
420,
422,
424,
427,
430,
437,
439,
436,
443,
451,
451,
454,
460,
453,
448,
449,
451,
453,
459,
461,
467,
481,
495,
512,
525,
524,
514,
500,
514,
517,
502,
479,
476,
489,
478,
484,
495,
502,
502,
500,
486,
475,
473,
475,
488,
502,
516,
524,
508,
496,
499,
503,
508,
514,
529,
543,
540,
527,
511,
495,
481,
467,
455,
447,
455,
461,
438,
436,
441,
449,
455,
461,
462,
478,
484,
487,
492,
500,
510,
522,
535,
536,
535,
536,
550,
564,
575,
586,
593,
602,
608,
616,
624,
636,
648,
661,
671,
680,
691,
696,
695,
700,
703,
703,
706,
709,
714,
713,
711,
707,
698,
690,
676,
660,
645,
632,
618,
608,
600,
593,
584,
573,
565,
559,
554,
551,
557,
556,
547,
533,
518,
501,
488,
478,
470,
463,
464,
464,
465,
468,
471,
466,
454,
441,
439,
436,
438,
445,
452,
451,
454,
449,
444,
438,
435,
436,
441,
455,
465,
475,
488,
490,
479,
475,
482,
485,
497,
512,
525,
530,
534,
526,
513,
502,
507,
514,
515,
513,
509,
505,
502,
495,
489,
478,
467,
456,
451,
457,
463,
462,
454,
446,
437,
427,
420,
416,
409,
404,
399,
392,
390,
380,
375,
366,
353,
340,
329,
329,
329,
337,
346,
352,
353,
345,
337,
329,
320,
315,
314,
312,
310,
308,
308,
308,
309,
310,
311,
312,
313,
318,
321,
323,
322,
318,
317,
319,
326,
326,
328,
326,
329,
339,
347,
351,
348,
347,
347,
338,
342,
352,
357,
356,
355,
350,
343,
335,
329,
325,
323,
324,
325,
326,
328,
329,
329,
329,
330,
332,
334,
337,
338,
339,
341,
343,
345,
348,
353,
359,
362,
363,
365,
367,
368,
366,
364,
363,
365,
372,
380,
387,
393,
400,
404,
396,
388,
393,
402,
405,
406,
407,
412,
416,
415,
412,
410,
406,
756,
752,
747,
740,
727,
714,
703,
697,
694,
687,
669,
648,
628,
610,
601,
597,
598,
594,
585,
577,
568,
563,
558,
546,
532,
517,
500,
479,
463,
478,
471,
459,
449,
438,
434,
437,
435,
437,
442,
448,
451,
444,
430,
423,
425,
426,
432,
442,
455,
454,
446,
432,
416,
414,
414,
415,
414,
415,
418,
420,
423,
424,
425,
426,
434,
445,
451,
445,
452,
464,
463,
465,
454,
451,
452,
451,
463,
469,
468,
474,
486,
498,
516,
528,
518,
500,
499,
513,
515,
502,
488,
476,
493,
482,
474,
486,
490,
488,
480,
471,
475,
488,
490,
490,
499,
513,
517,
495,
485,
490,
498,
503,
510,
524,
539,
530,
517,
507,
500,
489,
478,
464,
449,
443,
451,
437,
436,
442,
450,
459,
468,
470,
478,
489,
502,
509,
516,
527,
542,
550,
552,
552,
552,
554,
557,
564,
575,
582,
591,
595,
604,
614,
624,
634,
644,
656,
666,
675,
680,
691,
704,
711,
716,
721,
723,
721,
714,
709,
704,
700,
695,
684,
671,
654,
640,
628,
618,
608,
602,
595,
588,
577,
565,
553,
545,
542,
541,
537,
528,
517,
502,
487,
477,
474,
474,
477,
479,
480,
484,
482,
467,
458,
454,
449,
437,
443,
452,
456,
455,
452,
446,
439,
435,
437,
446,
455,
461,
474,
486,
499,
501,
491,
486,
495,
496,
500,
515,
529,
535,
537,
529,
516,
510,
514,
517,
518,
512,
503,
497,
490,
484,
478,
472,
465,
454,
441,
447,
453,
454,
452,
443,
434,
426,
419,
414,
406,
399,
392,
385,
379,
370,
363,
353,
344,
337,
333,
325,
325,
327,
336,
343,
343,
336,
330,
323,
316,
312,
311,
309,
308,
307,
307,
308,
309,
310,
312,
317,
317,
319,
326,
329,
332,
324,
319,
319,
324,
326,
327,
328,
331,
340,
352,
367,
366,
362,
352,
340,
343,
351,
349,
346,
341,
337,
329,
322,
322,
316,
319,
325,
328,
329,
333,
334,
333,
330,
330,
332,
336,
342,
345,
346,
347,
349,
348,
347,
354,
366,
370,
373,
378,
378,
371,
376,
373,
369,
367,
368,
378,
386,
396,
404,
405,
402,
394,
396,
408,
413,
416,
416,
418,
415,
409,
402,
400,
394,
751,
746,
738,
730,
717,
703,
688,
679,
680,
681,
669,
650,
629,
610,
595,
581,
573,
571,
564,
557,
550,
549,
547,
536,
524,
511,
493,
472,
477,
486,
481,
471,
460,
449,
438,
428,
428,
429,
434,
437,
442,
444,
434,
421,
418,
429,
441,
451,
452,
446,
437,
423,
412,
413,
413,
414,
415,
417,
420,
424,
424,
426,
431,
429,
433,
448,
460,
454,
466,
469,
464,
456,
447,
448,
456,
458,
463,
476,
479,
476,
492,
506,
523,
524,
508,
491,
500,
513,
517,
499,
475,
476,
491,
482,
463,
473,
475,
469,
460,
474,
487,
498,
503,
504,
508,
517,
515,
493,
479,
496,
507,
518,
525,
534,
538,
523,
508,
492,
484,
479,
473,
465,
451,
436,
440,
435,
437,
442,
452,
463,
476,
478,
482,
492,
505,
518,
530,
542,
556,
567,
570,
571,
572,
574,
575,
582,
592,
599,
606,
607,
613,
617,
619,
628,
636,
643,
654,
669,
686,
701,
713,
722,
728,
733,
729,
719,
708,
701,
694,
698,
704,
689,
674,
659,
649,
640,
631,
622,
614,
610,
598,
587,
573,
562,
546,
535,
525,
521,
516,
508,
492,
484,
484,
487,
489,
492,
491,
500,
498,
488,
478,
472,
465,
454,
446,
451,
459,
460,
451,
448,
441,
437,
436,
444,
456,
467,
474,
475,
491,
505,
511,
498,
494,
503,
510,
514,
524,
533,
540,
538,
531,
522,
518,
520,
523,
516,
510,
501,
492,
481,
472,
466,
463,
457,
449,
439,
436,
442,
444,
443,
438,
431,
423,
416,
409,
401,
390,
382,
376,
372,
364,
358,
353,
350,
345,
342,
333,
325,
321,
327,
335,
332,
327,
322,
316,
314,
311,
307,
306,
305,
306,
307,
308,
310,
312,
313,
317,
319,
325,
337,
342,
334,
327,
327,
324,
325,
327,
329,
334,
341,
353,
361,
376,
377,
362,
352,
343,
345,
342,
338,
332,
327,
323,
316,
316,
317,
319,
324,
330,
337,
339,
342,
343,
342,
336,
332,
336,
350,
355,
360,
359,
360,
359,
356,
352,
355,
365,
374,
380,
386,
387,
384,
388,
384,
381,
376,
372,
375,
383,
395,
403,
403,
401,
402,
403,
409,
415,
419,
418,
415,
409,
402,
394,
389,
388,
748,
739,
725,
717,
709,
696,
679,
666,
661,
663,
659,
648,
630,
611,
593,
578,
554,
546,
543,
537,
531,
532,
530,
524,
515,
501,
484,
476,
494,
496,
501,
483,
473,
462,
446,
435,
426,
424,
426,
427,
432,
437,
432,
419,
415,
424,
434,
442,
441,
433,
427,
416,
412,
412,
414,
416,
417,
420,
422,
426,
428,
432,
442,
447,
441,
455,
472,
475,
478,
463,
452,
443,
439,
449,
462,
468,
463,
476,
484,
487,
499,
516,
527,
515,
497,
486,
498,
511,
514,
494,
475,
476,
491,
486,
459,
452,
452,
460,
472,
486,
497,
508,
516,
522,
521,
524,
514,
494,
471,
489,
497,
509,
524,
535,
540,
528,
512,
499,
481,
464,
463,
458,
448,
436,
433,
436,
435,
441,
452,
464,
476,
484,
488,
496,
503,
515,
527,
542,
559,
575,
587,
592,
594,
595,
596,
599,
608,
618,
622,
625,
628,
632,
632,
634,
643,
655,
665,
679,
693,
708,
722,
732,
735,
733,
723,
713,
701,
693,
689,
694,
706,
694,
675,
664,
654,
647,
637,
632,
626,
618,
605,
592,
576,
565,
553,
542,
529,
516,
508,
498,
489,
490,
492,
484,
500,
499,
495,
499,
505,
497,
486,
475,
463,
453,
448,
454,
463,
455,
448,
441,
437,
437,
442,
449,
464,
477,
488,
480,
486,
501,
515,
507,
499,
506,
516,
524,
531,
539,
542,
546,
534,
524,
524,
525,
526,
518,
509,
499,
491,
481,
471,
460,
451,
448,
438,
429,
420,
428,
431,
432,
430,
425,
419,
416,
406,
398,
390,
383,
378,
370,
367,
359,
354,
351,
343,
343,
335,
323,
320,
321,
325,
324,
320,
315,
313,
311,
309,
306,
304,
305,
307,
308,
310,
311,
313,
316,
319,
324,
322,
327,
328,
331,
332,
331,
328,
329,
332,
336,
344,
353,
365,
374,
380,
370,
358,
349,
343,
336,
327,
322,
319,
316,
315,
316,
318,
322,
324,
328,
339,
346,
350,
351,
358,
352,
345,
337,
337,
344,
357,
365,
368,
375,
375,
364,
358,
358,
364,
373,
382,
391,
393,
394,
396,
393,
391,
385,
380,
374,
378,
391,
397,
399,
399,
404,
408,
414,
417,
417,
413,
406,
400,
394,
386,
378,
382,
744,
729,
714,
700,
695,
685,
673,
659,
642,
643,
646,
636,
627,
613,
594,
578,
562,
547,
535,
524,
514,
514,
515,
513,
505,
492,
484,
499,
509,
514,
514,
500,
488,
472,
459,
451,
438,
428,
424,
421,
423,
423,
422,
417,
414,
418,
426,
428,
428,
424,
419,
413,
412,
412,
414,
416,
419,
422,
423,
426,
436,
444,
456,
467,
467,
467,
477,
476,
471,
459,
444,
436,
443,
450,
465,
478,
479,
484,
492,
498,
510,
524,
522,
504,
481,
490,
499,
507,
503,
486,
466,
473,
491,
489,
460,
446,
451,
468,
488,
496,
508,
520,
527,
528,
530,
526,
515,
495,
473,
474,
477,
498,
511,
516,
533,
538,
519,
506,
494,
477,
465,
454,
450,
449,
438,
434,
435,
440,
449,
461,
474,
485,
494,
503,
511,
519,
531,
545,
562,
576,
593,
604,
611,
615,
622,
622,
625,
630,
634,
637,
642,
644,
647,
646,
648,
661,
678,
692,
706,
719,
731,
737,
732,
724,
715,
707,
699,
690,
682,
688,
697,
691,
675,
658,
646,
638,
629,
622,
618,
610,
603,
593,
582,
570,
562,
550,
540,
528,
515,
505,
501,
500,
490,
487,
486,
493,
494,
495,
492,
484,
476,
464,
453,
444,
445,
454,
459,
454,
445,
438,
437,
441,
448,
456,
467,
479,
495,
493,
498,
507,
515,
521,
517,
511,
513,
522,
531,
540,
548,
549,
539,
533,
530,
528,
526,
519,
510,
500,
490,
482,
473,
465,
455,
445,
434,
423,
413,
411,
414,
417,
414,
411,
411,
411,
405,
400,
392,
386,
378,
371,
366,
362,
354,
347,
340,
337,
333,
325,
319,
317,
319,
318,
314,
311,
309,
308,
307,
306,
304,
305,
307,
310,
311,
313,
314,
316,
326,
329,
328,
328,
325,
331,
340,
338,
333,
339,
344,
346,
353,
364,
377,
379,
370,
359,
349,
342,
333,
324,
316,
315,
316,
317,
317,
318,
322,
326,
329,
334,
344,
356,
362,
362,
367,
366,
356,
347,
340,
341,
343,
351,
360,
369,
372,
370,
366,
363,
365,
366,
376,
390,
397,
404,
405,
403,
399,
398,
393,
383,
376,
382,
390,
390,
398,
403,
407,
411,
413,
411,
406,
399,
388,
383,
375,
375,
380,
734,
720,
704,
685,
674,
669,
662,
653,
637,
624,
628,
621,
610,
606,
595,
584,
572,
561,
550,
537,
524,
509,
498,
500,
494,
485,
489,
502,
515,
524,
522,
513,
504,
486,
476,
464,
454,
448,
438,
425,
421,
417,
413,
414,
411,
416,
422,
420,
419,
417,
414,
412,
412,
412,
414,
416,
421,
424,
427,
431,
436,
447,
463,
475,
471,
477,
470,
464,
460,
448,
438,
439,
448,
459,
470,
484,
492,
496,
504,
514,
520,
526,
515,
490,
473,
489,
500,
500,
499,
480,
460,
469,
489,
489,
459,
445,
453,
469,
488,
502,
518,
527,
517,
513,
519,
517,
512,
498,
478,
462,
478,
491,
497,
512,
527,
526,
518,
507,
497,
485,
479,
474,
473,
460,
444,
429,
434,
436,
447,
460,
472,
483,
496,
507,
515,
524,
533,
547,
562,
576,
592,
603,
611,
617,
621,
627,
633,
638,
644,
650,
656,
658,
662,
662,
666,
668,
684,
701,
718,
730,
735,
733,
725,
715,
705,
695,
693,
687,
679,
681,
684,
684,
670,
654,
639,
626,
616,
608,
604,
600,
594,
586,
577,
568,
557,
548,
539,
530,
521,
513,
504,
498,
488,
476,
472,
474,
474,
477,
474,
466,
466,
454,
443,
438,
444,
453,
454,
450,
440,
437,
439,
444,
451,
462,
478,
490,
501,
509,
510,
518,
523,
529,
527,
521,
520,
522,
532,
537,
541,
543,
538,
534,
529,
528,
524,
517,
510,
501,
490,
483,
476,
468,
460,
449,
437,
425,
415,
401,
401,
402,
400,
396,
395,
396,
394,
391,
391,
388,
385,
381,
377,
366,
354,
344,
337,
331,
328,
321,
316,
315,
315,
313,
309,
306,
304,
306,
305,
304,
305,
307,
309,
312,
314,
315,
316,
318,
321,
324,
328,
327,
327,
328,
333,
346,
346,
354,
359,
360,
362,
369,
372,
365,
357,
346,
335,
326,
320,
315,
315,
317,
317,
319,
320,
321,
325,
329,
335,
343,
351,
361,
371,
376,
377,
375,
368,
356,
347,
342,
342,
343,
352,
363,
372,
377,
374,
372,
366,
373,
377,
384,
393,
402,
412,
413,
411,
410,
403,
393,
383,
379,
386,
390,
395,
400,
404,
406,
406,
403,
399,
393,
383,
373,
367,
372,
380,
722,
706,
689,
672,
657,
644,
644,
644,
635,
620,
612,
611,
590,
583,
576,
574,
573,
566,
561,
550,
538,
522,
506,
496,
487,
494,
503,
504,
510,
515,
519,
515,
510,
496,
483,
473,
467,
464,
445,
428,
424,
422,
418,
411,
413,
414,
418,
418,
416,
414,
412,
412,
411,
412,
415,
419,
423,
427,
427,
438,
452,
460,
466,
476,
465,
466,
458,
454,
450,
437,
438,
438,
448,
461,
473,
483,
494,
511,
512,
503,
507,
519,
514,
486,
468,
475,
485,
495,
492,
477,
458,
468,
490,
480,
456,
442,
450,
464,
483,
500,
523,
519,
503,
504,
505,
494,
497,
491,
477,
460,
476,
487,
502,
516,
510,
508,
505,
493,
485,
479,
480,
489,
482,
468,
450,
433,
431,
435,
447,
460,
472,
483,
494,
507,
515,
523,
532,
544,
558,
574,
590,
600,
606,
609,
616,
621,
632,
639,
648,
658,
668,
673,
677,
682,
686,
690,
695,
710,
725,
728,
721,
719,
715,
707,
695,
685,
677,
679,
670,
672,
680,
680,
672,
656,
642,
627,
615,
606,
597,
592,
586,
581,
574,
564,
551,
535,
526,
519,
510,
502,
494,
488,
484,
478,
466,
460,
457,
460,
458,
454,
451,
446,
437,
436,
440,
451,
448,
441,
436,
436,
440,
446,
453,
465,
482,
501,
510,
522,
522,
528,
535,
538,
534,
526,
525,
527,
527,
526,
527,
533,
532,
531,
526,
524,
518,
511,
505,
499,
493,
488,
478,
470,
461,
450,
436,
424,
416,
404,
392,
386,
382,
380,
378,
380,
378,
377,
376,
377,
378,
380,
379,
369,
357,
344,
332,
327,
320,
316,
315,
313,
313,
310,
307,
304,
304,
303,
304,
305,
307,
310,
312,
315,
317,
319,
319,
322,
326,
328,
332,
333,
332,
338,
343,
357,
362,
370,
374,
375,
369,
364,
359,
353,
345,
334,
321,
314,
312,
312,
316,
318,
320,
321,
322,
324,
327,
331,
340,
352,
363,
367,
378,
385,
387,
384,
377,
363,
356,
349,
344,
347,
355,
363,
373,
382,
386,
381,
377,
382,
387,
391,
399,
407,
414,
413,
415,
414,
406,
398,
387,
385,
384,
388,
391,
397,
402,
402,
400,
396,
392,
387,
380,
368,
364,
370,
378,
717,
698,
678,
659,
639,
625,
621,
624,
626,
621,
606,
596,
585,
578,
560,
548,
550,
552,
553,
553,
547,
532,
515,
504,
512,
510,
515,
519,
515,
511,
514,
513,
504,
490,
478,
466,
459,
462,
452,
440,
428,
430,
428,
415,
411,
413,
415,
417,
414,
412,
412,
411,
412,
413,
416,
421,
425,
432,
439,
438,
452,
462,
463,
459,
456,
454,
447,
437,
436,
431,
434,
441,
442,
449,
460,
476,
487,
498,
501,
490,
502,
510,
512,
485,
462,
466,
475,
487,
488,
471,
458,
466,
479,
472,
451,
438,
452,
469,
486,
502,
515,
509,
492,
498,
494,
481,
476,
477,
463,
468,
486,
500,
509,
504,
485,
488,
492,
481,
472,
470,
476,
480,
476,
468,
453,
444,
434,
433,
442,
455,
466,
480,
491,
502,
513,
519,
528,
539,
552,
568,
580,
591,
599,
604,
613,
624,
629,
635,
644,
652,
664,
676,
686,
695,
702,
708,
714,
721,
726,
722,
706,
704,
703,
699,
688,
670,
664,
664,
658,
660,
668,
672,
667,
657,
646,
636,
625,
614,
603,
590,
580,
573,
568,
563,
553,
543,
534,
522,
507,
497,
483,
474,
473,
470,
465,
452,
446,
446,
444,
443,
441,
439,
436,
436,
443,
444,
440,
437,
434,
436,
439,
446,
453,
465,
482,
498,
514,
526,
536,
540,
546,
542,
536,
530,
526,
526,
522,
520,
521,
525,
526,
524,
525,
525,
518,
510,
504,
498,
490,
486,
478,
469,
458,
446,
435,
425,
416,
406,
395,
386,
376,
368,
365,
365,
362,
358,
360,
362,
364,
368,
376,
370,
357,
345,
332,
322,
316,
314,
313,
312,
310,
307,
305,
303,
305,
305,
306,
308,
310,
312,
315,
318,
323,
328,
326,
326,
331,
337,
343,
344,
344,
351,
362,
372,
378,
382,
378,
369,
359,
350,
345,
338,
330,
322,
314,
308,
312,
314,
316,
317,
319,
320,
324,
326,
328,
329,
339,
353,
363,
368,
373,
380,
391,
391,
383,
376,
367,
356,
350,
351,
358,
365,
374,
384,
393,
394,
390,
391,
398,
402,
405,
414,
412,
409,
414,
410,
404,
399,
393,
390,
387,
388,
390,
395,
399,
397,
393,
388,
384,
379,
375,
365,
366,
367,
374,
712,
697,
679,
660,
642,
621,
605,
604,
612,
615,
609,
602,
598,
588,
567,
547,
534,
531,
536,
541,
545,
538,
524,
516,
526,
532,
530,
530,
526,
522,
515,
508,
500,
488,
479,
463,
451,
451,
451,
443,
436,
438,
432,
421,
411,
414,
414,
414,
414,
413,
412,
411,
412,
413,
415,
424,
428,
435,
447,
456,
457,
455,
449,
447,
443,
441,
435,
430,
426,
429,
433,
434,
438,
442,
448,
464,
478,
490,
489,
479,
491,
497,
503,
483,
463,
461,
471,
481,
484,
471,
452,
460,
472,
463,
451,
444,
452,
468,
484,
493,
495,
490,
483,
499,
492,
478,
465,
461,
460,
477,
491,
503,
490,
484,
479,
473,
476,
470,
462,
461,
464,
462,
460,
457,
462,
446,
430,
431,
438,
450,
460,
472,
485,
496,
506,
514,
521,
530,
541,
555,
568,
577,
585,
596,
611,
614,
615,
628,
635,
642,
655,
666,
677,
688,
699,
709,
721,
731,
724,
714,
699,
686,
689,
690,
685,
674,
657,
646,
640,
647,
656,
661,
658,
651,
647,
638,
629,
619,
607,
594,
580,
569,
561,
556,
549,
544,
547,
529,
515,
501,
485,
471,
462,
458,
454,
448,
438,
437,
436,
435,
436,
434,
430,
434,
440,
438,
435,
434,
434,
435,
439,
446,
454,
466,
479,
500,
516,
527,
539,
549,
546,
537,
530,
525,
521,
519,
516,
513,
513,
516,
517,
520,
524,
526,
522,
515,
507,
500,
488,
475,
471,
465,
455,
444,
434,
425,
418,
409,
399,
388,
381,
373,
365,
358,
352,
345,
340,
341,
348,
356,
363,
365,
354,
342,
332,
319,
315,
313,
311,
309,
308,
306,
304,
303,
303,
306,
308,
311,
313,
315,
317,
324,
331,
336,
335,
332,
339,
348,
358,
364,
364,
367,
377,
382,
375,
365,
361,
356,
347,
336,
331,
324,
319,
314,
310,
310,
312,
314,
315,
316,
317,
321,
326,
328,
333,
338,
340,
343,
352,
354,
361,
370,
384,
394,
392,
384,
372,
358,
356,
357,
364,
371,
379,
386,
394,
403,
400,
400,
405,
412,
414,
412,
408,
405,
405,
403,
406,
403,
398,
392,
390,
390,
391,
395,
395,
394,
390,
383,
377,
374,
368,
365,
365,
365,
368,
707,
691,
674,
656,
635,
620,
603,
591,
595,
606,
610,
601,
597,
582,
564,
548,
536,
524,
523,
530,
537,
542,
536,
529,
536,
541,
538,
533,
526,
520,
514,
504,
496,
488,
479,
468,
457,
446,
438,
439,
437,
436,
434,
430,
419,
411,
412,
412,
412,
412,
411,
412,
414,
415,
421,
429,
441,
450,
453,
453,
452,
448,
437,
435,
434,
429,
424,
425,
427,
430,
434,
439,
447,
455,
456,
459,
470,
482,
487,
473,
476,
491,
502,
482,
463,
460,
466,
474,
480,
472,
455,
453,
466,
459,
449,
443,
448,
463,
473,
474,
478,
476,
479,
500,
490,
478,
461,
462,
471,
478,
485,
492,
479,
469,
466,
466,
464,
457,
454,
451,
453,
454,
459,
463,
463,
439,
425,
430,
437,
446,
455,
462,
470,
483,
492,
501,
509,
520,
531,
542,
553,
561,
570,
586,
596,
596,
606,
618,
624,
633,
640,
649,
661,
676,
692,
710,
723,
730,
718,
702,
688,
674,
673,
676,
673,
671,
658,
640,
628,
634,
645,
650,
645,
643,
640,
633,
620,
611,
605,
595,
581,
568,
558,
549,
539,
535,
536,
528,
514,
498,
485,
472,
459,
450,
446,
440,
436,
430,
428,
428,
429,
430,
427,
429,
430,
432,
430,
431,
433,
436,
438,
444,
455,
471,
487,
504,
520,
528,
533,
530,
525,
523,
522,
518,
513,
512,
509,
507,
507,
508,
510,
513,
522,
528,
526,
518,
508,
497,
486,
475,
462,
456,
453,
446,
440,
433,
425,
415,
406,
397,
391,
385,
377,
367,
357,
349,
340,
329,
330,
342,
353,
354,
346,
337,
328,
317,
313,
310,
308,
308,
308,
304,
302,
302,
303,
306,
311,
314,
316,
319,
326,
331,
341,
349,
348,
344,
350,
356,
362,
362,
367,
366,
370,
364,
358,
350,
347,
343,
333,
320,
316,
313,
311,
311,
310,
310,
312,
314,
316,
318,
320,
324,
328,
333,
341,
352,
355,
349,
352,
361,
366,
368,
378,
388,
394,
388,
379,
367,
356,
365,
371,
377,
386,
392,
398,
404,
406,
408,
413,
414,
415,
407,
405,
400,
398,
398,
403,
404,
402,
400,
396,
394,
393,
393,
391,
390,
386,
380,
372,
367,
362,
356,
355,
361,
364,
706,
687,
668,
649,
630,
613,
598,
584,
586,
594,
596,
591,
588,
587,
571,
561,
549,
529,
519,
523,
527,
533,
534,
532,
537,
538,
533,
527,
521,
515,
507,
497,
489,
481,
473,
468,
460,
448,
437,
431,
426,
433,
436,
438,
430,
415,
409,
410,
410,
410,
410,
413,
415,
417,
421,
426,
437,
448,
445,
443,
437,
441,
431,
421,
423,
416,
424,
422,
426,
430,
436,
440,
453,
464,
473,
477,
479,
484,
489,
479,
472,
489,
497,
483,
463,
455,
461,
474,
479,
468,
453,
452,
467,
459,
446,
441,
442,
454,
459,
461,
462,
462,
482,
499,
494,
476,
456,
464,
462,
467,
474,
481,
480,
463,
459,
462,
449,
437,
443,
443,
446,
451,
460,
467,
454,
438,
425,
429,
434,
438,
448,
450,
452,
459,
469,
488,
496,
506,
518,
530,
540,
547,
562,
573,
575,
585,
599,
608,
615,
621,
637,
654,
672,
683,
697,
714,
727,
724,
710,
693,
673,
663,
656,
657,
655,
653,
648,
635,
619,
625,
638,
644,
636,
632,
632,
628,
620,
608,
596,
585,
576,
568,
557,
544,
530,
523,
524,
525,
514,
500,
486,
475,
462,
453,
444,
438,
432,
427,
426,
425,
428,
429,
427,
426,
425,
426,
426,
429,
431,
434,
437,
443,
452,
469,
487,
502,
500,
508,
512,
509,
506,
509,
511,
511,
507,
504,
502,
500,
501,
500,
506,
515,
524,
528,
526,
517,
506,
496,
486,
471,
457,
448,
440,
435,
432,
430,
426,
418,
415,
407,
400,
391,
381,
374,
363,
356,
346,
333,
321,
322,
342,
343,
338,
329,
320,
314,
309,
306,
305,
308,
304,
302,
302,
302,
304,
308,
314,
318,
324,
328,
337,
344,
356,
367,
364,
361,
366,
358,
349,
348,
355,
358,
355,
349,
344,
338,
332,
329,
321,
311,
307,
307,
309,
309,
310,
311,
313,
314,
316,
320,
323,
326,
330,
337,
345,
357,
366,
364,
357,
372,
380,
380,
387,
396,
402,
394,
384,
372,
363,
375,
380,
384,
390,
400,
404,
409,
412,
414,
417,
414,
407,
404,
401,
393,
389,
392,
397,
400,
399,
401,
401,
394,
394,
391,
384,
379,
377,
373,
368,
360,
355,
353,
355,
359,
360,
706,
686,
664,
643,
621,
611,
599,
583,
573,
579,
580,
577,
573,
581,
577,
566,
548,
531,
513,
511,
515,
517,
514,
514,
522,
527,
525,
520,
514,
506,
497,
488,
481,
473,
462,
457,
451,
443,
431,
427,
424,
425,
425,
436,
440,
423,
409,
408,
407,
405,
408,
410,
413,
412,
416,
426,
437,
432,
432,
431,
424,
427,
429,
415,
412,
420,
427,
430,
438,
441,
443,
445,
450,
462,
481,
494,
500,
496,
498,
484,
471,
480,
490,
482,
462,
453,
462,
471,
476,
461,
446,
450,
468,
465,
451,
438,
434,
441,
440,
449,
460,
474,
487,
501,
494,
474,
451,
454,
451,
454,
464,
473,
479,
467,
452,
454,
449,
438,
438,
438,
452,
460,
467,
470,
457,
439,
424,
426,
435,
442,
447,
451,
457,
467,
476,
486,
495,
500,
507,
513,
528,
541,
551,
558,
566,
587,
601,
612,
623,
633,
648,
665,
680,
693,
707,
722,
732,
722,
705,
688,
670,
654,
642,
641,
634,
635,
635,
623,
610,
619,
633,
636,
630,
624,
622,
620,
615,
609,
598,
584,
571,
560,
552,
541,
526,
512,
509,
512,
508,
497,
486,
476,
466,
454,
447,
439,
433,
427,
426,
426,
427,
427,
426,
424,
423,
424,
425,
428,
430,
433,
438,
445,
455,
469,
483,
485,
481,
491,
496,
496,
494,
497,
502,
503,
501,
496,
494,
494,
493,
494,
504,
513,
522,
528,
525,
515,
504,
493,
482,
470,
460,
451,
440,
430,
421,
414,
410,
407,
404,
402,
400,
394,
388,
380,
369,
358,
351,
339,
325,
314,
327,
332,
328,
321,
314,
308,
305,
305,
306,
303,
302,
302,
302,
303,
307,
316,
320,
326,
336,
340,
352,
363,
367,
369,
367,
363,
358,
349,
339,
337,
342,
346,
343,
337,
331,
326,
322,
316,
310,
304,
305,
307,
308,
309,
310,
312,
314,
316,
320,
324,
326,
328,
336,
344,
353,
364,
378,
378,
371,
378,
391,
394,
396,
403,
405,
398,
386,
375,
375,
384,
392,
395,
398,
404,
409,
414,
416,
417,
415,
410,
402,
394,
392,
384,
382,
387,
389,
391,
392,
400,
398,
390,
386,
382,
375,
368,
367,
365,
362,
357,
351,
352,
355,
356,
356,
699,
680,
661,
642,
624,
602,
589,
584,
568,
560,
563,
558,
563,
565,
563,
561,
556,
544,
528,
508,
499,
503,
498,
499,
509,
516,
516,
511,
503,
496,
487,
475,
469,
461,
452,
441,
438,
432,
427,
422,
427,
422,
424,
430,
432,
426,
412,
408,
404,
403,
406,
410,
415,
421,
430,
438,
433,
425,
424,
422,
415,
412,
417,
412,
419,
429,
433,
439,
439,
439,
439,
439,
448,
463,
482,
497,
512,
508,
502,
487,
471,
473,
482,
479,
460,
451,
457,
465,
471,
458,
438,
450,
468,
465,
453,
438,
430,
435,
446,
458,
473,
486,
492,
496,
501,
480,
463,
450,
447,
448,
456,
466,
474,
466,
450,
441,
441,
436,
445,
453,
459,
470,
478,
478,
461,
441,
424,
428,
436,
447,
456,
463,
473,
482,
493,
504,
515,
519,
521,
521,
524,
534,
542,
546,
562,
583,
602,
618,
631,
644,
657,
673,
687,
699,
713,
726,
727,
714,
699,
684,
665,
649,
633,
625,
617,
615,
614,
608,
598,
613,
627,
632,
623,
618,
611,
609,
604,
596,
588,
578,
568,
555,
538,
532,
525,
513,
499,
494,
494,
489,
481,
472,
462,
453,
447,
440,
435,
430,
427,
425,
427,
428,
426,
424,
422,
420,
423,
426,
427,
430,
437,
444,
454,
466,
475,
470,
472,
478,
483,
487,
483,
486,
492,
494,
492,
488,
486,
487,
487,
487,
500,
513,
521,
526,
528,
521,
510,
501,
490,
478,
466,
456,
445,
436,
425,
416,
408,
399,
391,
387,
382,
383,
384,
378,
368,
364,
353,
342,
329,
313,
313,
320,
318,
313,
308,
305,
305,
303,
302,
302,
302,
302,
303,
306,
312,
325,
334,
340,
351,
363,
368,
366,
364,
358,
352,
344,
343,
338,
329,
329,
338,
339,
336,
330,
322,
316,
314,
309,
304,
305,
306,
308,
309,
310,
312,
314,
316,
319,
324,
328,
330,
332,
340,
352,
361,
373,
380,
390,
389,
393,
399,
406,
407,
406,
406,
401,
390,
388,
389,
390,
399,
404,
405,
409,
413,
416,
417,
414,
409,
403,
397,
388,
380,
376,
378,
381,
382,
387,
389,
392,
392,
384,
375,
372,
366,
360,
356,
356,
355,
353,
350,
352,
353,
353,
353,
694,
676,
657,
640,
622,
602,
586,
572,
567,
562,
549,
547,
547,
549,
547,
548,
559,
556,
540,
522,
499,
488,
487,
491,
501,
505,
505,
500,
493,
486,
475,
467,
456,
444,
438,
430,
425,
425,
424,
416,
416,
418,
415,
423,
425,
418,
413,
409,
405,
402,
405,
412,
426,
430,
432,
437,
436,
421,
419,
429,
427,
414,
411,
417,
426,
428,
435,
427,
428,
434,
436,
443,
456,
469,
484,
499,
515,
522,
504,
488,
469,
463,
474,
477,
459,
447,
449,
458,
467,
455,
435,
449,
467,
472,
453,
442,
431,
432,
443,
458,
471,
479,
483,
483,
492,
489,
475,
460,
448,
451,
456,
463,
468,
465,
453,
436,
436,
438,
448,
458,
466,
475,
487,
476,
454,
438,
424,
427,
432,
441,
452,
466,
478,
493,
501,
502,
514,
530,
537,
540,
542,
545,
550,
556,
562,
578,
602,
620,
636,
649,
661,
674,
687,
700,
713,
722,
714,
702,
689,
676,
661,
644,
627,
610,
603,
599,
599,
592,
594,
609,
622,
626,
621,
614,
605,
595,
590,
583,
576,
567,
558,
551,
537,
524,
516,
510,
500,
486,
479,
477,
474,
467,
461,
453,
447,
441,
436,
430,
428,
429,
428,
431,
426,
423,
426,
428,
426,
424,
425,
427,
435,
442,
453,
460,
462,
459,
460,
465,
470,
472,
472,
474,
480,
482,
480,
478,
477,
479,
481,
487,
496,
504,
510,
517,
526,
524,
513,
503,
493,
481,
468,
458,
449,
440,
429,
421,
412,
405,
397,
388,
379,
371,
370,
368,
362,
358,
352,
343,
331,
316,
308,
307,
307,
305,
304,
303,
302,
302,
302,
302,
302,
303,
305,
311,
323,
334,
346,
354,
366,
371,
367,
356,
348,
344,
338,
329,
328,
327,
329,
332,
335,
333,
331,
325,
318,
312,
309,
305,
304,
306,
307,
309,
310,
312,
314,
316,
319,
324,
329,
334,
338,
340,
344,
355,
368,
379,
389,
397,
405,
407,
407,
406,
405,
403,
399,
396,
393,
391,
394,
395,
401,
407,
406,
408,
412,
411,
409,
406,
402,
396,
390,
382,
372,
368,
375,
377,
377,
381,
388,
391,
388,
377,
364,
362,
358,
353,
351,
350,
350,
349,
348,
349,
351,
352,
353,
684,
671,
656,
640,
624,
604,
591,
579,
568,
557,
548,
549,
538,
524,
528,
543,
558,
562,
548,
529,
511,
488,
480,
480,
486,
492,
492,
488,
481,
473,
467,
459,
448,
435,
439,
441,
441,
440,
436,
424,
418,
415,
416,
417,
417,
413,
412,
408,
405,
402,
403,
413,
424,
425,
424,
426,
429,
419,
414,
427,
428,
414,
411,
419,
425,
429,
424,
430,
439,
445,
444,
444,
453,
473,
486,
498,
512,
515,
503,
488,
468,
460,
469,
477,
461,
447,
442,
451,
461,
445,
435,
450,
467,
478,
466,
446,
436,
431,
437,
449,
457,
468,
472,
466,
478,
491,
478,
461,
442,
442,
451,
460,
466,
466,
450,
435,
438,
442,
448,
456,
469,
479,
484,
467,
456,
439,
425,
423,
427,
434,
445,
460,
471,
480,
476,
487,
502,
521,
537,
550,
557,
563,
571,
577,
580,
588,
599,
617,
634,
649,
662,
673,
683,
694,
706,
713,
701,
684,
672,
659,
651,
640,
626,
611,
593,
585,
582,
578,
590,
606,
620,
622,
616,
611,
604,
593,
580,
572,
562,
554,
546,
542,
537,
526,
512,
501,
494,
486,
474,
466,
462,
457,
451,
448,
439,
437,
431,
427,
425,
426,
426,
427,
436,
429,
435,
441,
439,
436,
433,
430,
434,
436,
445,
452,
450,
445,
450,
452,
456,
460,
462,
463,
467,
468,
468,
468,
471,
474,
474,
478,
484,
494,
500,
511,
520,
517,
510,
499,
489,
475,
463,
453,
445,
437,
430,
422,
416,
407,
402,
391,
379,
367,
352,
354,
351,
348,
345,
338,
315,
307,
304,
303,
302,
302,
302,
302,
302,
302,
302,
302,
303,
306,
312,
323,
329,
342,
358,
369,
366,
361,
356,
346,
334,
328,
324,
321,
327,
334,
333,
331,
331,
329,
322,
318,
312,
306,
304,
304,
305,
307,
309,
310,
312,
314,
315,
318,
324,
330,
335,
340,
345,
350,
354,
355,
365,
371,
379,
388,
395,
401,
405,
407,
405,
399,
389,
389,
387,
388,
391,
398,
403,
406,
405,
406,
404,
400,
400,
400,
392,
390,
382,
375,
365,
367,
370,
373,
376,
381,
386,
389,
381,
370,
360,
352,
352,
349,
347,
346,
346,
347,
346,
350,
351,
352,
353,
676,
663,
648,
633,
623,
610,
597,
587,
575,
562,
544,
538,
552,
540,
522,
532,
547,
561,
552,
539,
522,
502,
485,
472,
472,
474,
474,
471,
466,
461,
455,
450,
442,
450,
460,
461,
450,
442,
436,
427,
419,
415,
415,
414,
414,
414,
411,
407,
406,
403,
402,
411,
415,
414,
414,
422,
427,
415,
415,
421,
414,
410,
413,
428,
426,
426,
426,
434,
443,
452,
456,
459,
456,
463,
476,
488,
508,
513,
501,
483,
463,
461,
470,
476,
464,
452,
437,
443,
448,
435,
442,
457,
471,
483,
475,
455,
442,
432,
431,
439,
445,
456,
454,
451,
468,
488,
479,
461,
443,
433,
446,
459,
465,
459,
438,
437,
444,
453,
455,
452,
463,
474,
471,
455,
449,
437,
425,
420,
423,
428,
438,
447,
459,
460,
470,
485,
498,
509,
524,
540,
554,
569,
581,
593,
601,
605,
608,
613,
628,
642,
654,
663,
674,
685,
698,
704,
698,
684,
663,
644,
634,
627,
615,
603,
590,
576,
571,
571,
588,
606,
614,
612,
609,
602,
600,
597,
583,
569,
554,
544,
536,
532,
529,
524,
513,
502,
489,
483,
478,
469,
463,
457,
452,
446,
440,
434,
426,
424,
422,
416,
415,
434,
438,
440,
443,
449,
452,
451,
447,
443,
438,
436,
438,
440,
438,
438,
438,
442,
446,
450,
451,
453,
455,
459,
460,
462,
468,
475,
476,
482,
490,
498,
507,
514,
511,
508,
504,
498,
486,
473,
463,
455,
446,
438,
427,
414,
406,
396,
389,
381,
369,
358,
346,
341,
340,
338,
335,
316,
308,
302,
302,
302,
302,
302,
302,
302,
302,
302,
302,
303,
306,
312,
328,
335,
340,
353,
369,
362,
352,
347,
344,
336,
324,
316,
316,
325,
337,
343,
337,
330,
323,
317,
313,
310,
306,
303,
304,
305,
305,
308,
310,
311,
313,
315,
317,
323,
328,
335,
340,
345,
353,
361,
368,
370,
366,
370,
377,
383,
393,
402,
406,
403,
398,
392,
383,
381,
387,
390,
395,
400,
403,
406,
402,
399,
394,
390,
394,
392,
387,
383,
380,
371,
364,
366,
366,
372,
378,
382,
382,
379,
372,
363,
355,
348,
344,
343,
342,
343,
342,
342,
351,
353,
355,
357,
360,
666,
652,
635,
620,
610,
604,
598,
588,
574,
558,
542,
532,
542,
538,
521,
526,
544,
551,
549,
540,
526,
511,
496,
480,
465,
459,
457,
455,
452,
450,
448,
449,
451,
455,
458,
452,
439,
428,
424,
420,
419,
415,
414,
414,
413,
413,
410,
407,
404,
403,
402,
407,
410,
412,
414,
420,
416,
412,
414,
413,
410,
413,
421,
433,
433,
421,
428,
433,
440,
451,
455,
459,
466,
469,
477,
490,
505,
515,
499,
484,
464,
452,
461,
463,
466,
458,
443,
440,
438,
438,
450,
465,
479,
490,
484,
464,
447,
435,
428,
431,
438,
443,
438,
452,
471,
489,
478,
459,
444,
425,
432,
447,
454,
449,
433,
440,
452,
462,
468,
469,
474,
475,
466,
450,
434,
429,
419,
414,
428,
431,
436,
439,
448,
458,
474,
489,
509,
524,
532,
536,
546,
558,
575,
589,
604,
615,
622,
624,
625,
632,
643,
655,
669,
682,
693,
694,
688,
679,
660,
642,
626,
608,
602,
589,
580,
570,
562,
574,
592,
607,
605,
598,
598,
587,
588,
588,
583,
570,
554,
537,
526,
520,
517,
514,
508,
502,
497,
490,
484,
477,
470,
464,
454,
448,
441,
432,
424,
420,
417,
414,
420,
439,
445,
452,
456,
460,
464,
463,
458,
451,
445,
440,
439,
437,
439,
443,
449,
450,
450,
448,
449,
451,
454,
462,
467,
470,
475,
481,
486,
487,
493,
502,
508,
511,
504,
498,
497,
492,
489,
479,
470,
461,
451,
441,
429,
415,
402,
391,
380,
375,
364,
354,
342,
332,
330,
327,
315,
307,
302,
302,
302,
302,
302,
302,
302,
302,
302,
302,
303,
305,
310,
326,
336,
346,
356,
367,
368,
352,
338,
335,
332,
324,
316,
316,
322,
330,
340,
333,
326,
318,
311,
307,
306,
304,
303,
304,
305,
306,
306,
308,
310,
313,
314,
316,
321,
328,
335,
342,
348,
352,
359,
370,
380,
382,
379,
383,
385,
393,
401,
406,
403,
394,
390,
384,
376,
382,
391,
396,
401,
403,
405,
404,
397,
390,
387,
386,
388,
385,
380,
375,
372,
368,
364,
362,
366,
372,
377,
380,
379,
373,
363,
355,
349,
343,
340,
339,
340,
340,
341,
346,
353,
358,
361,
363,
366,
658,
646,
626,
610,
596,
588,
583,
578,
565,
552,
538,
525,
527,
521,
510,
524,
538,
538,
537,
533,
528,
518,
500,
490,
477,
462,
451,
446,
448,
450,
458,
462,
453,
451,
444,
441,
432,
423,
415,
414,
414,
414,
413,
413,
412,
411,
409,
406,
404,
402,
402,
402,
411,
415,
415,
413,
410,
412,
410,
411,
415,
424,
435,
441,
434,
414,
426,
426,
438,
440,
435,
443,
457,
466,
480,
492,
503,
511,
502,
485,
470,
452,
447,
448,
454,
459,
449,
436,
432,
443,
456,
469,
483,
491,
491,
473,
454,
434,
430,
427,
430,
435,
439,
457,
476,
483,
468,
451,
437,
421,
419,
432,
441,
435,
433,
437,
448,
460,
473,
479,
488,
474,
459,
442,
425,
412,
409,
415,
436,
436,
438,
442,
448,
459,
474,
488,
506,
523,
541,
554,
560,
559,
563,
577,
593,
610,
622,
635,
637,
640,
650,
662,
674,
685,
687,
680,
672,
658,
648,
636,
620,
596,
590,
581,
568,
560,
562,
578,
596,
600,
592,
583,
581,
575,
570,
574,
572,
565,
555,
541,
525,
510,
504,
503,
500,
493,
490,
490,
486,
476,
468,
459,
449,
440,
439,
427,
418,
414,
415,
413,
424,
440,
453,
464,
467,
474,
476,
475,
466,
459,
454,
452,
451,
445,
441,
450,
456,
460,
460,
456,
454,
461,
466,
472,
478,
480,
484,
491,
496,
498,
497,
499,
501,
504,
499,
491,
488,
484,
477,
476,
470,
462,
451,
440,
428,
415,
406,
398,
389,
379,
370,
358,
344,
332,
321,
314,
309,
302,
302,
302,
302,
302,
302,
302,
302,
302,
302,
303,
306,
309,
317,
327,
340,
354,
361,
360,
356,
343,
328,
324,
319,
315,
314,
318,
327,
333,
330,
322,
312,
307,
305,
303,
302,
302,
302,
303,
304,
305,
307,
309,
311,
313,
315,
318,
326,
333,
341,
350,
357,
364,
366,
379,
391,
394,
394,
398,
395,
402,
405,
403,
398,
388,
380,
375,
372,
382,
392,
401,
405,
404,
400,
396,
391,
383,
378,
380,
379,
376,
373,
365,
363,
361,
358,
364,
366,
373,
378,
378,
374,
367,
357,
347,
341,
339,
337,
338,
340,
342,
346,
351,
358,
364,
368,
372,
375,
646,
638,
625,
607,
586,
573,
561,
561,
556,
545,
533,
517,
516,
506,
508,
524,
530,
524,
522,
521,
522,
519,
512,
501,
488,
478,
479,
476,
468,
462,
463,
460,
454,
442,
428,
426,
426,
420,
417,
414,
413,
413,
413,
413,
412,
411,
410,
407,
405,
403,
402,
408,
415,
415,
411,
407,
402,
404,
408,
412,
420,
435,
451,
447,
433,
414,
419,
430,
433,
425,
425,
439,
448,
464,
475,
488,
495,
499,
494,
486,
474,
456,
439,
436,
442,
448,
452,
436,
427,
439,
450,
458,
470,
484,
488,
478,
464,
448,
436,
428,
426,
432,
447,
461,
476,
479,
464,
445,
429,
413,
418,
425,
430,
426,
429,
440,
446,
450,
464,
476,
478,
473,
452,
435,
417,
405,
409,
425,
440,
439,
440,
451,
456,
466,
478,
495,
510,
528,
544,
560,
571,
574,
574,
576,
585,
601,
616,
629,
637,
648,
658,
665,
675,
683,
681,
666,
648,
637,
629,
621,
606,
588,
578,
575,
562,
556,
569,
586,
599,
596,
582,
570,
563,
563,
553,
556,
556,
554,
549,
537,
522,
505,
492,
491,
490,
484,
478,
482,
480,
471,
460,
449,
442,
430,
424,
419,
413,
411,
413,
420,
432,
446,
455,
462,
463,
469,
477,
478,
473,
469,
466,
466,
460,
452,
450,
450,
461,
467,
467,
466,
462,
466,
472,
479,
485,
491,
497,
500,
502,
499,
493,
491,
498,
499,
493,
482,
472,
467,
463,
462,
460,
457,
452,
442,
432,
422,
412,
402,
393,
384,
375,
363,
348,
334,
323,
311,
306,
302,
302,
302,
302,
302,
302,
302,
303,
304,
305,
307,
313,
316,
322,
333,
345,
356,
351,
346,
343,
334,
323,
314,
315,
314,
316,
319,
328,
326,
320,
311,
305,
302,
302,
302,
302,
302,
302,
302,
304,
306,
308,
310,
312,
314,
316,
323,
331,
338,
347,
357,
368,
377,
380,
387,
400,
407,
410,
410,
406,
406,
403,
396,
390,
382,
373,
365,
369,
381,
392,
403,
405,
402,
393,
387,
384,
378,
373,
374,
371,
367,
364,
358,
354,
353,
358,
363,
368,
374,
377,
372,
366,
358,
348,
340,
336,
337,
338,
339,
342,
347,
352,
358,
365,
373,
378,
380,
382,
630,
628,
622,
606,
587,
564,
546,
547,
549,
541,
527,
509,
504,
498,
501,
517,
524,
514,
509,
509,
512,
514,
513,
506,
498,
482,
468,
466,
463,
454,
448,
446,
440,
431,
425,
424,
421,
416,
414,
414,
412,
413,
413,
413,
414,
414,
414,
413,
409,
404,
402,
406,
415,
412,
408,
404,
402,
405,
409,
413,
422,
434,
438,
438,
422,
413,
422,
427,
422,
414,
428,
433,
435,
446,
455,
466,
476,
493,
498,
495,
477,
463,
447,
438,
434,
437,
446,
438,
424,
432,
438,
449,
461,
477,
481,
482,
472,
458,
445,
432,
424,
428,
446,
454,
466,
468,
455,
437,
424,
412,
415,
427,
436,
432,
424,
436,
449,
458,
466,
466,
466,
468,
450,
429,
408,
402,
414,
428,
429,
429,
435,
443,
450,
460,
473,
491,
506,
522,
540,
552,
566,
576,
585,
588,
593,
600,
611,
622,
631,
640,
647,
654,
668,
675,
668,
650,
634,
619,
612,
604,
592,
579,
569,
565,
554,
551,
573,
589,
595,
583,
573,
556,
545,
546,
541,
539,
539,
539,
539,
532,
518,
503,
486,
478,
476,
472,
467,
471,
472,
466,
458,
448,
437,
423,
414,
410,
408,
410,
414,
424,
434,
440,
442,
446,
453,
466,
475,
482,
480,
479,
477,
475,
466,
463,
458,
452,
461,
470,
476,
474,
472,
471,
477,
484,
491,
499,
502,
501,
495,
487,
482,
483,
490,
492,
487,
474,
460,
451,
446,
443,
445,
446,
442,
436,
432,
427,
417,
408,
400,
391,
381,
368,
353,
340,
328,
317,
305,
302,
302,
302,
302,
302,
303,
304,
305,
310,
312,
313,
315,
322,
333,
343,
347,
347,
340,
332,
328,
323,
314,
306,
307,
312,
316,
321,
322,
317,
312,
306,
302,
302,
302,
302,
302,
302,
302,
303,
305,
307,
309,
312,
313,
315,
318,
329,
334,
342,
352,
363,
375,
386,
394,
402,
410,
416,
418,
416,
410,
404,
397,
388,
381,
376,
366,
363,
371,
383,
394,
403,
400,
395,
388,
378,
377,
372,
364,
366,
362,
356,
353,
355,
352,
356,
362,
367,
370,
371,
372,
366,
357,
349,
341,
336,
335,
336,
338,
341,
346,
351,
358,
366,
373,
380,
384,
386,
388,
616,
616,
611,
599,
581,
560,
541,
528,
529,
531,
525,
508,
493,
491,
505,
516,
518,
508,
497,
496,
500,
503,
503,
500,
492,
480,
459,
450,
448,
442,
436,
432,
426,
425,
424,
422,
420,
416,
414,
414,
413,
413,
413,
413,
414,
414,
414,
416,
411,
404,
402,
404,
406,
406,
405,
402,
403,
405,
408,
413,
420,
430,
438,
424,
416,
413,
417,
413,
413,
419,
429,
422,
425,
428,
442,
463,
481,
498,
502,
500,
488,
472,
458,
449,
439,
431,
439,
438,
425,
432,
443,
450,
458,
466,
471,
476,
478,
472,
455,
438,
425,
423,
432,
442,
451,
452,
442,
428,
414,
413,
419,
429,
439,
431,
425,
437,
449,
456,
457,
454,
460,
464,
451,
429,
406,
402,
408,
413,
413,
418,
424,
434,
445,
454,
470,
488,
500,
516,
530,
544,
560,
564,
569,
582,
593,
599,
600,
608,
619,
626,
630,
643,
659,
670,
661,
639,
621,
605,
596,
590,
580,
568,
558,
554,
547,
551,
572,
588,
579,
567,
560,
548,
534,
529,
528,
526,
527,
528,
529,
526,
515,
501,
487,
472,
463,
460,
458,
461,
461,
460,
454,
446,
438,
422,
407,
402,
408,
414,
415,
417,
424,
426,
434,
445,
456,
467,
476,
486,
488,
486,
488,
480,
475,
471,
464,
461,
464,
473,
478,
478,
476,
476,
484,
488,
491,
493,
494,
493,
488,
472,
466,
473,
480,
484,
476,
468,
454,
442,
434,
428,
428,
432,
430,
423,
420,
422,
417,
410,
399,
390,
378,
367,
354,
342,
332,
316,
308,
302,
302,
302,
302,
305,
306,
308,
309,
310,
313,
315,
317,
328,
342,
354,
341,
333,
328,
320,
314,
304,
304,
304,
310,
315,
319,
319,
316,
310,
306,
303,
302,
302,
302,
302,
302,
302,
302,
303,
306,
308,
311,
313,
315,
316,
322,
330,
334,
341,
351,
364,
374,
383,
394,
407,
417,
415,
414,
412,
405,
397,
390,
380,
371,
364,
362,
364,
374,
386,
394,
398,
392,
387,
381,
371,
365,
366,
354,
354,
351,
352,
354,
354,
356,
360,
365,
367,
369,
367,
368,
361,
350,
341,
337,
333,
332,
335,
338,
342,
348,
356,
364,
371,
378,
385,
390,
391,
390,
601,
601,
597,
590,
575,
563,
546,
527,
510,
510,
510,
502,
486,
490,
506,
514,
514,
503,
486,
482,
488,
491,
491,
488,
479,
471,
460,
444,
438,
434,
427,
426,
424,
423,
421,
420,
419,
417,
415,
415,
414,
414,
413,
413,
414,
414,
415,
415,
415,
403,
402,
402,
402,
403,
402,
402,
402,
404,
404,
411,
418,
426,
422,
418,
407,
407,
410,
413,
421,
426,
430,
415,
416,
432,
450,
467,
483,
492,
490,
491,
491,
486,
471,
459,
447,
434,
426,
431,
424,
430,
435,
441,
447,
452,
453,
464,
476,
478,
462,
441,
431,
421,
423,
428,
435,
436,
427,
419,
410,
416,
423,
433,
431,
428,
426,
438,
441,
443,
441,
447,
452,
454,
444,
428,
406,
401,
405,
414,
419,
420,
422,
426,
440,
453,
464,
476,
495,
509,
527,
542,
551,
548,
556,
567,
576,
582,
583,
595,
603,
609,
617,
634,
650,
660,
650,
631,
612,
592,
581,
577,
568,
556,
546,
539,
543,
558,
575,
575,
564,
552,
544,
535,
524,
516,
516,
514,
515,
516,
519,
520,
513,
499,
484,
471,
459,
449,
447,
452,
450,
452,
445,
437,
426,
417,
400,
404,
410,
415,
419,
421,
424,
428,
438,
450,
459,
467,
476,
485,
492,
497,
492,
488,
481,
476,
463,
464,
472,
476,
479,
480,
482,
484,
487,
489,
484,
480,
481,
481,
476,
467,
457,
461,
470,
476,
468,
461,
453,
438,
426,
417,
410,
413,
417,
411,
402,
406,
412,
405,
395,
385,
374,
363,
351,
341,
332,
321,
307,
303,
302,
302,
303,
306,
308,
309,
307,
311,
315,
317,
326,
337,
351,
354,
338,
324,
315,
309,
303,
304,
310,
309,
314,
318,
319,
315,
308,
305,
303,
302,
302,
302,
302,
302,
302,
302,
302,
304,
308,
310,
312,
315,
317,
323,
326,
328,
334,
341,
351,
354,
364,
376,
389,
404,
412,
407,
406,
404,
398,
390,
380,
372,
364,
356,
360,
369,
379,
390,
394,
393,
385,
375,
372,
364,
356,
355,
346,
343,
350,
352,
354,
354,
355,
358,
360,
363,
365,
364,
361,
356,
346,
335,
331,
330,
331,
335,
338,
342,
351,
360,
368,
377,
384,
390,
390,
385,
378,
581,
582,
584,
581,
574,
566,
555,
537,
518,
504,
494,
491,
486,
489,
502,
511,
508,
498,
483,
464,
472,
476,
471,
467,
464,
458,
448,
440,
433,
428,
425,
424,
423,
420,
418,
417,
417,
416,
415,
415,
416,
415,
413,
412,
411,
413,
414,
415,
416,
406,
401,
400,
400,
400,
400,
400,
401,
402,
400,
408,
417,
423,
416,
408,
405,
404,
406,
417,
430,
439,
429,
417,
415,
427,
445,
463,
478,
480,
472,
480,
482,
472,
466,
462,
452,
441,
426,
424,
427,
433,
436,
438,
441,
450,
456,
464,
478,
485,
473,
456,
438,
426,
420,
419,
422,
426,
418,
411,
413,
420,
428,
436,
431,
424,
423,
429,
430,
428,
437,
438,
439,
443,
432,
415,
399,
402,
408,
417,
428,
436,
439,
441,
441,
449,
462,
480,
499,
510,
522,
532,
538,
536,
544,
553,
557,
562,
567,
576,
587,
601,
616,
632,
646,
652,
638,
620,
605,
585,
570,
565,
557,
543,
534,
541,
554,
570,
574,
563,
547,
538,
529,
521,
515,
507,
503,
501,
503,
504,
506,
510,
509,
500,
487,
476,
461,
448,
438,
441,
439,
442,
435,
424,
416,
406,
401,
409,
416,
416,
422,
424,
425,
429,
437,
444,
451,
462,
473,
484,
495,
502,
497,
489,
483,
472,
472,
473,
478,
480,
478,
478,
478,
478,
479,
478,
472,
467,
466,
465,
463,
459,
450,
449,
459,
466,
462,
455,
452,
440,
425,
413,
401,
395,
403,
398,
388,
391,
402,
398,
391,
380,
370,
361,
350,
339,
330,
319,
307,
302,
302,
303,
305,
305,
306,
305,
306,
310,
316,
324,
334,
343,
352,
342,
331,
321,
312,
304,
306,
316,
311,
313,
317,
317,
316,
307,
305,
303,
302,
302,
302,
302,
302,
302,
302,
302,
303,
305,
309,
312,
314,
316,
325,
332,
334,
337,
343,
346,
355,
367,
375,
386,
399,
407,
404,
399,
398,
395,
390,
382,
374,
362,
354,
352,
365,
378,
384,
390,
391,
388,
381,
373,
361,
356,
348,
342,
338,
344,
353,
353,
352,
353,
352,
353,
355,
357,
357,
356,
353,
347,
338,
328,
329,
329,
332,
335,
339,
344,
353,
362,
371,
380,
388,
391,
383,
374,
365,
560,
565,
570,
571,
560,
556,
552,
542,
526,
513,
494,
483,
480,
484,
495,
498,
496,
490,
478,
462,
457,
460,
455,
449,
449,
446,
440,
434,
427,
424,
422,
421,
419,
418,
415,
414,
414,
414,
414,
412,
413,
414,
414,
411,
407,
407,
415,
414,
414,
404,
400,
399,
399,
399,
399,
399,
399,
399,
400,
404,
410,
416,
410,
405,
403,
404,
411,
420,
434,
442,
433,
418,
413,
425,
442,
455,
464,
460,
463,
474,
479,
470,
453,
448,
451,
444,
430,
424,
436,
447,
450,
452,
459,
465,
471,
477,
486,
490,
481,
463,
448,
429,
421,
416,
414,
413,
411,
408,
415,
424,
437,
442,
436,
427,
419,
416,
415,
427,
425,
429,
422,
431,
422,
410,
397,
402,
408,
417,
423,
435,
445,
452,
455,
460,
463,
478,
489,
498,
506,
513,
519,
523,
530,
536,
544,
556,
571,
586,
596,
607,
622,
638,
649,
647,
630,
612,
595,
577,
561,
553,
546,
532,
538,
552,
567,
575,
563,
551,
537,
525,
516,
509,
503,
497,
492,
487,
489,
487,
486,
491,
499,
492,
485,
474,
460,
448,
437,
429,
429,
430,
427,
418,
404,
400,
404,
412,
418,
426,
427,
428,
430,
438,
442,
447,
455,
466,
477,
486,
496,
498,
494,
488,
486,
479,
479,
478,
481,
479,
477,
473,
471,
470,
470,
468,
462,
457,
453,
452,
449,
448,
440,
439,
449,
456,
456,
448,
443,
437,
427,
414,
398,
387,
385,
384,
374,
381,
392,
390,
382,
373,
364,
356,
346,
336,
326,
317,
304,
302,
301,
302,
305,
305,
305,
304,
304,
308,
319,
332,
343,
343,
338,
331,
323,
315,
308,
302,
304,
305,
315,
317,
315,
316,
316,
306,
303,
302,
302,
302,
302,
302,
302,
302,
302,
303,
305,
308,
311,
314,
316,
324,
332,
338,
342,
341,
351,
356,
360,
365,
382,
396,
406,
404,
398,
389,
391,
389,
382,
373,
366,
355,
349,
359,
373,
390,
389,
386,
384,
380,
372,
363,
355,
346,
340,
338,
346,
355,
358,
354,
352,
348,
347,
348,
350,
350,
350,
348,
344,
340,
330,
326,
329,
330,
333,
336,
339,
345,
354,
364,
373,
380,
388,
386,
380,
369,
359,
556,
548,
554,
558,
548,
536,
539,
538,
528,
514,
495,
477,
477,
475,
486,
484,
480,
477,
468,
456,
447,
445,
443,
438,
437,
436,
432,
429,
426,
424,
420,
418,
417,
417,
416,
416,
415,
414,
414,
410,
410,
413,
414,
410,
404,
402,
409,
408,
406,
401,
398,
398,
398,
398,
398,
398,
398,
398,
400,
401,
407,
407,
406,
403,
402,
409,
417,
429,
440,
432,
421,
412,
412,
423,
433,
439,
446,
449,
457,
473,
483,
472,
452,
438,
436,
439,
426,
430,
440,
448,
459,
465,
468,
467,
470,
475,
482,
488,
480,
461,
438,
425,
420,
415,
410,
407,
402,
412,
421,
429,
440,
449,
449,
442,
431,
415,
409,
413,
412,
417,
412,
419,
416,
404,
398,
400,
403,
412,
414,
424,
431,
444,
453,
466,
469,
475,
490,
501,
501,
509,
513,
520,
527,
538,
552,
564,
580,
598,
610,
618,
630,
646,
652,
641,
624,
606,
587,
569,
554,
543,
538,
523,
540,
558,
573,
569,
553,
536,
525,
515,
506,
498,
493,
488,
484,
481,
478,
476,
475,
480,
485,
480,
475,
468,
458,
446,
433,
425,
420,
418,
417,
411,
400,
402,
407,
412,
420,
429,
437,
439,
437,
444,
452,
458,
463,
473,
482,
490,
492,
491,
490,
487,
484,
482,
481,
481,
479,
477,
473,
465,
459,
457,
458,
460,
456,
450,
441,
440,
438,
437,
434,
432,
438,
449,
454,
441,
431,
426,
419,
409,
394,
379,
370,
370,
362,
372,
382,
384,
377,
368,
358,
350,
342,
332,
322,
311,
302,
301,
300,
302,
305,
304,
305,
303,
303,
304,
320,
334,
341,
331,
324,
321,
315,
308,
304,
304,
303,
306,
314,
308,
306,
310,
307,
304,
302,
302,
302,
302,
302,
302,
302,
302,
302,
304,
309,
314,
314,
316,
321,
330,
338,
344,
351,
351,
352,
364,
370,
377,
389,
403,
403,
399,
391,
382,
382,
381,
374,
364,
356,
350,
352,
366,
379,
385,
383,
377,
378,
374,
363,
352,
347,
340,
338,
346,
355,
359,
357,
355,
348,
340,
341,
343,
344,
343,
342,
339,
334,
330,
329,
326,
329,
331,
333,
336,
340,
345,
354,
365,
374,
381,
381,
378,
374,
369,
360,
552,
534,
536,
544,
541,
524,
522,
524,
520,
508,
490,
474,
476,
468,
475,
473,
463,
464,
457,
448,
438,
437,
436,
434,
431,
426,
426,
424,
424,
419,
417,
414,
414,
414,
414,
416,
414,
413,
414,
410,
408,
408,
407,
405,
406,
402,
402,
403,
402,
398,
397,
397,
397,
397,
397,
397,
397,
398,
399,
401,
402,
403,
403,
402,
406,
414,
423,
431,
428,
416,
404,
407,
414,
424,
435,
446,
456,
463,
471,
476,
488,
481,
459,
441,
434,
425,
423,
432,
445,
453,
458,
457,
455,
456,
451,
461,
466,
475,
479,
463,
444,
427,
413,
413,
408,
402,
407,
414,
425,
436,
446,
454,
464,
454,
441,
426,
412,
402,
402,
405,
405,
411,
408,
400,
400,
402,
406,
412,
423,
431,
435,
440,
448,
459,
473,
478,
489,
504,
515,
522,
520,
528,
540,
551,
564,
574,
587,
601,
612,
623,
638,
650,
648,
632,
616,
600,
581,
561,
546,
534,
527,
522,
541,
558,
572,
562,
547,
529,
516,
505,
497,
491,
486,
482,
478,
473,
470,
466,
464,
471,
476,
470,
461,
457,
452,
446,
436,
424,
417,
412,
406,
402,
398,
399,
411,
419,
425,
432,
439,
447,
448,
448,
458,
467,
476,
480,
487,
490,
489,
484,
481,
479,
478,
476,
477,
477,
474,
469,
464,
458,
448,
444,
445,
449,
454,
448,
437,
432,
431,
427,
427,
424,
429,
440,
454,
440,
423,
414,
407,
402,
392,
375,
362,
353,
351,
365,
371,
374,
369,
360,
352,
342,
334,
326,
317,
305,
302,
300,
300,
301,
303,
305,
303,
301,
301,
301,
308,
320,
326,
318,
312,
310,
308,
307,
302,
303,
304,
308,
305,
304,
306,
308,
304,
302,
302,
302,
302,
302,
302,
302,
302,
302,
303,
304,
309,
314,
316,
320,
329,
334,
340,
349,
357,
362,
358,
364,
374,
386,
396,
398,
395,
392,
385,
376,
372,
372,
365,
352,
348,
348,
353,
367,
375,
379,
381,
368,
366,
366,
355,
344,
330,
328,
344,
352,
355,
354,
357,
355,
350,
342,
337,
337,
337,
335,
332,
329,
329,
329,
330,
329,
329,
331,
333,
337,
340,
346,
356,
366,
377,
380,
376,
369,
364,
362,
357,
552,
537,
514,
523,
530,
517,
507,
510,
507,
498,
484,
472,
467,
459,
463,
463,
450,
452,
449,
441,
436,
434,
432,
431,
429,
428,
425,
423,
421,
422,
419,
418,
417,
415,
414,
413,
413,
412,
409,
405,
405,
404,
404,
403,
403,
403,
402,
400,
398,
397,
396,
396,
396,
396,
396,
397,
398,
399,
400,
401,
401,
400,
401,
402,
412,
418,
416,
417,
410,
399,
401,
400,
416,
429,
441,
451,
463,
472,
483,
488,
490,
487,
464,
452,
448,
438,
423,
428,
440,
445,
449,
442,
437,
438,
438,
444,
454,
469,
479,
468,
455,
441,
425,
411,
406,
402,
408,
416,
428,
438,
448,
457,
465,
464,
455,
442,
424,
407,
399,
402,
402,
403,
401,
402,
402,
406,
412,
414,
425,
439,
449,
456,
459,
464,
480,
490,
496,
505,
517,
527,
524,
527,
539,
550,
563,
576,
585,
591,
602,
618,
636,
650,
638,
621,
605,
588,
572,
557,
541,
525,
520,
526,
543,
560,
566,
556,
540,
523,
508,
497,
488,
485,
481,
478,
473,
466,
460,
458,
453,
461,
465,
462,
455,
446,
439,
437,
431,
426,
417,
410,
402,
399,
395,
402,
415,
425,
432,
440,
446,
452,
458,
457,
462,
473,
479,
486,
490,
492,
486,
475,
471,
470,
470,
466,
467,
467,
466,
460,
453,
450,
441,
434,
434,
438,
442,
443,
433,
425,
424,
421,
419,
415,
419,
432,
447,
444,
425,
408,
391,
385,
379,
370,
357,
343,
345,
355,
358,
362,
360,
351,
344,
334,
326,
319,
310,
305,
302,
300,
299,
300,
302,
305,
301,
299,
298,
299,
299,
304,
316,
308,
304,
311,
310,
305,
303,
302,
304,
306,
303,
302,
305,
303,
302,
302,
302,
302,
302,
302,
302,
302,
302,
302,
304,
307,
310,
315,
319,
329,
335,
342,
345,
351,
359,
368,
368,
374,
385,
393,
392,
390,
388,
386,
380,
368,
361,
363,
353,
343,
342,
352,
359,
362,
369,
378,
374,
362,
353,
354,
344,
334,
325,
338,
348,
352,
352,
351,
352,
349,
344,
342,
333,
328,
330,
328,
325,
323,
329,
328,
328,
329,
329,
329,
333,
337,
340,
347,
357,
368,
378,
378,
368,
362,
355,
351,
349,
549,
530,
510,
505,
518,
513,
494,
493,
493,
486,
480,
469,
458,
450,
452,
450,
444,
440,
442,
439,
434,
429,
428,
428,
426,
425,
422,
423,
425,
427,
426,
424,
421,
421,
418,
414,
412,
411,
408,
406,
405,
405,
405,
405,
404,
402,
400,
398,
396,
395,
395,
395,
395,
395,
395,
395,
396,
396,
396,
397,
397,
398,
400,
402,
412,
413,
402,
402,
400,
398,
398,
402,
408,
417,
430,
438,
449,
465,
484,
487,
480,
481,
476,
470,
463,
451,
437,
423,
424,
427,
433,
425,
424,
429,
438,
447,
458,
466,
468,
470,
462,
449,
431,
413,
406,
402,
409,
414,
425,
437,
447,
456,
465,
467,
455,
442,
428,
415,
401,
396,
399,
398,
399,
402,
406,
408,
413,
422,
432,
445,
459,
471,
477,
477,
488,
500,
503,
509,
513,
516,
511,
520,
531,
541,
553,
560,
563,
579,
596,
613,
631,
636,
628,
613,
593,
573,
558,
545,
531,
516,
521,
530,
545,
558,
564,
552,
534,
518,
504,
493,
484,
478,
477,
473,
467,
461,
453,
450,
446,
450,
454,
456,
451,
441,
431,
426,
423,
418,
414,
409,
404,
399,
396,
403,
415,
426,
434,
443,
451,
458,
464,
469,
467,
474,
482,
488,
491,
488,
478,
466,
461,
461,
461,
458,
454,
455,
455,
452,
442,
439,
432,
423,
423,
428,
434,
436,
430,
418,
415,
416,
412,
409,
411,
424,
438,
440,
430,
414,
393,
374,
363,
356,
345,
333,
335,
344,
347,
352,
350,
342,
333,
325,
317,
310,
305,
304,
302,
301,
300,
300,
301,
301,
300,
298,
297,
297,
298,
299,
303,
303,
305,
316,
306,
305,
302,
301,
304,
303,
302,
302,
303,
302,
302,
302,
302,
302,
302,
302,
302,
302,
302,
303,
306,
309,
312,
316,
325,
332,
337,
344,
350,
357,
362,
368,
377,
387,
393,
392,
389,
384,
379,
380,
378,
365,
352,
349,
341,
340,
348,
357,
366,
371,
370,
374,
368,
355,
342,
341,
333,
326,
332,
345,
350,
349,
349,
347,
345,
341,
338,
334,
328,
325,
322,
320,
318,
323,
325,
327,
328,
329,
328,
332,
336,
339,
341,
348,
358,
369,
378,
376,
366,
355,
349,
342,
338,
540,
521,
499,
488,
503,
501,
487,
477,
476,
476,
472,
463,
454,
450,
447,
446,
443,
439,
436,
433,
428,
425,
429,
425,
424,
424,
430,
437,
436,
440,
442,
430,
426,
429,
426,
419,
415,
412,
409,
407,
410,
408,
408,
407,
404,
401,
398,
397,
396,
395,
394,
394,
394,
394,
394,
395,
395,
395,
395,
395,
395,
396,
397,
403,
404,
402,
399,
397,
397,
396,
399,
401,
408,
417,
425,
427,
447,
463,
478,
476,
459,
465,
474,
478,
476,
465,
450,
436,
423,
412,
417,
417,
430,
440,
445,
452,
453,
451,
450,
461,
461,
448,
430,
415,
402,
402,
408,
414,
429,
439,
449,
456,
462,
461,
441,
430,
422,
411,
400,
399,
396,
397,
396,
403,
403,
402,
411,
423,
437,
451,
469,
482,
491,
495,
499,
505,
504,
500,
496,
494,
497,
515,
526,
535,
542,
546,
556,
573,
591,
608,
620,
618,
612,
599,
585,
568,
547,
527,
514,
515,
528,
540,
552,
562,
561,
547,
531,
513,
499,
487,
478,
473,
472,
467,
462,
456,
449,
441,
438,
442,
445,
447,
447,
440,
428,
421,
414,
411,
407,
405,
403,
396,
391,
403,
414,
423,
430,
440,
451,
461,
467,
475,
477,
477,
485,
488,
485,
481,
473,
461,
452,
448,
451,
452,
443,
441,
447,
446,
437,
423,
415,
403,
412,
418,
425,
429,
427,
413,
402,
406,
405,
402,
403,
416,
428,
431,
426,
414,
398,
382,
366,
352,
340,
327,
326,
328,
334,
339,
340,
332,
324,
316,
309,
305,
303,
302,
301,
301,
301,
300,
300,
300,
298,
297,
297,
297,
297,
299,
298,
295,
297,
298,
300,
300,
300,
299,
299,
301,
300,
300,
301,
301,
302,
302,
302,
302,
302,
302,
302,
302,
303,
304,
308,
311,
314,
317,
323,
328,
333,
340,
347,
355,
364,
375,
382,
391,
390,
389,
383,
377,
369,
369,
369,
361,
342,
336,
338,
346,
356,
364,
372,
379,
378,
370,
362,
350,
339,
328,
326,
331,
344,
350,
347,
347,
344,
342,
340,
333,
330,
324,
318,
317,
316,
317,
318,
320,
324,
325,
327,
329,
331,
334,
337,
338,
341,
348,
357,
367,
375,
375,
369,
360,
351,
341,
335,
530,
511,
489,
480,
487,
482,
479,
477,
468,
460,
459,
455,
453,
450,
443,
440,
440,
439,
436,
438,
440,
439,
432,
426,
426,
435,
445,
453,
450,
452,
453,
436,
433,
438,
428,
422,
416,
414,
412,
414,
412,
416,
408,
405,
403,
402,
399,
397,
396,
394,
394,
394,
394,
394,
394,
394,
394,
394,
394,
394,
395,
395,
396,
397,
397,
397,
396,
396,
396,
397,
401,
402,
413,
428,
439,
447,
452,
462,
474,
466,
448,
451,
460,
464,
472,
468,
454,
438,
428,
416,
414,
423,
425,
437,
443,
443,
438,
432,
439,
452,
452,
438,
424,
412,
402,
403,
408,
416,
430,
440,
452,
456,
458,
454,
444,
428,
415,
408,
405,
406,
402,
393,
393,
399,
400,
400,
409,
422,
438,
455,
474,
489,
500,
504,
503,
501,
495,
486,
480,
479,
495,
508,
516,
523,
531,
538,
550,
565,
585,
597,
605,
600,
590,
586,
574,
560,
540,
521,
508,
520,
532,
543,
553,
560,
551,
539,
526,
510,
497,
484,
474,
470,
468,
464,
459,
452,
445,
438,
436,
437,
438,
438,
440,
436,
426,
418,
408,
401,
397,
398,
392,
392,
389,
398,
406,
414,
427,
435,
445,
454,
463,
470,
476,
480,
484,
480,
476,
471,
465,
455,
445,
435,
435,
440,
433,
425,
433,
441,
435,
420,
404,
391,
399,
407,
416,
415,
417,
409,
399,
396,
397,
395,
399,
412,
422,
422,
418,
410,
400,
385,
370,
358,
345,
332,
322,
318,
322,
327,
329,
323,
316,
310,
305,
303,
302,
301,
302,
301,
301,
300,
300,
299,
298,
298,
299,
300,
298,
297,
296,
294,
292,
292,
292,
294,
296,
296,
296,
297,
297,
298,
300,
301,
302,
302,
302,
302,
302,
302,
302,
303,
305,
305,
309,
316,
314,
316,
322,
328,
333,
340,
345,
353,
362,
375,
381,
386,
384,
383,
379,
373,
364,
356,
356,
349,
332,
329,
344,
354,
364,
370,
377,
378,
372,
365,
357,
347,
337,
326,
327,
336,
344,
346,
346,
346,
342,
334,
332,
326,
319,
316,
314,
317,
317,
318,
319,
321,
323,
325,
327,
329,
332,
335,
338,
340,
341,
348,
353,
363,
369,
369,
368,
361,
353,
345,
340,
526,
510,
494,
479,
472,
465,
468,
471,
466,
459,
452,
449,
448,
448,
443,
441,
438,
440,
445,
451,
451,
441,
429,
423,
431,
444,
456,
467,
473,
474,
460,
445,
442,
443,
435,
426,
420,
413,
414,
413,
414,
410,
402,
402,
402,
402,
402,
398,
396,
395,
394,
394,
394,
394,
394,
394,
394,
394,
394,
394,
394,
395,
395,
395,
395,
395,
395,
395,
396,
399,
407,
417,
419,
425,
440,
454,
464,
457,
460,
459,
441,
439,
450,
453,
463,
466,
451,
434,
421,
416,
412,
414,
426,
432,
430,
428,
421,
426,
439,
443,
438,
424,
414,
402,
405,
408,
412,
414,
426,
438,
449,
460,
460,
458,
451,
435,
417,
412,
414,
418,
414,
400,
390,
393,
398,
401,
411,
423,
436,
455,
470,
480,
488,
492,
492,
489,
481,
475,
467,
474,
485,
491,
500,
515,
525,
538,
546,
556,
566,
578,
581,
572,
570,
569,
560,
549,
531,
514,
511,
525,
535,
543,
548,
549,
543,
532,
520,
507,
492,
480,
469,
465,
465,
462,
456,
450,
442,
438,
436,
434,
433,
432,
433,
430,
424,
418,
407,
393,
388,
389,
387,
388,
386,
389,
398,
409,
419,
431,
441,
448,
457,
464,
473,
480,
480,
474,
464,
458,
455,
448,
439,
427,
419,
426,
423,
412,
417,
429,
429,
417,
401,
384,
386,
397,
405,
401,
406,
402,
392,
388,
389,
389,
397,
409,
415,
414,
410,
404,
396,
384,
369,
357,
344,
333,
323,
316,
314,
313,
314,
312,
308,
304,
303,
302,
301,
301,
301,
302,
302,
301,
301,
300,
300,
300,
301,
304,
307,
310,
310,
305,
298,
294,
292,
293,
294,
294,
294,
295,
295,
296,
298,
299,
301,
302,
302,
302,
303,
304,
304,
305,
305,
305,
314,
317,
315,
318,
324,
330,
335,
342,
347,
354,
359,
369,
380,
381,
378,
378,
379,
376,
367,
350,
340,
333,
326,
334,
347,
359,
369,
372,
373,
371,
368,
360,
351,
343,
334,
326,
327,
335,
342,
344,
343,
343,
338,
327,
322,
317,
315,
314,
315,
318,
318,
319,
320,
322,
324,
326,
327,
328,
330,
338,
339,
341,
345,
350,
351,
358,
362,
366,
364,
356,
351,
345,
341,
524,
516,
504,
492,
478,
464,
460,
462,
461,
458,
457,
455,
451,
448,
451,
451,
448,
449,
454,
452,
444,
433,
426,
428,
433,
443,
457,
470,
483,
485,
470,
460,
460,
455,
444,
430,
422,
417,
414,
411,
407,
402,
402,
402,
405,
403,
404,
401,
399,
398,
398,
397,
397,
397,
396,
395,
394,
394,
394,
394,
394,
394,
394,
394,
394,
394,
395,
396,
398,
399,
410,
428,
439,
441,
449,
459,
461,
444,
445,
452,
440,
428,
440,
454,
466,
471,
460,
441,
424,
416,
406,
410,
422,
422,
413,
413,
422,
433,
441,
433,
423,
415,
402,
402,
404,
414,
427,
432,
435,
442,
456,
467,
469,
461,
450,
434,
425,
428,
434,
434,
422,
404,
390,
390,
396,
403,
409,
418,
431,
446,
458,
465,
472,
475,
475,
471,
464,
461,
456,
464,
470,
476,
491,
509,
522,
529,
537,
547,
559,
567,
562,
555,
553,
554,
549,
538,
523,
509,
514,
528,
539,
540,
533,
534,
533,
522,
513,
500,
488,
475,
462,
459,
459,
457,
454,
448,
440,
434,
432,
432,
429,
425,
427,
424,
417,
416,
402,
392,
386,
386,
387,
389,
389,
392,
398,
404,
416,
429,
436,
443,
452,
462,
474,
484,
484,
471,
457,
446,
441,
439,
431,
419,
410,
412,
411,
400,
408,
415,
416,
414,
399,
380,
375,
382,
390,
387,
396,
390,
384,
380,
382,
385,
396,
404,
402,
401,
399,
394,
387,
376,
362,
351,
340,
329,
322,
315,
308,
304,
304,
306,
304,
303,
302,
302,
301,
301,
301,
302,
302,
302,
302,
301,
302,
304,
309,
316,
319,
319,
311,
305,
301,
295,
293,
292,
292,
293,
293,
293,
294,
294,
296,
298,
300,
301,
302,
302,
303,
304,
304,
305,
305,
305,
312,
310,
311,
317,
323,
329,
334,
340,
345,
349,
357,
368,
377,
376,
368,
367,
370,
368,
362,
347,
332,
324,
331,
343,
356,
366,
369,
367,
365,
364,
361,
357,
350,
341,
328,
318,
318,
329,
338,
342,
339,
334,
331,
320,
315,
314,
313,
314,
315,
316,
317,
318,
321,
323,
325,
327,
328,
330,
332,
338,
340,
344,
351,
353,
356,
358,
364,
365,
364,
354,
346,
339,
334,
512,
506,
499,
492,
485,
470,
462,
462,
462,
463,
470,
468,
461,
448,
454,
456,
453,
453,
451,
442,
436,
430,
431,
435,
438,
439,
451,
463,
481,
487,
482,
476,
464,
456,
447,
430,
425,
417,
414,
413,
408,
404,
408,
409,
409,
410,
413,
409,
406,
405,
402,
402,
402,
402,
400,
396,
395,
394,
394,
394,
394,
394,
394,
394,
395,
394,
393,
397,
401,
409,
418,
430,
447,
454,
456,
461,
449,
434,
431,
443,
438,
427,
438,
447,
458,
465,
461,
448,
437,
427,
410,
402,
408,
405,
413,
420,
427,
430,
429,
419,
412,
403,
397,
398,
408,
420,
433,
443,
451,
449,
454,
468,
477,
463,
454,
438,
444,
450,
452,
449,
428,
408,
390,
389,
395,
400,
407,
414,
427,
438,
442,
452,
459,
462,
459,
455,
452,
448,
449,
454,
463,
474,
490,
506,
522,
534,
549,
558,
563,
556,
545,
536,
537,
537,
533,
525,
513,
507,
514,
528,
532,
526,
516,
519,
520,
512,
504,
493,
482,
468,
460,
455,
454,
453,
451,
446,
439,
426,
428,
430,
429,
426,
422,
416,
412,
407,
399,
389,
383,
387,
393,
399,
403,
406,
408,
410,
414,
424,
431,
441,
452,
463,
475,
488,
485,
471,
456,
441,
429,
424,
414,
409,
401,
396,
392,
388,
396,
398,
403,
406,
397,
381,
367,
367,
374,
379,
392,
383,
375,
374,
374,
384,
393,
393,
389,
388,
387,
385,
378,
366,
355,
344,
336,
327,
317,
309,
305,
304,
303,
303,
303,
303,
305,
303,
303,
302,
302,
302,
303,
303,
303,
303,
304,
307,
318,
322,
320,
311,
304,
304,
300,
297,
294,
294,
293,
293,
293,
292,
292,
293,
294,
297,
300,
302,
303,
303,
303,
304,
305,
304,
306,
308,
305,
304,
306,
317,
321,
325,
330,
334,
337,
340,
350,
362,
370,
369,
362,
357,
358,
355,
347,
337,
324,
327,
338,
352,
360,
366,
365,
361,
359,
358,
356,
352,
348,
343,
332,
319,
315,
325,
334,
335,
332,
326,
321,
316,
312,
311,
312,
313,
314,
315,
316,
318,
321,
323,
326,
328,
331,
333,
337,
338,
342,
351,
357,
362,
363,
364,
366,
367,
367,
361,
352,
343,
335,
497,
488,
485,
481,
477,
471,
473,
473,
463,
468,
479,
480,
473,
464,
461,
456,
452,
447,
440,
435,
433,
435,
437,
443,
448,
444,
457,
470,
476,
471,
468,
461,
454,
446,
436,
424,
421,
417,
414,
412,
410,
407,
414,
415,
412,
414,
424,
418,
414,
409,
404,
402,
403,
403,
400,
399,
396,
395,
394,
394,
394,
394,
394,
394,
390,
391,
392,
395,
405,
420,
429,
439,
443,
442,
444,
447,
437,
422,
425,
440,
434,
417,
425,
436,
448,
458,
451,
444,
442,
430,
415,
402,
402,
408,
416,
421,
421,
417,
414,
403,
402,
396,
399,
407,
416,
423,
435,
450,
463,
463,
462,
463,
478,
470,
459,
453,
463,
464,
457,
448,
431,
411,
388,
391,
396,
402,
408,
414,
425,
423,
430,
440,
444,
444,
440,
435,
439,
452,
458,
463,
473,
484,
497,
512,
527,
539,
552,
563,
556,
542,
526,
521,
520,
523,
518,
511,
502,
506,
517,
527,
519,
510,
504,
507,
508,
504,
496,
484,
474,
464,
455,
450,
448,
447,
446,
442,
436,
426,
426,
430,
431,
427,
421,
412,
407,
400,
393,
386,
379,
387,
399,
407,
413,
417,
421,
422,
423,
427,
431,
438,
449,
463,
476,
488,
487,
472,
458,
445,
434,
424,
417,
411,
404,
396,
390,
380,
382,
387,
391,
393,
392,
380,
367,
363,
364,
380,
384,
376,
365,
366,
366,
379,
385,
382,
377,
376,
376,
376,
366,
355,
346,
338,
330,
321,
313,
304,
303,
304,
304,
304,
305,
305,
306,
306,
305,
306,
305,
304,
304,
304,
304,
304,
306,
309,
319,
319,
310,
304,
304,
303,
300,
298,
296,
295,
295,
294,
294,
293,
292,
293,
294,
296,
300,
306,
304,
303,
303,
304,
305,
306,
304,
302,
300,
300,
304,
310,
315,
319,
324,
327,
327,
333,
341,
357,
368,
367,
359,
348,
345,
341,
334,
327,
324,
332,
344,
356,
358,
359,
358,
354,
352,
351,
351,
347,
344,
342,
333,
320,
314,
323,
330,
328,
323,
316,
315,
312,
309,
310,
311,
312,
314,
315,
317,
320,
322,
324,
326,
329,
333,
337,
339,
341,
347,
356,
364,
368,
369,
370,
370,
368,
370,
366,
357,
348,
340,
504,
491,
483,
476,
473,
475,
483,
483,
467,
475,
482,
489,
481,
472,
462,
450,
438,
435,
436,
437,
435,
435,
442,
452,
458,
454,
455,
465,
466,
460,
453,
451,
446,
440,
430,
416,
418,
414,
414,
411,
412,
412,
416,
426,
419,
424,
432,
430,
418,
411,
404,
402,
403,
402,
398,
400,
400,
397,
395,
394,
395,
395,
397,
397,
395,
390,
392,
397,
406,
422,
434,
430,
425,
425,
426,
432,
425,
412,
423,
439,
426,
413,
428,
439,
451,
455,
452,
436,
431,
430,
421,
407,
401,
400,
404,
413,
410,
402,
400,
400,
398,
401,
408,
419,
429,
438,
444,
456,
466,
474,
473,
472,
478,
476,
464,
466,
477,
474,
454,
439,
425,
408,
391,
389,
396,
403,
404,
405,
408,
410,
418,
426,
428,
425,
425,
437,
454,
463,
473,
482,
488,
498,
512,
526,
535,
547,
556,
551,
540,
518,
507,
505,
506,
508,
502,
499,
500,
513,
519,
518,
507,
496,
489,
495,
496,
494,
488,
476,
468,
460,
452,
447,
443,
442,
440,
439,
433,
425,
425,
428,
429,
428,
425,
415,
407,
401,
394,
386,
377,
384,
398,
411,
423,
429,
432,
432,
433,
438,
441,
445,
451,
461,
473,
486,
489,
476,
462,
452,
440,
431,
424,
417,
410,
400,
391,
380,
371,
372,
375,
378,
378,
370,
357,
354,
367,
378,
374,
367,
358,
359,
363,
370,
374,
370,
364,
363,
363,
364,
358,
348,
338,
331,
322,
314,
306,
304,
304,
305,
305,
304,
304,
304,
306,
305,
305,
306,
305,
304,
305,
305,
306,
305,
303,
308,
311,
306,
302,
301,
303,
302,
301,
299,
298,
297,
296,
296,
295,
293,
292,
292,
293,
295,
302,
305,
304,
304,
305,
302,
303,
298,
295,
296,
295,
296,
300,
304,
306,
315,
318,
314,
321,
335,
347,
360,
370,
370,
360,
344,
334,
330,
323,
319,
329,
342,
350,
354,
353,
353,
352,
349,
344,
343,
344,
344,
339,
334,
327,
317,
310,
320,
323,
320,
315,
311,
309,
308,
308,
308,
310,
311,
314,
315,
318,
320,
323,
325,
327,
331,
340,
342,
343,
346,
352,
361,
368,
372,
369,
368,
369,
366,
364,
360,
356,
351,
342,
518,
501,
497,
492,
478,
474,
488,
486,
476,
478,
483,
476,
467,
455,
448,
438,
437,
441,
448,
439,
437,
439,
445,
453,
463,
464,
461,
459,
458,
454,
446,
437,
436,
430,
420,
412,
412,
412,
414,
416,
411,
413,
424,
435,
436,
432,
439,
433,
420,
404,
402,
405,
406,
404,
402,
404,
404,
399,
396,
395,
395,
397,
400,
415,
404,
394,
390,
394,
399,
411,
422,
414,
407,
405,
409,
420,
411,
410,
425,
434,
419,
414,
426,
434,
443,
453,
450,
432,
418,
428,
425,
411,
401,
396,
397,
398,
399,
398,
397,
397,
398,
405,
418,
430,
440,
450,
456,
463,
470,
479,
480,
485,
486,
481,
474,
478,
488,
480,
460,
444,
430,
406,
388,
389,
393,
399,
397,
398,
402,
408,
411,
412,
414,
424,
435,
447,
464,
477,
486,
499,
506,
515,
526,
535,
545,
552,
542,
532,
515,
500,
493,
491,
495,
502,
495,
499,
508,
510,
509,
504,
495,
479,
473,
478,
481,
480,
477,
466,
462,
457,
450,
443,
437,
436,
435,
432,
429,
424,
423,
426,
428,
427,
421,
414,
407,
399,
393,
386,
375,
376,
389,
412,
428,
439,
442,
442,
442,
447,
453,
457,
461,
470,
478,
488,
488,
480,
473,
458,
448,
440,
430,
422,
414,
401,
388,
377,
367,
362,
360,
360,
361,
355,
346,
356,
370,
369,
361,
354,
349,
352,
356,
357,
357,
354,
352,
350,
350,
354,
349,
341,
332,
324,
315,
307,
304,
304,
305,
306,
306,
305,
305,
304,
304,
304,
305,
305,
305,
305,
305,
306,
307,
306,
306,
306,
304,
302,
300,
302,
302,
302,
301,
300,
299,
298,
297,
296,
295,
294,
292,
292,
294,
298,
304,
306,
304,
305,
303,
296,
293,
290,
290,
291,
291,
292,
295,
299,
302,
304,
305,
316,
327,
340,
352,
366,
369,
366,
358,
344,
331,
320,
314,
322,
336,
346,
349,
348,
348,
346,
345,
344,
341,
337,
338,
340,
335,
330,
321,
306,
306,
315,
315,
312,
307,
305,
304,
306,
307,
306,
307,
311,
314,
317,
319,
321,
325,
326,
328,
332,
342,
346,
349,
351,
356,
366,
371,
371,
367,
365,
362,
358,
354,
351,
348,
346,
342,
521,
514,
514,
508,
486,
480,
486,
486,
479,
477,
466,
456,
445,
447,
450,
450,
449,
452,
454,
438,
436,
440,
449,
458,
466,
468,
463,
463,
461,
451,
441,
431,
422,
418,
413,
412,
413,
418,
422,
420,
415,
419,
425,
438,
450,
449,
447,
429,
411,
402,
402,
412,
413,
411,
412,
418,
417,
403,
398,
395,
396,
399,
406,
422,
422,
406,
390,
391,
388,
395,
403,
397,
397,
391,
401,
406,
401,
411,
427,
432,
419,
406,
414,
426,
438,
438,
442,
430,
412,
414,
419,
411,
396,
393,
394,
393,
395,
396,
397,
400,
400,
404,
418,
427,
440,
452,
460,
464,
467,
473,
477,
485,
490,
488,
486,
490,
493,
480,
461,
445,
432,
411,
388,
388,
393,
395,
389,
403,
415,
418,
426,
430,
434,
437,
447,
458,
474,
485,
499,
514,
526,
530,
536,
540,
544,
535,
523,
509,
499,
489,
476,
487,
500,
503,
506,
501,
500,
498,
498,
489,
481,
473,
464,
467,
468,
468,
466,
460,
455,
454,
450,
442,
434,
429,
428,
427,
424,
421,
420,
421,
422,
421,
417,
412,
405,
397,
388,
381,
375,
373,
390,
413,
427,
437,
446,
452,
453,
456,
461,
466,
468,
475,
477,
484,
486,
482,
476,
466,
459,
449,
438,
429,
418,
401,
384,
373,
364,
361,
353,
348,
341,
338,
342,
353,
356,
352,
348,
341,
340,
344,
344,
340,
336,
335,
336,
336,
337,
340,
337,
332,
326,
319,
312,
305,
304,
306,
306,
307,
307,
306,
306,
307,
306,
305,
304,
304,
304,
305,
306,
307,
308,
308,
308,
305,
304,
302,
302,
301,
302,
302,
302,
302,
301,
300,
298,
297,
295,
294,
292,
292,
294,
305,
306,
304,
302,
300,
298,
292,
290,
286,
288,
288,
289,
290,
292,
295,
299,
305,
314,
323,
334,
345,
357,
367,
365,
356,
349,
336,
325,
314,
317,
331,
341,
347,
344,
342,
342,
340,
338,
338,
335,
329,
329,
332,
331,
326,
319,
305,
302,
308,
308,
305,
303,
304,
304,
307,
305,
304,
306,
308,
311,
315,
318,
323,
328,
330,
331,
336,
342,
349,
355,
358,
363,
370,
375,
368,
362,
357,
351,
347,
344,
341,
338,
337,
335,
532,
526,
516,
500,
484,
485,
488,
486,
483,
480,
476,
468,
459,
454,
465,
464,
460,
461,
452,
437,
436,
441,
448,
456,
464,
472,
476,
471,
460,
440,
433,
428,
421,
413,
415,
417,
414,
422,
431,
430,
422,
428,
429,
436,
443,
454,
453,
434,
413,
409,
414,
425,
426,
420,
414,
426,
425,
408,
398,
396,
396,
399,
410,
428,
439,
427,
403,
390,
390,
389,
392,
391,
390,
391,
393,
392,
400,
408,
424,
430,
418,
401,
415,
425,
427,
426,
432,
422,
407,
401,
406,
398,
390,
390,
390,
391,
395,
397,
401,
405,
410,
412,
412,
418,
433,
444,
451,
453,
454,
456,
465,
477,
488,
490,
491,
499,
500,
478,
458,
440,
429,
414,
392,
389,
392,
392,
389,
402,
409,
417,
425,
435,
444,
452,
459,
470,
482,
498,
512,
524,
536,
542,
534,
536,
528,
517,
505,
494,
485,
470,
464,
479,
493,
503,
509,
501,
490,
485,
486,
476,
470,
465,
456,
457,
456,
456,
458,
454,
449,
448,
444,
439,
427,
423,
421,
419,
416,
415,
415,
416,
414,
415,
414,
408,
403,
395,
384,
372,
368,
375,
391,
407,
418,
429,
443,
455,
464,
467,
472,
473,
471,
471,
473,
482,
490,
493,
488,
475,
468,
457,
443,
430,
419,
403,
386,
375,
370,
368,
359,
351,
348,
342,
339,
338,
339,
338,
339,
336,
335,
335,
330,
328,
323,
320,
315,
321,
323,
326,
325,
322,
318,
314,
308,
304,
306,
308,
308,
308,
308,
308,
308,
308,
307,
306,
305,
305,
304,
304,
306,
308,
309,
309,
308,
304,
304,
303,
304,
304,
304,
303,
304,
303,
302,
302,
300,
297,
295,
293,
292,
292,
294,
296,
304,
301,
297,
294,
294,
292,
292,
288,
287,
287,
287,
288,
289,
291,
297,
303,
312,
323,
334,
345,
356,
363,
358,
348,
336,
324,
315,
314,
325,
337,
344,
343,
340,
337,
335,
334,
332,
330,
331,
326,
325,
326,
324,
319,
314,
306,
296,
303,
303,
302,
300,
304,
306,
305,
303,
304,
305,
306,
310,
314,
318,
325,
330,
334,
337,
339,
342,
350,
357,
365,
372,
379,
376,
366,
358,
351,
344,
338,
335,
332,
328,
327,
326,
518,
512,
504,
489,
488,
490,
490,
496,
487,
484,
488,
481,
470,
463,
476,
478,
474,
466,
453,
436,
438,
443,
450,
455,
461,
472,
480,
477,
461,
448,
433,
425,
420,
415,
418,
424,
421,
424,
431,
440,
436,
426,
439,
441,
445,
460,
460,
438,
426,
418,
429,
439,
442,
436,
425,
434,
432,
416,
403,
398,
398,
401,
406,
423,
438,
428,
416,
400,
394,
392,
389,
390,
389,
389,
389,
392,
400,
411,
421,
422,
410,
401,
409,
411,
415,
425,
427,
412,
400,
394,
393,
389,
391,
394,
395,
396,
397,
400,
406,
414,
418,
423,
427,
419,
425,
427,
432,
437,
441,
450,
460,
471,
476,
481,
487,
498,
500,
474,
450,
434,
419,
406,
394,
387,
390,
391,
389,
395,
398,
405,
412,
425,
442,
456,
471,
484,
495,
508,
524,
536,
540,
533,
524,
518,
508,
498,
486,
475,
461,
451,
450,
460,
475,
484,
492,
491,
481,
473,
474,
465,
460,
455,
448,
448,
447,
448,
452,
449,
442,
438,
438,
430,
422,
414,
414,
413,
411,
409,
410,
409,
407,
406,
406,
404,
398,
391,
380,
365,
370,
380,
393,
407,
419,
434,
446,
458,
466,
468,
474,
471,
464,
462,
474,
486,
501,
502,
500,
490,
475,
457,
438,
424,
410,
396,
381,
376,
380,
377,
366,
366,
360,
352,
346,
342,
340,
337,
336,
337,
337,
336,
334,
331,
328,
325,
318,
315,
313,
316,
316,
314,
313,
311,
306,
306,
307,
309,
310,
311,
311,
310,
310,
310,
309,
308,
307,
306,
305,
304,
304,
308,
311,
313,
307,
307,
306,
306,
308,
310,
311,
310,
309,
306,
304,
303,
301,
298,
296,
293,
292,
292,
292,
294,
294,
294,
292,
293,
292,
292,
291,
289,
287,
286,
286,
287,
288,
290,
294,
300,
306,
316,
328,
343,
354,
356,
350,
342,
326,
314,
313,
318,
331,
342,
343,
339,
332,
328,
329,
329,
326,
325,
324,
319,
317,
320,
318,
315,
308,
302,
292,
294,
295,
294,
300,
304,
303,
303,
302,
303,
304,
306,
309,
314,
317,
326,
332,
340,
343,
344,
345,
352,
360,
371,
377,
379,
371,
364,
355,
347,
340,
332,
327,
324,
321,
317,
316,
512,
500,
497,
495,
495,
496,
498,
497,
487,
487,
490,
488,
481,
477,
478,
476,
468,
463,
452,
437,
441,
447,
449,
454,
461,
468,
474,
474,
467,
458,
448,
440,
427,
420,
422,
427,
429,
425,
426,
440,
452,
440,
436,
451,
456,
463,
469,
456,
443,
433,
444,
454,
458,
449,
440,
448,
436,
418,
405,
399,
402,
405,
405,
420,
437,
428,
417,
402,
398,
395,
394,
395,
393,
389,
387,
390,
395,
404,
409,
408,
401,
402,
414,
421,
427,
429,
418,
404,
395,
389,
388,
389,
393,
398,
402,
400,
399,
402,
410,
415,
423,
430,
434,
430,
426,
435,
440,
439,
442,
454,
465,
474,
476,
477,
486,
496,
500,
475,
454,
435,
418,
402,
394,
388,
388,
388,
383,
386,
388,
394,
401,
414,
436,
455,
473,
490,
509,
524,
530,
531,
523,
515,
508,
499,
489,
479,
468,
455,
447,
439,
435,
445,
459,
470,
481,
480,
472,
462,
460,
455,
451,
447,
439,
440,
439,
439,
438,
441,
439,
430,
428,
424,
418,
412,
409,
408,
406,
403,
402,
402,
400,
400,
400,
396,
392,
384,
373,
364,
371,
382,
395,
407,
419,
434,
447,
458,
466,
466,
466,
463,
454,
456,
474,
486,
497,
500,
504,
494,
478,
461,
442,
428,
410,
396,
386,
384,
388,
384,
382,
382,
374,
368,
361,
354,
345,
341,
338,
340,
341,
341,
341,
338,
336,
333,
329,
319,
312,
314,
314,
314,
313,
312,
311,
311,
311,
312,
314,
313,
313,
312,
312,
313,
311,
310,
309,
308,
307,
307,
306,
309,
315,
314,
313,
312,
313,
312,
314,
317,
322,
325,
325,
318,
310,
304,
303,
300,
296,
293,
291,
291,
292,
292,
292,
292,
291,
291,
291,
291,
291,
290,
287,
286,
285,
285,
286,
288,
290,
296,
307,
318,
333,
346,
355,
355,
344,
327,
320,
311,
314,
322,
333,
339,
336,
332,
327,
322,
321,
321,
318,
320,
319,
317,
314,
317,
311,
306,
301,
294,
292,
292,
293,
294,
299,
300,
302,
302,
302,
302,
303,
305,
312,
317,
324,
329,
334,
341,
346,
350,
351,
353,
363,
375,
374,
375,
366,
358,
352,
342,
335,
329,
323,
318,
314,
310,
308,
532,
519,
508,
505,
500,
505,
505,
493,
486,
479,
479,
477,
474,
470,
464,
458,
455,
452,
444,
443,
452,
455,
455,
456,
458,
459,
461,
461,
460,
450,
446,
440,
427,
423,
428,
428,
438,
436,
436,
439,
455,
454,
444,
451,
465,
476,
485,
476,
462,
448,
458,
470,
477,
464,
460,
452,
436,
418,
406,
401,
406,
416,
414,
423,
439,
428,
416,
401,
398,
398,
402,
406,
401,
394,
388,
388,
390,
392,
391,
397,
399,
401,
409,
419,
429,
419,
407,
397,
390,
386,
388,
392,
399,
404,
411,
407,
401,
405,
413,
421,
429,
437,
441,
438,
438,
446,
453,
456,
454,
453,
462,
477,
484,
490,
491,
497,
500,
477,
459,
442,
427,
412,
411,
405,
395,
388,
378,
376,
376,
383,
397,
411,
428,
451,
471,
491,
512,
518,
517,
512,
506,
499,
491,
482,
471,
463,
452,
439,
438,
428,
422,
436,
454,
465,
469,
469,
461,
456,
449,
447,
442,
438,
432,
430,
427,
428,
428,
432,
431,
426,
417,
418,
415,
408,
404,
403,
402,
400,
398,
398,
396,
393,
389,
386,
382,
376,
364,
357,
370,
382,
394,
406,
419,
433,
444,
454,
456,
455,
455,
448,
444,
448,
465,
475,
484,
488,
499,
494,
478,
460,
443,
429,
411,
398,
388,
392,
395,
393,
399,
394,
384,
377,
367,
357,
349,
343,
345,
349,
350,
349,
348,
346,
345,
342,
335,
325,
316,
314,
315,
313,
313,
314,
315,
314,
315,
316,
315,
314,
314,
315,
315,
315,
314,
312,
311,
310,
309,
310,
310,
314,
315,
316,
316,
316,
317,
319,
323,
330,
337,
341,
338,
328,
318,
309,
304,
301,
296,
293,
291,
291,
291,
291,
291,
291,
291,
291,
291,
291,
291,
290,
288,
286,
285,
284,
285,
287,
289,
295,
307,
320,
333,
344,
353,
352,
342,
324,
315,
305,
314,
323,
331,
331,
328,
325,
321,
316,
315,
316,
314,
316,
315,
310,
305,
310,
308,
301,
294,
292,
293,
294,
294,
295,
297,
299,
300,
302,
302,
303,
305,
310,
315,
322,
331,
338,
341,
344,
351,
355,
354,
358,
365,
369,
358,
364,
360,
354,
348,
341,
331,
325,
318,
314,
308,
304,
301,
548,
536,
524,
520,
517,
518,
514,
503,
490,
480,
470,
462,
462,
460,
452,
450,
451,
444,
446,
451,
464,
470,
474,
462,
455,
450,
451,
450,
454,
451,
438,
431,
425,
425,
432,
439,
443,
452,
456,
453,
463,
472,
466,
458,
470,
487,
499,
492,
478,
471,
472,
480,
476,
469,
458,
443,
430,
414,
405,
404,
412,
424,
426,
436,
443,
433,
414,
403,
403,
401,
410,
419,
411,
401,
391,
387,
388,
389,
393,
397,
402,
403,
411,
418,
420,
408,
394,
388,
386,
387,
390,
395,
403,
412,
417,
416,
413,
408,
416,
426,
434,
441,
446,
447,
448,
455,
464,
469,
474,
470,
466,
474,
483,
497,
504,
505,
500,
476,
459,
440,
438,
435,
432,
423,
408,
398,
394,
388,
376,
375,
391,
406,
423,
444,
467,
485,
492,
494,
500,
497,
491,
482,
472,
461,
455,
450,
439,
427,
421,
413,
418,
434,
446,
452,
459,
459,
450,
448,
443,
439,
436,
430,
424,
416,
411,
413,
414,
418,
419,
416,
408,
407,
406,
404,
399,
397,
396,
394,
394,
394,
394,
391,
383,
378,
374,
367,
351,
354,
365,
382,
394,
405,
419,
431,
442,
447,
446,
443,
442,
436,
435,
443,
452,
462,
473,
482,
493,
489,
474,
452,
437,
422,
408,
398,
399,
402,
404,
403,
406,
402,
389,
377,
367,
360,
352,
351,
354,
359,
360,
358,
356,
356,
355,
352,
342,
331,
325,
322,
317,
316,
317,
317,
316,
317,
319,
318,
316,
316,
316,
317,
316,
316,
315,
313,
312,
312,
312,
314,
314,
316,
318,
319,
321,
324,
327,
330,
337,
345,
351,
353,
343,
331,
322,
313,
305,
302,
298,
293,
291,
291,
291,
291,
291,
291,
291,
291,
291,
291,
291,
290,
289,
286,
285,
284,
284,
286,
288,
292,
306,
321,
331,
341,
350,
346,
337,
326,
308,
304,
313,
321,
328,
330,
325,
319,
316,
313,
309,
308,
309,
311,
310,
307,
303,
305,
305,
296,
292,
292,
293,
295,
296,
296,
296,
298,
300,
302,
303,
305,
309,
315,
321,
328,
337,
345,
351,
354,
355,
358,
357,
356,
355,
355,
354,
353,
351,
346,
341,
338,
332,
324,
316,
309,
304,
300,
294,
567,
553,
541,
534,
529,
526,
519,
513,
501,
488,
476,
466,
462,
458,
455,
453,
451,
450,
450,
460,
476,
485,
490,
476,
463,
455,
449,
450,
443,
445,
441,
436,
430,
428,
430,
440,
450,
455,
463,
472,
478,
487,
484,
479,
482,
496,
496,
496,
492,
489,
480,
467,
460,
455,
446,
429,
418,
408,
404,
408,
418,
430,
437,
449,
451,
438,
426,
415,
414,
410,
420,
432,
426,
411,
399,
391,
387,
388,
392,
396,
402,
402,
408,
413,
410,
398,
390,
386,
386,
388,
390,
398,
405,
414,
417,
424,
422,
413,
415,
424,
430,
438,
449,
456,
459,
462,
471,
479,
486,
486,
478,
472,
482,
496,
506,
514,
501,
482,
470,
464,
460,
456,
450,
436,
417,
410,
413,
406,
393,
377,
375,
397,
413,
434,
452,
462,
469,
475,
481,
478,
470,
460,
452,
446,
444,
440,
430,
417,
405,
402,
417,
426,
432,
439,
449,
451,
444,
439,
435,
428,
426,
422,
415,
403,
395,
395,
398,
400,
404,
404,
397,
395,
396,
390,
389,
390,
392,
389,
390,
392,
391,
386,
377,
372,
368,
357,
350,
356,
365,
378,
392,
403,
416,
428,
435,
438,
436,
432,
430,
426,
440,
451,
461,
468,
474,
483,
490,
483,
466,
448,
433,
420,
412,
412,
411,
413,
413,
412,
412,
404,
390,
384,
378,
367,
360,
360,
362,
369,
369,
367,
365,
365,
366,
361,
347,
338,
336,
332,
327,
326,
325,
322,
317,
322,
324,
323,
319,
320,
317,
322,
322,
320,
316,
315,
314,
314,
314,
315,
316,
320,
323,
326,
329,
334,
338,
343,
351,
357,
357,
350,
339,
329,
320,
310,
304,
306,
300,
294,
291,
291,
291,
291,
291,
291,
291,
291,
291,
291,
291,
291,
290,
287,
286,
285,
284,
284,
288,
291,
303,
317,
331,
341,
347,
344,
334,
322,
310,
305,
311,
319,
324,
326,
322,
317,
313,
308,
304,
303,
305,
307,
309,
309,
303,
299,
297,
292,
291,
292,
294,
295,
294,
294,
296,
297,
299,
303,
304,
307,
314,
320,
329,
336,
342,
350,
358,
362,
357,
355,
352,
347,
343,
343,
341,
344,
342,
338,
333,
330,
330,
325,
317,
308,
301,
296,
293,
572,
558,
545,
536,
525,
516,
507,
502,
494,
483,
472,
463,
457,
453,
452,
449,
449,
450,
460,
472,
487,
494,
491,
480,
470,
454,
446,
447,
449,
438,
440,
436,
438,
440,
436,
437,
439,
444,
459,
477,
490,
500,
496,
490,
490,
487,
477,
475,
476,
474,
471,
456,
438,
436,
434,
421,
410,
404,
405,
412,
425,
442,
454,
463,
465,
454,
437,
433,
424,
422,
428,
442,
436,
422,
408,
395,
388,
386,
389,
392,
396,
399,
401,
406,
399,
392,
387,
386,
387,
389,
390,
398,
406,
414,
418,
423,
422,
419,
426,
433,
439,
446,
455,
465,
472,
469,
475,
487,
493,
497,
491,
482,
482,
491,
500,
500,
501,
493,
487,
482,
479,
472,
460,
443,
426,
421,
430,
426,
406,
386,
375,
386,
404,
418,
435,
447,
456,
463,
462,
457,
451,
445,
438,
432,
434,
427,
418,
399,
394,
405,
412,
414,
423,
429,
439,
442,
437,
426,
426,
418,
413,
412,
405,
397,
384,
380,
383,
386,
391,
390,
387,
387,
386,
382,
381,
387,
388,
386,
386,
389,
386,
379,
372,
368,
363,
353,
351,
356,
366,
376,
388,
400,
410,
419,
427,
430,
425,
422,
422,
432,
446,
458,
468,
476,
480,
482,
486,
476,
460,
445,
431,
423,
427,
429,
428,
427,
422,
420,
418,
411,
404,
395,
383,
373,
374,
373,
372,
377,
378,
378,
376,
375,
376,
368,
355,
350,
347,
342,
339,
337,
333,
330,
326,
325,
330,
329,
327,
326,
325,
328,
330,
326,
319,
317,
315,
314,
315,
317,
321,
327,
330,
335,
342,
348,
354,
360,
367,
363,
354,
342,
331,
322,
315,
314,
310,
304,
302,
297,
292,
291,
291,
291,
291,
291,
292,
292,
292,
293,
293,
293,
291,
289,
288,
286,
285,
284,
284,
292,
306,
319,
330,
339,
346,
346,
337,
323,
310,
304,
308,
316,
318,
320,
319,
316,
312,
306,
300,
300,
302,
304,
306,
307,
306,
300,
291,
290,
290,
292,
293,
292,
292,
294,
294,
295,
299,
302,
304,
307,
316,
323,
331,
343,
348,
356,
366,
361,
353,
346,
341,
336,
330,
331,
331,
332,
332,
330,
325,
321,
322,
320,
316,
307,
300,
294,
289,
561,
546,
538,
527,
518,
508,
494,
486,
482,
473,
462,
459,
453,
450,
450,
450,
448,
455,
468,
483,
493,
497,
485,
463,
456,
451,
438,
449,
451,
441,
437,
437,
444,
449,
438,
441,
443,
451,
463,
479,
483,
481,
478,
474,
474,
469,
459,
458,
457,
455,
456,
450,
434,
423,
417,
413,
406,
403,
409,
416,
426,
438,
454,
469,
478,
466,
451,
452,
439,
441,
444,
453,
440,
425,
409,
398,
390,
385,
385,
389,
392,
395,
397,
401,
393,
388,
386,
387,
391,
397,
397,
399,
401,
408,
411,
413,
414,
422,
432,
442,
451,
460,
467,
477,
484,
480,
484,
492,
500,
504,
499,
496,
487,
486,
492,
496,
500,
502,
500,
496,
491,
483,
470,
449,
434,
434,
441,
427,
404,
383,
376,
378,
393,
405,
416,
429,
443,
449,
447,
444,
437,
431,
424,
417,
423,
417,
406,
388,
393,
400,
411,
417,
424,
432,
434,
436,
430,
420,
413,
408,
399,
396,
392,
388,
377,
374,
375,
375,
378,
384,
380,
380,
380,
376,
376,
380,
381,
380,
382,
382,
380,
374,
366,
361,
355,
345,
351,
358,
365,
375,
385,
394,
401,
413,
421,
418,
413,
414,
426,
440,
452,
462,
468,
472,
472,
475,
479,
473,
456,
440,
433,
439,
442,
441,
440,
436,
433,
430,
425,
421,
414,
400,
388,
384,
387,
383,
382,
386,
388,
388,
384,
382,
381,
375,
369,
364,
359,
354,
351,
346,
340,
335,
329,
330,
333,
335,
336,
336,
333,
336,
336,
329,
324,
320,
316,
316,
316,
318,
326,
332,
339,
345,
353,
361,
368,
374,
368,
358,
347,
336,
327,
324,
325,
322,
316,
311,
308,
304,
295,
291,
291,
291,
291,
292,
292,
294,
296,
298,
302,
298,
292,
290,
291,
290,
285,
286,
284,
290,
303,
313,
321,
333,
344,
348,
339,
324,
310,
303,
305,
308,
315,
318,
316,
312,
307,
300,
294,
292,
297,
300,
301,
304,
305,
298,
292,
289,
289,
289,
291,
290,
292,
293,
293,
294,
299,
302,
304,
308,
315,
322,
329,
339,
346,
353,
357,
350,
342,
336,
331,
326,
320,
318,
318,
320,
321,
319,
314,
307,
311,
313,
309,
304,
298,
293,
287,
549,
539,
528,
518,
510,
500,
487,
479,
473,
467,
464,
457,
453,
454,
452,
450,
448,
456,
472,
486,
491,
489,
476,
464,
450,
442,
436,
439,
440,
441,
438,
442,
451,
454,
444,
439,
441,
453,
466,
467,
463,
463,
458,
454,
455,
452,
442,
440,
441,
433,
436,
435,
424,
414,
409,
407,
404,
405,
412,
428,
439,
447,
458,
471,
486,
478,
468,
467,
459,
458,
461,
452,
439,
421,
408,
400,
393,
387,
384,
386,
389,
391,
391,
390,
388,
386,
387,
389,
396,
403,
407,
404,
398,
401,
404,
412,
417,
428,
439,
448,
457,
466,
476,
484,
491,
492,
493,
500,
504,
511,
513,
503,
492,
488,
487,
493,
498,
505,
512,
508,
502,
489,
477,
466,
455,
448,
441,
424,
400,
379,
371,
369,
379,
391,
404,
419,
430,
433,
433,
431,
427,
420,
410,
402,
411,
407,
392,
387,
395,
406,
417,
425,
432,
432,
428,
424,
419,
412,
404,
396,
385,
380,
380,
380,
373,
369,
366,
368,
375,
381,
376,
374,
372,
367,
367,
377,
376,
374,
378,
378,
374,
367,
359,
354,
345,
341,
352,
357,
363,
372,
381,
390,
402,
411,
416,
412,
410,
423,
436,
448,
454,
457,
456,
461,
469,
479,
486,
475,
459,
450,
448,
453,
453,
453,
450,
447,
444,
438,
436,
430,
419,
402,
393,
400,
398,
393,
393,
395,
397,
397,
393,
391,
389,
385,
382,
376,
374,
369,
361,
353,
346,
339,
336,
337,
338,
343,
345,
347,
346,
346,
341,
334,
328,
322,
320,
321,
320,
321,
328,
338,
346,
354,
364,
374,
381,
376,
364,
352,
341,
334,
329,
337,
339,
337,
329,
323,
319,
309,
301,
294,
292,
292,
292,
293,
294,
296,
306,
308,
307,
303,
296,
297,
300,
300,
293,
286,
284,
286,
292,
302,
316,
331,
340,
345,
338,
324,
311,
303,
303,
306,
307,
311,
311,
307,
300,
295,
292,
292,
295,
295,
296,
298,
298,
293,
289,
288,
285,
287,
289,
289,
291,
292,
292,
295,
300,
303,
308,
315,
322,
328,
329,
336,
341,
350,
350,
340,
333,
326,
319,
315,
310,
308,
307,
308,
309,
306,
302,
298,
298,
300,
300,
296,
293,
291,
287,
538,
527,
517,
508,
501,
492,
485,
478,
476,
471,
462,
457,
450,
449,
451,
448,
447,
456,
468,
478,
480,
476,
467,
460,
454,
445,
441,
438,
436,
436,
437,
440,
450,
459,
453,
451,
451,
459,
454,
441,
437,
441,
440,
439,
437,
432,
425,
426,
427,
422,
419,
418,
414,
410,
409,
409,
407,
410,
417,
426,
426,
439,
456,
474,
488,
485,
488,
476,
466,
461,
457,
451,
438,
426,
416,
410,
397,
389,
385,
384,
387,
389,
387,
385,
386,
386,
390,
393,
400,
409,
416,
415,
405,
402,
403,
413,
414,
420,
431,
442,
451,
460,
468,
477,
492,
501,
502,
502,
496,
497,
501,
498,
488,
474,
484,
490,
497,
504,
502,
488,
484,
479,
476,
470,
467,
461,
447,
420,
396,
373,
366,
362,
369,
380,
393,
410,
418,
417,
415,
418,
417,
412,
398,
393,
402,
394,
380,
389,
399,
410,
424,
434,
439,
430,
419,
409,
404,
400,
396,
389,
375,
372,
374,
373,
368,
365,
363,
364,
373,
380,
379,
371,
365,
360,
358,
366,
367,
370,
372,
371,
366,
359,
353,
345,
338,
341,
349,
354,
359,
366,
376,
386,
397,
406,
408,
406,
414,
422,
431,
442,
445,
448,
452,
463,
473,
485,
491,
481,
471,
467,
465,
465,
465,
463,
459,
456,
452,
448,
443,
436,
421,
410,
413,
414,
408,
402,
405,
404,
407,
406,
401,
399,
396,
394,
393,
391,
388,
380,
370,
363,
352,
343,
341,
346,
345,
350,
354,
356,
356,
355,
346,
335,
329,
327,
327,
328,
329,
328,
328,
340,
352,
363,
374,
382,
380,
370,
358,
350,
344,
344,
340,
349,
354,
353,
344,
333,
327,
317,
309,
302,
298,
296,
296,
297,
300,
305,
315,
316,
308,
302,
303,
306,
311,
316,
313,
298,
287,
284,
293,
301,
313,
321,
334,
342,
333,
323,
312,
304,
302,
303,
304,
305,
307,
305,
300,
294,
291,
290,
292,
292,
292,
292,
292,
287,
281,
281,
284,
285,
288,
290,
293,
290,
292,
296,
300,
304,
311,
320,
330,
336,
341,
343,
344,
347,
343,
335,
326,
317,
312,
304,
302,
300,
299,
297,
296,
296,
293,
291,
291,
291,
291,
291,
289,
286,
281,
527,
522,
515,
510,
500,
492,
486,
490,
489,
477,
467,
470,
463,
453,
446,
447,
444,
451,
459,
462,
466,
462,
456,
448,
450,
447,
443,
442,
438,
436,
442,
446,
450,
464,
464,
463,
455,
450,
434,
428,
425,
425,
427,
427,
425,
423,
418,
417,
416,
414,
413,
412,
410,
411,
414,
416,
414,
416,
429,
430,
439,
451,
461,
474,
487,
479,
480,
467,
452,
443,
440,
439,
428,
414,
412,
409,
396,
390,
387,
385,
385,
387,
386,
384,
389,
390,
393,
399,
403,
412,
423,
426,
414,
414,
418,
412,
417,
418,
426,
435,
443,
450,
460,
472,
486,
496,
502,
494,
483,
484,
489,
488,
482,
470,
476,
483,
490,
500,
488,
473,
465,
463,
461,
459,
458,
454,
440,
414,
392,
373,
363,
357,
361,
370,
384,
396,
404,
401,
401,
406,
404,
399,
391,
384,
388,
378,
384,
391,
404,
415,
426,
439,
439,
424,
410,
397,
388,
383,
385,
381,
370,
364,
367,
367,
364,
361,
359,
361,
367,
372,
373,
369,
361,
352,
351,
356,
358,
366,
367,
366,
362,
358,
351,
340,
334,
338,
343,
347,
352,
362,
372,
384,
398,
405,
401,
412,
427,
431,
435,
436,
440,
450,
462,
474,
481,
489,
498,
493,
489,
485,
481,
478,
475,
468,
465,
463,
458,
454,
448,
438,
425,
428,
427,
424,
420,
414,
416,
416,
418,
417,
411,
407,
402,
404,
403,
402,
394,
385,
378,
366,
356,
348,
354,
356,
354,
358,
364,
367,
364,
359,
347,
339,
339,
332,
332,
335,
339,
340,
338,
340,
356,
369,
379,
381,
376,
366,
357,
354,
355,
356,
351,
362,
369,
363,
351,
342,
333,
324,
319,
313,
309,
305,
302,
303,
310,
320,
323,
320,
308,
308,
312,
317,
323,
330,
320,
306,
290,
284,
292,
293,
302,
318,
331,
336,
329,
320,
312,
304,
301,
302,
303,
304,
304,
301,
298,
293,
291,
290,
290,
290,
290,
290,
286,
283,
280,
281,
282,
284,
285,
288,
290,
290,
292,
297,
301,
306,
316,
325,
334,
342,
349,
351,
354,
353,
343,
334,
324,
316,
307,
301,
296,
292,
292,
291,
291,
291,
291,
289,
289,
289,
288,
286,
284,
281,
280,
536,
531,
525,
517,
505,
495,
497,
501,
497,
482,
479,
480,
470,
460,
452,
440,
440,
444,
455,
450,
453,
449,
449,
442,
440,
439,
434,
436,
429,
433,
442,
452,
456,
463,
466,
460,
444,
432,
425,
420,
420,
420,
420,
420,
420,
420,
418,
418,
415,
414,
415,
416,
414,
415,
427,
425,
416,
425,
441,
449,
449,
461,
474,
473,
482,
473,
476,
459,
441,
428,
425,
425,
418,
407,
401,
397,
392,
389,
387,
386,
384,
385,
384,
385,
387,
391,
397,
404,
414,
419,
428,
425,
423,
423,
429,
422,
424,
431,
423,
430,
436,
442,
455,
464,
474,
491,
492,
484,
474,
474,
476,
474,
471,
471,
476,
484,
490,
492,
488,
473,
459,
446,
439,
440,
442,
442,
431,
409,
389,
374,
358,
356,
355,
362,
371,
383,
387,
386,
388,
390,
390,
386,
383,
378,
378,
382,
387,
393,
407,
418,
428,
437,
430,
415,
400,
388,
376,
372,
370,
371,
365,
356,
360,
361,
360,
358,
355,
353,
353,
357,
358,
359,
356,
348,
339,
341,
348,
356,
359,
357,
353,
352,
348,
341,
329,
331,
336,
344,
353,
367,
378,
390,
403,
402,
399,
414,
430,
439,
446,
446,
449,
459,
472,
484,
490,
497,
501,
500,
500,
495,
491,
488,
479,
474,
468,
465,
462,
459,
453,
441,
442,
442,
438,
437,
432,
426,
427,
428,
428,
427,
421,
416,
413,
414,
414,
408,
397,
385,
375,
366,
355,
353,
364,
368,
366,
370,
376,
378,
375,
364,
351,
347,
345,
340,
339,
344,
351,
354,
352,
352,
359,
371,
379,
378,
371,
368,
364,
366,
366,
366,
365,
375,
380,
366,
357,
347,
338,
333,
328,
327,
322,
316,
306,
305,
319,
329,
331,
320,
314,
317,
323,
330,
333,
332,
318,
303,
291,
291,
284,
289,
299,
314,
331,
330,
322,
315,
309,
303,
300,
300,
300,
301,
301,
301,
304,
297,
294,
290,
288,
289,
290,
290,
293,
290,
280,
280,
281,
282,
284,
285,
287,
290,
293,
296,
302,
309,
319,
330,
340,
349,
356,
357,
357,
352,
343,
334,
325,
316,
307,
300,
293,
291,
290,
290,
290,
289,
288,
287,
287,
286,
285,
284,
282,
280,
280,
552,
544,
533,
521,
508,
499,
506,
508,
502,
487,
491,
489,
479,
469,
458,
445,
437,
439,
448,
446,
444,
439,
439,
439,
434,
429,
425,
426,
426,
429,
436,
439,
446,
455,
459,
450,
432,
424,
420,
420,
421,
424,
428,
431,
430,
426,
424,
425,
424,
420,
421,
427,
422,
421,
433,
441,
431,
434,
444,
460,
468,
473,
486,
477,
466,
462,
463,
454,
440,
418,
414,
411,
406,
399,
394,
393,
391,
389,
388,
386,
384,
382,
384,
385,
389,
394,
401,
408,
418,
424,
432,
434,
432,
432,
432,
426,
434,
440,
440,
439,
436,
436,
444,
451,
467,
480,
478,
472,
466,
464,
466,
471,
475,
478,
485,
490,
488,
483,
475,
469,
462,
451,
434,
417,
422,
424,
414,
400,
386,
374,
365,
356,
353,
354,
359,
365,
371,
367,
376,
377,
377,
372,
371,
373,
378,
386,
397,
405,
416,
421,
427,
433,
424,
409,
393,
380,
370,
364,
362,
360,
356,
351,
354,
356,
356,
356,
350,
343,
345,
346,
342,
342,
345,
341,
333,
332,
338,
346,
350,
349,
346,
345,
344,
341,
330,
326,
328,
342,
353,
370,
382,
393,
396,
392,
390,
409,
424,
439,
452,
460,
460,
469,
478,
490,
499,
499,
498,
500,
502,
499,
492,
486,
479,
474,
470,
466,
466,
462,
455,
453,
455,
452,
449,
447,
442,
437,
439,
439,
439,
438,
434,
426,
427,
429,
428,
414,
400,
387,
376,
370,
366,
363,
374,
380,
379,
382,
387,
389,
382,
365,
355,
355,
355,
346,
344,
354,
362,
366,
371,
371,
372,
369,
374,
376,
379,
377,
375,
378,
378,
376,
377,
387,
388,
377,
362,
353,
350,
345,
343,
340,
336,
327,
315,
306,
321,
335,
337,
324,
320,
326,
334,
342,
342,
331,
318,
310,
304,
293,
284,
284,
292,
304,
316,
319,
312,
308,
305,
299,
293,
297,
297,
298,
299,
302,
305,
306,
301,
294,
291,
289,
289,
292,
293,
293,
281,
278,
280,
281,
282,
283,
284,
287,
291,
296,
303,
312,
321,
332,
343,
354,
359,
357,
354,
348,
341,
330,
321,
313,
304,
297,
292,
291,
289,
288,
287,
287,
286,
284,
285,
284,
282,
281,
280,
279,
281,
558,
551,
540,
523,
510,
513,
519,
519,
512,
499,
504,
502,
488,
474,
456,
447,
439,
436,
438,
439,
438,
436,
434,
436,
431,
431,
429,
426,
424,
424,
426,
430,
449,
454,
453,
433,
425,
420,
420,
422,
424,
430,
450,
449,
450,
435,
430,
436,
437,
432,
426,
435,
439,
432,
438,
452,
448,
448,
459,
466,
482,
490,
486,
465,
450,
445,
443,
443,
428,
415,
405,
399,
397,
394,
394,
396,
396,
394,
390,
387,
384,
382,
383,
386,
390,
399,
407,
416,
425,
433,
438,
442,
442,
444,
443,
436,
433,
440,
445,
445,
442,
443,
440,
450,
460,
466,
464,
462,
461,
464,
468,
474,
481,
488,
492,
491,
479,
466,
459,
456,
453,
450,
432,
409,
406,
415,
397,
388,
379,
369,
363,
357,
352,
349,
349,
352,
356,
358,
360,
363,
364,
366,
370,
377,
381,
394,
406,
417,
425,
431,
436,
431,
420,
403,
391,
378,
365,
361,
357,
354,
349,
345,
350,
352,
351,
349,
343,
337,
335,
336,
335,
334,
333,
330,
322,
325,
332,
335,
340,
341,
339,
338,
338,
336,
331,
324,
326,
338,
351,
366,
378,
386,
386,
386,
396,
416,
430,
443,
457,
468,
475,
478,
489,
496,
500,
499,
496,
494,
498,
494,
489,
484,
478,
474,
471,
468,
466,
464,
464,
464,
463,
462,
461,
458,
453,
448,
452,
452,
451,
448,
446,
442,
442,
442,
436,
420,
407,
391,
384,
381,
374,
372,
382,
390,
392,
393,
395,
395,
384,
369,
368,
364,
365,
355,
352,
362,
372,
380,
383,
380,
380,
377,
373,
377,
382,
388,
388,
390,
390,
387,
388,
394,
394,
382,
373,
368,
361,
359,
357,
354,
347,
336,
320,
316,
323,
338,
343,
329,
330,
336,
345,
347,
341,
328,
316,
310,
304,
295,
288,
284,
285,
292,
305,
310,
303,
302,
298,
292,
293,
296,
294,
294,
297,
303,
309,
318,
309,
297,
291,
288,
288,
290,
290,
288,
281,
279,
278,
279,
280,
281,
280,
285,
292,
300,
308,
316,
324,
332,
343,
354,
354,
352,
348,
343,
336,
327,
318,
309,
300,
292,
290,
288,
286,
284,
284,
283,
282,
282,
282,
282,
280,
280,
280,
280,
281,
562,
554,
548,
524,
523,
529,
530,
530,
529,
523,
518,
510,
488,
471,
461,
452,
444,
436,
435,
436,
434,
436,
435,
438,
441,
439,
436,
430,
422,
421,
422,
425,
428,
431,
429,
425,
420,
420,
422,
426,
434,
447,
469,
469,
466,
448,
438,
453,
454,
448,
442,
436,
448,
448,
445,
457,
463,
460,
471,
482,
490,
488,
477,
458,
440,
428,
427,
424,
417,
407,
399,
394,
394,
396,
400,
405,
410,
400,
397,
390,
382,
382,
383,
387,
395,
406,
419,
428,
434,
439,
445,
449,
452,
453,
453,
439,
437,
444,
452,
453,
454,
459,
459,
452,
451,
456,
459,
464,
470,
473,
473,
479,
487,
492,
494,
486,
474,
460,
438,
436,
439,
435,
422,
402,
393,
403,
390,
376,
372,
366,
360,
357,
351,
347,
344,
344,
347,
351,
352,
354,
358,
366,
376,
391,
401,
405,
416,
428,
438,
440,
443,
432,
414,
400,
385,
373,
363,
357,
352,
348,
342,
340,
346,
345,
342,
340,
335,
328,
326,
326,
324,
325,
323,
319,
316,
320,
324,
326,
330,
333,
334,
332,
331,
330,
327,
320,
317,
334,
344,
352,
364,
371,
375,
390,
400,
413,
426,
440,
454,
469,
482,
488,
494,
501,
502,
501,
499,
494,
491,
489,
486,
481,
477,
473,
471,
469,
467,
466,
476,
470,
464,
467,
466,
465,
462,
462,
462,
465,
463,
460,
457,
454,
454,
453,
443,
430,
416,
400,
398,
389,
387,
380,
387,
398,
403,
404,
404,
402,
389,
384,
381,
376,
374,
363,
360,
371,
382,
390,
391,
385,
380,
376,
376,
378,
387,
394,
398,
402,
404,
397,
399,
404,
400,
392,
385,
377,
376,
371,
366,
359,
351,
340,
321,
322,
332,
342,
348,
340,
343,
348,
353,
349,
340,
330,
316,
310,
303,
291,
287,
283,
282,
287,
295,
302,
295,
295,
294,
291,
293,
295,
292,
292,
292,
296,
304,
314,
311,
298,
291,
288,
284,
282,
283,
282,
280,
278,
278,
278,
278,
279,
279,
287,
296,
304,
314,
324,
333,
341,
348,
349,
345,
342,
340,
336,
332,
324,
315,
305,
295,
291,
288,
285,
282,
282,
282,
281,
280,
280,
280,
280,
279,
280,
280,
280,
280,
565,
556,
545,
535,
546,
548,
546,
545,
546,
542,
531,
510,
494,
482,
469,
459,
455,
446,
438,
433,
443,
451,
452,
458,
465,
459,
448,
437,
425,
421,
421,
421,
422,
421,
421,
421,
421,
422,
424,
432,
457,
472,
486,
487,
479,
461,
448,
457,
467,
466,
459,
445,
449,
460,
463,
463,
476,
477,
485,
488,
482,
472,
464,
452,
433,
416,
413,
410,
405,
399,
396,
396,
398,
402,
409,
419,
422,
414,
401,
389,
382,
382,
384,
388,
397,
411,
426,
434,
441,
446,
450,
452,
452,
452,
451,
440,
441,
446,
455,
463,
465,
463,
463,
459,
462,
464,
464,
477,
484,
489,
491,
491,
496,
499,
490,
478,
466,
456,
439,
416,
418,
417,
410,
392,
383,
386,
382,
372,
366,
362,
358,
355,
352,
349,
346,
344,
343,
343,
345,
347,
354,
364,
388,
398,
408,
417,
424,
425,
435,
439,
439,
423,
408,
394,
381,
368,
358,
353,
348,
344,
339,
333,
340,
340,
332,
328,
324,
318,
317,
317,
316,
316,
315,
314,
313,
316,
318,
319,
322,
326,
327,
326,
324,
322,
319,
316,
319,
330,
339,
349,
357,
364,
380,
400,
414,
420,
429,
440,
456,
471,
483,
491,
496,
500,
501,
499,
496,
492,
488,
485,
480,
477,
475,
474,
472,
469,
467,
468,
473,
467,
463,
463,
463,
464,
467,
468,
471,
473,
470,
467,
465,
465,
465,
463,
453,
442,
427,
418,
415,
401,
402,
396,
394,
407,
412,
413,
412,
405,
399,
398,
393,
387,
381,
370,
372,
381,
392,
394,
390,
384,
378,
375,
374,
381,
391,
397,
403,
405,
406,
404,
405,
408,
405,
402,
396,
389,
382,
373,
364,
355,
346,
334,
325,
328,
339,
353,
356,
354,
356,
356,
354,
346,
337,
326,
314,
302,
292,
288,
286,
282,
281,
285,
291,
293,
291,
292,
292,
292,
294,
294,
292,
291,
291,
292,
296,
303,
304,
295,
289,
284,
282,
280,
279,
279,
278,
277,
276,
276,
275,
276,
280,
290,
300,
311,
321,
331,
339,
344,
344,
341,
334,
332,
332,
328,
325,
319,
311,
301,
292,
288,
285,
282,
280,
279,
279,
279,
278,
278,
278,
278,
278,
278,
278,
278,
278,
566,
553,
547,
554,
563,
567,
568,
567,
564,
560,
541,
524,
508,
498,
482,
467,
456,
448,
437,
444,
455,
469,
472,
476,
483,
478,
464,
454,
435,
424,
421,
421,
421,
421,
421,
422,
425,
428,
432,
437,
457,
478,
495,
504,
495,
478,
458,
459,
477,
481,
471,
460,
456,
466,
475,
480,
488,
489,
479,
470,
464,
456,
448,
436,
422,
411,
403,
400,
399,
398,
399,
403,
407,
411,
423,
429,
425,
411,
399,
389,
382,
382,
382,
390,
400,
416,
429,
437,
440,
440,
441,
445,
448,
449,
450,
452,
452,
449,
456,
464,
467,
466,
463,
468,
474,
478,
473,
485,
495,
501,
504,
507,
508,
500,
483,
464,
451,
447,
435,
413,
399,
401,
398,
384,
373,
374,
378,
370,
365,
362,
360,
358,
357,
355,
353,
350,
346,
343,
342,
343,
347,
357,
371,
382,
392,
402,
406,
411,
419,
425,
424,
419,
406,
389,
375,
362,
353,
346,
341,
338,
331,
325,
327,
326,
312,
312,
312,
312,
311,
311,
311,
311,
311,
311,
310,
311,
307,
314,
315,
318,
319,
320,
316,
316,
314,
315,
327,
336,
348,
363,
370,
371,
383,
398,
412,
427,
439,
450,
462,
475,
485,
492,
496,
498,
498,
496,
495,
491,
489,
484,
477,
473,
473,
474,
473,
471,
465,
463,
464,
454,
458,
458,
460,
462,
466,
470,
473,
476,
475,
472,
471,
471,
476,
471,
464,
452,
439,
438,
431,
416,
416,
411,
403,
417,
424,
421,
419,
413,
412,
411,
406,
399,
393,
390,
390,
395,
399,
399,
389,
382,
376,
374,
375,
381,
390,
395,
399,
402,
403,
405,
403,
404,
404,
403,
403,
393,
381,
370,
362,
352,
344,
334,
332,
342,
351,
359,
359,
355,
354,
353,
346,
338,
329,
319,
308,
298,
291,
289,
286,
282,
280,
283,
287,
290,
289,
290,
291,
293,
294,
292,
291,
290,
290,
291,
292,
293,
295,
290,
285,
280,
280,
279,
278,
278,
277,
276,
275,
275,
275,
278,
285,
295,
305,
316,
327,
335,
342,
339,
334,
328,
322,
322,
322,
320,
317,
312,
305,
296,
289,
284,
282,
280,
279,
278,
277,
277,
276,
276,
276,
276,
276,
276,
276,
276,
276,
566,
558,
560,
572,
576,
581,
585,
584,
580,
570,
555,
536,
512,
493,
474,
460,
449,
442,
442,
450,
468,
488,
494,
498,
501,
495,
480,
468,
457,
435,
427,
425,
422,
422,
424,
430,
438,
454,
446,
443,
461,
477,
495,
512,
507,
484,
463,
464,
480,
490,
486,
478,
470,
472,
482,
490,
484,
473,
466,
455,
446,
439,
428,
421,
412,
404,
399,
398,
401,
404,
406,
412,
418,
422,
426,
433,
425,
409,
394,
388,
382,
382,
382,
392,
405,
420,
428,
426,
424,
426,
432,
439,
444,
449,
454,
459,
463,
459,
464,
474,
473,
468,
471,
479,
488,
492,
487,
492,
505,
512,
519,
516,
501,
487,
472,
457,
441,
433,
428,
412,
390,
382,
384,
376,
366,
369,
369,
368,
365,
365,
366,
366,
365,
363,
362,
358,
355,
350,
345,
343,
342,
344,
357,
368,
377,
387,
388,
398,
404,
404,
410,
412,
402,
387,
373,
360,
349,
341,
335,
330,
325,
317,
312,
312,
312,
312,
312,
313,
313,
314,
314,
314,
314,
313,
308,
307,
307,
307,
307,
307,
313,
314,
313,
313,
315,
323,
334,
344,
365,
375,
382,
387,
392,
402,
416,
427,
439,
452,
463,
475,
486,
492,
496,
500,
498,
496,
490,
488,
485,
480,
475,
469,
472,
476,
475,
471,
465,
461,
456,
452,
452,
452,
455,
461,
464,
467,
470,
473,
475,
472,
476,
474,
472,
472,
466,
463,
455,
450,
440,
429,
430,
425,
419,
428,
435,
434,
428,
425,
423,
424,
414,
412,
409,
407,
408,
411,
410,
402,
392,
385,
378,
376,
376,
381,
389,
391,
394,
396,
396,
398,
400,
403,
404,
402,
404,
396,
388,
381,
370,
364,
355,
340,
345,
356,
364,
364,
354,
346,
339,
335,
332,
327,
320,
312,
302,
295,
291,
288,
287,
282,
280,
281,
284,
287,
288,
288,
290,
291,
292,
292,
291,
290,
290,
290,
291,
290,
290,
286,
282,
280,
279,
278,
277,
276,
276,
275,
274,
275,
276,
278,
287,
299,
310,
320,
330,
330,
328,
324,
322,
318,
313,
310,
312,
311,
308,
303,
296,
290,
284,
280,
279,
278,
278,
277,
276,
275,
275,
274,
274,
274,
274,
273,
273,
274,
274,
570,
569,
574,
574,
575,
584,
586,
586,
585,
574,
560,
538,
513,
497,
480,
463,
447,
443,
444,
455,
475,
499,
513,
520,
522,
512,
496,
481,
461,
444,
444,
433,
430,
431,
437,
451,
458,
468,
464,
454,
471,
485,
498,
516,
516,
502,
478,
476,
486,
487,
480,
471,
468,
475,
475,
477,
466,
458,
448,
440,
429,
425,
417,
410,
405,
402,
402,
403,
409,
414,
417,
421,
430,
436,
441,
438,
421,
405,
390,
387,
382,
382,
382,
396,
409,
417,
418,
414,
414,
417,
424,
432,
439,
447,
456,
465,
473,
472,
476,
484,
481,
476,
480,
490,
500,
505,
504,
504,
516,
526,
526,
508,
489,
477,
465,
453,
437,
424,
415,
406,
392,
375,
371,
366,
366,
367,
368,
368,
368,
370,
376,
379,
376,
373,
371,
370,
366,
360,
354,
348,
342,
340,
344,
351,
366,
371,
378,
387,
387,
388,
396,
406,
402,
386,
372,
358,
347,
338,
331,
324,
317,
312,
312,
312,
312,
312,
314,
318,
319,
320,
320,
320,
320,
318,
316,
311,
310,
307,
307,
307,
307,
309,
311,
316,
321,
331,
340,
360,
375,
382,
392,
402,
409,
419,
429,
434,
442,
454,
465,
477,
488,
493,
499,
501,
500,
497,
488,
483,
479,
474,
468,
462,
468,
476,
477,
474,
467,
460,
455,
450,
449,
449,
452,
455,
460,
464,
468,
465,
466,
469,
471,
470,
468,
465,
466,
465,
465,
457,
447,
437,
442,
436,
434,
434,
443,
442,
437,
435,
436,
431,
424,
426,
425,
422,
423,
422,
416,
405,
396,
389,
381,
378,
376,
380,
382,
383,
388,
389,
388,
389,
395,
402,
404,
403,
402,
395,
401,
391,
383,
375,
361,
351,
357,
366,
370,
363,
350,
339,
330,
324,
320,
316,
310,
306,
298,
292,
290,
288,
285,
284,
280,
280,
282,
285,
286,
287,
288,
289,
290,
290,
289,
289,
289,
289,
288,
288,
285,
282,
280,
279,
278,
276,
276,
275,
274,
274,
274,
275,
276,
279,
290,
303,
317,
321,
322,
319,
315,
312,
309,
308,
304,
301,
302,
302,
296,
291,
287,
282,
281,
279,
278,
277,
276,
275,
275,
274,
273,
272,
272,
272,
271,
271,
271,
272,
272,
575,
575,
570,
565,
571,
575,
576,
576,
575,
573,
560,
540,
524,
504,
485,
464,
451,
442,
443,
458,
476,
501,
517,
528,
528,
525,
506,
485,
461,
460,
457,
438,
437,
448,
457,
466,
471,
481,
480,
468,
473,
488,
500,
513,
523,
506,
494,
489,
486,
474,
465,
458,
456,
458,
460,
458,
451,
442,
436,
425,
414,
411,
408,
406,
405,
406,
409,
412,
417,
425,
428,
428,
442,
447,
450,
438,
420,
404,
391,
386,
382,
382,
387,
395,
404,
408,
408,
402,
403,
412,
420,
427,
436,
447,
456,
466,
474,
481,
490,
489,
482,
479,
486,
498,
510,
522,
521,
521,
527,
537,
521,
499,
484,
471,
460,
450,
438,
425,
405,
389,
385,
374,
364,
358,
361,
364,
370,
373,
371,
375,
384,
393,
391,
385,
383,
384,
380,
371,
368,
356,
346,
338,
337,
338,
354,
352,
367,
371,
372,
377,
391,
404,
403,
386,
371,
356,
346,
337,
329,
322,
312,
312,
313,
317,
320,
321,
325,
326,
328,
329,
329,
329,
328,
326,
326,
321,
317,
314,
310,
307,
307,
307,
308,
314,
322,
333,
345,
366,
378,
381,
388,
402,
418,
430,
439,
439,
447,
455,
465,
478,
488,
494,
500,
502,
501,
495,
488,
479,
472,
467,
463,
460,
464,
473,
478,
476,
469,
462,
455,
449,
445,
448,
448,
450,
455,
462,
465,
461,
461,
463,
464,
464,
464,
462,
463,
463,
464,
464,
455,
448,
452,
450,
444,
439,
448,
452,
446,
445,
445,
438,
435,
436,
436,
434,
429,
423,
414,
404,
396,
388,
380,
376,
375,
370,
372,
374,
380,
381,
380,
387,
392,
397,
400,
396,
399,
400,
402,
404,
395,
382,
368,
366,
370,
371,
369,
360,
351,
340,
328,
315,
309,
305,
300,
296,
292,
291,
290,
287,
285,
283,
280,
279,
281,
284,
284,
284,
285,
286,
287,
288,
288,
288,
288,
287,
287,
286,
284,
280,
279,
278,
276,
275,
274,
274,
273,
273,
273,
274,
277,
280,
294,
306,
316,
314,
310,
307,
303,
300,
298,
297,
296,
292,
291,
289,
285,
280,
280,
279,
278,
277,
277,
276,
275,
274,
273,
272,
271,
271,
270,
270,
269,
269,
269,
270,
270,
576,
573,
563,
555,
560,
561,
562,
563,
564,
562,
547,
533,
518,
502,
483,
463,
449,
442,
439,
450,
471,
494,
504,
511,
513,
512,
501,
480,
465,
474,
463,
441,
440,
451,
465,
476,
486,
494,
495,
486,
479,
491,
504,
514,
518,
505,
493,
486,
473,
461,
452,
443,
436,
439,
439,
439,
440,
433,
429,
416,
409,
406,
407,
409,
412,
416,
420,
423,
426,
433,
445,
446,
452,
449,
438,
429,
416,
401,
391,
387,
387,
383,
379,
386,
390,
397,
397,
393,
402,
414,
426,
435,
442,
448,
455,
461,
467,
471,
478,
472,
467,
480,
493,
506,
522,
532,
538,
538,
538,
534,
515,
494,
480,
464,
455,
446,
435,
425,
408,
390,
376,
370,
360,
357,
357,
362,
367,
373,
378,
382,
390,
403,
406,
398,
396,
396,
390,
382,
378,
364,
353,
342,
336,
331,
332,
342,
356,
356,
369,
383,
397,
408,
408,
388,
371,
357,
345,
336,
328,
318,
312,
312,
318,
328,
332,
331,
335,
336,
339,
339,
338,
338,
335,
335,
336,
332,
328,
322,
318,
313,
309,
307,
307,
309,
316,
325,
342,
357,
364,
366,
376,
395,
415,
430,
441,
443,
452,
458,
466,
477,
486,
491,
494,
498,
495,
489,
484,
478,
468,
461,
459,
455,
459,
467,
473,
472,
467,
462,
455,
448,
440,
442,
445,
448,
450,
454,
456,
454,
453,
456,
461,
461,
460,
460,
461,
462,
464,
465,
463,
454,
459,
459,
453,
448,
455,
459,
457,
456,
454,
448,
446,
443,
443,
440,
430,
420,
410,
400,
391,
385,
377,
377,
378,
371,
363,
364,
370,
374,
377,
384,
391,
392,
392,
392,
397,
399,
401,
403,
402,
393,
387,
381,
376,
365,
356,
351,
344,
336,
327,
312,
303,
301,
297,
292,
291,
290,
289,
287,
284,
282,
279,
279,
280,
283,
282,
282,
282,
284,
285,
286,
287,
288,
287,
286,
286,
284,
282,
281,
279,
277,
276,
274,
273,
273,
272,
272,
272,
273,
276,
283,
295,
307,
306,
302,
299,
297,
292,
290,
288,
286,
286,
283,
280,
280,
280,
279,
278,
277,
276,
276,
275,
275,
274,
273,
272,
271,
270,
269,
269,
268,
268,
268,
268,
268,
269,
558,
563,
561,
549,
541,
545,
547,
549,
549,
542,
532,
521,
506,
492,
473,
458,
448,
437,
446,
455,
465,
480,
488,
491,
497,
508,
500,
480,
474,
482,
469,
444,
442,
449,
459,
472,
489,
502,
506,
504,
495,
498,
512,
517,
506,
494,
483,
471,
460,
448,
441,
434,
430,
428,
427,
426,
425,
423,
418,
412,
409,
410,
412,
418,
425,
430,
435,
437,
438,
437,
440,
452,
456,
446,
432,
421,
410,
401,
399,
397,
398,
389,
382,
377,
379,
387,
388,
390,
404,
416,
427,
433,
440,
443,
448,
454,
451,
451,
458,
458,
470,
489,
500,
511,
514,
521,
528,
528,
528,
525,
514,
492,
477,
460,
449,
441,
431,
420,
406,
391,
375,
364,
356,
355,
356,
360,
373,
376,
382,
393,
398,
410,
418,
413,
412,
411,
402,
396,
387,
375,
362,
350,
340,
333,
330,
334,
342,
355,
369,
385,
401,
414,
411,
388,
370,
353,
342,
333,
323,
312,
312,
316,
328,
339,
342,
339,
344,
345,
352,
351,
348,
346,
342,
345,
346,
342,
336,
331,
328,
324,
316,
309,
307,
308,
311,
318,
332,
345,
351,
364,
377,
394,
412,
422,
434,
441,
452,
459,
466,
474,
480,
484,
488,
488,
484,
480,
477,
473,
467,
460,
454,
450,
455,
462,
467,
465,
461,
455,
453,
446,
438,
439,
441,
444,
448,
450,
449,
449,
450,
453,
456,
457,
455,
456,
459,
461,
464,
465,
465,
463,
464,
467,
464,
456,
459,
466,
465,
464,
461,
456,
454,
448,
439,
429,
422,
414,
405,
398,
391,
383,
383,
389,
388,
383,
378,
366,
364,
365,
374,
380,
384,
385,
386,
390,
392,
400,
402,
404,
403,
396,
389,
381,
372,
362,
349,
340,
336,
330,
320,
307,
305,
308,
305,
298,
292,
291,
289,
287,
285,
283,
281,
279,
279,
280,
280,
280,
282,
282,
283,
286,
288,
288,
287,
286,
285,
284,
282,
280,
279,
277,
276,
274,
273,
272,
271,
270,
270,
271,
275,
282,
291,
297,
298,
294,
290,
288,
283,
280,
280,
280,
280,
279,
278,
278,
277,
276,
276,
275,
275,
274,
274,
273,
272,
271,
270,
269,
268,
268,
268,
267,
267,
267,
267,
268,
268,
554,
541,
545,
544,
524,
523,
527,
529,
532,
521,
509,
500,
490,
476,
464,
453,
443,
451,
458,
464,
469,
474,
478,
486,
502,
516,
508,
491,
479,
490,
476,
451,
455,
460,
464,
470,
482,
496,
509,
519,
516,
516,
526,
514,
495,
478,
467,
453,
445,
436,
432,
430,
429,
430,
428,
424,
420,
419,
418,
416,
415,
418,
421,
428,
438,
448,
450,
456,
458,
457,
453,
454,
460,
450,
442,
426,
415,
414,
402,
411,
414,
405,
389,
381,
377,
380,
384,
389,
401,
408,
413,
426,
433,
437,
441,
444,
439,
437,
441,
456,
476,
483,
486,
490,
499,
507,
510,
508,
506,
509,
506,
489,
471,
454,
441,
433,
426,
414,
399,
386,
375,
366,
357,
354,
356,
364,
375,
383,
389,
403,
408,
418,
431,
430,
422,
425,
418,
410,
397,
386,
373,
359,
346,
338,
330,
328,
339,
352,
373,
392,
405,
414,
404,
384,
368,
353,
342,
329,
317,
313,
313,
319,
332,
342,
347,
347,
352,
354,
363,
364,
358,
354,
350,
356,
356,
349,
341,
339,
340,
336,
329,
312,
307,
307,
309,
313,
324,
331,
345,
361,
376,
392,
406,
416,
428,
435,
449,
455,
461,
467,
471,
475,
478,
478,
474,
471,
468,
466,
464,
460,
452,
450,
451,
456,
460,
462,
458,
450,
448,
442,
438,
436,
438,
441,
445,
447,
449,
451,
452,
455,
455,
454,
452,
452,
455,
460,
463,
465,
466,
468,
467,
467,
466,
463,
463,
469,
468,
469,
466,
462,
455,
448,
438,
429,
418,
409,
400,
395,
389,
388,
395,
400,
398,
394,
388,
379,
363,
355,
366,
374,
377,
376,
383,
391,
396,
402,
403,
402,
398,
392,
381,
374,
367,
358,
348,
338,
327,
321,
314,
312,
316,
317,
309,
302,
294,
291,
289,
287,
285,
283,
281,
280,
279,
279,
280,
279,
280,
280,
282,
291,
293,
290,
288,
286,
284,
283,
282,
280,
279,
277,
276,
274,
273,
272,
270,
269,
269,
270,
273,
279,
286,
290,
290,
286,
281,
280,
279,
278,
278,
278,
277,
277,
276,
275,
275,
274,
274,
273,
273,
273,
272,
272,
271,
271,
270,
269,
268,
268,
268,
267,
267,
267,
267,
267,
267,
559,
541,
523,
523,
519,
506,
508,
508,
505,
498,
489,
480,
472,
463,
455,
451,
452,
463,
474,
476,
474,
484,
492,
494,
510,
524,
524,
502,
498,
492,
482,
460,
465,
476,
475,
476,
479,
492,
508,
526,
534,
533,
527,
509,
487,
467,
455,
446,
437,
433,
432,
432,
435,
439,
437,
432,
426,
425,
426,
426,
426,
430,
434,
438,
450,
465,
470,
473,
479,
476,
468,
465,
470,
464,
452,
436,
424,
415,
421,
426,
426,
412,
401,
386,
379,
377,
380,
386,
391,
397,
407,
416,
420,
424,
428,
427,
422,
431,
443,
456,
464,
470,
474,
478,
489,
496,
494,
486,
485,
490,
495,
482,
462,
448,
431,
421,
415,
402,
387,
375,
365,
359,
356,
354,
357,
368,
380,
390,
397,
408,
419,
430,
441,
437,
438,
437,
428,
423,
412,
396,
382,
367,
354,
346,
337,
330,
334,
349,
373,
391,
395,
397,
389,
376,
361,
347,
336,
324,
314,
313,
318,
329,
337,
346,
354,
352,
359,
366,
369,
371,
368,
361,
357,
364,
366,
357,
352,
352,
353,
347,
339,
317,
307,
307,
307,
310,
316,
331,
349,
364,
377,
392,
404,
414,
421,
430,
440,
447,
453,
458,
461,
465,
468,
466,
463,
464,
460,
458,
455,
455,
453,
448,
448,
450,
455,
455,
454,
449,
443,
439,
436,
438,
442,
447,
451,
449,
450,
453,
452,
452,
452,
452,
449,
450,
454,
459,
464,
466,
467,
467,
468,
468,
467,
466,
460,
466,
468,
467,
464,
463,
456,
450,
441,
431,
420,
410,
404,
404,
402,
404,
406,
404,
403,
396,
390,
381,
368,
352,
357,
365,
369,
378,
385,
391,
396,
400,
400,
399,
393,
386,
377,
370,
361,
352,
345,
337,
328,
316,
316,
322,
325,
320,
311,
303,
296,
292,
290,
288,
286,
284,
282,
282,
280,
279,
279,
279,
279,
280,
280,
286,
291,
290,
289,
286,
282,
282,
281,
280,
279,
277,
276,
274,
273,
272,
270,
269,
269,
269,
271,
275,
280,
284,
284,
281,
279,
278,
278,
277,
276,
276,
275,
274,
274,
273,
273,
272,
272,
271,
271,
271,
271,
271,
271,
270,
268,
268,
268,
268,
268,
267,
267,
267,
267,
267,
267,
562,
543,
522,
500,
505,
498,
495,
493,
488,
482,
475,
469,
463,
457,
454,
449,
459,
474,
489,
493,
487,
495,
508,
514,
522,
529,
526,
508,
487,
486,
484,
462,
470,
478,
489,
492,
485,
495,
508,
525,
538,
536,
521,
506,
485,
467,
445,
441,
439,
439,
437,
436,
443,
455,
453,
445,
431,
435,
444,
441,
436,
446,
451,
449,
453,
470,
483,
490,
496,
495,
482,
483,
481,
470,
454,
437,
433,
431,
438,
441,
430,
412,
397,
386,
380,
378,
377,
380,
384,
388,
396,
405,
409,
410,
412,
413,
421,
432,
441,
446,
451,
458,
466,
474,
479,
480,
477,
471,
469,
478,
480,
470,
455,
439,
425,
409,
402,
387,
374,
365,
353,
351,
350,
351,
356,
371,
384,
396,
405,
415,
424,
437,
442,
448,
452,
450,
444,
440,
423,
404,
389,
370,
356,
355,
349,
335,
332,
342,
359,
368,
376,
377,
372,
364,
354,
339,
329,
319,
313,
313,
325,
338,
347,
350,
356,
357,
366,
370,
376,
379,
375,
370,
365,
370,
371,
367,
364,
365,
366,
358,
346,
325,
309,
307,
307,
309,
314,
329,
349,
366,
379,
390,
400,
406,
411,
422,
433,
441,
445,
448,
450,
456,
458,
456,
454,
454,
452,
450,
449,
451,
452,
452,
444,
443,
449,
453,
450,
445,
440,
437,
436,
442,
450,
453,
451,
451,
449,
448,
446,
446,
447,
447,
448,
452,
456,
460,
464,
466,
466,
467,
468,
468,
467,
466,
465,
466,
467,
466,
467,
464,
458,
451,
440,
430,
422,
419,
417,
415,
413,
411,
405,
400,
395,
390,
385,
379,
368,
353,
352,
363,
371,
380,
385,
390,
394,
396,
398,
396,
392,
384,
376,
364,
354,
345,
339,
331,
323,
316,
324,
331,
330,
321,
312,
303,
297,
292,
290,
288,
286,
284,
283,
282,
281,
279,
279,
279,
279,
279,
277,
281,
287,
293,
292,
286,
282,
280,
280,
279,
278,
278,
276,
275,
274,
272,
271,
270,
269,
269,
269,
272,
276,
278,
278,
278,
276,
276,
275,
275,
274,
273,
273,
272,
272,
271,
271,
270,
270,
269,
269,
269,
270,
270,
270,
268,
270,
269,
268,
268,
267,
267,
267,
267,
267,
267,
266,
554,
536,
515,
507,
501,
497,
495,
492,
486,
481,
476,
472,
467,
460,
460,
463,
465,
477,
495,
507,
500,
504,
517,
528,
534,
535,
520,
493,
487,
482,
473,
473,
480,
487,
501,
509,
506,
499,
511,
523,
534,
524,
512,
500,
478,
462,
446,
440,
443,
448,
450,
438,
447,
464,
469,
461,
444,
440,
456,
458,
450,
456,
467,
468,
467,
466,
473,
476,
495,
502,
500,
491,
479,
465,
450,
450,
452,
448,
446,
439,
421,
405,
389,
385,
382,
380,
379,
377,
377,
379,
384,
391,
395,
399,
407,
422,
431,
431,
431,
436,
444,
453,
463,
462,
462,
464,
463,
459,
451,
463,
467,
458,
446,
432,
416,
397,
383,
371,
361,
352,
342,
344,
345,
347,
355,
368,
383,
393,
401,
409,
417,
427,
441,
450,
449,
445,
443,
434,
425,
411,
392,
379,
376,
369,
354,
343,
331,
332,
340,
350,
358,
360,
356,
350,
343,
333,
322,
314,
313,
317,
332,
345,
356,
357,
357,
366,
364,
372,
380,
384,
383,
378,
376,
380,
380,
377,
379,
380,
376,
364,
348,
340,
315,
306,
306,
308,
312,
322,
349,
365,
377,
385,
391,
399,
408,
420,
429,
434,
436,
437,
442,
446,
447,
446,
443,
442,
445,
445,
446,
447,
448,
449,
444,
437,
441,
449,
447,
442,
436,
436,
437,
442,
450,
451,
452,
448,
442,
439,
439,
438,
439,
445,
450,
453,
453,
461,
465,
465,
463,
465,
466,
467,
466,
466,
466,
466,
467,
466,
464,
461,
457,
451,
444,
439,
434,
429,
426,
419,
415,
408,
403,
394,
387,
383,
376,
371,
362,
351,
354,
364,
370,
378,
383,
388,
391,
394,
395,
394,
390,
380,
371,
362,
351,
342,
334,
329,
327,
330,
330,
336,
333,
324,
315,
307,
299,
292,
290,
288,
286,
284,
283,
280,
280,
280,
279,
279,
279,
279,
280,
280,
288,
291,
289,
285,
281,
280,
279,
279,
279,
278,
277,
276,
274,
273,
272,
270,
269,
269,
269,
269,
272,
274,
274,
274,
274,
273,
273,
273,
272,
271,
271,
270,
270,
269,
269,
269,
268,
268,
268,
268,
269,
269,
268,
269,
269,
269,
269,
268,
267,
267,
267,
269,
269,
267,
266,
534,
523,
513,
507,
506,
507,
508,
504,
499,
493,
488,
484,
476,
469,
469,
474,
474,
487,
503,
516,
520,
521,
532,
541,
546,
535,
514,
493,
504,
502,
493,
485,
489,
502,
514,
526,
527,
519,
522,
531,
531,
516,
501,
483,
464,
454,
446,
445,
449,
460,
461,
454,
449,
468,
479,
474,
460,
449,
463,
472,
467,
461,
476,
487,
488,
487,
487,
491,
500,
511,
502,
486,
471,
462,
461,
464,
464,
457,
438,
425,
409,
394,
387,
384,
382,
381,
380,
378,
376,
376,
376,
378,
385,
398,
414,
420,
424,
426,
417,
427,
437,
440,
441,
441,
443,
452,
452,
450,
442,
449,
454,
446,
436,
425,
403,
380,
366,
355,
347,
341,
338,
339,
340,
343,
348,
359,
370,
379,
387,
399,
402,
413,
430,
438,
427,
423,
425,
419,
413,
408,
398,
389,
382,
370,
356,
343,
322,
322,
330,
338,
342,
341,
339,
334,
328,
321,
313,
313,
313,
322,
335,
348,
361,
367,
362,
368,
370,
374,
380,
390,
391,
386,
387,
391,
391,
390,
391,
389,
377,
364,
356,
354,
323,
306,
306,
307,
311,
319,
347,
358,
368,
376,
387,
397,
407,
416,
424,
426,
426,
425,
434,
439,
439,
438,
436,
436,
438,
440,
444,
439,
440,
441,
441,
438,
438,
442,
441,
440,
433,
432,
436,
442,
447,
448,
448,
444,
439,
436,
437,
440,
442,
444,
445,
448,
452,
457,
461,
458,
459,
463,
465,
465,
465,
463,
463,
466,
462,
461,
459,
455,
454,
452,
446,
441,
436,
427,
421,
415,
408,
402,
395,
386,
378,
376,
371,
361,
354,
346,
351,
358,
366,
377,
381,
386,
391,
395,
393,
393,
387,
376,
363,
353,
345,
344,
341,
344,
338,
340,
339,
342,
334,
327,
318,
307,
299,
292,
290,
288,
286,
284,
282,
281,
280,
280,
279,
279,
279,
279,
279,
283,
285,
287,
286,
281,
280,
279,
279,
279,
279,
280,
278,
277,
276,
274,
273,
271,
270,
269,
269,
269,
270,
271,
271,
271,
271,
271,
271,
271,
270,
269,
269,
269,
268,
268,
268,
268,
268,
268,
268,
268,
268,
268,
269,
269,
269,
270,
270,
270,
269,
268,
271,
274,
272,
268,
265,
520,
514,
510,
510,
515,
522,
528,
526,
518,
513,
511,
496,
484,
477,
482,
490,
489,
493,
510,
525,
536,
538,
548,
548,
544,
528,
505,
505,
518,
518,
510,
497,
493,
509,
526,
540,
545,
538,
539,
539,
527,
509,
486,
465,
460,
454,
453,
455,
455,
461,
474,
473,
467,
474,
489,
485,
472,
457,
471,
485,
484,
477,
484,
498,
506,
507,
508,
511,
513,
507,
496,
483,
474,
477,
476,
475,
471,
460,
440,
422,
404,
390,
386,
385,
383,
382,
380,
378,
380,
379,
378,
376,
378,
384,
394,
396,
404,
403,
398,
408,
417,
421,
421,
423,
427,
433,
436,
437,
429,
439,
443,
437,
428,
410,
390,
366,
353,
344,
339,
334,
333,
334,
336,
338,
340,
347,
354,
362,
375,
385,
390,
404,
412,
422,
414,
404,
404,
402,
392,
393,
395,
390,
378,
364,
353,
341,
322,
324,
322,
326,
329,
329,
326,
322,
318,
313,
313,
313,
317,
329,
340,
353,
365,
373,
365,
369,
378,
377,
380,
390,
395,
391,
394,
402,
402,
404,
400,
388,
377,
372,
370,
364,
332,
310,
306,
307,
309,
316,
327,
357,
369,
378,
384,
392,
402,
410,
416,
418,
415,
418,
427,
431,
432,
431,
429,
432,
435,
439,
442,
441,
437,
436,
435,
432,
433,
437,
436,
431,
427,
428,
435,
442,
444,
443,
443,
439,
437,
432,
439,
441,
443,
440,
439,
446,
451,
454,
454,
450,
451,
456,
461,
461,
462,
464,
454,
459,
452,
453,
451,
449,
449,
446,
440,
434,
429,
422,
415,
408,
401,
392,
385,
380,
372,
367,
363,
357,
348,
341,
347,
357,
367,
372,
377,
381,
391,
393,
391,
389,
383,
377,
367,
362,
354,
359,
361,
354,
350,
347,
344,
341,
332,
323,
316,
305,
298,
292,
290,
288,
286,
284,
282,
280,
280,
280,
280,
280,
280,
279,
279,
279,
281,
284,
284,
282,
280,
279,
279,
280,
280,
281,
280,
278,
276,
275,
274,
273,
271,
270,
269,
269,
269,
269,
269,
269,
269,
269,
269,
269,
269,
269,
269,
270,
270,
270,
270,
270,
270,
270,
269,
269,
268,
268,
269,
270,
271,
272,
273,
273,
272,
273,
278,
281,
276,
268,
264,
514,
509,
510,
516,
526,
540,
543,
544,
541,
535,
527,
504,
490,
484,
492,
502,
509,
507,
512,
530,
549,
554,
561,
556,
539,
518,
508,
515,
526,
531,
524,
512,
499,
511,
530,
547,
561,
552,
542,
533,
519,
503,
483,
468,
461,
463,
466,
472,
472,
465,
477,
488,
488,
482,
493,
499,
484,
474,
477,
492,
501,
497,
499,
513,
512,
514,
515,
511,
503,
492,
483,
476,
488,
491,
490,
485,
470,
452,
434,
416,
399,
389,
387,
386,
385,
385,
384,
384,
385,
387,
387,
386,
376,
376,
375,
376,
384,
384,
384,
391,
391,
400,
402,
405,
410,
415,
418,
422,
423,
431,
436,
428,
411,
394,
377,
355,
347,
340,
338,
333,
330,
331,
332,
333,
335,
338,
341,
354,
364,
368,
377,
389,
398,
404,
400,
392,
384,
382,
378,
382,
393,
385,
368,
356,
344,
322,
322,
322,
318,
319,
321,
320,
318,
313,
313,
315,
316,
318,
329,
337,
344,
358,
371,
380,
372,
371,
381,
384,
386,
391,
397,
399,
403,
411,
415,
414,
401,
389,
386,
383,
378,
364,
347,
320,
306,
306,
306,
312,
323,
353,
362,
367,
373,
385,
395,
403,
408,
407,
402,
407,
417,
424,
427,
425,
424,
428,
430,
437,
440,
440,
435,
426,
428,
426,
428,
431,
429,
421,
418,
427,
437,
442,
441,
439,
440,
433,
428,
434,
439,
440,
437,
435,
436,
442,
449,
452,
448,
442,
446,
452,
454,
454,
455,
454,
453,
450,
445,
444,
444,
443,
443,
442,
437,
425,
419,
414,
407,
400,
394,
386,
377,
370,
365,
357,
357,
353,
345,
339,
346,
353,
362,
366,
374,
381,
388,
390,
385,
380,
375,
370,
375,
375,
367,
374,
372,
368,
363,
354,
347,
338,
329,
320,
310,
303,
297,
293,
291,
288,
286,
283,
281,
280,
280,
280,
280,
281,
280,
280,
279,
279,
279,
279,
284,
282,
281,
279,
280,
281,
282,
282,
280,
278,
278,
277,
276,
274,
273,
271,
270,
269,
269,
269,
269,
269,
269,
269,
269,
269,
269,
270,
273,
274,
275,
275,
275,
274,
274,
274,
273,
271,
270,
271,
271,
273,
275,
276,
277,
278,
278,
281,
284,
284,
277,
269,
262,
512,
509,
513,
523,
540,
550,
558,
561,
558,
543,
527,
507,
493,
488,
495,
510,
521,
527,
525,
534,
552,
562,
570,
548,
536,
513,
522,
527,
532,
539,
542,
528,
512,
514,
532,
548,
564,
544,
532,
521,
508,
490,
476,
469,
471,
477,
483,
484,
489,
482,
476,
493,
502,
498,
497,
508,
504,
493,
486,
490,
499,
507,
513,
513,
498,
497,
502,
494,
485,
478,
471,
474,
487,
495,
490,
477,
460,
440,
423,
407,
390,
387,
385,
387,
389,
388,
388,
388,
392,
400,
399,
398,
392,
386,
380,
375,
372,
372,
373,
373,
374,
378,
383,
391,
394,
398,
406,
412,
415,
424,
425,
411,
395,
379,
365,
352,
345,
343,
343,
338,
333,
332,
333,
332,
331,
333,
338,
345,
353,
358,
370,
381,
387,
388,
383,
374,
367,
366,
371,
386,
392,
382,
366,
354,
340,
322,
322,
320,
319,
319,
319,
318,
313,
314,
316,
319,
320,
323,
328,
339,
349,
360,
374,
382,
378,
377,
387,
394,
393,
396,
404,
402,
408,
418,
427,
417,
402,
397,
396,
390,
380,
365,
349,
334,
313,
306,
307,
310,
318,
341,
347,
352,
364,
375,
386,
396,
399,
394,
388,
395,
407,
415,
418,
418,
419,
422,
427,
432,
436,
435,
430,
423,
420,
419,
420,
427,
422,
412,
417,
428,
438,
440,
437,
435,
433,
427,
427,
433,
436,
436,
432,
426,
432,
439,
445,
447,
444,
437,
438,
442,
448,
450,
447,
447,
442,
443,
438,
436,
436,
436,
437,
438,
438,
428,
416,
405,
400,
391,
384,
378,
370,
362,
355,
352,
350,
344,
342,
340,
348,
356,
362,
366,
374,
380,
383,
382,
377,
370,
364,
366,
378,
386,
386,
383,
379,
370,
364,
355,
345,
336,
326,
315,
308,
302,
297,
294,
291,
288,
286,
283,
281,
280,
280,
280,
280,
281,
280,
280,
280,
281,
279,
279,
279,
279,
279,
279,
281,
283,
283,
282,
280,
280,
280,
279,
278,
276,
275,
273,
272,
270,
269,
269,
269,
269,
269,
269,
269,
270,
272,
275,
279,
281,
282,
283,
281,
280,
279,
278,
276,
275,
275,
277,
279,
278,
280,
280,
284,
286,
289,
289,
291,
284,
277,
266,
262,
512,
513,
519,
529,
545,
558,
572,
574,
555,
541,
524,
506,
494,
491,
498,
507,
522,
536,
542,
545,
556,
570,
572,
549,
532,
528,
539,
547,
547,
546,
552,
540,
533,
534,
541,
555,
557,
536,
519,
503,
491,
482,
476,
478,
486,
496,
504,
501,
504,
492,
486,
495,
510,
513,
509,
517,
520,
511,
504,
507,
512,
512,
510,
508,
497,
483,
489,
482,
465,
458,
459,
472,
487,
493,
480,
464,
444,
428,
413,
399,
390,
386,
386,
389,
393,
391,
392,
393,
400,
407,
412,
412,
407,
393,
385,
377,
370,
367,
366,
366,
368,
370,
372,
370,
374,
382,
392,
392,
399,
405,
405,
390,
379,
367,
356,
347,
344,
354,
355,
345,
341,
339,
342,
334,
331,
331,
333,
338,
346,
356,
366,
371,
372,
370,
366,
359,
354,
360,
374,
388,
390,
379,
364,
351,
334,
322,
322,
321,
322,
324,
324,
321,
317,
320,
324,
328,
330,
328,
335,
341,
350,
360,
374,
388,
384,
385,
394,
400,
400,
403,
409,
411,
415,
424,
430,
423,
409,
408,
402,
392,
380,
365,
354,
338,
322,
309,
306,
306,
313,
318,
330,
345,
356,
365,
378,
388,
388,
380,
374,
383,
396,
406,
410,
411,
413,
415,
420,
426,
429,
429,
423,
415,
410,
411,
414,
419,
415,
407,
418,
430,
434,
435,
433,
430,
429,
425,
427,
428,
431,
429,
428,
425,
428,
436,
441,
441,
439,
432,
427,
435,
441,
442,
439,
439,
437,
434,
429,
427,
427,
428,
426,
434,
438,
430,
420,
410,
401,
392,
387,
381,
378,
374,
368,
363,
355,
346,
340,
339,
347,
356,
364,
368,
373,
378,
378,
374,
368,
363,
360,
370,
379,
389,
386,
381,
374,
364,
357,
348,
339,
330,
322,
314,
305,
300,
295,
292,
290,
288,
286,
283,
281,
280,
280,
281,
282,
283,
281,
281,
280,
280,
280,
281,
279,
279,
279,
280,
282,
284,
284,
282,
282,
282,
282,
280,
279,
278,
276,
275,
274,
273,
271,
271,
271,
271,
270,
270,
271,
274,
278,
282,
286,
289,
290,
290,
288,
286,
285,
284,
280,
277,
281,
287,
293,
289,
287,
290,
294,
296,
302,
305,
293,
283,
274,
264,
261,
514,
520,
530,
537,
550,
562,
574,
564,
551,
535,
516,
504,
496,
496,
504,
515,
526,
538,
550,
563,
568,
576,
565,
546,
534,
541,
553,
561,
562,
559,
563,
558,
556,
558,
562,
567,
551,
530,
510,
494,
483,
483,
487,
493,
502,
517,
524,
521,
519,
502,
499,
504,
513,
527,
528,
530,
534,
529,
526,
527,
516,
500,
494,
502,
495,
475,
474,
474,
461,
450,
456,
472,
484,
480,
465,
447,
432,
419,
406,
395,
388,
386,
388,
397,
402,
400,
402,
404,
400,
409,
419,
424,
419,
403,
389,
379,
374,
371,
368,
367,
366,
366,
366,
366,
367,
368,
371,
372,
374,
378,
378,
372,
362,
355,
348,
344,
345,
350,
353,
354,
345,
355,
349,
337,
332,
330,
330,
334,
343,
355,
360,
357,
357,
356,
350,
348,
350,
360,
374,
379,
384,
374,
358,
344,
327,
322,
323,
328,
335,
338,
338,
329,
324,
325,
332,
339,
338,
336,
342,
351,
358,
369,
381,
393,
390,
389,
404,
407,
409,
412,
417,
418,
424,
432,
437,
427,
418,
415,
404,
393,
384,
371,
357,
340,
333,
321,
311,
306,
308,
311,
317,
332,
342,
356,
367,
376,
376,
368,
363,
374,
390,
400,
404,
405,
406,
409,
412,
414,
418,
419,
414,
403,
401,
403,
413,
416,
408,
399,
417,
428,
429,
430,
429,
424,
424,
420,
424,
425,
425,
423,
424,
421,
425,
433,
435,
433,
430,
427,
418,
430,
433,
433,
431,
428,
430,
427,
420,
419,
419,
418,
423,
429,
431,
429,
420,
410,
403,
396,
392,
387,
381,
376,
368,
362,
356,
348,
340,
339,
345,
353,
360,
366,
370,
371,
365,
363,
357,
352,
365,
376,
382,
384,
379,
373,
365,
357,
350,
342,
332,
323,
315,
308,
302,
296,
292,
290,
289,
287,
285,
283,
281,
280,
281,
282,
282,
283,
282,
282,
281,
282,
283,
283,
283,
282,
279,
282,
284,
285,
285,
285,
285,
285,
284,
283,
282,
280,
278,
276,
275,
275,
274,
274,
274,
273,
273,
273,
275,
279,
284,
292,
294,
296,
296,
296,
292,
290,
292,
290,
284,
280,
285,
293,
300,
299,
294,
295,
299,
302,
305,
304,
292,
282,
272,
262,
260,
520,
529,
542,
551,
559,
563,
559,
552,
540,
523,
505,
503,
503,
504,
505,
522,
537,
546,
559,
572,
576,
571,
555,
547,
546,
547,
559,
570,
574,
572,
576,
576,
570,
566,
563,
554,
538,
523,
503,
488,
484,
492,
504,
514,
524,
532,
539,
541,
534,
521,
517,
523,
521,
534,
544,
548,
535,
534,
537,
530,
515,
498,
485,
493,
492,
476,
461,
457,
457,
452,
448,
464,
471,
462,
448,
435,
422,
410,
399,
390,
387,
388,
392,
403,
412,
412,
413,
415,
410,
414,
421,
430,
428,
409,
389,
378,
377,
380,
377,
375,
376,
378,
379,
366,
366,
365,
362,
360,
358,
359,
358,
351,
350,
348,
347,
345,
348,
355,
356,
351,
341,
339,
341,
336,
332,
330,
330,
332,
342,
353,
350,
341,
340,
339,
338,
340,
346,
353,
360,
366,
375,
368,
352,
339,
328,
322,
328,
342,
352,
354,
348,
335,
330,
331,
341,
353,
350,
342,
348,
356,
365,
379,
392,
401,
400,
399,
409,
416,
416,
419,
424,
428,
434,
441,
440,
428,
428,
420,
410,
399,
389,
377,
362,
354,
350,
344,
326,
316,
306,
306,
311,
316,
330,
344,
357,
365,
364,
357,
354,
371,
385,
393,
397,
398,
399,
401,
403,
404,
406,
410,
407,
398,
391,
396,
405,
405,
398,
390,
412,
425,
427,
425,
427,
419,
413,
418,
427,
424,
421,
418,
417,
414,
423,
428,
427,
423,
417,
411,
410,
424,
424,
423,
420,
422,
425,
417,
412,
412,
411,
410,
418,
422,
426,
426,
418,
408,
398,
392,
388,
382,
375,
369,
363,
357,
353,
346,
341,
334,
341,
350,
358,
364,
367,
364,
356,
353,
349,
355,
370,
379,
382,
379,
375,
364,
356,
351,
340,
333,
325,
316,
308,
302,
298,
293,
291,
289,
288,
286,
285,
283,
281,
280,
281,
283,
283,
283,
284,
284,
284,
285,
286,
286,
285,
286,
284,
284,
286,
287,
287,
288,
288,
288,
289,
289,
287,
280,
279,
278,
277,
276,
276,
276,
276,
276,
276,
277,
279,
284,
291,
296,
300,
302,
304,
301,
295,
296,
297,
296,
292,
288,
287,
294,
303,
305,
302,
300,
299,
300,
299,
300,
291,
282,
273,
264,
260,
528,
537,
547,
551,
552,
549,
546,
539,
527,
508,
508,
505,
516,
518,
510,
518,
530,
539,
550,
565,
574,
565,
561,
564,
567,
568,
562,
575,
585,
593,
588,
569,
553,
544,
543,
537,
525,
510,
496,
488,
492,
506,
526,
534,
541,
550,
556,
559,
549,
543,
537,
539,
538,
541,
545,
543,
523,
520,
522,
527,
512,
498,
478,
486,
489,
477,
459,
446,
450,
452,
437,
449,
454,
444,
432,
422,
411,
401,
393,
388,
389,
394,
403,
407,
417,
424,
427,
420,
423,
425,
426,
431,
419,
401,
386,
385,
391,
396,
390,
385,
391,
391,
395,
398,
385,
374,
365,
359,
355,
354,
353,
351,
351,
353,
354,
354,
352,
357,
355,
350,
343,
342,
339,
334,
333,
331,
329,
330,
335,
342,
337,
334,
331,
331,
332,
337,
338,
341,
347,
354,
365,
362,
349,
334,
322,
322,
330,
356,
366,
355,
347,
340,
342,
343,
350,
362,
364,
356,
357,
367,
376,
387,
402,
413,
412,
411,
413,
421,
426,
428,
432,
438,
443,
447,
440,
435,
433,
427,
416,
402,
391,
379,
374,
372,
370,
366,
351,
335,
319,
306,
306,
310,
315,
334,
350,
354,
351,
344,
350,
368,
379,
386,
390,
392,
392,
394,
396,
396,
397,
401,
402,
392,
385,
387,
393,
391,
381,
386,
405,
418,
420,
418,
417,
414,
413,
419,
427,
421,
418,
414,
409,
410,
418,
418,
415,
408,
402,
399,
404,
412,
411,
412,
410,
415,
416,
407,
401,
405,
401,
402,
408,
412,
419,
420,
416,
407,
396,
387,
382,
378,
369,
362,
356,
352,
347,
343,
337,
330,
339,
348,
355,
359,
359,
354,
344,
342,
353,
364,
372,
378,
378,
378,
374,
369,
363,
355,
346,
337,
324,
313,
303,
300,
293,
291,
290,
288,
287,
286,
284,
283,
282,
280,
281,
282,
284,
284,
285,
287,
289,
290,
290,
289,
288,
288,
288,
289,
289,
290,
289,
290,
290,
291,
292,
294,
293,
288,
282,
280,
279,
278,
278,
278,
278,
278,
278,
278,
280,
286,
294,
302,
306,
308,
308,
306,
301,
302,
305,
304,
298,
291,
290,
294,
300,
305,
303,
301,
299,
298,
296,
294,
290,
281,
273,
265,
261,
522,
529,
534,
536,
536,
535,
535,
536,
521,
520,
510,
512,
527,
534,
521,
515,
523,
537,
554,
571,
582,
576,
585,
587,
587,
588,
572,
583,
598,
598,
583,
557,
540,
529,
525,
521,
510,
500,
491,
490,
501,
524,
541,
554,
560,
568,
573,
574,
568,
561,
557,
557,
551,
537,
530,
534,
520,
506,
509,
516,
506,
488,
477,
477,
478,
470,
459,
447,
437,
438,
427,
436,
434,
426,
418,
411,
403,
394,
391,
392,
395,
403,
414,
415,
423,
434,
434,
426,
433,
436,
437,
435,
420,
403,
387,
387,
400,
410,
402,
401,
408,
411,
414,
417,
403,
391,
376,
363,
363,
364,
357,
359,
361,
366,
371,
365,
365,
360,
354,
349,
345,
341,
338,
336,
334,
330,
328,
329,
330,
332,
330,
327,
325,
326,
329,
330,
331,
333,
338,
347,
353,
350,
344,
332,
322,
323,
329,
337,
344,
343,
340,
345,
353,
356,
356,
367,
375,
371,
370,
379,
393,
402,
413,
423,
424,
422,
423,
429,
436,
440,
441,
449,
453,
452,
442,
437,
440,
433,
421,
407,
399,
394,
392,
390,
383,
376,
364,
349,
333,
316,
306,
306,
309,
328,
341,
342,
339,
335,
347,
364,
372,
379,
382,
384,
384,
387,
388,
390,
388,
390,
395,
391,
380,
376,
375,
373,
367,
381,
400,
412,
413,
408,
410,
413,
414,
419,
425,
416,
412,
403,
400,
407,
412,
411,
404,
394,
389,
389,
399,
403,
399,
399,
401,
409,
408,
400,
393,
394,
391,
395,
401,
411,
416,
416,
413,
404,
394,
385,
375,
370,
364,
357,
352,
346,
342,
337,
333,
328,
336,
345,
351,
353,
350,
343,
339,
351,
362,
366,
369,
375,
378,
380,
378,
372,
367,
357,
347,
336,
324,
313,
303,
298,
293,
292,
290,
288,
287,
286,
284,
283,
281,
280,
282,
283,
284,
286,
287,
289,
291,
293,
293,
292,
290,
290,
290,
291,
292,
292,
291,
291,
292,
295,
302,
305,
301,
296,
291,
288,
285,
284,
283,
284,
284,
284,
283,
280,
280,
287,
295,
304,
311,
314,
314,
309,
304,
305,
308,
308,
304,
297,
292,
293,
300,
304,
302,
300,
298,
296,
295,
293,
287,
280,
272,
265,
261,
518,
522,
525,
527,
527,
527,
530,
533,
535,
533,
531,
522,
529,
544,
539,
530,
535,
547,
569,
583,
594,
594,
603,
609,
608,
603,
594,
592,
596,
587,
574,
558,
540,
524,
507,
504,
498,
494,
493,
494,
507,
530,
549,
564,
575,
581,
584,
586,
586,
576,
572,
561,
547,
526,
515,
522,
520,
502,
495,
501,
501,
484,
473,
466,
466,
458,
451,
452,
437,
429,
425,
420,
416,
410,
405,
401,
396,
391,
396,
400,
406,
411,
423,
429,
428,
440,
441,
437,
441,
446,
447,
439,
421,
409,
392,
396,
410,
419,
419,
419,
426,
431,
434,
435,
425,
407,
394,
383,
381,
378,
376,
372,
375,
386,
387,
382,
374,
362,
354,
349,
342,
340,
338,
339,
337,
333,
331,
329,
328,
328,
326,
324,
324,
324,
324,
324,
324,
323,
323,
338,
341,
339,
334,
322,
325,
324,
329,
331,
331,
328,
333,
345,
355,
366,
369,
377,
388,
389,
388,
390,
405,
416,
422,
431,
435,
433,
435,
436,
442,
451,
453,
457,
459,
453,
446,
444,
449,
440,
426,
419,
412,
408,
405,
400,
392,
384,
375,
358,
342,
324,
310,
306,
305,
313,
329,
329,
327,
328,
343,
357,
368,
371,
375,
378,
375,
379,
379,
382,
380,
380,
388,
389,
381,
371,
361,
352,
359,
379,
394,
402,
401,
400,
406,
415,
423,
418,
417,
409,
399,
390,
392,
402,
404,
402,
394,
382,
376,
383,
392,
392,
389,
386,
397,
402,
400,
393,
386,
384,
387,
392,
403,
406,
409,
412,
410,
404,
394,
384,
374,
364,
357,
352,
350,
343,
339,
335,
329,
327,
333,
342,
346,
345,
341,
337,
344,
357,
367,
367,
368,
373,
378,
378,
376,
370,
366,
353,
340,
330,
320,
309,
303,
298,
295,
292,
290,
288,
287,
286,
284,
283,
281,
282,
282,
284,
286,
287,
288,
289,
290,
292,
295,
292,
291,
292,
292,
294,
296,
295,
292,
293,
297,
303,
309,
313,
312,
307,
302,
298,
297,
297,
296,
296,
297,
296,
294,
286,
283,
287,
292,
304,
312,
318,
318,
314,
308,
307,
316,
316,
310,
304,
300,
300,
301,
303,
301,
299,
297,
295,
293,
290,
284,
279,
272,
266,
262,
516,
518,
521,
522,
523,
525,
525,
538,
550,
552,
548,
536,
532,
552,
553,
548,
556,
551,
578,
598,
611,
615,
621,
626,
626,
611,
590,
581,
579,
570,
559,
546,
534,
517,
496,
494,
494,
497,
501,
503,
505,
526,
549,
566,
584,
589,
581,
579,
579,
573,
558,
550,
542,
529,
511,
510,
513,
501,
486,
494,
497,
483,
466,
453,
451,
446,
442,
437,
432,
427,
422,
413,
403,
397,
391,
391,
391,
401,
411,
412,
414,
415,
426,
436,
438,
442,
443,
446,
453,
457,
457,
448,
427,
412,
400,
405,
417,
428,
433,
436,
440,
447,
453,
451,
438,
425,
411,
404,
402,
392,
384,
377,
384,
397,
400,
390,
376,
363,
356,
352,
344,
340,
341,
342,
340,
338,
334,
330,
328,
329,
330,
326,
330,
345,
346,
340,
335,
330,
323,
323,
323,
323,
322,
329,
331,
324,
324,
324,
324,
324,
328,
338,
350,
362,
368,
377,
393,
404,
403,
404,
415,
425,
432,
440,
444,
442,
444,
446,
450,
459,
464,
466,
464,
454,
450,
454,
456,
447,
438,
430,
425,
421,
416,
409,
401,
391,
380,
366,
349,
332,
306,
306,
303,
304,
310,
315,
315,
325,
340,
352,
359,
364,
365,
368,
370,
371,
371,
376,
375,
371,
378,
383,
380,
371,
358,
350,
357,
372,
382,
389,
390,
397,
408,
419,
423,
415,
410,
400,
387,
377,
383,
393,
393,
392,
386,
375,
366,
376,
382,
382,
377,
383,
391,
391,
390,
386,
378,
377,
387,
393,
398,
401,
403,
409,
414,
407,
396,
385,
375,
366,
361,
356,
352,
347,
343,
338,
330,
327,
330,
337,
339,
340,
337,
340,
349,
360,
368,
368,
368,
370,
374,
373,
370,
368,
362,
351,
339,
326,
314,
304,
301,
298,
293,
291,
290,
289,
287,
286,
285,
284,
284,
283,
284,
286,
287,
288,
289,
289,
290,
292,
295,
294,
293,
294,
296,
298,
300,
302,
300,
299,
303,
308,
314,
320,
323,
320,
315,
311,
308,
308,
309,
310,
310,
306,
297,
288,
288,
290,
291,
302,
309,
318,
318,
317,
315,
313,
316,
320,
315,
310,
305,
304,
303,
301,
300,
298,
296,
294,
292,
288,
281,
277,
272,
267,
262,
519,
520,
521,
526,
537,
542,
537,
534,
547,
562,
562,
550,
541,
555,
569,
568,
573,
568,
578,
597,
618,
627,
626,
621,
612,
596,
571,
550,
555,
554,
542,
524,
516,
506,
499,
498,
503,
509,
514,
517,
510,
520,
544,
562,
582,
578,
560,
559,
562,
564,
545,
530,
529,
530,
520,
500,
501,
496,
485,
490,
488,
471,
457,
447,
446,
443,
440,
435,
431,
426,
419,
407,
397,
391,
391,
391,
402,
415,
420,
425,
424,
423,
429,
437,
444,
451,
452,
450,
459,
466,
463,
445,
429,
412,
402,
413,
424,
438,
448,
455,
461,
467,
473,
472,
451,
438,
423,
422,
421,
404,
389,
387,
394,
408,
409,
392,
377,
366,
361,
354,
352,
345,
349,
354,
345,
339,
338,
334,
334,
336,
339,
342,
347,
360,
364,
352,
344,
337,
331,
327,
323,
326,
322,
335,
345,
348,
357,
348,
340,
332,
328,
338,
345,
354,
359,
372,
387,
404,
414,
417,
424,
432,
442,
450,
454,
454,
454,
454,
456,
466,
474,
475,
466,
456,
457,
464,
464,
455,
448,
441,
437,
431,
423,
415,
405,
394,
381,
367,
351,
336,
320,
306,
305,
302,
303,
306,
311,
323,
334,
342,
348,
355,
358,
359,
360,
363,
360,
366,
368,
362,
368,
377,
378,
369,
354,
341,
348,
362,
370,
372,
379,
395,
405,
410,
409,
406,
403,
388,
378,
366,
374,
382,
383,
380,
378,
369,
362,
367,
372,
369,
370,
380,
382,
382,
378,
375,
369,
375,
382,
388,
391,
396,
402,
406,
414,
405,
397,
386,
379,
374,
368,
362,
358,
353,
346,
342,
332,
326,
330,
333,
329,
331,
338,
347,
356,
365,
367,
364,
366,
366,
364,
363,
362,
362,
356,
348,
336,
325,
314,
304,
301,
299,
294,
292,
291,
290,
288,
287,
286,
285,
285,
285,
286,
287,
288,
289,
290,
290,
291,
292,
293,
296,
297,
298,
300,
303,
306,
308,
307,
304,
305,
312,
318,
326,
331,
331,
328,
324,
323,
323,
325,
321,
317,
306,
296,
289,
292,
296,
298,
301,
306,
315,
316,
317,
317,
315,
316,
318,
318,
314,
308,
305,
303,
301,
299,
297,
296,
294,
291,
286,
281,
276,
271,
266,
263,
524,
528,
531,
534,
549,
556,
553,
545,
546,
560,
573,
568,
557,
559,
580,
587,
594,
588,
587,
606,
622,
613,
600,
597,
596,
588,
572,
541,
531,
536,
523,
515,
507,
505,
504,
507,
515,
520,
535,
536,
528,
521,
535,
558,
575,
570,
547,
536,
549,
554,
541,
522,
517,
524,
526,
507,
488,
490,
480,
479,
478,
461,
450,
448,
447,
447,
446,
440,
438,
432,
413,
401,
393,
391,
394,
401,
410,
419,
425,
431,
433,
432,
432,
433,
438,
449,
458,
461,
463,
470,
461,
446,
429,
414,
414,
422,
433,
447,
465,
476,
481,
486,
492,
485,
468,
448,
435,
439,
432,
417,
398,
400,
409,
418,
418,
402,
391,
378,
365,
359,
357,
354,
356,
353,
341,
344,
343,
338,
340,
346,
350,
356,
360,
372,
379,
366,
352,
344,
337,
329,
325,
322,
330,
345,
358,
365,
372,
361,
348,
337,
331,
330,
340,
348,
359,
373,
384,
396,
406,
413,
425,
435,
443,
454,
462,
465,
465,
465,
464,
474,
483,
482,
471,
463,
469,
474,
472,
464,
457,
451,
442,
434,
425,
416,
406,
394,
381,
368,
354,
341,
329,
316,
306,
302,
301,
301,
306,
314,
324,
330,
338,
346,
351,
352,
352,
351,
350,
354,
357,
352,
360,
370,
372,
366,
351,
338,
339,
346,
352,
360,
376,
391,
397,
397,
395,
394,
389,
382,
372,
362,
367,
371,
373,
370,
370,
365,
353,
358,
363,
363,
369,
374,
373,
371,
366,
361,
364,
370,
377,
379,
387,
393,
399,
402,
405,
402,
397,
391,
386,
380,
376,
368,
364,
355,
349,
341,
332,
326,
328,
327,
328,
334,
342,
353,
359,
366,
364,
362,
366,
365,
360,
359,
357,
356,
351,
342,
332,
321,
311,
304,
304,
302,
298,
295,
292,
291,
289,
287,
286,
286,
286,
287,
288,
288,
289,
290,
291,
292,
292,
293,
296,
299,
301,
303,
304,
308,
315,
318,
316,
309,
305,
314,
322,
329,
337,
341,
340,
338,
337,
337,
332,
324,
316,
304,
294,
291,
296,
303,
306,
307,
304,
310,
317,
320,
318,
318,
318,
318,
317,
313,
308,
305,
303,
301,
299,
297,
295,
293,
289,
284,
280,
276,
270,
266,
264,
533,
544,
547,
539,
549,
563,
568,
560,
550,
556,
572,
582,
579,
580,
588,
608,
613,
607,
606,
618,
624,
604,
584,
575,
579,
578,
571,
546,
523,
517,
514,
507,
506,
510,
513,
518,
532,
538,
547,
553,
546,
541,
544,
560,
576,
568,
548,
526,
530,
541,
538,
520,
510,
518,
526,
511,
496,
484,
475,
466,
462,
455,
450,
448,
451,
460,
462,
452,
429,
418,
405,
397,
394,
395,
401,
412,
421,
425,
434,
438,
442,
444,
448,
441,
445,
457,
467,
473,
476,
476,
467,
454,
438,
428,
425,
434,
444,
460,
475,
485,
498,
504,
506,
495,
479,
457,
450,
452,
442,
422,
410,
418,
422,
428,
425,
411,
400,
387,
368,
366,
363,
359,
354,
350,
350,
356,
346,
339,
343,
354,
359,
369,
379,
385,
388,
378,
365,
357,
345,
332,
326,
329,
344,
354,
369,
382,
382,
367,
355,
341,
332,
331,
343,
344,
353,
365,
379,
389,
394,
401,
418,
427,
434,
445,
456,
466,
476,
478,
478,
483,
491,
488,
479,
472,
477,
479,
476,
468,
460,
450,
442,
434,
428,
418,
409,
397,
384,
370,
357,
341,
329,
318,
308,
302,
302,
301,
301,
303,
309,
317,
325,
336,
343,
345,
343,
340,
338,
344,
346,
344,
351,
361,
368,
362,
348,
332,
329,
337,
343,
356,
370,
383,
386,
384,
381,
381,
381,
375,
365,
353,
359,
363,
363,
362,
361,
358,
351,
352,
353,
357,
365,
365,
362,
359,
354,
353,
359,
363,
367,
376,
383,
389,
392,
396,
401,
402,
400,
396,
391,
386,
379,
370,
365,
356,
351,
344,
337,
329,
325,
326,
329,
336,
344,
354,
359,
359,
356,
358,
361,
361,
358,
355,
353,
351,
346,
338,
327,
317,
312,
309,
310,
307,
303,
298,
294,
292,
290,
288,
288,
288,
288,
289,
289,
289,
290,
291,
292,
293,
295,
296,
300,
302,
304,
308,
312,
317,
324,
328,
324,
319,
315,
314,
322,
331,
339,
344,
346,
349,
348,
344,
334,
322,
313,
302,
292,
294,
302,
308,
314,
317,
315,
310,
313,
317,
318,
318,
318,
318,
315,
310,
306,
304,
302,
300,
298,
296,
294,
292,
287,
282,
278,
275,
270,
267,
265,
536,
552,
560,
546,
551,
566,
580,
576,
566,
560,
578,
597,
602,
602,
607,
620,
625,
627,
624,
626,
616,
602,
582,
558,
554,
561,
558,
544,
527,
515,
509,
508,
511,
518,
528,
523,
533,
547,
562,
571,
568,
564,
563,
575,
584,
570,
549,
530,
516,
526,
530,
518,
507,
513,
520,
520,
506,
496,
481,
462,
455,
451,
450,
452,
457,
464,
455,
438,
421,
407,
400,
398,
398,
402,
410,
423,
434,
442,
446,
448,
448,
454,
458,
449,
458,
466,
478,
488,
492,
490,
476,
460,
450,
442,
436,
442,
454,
466,
475,
485,
498,
507,
514,
500,
482,
470,
465,
456,
437,
426,
423,
430,
432,
434,
430,
418,
405,
382,
380,
375,
368,
363,
358,
360,
363,
357,
350,
344,
349,
356,
366,
376,
387,
398,
401,
390,
376,
359,
345,
335,
330,
334,
350,
363,
383,
395,
388,
370,
357,
344,
334,
331,
334,
340,
352,
365,
373,
379,
379,
392,
407,
412,
425,
438,
451,
463,
473,
484,
491,
495,
501,
494,
485,
480,
480,
476,
471,
465,
457,
448,
438,
432,
426,
418,
409,
398,
384,
370,
355,
343,
329,
318,
314,
311,
307,
303,
303,
300,
301,
302,
311,
323,
332,
336,
335,
330,
326,
331,
335,
332,
343,
351,
357,
357,
345,
328,
327,
333,
342,
354,
366,
375,
374,
371,
368,
373,
373,
368,
356,
349,
353,
356,
354,
353,
352,
351,
342,
344,
346,
353,
357,
354,
350,
348,
346,
351,
353,
362,
368,
375,
378,
384,
388,
394,
401,
404,
404,
400,
392,
386,
378,
367,
359,
352,
349,
344,
339,
328,
326,
327,
329,
335,
342,
353,
355,
353,
351,
354,
356,
356,
355,
351,
348,
345,
342,
334,
324,
316,
315,
316,
318,
312,
305,
297,
293,
291,
290,
289,
289,
290,
291,
290,
290,
290,
291,
292,
294,
296,
298,
300,
303,
306,
311,
316,
319,
324,
331,
334,
333,
328,
322,
318,
322,
330,
337,
343,
347,
353,
354,
345,
335,
323,
313,
304,
292,
298,
306,
312,
318,
318,
321,
318,
315,
316,
318,
318,
318,
316,
312,
308,
304,
303,
301,
299,
297,
295,
294,
291,
286,
281,
278,
275,
272,
270,
268,
547,
551,
562,
554,
554,
571,
587,
591,
583,
576,
582,
606,
622,
630,
629,
623,
618,
617,
619,
617,
609,
596,
582,
562,
536,
534,
540,
530,
521,
513,
511,
515,
520,
524,
539,
541,
539,
552,
571,
586,
587,
586,
579,
584,
586,
570,
547,
531,
515,
511,
515,
513,
500,
499,
511,
516,
514,
500,
480,
462,
453,
452,
452,
458,
465,
459,
443,
427,
412,
404,
403,
405,
406,
407,
416,
431,
448,
459,
465,
462,
462,
466,
470,
464,
475,
478,
488,
502,
505,
500,
481,
474,
468,
450,
447,
449,
449,
460,
471,
478,
488,
499,
511,
509,
492,
489,
475,
467,
455,
439,
431,
439,
441,
442,
439,
429,
410,
397,
390,
379,
369,
366,
367,
369,
373,
366,
357,
352,
356,
364,
376,
386,
393,
402,
402,
390,
373,
359,
344,
336,
333,
339,
352,
366,
384,
402,
393,
372,
358,
344,
339,
332,
332,
341,
348,
356,
363,
365,
374,
387,
394,
402,
417,
430,
443,
455,
466,
479,
493,
503,
503,
498,
490,
483,
479,
473,
464,
457,
451,
444,
438,
430,
422,
414,
406,
396,
383,
370,
357,
343,
330,
327,
328,
327,
320,
316,
309,
306,
301,
300,
303,
309,
318,
326,
326,
321,
315,
321,
323,
326,
334,
342,
347,
350,
340,
324,
321,
328,
342,
353,
361,
365,
364,
357,
356,
366,
365,
359,
351,
340,
346,
348,
348,
346,
344,
344,
338,
337,
339,
346,
348,
345,
341,
340,
341,
351,
356,
366,
369,
372,
376,
382,
390,
397,
403,
405,
403,
398,
391,
386,
378,
369,
359,
351,
341,
341,
335,
329,
327,
327,
328,
334,
341,
347,
349,
347,
347,
351,
354,
354,
350,
346,
342,
340,
338,
329,
325,
323,
325,
321,
317,
309,
302,
295,
292,
291,
290,
290,
290,
291,
292,
292,
291,
291,
291,
293,
296,
297,
300,
302,
306,
310,
317,
319,
322,
328,
334,
341,
342,
338,
330,
325,
325,
328,
334,
339,
342,
348,
353,
346,
337,
325,
316,
309,
301,
305,
307,
313,
319,
319,
320,
319,
318,
318,
318,
318,
317,
313,
309,
305,
303,
302,
300,
298,
296,
294,
292,
290,
285,
280,
278,
276,
273,
271,
268,
562,
560,
567,
565,
559,
568,
586,
603,
604,
596,
595,
610,
630,
644,
638,
617,
599,
596,
597,
598,
594,
584,
575,
563,
543,
517,
521,
519,
517,
515,
517,
524,
534,
531,
544,
557,
559,
561,
577,
585,
581,
574,
569,
571,
576,
571,
552,
540,
528,
512,
499,
502,
496,
489,
504,
515,
504,
488,
471,
459,
456,
459,
466,
470,
464,
451,
433,
417,
407,
407,
412,
415,
418,
414,
418,
435,
454,
473,
481,
480,
482,
484,
486,
479,
491,
497,
499,
513,
516,
502,
500,
491,
475,
453,
458,
463,
460,
460,
472,
480,
492,
503,
513,
517,
503,
505,
496,
490,
470,
460,
443,
454,
455,
452,
450,
441,
425,
415,
398,
379,
376,
378,
378,
379,
380,
371,
362,
362,
368,
373,
386,
396,
404,
407,
399,
387,
371,
358,
342,
336,
335,
343,
356,
370,
385,
406,
396,
375,
360,
353,
346,
338,
333,
334,
340,
341,
351,
357,
370,
379,
388,
402,
413,
425,
437,
449,
463,
480,
494,
503,
501,
498,
490,
484,
474,
466,
460,
450,
445,
438,
430,
427,
417,
409,
399,
388,
378,
369,
354,
342,
339,
341,
344,
343,
338,
335,
329,
320,
312,
305,
300,
301,
306,
313,
318,
311,
307,
308,
312,
317,
326,
332,
337,
342,
335,
317,
319,
330,
342,
350,
352,
355,
354,
349,
350,
356,
360,
355,
344,
336,
338,
342,
342,
339,
337,
336,
332,
328,
333,
339,
338,
336,
338,
340,
344,
352,
358,
365,
367,
372,
376,
383,
391,
400,
406,
404,
399,
393,
389,
385,
380,
372,
362,
352,
344,
338,
332,
328,
326,
326,
327,
332,
339,
344,
343,
340,
341,
347,
349,
348,
346,
341,
339,
338,
336,
331,
331,
331,
328,
320,
312,
302,
296,
293,
291,
291,
291,
291,
291,
292,
294,
293,
292,
293,
293,
293,
296,
299,
301,
304,
308,
312,
317,
320,
324,
328,
334,
343,
347,
347,
340,
332,
328,
327,
330,
336,
339,
342,
345,
340,
334,
325,
315,
304,
304,
306,
305,
310,
315,
319,
319,
320,
319,
318,
317,
316,
315,
310,
307,
304,
303,
301,
299,
298,
296,
294,
292,
289,
285,
282,
279,
277,
274,
271,
268,
568,
576,
576,
581,
571,
572,
586,
607,
621,
623,
622,
626,
638,
649,
642,
622,
600,
582,
577,
577,
577,
573,
564,
553,
537,
521,
519,
518,
517,
520,
525,
530,
547,
548,
549,
565,
580,
580,
577,
574,
565,
554,
550,
555,
563,
566,
562,
550,
536,
525,
510,
501,
493,
492,
501,
503,
491,
476,
465,
460,
463,
472,
478,
476,
458,
439,
423,
410,
407,
418,
426,
428,
427,
428,
425,
434,
454,
478,
496,
502,
503,
506,
506,
498,
505,
513,
512,
523,
524,
520,
510,
494,
484,
462,
473,
480,
476,
470,
472,
474,
486,
500,
511,
518,
525,
525,
517,
499,
489,
475,
454,
465,
477,
476,
470,
454,
438,
421,
406,
389,
386,
387,
388,
386,
383,
376,
373,
371,
380,
387,
398,
409,
416,
406,
396,
385,
371,
358,
342,
338,
338,
347,
360,
373,
387,
409,
399,
380,
367,
367,
353,
348,
339,
334,
337,
343,
348,
353,
363,
372,
392,
411,
425,
433,
443,
455,
469,
483,
492,
495,
491,
494,
492,
486,
477,
467,
459,
452,
444,
437,
427,
416,
409,
400,
389,
378,
366,
362,
354,
353,
355,
359,
360,
360,
358,
353,
346,
338,
329,
317,
306,
300,
299,
302,
303,
302,
298,
297,
301,
307,
319,
322,
328,
334,
330,
314,
320,
333,
343,
345,
343,
344,
343,
340,
346,
354,
355,
350,
342,
330,
329,
334,
336,
333,
330,
329,
328,
327,
329,
328,
328,
334,
340,
344,
349,
353,
357,
362,
366,
372,
378,
385,
393,
402,
404,
399,
393,
388,
384,
380,
379,
377,
366,
356,
350,
343,
337,
329,
325,
326,
327,
329,
335,
340,
337,
335,
338,
343,
344,
343,
341,
338,
336,
335,
334,
333,
331,
329,
321,
313,
306,
297,
293,
292,
292,
292,
292,
292,
293,
293,
295,
296,
295,
296,
297,
298,
300,
302,
304,
304,
308,
313,
316,
321,
325,
328,
335,
343,
352,
355,
348,
341,
337,
335,
337,
338,
339,
341,
347,
342,
336,
326,
316,
304,
304,
305,
305,
308,
312,
316,
319,
319,
319,
319,
317,
314,
311,
306,
304,
303,
302,
300,
299,
297,
295,
294,
293,
289,
285,
283,
280,
277,
274,
271,
268,
559,
571,
585,
591,
592,
591,
599,
615,
632,
635,
627,
635,
638,
642,
643,
628,
605,
588,
561,
558,
561,
559,
552,
539,
527,
525,
523,
521,
522,
527,
535,
538,
553,
562,
562,
572,
588,
573,
555,
554,
551,
533,
530,
540,
546,
555,
552,
538,
527,
523,
516,
504,
493,
487,
494,
486,
478,
470,
466,
469,
474,
486,
485,
470,
449,
430,
414,
406,
410,
425,
432,
437,
440,
441,
439,
437,
456,
477,
499,
515,
522,
524,
523,
514,
518,
525,
526,
528,
529,
528,
515,
501,
487,
475,
487,
498,
486,
481,
490,
487,
486,
487,
504,
512,
522,
537,
531,
517,
503,
482,
461,
476,
493,
500,
486,
465,
442,
425,
404,
401,
396,
400,
398,
395,
392,
390,
391,
381,
390,
401,
412,
420,
415,
403,
393,
383,
371,
359,
347,
341,
341,
354,
368,
380,
397,
415,
405,
390,
386,
377,
371,
364,
350,
342,
339,
340,
353,
363,
364,
377,
396,
411,
424,
436,
448,
461,
472,
479,
484,
483,
482,
490,
499,
490,
481,
472,
462,
454,
445,
435,
426,
416,
405,
396,
386,
377,
373,
372,
371,
370,
370,
373,
374,
373,
371,
367,
367,
361,
345,
330,
318,
308,
301,
298,
297,
296,
293,
292,
293,
298,
306,
312,
317,
328,
322,
314,
323,
332,
340,
339,
336,
335,
338,
341,
345,
349,
349,
344,
339,
329,
326,
327,
331,
329,
322,
319,
322,
326,
328,
329,
334,
340,
344,
347,
354,
356,
358,
363,
367,
370,
376,
384,
393,
402,
399,
390,
387,
382,
379,
376,
375,
372,
367,
362,
355,
348,
343,
334,
327,
326,
326,
328,
333,
334,
331,
328,
336,
341,
339,
337,
334,
331,
329,
330,
333,
331,
329,
321,
312,
305,
298,
292,
292,
292,
292,
294,
294,
295,
295,
296,
297,
299,
298,
300,
302,
303,
304,
304,
307,
308,
310,
315,
318,
322,
326,
330,
337,
343,
354,
357,
354,
349,
346,
344,
341,
340,
340,
342,
342,
344,
338,
330,
321,
309,
304,
305,
307,
311,
313,
315,
317,
319,
318,
318,
316,
312,
308,
304,
303,
302,
301,
300,
298,
296,
294,
293,
291,
288,
285,
282,
280,
278,
276,
272,
269,
563,
570,
584,
599,
612,
616,
623,
633,
637,
618,
608,
618,
624,
627,
630,
623,
606,
590,
569,
555,
546,
540,
536,
533,
529,
528,
531,
533,
528,
533,
544,
554,
562,
570,
578,
585,
576,
562,
540,
532,
538,
526,
519,
524,
533,
547,
549,
535,
521,
511,
509,
503,
490,
482,
477,
475,
472,
472,
477,
485,
492,
495,
485,
462,
441,
421,
407,
406,
416,
426,
438,
448,
454,
456,
455,
449,
454,
471,
493,
514,
533,
539,
537,
530,
530,
533,
533,
531,
525,
521,
507,
491,
484,
484,
497,
506,
502,
487,
504,
508,
505,
496,
501,
515,
527,
543,
540,
530,
506,
483,
478,
489,
507,
508,
491,
468,
445,
427,
417,
415,
409,
406,
408,
408,
403,
405,
408,
401,
404,
417,
427,
427,
414,
400,
390,
381,
370,
359,
349,
344,
344,
353,
368,
383,
402,
418,
416,
407,
399,
389,
385,
375,
363,
354,
346,
342,
341,
358,
373,
381,
395,
406,
414,
428,
444,
459,
467,
472,
472,
471,
474,
486,
501,
497,
489,
479,
468,
457,
447,
440,
429,
417,
406,
394,
388,
385,
384,
386,
386,
384,
382,
382,
380,
378,
374,
372,
370,
367,
358,
344,
332,
321,
308,
304,
300,
297,
293,
290,
290,
292,
297,
302,
315,
322,
317,
312,
324,
331,
332,
332,
328,
329,
336,
342,
342,
342,
342,
338,
333,
325,
317,
321,
325,
323,
315,
317,
320,
329,
332,
332,
338,
342,
344,
348,
351,
354,
355,
360,
364,
366,
375,
382,
391,
396,
391,
385,
379,
377,
375,
373,
368,
367,
368,
366,
359,
355,
350,
343,
334,
324,
325,
327,
330,
328,
327,
329,
333,
334,
331,
330,
329,
327,
328,
332,
333,
330,
322,
313,
305,
297,
292,
292,
292,
292,
293,
295,
296,
296,
297,
298,
300,
302,
305,
309,
309,
309,
309,
312,
314,
314,
314,
316,
319,
323,
328,
334,
341,
348,
354,
359,
357,
356,
355,
350,
345,
341,
339,
340,
340,
338,
335,
331,
322,
312,
304,
307,
310,
314,
316,
318,
318,
318,
318,
317,
316,
312,
306,
304,
303,
302,
301,
299,
298,
295,
294,
292,
290,
288,
284,
280,
278,
276,
274,
271,
271,
575,
577,
588,
604,
623,
634,
639,
637,
628,
611,
592,
595,
601,
605,
609,
607,
594,
583,
568,
550,
538,
531,
534,
532,
528,
533,
541,
545,
544,
542,
553,
566,
574,
580,
585,
576,
564,
549,
530,
517,
516,
512,
511,
516,
524,
535,
537,
534,
524,
509,
498,
494,
487,
476,
470,
470,
474,
481,
494,
502,
503,
498,
482,
455,
433,
416,
405,
409,
426,
432,
446,
456,
465,
470,
473,
473,
469,
473,
491,
510,
529,
549,
546,
543,
541,
540,
538,
532,
523,
511,
500,
501,
502,
484,
497,
511,
513,
496,
513,
522,
515,
500,
505,
522,
536,
549,
541,
523,
500,
487,
499,
508,
517,
508,
490,
469,
447,
436,
433,
430,
422,
414,
417,
421,
420,
421,
427,
418,
424,
434,
437,
426,
415,
399,
388,
379,
369,
360,
352,
348,
346,
354,
366,
381,
402,
420,
430,
424,
412,
405,
396,
387,
380,
366,
355,
346,
344,
348,
367,
380,
392,
403,
413,
427,
443,
453,
458,
460,
461,
460,
468,
485,
501,
502,
497,
484,
472,
460,
452,
442,
430,
420,
411,
406,
404,
399,
400,
403,
403,
396,
391,
384,
379,
374,
369,
364,
361,
357,
353,
347,
341,
333,
325,
319,
310,
302,
297,
291,
289,
290,
293,
299,
311,
321,
313,
308,
320,
324,
330,
330,
326,
329,
332,
334,
335,
334,
334,
331,
326,
320,
314,
316,
318,
317,
314,
318,
321,
328,
328,
329,
333,
338,
340,
340,
342,
347,
352,
357,
361,
365,
374,
381,
388,
392,
385,
379,
375,
372,
370,
369,
367,
364,
362,
359,
357,
355,
354,
346,
343,
327,
319,
328,
326,
326,
325,
328,
329,
330,
328,
327,
326,
327,
330,
335,
339,
330,
318,
306,
300,
292,
293,
292,
292,
293,
295,
297,
298,
298,
299,
300,
302,
305,
308,
315,
317,
318,
317,
318,
320,
320,
317,
316,
318,
324,
329,
338,
345,
352,
357,
357,
358,
355,
353,
348,
343,
340,
337,
334,
332,
330,
329,
328,
320,
310,
306,
309,
313,
317,
318,
318,
319,
318,
318,
317,
317,
312,
306,
304,
302,
302,
301,
300,
298,
296,
295,
293,
291,
288,
284,
280,
278,
275,
273,
271,
270,
572,
580,
589,
605,
622,
634,
631,
625,
615,
601,
584,
580,
577,
585,
593,
592,
581,
569,
551,
543,
539,
537,
537,
536,
536,
536,
552,
561,
561,
556,
556,
569,
576,
575,
570,
561,
551,
534,
520,
512,
514,
509,
506,
512,
524,
522,
520,
520,
518,
511,
493,
484,
477,
473,
469,
471,
479,
496,
507,
513,
508,
497,
479,
451,
431,
416,
406,
418,
430,
438,
451,
461,
470,
478,
485,
490,
490,
485,
490,
505,
522,
540,
552,
552,
551,
547,
544,
540,
528,
522,
524,
519,
502,
487,
504,
517,
524,
509,
518,
531,
522,
506,
513,
525,
539,
546,
540,
520,
497,
504,
517,
526,
528,
504,
487,
472,
461,
455,
449,
443,
433,
426,
425,
432,
434,
433,
437,
436,
442,
445,
437,
425,
412,
399,
388,
377,
366,
362,
358,
350,
349,
358,
378,
391,
406,
422,
440,
438,
425,
417,
408,
403,
392,
376,
361,
352,
346,
350,
361,
376,
390,
403,
415,
428,
437,
442,
443,
445,
447,
450,
466,
481,
497,
505,
503,
490,
474,
466,
456,
447,
439,
432,
429,
425,
420,
416,
412,
407,
404,
396,
388,
380,
370,
365,
359,
353,
347,
343,
341,
337,
332,
328,
323,
320,
318,
311,
306,
292,
290,
289,
292,
299,
309,
318,
308,
304,
312,
320,
329,
326,
322,
325,
327,
328,
328,
328,
327,
322,
318,
314,
308,
308,
309,
311,
316,
318,
322,
323,
325,
326,
328,
332,
336,
337,
340,
344,
351,
354,
362,
367,
374,
380,
387,
384,
378,
374,
371,
367,
364,
364,
363,
359,
355,
352,
351,
350,
347,
343,
338,
330,
317,
318,
321,
322,
322,
325,
326,
325,
326,
326,
328,
330,
334,
340,
343,
333,
321,
309,
299,
292,
293,
292,
293,
295,
297,
298,
300,
300,
301,
302,
303,
305,
314,
316,
318,
322,
325,
326,
328,
326,
324,
321,
320,
326,
329,
336,
344,
352,
356,
356,
355,
351,
347,
344,
341,
339,
335,
330,
328,
327,
326,
322,
317,
313,
308,
312,
314,
318,
319,
319,
319,
318,
318,
318,
317,
312,
306,
304,
302,
302,
302,
301,
299,
297,
295,
294,
292,
288,
284,
282,
279,
276,
272,
270,
268,
570,
579,
592,
607,
621,
626,
620,
612,
601,
594,
592,
586,
567,
571,
576,
577,
567,
553,
547,
541,
540,
542,
546,
548,
550,
541,
548,
560,
572,
570,
564,
569,
568,
560,
555,
546,
531,
520,
522,
528,
528,
512,
510,
512,
517,
512,
506,
504,
508,
505,
493,
483,
476,
473,
474,
478,
482,
501,
516,
524,
508,
497,
479,
455,
434,
417,
408,
419,
434,
446,
454,
466,
476,
484,
492,
500,
505,
506,
499,
505,
520,
532,
545,
550,
553,
554,
553,
549,
542,
540,
531,
512,
499,
496,
508,
524,
534,
523,
528,
539,
524,
512,
522,
537,
543,
561,
541,
522,
504,
522,
536,
542,
535,
509,
487,
488,
487,
473,
468,
454,
441,
436,
438,
444,
451,
450,
454,
457,
455,
446,
433,
418,
407,
396,
387,
379,
372,
367,
358,
351,
351,
362,
380,
395,
409,
423,
439,
444,
436,
429,
421,
410,
398,
384,
369,
356,
349,
354,
369,
374,
382,
395,
406,
416,
422,
424,
427,
428,
435,
442,
455,
472,
490,
505,
510,
496,
484,
474,
462,
456,
451,
443,
442,
437,
429,
421,
414,
402,
395,
388,
380,
375,
367,
356,
347,
341,
336,
330,
329,
327,
322,
318,
315,
312,
311,
306,
297,
291,
290,
289,
291,
299,
309,
314,
305,
302,
305,
318,
321,
320,
316,
320,
324,
324,
324,
321,
318,
316,
309,
305,
303,
303,
304,
314,
318,
319,
325,
327,
327,
328,
328,
330,
338,
340,
341,
344,
350,
353,
362,
370,
376,
381,
381,
378,
374,
368,
366,
365,
361,
357,
357,
356,
353,
348,
343,
340,
338,
335,
331,
325,
316,
318,
320,
319,
318,
320,
321,
321,
321,
328,
330,
334,
338,
344,
347,
338,
326,
313,
301,
292,
293,
293,
296,
296,
299,
300,
302,
302,
303,
303,
304,
306,
310,
314,
319,
324,
330,
332,
333,
333,
330,
328,
328,
327,
326,
332,
339,
345,
349,
350,
349,
345,
341,
340,
339,
337,
334,
330,
328,
326,
324,
320,
317,
315,
314,
314,
317,
318,
319,
319,
319,
318,
317,
316,
314,
309,
306,
304,
302,
303,
303,
303,
301,
297,
295,
294,
292,
288,
284,
282,
279,
275,
271,
268,
267,
581,
584,
596,
613,
627,
627,
620,
613,
610,
602,
592,
578,
563,
560,
560,
560,
553,
547,
543,
541,
542,
551,
561,
562,
562,
548,
552,
556,
565,
579,
578,
574,
555,
546,
538,
535,
535,
537,
539,
542,
530,
511,
514,
525,
525,
504,
492,
492,
487,
494,
492,
481,
477,
478,
484,
492,
488,
500,
516,
518,
506,
494,
479,
455,
434,
416,
408,
420,
437,
451,
459,
471,
483,
490,
498,
505,
512,
518,
517,
514,
517,
526,
532,
537,
549,
560,
564,
561,
558,
549,
523,
522,
514,
508,
508,
522,
541,
538,
543,
544,
526,
523,
532,
542,
552,
560,
541,
522,
524,
533,
545,
550,
536,
514,
497,
506,
506,
495,
484,
462,
451,
448,
452,
458,
466,
468,
469,
465,
453,
442,
430,
416,
404,
393,
385,
380,
373,
368,
358,
351,
352,
365,
377,
389,
408,
423,
438,
450,
448,
440,
429,
416,
404,
390,
372,
355,
352,
357,
374,
380,
382,
388,
395,
398,
405,
410,
414,
428,
436,
443,
460,
476,
490,
507,
513,
502,
492,
480,
471,
465,
455,
449,
440,
432,
424,
417,
407,
395,
386,
377,
369,
366,
361,
355,
347,
335,
325,
319,
316,
314,
311,
307,
305,
303,
302,
299,
292,
291,
289,
291,
297,
303,
308,
311,
305,
303,
301,
310,
315,
315,
315,
319,
321,
320,
319,
318,
312,
308,
303,
300,
301,
304,
312,
319,
324,
326,
330,
333,
333,
334,
335,
337,
340,
344,
348,
352,
353,
359,
367,
374,
379,
380,
377,
372,
368,
365,
361,
359,
356,
352,
352,
351,
348,
344,
340,
335,
329,
328,
326,
320,
318,
317,
320,
318,
316,
318,
318,
317,
320,
326,
331,
337,
342,
350,
354,
344,
332,
318,
303,
292,
293,
295,
297,
298,
300,
303,
304,
304,
306,
307,
306,
305,
309,
312,
316,
321,
327,
332,
338,
340,
338,
336,
334,
331,
328,
327,
332,
338,
341,
343,
343,
341,
339,
338,
337,
335,
333,
331,
330,
327,
324,
321,
317,
316,
315,
317,
317,
318,
319,
319,
318,
317,
316,
313,
311,
307,
304,
304,
304,
305,
305,
304,
301,
297,
295,
293,
291,
288,
284,
282,
279,
276,
270,
266,
268,
596,
594,
606,
619,
630,
624,
619,
607,
600,
598,
586,
572,
563,
554,
548,
546,
545,
543,
544,
544,
547,
555,
571,
578,
574,
565,
567,
560,
567,
579,
588,
580,
565,
554,
543,
548,
553,
552,
550,
539,
521,
504,
514,
524,
516,
495,
485,
483,
483,
483,
483,
480,
473,
484,
500,
507,
512,
514,
520,
514,
502,
489,
475,
451,
430,
415,
409,
424,
442,
454,
460,
473,
486,
496,
504,
510,
517,
524,
530,
531,
530,
526,
532,
535,
542,
552,
562,
566,
560,
547,
540,
538,
532,
523,
516,
528,
546,
556,
556,
546,
534,
535,
545,
554,
564,
560,
541,
542,
546,
547,
557,
560,
540,
519,
514,
525,
524,
514,
491,
471,
463,
460,
468,
473,
479,
478,
472,
461,
449,
439,
429,
418,
406,
395,
386,
377,
373,
365,
358,
352,
352,
366,
377,
396,
414,
430,
442,
454,
456,
447,
440,
425,
411,
396,
373,
355,
347,
360,
377,
390,
398,
404,
408,
414,
419,
421,
424,
434,
449,
454,
461,
478,
494,
511,
508,
502,
497,
486,
479,
468,
457,
446,
436,
426,
415,
406,
398,
389,
378,
367,
359,
352,
349,
344,
339,
330,
319,
315,
314,
307,
303,
300,
300,
295,
292,
292,
291,
289,
289,
294,
306,
306,
306,
308,
305,
301,
300,
306,
307,
309,
317,
319,
318,
316,
314,
310,
306,
301,
301,
303,
306,
312,
317,
325,
332,
334,
338,
342,
342,
342,
342,
343,
344,
351,
356,
364,
366,
365,
371,
379,
382,
380,
374,
366,
363,
360,
355,
354,
352,
349,
346,
344,
342,
340,
337,
333,
328,
325,
320,
317,
317,
317,
318,
316,
316,
317,
317,
319,
324,
329,
334,
339,
346,
353,
357,
347,
333,
320,
303,
292,
294,
296,
298,
300,
302,
304,
306,
307,
309,
313,
312,
314,
310,
313,
316,
319,
325,
331,
338,
343,
343,
343,
342,
341,
339,
336,
334,
339,
340,
339,
339,
339,
338,
336,
335,
333,
332,
332,
330,
329,
326,
322,
319,
316,
314,
317,
317,
318,
318,
318,
318,
317,
316,
312,
308,
305,
303,
304,
305,
305,
305,
305,
301,
296,
295,
292,
290,
287,
284,
282,
279,
275,
269,
268,
269,
600,
608,
615,
621,
614,
608,
600,
590,
586,
584,
582,
572,
559,
552,
544,
541,
541,
543,
548,
552,
548,
555,
569,
586,
587,
584,
592,
583,
579,
585,
594,
591,
579,
563,
559,
563,
565,
555,
543,
533,
516,
498,
504,
507,
505,
490,
475,
478,
478,
478,
479,
475,
483,
486,
500,
517,
526,
526,
521,
512,
497,
483,
469,
449,
428,
414,
411,
426,
442,
452,
459,
470,
483,
496,
505,
514,
521,
530,
537,
541,
540,
537,
533,
531,
539,
546,
561,
570,
562,
549,
556,
554,
550,
532,
529,
538,
552,
567,
565,
552,
550,
551,
560,
572,
575,
565,
548,
559,
564,
564,
570,
561,
545,
537,
535,
541,
537,
522,
500,
485,
475,
473,
486,
491,
488,
475,
466,
456,
446,
437,
429,
418,
407,
399,
392,
383,
375,
364,
357,
355,
356,
370,
386,
402,
418,
433,
447,
459,
466,
454,
444,
432,
418,
401,
380,
361,
354,
362,
376,
391,
407,
421,
429,
433,
437,
439,
440,
442,
450,
467,
473,
478,
494,
511,
508,
503,
501,
492,
482,
471,
458,
446,
435,
425,
414,
400,
388,
378,
370,
360,
351,
345,
339,
338,
333,
328,
326,
324,
320,
315,
308,
305,
301,
297,
292,
288,
290,
289,
290,
294,
301,
302,
301,
303,
306,
301,
292,
299,
303,
310,
318,
317,
313,
309,
304,
300,
294,
300,
305,
308,
314,
320,
325,
331,
338,
342,
343,
344,
346,
346,
349,
350,
352,
353,
362,
368,
372,
374,
377,
380,
381,
378,
372,
365,
359,
355,
351,
349,
346,
343,
341,
339,
336,
335,
333,
330,
327,
323,
318,
316,
315,
316,
316,
316,
316,
315,
317,
322,
327,
332,
337,
343,
350,
355,
355,
343,
330,
316,
299,
292,
294,
296,
298,
300,
302,
304,
307,
310,
315,
317,
317,
317,
313,
313,
316,
317,
322,
329,
336,
342,
344,
345,
350,
350,
348,
345,
343,
341,
340,
340,
340,
337,
337,
335,
333,
331,
330,
331,
331,
330,
327,
323,
319,
316,
316,
317,
318,
318,
318,
318,
318,
317,
314,
310,
306,
304,
303,
305,
307,
306,
304,
304,
300,
295,
294,
291,
288,
286,
283,
281,
279,
275,
271,
271,
270,
608,
619,
619,
606,
596,
593,
587,
576,
568,
569,
575,
573,
559,
550,
544,
542,
544,
547,
552,
564,
567,
564,
570,
588,
599,
600,
606,
598,
594,
597,
600,
598,
589,
578,
570,
565,
559,
548,
534,
523,
511,
499,
494,
492,
489,
482,
475,
476,
473,
475,
480,
485,
494,
495,
502,
518,
535,
535,
520,
507,
493,
481,
464,
447,
426,
414,
412,
428,
442,
448,
454,
465,
477,
491,
507,
518,
526,
534,
540,
548,
549,
544,
537,
532,
536,
544,
556,
573,
574,
563,
568,
572,
565,
547,
543,
554,
561,
577,
577,
571,
570,
564,
576,
593,
588,
574,
561,
576,
584,
584,
579,
568,
560,
560,
558,
555,
547,
529,
514,
500,
490,
492,
502,
498,
485,
472,
464,
453,
444,
435,
428,
417,
409,
404,
394,
388,
372,
363,
357,
359,
365,
379,
395,
408,
420,
431,
444,
456,
466,
463,
450,
437,
420,
402,
383,
370,
360,
363,
374,
393,
409,
424,
434,
440,
444,
448,
454,
458,
460,
469,
486,
495,
503,
512,
510,
503,
500,
490,
481,
469,
456,
446,
436,
426,
414,
405,
397,
386,
377,
365,
356,
356,
356,
351,
346,
337,
334,
333,
328,
323,
318,
313,
307,
300,
293,
291,
289,
289,
289,
290,
293,
295,
293,
294,
296,
294,
290,
296,
305,
309,
313,
310,
305,
300,
297,
295,
297,
302,
307,
312,
318,
323,
330,
336,
342,
345,
346,
345,
349,
351,
354,
357,
360,
365,
366,
371,
375,
379,
380,
380,
379,
374,
368,
362,
356,
351,
348,
343,
340,
338,
335,
333,
331,
329,
329,
327,
324,
321,
318,
316,
315,
314,
314,
314,
315,
316,
320,
324,
330,
335,
340,
347,
354,
356,
349,
337,
324,
310,
296,
293,
295,
297,
299,
301,
302,
304,
307,
310,
315,
316,
317,
317,
314,
316,
316,
316,
320,
325,
331,
337,
341,
344,
350,
354,
353,
351,
345,
343,
344,
343,
342,
340,
336,
334,
332,
329,
328,
329,
329,
329,
328,
324,
321,
317,
318,
318,
318,
318,
318,
318,
317,
315,
312,
308,
304,
303,
303,
303,
310,
308,
304,
301,
298,
294,
292,
290,
287,
284,
282,
279,
277,
275,
273,
271,
270,
610,
611,
603,
590,
575,
572,
565,
561,
558,
561,
564,
565,
557,
549,
543,
543,
548,
554,
553,
560,
574,
579,
587,
598,
613,
611,
608,
601,
598,
595,
591,
588,
586,
578,
562,
548,
545,
542,
533,
520,
508,
498,
490,
483,
481,
476,
471,
473,
473,
475,
485,
489,
505,
513,
510,
512,
524,
528,
516,
500,
488,
477,
460,
444,
425,
414,
414,
428,
441,
447,
452,
463,
475,
493,
510,
521,
530,
537,
543,
547,
550,
550,
542,
534,
536,
545,
559,
578,
586,
580,
588,
588,
577,
566,
564,
572,
579,
588,
595,
592,
588,
581,
585,
594,
594,
587,
578,
590,
600,
603,
593,
586,
577,
574,
574,
565,
555,
543,
530,
516,
506,
500,
493,
487,
479,
469,
462,
452,
440,
431,
426,
417,
406,
398,
388,
380,
370,
361,
360,
364,
372,
384,
395,
405,
416,
427,
439,
451,
464,
464,
453,
438,
420,
401,
383,
374,
362,
365,
374,
393,
410,
424,
430,
427,
425,
430,
443,
459,
476,
483,
493,
504,
512,
511,
507,
501,
497,
488,
476,
463,
453,
445,
437,
430,
425,
415,
404,
391,
379,
368,
370,
372,
369,
367,
357,
347,
344,
342,
337,
332,
326,
319,
311,
302,
298,
294,
289,
288,
288,
289,
289,
289,
289,
289,
289,
290,
288,
299,
309,
307,
307,
303,
299,
296,
294,
295,
298,
306,
309,
314,
319,
325,
332,
340,
343,
344,
342,
342,
347,
350,
355,
360,
366,
369,
370,
370,
375,
377,
380,
381,
378,
370,
364,
359,
353,
349,
345,
341,
337,
333,
329,
328,
327,
326,
325,
323,
320,
318,
316,
316,
314,
314,
313,
314,
314,
317,
320,
326,
331,
337,
344,
353,
355,
348,
339,
328,
316,
305,
300,
298,
297,
299,
300,
302,
303,
305,
307,
311,
316,
317,
316,
316,
316,
319,
321,
322,
320,
326,
328,
332,
337,
340,
345,
349,
349,
346,
344,
343,
342,
343,
342,
339,
334,
332,
331,
329,
328,
328,
328,
328,
327,
324,
322,
320,
319,
318,
318,
318,
318,
317,
315,
313,
310,
306,
304,
303,
302,
307,
315,
310,
304,
300,
297,
293,
290,
288,
285,
283,
280,
278,
276,
274,
271,
270,
269,
605,
596,
583,
578,
576,
571,
566,
561,
556,
551,
552,
553,
550,
547,
544,
546,
554,
564,
569,
568,
574,
590,
603,
610,
611,
595,
584,
580,
577,
571,
564,
566,
568,
568,
554,
534,
526,
532,
530,
523,
514,
503,
490,
476,
478,
472,
482,
474,
473,
482,
495,
501,
512,
524,
528,
522,
525,
523,
509,
490,
479,
466,
453,
439,
420,
413,
414,
427,
440,
446,
453,
462,
478,
496,
511,
524,
534,
539,
541,
555,
558,
553,
545,
535,
536,
548,
566,
587,
596,
595,
605,
602,
594,
590,
585,
589,
598,
602,
612,
611,
602,
596,
596,
600,
602,
601,
595,
604,
613,
613,
605,
595,
578,
567,
563,
562,
554,
539,
525,
510,
498,
491,
483,
474,
468,
463,
455,
444,
433,
426,
422,
415,
404,
395,
383,
375,
367,
362,
363,
371,
378,
389,
398,
409,
419,
427,
440,
453,
466,
464,
454,
436,
424,
409,
395,
377,
370,
368,
372,
388,
406,
417,
415,
411,
412,
424,
435,
452,
476,
491,
501,
504,
503,
498,
496,
494,
492,
488,
477,
463,
454,
447,
441,
432,
425,
415,
404,
392,
382,
382,
385,
391,
384,
379,
368,
354,
356,
353,
349,
342,
334,
325,
316,
310,
303,
296,
292,
289,
288,
287,
287,
287,
286,
286,
287,
287,
290,
302,
306,
304,
298,
296,
294,
293,
293,
295,
301,
305,
308,
314,
320,
326,
333,
342,
342,
338,
336,
339,
340,
348,
354,
360,
364,
366,
366,
369,
373,
375,
378,
379,
373,
366,
359,
353,
350,
346,
343,
339,
335,
332,
328,
326,
325,
323,
321,
319,
317,
316,
316,
315,
314,
313,
312,
312,
313,
316,
320,
325,
331,
339,
350,
356,
350,
340,
329,
319,
310,
303,
301,
300,
301,
301,
302,
303,
304,
306,
307,
309,
313,
314,
315,
316,
317,
322,
327,
327,
327,
326,
327,
329,
333,
338,
341,
344,
342,
341,
340,
339,
340,
340,
338,
334,
332,
330,
329,
328,
327,
327,
326,
326,
325,
324,
322,
321,
320,
319,
318,
318,
318,
316,
315,
312,
307,
305,
303,
302,
302,
310,
319,
314,
304,
299,
296,
292,
290,
287,
284,
282,
279,
277,
275,
272,
270,
268,
267,
600,
589,
591,
595,
598,
585,
576,
566,
557,
552,
549,
548,
546,
544,
546,
552,
553,
563,
577,
586,
588,
602,
620,
620,
606,
586,
566,
552,
546,
542,
538,
537,
546,
554,
552,
535,
519,
514,
518,
515,
506,
496,
482,
472,
477,
473,
475,
476,
475,
488,
502,
514,
523,
533,
540,
535,
527,
514,
496,
481,
467,
454,
444,
430,
414,
413,
416,
426,
439,
446,
454,
466,
481,
498,
512,
526,
537,
544,
548,
558,
562,
559,
546,
535,
533,
550,
571,
593,
611,
612,
618,
617,
610,
605,
601,
604,
611,
613,
627,
624,
611,
610,
608,
607,
611,
612,
608,
610,
612,
604,
594,
583,
568,
553,
545,
545,
544,
531,
512,
498,
491,
483,
474,
464,
454,
450,
444,
435,
426,
416,
418,
413,
402,
393,
383,
373,
365,
363,
368,
377,
382,
390,
400,
412,
423,
432,
443,
456,
466,
461,
450,
438,
429,
416,
402,
387,
375,
370,
370,
382,
395,
404,
401,
404,
419,
433,
447,
461,
476,
493,
501,
499,
495,
488,
487,
488,
492,
487,
477,
464,
454,
446,
437,
426,
417,
410,
401,
397,
394,
398,
400,
404,
397,
390,
377,
371,
371,
367,
366,
354,
344,
334,
329,
320,
306,
302,
296,
292,
294,
293,
292,
288,
286,
286,
286,
287,
289,
296,
296,
295,
294,
293,
293,
293,
293,
297,
299,
302,
307,
314,
320,
327,
333,
337,
336,
330,
329,
332,
339,
348,
354,
358,
361,
363,
364,
366,
367,
370,
374,
374,
368,
359,
354,
350,
346,
343,
340,
336,
333,
330,
327,
324,
322,
320,
318,
316,
315,
315,
314,
314,
313,
312,
311,
311,
310,
315,
317,
322,
329,
340,
350,
348,
341,
332,
322,
314,
306,
301,
301,
301,
301,
302,
304,
305,
305,
306,
308,
310,
312,
313,
315,
317,
321,
325,
330,
331,
330,
330,
327,
328,
331,
338,
339,
340,
338,
336,
334,
334,
334,
333,
332,
331,
329,
328,
327,
327,
326,
325,
325,
325,
324,
323,
322,
321,
320,
319,
318,
318,
318,
317,
316,
310,
306,
303,
301,
300,
300,
307,
318,
312,
304,
296,
293,
291,
289,
286,
283,
281,
278,
276,
274,
271,
269,
267,
266,
597,
599,
607,
605,
602,
592,
580,
570,
561,
551,
546,
544,
542,
542,
546,
559,
568,
565,
575,
594,
607,
611,
618,
611,
594,
588,
575,
557,
542,
531,
522,
522,
526,
535,
544,
539,
522,
502,
502,
502,
495,
486,
488,
476,
475,
474,
474,
473,
485,
494,
503,
519,
533,
546,
550,
538,
526,
504,
489,
471,
455,
442,
435,
425,
412,
413,
418,
430,
438,
447,
456,
469,
482,
496,
512,
526,
538,
548,
549,
551,
552,
551,
542,
534,
536,
551,
571,
592,
614,
624,
629,
635,
626,
617,
616,
618,
623,
626,
637,
634,
626,
621,
612,
612,
612,
611,
612,
613,
610,
598,
577,
570,
559,
548,
532,
526,
531,
526,
509,
490,
481,
476,
467,
455,
445,
437,
430,
424,
416,
411,
415,
409,
399,
389,
379,
370,
364,
362,
367,
375,
383,
394,
403,
414,
425,
435,
447,
459,
467,
459,
445,
435,
427,
417,
405,
391,
375,
370,
369,
374,
384,
390,
399,
408,
414,
425,
445,
463,
481,
492,
494,
493,
490,
486,
485,
487,
490,
484,
478,
469,
459,
449,
438,
426,
424,
419,
411,
409,
406,
411,
412,
415,
406,
401,
391,
387,
386,
384,
378,
368,
358,
347,
337,
326,
317,
313,
304,
304,
304,
307,
306,
295,
291,
288,
286,
286,
287,
289,
291,
291,
292,
292,
294,
296,
298,
300,
301,
304,
308,
315,
320,
327,
332,
332,
329,
327,
327,
335,
342,
351,
355,
357,
357,
358,
361,
365,
367,
367,
368,
368,
364,
357,
351,
346,
342,
339,
336,
333,
330,
328,
325,
322,
319,
317,
316,
315,
314,
314,
313,
312,
312,
311,
310,
309,
309,
306,
304,
317,
328,
339,
343,
337,
329,
322,
316,
307,
303,
301,
301,
301,
302,
303,
304,
307,
307,
308,
309,
311,
313,
314,
316,
318,
321,
326,
330,
330,
330,
330,
330,
330,
331,
336,
336,
335,
332,
330,
329,
329,
329,
328,
328,
328,
327,
326,
325,
324,
324,
324,
324,
323,
323,
322,
321,
320,
320,
319,
318,
317,
317,
316,
313,
308,
306,
302,
300,
298,
296,
302,
309,
306,
299,
293,
291,
289,
287,
285,
282,
280,
278,
275,
273,
270,
268,
267,
266,
605,
610,
608,
600,
593,
586,
573,
565,
556,
547,
541,
539,
537,
538,
545,
559,
576,
584,
584,
592,
610,
623,
619,
597,
578,
567,
570,
566,
561,
548,
537,
522,
511,
520,
531,
538,
525,
506,
490,
488,
486,
482,
485,
477,
473,
472,
472,
474,
486,
498,
507,
524,
539,
553,
549,
535,
516,
497,
481,
462,
447,
436,
425,
412,
410,
414,
421,
435,
443,
449,
460,
471,
481,
494,
510,
525,
537,
542,
546,
547,
549,
545,
536,
535,
540,
549,
566,
589,
610,
623,
628,
640,
646,
634,
634,
636,
639,
640,
649,
643,
635,
623,
609,
596,
596,
594,
599,
609,
600,584,567,554,549,538,522,510,516,516,503,489,472,464,459,449,440,431,421,413,409,404,407,404,393,384,375,368,365,366,373,377,378,392,405,418,430,440,450,461,465,456,444,432,421,411,404,393,378,371,368,375,386,397,409,420,430,435,442,457,478,490,489,487,486,484,483,486,488,481,478,472,464,454,447,443,440,435,430,427,422,423,418,415,410,404,400,399,400,394,388,379,370,356,343,337,331,326,320,320,319,318,311,305,303,293,288,287,286,287,287,289,290,292,295,300,303,303,303,304,309,314,320,324,326,324,320,318,329,337,346,354,355,354,352,354,358,360,362,362,362,363,358,352,348,343,339,336,332,330,328,326,323,320,317,316,315,314,313,312,312,311,311,310,309,308,306,305,304,314,326,334,335,327,319,316,309,304,301,301,301,302,302,304,307,308,309,310,311,312,314,315,316,318,322,325,329,330,330,330,330,331,333,333,332,331,329,328,327,327,327,326,326,326,325,324,322,321,320,322,322,322,321,321,320,320,319,319,318,317,314,313,309,307,304,302,299,295,294,297,300,298,293,291,289,288,286,284,281,279,277,274,271,269,268,267,266,612,604,592,582,572,570,563,553,546,540,537,534,532,534,545,562,582,596,599,602,614,621,616,594,571,546,546,554,560,552,545,536,509,509,517,523,521,508,490,479,480,475,473,471,467,470,469,475,490,507,521,531,544,553,545,532,518,505,490,479,470,448,431,415,413,414,421,435,442,448,460,469,478,488,506,522,532,539,542,544,546,541,535,538,544,549,565,586,598,607,619,633,647,644,647,650,654,654,658,646,636,624,609,594,582,578,589,597,592,572,557,543,531,525,513,498,500,502,498,488,468,457,448,442,434,425,417,406,401,399,400,399,391,381,371,366,368,373,380,388,389,393,402,418,430,444,456,464,460,452,443,434,422,412,400,389,380,371,369,375,385,399,409,418,430,441,452,465,478,487,484,482,482,481,480,478,477,473,471,468,464,458,454,453,448,443,437,432,429,424,416,413,410,408,406,405,402,397,392,386,380,370,361,354,349,341,336,331,327,320,314,308,306,300,294,288,286,286,286,287,288,290,296,303,306,306,306,307,309,313,317,319,320,315,314,321,331,339,344,350,352,351,350,352,354,356,356,355,354,357,354,349,344,340,336,333,329,327,326,324,322,320,318,316,315,314,312,311,311,310,310,309,308,308,306,306,304,304,321,330,329,320,315,314,305,301,301,301,302,302,303,307,309,310,312,312,313,314,315,316,317,319,323,327,328,329,330,330,331,331,331,331,330,329,328,327,326,326,325,324,324,323,322,321,320,319,318,320,321,321,320,320,320,319,319,318,318,317,313,310,306,304,302,300,296,293,292,292,292,292,291,289,287,286,284,282,281,278,276,273,270,268,267,267,266,596,588,581,574,564,558,553,549,545,542,539,536,532,531,541,563,575,588,598,606,615,617,610,596,579,554,534,530,538,544,546,538,516,496,498,504,513,507,491,474,479,475,464,463,465,470,475,486,499,513,527,539,548,551,541,534,526,511,498,491,482,466,445,423,415,414,420,432,441,447,456,467,476,486,502,517,528,534,538,542,544,537,536,542,548,554,564,576,596,609,624,633,635,637,639,645,649,656,646,633,626,623,609,595,580,567,571,580,587,571,554,537,524,511,502,490,484,483,482,481,473,458,448,436,426,417,411,402,396,394,394,391,390,377,369,366,369,377,389,399,405,404,409,419,432,446,458,463,455,447,440,432,424,415,403,392,379,371,370,377,385,392,395,403,417,429,438,456,470,477,476,475,478,480,476,473,470,467,462,460,457,454,454,450,444,439,432,426,424,420,415,410,406,406,405,404,399,394,388,384,380,375,369,366,357,356,348,340,331,322,315,310,305,300,296,290,286,285,286,286,286,288,291,300,305,307,305,303,304,306,310,313,312,307,315,324,332,339,344,345,345,345,347,349,351,353,352,350,350,351,351,345,340,336,333,330,328,326,324,323,321,320,318,316,315,313,312,310,310,309,309,309,308,308,307,306,305,304,312,321,320,313,306,305,303,301,301,302,302,303,306,308,310,313,314,315,315,315,317,318,320,322,325,326,327,326,328,329,329,329,328,328,328,327,326,325,324,324,323,322,322,321,320,319,318,317,318,318,319,320,320,319,319,319,318,318,318,315,312,308,304,302,300,298,296,294,292,290,289,289,288,287,284,284,283,282,280,278,276,272,270,268,267,267,267,593,581,570,558,554,548,548,552,555,554,550,545,537,530,535,547,558,577,590,596,606,603,590,588,583,564,542,523,519,526,532,536,520,500,489,494,502,502,490,473,476,470,461,462,466,473,479,486,498,512,526,536,547,542,532,524,528,519,508,499,488,470,453,429,417,411,420,431,440,441,450,460,468,479,492,510,524,532,535,538,543,545,542,547,558,563,574,588,598,610,622,624,627,630,626,631,637,644,636,621,608,609,606,596,578,560,559,571,580,572,550,535,522,510,500,485,474,469,469,472,469,462,451,440,432,422,408,398,390,384,385,389,379,373,368,369,372,378,392,402,416,418,423,430,438,452,464,461,454,445,436,427,421,412,399,387,377,373,376,380,385,391,395,401,404,414,429,444,460,468,472,473,476,478,479,476,475,475,471,465,456,448,452,445,437,432,426,422,416,415,412,406,401,402,401,400,394,390,385,379,375,369,366,362,359,355,351,340,331,323,315,309,304,301,296,293,287,285,285,285,285,285,288,292,300,305,302,300,300,302,304,307,305,304,315,325,331,337,341,340,340,341,344,345,347,348,347,344,344,344,345,342,338,334,329,328,326,325,324,322,319,318,317,316,314,313,311,310,309,309,309,308,308,307,307,307,306,304,305,314,314,309,304,302,301,301,302,302,303,305,308,310,312,315,317,317,316,316,317,319,321,324,327,327,326,326,326,327,326,326,325,326,325,325,324,323,323,322,320,320,320,320,318,317,317,316,317,318,318,319,319,319,318,318,318,318,317,314,310,307,304,301,299,296,294,293,292,290,288,287,285,283,282,282,281,280,279,277,274,271,269,268,267,267,267,600,591,582,576,574,572,567,573,574,573,571,561,544,533,529,537,546,573,578,581,594,595,570,562,566,566,552,535,514,507,511,520,522,510,488,486,492,493,484,474,464,463,463,463,466,471,476,487,502,513,527,538,537,532,523,524,524,520,511,499,484,467,450,429,417,415,421,427,435,437,444,450,462,476,491,508,525,537,539,539,545,548,552,561,563,560,569,581,590,603,612,621,626,625,616,612,626,634,625,613,600,589,592,590,579,562,549,562,574,572,558,542,513,498,494,486,468,461,458,461,461,457,453,445,439,426,413,402,393,382,379,377,374,372,372,376,377,379,388,401,420,431,437,442,448,459,466,459,451,443,434,424,416,408,397,385,376,376,385,393,400,406,410,413,413,417,425,438,453,464,468,470,473,475,477,476,477,476,472,464,454,446,443,440,432,425,421,417,412,408,405,402,397,396,396,393,390,385,381,377,372,366,362,356,354,353,348,342,334,328,319,312,305,301,298,292,290,286,285,285,282,280,283,288,295,300,298,294,294,296,298,297,298,307,317,328,332,332,337,338,338,340,341,342,344,344,344,341,339,340,341,338,334,330,328,326,326,325,324,320,318,316,316,314,314,313,311,310,309,309,308,308,308,307,307,307,306,305,304,306,307,305,303,302,302,302,303,304,305,306,309,311,314,317,318,320,319,317,317,320,323,325,329,327,326,325,323,324,323,323,322,321,321,321,321,321,321,320,319,317,318,318,317,315,315,315,316,317,318,318,318,318,317,317,316,316,315,311,308,305,302,300,297,295,292,292,290,290,288,284,283,281,280,280,279,279,277,275,272,270,269,268,267,267,267,600,596,590,591,588,584,577,574,570,573,576,565,548,538,529,530,544,558,554,564,579,587,560,546,547,550,550,540,519,499,497,507,512,510,494,482,483,484,477,466,457,461,463,464,463,470,474,485,504,519,529,530,520,514,514,526,525,518,508,495,480,462,442,430,419,419,427,429,441,444,449,460,472,479,490,507,523,531,533,536,540,549,557,559,550,557,570,582,593,606,619,623,620,611,598,596,607,621,619,607,590,574,574,581,578,563,545,544,558,561,557,540,513,488,484,477,466,456,450,449,450,448,447,440,431,418,406,396,386,379,378,378,377,377,380,385,385,383,387,401,417,433,443,451,457,465,464,456,447,440,432,423,411,401,393,382,377,378,388,400,410,421,427,429,432,435,442,450,462,466,466,465,465,471,476,474,469,462,461,454,445,437,439,433,427,420,414,413,409,404,399,394,391,390,392,389,385,381,377,373,369,364,359,354,350,348,344,340,337,331,323,314,306,304,304,299,294,289,286,285,284,281,280,281,292,296,295,291,291,292,292,291,296,306,318,323,329,329,332,336,337,337,338,339,341,341,340,338,335,335,334,334,332,328,327,326,326,326,325,324,320,317,316,315,314,314,312,311,310,309,309,309,308,307,305,305,306,306,304,303,303,304,304,303,303,304,306,307,307,308,311,313,315,317,320,323,322,322,321,322,324,327,329,329,327,323,320,320,320,320,319,318,318,318,319,320,320,319,318,316,316,317,316,314,314,314,315,316,317,318,317,317,315,315,314,313,312,309,306,304,302,298,295,293,292,291,291,291,288,285,283,282,280,278,278,277,276,274,271,269,268,267,267,267,267,584,583,592,596,595,583,570,561,556,562,565,566,556,543,532,527,530,534,536,548,565,575,558,538,530,532,535,538,520,501,487,495,498,500,494,474,475,476,467,460,459,460,461,463,469,476,479,476,498,513,521,519,511,508,515,525,521,513,500,485,472,456,443,428,422,422,441,450,445,451,455,462,471,478,485,501,512,522,522,534,534,546,550,543,539,548,559,576,591,605,613,608,600,589,580,582,593,607,611,599,586,572,560,569,574,563,546,533,542,548,545,534,512,488,476,469,462,453,445,440,437,436,435,431,424,414,399,386,381,379,380,384,386,388,392,396,394,390,388,398,409,422,437,445,454,462,462,453,443,435,427,419,409,398,386,375,377,381,387,397,408,421,433,441,445,452,456,464,465,464,462,457,459,466,467,465,460,452,449,445,437,428,429,425,420,414,411,410,407,404,397,388,384,384,387,385,381,377,373,370,366,361,356,352,346,343,342,334,331,332,326,317,313,312,311,307,301,294,289,286,285,283,281,280,286,293,291,290,289,290,289,292,296,303,311,316,322,327,332,334,335,334,335,335,336,336,335,334,330,329,331,330,328,325,325,326,327,329,329,326,324,322,320,317,316,315,314,312,311,311,310,310,309,308,306,304,306,305,305,304,303,304,304,305,305,307,308,310,310,312,313,314,316,320,324,329,328,327,324,323,323,327,323,322,321,318,317,318,317,317,316,316,316,317,318,318,319,318,318,316,314,315,314,312,312,313,314,316,318,317,317,316,316,315,314,312,309,306,304,302,300,296,293,292,293,293,292,291,288,285,282,280,279,279,278,277,275,273,270,268,267,267,267,268,268,574,572,579,581,583,581,567,549,542,543,546,548,551,549,535,525,521,523,531,543,554,560,556,538,517,513,518,522,520,503,485,481,485,491,490,474,468,466,461,460,459,458,461,466,474,484,493,490,487,500,504,502,506,502,504,510,512,506,495,478,458,442,434,426,423,435,452,464,462,452,458,462,462,461,471,478,494,503,512,517,526,538,540,532,541,551,562,573,588,602,606,592,580,574,564,567,580,593,598,588,574,564,552,558,565,561,542,523,530,536,533,527,512,490,474,461,456,450,438,434,429,422,421,422,416,407,394,384,379,380,385,392,398,401,407,411,407,400,396,403,406,416,426,437,448,456,455,451,440,429,421,415,406,395,382,378,378,384,394,403,411,423,434,442,446,451,456,460,457,454,451,447,456,464,466,456,449,442,436,434,430,424,420,418,414,409,404,407,406,404,398,387,377,376,380,379,380,376,370,365,361,356,352,348,343,338,331,327,327,326,327,321,318,322,321,314,307,299,293,288,286,283,281,280,281,283,287,289,290,292,296,303,306,306,306,315,321,327,331,332,333,332,332,331,331,331,330,330,327,326,326,326,323,320,322,325,328,330,331,330,329,327,324,322,320,317,316,314,314,313,312,312,311,310,308,308,306,305,305,304,304,303,304,306,308,310,311,312,313,314,314,315,318,322,326,330,329,328,324,322,322,321,320,318,318,316,315,316,315,315,314,314,315,316,317,318,318,318,316,314,312,312,310,309,310,311,314,317,317,317,316,315,316,316,316,315,312,307,305,303,302,297,294,295,295,295,293,291,289,290,290,288,281,279,278,277,274,271,269,268,267,267,268,268,269,568,558,561,564,573,575,572,555,538,532,533,536,538,539,538,524,517,521,535,539,544,551,552,538,518,498,501,506,512,506,490,474,474,482,485,473,462,461,458,457,458,462,465,472,480,491,502,509,503,497,500,508,511,496,491,495,497,496,488,470,453,434,427,425,431,447,466,478,479,479,477,474,477,481,480,479,484,483,490,501,513,523,525,534,543,555,570,582,589,597,590,578,567,557,548,553,567,577,585,582,565,553,546,549,554,552,538,517,510,520,523,514,509,490,474,459,451,443,434,428,419,413,407,408,406,398,389,382,375,385,393,399,408,414,423,424,420,412,401,396,397,413,428,437,448,454,452,441,431,423,413,406,401,391,382,378,380,390,397,405,416,424,431,435,438,445,451,452,447,443,438,440,451,461,466,458,446,435,425,425,422,414,413,412,408,402,400,403,403,401,394,387,377,368,373,373,373,372,368,362,354,351,348,344,343,339,333,326,318,318,321,322,325,328,330,320,311,302,295,290,287,284,281,279,280,283,286,289,292,297,305,310,317,316,316,318,322,328,331,332,330,330,329,327,326,326,325,324,322,321,320,320,318,316,319,323,327,331,334,337,336,333,328,326,325,321,318,316,315,315,314,314,312,312,312,313,312,310,307,305,304,304,306,308,310,313,314,315,315,316,316,316,318,322,325,329,329,326,323,321,319,318,316,316,315,314,314,314,313,313,313,313,314,314,316,318,320,319,317,313,308,307,307,306,306,311,314,317,322,322,321,316,316,319,318,317,312,307,304,303,301,300,299,298,297,296,293,291,293,300,304,301,291,281,279,277,274,271,269,268,268,268,268,269,270,557,553,543,543,553,559,560,563,554,544,536,525,525,526,528,523,514,515,523,526,535,545,548,537,517,497,487,493,498,497,485,473,467,474,472,463,459,456,455,455,455,463,470,479,492,507,514,524,517,502,514,521,516,497,484,477,485,485,477,459,443,432,426,428,440,457,477,492,497,496,494,486,486,498,502,503,504,499,502,499,501,511,526,538,549,560,568,575,581,586,575,564,555,543,534,543,553,560,573,574,565,552,535,541,546,545,539,521,500,502,508,498,502,489,474,459,447,435,426,423,416,406,400,392,393,388,386,384,385,394,404,409,415,423,434,436,433,427,410,399,402,419,434,445,453,451,448,440,425,414,407,399,390,384,380,377,381,391,396,402,406,416,427,426,433,438,440,437,434,429,428,440,452,462,461,454,448,440,428,418,414,410,406,405,404,399,391,395,396,392,388,383,376,364,365,364,366,367,365,357,350,345,343,339,336,333,332,329,320,314,317,319,320,322,325,322,314,304,295,291,288,284,281,279,281,284,288,292,296,303,309,315,322,325,322,323,326,329,330,330,328,326,325,322,320,320,320,319,317,316,317,317,316,316,319,323,328,331,336,342,343,341,335,332,330,326,321,318,318,317,317,317,316,316,317,318,318,315,310,305,306,307,309,312,314,316,317,318,318,319,319,319,321,322,324,327,326,324,321,318,317,316,315,314,313,312,312,311,310,310,311,312,314,316,317,322,322,321,318,311,305,306,306,304,307,311,316,322,328,329,326,320,321,324,321,318,313,307,305,303,302,302,302,302,300,296,294,294,302,308,308,306,296,286,279,274,273,271,270,269,269,269,269,270,272,540,542,539,529,535,543,548,559,562,561,549,538,523,516,515,512,508,507,511,518,528,534,536,532,517,499,486,480,479,481,475,464,462,464,462,456,452,452,452,452,461,469,476,478,494,513,520,534,527,512,524,530,520,500,487,470,470,476,467,452,426,426,429,439,457,470,480,497,511,513,509,500,497,512,519,521,524,518,521,518,514,523,529,537,545,556,564,570,575,576,566,554,544,532,524,534,536,553,565,562,556,545,530,533,538,537,531,517,500,487,495,487,492,488,474,458,446,437,423,417,413,404,395,388,384,387,388,388,399,409,418,422,425,432,443,450,442,430,412,397,409,422,435,443,443,443,441,440,427,416,406,398,389,382,381,378,379,388,393,395,401,413,418,418,427,428,426,426,423,424,432,444,454,457,456,447,437,433,426,415,409,406,402,398,397,392,387,388,390,386,380,378,375,366,355,353,358,361,357,351,343,338,337,333,328,327,327,324,318,312,312,314,314,316,319,319,316,305,296,291,288,284,280,279,282,285,289,293,298,304,310,317,328,328,327,324,325,327,327,328,325,322,320,318,315,315,314,315,314,315,314,314,314,316,318,323,327,329,336,341,350,348,339,336,333,328,325,324,323,324,324,324,323,323,326,328,323,317,311,307,309,310,312,314,317,318,319,320,321,323,325,325,323,323,323,324,323,322,320,317,315,314,313,311,310,310,309,309,308,307,307,311,314,317,322,328,329,323,318,310,304,306,305,304,305,311,317,324,331,333,329,326,327,330,325,317,313,305,303,302,303,304,306,305,300,297,301,302,305,310,308,304,295,285,279,275,273,272,271,271,271,271,271,272,273,539,531,524,520,519,524,532,539,543,550,555,550,536,521,510,506,505,501,503,509,520,521,518,521,515,501,483,472,468,467,466,460,456,454,452,450,449,450,451,457,464,476,488,494,497,507,520,537,533,524,538,532,512,497,479,466,455,464,458,442,426,429,440,446,465,481,492,498,510,521,521,508,510,524,534,542,542,539,540,533,525,527,533,538,546,564,575,575,570,559,554,544,533,520,518,524,536,550,556,549,540,537,522,521,532,536,519,507,493,476,478,475,481,480,471,451,440,432,419,412,408,402,393,388,385,391,395,399,407,420,429,433,435,436,448,453,442,430,414,400,411,425,432,434,434,435,432,428,419,413,410,405,399,392,386,381,377,381,385,390,399,406,408,410,417,417,412,413,415,424,434,443,452,457,454,446,436,427,416,410,402,402,398,389,389,387,380,378,382,379,375,372,370,367,355,350,355,355,350,344,338,332,330,328,324,321,320,318,314,310,306,308,309,311,313,316,314,306,298,292,288,283,280,279,282,286,290,294,300,304,311,318,328,327,323,322,320,322,322,323,321,318,317,315,310,308,308,309,312,314,313,314,313,315,318,324,328,331,337,341,352,352,344,341,335,329,328,328,328,331,333,332,331,331,332,332,327,318,313,311,313,313,315,317,319,320,320,321,324,329,328,326,324,323,322,322,321,320,317,316,314,312,311,309,308,307,307,307,306,305,307,310,316,321,326,331,328,321,317,307,304,303,303,303,304,309,315,322,330,334,330,328,330,330,324,317,312,306,304,304,304,307,307,303,302,301,303,305,307,311,304,296,291,283,278,275,273,272,272,273,273,273,273,274,274,554,546,537,527,511,510,518,524,531,540,548,553,549,533,519,512,509,504,498,499,503,503,499,502,503,500,485,468,462,460,457,454,451,449,448,446,449,452,456,462,471,474,493,506,513,517,530,539,537,538,542,533,516,498,480,464,447,440,442,426,429,434,451,462,462,476,491,503,514,520,526,517,513,529,544,555,560,557,561,557,542,534,538,546,556,569,569,562,555,542,535,537,532,512,515,525,537,544,543,538,524,523,513,511,526,528,516,498,481,465,461,465,470,468,467,458,446,433,420,409,403,398,390,386,392,398,406,414,416,426,439,443,445,447,453,451,440,429,414,402,407,417,425,423,426,426,425,426,428,428,425,417,413,407,399,387,380,379,382,388,393,397,400,404,403,404,412,415,422,428,435,444,452,455,452,443,433,422,413,405,400,396,393,387,378,380,376,370,373,372,368,366,367,361,351,345,351,350,344,339,333,328,324,323,319,316,315,312,309,306,303,304,304,305,307,307,305,307,300,292,288,283,280,279,284,288,292,296,300,305,312,319,329,328,322,319,317,318,319,319,318,314,312,310,306,304,304,307,309,310,312,313,315,318,322,325,330,335,340,346,354,354,350,344,338,334,334,336,338,341,343,342,339,340,338,331,325,316,315,314,314,316,318,320,324,321,323,324,326,328,329,328,327,323,321,321,320,319,317,315,314,311,309,307,305,305,304,304,304,303,307,312,318,319,324,328,322,317,309,304,302,302,302,303,304,306,312,318,325,331,332,333,330,330,321,316,312,309,309,309,309,306,303,301,302,304,309,314,315,307,301,293,284,277,276,275,274,273,274,274,274,275,275,275,276,544,538,529,518,510,507,510,513,521,529,535,549,558,545,532,528,520,506,497,492,493,492,488,486,484,488,481,471,462,457,452,450,449,448,447,448,451,455,460,465,478,487,494,512,525,531,538,535,539,546,546,533,517,502,487,470,454,438,426,426,431,439,455,463,477,475,485,494,502,512,526,526,524,531,541,555,566,574,572,568,559,545,545,553,564,562,552,537,540,533,521,522,527,512,511,524,532,532,527,531,525,510,500,501,513,526,515,500,479,463,449,455,459,459,464,460,446,433,420,407,398,392,388,389,398,407,419,428,433,436,443,450,454,456,453,445,435,424,411,401,402,411,411,413,418,422,426,431,436,438,436,432,428,418,406,392,383,379,381,386,391,395,400,399,402,406,413,420,428,435,442,451,454,452,445,437,429,418,410,401,396,390,388,382,377,373,368,362,366,365,361,359,359,355,341,340,345,344,341,336,331,326,319,317,316,314,312,306,305,303,301,301,301,302,302,303,304,305,299,291,286,282,279,281,286,290,293,297,302,306,312,318,325,324,321,317,315,315,316,315,314,310,306,304,301,300,305,307,307,310,312,315,317,322,328,328,333,340,345,352,357,360,355,348,340,340,342,347,351,353,354,351,348,345,340,330,326,317,316,315,317,318,321,324,327,330,326,326,327,328,329,326,324,321,320,319,319,318,316,314,312,310,307,306,304,304,303,302,302,303,304,307,313,316,318,321,317,310,305,302,302,302,303,305,306,308,312,314,317,324,330,332,331,325,316,314,314,314,315,315,313,305,304,301,303,304,317,319,306,301,295,288,281,278,275,274,275,274,275,276,276,277,277,277,277,528,521,521,523,526,513,497,504,510,518,527,542,557,551,545,539,531,515,494,492,490,487,481,475,471,472,472,465,461,455,452,449,448,446,445,447,450,460,467,471,483,501,506,517,525,530,525,525,538,548,544,528,512,496,479,467,448,434,427,429,438,448,462,471,475,490,499,505,506,510,512,516,524,534,540,548,559,567,569,572,564,556,552,555,557,551,538,522,525,530,519,511,520,512,510,520,521,516,512,524,525,508,493,492,506,513,509,497,478,462,440,439,445,450,460,460,446,431,418,406,395,388,388,395,404,418,429,439,443,446,451,453,453,453,450,440,430,417,408,401,408,411,417,417,422,426,430,438,445,451,452,445,434,422,408,396,386,380,380,387,393,400,404,409,413,416,418,423,426,437,444,450,451,446,440,428,421,417,408,401,395,391,386,380,374,367,362,353,355,356,351,353,354,348,338,334,340,340,336,341,329,322,316,313,313,312,309,306,303,302,300,298,297,296,297,296,297,297,294,289,285,281,279,282,289,293,296,300,302,306,312,316,319,318,318,316,313,313,312,312,310,305,300,296,292,296,302,304,307,309,312,316,321,327,333,335,338,344,351,358,364,366,358,350,345,349,353,360,367,369,366,361,357,351,341,338,331,325,317,317,320,322,325,327,329,329,328,327,328,329,326,323,320,318,317,316,316,315,314,313,311,307,306,304,302,303,302,301,302,302,301,304,312,317,318,315,309,304,303,302,303,304,308,315,317,318,316,315,314,317,322,326,324,320,314,317,320,320,321,320,315,309,305,297,302,313,322,314,301,291,284,280,277,277,276,275,275,276,276,278,279,279,279,279,279,530,535,536,543,537,518,499,498,500,507,519,536,547,538,536,534,529,520,501,493,498,490,480,472,464,460,459,456,453,451,450,449,446,445,445,444,451,464,478,482,486,502,516,523,524,518,517,532,542,546,539,523,507,488,463,449,435,430,427,432,438,450,465,479,487,490,503,514,520,514,520,523,523,524,533,539,548,558,567,567,556,546,544,543,540,534,522,511,514,525,517,508,514,512,509,515,513,510,510,521,524,509,492,485,496,499,494,486,471,456,439,433,436,443,455,453,444,427,415,404,394,388,392,403,417,427,438,448,450,445,448,446,445,446,444,434,423,412,406,407,416,416,413,425,428,430,435,442,452,457,454,443,432,422,409,397,389,383,382,391,401,407,415,419,424,429,431,429,431,436,441,444,442,439,431,424,415,410,402,397,392,390,385,381,376,368,359,351,347,346,341,345,347,345,337,328,329,330,330,329,323,317,312,308,316,316,310,307,303,300,298,295,293,293,293,292,291,290,290,284,282,279,280,285,291,296,299,302,305,305,308,313,314,314,313,313,310,310,310,308,303,298,296,292,292,295,300,304,306,308,313,317,325,331,338,342,345,350,357,364,368,368,361,353,352,359,367,374,379,381,376,370,366,359,353,349,342,332,324,320,322,326,329,330,330,328,328,328,329,326,324,320,317,316,315,314,313,313,313,313,312,312,310,309,306,303,301,300,301,303,306,310,316,318,314,307,303,302,302,303,305,309,319,324,328,328,326,322,318,315,316,318,321,318,318,326,328,329,325,316,310,304,299,292,295,309,312,303,293,281,276,277,277,276,276,276,277,278,278,279,280,280,280,280,279,546,553,555,552,536,521,504,494,492,499,513,531,541,525,514,512,510,508,505,508,507,497,484,468,460,454,450,450,450,449,448,446,445,444,445,446,451,468,485,496,498,506,518,518,513,507,520,538,542,538,530,518,502,487,463,441,433,427,428,434,442,451,462,475,492,501,499,510,523,523,525,532,534,532,538,545,549,556,566,565,552,536,528,524,520,516,511,508,509,517,513,500,509,510,504,512,502,504,511,523,518,504,486,479,484,482,476,471,462,449,438,430,425,436,446,445,439,423,412,402,393,387,394,403,419,430,439,447,446,441,438,437,436,438,438,427,418,412,410,414,417,419,419,425,429,433,438,446,455,463,450,440,430,420,410,400,390,385,385,392,400,406,413,415,423,431,439,440,440,440,442,438,432,425,420,414,412,404,399,390,388,385,382,380,375,366,358,350,341,338,337,337,340,338,332,326,321,320,320,321,317,313,307,304,310,311,308,305,301,298,296,295,292,291,291,290,290,281,282,281,280,279,282,288,293,298,303,305,305,307,306,307,309,308,309,307,306,307,308,309,303,296,295,292,293,296,299,302,304,308,314,320,329,336,343,347,352,358,364,369,372,369,364,358,360,367,376,383,391,392,386,378,368,366,361,354,345,333,328,325,325,329,329,329,331,329,329,328,326,324,321,318,316,314,313,312,311,310,311,314,316,316,318,317,313,306,300,297,298,303,309,316,319,315,305,300,298,302,303,305,309,319,326,332,334,338,335,330,325,321,317,316,320,324,328,334,337,334,322,313,304,301,292,290,289,297,297,292,281,282,280,279,278,277,277,277,278,279,280,282,284,284,283,284,282,556,560,565,551,534,517,506,493,487,493,508,527,533,521,501,490,491,486,496,502,498,485,469,460,455,451,449,448,448,447,446,445,444,444,444,450,460,473,491,506,513,514,513,503,499,506,520,535,532,526,518,508,495,482,459,441,431,427,430,438,448,454,466,478,497,509,512,511,520,526,524,529,533,528,537,544,550,552,553,561,557,539,528,513,508,506,503,500,505,509,507,498,503,503,499,501,499,508,521,523,514,498,480,474,476,471,466,460,454,444,429,428,422,430,431,429,422,417,406,398,391,388,397,412,422,429,435,431,433,428,426,428,430,430,430,426,419,415,413,410,414,417,424,427,429,436,441,449,459,463,452,441,433,426,418,406,391,387,386,389,395,400,404,410,418,423,432,442,440,437,433,427,423,421,414,412,410,405,398,390,383,378,379,377,374,367,360,353,344,338,330,327,329,329,326,320,315,315,314,316,313,308,304,303,306,305,304,302,296,294,293,293,291,290,289,288,283,281,280,279,279,282,286,291,295,300,305,305,306,307,304,304,304,305,305,304,303,305,306,306,306,304,293,291,293,296,298,301,304,308,314,321,330,338,344,350,357,365,371,376,376,374,368,364,361,364,373,379,384,384,380,375,364,363,354,348,340,330,327,326,327,328,328,329,329,328,326,325,323,321,318,316,315,313,309,308,308,307,308,316,322,325,324,321,317,307,300,294,295,302,305,314,313,305,299,296,300,303,306,310,319,322,321,326,331,340,343,338,332,327,323,323,324,329,330,342,342,333,323,313,308,302,297,293,290,290,291,289,282,282,280,279,278,278,278,279,279,280,282,285,288,289,290,290,287,553,570,570,556,541,523,510,495,485,486,500,517,520,516,503,487,472,472,488,494,490,472,459,451,451,449,447,447,446,445,444,443,443,444,444,450,462,478,494,512,519,514,500,492,499,510,516,514,512,509,503,495,484,473,452,437,429,427,433,443,459,463,462,473,487,502,513,514,516,516,510,520,517,515,526,533,535,538,550,555,554,548,540,528,512,498,499,496,501,498,499,497,502,502,498,498,499,506,519,524,516,500,486,472,464,461,455,447,441,433,424,420,414,419,419,414,410,407,400,392,389,388,398,411,417,420,424,426,423,419,417,424,423,420,424,425,423,418,414,412,413,415,424,429,433,437,444,451,452,455,451,442,439,431,424,407,392,387,386,388,389,390,396,403,409,412,424,431,428,425,423,426,426,423,420,417,414,405,397,388,380,376,374,374,370,367,362,356,349,343,334,327,321,320,317,316,314,308,307,308,308,307,304,302,303,302,300,297,293,292,292,292,291,290,289,286,283,283,281,279,280,284,288,291,294,297,300,302,304,304,302,301,301,301,301,302,304,306,306,305,305,300,292,291,292,295,298,301,304,308,315,322,330,338,344,352,361,370,377,382,383,380,372,363,354,358,364,369,373,373,369,363,357,352,346,341,334,329,327,326,325,325,325,325,324,324,322,321,320,318,316,315,314,312,310,306,305,306,311,320,329,330,326,320,314,307,299,292,291,299,307,307,304,299,295,295,301,306,313,321,325,319,313,314,323,330,337,343,338,332,329,327,329,325,333,337,329,326,320,315,312,309,307,301,295,292,291,287,284,281,279,279,279,279,279,280,281,283,285,289,292,295,297,296,291,546,562,574,561,552,536,516,500,486,481,489,500,506,503,496,480,470,463,477,483,481,469,456,450,448,447,446,446,445,444,443,443,443,442,448,459,471,485,499,514,518,513,500,486,500,508,503,497,498,496,488,481,471,456,441,432,427,428,432,442,457,469,476,479,484,496,501,508,507,498,498,500,507,510,512,519,522,530,537,547,550,551,542,525,512,500,493,491,491,487,490,490,500,500,498,499,502,506,512,512,515,504,491,476,461,450,444,437,427,427,428,424,414,406,406,405,402,400,397,393,390,390,393,401,406,412,416,418,417,414,411,412,413,413,420,427,430,427,420,414,416,421,427,430,434,438,447,451,450,450,446,437,438,429,420,405,391,384,387,388,387,388,389,395,399,406,415,419,418,415,424,427,428,428,426,417,410,402,394,387,381,372,368,369,368,367,362,356,349,342,334,325,317,315,316,314,311,307,304,304,305,306,304,302,301,300,299,297,294,294,293,291,290,290,289,288,287,283,281,279,280,284,287,290,293,294,296,298,300,300,299,297,296,296,296,298,302,303,301,299,298,295,291,291,292,295,298,301,304,310,317,324,332,339,346,355,364,374,383,390,392,387,375,363,354,352,355,360,363,363,359,356,351,344,339,334,329,327,326,323,322,321,320,321,320,318,318,319,318,316,314,313,314,313,309,308,304,303,306,314,320,326,322,317,309,303,296,292,290,296,304,300,297,295,294,297,304,311,319,330,324,313,304,304,315,318,323,333,333,329,328,322,328,326,322,323,318,314,308,311,317,319,317,313,307,299,292,290,284,280,281,280,280,280,280,281,283,285,289,292,296,303,306,302,293,533,549,555,563,558,544,526,506,490,479,478,483,486,484,477,470,465,463,468,469,466,459,452,447,447,446,445,444,444,443,442,442,444,447,458,471,484,496,505,500,500,508,498,480,492,495,492,477,483,480,476,466,453,441,432,427,427,428,434,446,460,473,482,488,488,490,496,502,493,495,504,512,522,527,522,519,527,538,539,535,541,547,540,528,514,504,495,486,485,486,487,489,495,497,497,499,502,504,508,509,513,509,501,490,474,459,449,439,434,432,433,429,421,408,399,399,401,400,397,394,391,392,393,397,401,405,409,410,409,406,404,405,408,413,418,424,428,427,426,423,419,422,427,431,435,440,448,450,448,442,439,430,432,427,414,402,390,386,388,390,391,392,393,397,400,401,402,410,411,415,425,428,427,428,423,415,406,398,390,384,380,371,364,365,364,366,361,356,350,340,333,325,317,316,317,317,315,311,307,304,302,304,302,298,298,299,299,299,296,294,293,292,291,291,291,292,288,285,282,280,279,282,284,286,288,290,293,295,295,296,296,294,292,292,293,296,297,297,296,292,292,292,290,291,292,295,297,301,305,311,317,324,332,340,348,357,369,379,389,398,399,390,379,366,355,350,347,350,351,351,348,345,342,336,331,328,326,324,322,320,320,319,318,317,316,316,317,316,315,313,312,313,314,317,319,314,304,299,303,310,318,323,319,314,306,299,293,290,289,292,295,294,292,292,294,302,310,317,324,329,321,310,298,295,304,305,313,324,324,318,316,318,319,321,318,310,307,303,303,310,318,323,326,324,318,309,299,292,283,282,282,281,282,282,282,283,284,287,290,295,301,309,314,306,297,520,528,541,552,551,548,532,512,490,481,473,472,472,471,468,465,462,458,456,456,454,451,448,446,445,445,444,444,443,442,442,442,443,451,467,482,496,501,496,486,491,496,490,470,472,477,478,462,463,465,457,449,440,431,428,427,427,429,434,447,460,470,482,490,489,486,488,489,488,501,502,511,523,538,536,526,538,533,525,522,532,537,533,527,517,507,497,486,485,488,491,492,493,495,494,497,499,500,504,511,499,495,486,483,474,463,452,438,427,433,439,437,430,423,420,417,409,401,397,394,393,393,394,396,399,402,404,404,403,401,402,401,406,411,416,423,427,428,428,428,425,424,427,432,436,440,449,455,450,439,434,426,426,424,410,396,386,388,388,389,392,397,403,405,408,405,405,407,412,414,422,424,427,425,419,412,402,395,388,380,376,369,364,361,360,360,357,354,348,340,331,327,322,324,324,321,318,314,309,306,304,302,297,295,295,296,306,299,294,293,292,292,291,291,293,292,289,288,285,283,279,279,280,282,283,285,288,291,292,293,293,291,290,290,292,295,294,293,292,291,291,291,290,291,292,293,297,301,305,310,316,324,332,340,349,360,371,383,394,404,400,392,383,371,359,349,341,340,340,340,339,339,334,329,327,325,323,321,319,318,317,316,316,314,314,314,313,312,312,311,308,312,317,317,316,313,304,297,302,311,319,319,315,308,302,295,291,289,287,288,290,289,287,290,297,307,315,320,321,320,314,304,293,291,298,304,314,319,315,307,308,318,321,320,316,308,299,297,301,307,313,321,330,327,317,306,296,290,286,283,283,283,284,284,284,285,286,288,292,297,303,314,319,311,301,510,519,532,537,542,542,537,519,500,483,470,471,469,468,468,467,465,458,452,450,448,447,445,444,444,444,443,442,442,441,440,440,446,458,476,489,494,491,485,473,477,484,479,462,450,461,462,454,450,447,443,437,432,428,428,429,431,430,432,439,451,467,476,483,482,477,480,484,488,488,490,506,524,538,535,525,528,522,509,516,528,529,524,518,509,499,498,497,486,488,490,490,491,492,493,494,496,499,502,501,485,480,475,470,463,454,444,432,426,433,440,441,436,430,428,419,409,401,397,394,394,394,395,396,398,399,401,402,400,401,401,402,406,412,419,427,429,428,428,427,427,427,430,436,439,446,452,452,440,433,428,418,419,418,408,394,386,388,390,392,393,398,402,409,416,415,414,415,414,417,422,425,426,422,415,407,400,392,384,377,370,365,360,357,356,357,355,350,342,338,329,326,330,330,328,323,319,316,310,307,304,299,294,292,293,294,295,293,292,293,292,291,292,292,292,291,291,291,292,286,282,279,279,279,279,281,283,286,288,290,290,289,289,290,291,293,292,291,290,290,290,290,290,290,291,292,296,300,304,310,316,324,333,341,350,361,374,386,397,399,391,388,382,373,361,350,341,337,334,333,333,331,329,327,324,322,320,318,316,316,314,314,314,313,312,311,310,309,308,307,306,311,319,316,317,306,296,293,301,308,314,312,306,302,296,292,289,287,285,285,285,284,280,287,296,306,314,315,312,308,302,296,290,293,302,308,308,309,305,302,309,315,319,318,311,299,295,295,299,302,312,324,331,318,304,297,290,288,286,285,285,285,285,285,286,287,288,289,292,297,303,314,321,315,306,511,508,513,519,524,526,528,524,508,491,482,478,472,474,472,480,472,463,455,449,447,446,444,444,443,442,442,441,441,440,439,440,450,464,478,480,477,475,467,459,469,471,466,453,439,444,444,442,439,436,433,431,430,428,429,434,437,434,431,437,452,460,469,474,472,472,475,482,481,476,485,500,518,524,524,512,516,514,498,509,520,527,516,510,505,506,510,500,490,482,487,488,487,490,491,490,490,489,494,489,477,468,462,458,451,443,434,427,425,432,440,439,430,425,423,415,402,398,396,394,394,394,395,396,398,399,400,402,402,402,402,402,407,413,422,428,428,428,428,427,426,427,433,438,444,451,453,446,437,425,419,412,410,412,404,391,386,388,391,394,398,400,400,405,414,417,418,419,420,423,427,429,428,420,412,404,396,388,381,374,366,361,355,352,352,352,352,344,340,337,329,332,332,331,328,322,317,314,310,307,305,300,294,293,293,293,293,293,293,293,292,293,293,293,291,291,292,293,292,293,285,280,279,278,278,279,280,282,284,285,287,288,290,292,292,290,290,290,289,289,289,289,289,290,291,293,296,300,304,310,316,324,333,342,351,362,377,388,395,389,380,376,375,369,360,350,340,333,328,328,328,327,325,322,320,318,317,316,314,314,312,312,311,311,309,308,306,305,305,304,304,311,318,314,306,300,293,292,298,302,302,301,302,300,293,290,288,284,283,282,282,281,279,282,294,304,308,306,301,297,293,291,288,293,304,304,299,298,299,302,310,315,317,303,298,295,293,294,299,308,315,326,330,318,304,292,289,287,286,286,286,286,287,287,288,288,289,290,292,297,303,315,326,317,310,512,498,498,502,504,508,511,515,512,499,494,488,487,487,485,488,483,473,459,451,449,446,445,444,443,442,440,439,439,439,439,439,449,461,466,461,460,452,448,456,463,458,449,439,437,435,434,432,432,431,431,428,428,430,430,438,448,443,436,435,444,450,462,457,458,471,478,483,486,475,475,491,505,506,508,498,501,501,496,501,511,522,526,522,519,520,516,510,500,475,471,472,482,489,488,488,487,478,480,476,469,461,450,445,440,430,424,417,420,427,436,430,424,419,416,407,397,396,395,394,394,395,396,397,398,400,402,403,405,403,404,406,411,415,424,430,428,427,428,428,426,428,436,441,448,452,444,435,429,419,411,402,399,400,393,388,389,388,392,397,401,404,402,405,412,416,421,424,424,426,429,432,429,419,411,402,393,384,376,371,364,357,354,350,343,346,346,342,339,336,334,334,333,331,327,321,317,313,310,307,304,301,293,293,293,293,293,293,293,293,293,293,293,293,292,290,292,292,292,292,286,283,281,279,278,278,279,280,280,282,284,286,290,293,292,289,288,288,288,287,288,288,289,290,291,293,297,300,304,309,315,323,331,342,352,364,379,391,392,381,365,363,362,359,354,347,338,332,328,325,324,323,321,318,316,315,314,314,312,311,309,309,309,309,307,306,304,303,302,302,302,307,307,305,298,294,291,292,293,293,295,301,302,304,297,291,287,283,281,280,280,279,280,287,295,304,308,302,296,289,288,288,287,292,295,296,295,295,299,310,306,306,301,296,293,292,292,294,298,303,312,322,323,316,303,293,287,287,287,287,288,288,288,289,290,290,290,292,294,299,304,317,330,320,312,521,509,493,482,485,490,494,497,501,505,509,501,500,500,501,499,491,477,458,451,448,448,446,446,445,443,444,444,443,442,441,439,439,447,449,447,439,439,439,447,445,439,440,436,434,432,430,430,428,428,428,435,432,430,433,447,460,459,443,437,437,441,445,441,452,466,475,478,481,476,464,480,489,489,490,489,484,489,497,504,510,515,521,529,529,531,525,515,499,476,459,461,476,477,477,476,475,464,467,463,456,451,440,426,429,428,417,412,421,430,431,425,418,412,408,402,396,395,394,394,394,395,396,397,398,400,402,404,404,404,404,406,410,415,427,428,426,425,423,428,428,432,438,447,444,442,429,423,417,413,411,414,401,390,389,390,390,390,392,398,403,413,406,406,415,419,423,427,430,433,437,437,426,414,406,399,389,381,374,367,360,354,348,345,340,340,340,340,340,338,336,333,330,328,324,319,316,311,307,304,303,301,298,296,293,293,293,293,294,294,294,294,293,293,292,292,291,291,291,290,288,285,283,281,279,278,279,279,280,281,282,283,286,290,289,288,286,286,286,286,286,287,289,290,292,294,297,300,303,307,313,320,328,340,352,365,381,390,383,373,361,352,350,346,345,341,335,329,325,321,320,319,318,316,314,313,312,311,310,308,306,306,307,311,311,308,305,303,300,299,299,302,300,295,291,289,287,287,290,290,296,303,305,307,304,292,287,283,280,279,279,278,280,288,294,302,308,304,296,288,282,282,284,288,291,293,294,297,307,307,303,296,295,292,292,292,293,294,297,301,304,315,316,307,298,291,289,288,288,289,290,290,290,290,291,292,292,293,297,302,309,320,326,323,317,538,522,508,490,474,476,478,487,500,505,511,515,514,522,514,499,483,466,456,450,449,449,449,446,447,450,451,450,454,451,447,442,442,439,439,439,439,442,439,439,438,437,436,435,434,433,431,432,434,437,443,441,434,431,434,448,451,452,452,441,437,437,436,440,449,458,462,462,466,463,459,470,477,475,472,474,487,501,510,517,525,526,531,536,537,542,535,514,498,476,456,447,460,463,462,463,460,455,455,451,444,438,431,423,416,417,413,413,421,429,428,419,416,414,401,398,396,394,393,394,394,395,396,397,398,399,401,402,402,401,401,402,406,411,418,420,417,414,420,428,434,438,447,454,446,436,427,427,426,419,422,417,409,396,390,392,393,394,397,402,408,414,414,412,413,420,425,431,436,439,441,439,422,411,399,390,386,378,370,364,358,350,340,340,331,332,336,338,341,340,339,334,329,326,320,316,314,309,304,304,305,304,302,298,296,295,294,294,294,294,294,294,294,293,292,291,290,290,291,291,288,287,286,284,281,279,278,279,279,279,280,281,282,284,285,284,285,283,284,284,285,286,288,290,292,293,295,299,302,304,312,320,330,343,355,369,381,380,374,367,359,351,342,339,337,331,328,325,320,316,317,316,314,312,311,309,309,308,308,306,304,304,308,316,320,317,308,303,298,296,295,294,293,292,287,284,283,282,286,293,305,310,312,312,304,291,286,282,280,279,278,280,283,290,296,304,308,308,297,288,282,281,280,284,289,292,295,302,309,312,303,295,291,291,292,293,298,305,306,304,303,309,309,299,293,288,289,289,290,291,292,292,292,292,292,294,294,295,300,304,313,321,323,319,320,541,534,520,504,488,473,473,482,486,490,497,506,512,514,505,492,474,460,452,449,449,451,456,451,452,464,466,461,468,468,463,448,441,441,443,450,458,459,444,438,438,438,438,438,437,436,438,441,438,447,452,440,434,432,432,436,439,439,445,446,439,437,438,440,441,448,451,450,452,450,454,461,462,460,463,475,486,501,518,528,536,538,533,537,541,551,531,509,491,473,456,438,446,451,450,452,448,446,442,441,434,426,419,414,412,411,410,412,417,423,420,413,412,411,400,397,395,394,394,394,395,396,396,397,398,399,400,401,402,402,402,402,402,404,410,411,412,414,421,431,436,442,453,451,438,435,434,432,428,426,428,426,412,399,393,394,395,397,399,402,409,414,412,414,416,422,426,434,440,443,442,434,420,409,396,387,380,376,369,364,358,352,341,330,328,330,333,335,339,340,341,335,329,325,319,316,314,310,308,307,305,303,302,299,297,296,295,295,295,294,294,294,293,292,291,290,288,289,293,292,290,289,287,288,284,281,278,278,279,279,279,280,281,282,282,281,282,282,282,283,284,286,287,289,290,292,294,298,302,306,315,324,333,345,358,370,379,371,365,360,355,349,343,337,331,326,324,321,318,315,314,312,311,309,308,306,306,306,307,305,303,304,308,320,330,322,311,302,296,294,291,290,288,286,283,281,281,280,287,296,306,316,317,310,296,284,283,281,280,279,279,280,284,290,295,300,306,309,301,290,282,279,281,285,289,292,296,306,316,313,302,289,285,290,293,297,305,311,315,315,312,307,298,292,287,288,288,290,291,292,294,294,295,295,294,295,296,298,302,304,313,321,318,314,317,523,525,528,512,492,472,463,465,470,475,485,494,498,501,495,480,464,454,450,451,452,455,462,451,462,477,488,475,480,482,471,459,459,461,467,472,470,463,442,442,443,445,438,443,445,442,442,445,440,450,447,438,433,432,432,434,435,438,440,440,439,439,439,439,440,439,441,447,450,450,450,451,459,468,477,486,498,512,520,535,534,524,524,526,534,537,524,502,485,469,454,435,433,439,440,439,438,435,427,426,422,416,413,411,408,406,404,406,411,415,413,406,406,404,399,397,395,394,394,395,396,397,397,398,398,399,400,402,402,402,402,402,402,402,402,409,412,416,423,431,438,446,454,449,442,442,441,437,431,426,423,423,412,400,395,397,399,401,402,402,407,410,412,416,423,426,431,438,442,442,436,428,416,404,394,384,376,369,368,364,358,352,343,332,327,326,330,333,342,341,341,336,330,326,322,318,315,313,310,308,305,303,302,300,298,297,296,295,294,294,293,292,292,292,290,288,288,288,293,292,290,289,288,286,285,283,281,279,278,278,279,280,280,281,282,280,280,281,281,281,282,285,287,288,290,292,295,300,303,310,320,329,338,349,361,370,371,363,354,352,349,344,341,334,330,325,319,318,316,313,311,309,307,306,305,304,304,305,309,308,304,300,308,316,325,322,312,300,292,294,291,286,284,283,282,281,280,281,287,294,305,315,318,310,297,281,280,280,279,278,278,279,283,286,290,292,299,305,305,293,284,279,281,285,290,293,296,302,306,307,296,287,283,287,293,301,308,314,318,320,312,301,290,285,285,286,288,290,292,294,295,297,299,300,299,298,298,300,303,304,310,317,313,308,312,508,513,521,518,504,485,470,463,459,466,476,480,484,489,478,466,455,450,450,453,448,462,470,462,461,476,483,484,492,492,480,478,480,480,479,476,465,450,447,447,455,454,447,449,452,451,449,453,448,443,439,435,433,432,433,434,435,437,438,439,438,437,443,441,439,445,449,449,452,454,454,458,467,481,495,505,510,516,525,538,533,519,511,515,524,525,510,494,478,464,452,437,425,428,432,434,431,425,422,417,413,412,410,408,405,402,401,402,405,405,404,401,400,401,399,397,396,396,396,397,398,398,398,399,399,400,401,402,402,402,402,402,402,404,404,411,415,418,423,431,441,451,461,460,452,453,449,437,427,414,416,416,408,399,398,399,401,404,405,407,410,411,414,421,428,434,437,440,440,436,427,417,407,399,390,380,371,364,362,360,356,351,344,334,323,319,325,332,336,338,338,334,332,329,325,321,318,314,311,307,304,303,301,300,298,297,296,294,294,292,292,292,291,290,291,291,286,286,290,290,288,287,287,287,285,283,281,280,279,278,278,279,280,281,281,281,279,280,280,280,282,284,286,288,290,292,296,301,304,314,324,335,346,355,363,367,366,358,350,342,342,338,333,329,325,320,316,315,312,310,308,306,304,303,302,301,303,309,317,311,302,297,302,311,317,320,311,298,290,293,293,288,284,281,281,280,280,283,288,295,305,315,320,312,299,285,280,279,279,278,278,279,280,286,292,295,301,305,307,296,286,279,279,281,283,288,291,294,298,300,290,283,281,284,290,296,304,309,314,313,302,291,284,280,282,284,287,290,293,295,298,300,303,305,303,302,301,301,302,304,307,309,305,303,306,497,498,508,516,514,499,483,473,460,463,468,468,472,476,465,457,451,450,454,459,452,470,483,480,465,465,474,489,502,502,496,497,492,485,473,462,453,451,449,449,464,466,455,460,462,462,458,452,439,435,436,434,433,433,434,436,437,438,438,438,438,440,448,446,440,446,449,450,453,456,460,469,479,492,506,515,516,514,520,525,526,516,508,508,514,511,500,486,474,459,447,433,423,427,430,430,428,419,417,414,412,410,407,405,403,400,400,400,401,401,400,399,399,399,399,398,398,397,398,399,401,400,400,399,399,400,400,401,402,402,402,402,404,406,407,411,413,416,425,434,444,456,465,460,454,448,441,434,426,416,410,405,399,396,399,400,402,405,408,411,413,414,418,426,434,441,447,447,437,428,420,410,400,392,386,377,367,359,353,354,354,349,345,339,325,315,321,328,329,330,333,331,329,328,324,320,317,314,310,308,304,303,301,300,298,296,294,294,293,292,291,291,290,290,292,293,287,285,287,287,287,288,287,286,285,282,281,280,279,278,278,278,279,280,281,281,279,279,280,280,282,283,286,288,291,294,299,304,312,322,331,344,354,360,358,359,356,352,345,338,332,329,326,322,318,316,313,312,309,307,305,303,301,300,299,300,303,312,319,309,296,292,300,310,318,319,308,296,292,289,293,294,287,283,281,281,280,283,286,295,304,313,318,311,301,288,280,279,278,277,277,278,282,288,295,302,306,308,307,298,288,280,278,279,280,283,285,290,293,294,281,281,280,284,290,294,302,308,306,300,290,284,279,280,281,284,287,290,292,295,299,302,305,307,305,304,302,302,302,303,304,304,303,301,306,494,489,494,504,514,504,492,477,459,461,464,461,461,460,457,453,452,452,456,462,462,476,490,493,488,474,476,489,499,498,502,498,488,476,461,452,457,456,455,457,465,477,471,476,481,475,466,451,444,438,435,434,434,434,437,438,441,442,440,438,439,441,442,445,444,448,451,451,451,452,462,474,486,500,512,523,514,505,507,512,513,508,498,500,503,501,487,480,475,465,452,436,424,422,428,427,421,414,413,412,410,407,404,403,401,400,400,400,400,400,399,398,398,398,399,400,400,399,398,401,402,402,401,400,400,400,400,401,402,402,403,405,406,409,413,414,414,421,430,441,450,460,462,448,439,438,434,427,422,417,409,401,398,399,399,400,403,407,410,414,419,422,426,432,440,449,452,442,431,420,411,402,392,385,380,373,366,357,350,347,347,345,342,338,329,315,316,320,322,326,330,327,324,322,320,317,315,313,310,308,306,304,302,299,297,296,294,292,292,292,292,291,290,288,290,291,287,286,285,286,286,286,285,285,283,282,280,279,279,278,277,277,278,279,280,280,281,280,279,280,280,283,286,289,292,297,302,311,322,332,344,354,365,362,352,350,347,344,340,333,326,322,320,317,315,313,311,309,306,304,301,298,297,296,295,296,302,308,311,302,292,291,300,310,318,318,305,295,287,288,292,293,295,289,285,283,280,280,283,290,297,304,309,308,299,289,280,278,277,276,277,279,280,289,296,303,303,304,306,300,288,281,278,278,279,280,280,289,292,295,282,279,280,284,292,296,301,306,303,288,284,280,279,280,282,284,288,290,292,296,300,303,305,306,305,305,304,303,302,303,303,302,304,305,310,494,485,483,489,498,500,496,485,463,460,458,456,455,455,455,455,457,463,463,467,460,475,484,488,489,478,475,486,488,493,490,484,477,465,460,463,465,466,471,474,474,487,487,488,487,488,472,457,444,438,435,434,434,437,442,450,450,445,441,438,440,442,443,444,444,448,449,448,450,455,468,479,493,509,516,516,506,495,494,496,497,496,487,487,489,487,476,471,468,464,452,435,422,413,416,415,414,412,410,408,407,405,402,401,401,400,400,400,401,400,399,398,398,400,400,403,405,402,401,405,406,405,402,401,401,401,401,402,402,403,404,404,406,410,413,417,422,426,436,445,453,459,456,445,437,433,429,424,419,413,405,400,400,401,401,401,405,410,413,417,426,431,435,437,441,452,447,435,424,414,406,398,388,377,370,367,363,356,348,340,340,341,338,333,330,318,317,315,315,320,322,320,318,317,315,313,311,308,309,307,305,303,302,299,296,295,293,292,292,292,293,292,289,288,286,288,287,286,285,283,283,283,284,283,282,281,279,279,279,278,277,277,277,278,279,280,280,279,280,279,281,283,287,290,293,299,307,318,331,343,354,366,368,356,343,339,339,336,332,328,322,316,314,312,309,308,307,307,304,302,298,293,293,293,291,294,298,302,301,296,292,290,296,305,312,312,302,292,282,284,288,290,297,297,291,286,285,280,283,286,291,296,300,300,293,285,280,278,276,276,276,278,281,290,296,297,293,293,297,298,290,282,278,278,278,279,283,289,294,292,280,279,280,283,288,293,296,303,296,286,280,279,280,282,284,286,288,290,292,297,302,305,305,306,305,305,305,303,302,302,302,302,304,310,317,488,481,476,477,482,489,484,476,465,459,457,455,452,454,460,460,464,466,476,476,462,464,474,477,481,480,475,487,489,482,479,475,468,461,464,468,479,484,491,491,489,490,492,488,486,482,468,451,442,437,434,434,435,438,447,460,455,444,441,439,442,446,447,447,451,449,453,450,452,461,471,484,497,513,507,501,495,485,484,484,483,485,489,479,473,471,466,459,455,452,441,430,420,414,411,411,410,410,408,406,405,404,401,401,400,400,400,401,402,402,399,399,399,401,405,415,412,406,405,410,416,413,404,402,403,402,403,403,403,404,402,403,406,410,413,418,427,432,440,447,453,452,447,442,440,430,423,419,416,409,401,400,402,404,403,403,408,413,418,424,432,440,444,446,444,444,441,430,417,402,398,391,383,375,365,361,358,353,346,340,335,334,333,329,327,320,316,315,314,316,316,314,312,311,310,309,307,306,307,306,305,303,300,297,296,294,292,291,291,291,292,292,289,288,286,285,286,284,282,281,280,280,282,282,282,279,279,279,279,278,277,277,277,277,278,278,280,279,280,280,282,284,287,290,294,300,309,322,335,348,363,364,364,351,339,330,330,329,324,320,318,313,308,306,305,304,305,306,304,300,296,292,291,290,289,292,294,294,293,291,291,290,291,296,305,308,300,290,283,281,281,291,298,302,296,291,286,280,282,283,286,289,293,292,288,282,280,278,276,275,276,278,280,288,294,295,289,280,287,291,291,281,278,277,278,279,282,287,292,279,279,280,280,282,282,290,292,297,288,282,279,279,281,283,284,286,288,290,292,298,304,305,306,306,305,304,304,303,302,301,301,302,304,312,319,482,479,475,472,471,471,468,461,462,460,458,458,461,460,465,462,468,473,477,474,462,468,474,474,476,484,483,489,486,480,474,470,466,467,468,471,475,478,487,488,488,488,488,484,479,476,463,451,442,436,434,435,437,441,454,466,454,446,441,441,442,447,453,450,455,452,461,457,457,462,472,487,501,500,494,488,484,479,474,474,475,486,481,474,464,458,455,451,440,436,430,418,413,412,410,410,410,409,407,405,403,402,401,400,401,401,401,402,402,401,400,399,400,403,409,416,415,411,412,416,423,420,411,404,405,407,410,404,404,407,404,402,406,410,413,418,427,432,440,447,453,447,437,433,432,429,419,413,412,405,401,401,404,406,407,408,412,416,424,430,439,449,452,444,438,434,435,428,418,404,392,385,376,367,362,357,351,349,343,338,330,327,328,322,318,317,315,313,310,308,304,305,306,306,305,305,304,304,304,304,305,302,298,296,294,293,292,291,289,290,290,292,289,287,286,284,283,282,280,280,280,280,280,281,280,279,279,279,279,278,277,277,276,276,276,277,277,278,280,282,283,284,287,290,293,299,306,320,334,350,365,359,356,343,334,326,322,320,318,313,310,307,304,301,302,301,304,308,307,303,304,294,290,288,289,290,290,289,289,286,288,289,290,292,297,303,298,286,282,279,279,291,299,306,301,294,288,283,280,281,282,284,287,287,286,280,279,276,274,274,275,277,279,285,292,295,289,280,280,283,286,282,278,277,277,278,280,282,290,279,283,280,280,282,286,292,293,286,282,279,278,279,281,283,285,287,289,291,293,299,303,305,305,305,305,304,303,302,301,300,299,300,304,311,319,485,483,480,475,468,464,462,461,461,461,461,464,466,466,469,471,473,477,477,473,467,472,478,482,484,489,485,485,481,477,474,474,474,472,474,472,472,473,476,477,476,477,476,476,475,476,464,450,440,435,434,436,440,445,452,465,457,448,441,446,451,447,449,454,458,462,468,471,472,473,476,484,490,489,486,480,474,474,471,473,477,472,466,460,453,442,440,434,428,424,420,414,413,410,410,410,410,410,406,405,402,402,402,402,402,402,402,402,402,401,400,400,400,401,406,413,415,413,416,423,430,427,413,408,407,408,414,409,414,417,415,407,408,412,412,416,422,429,436,445,453,444,434,426,424,424,418,414,408,403,402,402,405,413,410,411,413,420,429,436,445,452,443,436,428,430,426,427,419,408,398,388,380,372,365,352,349,342,340,333,328,324,318,316,315,315,315,313,311,308,305,304,303,303,303,303,303,303,302,301,300,297,295,293,292,291,291,290,288,287,288,291,288,287,286,284,283,282,280,280,279,279,279,279,279,279,279,279,279,280,278,277,276,276,276,275,276,277,280,283,283,284,286,290,291,296,304,319,334,348,353,348,344,337,328,321,315,314,311,308,304,302,299,296,297,300,304,307,305,305,302,294,291,287,286,286,284,282,280,282,284,286,288,290,292,297,295,287,282,279,280,286,294,302,303,295,291,284,280,279,280,281,283,281,283,280,276,274,272,273,274,276,279,282,288,293,291,283,278,280,281,281,277,275,275,277,279,281,279,283,281,280,280,282,284,289,283,282,280,278,279,280,281,284,286,288,290,292,295,300,304,305,305,305,304,304,303,303,301,299,297,294,301,308,313,493,492,487,479,469,462,461,461,461,464,464,463,470,476,475,477,474,477,478,475,473,481,484,490,485,491,488,488,486,486,480,479,475,479,477,473,469,466,466,463,463,462,464,464,464,470,461,443,438,435,436,436,445,454,456,464,456,449,447,445,451,450,450,456,465,475,482,489,486,481,480,484,485,483,479,476,474,470,466,464,460,458,451,449,446,439,430,426,425,424,421,419,416,414,411,411,412,414,413,407,406,406,406,405,402,403,403,403,401,401,400,400,400,400,405,412,415,418,423,430,437,432,417,412,410,411,412,413,415,420,418,412,413,414,412,412,416,424,431,438,452,450,440,431,424,417,413,412,407,405,405,406,409,414,412,414,417,423,431,439,444,441,434,424,418,414,415,419,420,412,400,390,380,373,367,356,347,340,339,333,329,327,320,319,318,317,316,315,314,309,307,304,303,303,303,303,303,303,302,300,296,294,292,292,292,292,291,290,288,287,286,287,287,287,285,284,282,281,280,280,279,281,280,279,279,279,279,279,279,280,279,277,277,276,275,275,275,277,281,283,283,283,283,287,290,294,305,319,334,343,346,334,334,328,322,316,311,308,305,303,299,296,293,292,295,302,306,306,303,297,296,293,290,285,282,282,282,280,280,280,283,284,285,288,290,293,292,288,283,279,280,282,289,296,297,298,294,285,280,279,279,279,279,279,280,279,276,272,269,271,272,275,278,279,282,287,288,284,278,278,279,277,274,272,273,275,278,280,280,279,279,279,280,281,283,281,281,279,278,279,279,281,283,284,287,289,291,293,297,301,305,306,306,305,303,303,304,304,302,296,294,292,295,302,306,495,496,489,478,468,462,461,462,462,464,466,466,472,477,479,480,478,479,481,475,479,488,492,496,491,498,498,495,496,493,488,488,484,486,478,474,468,463,461,463,460,458,454,451,452,458,454,442,437,435,437,440,444,454,464,464,457,452,448,447,451,457,457,462,475,486,495,502,497,487,485,488,489,488,486,487,477,474,469,467,465,465,460,457,452,442,437,432,430,428,426,425,422,419,415,416,421,420,413,411,410,410,410,411,414,408,403,402,401,401,400,400,400,402,406,411,416,423,428,433,439,437,424,412,412,412,413,418,424,425,423,420,418,418,414,414,413,421,430,439,447,448,438,430,422,415,412,411,410,409,410,412,412,414,414,420,424,427,433,436,437,432,426,416,409,402,404,412,418,413,400,389,379,372,364,354,351,348,343,339,333,330,326,322,321,319,317,317,312,310,310,307,304,303,303,303,303,303,304,300,296,294,293,292,292,292,293,292,291,288,287,286,286,286,285,282,281,279,279,279,283,282,281,280,280,281,281,280,280,279,279,278,278,277,276,275,274,276,279,282,283,282,281,284,289,293,304,319,330,337,337,325,323,318,314,310,305,302,298,295,293,292,291,291,293,300,304,302,295,291,291,290,284,282,280,280,281,279,280,281,282,282,282,282,284,289,290,286,281,279,279,281,284,293,295,294,292,284,280,278,278,278,278,278,279,276,273,269,268,270,271,273,275,277,279,281,282,282,278,276,273,272,271,269,271,274,276,278,278,277,276,277,278,280,280,280,279,278,278,279,280,281,283,285,287,290,292,295,298,302,305,305,304,303,305,304,304,304,304,300,296,292,291,295,299,488,490,485,473,465,461,462,463,465,464,469,470,473,475,479,483,485,486,487,487,486,498,506,509,502,505,506,500,504,503,498,490,490,492,485,480,475,472,469,468,463,463,461,451,447,447,440,440,436,436,436,438,442,453,465,461,450,449,450,450,459,465,462,472,483,494,505,514,508,499,496,491,492,493,491,488,478,477,479,478,486,476,464,464,454,446,439,437,436,434,432,428,429,425,423,425,430,423,413,414,414,413,412,415,416,412,406,403,402,401,401,401,402,404,408,413,419,428,434,433,433,430,424,413,412,412,415,420,429,432,430,429,428,424,418,415,412,418,426,435,439,440,435,428,417,413,413,412,411,413,414,417,416,420,424,429,432,430,438,432,430,426,418,411,401,393,395,406,416,417,408,399,389,378,368,363,360,355,349,344,340,332,327,323,318,316,314,313,312,310,311,315,306,304,305,305,305,304,304,301,295,294,293,293,293,293,294,293,291,289,288,287,286,285,285,284,281,281,281,284,285,284,282,281,281,281,281,280,280,280,278,280,280,280,278,276,275,275,277,280,279,280,280,281,285,291,302,315,323,325,323,320,316,310,306,303,299,296,292,292,291,290,290,290,292,295,296,294,291,289,288,286,282,280,280,279,280,280,280,282,283,281,280,280,280,285,286,284,280,279,279,280,280,287,290,289,288,282,280,278,277,277,277,277,276,275,272,270,269,268,270,271,272,274,276,278,280,280,277,274,270,268,269,268,270,272,275,277,276,274,274,274,276,278,279,278,278,278,278,279,280,281,283,285,288,290,293,295,300,304,306,304,303,303,305,304,304,305,303,300,296,293,291,292,294,479,480,476,467,462,462,463,464,467,467,470,474,475,476,483,487,490,491,491,498,499,507,519,524,513,511,513,509,514,513,508,498,499,501,496,487,485,481,479,474,471,468,464,450,448,444,440,438,437,437,437,439,443,449,459,461,457,456,458,454,460,462,472,481,490,500,505,505,502,495,489,490,494,497,499,494,491,487,492,491,487,474,476,470,460,451,448,445,444,441,437,436,439,431,430,433,432,424,418,420,422,418,414,420,416,412,408,405,403,402,401,402,404,406,411,415,421,430,439,433,427,423,416,412,410,414,423,428,432,439,438,439,432,426,421,417,413,414,422,428,429,430,429,427,420,416,417,414,413,416,418,421,424,429,434,439,438,431,432,428,422,418,412,403,396,388,388,400,410,418,416,407,398,386,377,372,368,361,355,351,345,337,330,326,318,316,315,312,310,307,309,311,312,306,308,306,306,305,303,299,295,292,293,293,293,293,293,292,291,290,288,287,287,286,285,285,284,285,286,287,286,285,284,283,282,282,282,282,282,281,280,282,284,284,280,277,275,275,275,276,277,278,278,280,283,288,296,305,310,311,311,308,306,302,298,295,293,292,291,290,290,289,289,289,290,292,292,291,289,288,285,283,281,280,279,278,279,279,280,282,281,279,279,279,280,282,283,280,280,279,279,278,279,281,283,283,284,284,280,279,277,276,276,276,275,274,273,270,270,268,268,268,269,271,273,276,277,278,280,280,274,267,268,268,269,271,275,276,275,272,271,272,273,275,277,277,277,277,278,278,280,280,283,286,288,290,292,296,301,305,306,303,301,302,305,304,304,305,303,300,297,294,291,290,291,472,472,469,464,462,463,465,465,470,472,473,475,480,483,487,495,500,500,499,504,512,520,533,539,526,521,524,514,523,525,517,508,510,513,509,500,501,494,487,478,478,470,462,455,451,446,441,439,438,438,438,439,441,446,450,452,452,455,459,451,458,464,477,488,492,493,492,491,489,485,483,488,493,500,504,503,503,503,503,498,485,481,482,474,467,460,458,454,454,448,444,439,442,441,441,442,442,429,424,429,432,424,424,425,418,413,410,407,405,404,403,402,405,408,412,416,421,428,434,432,426,418,413,410,412,416,427,435,437,442,441,440,432,426,424,420,417,417,419,423,423,426,425,423,422,420,418,415,413,415,421,426,431,438,443,444,434,425,421,418,414,411,406,400,390,380,382,392,402,412,413,412,403,395,387,379,373,367,363,357,349,342,332,325,318,317,316,312,306,305,305,308,308,308,308,306,304,304,304,300,295,294,293,293,294,294,293,292,292,290,291,289,288,287,286,286,287,286,287,287,286,285,284,284,283,283,283,283,284,284,285,286,287,287,283,279,277,275,274,274,275,276,277,278,280,284,289,293,296,298,298,296,294,292,291,291,290,290,289,288,288,287,287,288,289,289,289,288,286,284,283,283,282,280,279,278,278,278,280,282,280,279,277,279,280,281,281,280,280,279,279,277,276,275,277,278,279,282,280,279,276,275,275,275,275,275,274,274,273,269,267,267,267,268,271,272,274,276,278,280,274,267,268,268,271,274,278,277,275,268,268,270,271,273,275,275,276,276,277,278,279,281,284,286,289,291,294,297,302,304,307,303,298,301,304,305,303,302,302,297,296,294,292,289,289,477,468,465,464,464,466,468,471,471,474,476,475,484,491,496,504,512,510,508,508,516,532,548,552,538,535,537,524,530,537,526,515,525,527,523,515,517,508,496,488,477,474,463,458,451,448,443,440,439,439,441,442,442,442,445,447,449,451,450,452,460,465,478,485,482,480,476,477,476,484,489,491,497,504,512,514,516,519,519,506,491,491,491,483,475,473,470,466,461,453,452,449,447,450,451,452,450,437,437,442,440,427,433,430,427,412,409,409,407,406,405,404,406,409,411,412,414,423,429,430,429,422,414,411,413,416,427,439,441,449,448,436,429,426,425,426,423,421,424,425,427,431,431,428,427,425,423,416,415,419,424,430,438,448,453,439,427,417,412,408,405,404,404,399,393,381,376,384,394,401,402,407,408,403,394,387,379,372,366,357,349,340,332,328,325,322,318,313,309,304,305,304,305,306,305,305,304,303,303,300,297,295,294,294,294,295,294,293,292,292,292,290,288,286,286,286,286,287,288,287,285,285,285,284,284,284,284,285,286,286,287,288,287,287,287,283,280,277,275,274,274,274,275,276,278,280,283,285,287,289,289,289,289,288,288,288,287,287,286,286,286,285,285,285,286,286,286,283,283,280,280,281,282,280,279,278,276,276,278,278,276,276,277,279,280,280,280,280,280,281,279,277,274,272,272,273,275,278,276,275,274,274,274,274,275,275,276,279,280,275,267,267,266,267,269,270,272,274,273,273,270,267,268,268,271,274,278,281,274,267,268,269,270,272,273,274,275,276,277,279,280,283,285,287,289,292,295,300,302,305,305,302,296,296,298,298,298,297,295,294,295,294,292,290,288,473,468,466,467,467,469,472,474,474,475,478,482,486,497,508,512,522,522,514,515,520,534,551,563,552,552,552,539,540,550,540,524,536,545,535,532,530,518,504,490,488,480,470,462,453,448,445,442,439,439,445,448,448,446,446,446,448,451,452,453,460,465,471,466,468,473,476,478,482,486,495,503,504,508,512,521,532,535,535,516,502,499,503,498,487,489,483,478,468,463,461,455,451,459,467,467,459,449,452,452,441,435,440,428,421,413,412,410,409,408,407,408,411,412,413,411,414,419,424,427,427,421,414,413,418,417,425,436,446,454,442,436,434,433,436,435,424,426,429,431,436,440,439,435,434,433,430,427,425,424,426,435,445,452,448,439,429,421,412,404,399,396,399,394,389,380,374,377,385,386,394,402,404,405,402,393,383,376,367,356,350,342,339,335,333,328,324,320,316,309,304,303,303,303,303,303,303,302,302,300,298,297,296,295,295,295,294,293,293,293,292,292,290,288,287,286,287,287,287,286,286,285,286,285,285,285,286,286,286,287,287,287,288,288,286,287,283,280,275,274,274,274,274,275,276,278,279,280,281,282,283,284,284,284,284,284,284,284,283,283,284,284,283,283,283,284,283,282,280,279,279,280,283,280,280,277,276,275,274,274,274,274,276,278,280,280,280,280,279,279,279,278,276,273,271,271,271,272,271,272,272,273,273,274,274,275,277,279,280,281,268,267,265,266,268,269,272,272,272,268,269,267,268,269,270,272,276,277,273,267,268,270,271,272,273,274,275,277,278,280,282,284,285,287,289,291,294,298,302,305,304,298,292,292,294,294,294,292,291,291,294,294,293,288,286,472,470,470,471,472,473,474,477,478,480,483,486,494,502,514,526,537,538,528,526,530,534,551,569,572,570,567,555,553,561,554,545,554,560,551,550,539,520,510,504,496,490,478,466,458,450,446,441,439,440,453,462,460,457,451,449,448,450,451,452,455,460,464,468,466,474,479,486,493,498,498,507,515,514,525,525,540,549,547,533,518,511,518,510,506,504,496,489,477,482,471,462,462,471,478,484,474,462,460,453,448,441,441,431,422,414,414,413,412,410,409,411,414,415,415,415,413,416,420,422,420,417,414,413,424,428,432,436,447,454,447,439,439,443,445,437,428,429,432,439,437,439,434,431,431,439,429,428,431,427,433,441,452,448,437,431,428,421,415,407,399,390,388,390,389,382,374,367,372,376,389,395,398,398,396,392,385,377,370,357,355,352,347,342,341,336,333,329,321,312,307,306,305,304,303,303,303,303,302,301,300,298,297,296,296,295,295,294,294,294,293,293,291,289,288,287,287,288,287,287,286,286,286,286,285,285,286,286,287,287,287,287,286,286,286,284,282,280,277,276,275,274,274,274,275,275,276,277,278,278,279,280,280,280,281,281,282,281,281,281,282,282,282,281,281,282,282,281,279,278,278,280,282,282,278,278,275,274,274,274,274,274,274,275,278,279,280,280,279,279,280,279,280,277,275,273,271,269,269,269,269,271,272,273,274,275,276,278,279,281,276,269,265,265,266,269,271,271,270,270,268,267,268,269,270,270,269,273,267,268,269,269,271,271,272,274,276,278,279,280,281,284,286,287,289,290,293,295,300,304,299,294,290,289,290,290,289,289,288,289,293,293,290,285,282,473,473,474,475,475,476,479,482,483,485,487,488,497,507,520,536,552,554,542,536,550,543,550,572,590,592,583,568,567,573,569,562,574,581,575,561,541,524,526,516,502,489,477,464,454,449,444,441,439,442,450,457,462,465,461,449,449,450,451,453,456,459,465,476,478,473,480,487,500,508,509,513,527,530,537,538,545,557,561,545,533,523,536,527,525,520,509,504,500,496,479,470,472,485,488,490,481,475,472,465,456,451,450,437,429,422,422,417,413,411,411,412,414,417,420,424,419,415,419,419,418,414,412,414,423,431,440,445,450,460,454,449,451,455,451,439,435,432,435,434,427,427,422,422,428,422,415,427,430,435,440,448,454,446,434,423,418,415,413,408,402,392,388,379,379,379,373,365,363,372,382,388,390,390,386,384,381,377,370,363,360,359,355,354,350,346,342,332,322,315,313,312,309,306,306,305,304,304,303,302,301,299,298,297,296,295,295,295,294,295,294,293,292,290,289,288,288,288,288,287,286,287,287,286,285,285,285,286,286,286,286,285,285,285,282,280,280,278,277,276,276,275,274,274,274,274,274,275,275,276,276,277,277,277,278,279,278,278,279,279,280,280,280,279,280,280,281,281,278,277,277,280,281,280,276,274,274,274,274,274,274,274,274,276,277,278,280,281,281,281,281,282,282,282,282,278,274,271,269,268,269,270,271,272,272,273,274,276,278,280,281,269,267,265,265,266,270,270,269,271,268,268,267,268,268,268,268,269,267,267,268,268,269,271,272,275,277,279,281,282,283,284,286,287,289,291,292,294,295,295,296,292,288,285,284,284,284,284,284,286,288,287,284,281,279,475,477,478,480,481,484,489,486,489,490,490,494,498,506,519,538,556,568,558,553,565,560,554,572,598,609,596,585,583,587,586,581,592,597,586,566,547,546,537,524,501,491,477,464,450,446,445,442,439,440,444,447,451,457,454,451,449,450,451,454,458,464,464,477,488,478,484,488,496,507,514,521,534,549,555,557,558,561,572,563,549,539,550,548,545,535,526,519,515,502,489,480,490,497,496,501,501,499,491,478,463,462,456,447,432,430,430,425,415,411,412,412,414,418,423,428,426,423,420,419,416,414,419,428,429,434,441,452,461,466,466,462,464,460,452,440,434,430,430,428,417,413,412,408,412,404,404,411,421,434,439,442,449,443,432,418,406,404,405,405,404,399,390,381,374,368,365,364,361,366,375,377,381,381,376,374,374,373,367,358,354,366,366,366,362,354,343,333,329,322,319,319,316,312,310,308,308,307,304,303,301,300,299,297,296,295,295,294,294,294,293,292,292,291,290,289,289,288,288,288,288,287,287,286,285,285,285,285,284,284,283,283,284,279,279,279,279,279,278,277,276,276,275,275,274,274,274,274,274,274,274,275,275,275,275,276,276,276,276,276,277,278,278,278,278,279,279,278,277,276,275,275,275,275,274,274,274,274,274,274,274,274,275,275,276,278,279,281,281,281,282,282,282,283,285,283,280,274,270,268,269,271,271,270,270,271,272,274,276,278,279,277,268,265,265,265,266,268,268,269,269,268,267,268,268,268,268,268,267,269,269,268,268,270,271,276,278,281,282,283,284,284,286,287,288,290,291,292,292,292,292,287,283,280,280,280,280,280,280,283,283,282,280,278,278,477,481,482,485,489,485,491,491,496,500,498,499,502,507,517,535,555,572,574,572,583,583,572,583,601,617,611,604,602,606,606,601,610,609,593,574,566,557,540,522,517,501,480,465,450,446,444,442,440,441,444,448,449,450,449,449,451,450,452,458,465,474,472,479,488,483,490,494,503,513,522,533,536,547,564,574,575,571,584,577,561,557,567,570,560,550,546,538,527,511,494,495,504,511,508,511,518,514,504,488,475,470,463,452,438,440,438,430,418,413,412,413,414,418,424,427,427,425,422,419,417,414,419,427,430,435,441,452,465,469,468,462,452,447,441,434,427,424,418,414,408,401,405,398,395,392,402,415,426,431,428,432,438,439,429,417,404,397,398,397,399,402,396,389,382,378,370,364,360,361,366,367,373,374,367,364,366,366,365,355,350,360,370,376,369,357,349,343,336,328,328,324,319,317,318,318,314,308,304,303,302,300,299,298,296,294,294,294,294,294,293,292,292,291,290,290,289,289,289,288,288,288,287,286,286,285,284,284,283,282,281,280,279,282,282,280,280,279,279,278,278,277,276,276,275,274,274,274,274,274,273,273,274,274,274,274,274,274,274,275,275,275,276,276,276,276,276,276,275,274,274,273,273,272,274,274,274,274,274,274,274,274,274,274,275,275,276,278,280,281,281,282,282,283,283,283,282,278,273,268,269,269,270,269,269,270,270,271,273,274,276,273,271,266,265,265,265,266,267,268,268,268,267,267,267,267,267,267,268,269,268,268,268,268,269,275,278,282,282,283,284,284,285,286,286,287,288,289,289,287,286,283,279,277,276,276,276,276,277,279,279,278,278,276,276,488,486,486,486,490,489,494,499,501,506,508,506,510,512,513,527,549,564,577,591,597,602,598,600,612,623,624,619,614,622,624,623,629,618,605,593,578,560,549,540,528,508,486,468,452,450,446,444,443,444,446,449,451,452,451,452,452,453,458,463,476,478,475,480,485,490,494,498,508,517,530,549,560,560,569,584,587,583,591,590,575,572,582,591,579,570,560,552,540,519,500,505,519,527,524,527,531,524,512,496,486,478,469,459,452,448,439,429,421,416,412,413,414,417,420,424,426,426,422,421,418,418,418,425,428,433,439,451,458,464,464,448,437,430,425,421,412,409,409,404,402,395,396,392,388,396,407,421,426,421,422,422,426,428,428,419,408,398,390,390,390,395,398,394,392,387,379,370,363,355,356,358,364,366,359,353,357,365,360,351,348,356,368,378,370,363,356,351,344,339,334,328,326,325,321,318,314,307,304,302,301,300,299,298,296,295,293,293,293,292,292,292,292,291,290,290,290,290,289,289,288,288,287,287,286,285,284,282,281,281,280,279,279,281,281,280,281,280,280,279,279,278,277,277,276,275,275,274,274,274,274,274,273,273,273,273,273,273,274,274,274,274,274,274,274,274,274,274,274,273,273,272,272,273,274,274,274,274,274,273,273,273,273,273,273,274,274,275,278,279,280,280,281,282,283,283,283,282,278,271,268,268,268,268,268,269,269,269,270,271,272,271,270,269,266,265,265,265,266,267,267,267,267,267,267,267,267,268,268,268,267,267,267,268,269,273,280,281,282,282,283,284,284,284,284,284,286,286,285,283,281,279,276,273,271,271,271,272,273,274,274,273,272,272,273,499,493,491,492,496,500,499,502,508,513,517,515,514,521,524,522,537,556,574,589,606,615,614,611,617,623,623,622,619,621,626,632,635,628,618,603,588,574,558,543,529,511,491,472,459,451,447,445,445,446,448,451,453,452,451,454,456,459,462,471,478,482,484,486,494,505,511,503,511,525,539,556,569,578,580,590,598,596,596,600,593,590,598,606,602,589,575,560,544,525,512,520,532,540,540,539,537,526,513,506,498,487,477,467,458,452,437,426,423,418,414,412,414,417,420,422,425,426,424,423,422,422,423,422,426,432,438,444,449,456,459,446,432,420,413,414,405,394,401,393,393,388,388,388,392,400,411,426,423,419,416,414,415,418,420,418,410,401,392,387,387,387,391,392,394,391,383,375,367,357,349,350,352,354,351,347,351,356,354,345,341,354,363,367,363,356,354,353,345,344,340,333,331,328,322,317,312,305,304,302,302,300,299,298,296,294,293,292,292,292,292,292,292,291,291,291,290,290,290,290,289,288,288,287,286,285,284,282,281,280,279,280,280,280,282,283,283,283,282,280,280,279,278,278,277,276,276,275,275,274,274,274,274,274,274,274,274,274,273,273,273,273,273,273,273,273,273,273,273,273,273,272,272,273,273,274,274,274,273,273,273,273,272,272,272,271,269,269,275,276,278,278,280,281,282,282,283,282,278,270,268,268,268,268,267,267,268,268,268,269,270,269,269,268,268,265,265,266,266,266,266,266,266,266,266,267,267,268,268,267,267,267,267,267,268,271,279,281,281,282,283,283,283,283,281,282,283,283,283,280,278,275,272,269,268,268,268,268,268,270,270,269,269,268,270,510,500,496,500,505,511,509,506,509,517,523,527,527,531,538,533,534,551,565,577,595,612,620,616,618,621,624,622,619,620,626,629,628,626,620,608,593,576,561,542,526,504,484,472,462,456,446,446,447,449,447,450,453,456,456,460,460,462,467,476,481,486,490,497,507,522,528,525,520,529,540,560,575,592,597,598,605,608,605,610,608,608,610,619,622,604,584,566,555,544,525,531,542,554,559,543,531,523,517,513,506,496,484,471,460,450,438,430,425,420,416,413,415,419,421,424,424,426,424,425,426,426,425,425,426,430,434,438,444,452,453,444,432,416,406,401,395,387,388,385,385,380,382,389,397,408,419,425,422,418,414,406,405,409,412,414,410,403,394,387,380,380,383,386,391,392,387,380,378,368,354,346,344,345,344,340,344,347,345,340,341,349,354,356,353,347,341,341,342,338,333,327,324,321,318,314,310,306,304,303,302,301,300,298,297,295,294,293,293,293,293,292,292,291,291,291,291,291,291,290,290,289,288,287,286,285,284,283,281,282,282,281,281,283,285,286,286,286,284,282,281,280,279,279,278,277,276,276,275,275,275,274,274,274,274,274,274,274,274,274,274,273,273,273,273,273,273,273,273,273,272,272,272,273,272,274,274,273,273,273,272,272,271,271,271,270,269,268,268,274,275,276,276,277,279,279,278,274,269,268,268,267,267,267,267,267,267,267,267,268,268,268,268,268,267,265,264,265,265,265,266,266,266,266,266,267,268,268,268,267,266,266,266,266,267,268,270,274,277,279,280,281,282,280,279,281,281,281,280,278,275,270,268,268,267,267,267,267,267,268,268,267,267,267,271,506,505,501,504,514,515,513,510,512,518,528,537,542,544,552,548,536,546,557,568,588,610,620,619,618,621,622,619,615,616,624,625,620,622,617,607,594,576,558,541,520,499,489,486,475,462,450,448,447,451,450,455,461,460,461,468,470,463,467,473,478,486,492,499,514,536,540,538,535,535,540,562,581,599,609,609,614,613,611,616,615,616,621,626,622,609,588,578,568,556,539,540,552,562,560,538,524,520,518,514,509,500,485,474,463,455,446,436,429,423,416,414,417,422,424,426,426,426,426,427,430,430,428,426,428,431,434,437,441,446,449,439,426,414,406,399,392,388,378,376,378,377,386,397,411,421,427,426,420,417,411,402,399,402,404,406,406,403,396,388,380,376,376,380,384,386,387,384,382,375,362,350,340,339,339,337,339,341,340,335,334,341,344,346,344,338,330,333,334,331,326,320,315,313,312,309,306,304,303,303,303,303,301,299,297,295,295,294,293,293,292,292,292,292,292,292,292,292,292,292,291,289,287,287,286,286,285,285,285,285,286,286,286,286,288,288,288,288,286,284,283,281,280,280,279,278,278,277,276,276,275,275,275,275,275,275,275,275,275,275,275,274,274,274,273,273,273,273,273,273,272,272,272,272,273,272,274,273,273,272,272,271,271,271,270,270,269,268,268,268,268,270,271,273,273,272,268,267,266,268,267,267,266,266,266,266,266,266,266,266,264,263,262,263,263,264,264,264,264,264,265,265,265,265,265,267,268,268,267,266,265,265,265,265,266,267,267,268,271,273,275,276,277,276,276,277,276,276,275,274,274,272,267,265,264,266,265,266,266,266,267,267,266,269,276,500,510,509,509,511,519,517,512,522,523,527,545,558,561,568,565,552,555,575,583,599,615,622,622,620,617,614,613,612,607,615,617,611,614,611,603,593,580,563,546,530,516,508,501,485,465,454,449,450,458,461,463,465,463,466,478,478,468,470,477,485,488,499,513,522,540,553,563,561,555,551,565,585,605,614,622,622,615,616,617,618,618,624,626,625,615,600,590,576,564,553,556,561,574,567,544,526,521,516,514,508,501,490,480,472,461,451,439,431,425,418,417,422,427,428,431,433,429,431,434,436,439,436,430,431,434,435,434,437,442,447,439,431,424,416,408,401,396,386,377,373,379,391,404,418,427,428,430,427,417,409,400,393,394,397,399,402,403,396,388,379,374,373,376,377,379,379,381,382,379,370,360,346,340,333,332,334,334,334,331,329,331,336,337,335,331,325,326,326,323,319,316,306,304,304,305,304,303,303,304,305,303,301,299,297,296,294,293,292,292,292,292,293,292,293,293,292,292,292,292,292,291,288,287,286,286,286,287,288,288,290,292,291,291,290,290,289,288,287,285,284,283,282,281,280,279,279,278,277,276,276,276,275,275,275,275,275,275,275,275,275,275,275,275,274,274,273,273,273,273,272,272,273,273,272,272,273,273,272,272,272,271,271,271,270,270,269,269,268,268,268,268,268,268,269,268,266,265,265,265,265,266,266,266,266,266,266,266,266,265,264,263,262,262,262,262,263,263,263,264,264,264,264,264,264,265,266,266,265,265,264,264,264,264,264,265,266,266,267,268,268,268,270,271,272,272,271,270,268,270,272,272,269,265,262,263,264,265,266,266,267,268,269,274,282,512,515,512,514,515,522,527,523,532,535,534,539,562,574,583,585,573,567,586,598,613,622,620,622,618,613,609,612,606,599,604,608,604,605,603,595,588,580,567,550,533,519,509,496,480,466,453,448,451,454,461,466,464,470,474,480,475,470,475,484,495,504,511,527,538,544,561,576,580,570,566,575,592,610,612,618,619,619,619,619,619,620,621,623,619,615,606,594,585,575,566,568,570,574,564,547,530,521,514,510,504,500,489,482,477,465,452,440,431,425,422,422,425,429,432,434,435,436,438,442,446,449,439,434,435,437,437,437,437,439,444,440,432,424,417,412,406,403,390,377,372,379,392,406,418,416,415,421,424,415,408,399,390,389,390,390,396,399,394,386,378,373,367,367,369,371,373,375,378,380,377,366,356,344,333,328,329,330,330,329,328,326,328,329,328,324,319,316,317,317,316,308,305,305,305,305,305,305,305,303,303,302,301,299,298,295,294,292,293,293,293,294,294,294,294,294,293,293,294,294,293,291,290,288,287,286,286,288,290,292,295,298,298,296,292,291,290,288,286,286,286,284,282,281,280,280,279,278,278,277,277,276,276,276,276,276,276,276,275,275,275,275,275,274,274,274,274,274,274,273,273,273,273,273,274,274,272,272,272,272,271,271,271,271,270,270,269,269,269,269,268,268,268,268,267,266,265,264,264,264,264,264,264,264,264,264,266,266,265,265,264,263,262,262,262,262,262,262,263,263,263,264,264,264,263,263,263,264,264,263,264,263,263,263,263,263,264,264,265,266,266,267,267,267,268,269,267,267,267,270,271,270,268,264,262,263,264,264,265,267,269,271,273,277,283,512,520,520,520,524,528,536,540,545,553,550,546,554,576,592,598,592,588,594,609,618,620,617,620,620,614,610,604,599,591,592,594,594,595,595,584,573,566,558,549,535,521,511,499,480,465,453,450,450,452,455,460,460,464,472,473,467,474,481,492,504,518,530,537,546,562,571,585,596,588,580,592,604,611,610,612,617,619,622,618,616,621,614,614,610,605,601,597,588,582,576,577,582,582,564,547,532,522,515,509,501,492,484,476,474,464,451,441,431,424,424,424,427,431,435,437,438,441,446,450,455,456,445,438,439,442,446,445,443,445,444,437,427,416,410,404,402,395,384,377,374,378,392,404,408,405,404,411,416,413,407,400,390,384,384,385,391,393,391,384,377,372,366,363,364,365,366,366,370,374,375,369,358,346,337,331,328,327,327,326,327,326,325,322,320,316,316,315,316,316,314,312,309,307,306,308,307,306,305,304,303,302,301,300,299,297,294,294,294,295,295,295,295,296,296,295,294,295,295,294,293,292,291,289,288,288,289,290,291,295,301,307,306,302,296,292,289,287,285,286,287,286,284,282,282,280,280,279,279,278,277,277,276,276,276,276,276,276,276,275,275,274,274,275,275,274,274,274,274,273,273,273,273,273,274,274,272,272,272,272,271,271,271,271,270,270,269,269,269,269,269,268,268,268,267,266,265,264,264,264,264,264,264,264,264,264,264,264,265,264,264,263,263,262,262,262,262,262,262,263,263,263,263,263,263,262,262,263,263,263,262,262,262,262,262,262,262,262,263,264,264,265,265,266,266,266,264,263,266,268,268,268,265,262,262,262,263,263,264,267,269,269,274,279,288,524,526,523,523,524,532,543,556,563,568,568,561,557,574,594,610,608,605,607,605,609,613,617,621,619,613,609,596,591,584,581,582,584,584,586,578,556,544,540,535,528,518,507,494,475,461,453,450,450,452,453,455,459,467,462,460,468,480,490,502,517,530,540,548,561,572,585,600,608,605,600,607,616,610,606,612,617,621,621,614,612,608,605,606,600,595,594,588,584,582,579,583,584,576,562,549,534,523,517,510,503,495,486,479,476,468,459,448,434,425,425,428,430,434,438,441,445,450,455,459,464,465,455,447,449,451,458,458,454,452,448,436,421,402,400,392,387,383,378,375,372,374,383,393,391,390,395,401,408,409,405,398,390,381,377,378,385,388,386,380,376,372,367,363,363,363,361,362,364,366,367,368,360,349,341,337,334,331,328,327,326,326,324,319,317,317,318,317,317,316,314,314,312,310,308,309,309,307,307,305,303,302,301,300,300,299,298,297,297,296,297,297,297,297,297,296,296,296,296,295,293,292,291,291,290,291,292,291,292,299,306,317,315,303,295,291,288,286,285,285,286,286,284,282,282,281,280,280,279,279,278,277,277,277,277,277,276,276,276,275,275,274,274,274,274,274,274,274,274,274,273,273,273,273,274,274,273,272,272,271,271,271,271,270,270,270,269,269,269,269,269,268,268,268,267,265,264,264,264,264,264,264,264,264,264,264,264,264,264,264,264,264,263,263,262,262,262,262,262,263,263,263,263,263,262,262,262,262,262,262,262,262,262,262,261,261,261,262,262,263,262,263,264,264,263,263,263,262,263,263,263,262,262,262,261,262,262,262,262,263,265,267,273,280,287,524,530,527,533,541,540,540,563,575,586,587,585,576,587,597,611,617,616,617,613,611,618,621,620,617,612,605,595,584,573,571,570,571,570,575,575,560,539,516,512,510,501,486,478,466,455,452,450,452,453,453,454,456,462,458,461,469,482,492,505,520,531,542,554,568,582,600,610,614,614,617,622,621,608,603,612,615,620,617,607,599,596,595,595,597,598,592,582,579,579,576,574,570,562,556,549,536,525,517,510,503,495,486,479,475,467,458,447,434,425,426,430,434,437,441,445,451,458,465,470,473,473,462,458,460,465,468,466,463,456,446,432,413,404,400,390,386,379,377,377,370,368,374,380,382,382,388,394,401,404,401,394,388,381,376,376,380,382,380,376,373,371,368,365,363,359,357,358,359,361,362,363,358,353,347,343,343,339,334,332,330,328,328,324,321,320,321,320,319,318,317,317,316,314,312,312,311,310,309,306,304,302,301,301,301,301,301,300,300,300,299,299,298,298,298,298,298,297,297,295,294,292,292,292,292,294,295,294,296,302,306,315,309,300,292,290,287,285,282,282,282,283,282,282,282,281,281,280,280,279,278,278,277,277,277,277,277,276,276,275,275,275,274,274,274,274,274,274,274,274,274,274,273,273,274,273,273,272,272,271,271,271,271,270,270,269,269,269,268,268,268,268,268,268,266,264,264,264,264,264,264,264,264,264,264,264,264,264,264,264,264,264,263,263,263,262,262,262,262,263,263,263,263,262,262,262,262,262,262,262,261,261,261,261,261,261,261,261,261,262,262,262,262,263,262,262,262,261,262,262,261,261,261,261,261,261,262,261,261,262,262,267,271,277,281,529,539,536,541,555,556,548,551,574,589,600,600,595,604,612,620,627,627,626,623,622,617,617,614,612,610,604,598,589,576,563,559,554,555,562,567,562,547,528,502,494,488,478,470,460,455,452,453,455,456,455,454,455,456,457,460,466,477,485,500,506,512,536,554,568,576,585,596,603,605,616,623,616,604,597,607,614,622,614,608,599,584,589,597,606,600,588,575,572,575,573,569,564,557,551,544,533,522,514,509,502,493,485,478,471,461,454,443,434,430,426,430,436,441,443,447,453,463,473,480,487,481,474,468,471,475,472,465,460,453,432,424,419,416,406,394,390,392,390,381,371,364,366,371,375,376,383,390,395,400,397,390,385,379,376,375,373,378,375,373,372,373,374,370,364,359,353,353,354,356,357,357,356,355,347,349,352,346,340,337,334,330,329,328,326,324,323,323,322,321,320,319,318,316,315,314,314,315,310,307,305,302,301,302,304,303,303,303,303,302,302,300,300,300,299,299,299,299,298,298,296,295,295,295,298,300,301,302,302,305,310,308,303,296,290,288,285,282,280,280,279,279,280,282,281,281,281,280,280,279,278,278,278,278,278,278,277,276,276,275,275,274,275,275,275,275,275,274,274,274,274,274,273,273,273,273,273,272,272,271,271,271,271,270,270,269,268,268,268,268,268,268,269,266,265,264,264,264,264,264,264,264,264,264,264,264,264,264,264,264,264,264,264,263,263,263,263,262,262,262,263,263,262,262,262,262,262,262,262,261,261,261,260,260,260,260,260,261,261,261,261,261,262,262,262,261,261,260,261,261,260,260,260,260,260,260,261,260,261,261,262,263,266,272,276,545,553,551,546,563,572,563,558,565,583,606,614,614,620,630,633,637,636,632,625,619,612,611,610,607,602,596,592,586,576,562,546,541,542,550,558,552,542,528,512,492,479,475,466,458,454,452,455,462,460,455,455,457,457,459,459,462,468,479,488,493,512,531,547,556,558,564,575,587,594,610,616,614,604,594,600,611,614,607,599,590,584,590,597,604,602,589,576,571,570,567,564,559,554,547,535,524,512,502,503,500,488,482,478,470,462,450,440,434,429,426,430,436,442,446,450,456,462,473,487,486,487,482,476,478,477,470,462,456,452,442,436,427,416,407,402,406,405,395,385,374,364,362,364,368,373,378,383,390,394,393,390,384,378,375,369,366,368,369,366,371,376,380,378,368,361,355,353,352,353,353,353,353,351,349,348,353,345,342,341,337,333,331,330,328,327,326,325,324,323,322,320,318,316,315,315,317,315,311,307,303,302,302,303,304,304,306,307,306,304,302,302,301,301,301,301,300,300,300,300,301,300,301,302,304,306,306,305,304,307,306,302,296,291,288,285,282,280,278,278,277,278,278,279,279,281,280,280,280,279,279,278,278,278,278,278,278,277,275,274,274,275,275,275,275,275,275,274,274,274,274,274,274,274,273,273,273,272,272,272,271,271,270,270,269,269,268,268,268,268,268,268,267,266,264,264,264,264,265,265,264,264,264,264,264,264,264,264,264,264,264,264,264,263,263,263,263,263,262,262,262,262,262,262,262,262,262,262,261,261,260,260,260,260,260,260,260,260,260,260,261,261,261,261,260,260,260,260,260,260,260,259,259,259,259,260,260,260,260,260,260,261,263,269,272,556,567,567,558,561,582,584,574,584,584,605,624,632,640,648,648,645,636,627,620,612,607,602,601,598,598,589,581,574,568,556,538,525,529,539,548,539,529,516,505,494,477,470,464,457,453,452,458,459,459,459,460,462,463,463,465,461,466,474,480,494,507,520,531,538,544,550,562,570,580,595,610,610,602,594,596,605,603,597,590,584,588,587,587,593,597,586,572,569,573,563,559,554,549,538,526,515,502,493,497,493,485,478,476,469,460,450,440,433,427,428,432,438,442,447,451,460,468,476,487,481,482,476,475,476,476,466,458,451,447,444,442,437,424,419,421,420,412,399,387,376,364,356,354,362,368,373,376,382,388,390,390,384,379,373,369,364,365,364,363,368,374,379,378,370,362,355,352,351,352,352,351,350,347,346,346,345,345,342,341,338,335,332,331,330,330,329,328,327,326,324,320,317,315,314,314,316,313,306,306,303,302,303,305,308,311,314,316,312,306,304,302,302,302,302,302,302,301,301,302,303,304,306,307,308,307,307,307,306,307,305,299,293,288,284,281,280,278,277,277,277,277,277,277,278,278,278,278,278,277,277,277,278,279,279,279,278,278,276,275,274,274,274,275,275,275,275,275,274,274,274,274,274,274,274,273,273,273,272,272,272,271,270,270,269,269,269,268,268,268,268,268,267,265,264,264,264,265,265,265,266,266,264,264,264,264,264,264,264,264,264,264,264,263,263,263,262,262,262,262,262,262,262,262,262,262,262,262,261,260,260,260,260,260,260,260,260,260,260,260,260,260,260,260,260,260,259,259,259,259,259,259,258,258,258,259,259,259,259,260,261,261,262,265,268,562,577,584,577,571,583,593,594,601,613,616,635,647,647,646,644,639,632,626,619,613,604,595,591,588,590,587,575,559,554,549,530,512,517,525,532,526,515,506,495,486,477,467,460,455,452,453,454,458,459,462,463,468,476,478,479,474,471,472,476,488,497,506,515,518,526,535,552,561,573,588,600,598,593,584,592,597,594,588,582,582,587,577,579,584,586,578,565,564,569,560,552,549,540,527,516,509,500,485,487,486,478,472,469,465,456,445,438,435,428,430,435,439,443,447,452,462,472,477,486,472,469,465,463,464,468,462,452,447,443,438,439,442,440,440,440,432,412,405,394,382,372,359,352,357,362,367,372,377,378,384,385,381,378,373,368,362,360,359,360,365,368,375,375,367,362,356,366,353,351,352,350,346,342,341,341,341,340,340,338,337,334,332,331,330,331,333,330,329,329,327,323,318,316,313,312,310,306,307,303,302,302,304,309,315,320,324,322,316,309,304,303,303,304,304,303,303,302,302,303,304,310,315,315,310,308,307,308,308,303,299,295,290,286,281,279,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,278,280,280,280,279,279,278,276,275,274,275,276,276,276,275,275,274,274,274,274,274,274,274,274,274,273,273,273,272,271,270,269,269,269,269,268,268,268,268,267,266,265,265,265,265,265,265,266,266,266,266,266,264,264,264,264,264,264,264,264,264,263,263,262,262,262,262,262,262,262,262,262,262,262,262,262,261,260,260,259,259,259,259,259,259,259,259,259,259,259,259,259,259,259,259,258,258,258,258,258,258,257,258,258,258,257,258,258,258,259,260,261,262,560,577,594,598,592,594,610,617,625,631,634,642,644,641,640,636,634,628,621,612,608,602,598,583,576,577,578,573,554,536,534,526,506,504,512,516,514,502,494,488,478,470,462,457,454,453,452,456,459,461,465,472,479,486,493,496,488,486,484,479,483,487,490,498,503,515,526,536,555,569,581,587,584,582,583,586,588,585,579,574,578,579,568,570,572,575,569,559,562,562,557,550,537,525,516,508,501,492,481,476,478,473,468,462,460,452,438,434,438,434,434,437,441,444,448,454,462,472,479,476,469,459,456,455,456,459,456,449,442,438,434,435,438,441,443,443,435,423,411,402,388,376,363,354,352,357,364,366,368,373,377,379,377,376,371,367,361,354,353,357,361,365,370,368,364,362,360,363,359,356,353,351,348,343,342,341,340,339,337,336,334,333,330,330,330,330,331,335,331,332,330,325,320,316,313,310,308,305,304,302,302,303,306,313,320,327,330,326,318,312,306,304,305,307,307,306,305,304,303,304,307,317,318,316,310,307,307,308,306,299,292,290,286,283,279,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,278,280,280,280,280,280,279,279,277,279,279,277,276,276,274,274,275,274,274,274,274,274,274,274,274,274,274,274,273,272,270,269,268,268,268,268,268,268,267,267,266,266,266,265,265,266,266,266,266,266,266,266,266,265,265,265,265,265,264,264,264,263,263,262,262,262,262,262,262,262,262,262,262,262,262,261,261,260,259,259,259,259,259,259,259,259,259,259,258,258,258,258,258,258,258,258,258,258,257,257,257,256,257,257,257,256,256,256,257,258,258,259,260,557,569,588,608,613,614,626,636,637,640,636,634,635,634,633,630,624,623,618,609,600,598,594,579,561,559,560,561,553,538,521,517,506,497,499,503,500,493,482,478,470,462,458,455,454,454,456,459,461,465,475,480,490,501,512,510,507,510,502,493,484,487,494,502,504,510,526,540,550,563,572,572,573,578,580,582,585,585,572,570,575,575,560,555,561,563,559,554,559,557,552,550,541,526,509,500,492,484,476,465,465,465,467,462,451,441,438,442,441,438,436,439,441,440,448,455,464,472,472,464,457,448,447,448,449,453,454,450,440,436,429,429,432,436,438,436,428,414,406,401,392,378,366,354,350,353,357,359,362,366,370,372,371,371,369,368,362,353,350,353,359,367,368,369,365,356,358,361,367,365,356,352,351,353,354,349,343,340,338,332,331,330,330,329,329,329,331,334,340,338,333,326,321,316,314,311,307,305,303,302,302,304,309,317,326,334,338,328,319,315,306,307,311,313,314,313,311,306,304,304,309,318,316,310,305,303,304,305,301,294,289,285,282,280,278,277,277,277,277,277,277,277,277,277,277,277,277,277,277,277,278,279,281,283,284,284,284,282,280,280,280,280,280,279,277,275,275,275,276,275,275,274,274,274,274,274,275,274,274,274,273,272,270,269,268,268,268,268,267,267,267,267,267,266,266,266,266,266,266,266,266,266,266,266,266,266,265,265,265,265,264,264,263,263,262,262,262,262,262,262,262,262,262,261,261,261,261,261,261,260,259,259,259,259,259,259,258,258,258,258,258,258,258,257,257,257,257,257,257,257,257,256,256,256,256,257,256,256,256,256,256,256,257,258,259,574,575,587,612,627,631,631,643,638,638,630,628,628,630,631,625,619,614,612,608,597,590,587,575,553,541,543,546,547,538,518,509,505,494,488,492,491,486,475,467,466,460,456,455,454,456,458,461,466,476,483,489,499,511,523,526,523,525,517,506,495,501,513,526,529,533,542,553,562,562,569,573,575,577,576,576,578,578,575,570,571,563,549,544,546,548,548,547,554,552,548,543,535,517,502,492,483,474,469,460,451,456,463,462,454,451,452,451,444,440,434,437,434,436,443,453,464,468,464,452,443,438,438,440,443,447,452,454,444,434,426,423,420,422,422,425,420,411,395,387,388,381,376,365,355,350,350,350,355,357,365,365,364,366,366,367,362,353,349,353,360,368,369,368,367,356,351,359,365,361,354,353,352,357,364,355,347,341,337,332,329,329,328,328,326,327,329,333,339,342,334,327,321,316,314,310,307,304,303,302,303,306,313,321,330,340,338,327,317,313,313,314,317,322,323,318,314,307,305,305,307,313,311,304,302,300,298,297,294,290,285,280,279,278,277,277,277,277,277,277,277,277,277,277,277,277,277,278,278,279,280,282,285,288,288,289,289,288,285,283,282,282,280,280,280,274,276,276,276,276,275,274,274,274,275,275,275,275,275,274,273,272,270,270,268,268,268,267,267,267,267,267,267,267,266,266,266,266,266,267,267,266,266,266,266,266,266,265,265,265,264,264,263,263,262,262,262,262,262,262,262,262,261,261,261,261,261,261,261,260,260,259,259,259,259,258,258,258,258,257,257,257,257,257,257,256,256,256,256,256,256,256,256,256,256,256,256,255,255,255,255,255,255,256,256,589,597,605,620,636,641,638,641,636,632,624,620,621,622,624,616,611,605,603,599,592,584,579,573,552,536,526,528,530,529,519,501,500,496,485,485,485,473,465,460,461,457,457,456,456,458,460,463,475,480,487,497,510,523,534,541,542,540,529,518,506,502,522,538,547,549,561,569,573,565,564,570,573,573,572,571,572,579,576,572,573,562,555,560,553,546,538,538,547,549,544,538,523,507,496,490,479,467,459,453,446,447,454,456,456,453,452,448,440,434,427,427,426,431,438,446,460,464,458,446,437,434,434,434,436,441,452,456,450,435,425,418,412,412,413,414,414,406,392,388,382,380,377,369,366,354,348,340,341,347,355,357,359,360,361,362,358,351,345,349,357,366,366,364,366,356,346,352,353,352,351,351,350,352,353,350,343,340,338,333,332,331,330,329,327,327,328,329,334,338,336,328,322,317,314,312,308,306,304,303,305,310,316,323,330,329,332,325,317,316,317,318,320,327,325,317,311,304,305,305,302,308,306,304,301,296,294,292,290,286,282,279,277,277,277,277,277,277,277,277,277,277,277,277,277,279,280,280,281,283,285,287,290,293,293,292,293,293,290,289,289,286,283,280,280,278,277,277,277,276,276,275,275,275,276,275,275,275,274,274,273,271,270,269,269,268,268,267,267,267,267,267,268,267,267,267,267,267,267,267,267,267,267,265,266,265,265,265,266,265,264,263,262,262,262,262,262,262,262,262,262,262,261,261,261,261,262,262,261,261,260,260,259,259,259,258,258,258,257,257,257,257,257,257,256,256,256,256,255,255,256,256,257,258,257,256,255,255,254,254,254,254,254,254,255,606,609,623,635,637,635,637,635,630,621,619,617,620,624,622,614,605,597,593,590,586,576,570,566,554,537,522,511,513,516,514,501,490,490,480,475,474,463,460,462,462,460,459,458,458,459,463,472,478,484,487,497,513,526,540,554,559,557,546,534,522,512,523,543,562,567,572,573,569,568,568,569,570,571,573,574,578,584,587,585,586,575,570,575,568,560,548,532,536,542,541,536,520,503,487,483,477,465,449,447,441,437,446,452,451,448,445,440,432,425,419,418,419,426,432,438,448,452,448,442,435,430,427,429,432,438,448,454,454,440,427,416,411,402,403,406,408,405,400,392,385,376,373,366,364,358,355,347,339,339,345,351,353,354,356,357,354,346,341,346,353,358,358,358,359,354,341,343,345,343,345,345,348,350,350,344,340,339,336,334,332,331,331,329,327,326,326,328,329,332,330,329,324,319,315,312,309,306,304,304,307,312,317,324,329,328,325,326,320,318,317,317,320,320,318,315,309,304,304,304,302,303,303,302,301,296,291,289,286,282,280,278,277,277,277,277,277,277,277,277,277,277,277,278,279,282,285,285,286,288,290,291,293,294,294,294,294,294,293,293,293,293,289,285,282,280,279,278,278,277,277,276,276,276,276,276,275,274,274,274,273,272,270,269,268,268,268,267,267,267,268,268,268,268,267,267,267,267,267,267,267,266,265,265,265,265,265,265,266,265,264,262,262,262,262,262,262,262,262,262,262,262,261,261,261,261,262,263,263,261,261,260,260,260,259,258,258,257,257,257,257,257,257,256,256,256,256,257,257,257,257,258,258,257,258,256,255,254,254,253,253,253,253,254,254,616,616,626,634,632,630,630,623,621,613,609,615,619,619,615,608,598,590,584,582,577,567,556,557,551,539,526,510,496,496,500,498,485,482,475,470,467,461,463,464,464,462,461,460,459,462,469,478,485,491,493,498,509,519,533,555,573,578,565,553,537,529,524,538,561,572,573,566,570,570,569,568,570,570,573,577,584,595,597,593,587,581,583,585,579,572,554,536,523,530,536,525,513,500,484,476,471,461,449,440,434,430,439,443,441,439,435,426,422,414,408,407,414,421,428,435,441,437,436,434,429,424,421,423,426,433,440,447,450,437,428,418,407,400,398,401,402,402,404,399,388,377,374,368,364,360,357,352,341,341,341,345,348,349,352,353,352,346,340,342,347,352,352,352,354,350,342,341,340,339,340,342,345,346,345,341,340,337,336,333,331,329,329,328,327,325,325,325,326,326,327,327,323,319,317,313,310,306,304,306,310,313,317,322,327,329,326,323,322,319,317,315,316,318,318,315,309,304,304,304,302,301,300,298,295,294,290,287,284,280,278,277,277,278,278,277,277,277,277,277,277,278,280,280,283,287,292,293,291,291,293,294,295,296,295,294,294,294,294,294,293,293,293,291,287,281,280,279,279,278,278,278,278,277,277,276,275,274,274,274,274,273,270,268,268,268,268,267,267,268,268,268,269,268,268,267,267,267,267,268,268,267,265,265,265,265,265,265,265,266,265,264,263,262,262,262,262,262,262,262,262,262,261,261,261,261,262,262,262,262,261,261,261,260,260,259,258,257,257,257,257,257,256,256,256,256,258,259,259,259,259,259,258,257,256,255,254,254,253,252,252,252,252,253,254,623,618,624,628,626,622,621,611,608,606,611,611,611,608,607,600,591,584,577,573,569,561,547,539,542,539,529,515,499,486,484,486,478,473,472,468,465,463,466,467,466,463,461,461,461,468,477,488,494,502,503,505,511,520,537,555,575,588,579,568,551,545,534,541,556,563,561,570,573,572,571,569,572,572,573,578,583,594,599,594,592,588,591,590,586,572,552,538,523,513,514,512,501,491,480,471,465,456,448,439,426,423,427,436,427,423,420,415,414,406,400,400,409,417,424,431,433,432,426,420,418,416,412,415,418,428,430,437,438,431,423,414,404,396,394,395,396,398,403,400,388,383,379,372,367,362,356,353,345,340,340,341,342,344,346,346,347,345,340,340,342,345,344,346,348,347,341,337,338,336,337,340,343,343,342,341,340,339,335,332,329,328,328,327,326,325,323,322,320,321,322,323,321,319,317,314,310,306,307,309,311,315,316,319,324,323,320,320,320,319,315,313,312,313,313,309,306,304,304,304,302,300,297,295,294,291,288,285,282,279,277,277,278,278,277,277,277,277,277,277,278,282,282,282,286,291,294,294,294,294,295,296,297,297,296,296,295,295,294,294,294,294,294,292,287,281,280,280,280,279,279,279,279,279,278,277,276,275,275,275,274,273,270,268,268,268,268,267,268,268,268,269,269,268,268,268,267,268,268,268,268,266,265,265,265,265,265,265,265,266,266,265,264,263,263,263,263,263,263,262,262,262,262,261,260,262,262,263,263,263,263,263,263,261,262,261,258,257,257,257,257,257,256,256,257,259,261,261,261,261,261,260,258,256,255,254,254,253,252,252,252,252,252,253,253,623,620,621,623,620,614,609,600,602,610,609,604,600,597,594,588,583,574,568,560,561,555,543,527,531,534,526,515,501,487,475,474,472,469,472,472,470,469,468,468,465,463,463,465,464,474,486,496,506,516,521,525,531,536,547,564,581,590,587,582,569,556,550,555,558,560,565,573,576,577,576,574,575,578,580,586,586,593,592,582,576,586,590,588,578,560,545,535,523,506,497,489,485,478,473,464,456,450,441,432,421,414,418,426,421,411,405,404,403,398,393,398,402,411,421,429,436,433,428,419,411,404,402,406,411,416,422,428,431,429,417,407,397,390,390,390,389,396,403,399,394,391,381,374,365,362,356,352,348,341,338,340,339,340,342,343,343,342,340,339,340,341,341,341,344,343,341,336,331,331,333,338,341,340,340,340,341,340,338,333,331,330,330,329,326,326,322,319,319,317,318,320,320,320,318,315,311,310,309,310,312,314,316,317,320,320,318,317,318,318,316,311,305,306,306,305,304,302,303,304,304,299,296,294,291,288,284,282,280,278,277,277,278,277,277,277,277,277,279,279,278,280,286,289,290,293,294,295,296,296,297,298,298,298,297,297,296,295,295,294,294,293,291,288,285,281,280,280,281,280,280,280,280,280,279,278,277,276,276,275,274,273,271,270,270,269,268,267,268,268,269,270,270,269,268,268,268,268,268,269,269,266,265,265,265,265,265,265,266,267,266,266,265,264,264,264,264,264,263,262,262,263,262,262,261,263,264,264,264,265,266,266,267,268,265,263,258,256,256,256,256,256,256,259,265,266,265,264,264,263,262,260,258,255,255,254,253,252,252,252,252,252,252,252,253,626,622,616,617,614,607,597,593,601,601,597,594,588,586,584,575,572,564,557,548,547,545,534,523,523,523,516,504,489,481,475,475,471,474,475,479,475,475,473,470,465,462,463,467,471,478,490,505,518,532,540,543,548,556,560,572,581,584,586,584,573,560,560,565,570,568,572,573,581,584,584,586,584,581,584,591,598,598,585,569,562,574,581,579,566,550,537,526,517,506,490,482,476,468,464,458,448,443,436,429,418,409,408,415,416,407,398,395,393,388,390,393,399,407,419,429,439,440,438,429,419,409,400,400,401,406,416,420,425,427,418,400,390,388,388,387,387,393,403,402,396,390,381,374,368,365,364,362,356,348,340,340,337,338,339,340,340,340,339,338,339,338,338,338,338,340,338,333,330,328,330,332,333,339,337,340,341,340,338,335,336,338,338,335,332,328,324,319,317,317,315,317,318,318,317,314,312,310,310,311,312,313,314,316,318,318,317,315,315,316,314,308,305,303,303,303,302,301,302,303,304,301,295,292,288,283,280,279,278,277,277,278,277,278,280,278,278,279,282,284,283,282,290,293,293,294,295,297,297,298,299,299,299,298,298,297,296,296,294,294,293,292,289,286,284,281,280,281,282,281,280,281,281,280,279,279,279,277,276,275,274,273,272,274,272,268,268,267,268,269,269,270,270,269,268,268,268,268,269,269,269,269,265,265,265,265,265,265,268,268,267,267,266,265,265,265,265,265,264,264,264,264,263,262,262,264,266,266,266,267,268,269,269,269,268,262,259,257,256,256,256,257,261,266,269,269,268,266,266,266,264,261,258,256,255,254,254,252,252,252,252,252,252,252,252,620,613,608,612,612,604,590,583,589,587,585,582,577,575,576,566,558,553,546,541,533,531,525,515,514,512,503,493,486,487,482,478,478,484,486,487,486,483,477,473,466,463,466,469,474,480,491,504,519,535,544,553,562,571,567,570,573,576,577,579,576,572,574,574,578,577,581,582,584,594,599,599,593,586,586,594,596,583,571,563,550,562,570,567,554,541,532,521,509,498,487,478,467,459,454,451,443,436,431,423,414,406,400,407,410,406,394,386,382,379,386,388,396,403,412,426,435,445,451,438,427,417,405,392,393,402,406,411,418,420,416,403,390,386,386,386,387,392,396,398,395,392,384,379,373,369,368,367,361,350,340,339,335,335,336,337,338,338,337,337,336,336,331,332,332,333,332,333,332,329,329,331,333,334,338,339,341,339,338,338,341,345,345,343,339,334,328,321,317,316,315,315,315,316,316,314,312,310,310,310,310,311,312,314,316,316,314,312,310,309,308,308,305,304,302,301,300,300,300,299,297,296,294,290,286,282,279,278,277,277,277,277,277,279,282,280,280,279,285,292,292,290,290,293,294,295,296,298,299,299,299,299,299,298,298,297,296,295,294,293,292,291,288,286,285,283,282,282,282,282,282,282,282,281,280,280,279,278,277,275,274,274,276,279,274,268,268,267,268,269,270,270,270,270,269,268,268,268,269,269,269,266,265,265,265,265,265,266,268,268,268,267,266,266,266,266,266,266,267,268,267,266,264,263,264,265,266,268,268,269,270,272,271,271,270,266,263,261,260,260,262,264,267,269,271,271,270,268,268,267,266,264,261,257,255,255,256,252,252,252,252,252,252,252,252,610,604,602,611,607,599,590,576,572,571,573,572,565,567,566,561,546,543,536,532,526,519,515,509,502,498,496,492,500,496,485,487,489,496,499,498,500,498,490,477,469,467,469,472,473,476,484,494,507,522,535,550,558,565,563,561,563,569,572,579,586,589,589,585,586,586,590,594,595,608,612,610,596,589,586,580,578,574,565,554,543,547,557,555,540,532,526,517,503,491,481,471,459,450,445,441,438,426,425,418,410,402,394,400,404,402,392,382,375,377,380,386,391,397,404,424,434,443,449,440,429,418,407,395,389,394,398,406,414,417,413,400,389,386,385,383,384,387,388,392,393,390,383,377,373,369,367,363,356,347,340,331,330,332,333,335,335,335,335,335,334,332,330,329,329,330,334,342,334,328,328,329,331,334,339,342,342,341,342,342,348,352,355,353,344,337,329,322,318,316,315,314,312,313,314,312,310,309,308,308,309,309,310,312,312,312,309,307,305,305,306,306,308,305,303,301,299,298,297,295,294,293,291,288,284,281,279,278,277,277,277,278,279,279,282,284,282,281,289,293,293,293,293,293,295,297,298,298,298,298,299,299,299,299,298,297,296,295,294,293,293,292,289,287,286,286,284,283,283,284,284,284,283,282,282,280,280,279,279,277,276,275,278,282,277,268,267,270,269,268,270,271,271,270,268,268,268,269,269,269,269,266,265,265,265,265,266,268,268,268,268,267,266,266,266,266,267,268,268,269,268,267,266,266,267,267,268,269,270,272,272,275,274,275,271,269,267,264,263,266,267,268,270,272,272,273,272,271,271,270,268,266,265,261,258,256,256,252,252,252,252,252,252,252,252,605,596,596,601,598,592,586,575,560,558,563,561,552,556,556,552,540,533,529,522,521,512,509,510,510,502,497,498,500,498,489,497,505,510,515,515,515,512,501,483,471,469,473,474,479,484,485,486,496,509,523,537,549,554,555,557,561,571,576,592,601,606,606,602,599,599,603,610,610,618,621,610,592,578,571,560,559,558,554,547,536,536,546,543,525,518,518,514,503,487,476,465,454,445,438,429,427,417,413,411,403,396,388,392,397,398,388,378,371,365,374,381,387,396,408,422,429,437,442,429,420,412,405,391,387,389,392,399,403,408,410,401,390,386,384,381,378,380,383,388,390,384,377,369,369,366,361,356,351,345,339,331,329,330,331,332,333,333,333,333,333,332,331,329,328,328,329,333,332,329,328,328,330,333,338,342,346,346,347,347,345,346,346,348,344,339,331,325,318,316,314,312,310,310,310,310,308,307,306,306,306,307,307,308,309,308,307,306,302,302,303,304,305,304,302,299,298,296,294,293,291,290,287,284,281,280,278,277,277,278,278,280,281,281,285,289,291,290,290,293,294,294,294,295,296,298,299,299,299,299,299,300,300,299,298,298,297,296,294,294,293,293,288,287,286,285,284,285,285,285,285,285,285,285,284,283,281,280,280,278,274,274,274,275,274,267,267,271,272,271,269,274,274,270,270,270,270,270,269,269,266,265,265,265,265,266,268,268,268,268,268,267,267,267,267,267,268,268,269,269,269,268,268,267,268,270,270,271,272,273,274,278,277,276,276,273,270,267,266,270,271,272,273,274,275,275,276,275,275,274,272,270,268,266,263,262,259,253,252,252,252,252,252,252,252,598,591,584,588,588,582,578,570,560,545,551,550,545,538,539,545,542,535,528,524,522,520,525,526,524,511,500,498,503,503,496,507,517,527,533,535,530,517,499,480,470,472,476,483,487,498,501,491,494,499,510,522,536,538,550,556,565,578,587,600,612,624,625,620,615,613,616,626,628,629,621,605,592,582,573,567,558,542,536,536,533,533,541,537,515,509,509,508,502,488,474,460,449,441,430,424,420,414,401,403,398,390,383,386,390,393,387,376,369,363,367,378,390,397,404,415,417,431,440,426,416,407,401,390,385,386,388,391,394,399,403,399,391,386,381,379,376,378,377,381,381,378,370,363,360,358,352,349,346,342,338,333,331,329,330,331,331,332,331,332,332,332,332,329,328,328,328,328,329,327,327,327,329,334,339,344,348,353,352,349,341,338,337,338,339,339,331,327,319,316,314,308,307,307,307,308,307,306,305,304,304,305,305,306,306,306,304,303,301,300,300,300,300,300,299,297,296,295,293,292,290,287,285,282,279,278,278,278,278,278,279,280,283,284,288,292,299,295,292,293,294,295,296,296,296,298,299,299,299,300,300,300,300,300,299,299,297,296,294,293,292,289,286,286,286,285,285,286,286,286,286,287,286,287,287,287,284,282,281,279,274,274,273,271,269,267,267,270,274,274,274,274,273,273,272,272,272,271,270,268,266,265,265,265,266,268,268,268,268,268,268,267,267,267,267,268,268,269,269,270,269,269,268,268,269,270,272,272,274,275,278,281,279,278,279,280,273,268,266,272,276,277,277,277,276,278,279,278,278,278,276,275,272,270,268,267,266,261,253,252,252,252,252,252,252,590,583,575,576,576,570,569,564,557,540,535,538,539,537,547,554,557,555,550,540,532,535,537,533,522,510,498,499,505,506,505,514,528,542,549,547,527,509,493,476,472,475,485,496,508,513,526,516,510,516,524,526,535,534,549,559,569,584,596,605,620,631,637,638,636,634,628,634,636,628,614,597,586,577,574,568,562,549,536,527,525,532,538,523,510,503,502,500,493,487,474,460,448,437,429,422,416,405,394,391,392,384,378,380,387,393,387,375,366,360,364,373,381,392,399,402,409,427,432,431,426,420,410,397,386,382,382,384,386,391,394,392,387,383,380,376,375,374,369,373,375,370,365,359,352,350,346,342,340,339,336,333,330,329,329,330,330,329,329,330,331,332,331,329,326,326,327,326,328,326,326,326,329,334,340,345,350,355,355,350,341,336,330,329,330,331,331,326,320,316,312,307,305,305,306,307,307,306,305,305,304,303,303,303,301,300,299,298,297,297,297,296,295,296,296,296,295,294,293,291,288,286,284,281,279,278,278,278,278,279,279,279,281,285,288,292,300,298,294,294,295,296,297,297,297,298,299,299,299,300,300,300,300,300,299,299,296,293,292,291,288,286,286,286,285,286,286,287,287,288,288,289,290,291,291,291,286,284,281,280,276,275,275,274,270,267,267,271,274,276,275,274,274,274,274,273,273,272,267,266,265,265,267,267,269,269,268,268,268,268,268,268,268,268,268,268,269,269,270,270,269,269,268,269,270,271,273,274,279,283,288,290,286,281,287,288,279,271,267,275,280,282,281,280,278,280,287,290,290,288,283,279,277,274,272,270,270,268,254,252,252,252,252,252,252,583,572,566,564,565,558,559,555,549,535,530,531,535,551,561,566,571,569,563,548,546,550,539,524,513,504,497,500,512,513,510,519,532,549,551,537,519,503,487,472,475,483,495,512,525,530,538,538,526,526,539,541,548,538,550,562,574,585,594,604,612,625,643,648,649,645,638,639,635,623,607,587,580,572,561,552,545,539,541,538,527,523,525,515,505,496,493,489,482,476,468,458,448,437,428,420,410,400,391,386,385,380,376,375,383,392,388,381,372,361,360,365,377,391,392,392,406,415,418,426,431,428,416,403,389,382,377,378,380,385,386,380,379,381,380,376,372,371,367,364,364,361,358,355,349,342,340,338,336,334,332,332,330,329,327,328,329,328,328,329,330,330,330,329,326,325,324,324,324,324,325,326,329,334,340,345,351,357,360,354,345,337,329,327,326,325,325,322,319,316,313,308,305,304,305,306,306,306,305,305,305,304,303,294,293,293,293,294,294,295,294,292,293,293,294,294,294,294,293,290,287,285,283,281,279,279,278,278,280,281,281,282,284,288,290,292,299,297,296,297,296,297,298,298,298,298,298,299,299,299,300,300,300,300,299,298,294,292,291,290,287,287,287,286,287,287,288,289,290,289,290,290,293,293,293,293,288,284,280,279,278,276,276,275,271,269,272,274,275,276,275,275,274,274,275,274,274,268,266,265,265,267,269,269,269,268,268,268,268,268,268,268,268,268,269,269,270,270,270,270,270,269,269,270,272,273,274,279,286,294,301,303,298,294,302,296,284,275,272,280,290,294,293,287,280,286,299,306,308,304,298,293,284,278,275,273,272,270,255,252,252,252,252,252,252,578,570,561,552,554,549,543,540,533,527,519,527,542,557,570,579,585,583,573,560,560,549,527,516,509,501,505,510,514,516,516,521,531,539,538,525,513,495,480,471,482,496,509,521,536,545,551,546,533,535,543,552,553,548,548,558,567,572,587,596,610,628,641,649,652,647,644,639,634,623,610,601,589,579,572,563,547,533,530,537,538,521,507,501,498,489,482,479,474,468,462,457,453,441,428,418,407,398,390,383,377,375,371,366,379,390,392,386,380,364,356,364,377,389,379,391,395,396,404,422,431,424,413,405,390,383,377,375,378,379,378,375,376,378,377,373,369,368,366,359,353,353,349,345,341,339,339,337,330,330,329,330,331,328,327,326,327,327,327,327,328,329,329,328,325,323,320,320,319,322,325,326,328,333,339,344,351,359,366,359,348,339,334,330,327,325,320,318,318,315,312,309,306,305,304,305,305,305,305,306,306,304,304,296,293,293,292,292,292,292,292,292,292,292,292,292,292,292,290,288,286,284,282,280,279,279,279,278,280,282,283,284,287,291,292,293,294,296,297,297,298,298,298,298,298,298,298,299,297,299,299,300,299,299,296,293,292,291,291,289,288,289,289,289,289,290,290,291,292,292,292,292,294,294,294,293,290,285,279,275,275,276,275,274,273,275,277,278,276,274,273,274,275,275,275,275,268,266,265,265,266,269,269,268,269,268,268,268,268,268,268,268,268,269,269,270,270,271,272,272,271,271,271,273,273,274,278,285,294,304,318,307,318,310,314,306,289,279,276,288,299,306,305,297,288,289,305,315,318,317,308,300,290,279,278,275,273,270,259,256,255,254,254,253,253,584,575,564,547,538,534,529,527,524,523,524,533,538,556,573,586,598,598,583,573,561,540,527,521,512,509,514,522,511,509,508,509,517,522,524,515,501,487,475,479,493,508,522,536,544,551,550,542,538,548,551,555,561,562,555,558,567,576,583,593,606,628,645,651,651,650,646,639,638,634,624,609,593,583,576,568,549,534,519,524,525,514,502,486,482,478,472,471,468,464,458,453,451,443,433,422,409,398,389,383,376,372,367,364,374,382,386,383,378,365,352,361,375,379,373,375,378,394,409,421,428,420,411,399,388,382,378,376,374,374,373,371,371,370,370,368,366,364,363,357,352,347,341,339,340,340,340,339,332,330,329,328,328,327,325,325,325,325,325,326,327,328,326,326,322,320,319,317,318,322,325,328,331,335,339,341,347,355,366,362,351,346,341,334,328,326,320,316,316,314,312,309,307,306,305,305,304,303,303,303,304,304,303,296,295,295,294,293,293,293,292,291,292,292,292,291,288,287,287,287,285,283,281,280,280,280,280,280,279,282,285,286,289,292,293,295,297,298,298,298,298,299,299,299,299,299,298,298,298,298,299,297,297,294,293,292,291,291,291,291,290,291,291,292,292,292,292,293,293,294,294,294,295,295,295,294,293,287,281,277,274,275,276,276,278,279,280,279,277,276,275,271,273,275,274,268,266,265,265,266,269,269,269,269,269,268,268,268,268,268,268,268,268,269,269,270,271,272,273,273,273,273,272,273,274,278,284,292,302,318,324,314,319,321,318,309,291,276,282,297,308,315,312,305,291,291,306,318,322,320,312,302,294,286,280,277,273,268,266,267,268,260,255,255,255,577,573,564,546,526,520,519,518,520,525,534,542,550,562,572,585,596,600,590,578,551,542,539,528,515,512,515,516,502,498,498,498,499,507,509,500,489,481,480,487,502,516,531,540,548,547,547,548,546,552,560,566,569,575,569,564,573,584,596,607,612,632,648,655,656,646,641,635,623,620,611,597,583,571,564,562,552,538,522,509,511,506,499,486,473,462,462,465,462,457,451,445,440,436,430,425,417,405,396,389,381,375,366,362,367,373,378,374,369,360,352,358,370,372,363,367,378,396,411,421,419,414,406,394,384,380,376,372,369,369,368,366,365,366,365,363,362,360,358,355,353,350,345,344,348,348,345,341,337,334,331,328,327,325,323,322,322,323,324,324,325,325,324,322,320,320,318,317,320,323,327,331,336,340,341,340,342,350,358,366,360,353,346,340,334,329,322,317,315,314,311,309,309,308,306,306,306,305,304,303,302,302,300,298,296,295,294,294,294,293,294,293,292,292,288,287,286,286,286,286,284,283,281,282,282,282,284,284,283,283,286,288,290,293,295,298,299,299,299,299,299,299,299,299,299,298,299,299,298,299,298,296,294,293,292,291,291,292,292,292,292,293,293,294,294,294,295,295,296,295,295,295,296,297,298,297,294,290,285,279,274,274,275,276,278,280,281,278,274,275,277,271,268,268,267,266,265,266,268,268,269,269,269,269,268,268,268,268,268,268,268,268,268,268,268,270,272,273,274,274,274,273,273,274,276,281,289,299,313,322,323,317,320,317,308,297,279,278,295,306,320,324,318,309,294,289,306,317,323,322,316,307,299,292,281,278,277,274,271,272,271,268,262,260,258,566,561,554,542,527,510,512,514,520,534,544,554,563,571,582,589,593,598,597,573,557,554,549,534,522,523,520,501,496,494,493,492,491,491,490,487,483,484,485,489,504,522,537,541,545,552,559,561,557,561,569,579,583,587,578,574,586,597,608,623,631,641,650,658,655,644,635,632,620,605,591,583,569,554,548,547,545,537,527,512,497,496,493,486,475,456,450,454,450,448,443,436,430,426,425,419,414,407,403,392,385,379,370,360,361,365,366,366,361,354,352,357,365,365,367,381,388,398,413,420,409,395,392,387,379,374,371,368,365,365,364,362,360,360,359,358,356,359,361,361,358,356,354,355,358,358,354,347,341,336,332,328,326,323,320,319,320,321,322,323,323,323,321,320,319,319,318,316,320,326,330,336,341,344,346,345,348,352,358,366,368,360,352,344,340,332,326,319,316,315,311,307,307,307,306,306,306,305,304,302,301,299,296,295,295,295,294,294,293,294,293,292,292,288,287,286,286,286,286,286,285,284,284,284,284,284,285,285,291,288,287,289,291,293,297,299,299,299,299,299,299,299,299,299,299,299,298,298,298,297,294,293,292,292,291,292,292,293,293,293,294,295,295,296,296,297,297,299,297,296,296,297,297,298,297,295,293,288,282,278,275,275,275,275,274,277,274,274,274,274,273,269,266,266,266,266,267,269,268,269,269,269,269,268,268,268,268,268,268,268,268,268,268,269,269,272,273,274,275,275,274,273,274,274,278,284,294,307,319,318,303,304,308,304,295,280,277,292,306,320,331,333,328,311,296,291,306,317,324,324,319,311,302,291,281,279,280,280,278,276,274,270,268,264,262,556,541,534,531,522,512,510,511,524,539,550,563,574,580,586,598,602,598,596,578,573,565,553,538,531,526,512,497,500,499,494,490,487,485,485,486,487,486,492,499,508,521,532,534,544,553,562,566,572,576,579,589,598,600,590,586,589,607,619,634,642,650,654,653,646,636,630,624,617,606,590,576,563,552,541,530,523,520,511,502,490,482,482,479,474,462,440,438,438,440,438,430,426,423,417,415,408,400,396,393,388,382,374,363,354,355,356,360,356,352,353,356,358,364,373,389,404,408,416,418,404,389,383,377,373,368,366,365,363,362,360,358,354,353,353,353,354,359,365,367,365,366,364,366,367,366,360,352,345,338,334,329,327,321,317,317,319,320,321,321,321,321,320,318,317,318,317,316,322,328,334,341,344,349,353,354,355,358,366,368,366,356,348,341,335,330,324,318,315,313,310,307,305,304,305,304,304,303,303,301,299,295,293,293,294,295,293,292,293,293,293,292,288,287,286,286,286,286,286,286,285,285,286,285,285,285,285,286,287,290,290,291,292,294,296,298,299,299,299,299,299,299,299,299,298,298,298,298,298,297,296,293,293,292,292,292,294,294,294,295,296,297,298,298,298,298,299,300,298,296,297,298,298,297,293,288,285,280,279,276,275,274,274,274,275,274,273,273,271,270,270,267,266,267,269,270,269,269,269,269,269,269,268,268,268,268,268,268,268,268,268,268,268,269,271,273,274,275,276,275,274,274,274,276,279,287,300,315,324,318,306,298,292,290,286,283,296,305,317,330,342,339,328,314,307,299,304,317,326,327,321,313,302,292,285,287,292,289,285,280,277,273,268,267,266,556,540,524,515,511,508,510,521,528,543,555,568,581,591,595,605,611,605,600,592,582,569,556,543,531,515,504,498,514,508,497,491,487,488,492,494,495,493,498,506,518,526,529,534,539,545,553,566,577,587,597,604,612,614,609,610,608,617,630,640,647,650,648,646,637,631,624,613,611,608,601,586,571,557,545,536,531,524,509,493,478,470,467,469,467,462,442,427,426,427,425,422,420,419,417,411,400,393,388,386,384,381,378,370,361,351,352,356,353,351,352,353,359,365,378,391,408,418,418,415,407,394,381,372,366,364,363,362,360,359,357,355,352,352,351,352,355,360,365,372,374,374,373,372,373,370,365,355,348,341,339,334,329,323,316,316,318,319,319,320,320,320,320,318,318,316,316,317,322,330,333,342,347,352,354,360,365,367,374,372,362,351,341,336,328,327,321,317,316,315,314,310,307,304,303,303,302,302,302,300,298,295,294,293,293,292,293,293,292,293,293,289,287,286,286,286,286,286,286,286,287,288,289,288,287,287,286,286,286,288,292,292,293,293,295,297,298,298,299,299,298,299,298,298,298,298,298,298,297,296,296,295,294,294,293,294,295,295,296,297,298,299,299,299,300,299,299,299,298,298,298,298,296,294,290,284,280,279,277,276,274,274,274,274,274,274,274,273,271,269,267,267,267,270,270,271,270,271,270,270,269,269,268,268,268,268,268,268,269,268,268,269,269,270,273,274,275,276,276,276,275,276,278,279,280,288,303,317,324,322,315,302,290,285,290,296,306,317,328,338,346,342,327,321,314,304,304,318,330,330,323,314,304,291,291,298,300,299,296,289,280,278,276,272,268,552,538,524,510,504,498,512,528,540,552,560,576,588,599,596,608,608,598,588,585,582,568,554,541,528,511,508,511,521,516,503,495,492,494,500,503,502,498,503,511,527,537,539,539,547,548,555,562,575,590,607,618,626,628,626,625,626,626,634,645,647,646,645,642,634,634,626,609,596,600,600,585,572,563,556,550,541,526,510,495,484,471,460,451,456,453,440,426,420,416,417,415,412,414,415,409,400,389,382,378,374,372,372,371,367,357,349,353,351,349,353,358,365,378,388,401,414,420,412,402,394,386,377,368,364,361,360,358,356,356,355,353,350,351,353,356,360,363,368,376,381,382,381,379,378,374,366,359,352,348,344,341,334,329,318,316,317,318,318,318,318,320,321,320,317,315,315,317,322,327,331,338,345,352,358,360,367,375,380,375,366,353,345,340,334,328,325,322,319,318,317,314,314,312,307,304,303,302,302,300,298,297,295,293,293,293,292,292,292,292,292,287,286,286,286,286,286,286,286,287,288,292,292,292,292,291,287,286,286,286,291,292,292,293,294,296,297,298,299,299,298,298,298,298,298,298,298,298,298,297,296,296,295,295,296,296,296,296,297,298,299,300,299,299,299,299,299,299,298,298,299,295,293,290,286,284,280,279,276,274,274,274,274,274,275,274,274,272,271,269,268,268,269,271,273,273,274,272,271,270,270,269,269,269,269,268,268,269,269,268,268,269,270,271,273,275,276,276,277,277,278,279,282,286,290,293,304,310,316,317,310,298,290,286,288,293,304,315,328,337,344,345,338,330,319,308,302,319,332,332,326,316,307,292,299,308,312,311,305,298,291,286,283,278,271,533,528,515,506,497,508,521,533,548,562,572,584,595,598,600,598,590,588,571,568,565,559,550,540,525,512,523,526,527,524,510,499,499,500,504,512,508,510,513,516,528,541,548,548,554,559,568,573,586,597,611,624,634,637,639,641,642,634,634,641,644,643,642,639,637,634,624,609,597,591,600,589,576,568,560,550,535,522,510,501,494,485,470,456,439,439,426,422,417,412,406,402,399,402,405,403,397,389,379,373,366,362,362,365,366,358,351,350,347,350,354,357,368,377,388,404,416,416,402,390,381,375,367,363,359,357,357,356,353,353,352,348,345,348,355,360,364,368,372,379,388,390,390,387,384,379,372,364,358,352,348,344,340,332,330,316,316,317,317,317,318,322,324,323,321,315,316,317,320,325,329,337,344,350,353,354,359,369,375,368,366,362,354,346,339,331,329,328,323,327,324,321,321,319,313,305,304,304,302,300,299,297,296,294,294,293,292,293,292,292,288,286,286,286,286,286,286,286,286,287,289,297,305,306,301,293,289,288,286,286,288,290,292,292,294,295,296,297,297,297,297,298,298,298,298,298,298,298,297,297,297,297,296,296,297,297,297,298,298,299,299,299,299,299,299,299,299,299,298,297,295,291,287,285,284,282,282,279,274,275,274,274,274,275,276,274,274,273,273,272,270,271,272,273,274,274,274,272,272,271,270,270,270,270,269,269,269,269,269,268,268,269,271,272,273,275,276,277,278,278,279,281,291,298,304,307,311,310,308,304,298,292,291,288,289,290,298,309,320,330,337,342,344,338,325,315,303,320,333,337,331,320,308,302,310,318,324,323,318,312,303,298,293,285,277,519,512,506,502,506,518,531,543,554,566,576,585,588,590,590,584,573,575,567,549,548,550,544,534,522,520,519,524,528,526,518,505,500,505,509,515,511,516,526,527,537,548,555,558,565,572,580,589,597,611,616,626,634,642,647,646,643,638,641,646,645,645,643,637,631,624,609,600,590,584,595,588,578,572,561,553,542,531,525,512,504,494,475,463,445,439,440,432,423,417,406,397,392,390,395,395,392,387,379,371,366,356,352,357,358,356,350,344,346,349,351,354,363,372,385,400,411,406,395,385,373,365,361,357,354,353,354,353,350,349,346,344,341,349,356,362,367,369,374,378,388,401,405,399,392,384,378,371,363,356,353,347,344,339,331,320,313,315,316,316,320,324,328,331,324,315,315,316,320,324,329,335,340,342,341,342,353,362,364,354,356,359,355,345,339,333,331,328,328,330,330,329,328,323,319,313,308,306,303,300,298,296,296,295,294,295,295,294,292,292,288,286,286,286,286,286,286,287,287,287,288,295,305,308,307,303,297,290,288,286,287,288,289,291,292,294,295,296,296,297,297,298,298,298,299,299,299,298,298,298,298,298,297,297,298,298,298,298,299,299,299,299,299,299,299,299,299,298,295,293,291,288,283,280,280,280,277,275,275,274,275,274,274,275,275,274,274,274,274,273,272,273,274,275,274,274,273,272,272,272,271,271,271,270,270,270,270,270,269,269,269,270,271,272,273,275,276,278,279,280,280,285,299,308,313,316,319,320,314,309,303,298,297,293,295,298,299,303,310,319,325,333,343,345,334,322,312,324,337,340,334,319,312,312,320,330,335,336,330,323,314,301,290,281,276,519,507,499,499,514,527,537,544,549,556,566,574,578,577,577,572,560,548,554,551,537,534,534,524,522,526,523,525,528,530,524,510,506,512,511,516,514,525,537,542,549,556,565,570,576,586,593,606,614,618,624,630,637,643,648,643,642,640,646,649,648,647,647,630,621,609,598,592,585,585,590,583,575,568,561,554,548,538,532,520,506,492,471,459,454,454,453,441,430,422,415,405,398,388,388,382,380,378,374,367,356,352,351,350,349,348,342,343,344,346,353,359,366,374,385,398,401,393,384,375,365,360,355,352,350,350,352,350,347,345,342,341,341,352,357,364,368,370,374,377,386,398,410,413,398,388,382,376,368,364,356,349,343,338,330,320,315,315,315,315,319,323,329,332,323,313,314,317,320,324,328,330,331,331,336,343,351,354,353,348,350,356,351,341,331,329,326,322,322,326,327,330,332,326,317,315,312,309,304,299,295,295,294,294,293,293,292,292,291,290,288,286,286,286,286,286,287,288,288,288,290,293,295,295,301,305,301,294,292,287,286,287,288,290,292,293,294,295,296,297,297,297,298,299,299,299,299,299,299,299,298,299,298,298,299,299,299,299,299,299,299,299,299,299,298,298,298,296,293,290,286,284,280,278,279,277,276,274,274,275,274,274,274,276,276,275,274,274,274,273,274,274,275,274,274,274,273,273,272,272,272,272,271,271,271,271,271,271,270,270,271,272,273,272,273,276,278,280,281,284,286,291,306,313,319,321,327,330,327,320,312,306,306,302,303,309,312,306,303,312,320,334,346,349,340,327,318,331,342,344,334,324,321,325,333,340,344,344,333,318,299,285,277,273,270,514,501,494,492,512,523,532,532,536,545,559,564,561,563,558,552,554,539,537,540,538,537,531,524,525,531,530,532,534,536,525,510,510,515,520,524,521,532,546,555,561,569,575,582,589,601,607,617,626,630,631,636,641,643,648,646,645,643,648,649,649,637,628,620,608,596,591,587,573,582,587,575,563,552,545,542,540,539,534,521,507,492,479,473,469,466,458,446,439,425,418,417,405,395,388,380,376,375,368,364,362,362,359,353,344,341,341,342,341,344,353,360,368,372,382,391,387,380,375,370,362,354,350,347,346,346,348,346,345,343,340,340,344,354,358,363,366,368,373,376,382,391,402,403,402,393,384,376,370,364,356,350,342,335,328,322,318,314,313,314,317,322,329,330,317,314,313,317,319,321,324,325,326,328,334,342,342,342,347,341,346,352,344,336,327,321,319,316,316,318,324,330,331,329,319,316,314,312,304,295,294,294,293,292,292,292,292,291,290,289,286,286,286,286,286,287,290,290,289,289,290,292,293,294,295,296,296,294,294,289,286,286,288,289,290,292,293,295,296,297,298,298,299,300,300,300,300,300,300,300,299,299,299,299,300,300,300,300,300,299,299,299,298,298,297,297,296,294,293,290,286,283,280,279,279,277,276,276,274,274,274,275,276,276,278,274,277,274,274,274,275,275,275,274,274,273,273,273,273,273,272,272,272,272,272,272,272,272,272,272,273,273,274,273,274,275,279,284,294,298,300,300,307,314,317,314,321,332,333,326,315,318,317,308,310,320,321,317,306,315,329,342,351,353,343,328,327,341,349,350,341,337,338,340,346,350,343,330,312,299,289,278,274,268,266,502,494,488,489,498,515,519,519,531,545,556,551,546,550,540,539,537,529,522,523,523,532,538,536,534,538,538,536,541,539,523,517,514,521,529,534,530,535,549,565,576,586,590,595,605,618,624,625,635,636,636,642,647,646,653,652,646,645,647,644,637,620,611,606,599,590,582,580,571,575,582,577,564,543,532,525,526,526,526,522,513,500,491,487,486,471,460,453,442,430,430,427,415,404,397,391,388,378,368,371,373,374,374,370,353,341,338,341,338,341,353,359,363,366,378,381,377,374,370,364,357,351,344,341,341,344,345,344,342,340,341,341,346,353,357,361,364,368,372,374,378,387,394,397,401,394,383,375,368,362,356,348,340,331,327,321,316,315,311,311,314,319,324,320,311,308,312,315,317,319,320,321,324,328,331,334,334,333,334,340,341,346,341,333,325,317,314,313,313,320,330,323,327,331,325,320,316,308,304,296,292,292,292,292,292,292,292,292,291,288,286,286,286,287,288,292,292,291,292,290,291,293,295,297,297,293,298,300,299,290,286,286,288,289,290,291,292,294,296,298,299,299,300,300,300,301,301,301,301,301,300,301,301,301,301,301,300,300,300,300,299,299,298,298,297,296,295,294,293,289,287,283,280,280,279,278,277,276,274,274,275,276,277,278,278,278,280,277,275,275,276,276,275,274,274,274,273,273,273,273,273,273,273,273,273,272,272,272,272,273,273,274,275,274,276,279,283,294,305,310,310,309,306,305,303,302,314,329,333,329,326,328,323,315,319,328,331,322,312,322,337,350,358,359,353,338,336,349,359,360,356,353,358,358,357,340,324,307,302,293,288,281,278,279,274,500,490,483,481,490,504,497,514,528,543,550,536,532,537,530,521,521,515,508,510,512,522,535,544,544,547,548,548,550,543,525,523,521,525,538,547,545,541,548,567,586,597,607,613,621,628,634,633,633,638,642,646,654,658,659,659,655,651,650,642,624,611,598,594,585,584,576,572,571,570,576,576,569,549,528,511,510,510,510,509,506,500,490,488,483,473,465,464,452,441,441,431,424,413,404,401,394,385,380,382,383,384,384,378,360,347,340,337,337,343,353,357,355,364,368,374,371,369,364,358,354,348,340,339,340,342,343,341,339,338,339,343,346,352,356,358,364,368,370,373,377,386,392,395,401,393,384,376,367,362,354,347,340,334,331,326,319,314,310,308,309,312,316,311,305,306,309,313,315,316,316,317,320,323,328,327,327,328,332,337,341,344,342,332,324,316,310,306,310,320,324,318,316,322,324,318,313,305,302,298,294,293,292,294,297,300,300,297,296,289,286,286,287,290,294,292,295,299,294,290,292,296,300,303,305,305,302,304,305,292,286,286,288,288,289,289,291,294,297,298,299,300,300,301,301,301,302,302,302,302,303,303,303,303,302,301,301,300,300,300,299,299,298,297,296,295,294,293,291,287,285,284,283,282,280,280,280,278,278,280,278,277,278,280,280,280,280,280,278,277,276,276,275,274,274,274,274,274,274,274,274,274,273,273,274,273,273,273,273,273,274,275,277,279,281,288,293,300,310,318,318,312,304,298,297,301,314,327,331,332,333,333,331,322,324,333,336,328,316,328,344,355,366,368,361,350,345,359,370,374,371,373,372,361,342,328,316,312,307,301,297,293,289,285,279,503,490,480,475,478,483,486,504,522,536,534,524,516,524,520,507,502,498,501,508,516,527,540,554,558,557,561,560,560,552,541,531,521,532,546,557,564,561,559,564,584,602,616,625,631,635,637,636,637,640,644,649,655,659,660,657,658,643,628,623,610,603,595,590,583,576,573,569,563,563,570,576,569,547,529,512,498,498,498,497,493,489,479,477,475,468,468,470,461,454,445,440,432,421,414,410,400,389,390,391,392,392,385,372,362,349,340,336,338,343,348,350,355,363,368,368,368,366,362,357,351,344,340,337,338,341,342,339,337,337,338,342,345,349,351,354,357,364,365,368,376,380,385,392,395,390,387,381,373,366,357,351,346,340,335,329,322,316,311,306,306,307,308,305,304,306,308,311,314,313,314,316,317,321,324,327,326,329,334,340,343,343,338,330,322,315,308,304,307,312,311,311,314,316,324,317,309,304,301,302,302,297,296,298,306,317,320,309,296,289,286,287,290,295,303,292,299,306,301,292,294,300,304,306,306,305,306,306,306,292,286,286,287,287,287,293,288,297,298,298,299,300,300,301,302,302,302,304,304,303,304,304,305,304,303,302,301,300,300,300,299,298,298,297,296,294,293,292,291,288,287,287,287,285,283,281,280,280,281,280,280,279,278,279,280,280,280,280,279,278,277,276,275,275,275,274,274,275,275,275,274,274,274,274,274,274,274,273,274,274,275,277,279,284,293,301,305,304,305,314,320,312,300,291,294,305,311,319,324,330,338,341,337,331,330,337,343,337,326,328,344,359,370,377,372,361,355,369,379,381,381,379,369,352,332,323,320,318,311,306,295,293,289,284,281,506,490,479,469,466,472,480,505,525,522,516,512,503,508,506,498,491,499,506,509,519,528,539,562,572,570,577,575,574,565,552,536,527,535,550,565,576,579,578,579,588,598,613,626,634,636,638,640,642,643,643,646,652,650,643,634,642,632,609,607,604,596,585,585,579,576,571,564,559,559,566,574,565,547,530,515,497,487,487,485,482,477,467,462,462,462,472,477,468,458,450,447,438,429,427,417,405,398,400,402,396,387,378,365,354,346,334,332,335,340,341,348,356,360,365,363,362,362,360,356,352,345,339,336,339,340,342,336,332,333,337,340,342,344,347,350,353,360,364,369,373,377,383,387,393,394,392,387,379,374,368,359,352,345,340,332,324,317,313,308,305,305,305,304,304,307,308,310,313,316,318,322,324,326,329,328,328,332,337,341,343,339,331,325,318,314,308,304,304,304,307,308,314,322,330,322,314,307,304,305,305,302,302,304,311,319,320,309,294,289,286,288,294,301,306,293,301,307,304,295,294,302,307,306,306,306,306,305,300,293,287,286,287,289,290,297,299,298,298,299,300,300,302,302,303,303,303,304,305,304,304,304,305,304,302,301,300,300,300,300,299,298,298,297,296,295,294,294,293,293,292,288,287,287,286,283,282,282,281,280,280,279,279,279,280,280,280,280,279,278,278,277,277,276,276,276,276,276,276,276,275,275,275,275,275,274,274,274,274,275,277,279,284,293,297,313,311,302,295,304,312,309,298,289,291,298,302,312,320,329,338,343,341,334,336,342,347,347,338,331,341,360,374,383,382,374,370,376,383,386,382,367,352,338,325,315,316,311,308,301,291,286,286,282,257,496,490,476,465,459,464,486,509,522,514,497,494,492,490,491,487,486,502,514,517,526,533,547,562,578,588,594,591,584,572,552,540,535,538,551,568,584,595,598,598,605,614,622,628,629,631,635,644,646,646,645,646,647,633,622,618,624,619,607,595,592,590,586,580,572,572,571,563,560,557,562,568,558,543,528,514,496,479,475,476,470,463,458,454,458,462,474,487,476,466,458,451,441,434,429,417,410,410,410,402,392,381,374,366,358,347,330,329,332,336,339,344,352,354,353,354,359,358,356,353,348,344,339,334,334,336,339,334,329,331,333,337,340,340,342,349,354,359,365,368,370,376,380,386,391,392,392,388,381,375,366,362,354,347,342,332,325,318,314,309,307,305,304,304,304,306,305,307,314,318,322,326,330,331,331,329,331,331,335,338,339,334,327,318,313,307,307,305,303,303,305,310,314,317,327,321,312,304,305,305,305,304,303,302,310,317,317,306,292,288,287,291,292,301,301,296,305,305,305,301,301,301,305,306,306,306,305,302,299,294,289,288,289,292,295,299,299,299,299,300,301,302,303,303,304,303,304,304,304,304,304,304,304,303,302,301,300,299,299,299,298,298,297,296,296,295,294,294,294,293,292,289,288,285,285,284,282,280,280,280,280,280,280,280,280,279,280,279,279,279,278,278,279,278,279,278,278,278,277,277,277,277,276,276,276,275,274,274,275,276,278,281,292,298,302,313,314,301,291,299,304,301,292,291,295,301,308,315,322,328,335,341,334,330,333,341,351,355,354,347,340,358,376,387,392,387,383,384,383,379,367,353,331,324,320,314,308,303,298,294,289,281,279,280,256,484,479,468,459,453,458,478,496,507,501,488,477,481,477,477,485,495,508,522,529,533,547,558,575,591,605,614,606,591,574,559,555,546,547,552,566,583,598,611,614,616,623,630,637,636,632,636,643,648,648,648,646,637,622,607,602,606,600,598,584,582,582,579,577,575,571,563,560,560,556,560,561,557,539,526,512,496,481,471,464,461,455,451,451,457,462,477,489,480,468,458,450,443,437,427,423,418,417,408,397,388,380,373,368,360,351,335,328,329,332,336,340,345,349,350,351,355,355,352,346,344,342,334,329,332,331,331,329,327,329,333,336,340,344,346,350,354,358,362,364,366,371,376,379,385,386,385,382,376,365,358,357,353,346,338,329,323,317,314,311,308,306,305,304,304,306,304,306,312,317,318,322,324,327,328,328,328,328,332,330,332,330,324,316,309,305,305,304,304,303,304,310,308,312,318,318,309,306,302,305,304,305,304,304,310,317,313,297,290,289,289,292,295,297,304,302,305,308,305,305,304,304,304,306,306,305,305,301,298,295,293,291,294,294,297,300,300,300,300,301,302,303,304,304,305,304,304,305,305,305,304,303,303,301,300,300,299,299,298,298,298,297,296,296,295,295,294,294,293,292,292,291,288,287,285,284,283,281,281,280,280,281,281,280,280,280,280,280,280,279,279,279,280,280,280,280,280,279,278,279,279,279,278,278,277,276,276,275,276,278,279,284,293,299,307,312,315,301,291,294,295,292,290,292,296,304,309,317,322,328,332,333,326,326,331,344,354,363,367,365,360,362,378,391,398,398,393,386,373,365,352,334,321,312,307,310,313,300,286,280,282,278,272,266,255,472,466,458,451,446,447,468,479,491,488,477,471,472,472,476,486,504,520,536,545,542,556,574,585,604,622,625,613,593,580,577,569,556,556,565,568,578,593,611,623,626,629,635,643,645,634,636,641,646,646,638,626,619,614,604,585,588,585,585,577,572,573,571,569,572,574,567,563,559,554,558,560,556,539,526,510,493,476,466,460,454,450,448,451,458,467,478,485,476,466,454,447,442,440,434,428,423,414,406,396,386,378,366,362,356,346,337,329,327,329,333,338,342,345,346,348,350,351,347,341,338,336,330,328,330,328,327,329,332,335,337,341,348,354,353,352,353,356,356,359,363,367,370,374,380,379,375,374,368,359,350,347,349,345,340,328,319,316,314,311,308,306,305,303,303,304,303,305,308,313,314,314,316,319,320,322,324,326,328,327,325,324,320,314,305,304,305,304,302,302,304,307,304,305,311,316,306,305,305,303,304,305,304,307,315,320,309,292,290,292,291,292,296,299,300,302,305,308,309,311,308,307,309,306,306,305,302,301,299,298,296,296,298,300,301,303,302,301,301,302,303,304,305,305,305,305,304,304,304,303,303,302,301,300,299,298,298,298,298,297,297,296,296,295,295,294,294,293,293,292,291,290,289,287,286,285,283,282,281,281,281,281,281,281,281,281,281,281,281,280,280,280,280,281,281,281,280,280,280,280,281,280,280,279,278,278,278,277,278,279,280,287,292,305,314,314,312,298,289,292,292,290,290,291,296,301,306,314,319,321,326,325,318,325,334,347,357,365,376,377,376,374,379,392,404,403,393,378,361,346,335,321,311,302,294,307,317,307,293,277,271,280,267,256,254,459,453,446,439,438,440,449,464,476,474,467,463,466,473,480,494,512,532,550,562,564,564,579,600,614,629,633,618,596,594,590,580,564,563,577,582,587,593,608,623,634,636,639,643,646,640,644,640,638,629,622,612,603,604,600,587,572,572,575,570,567,564,560,566,573,570,565,564,562,552,551,558,551,537,521,503,490,478,468,459,450,448,445,450,459,468,478,481,477,465,455,447,437,439,436,430,424,416,408,400,389,376,362,352,346,340,334,330,327,328,331,336,339,342,342,342,346,345,344,340,331,331,328,326,324,326,328,337,341,344,344,348,354,354,354,352,352,352,352,353,356,358,363,370,377,372,365,365,362,355,342,339,342,344,339,327,317,315,313,311,308,306,304,303,303,304,303,304,307,308,307,307,314,314,315,317,323,322,319,318,316,319,316,309,304,303,303,302,302,302,303,304,303,304,305,312,311,311,307,303,304,298,299,306,316,322,307,292,291,293,295,296,300,302,302,304,306,310,310,311,311,311,309,307,305,303,302,301,300,300,300,300,301,304,305,304,304,302,302,302,304,305,305,305,305,304,304,303,303,302,302,301,300,299,299,298,298,297,297,296,296,296,295,295,294,294,294,293,293,292,292,290,289,288,287,286,284,283,282,282,282,282,282,282,281,281,282,282,281,281,280,280,281,282,282,282,281,281,280,282,281,281,281,280,280,280,279,279,280,280,286,291,298,308,318,317,304,292,288,287,287,287,287,290,292,296,304,312,317,316,319,316,314,323,337,350,359,365,381,388,389,388,392,399,396,380,380,368,350,335,322,309,298,291,285,295,305,312,296,281,268,261,256,255,254,444,440,432,431,432,437,446,455,463,462,458,462,468,476,484,495,510,530,551,567,579,587,592,599,622,637,642,627,618,615,607,592,576,573,584,598,605,613,614,622,635,641,646,649,651,647,648,643,631,616,607,597,591,593,595,586,572,562,564,560,563,558,564,573,562,563,558,557,552,547,551,552,549,539,524,514,503,490,473,461,450,443,442,448,457,466,477,476,477,469,459,448,436,432,430,426,422,416,408,399,390,378,362,352,343,338,331,328,326,329,330,332,335,336,338,339,342,342,340,333,327,326,326,322,325,329,336,344,349,352,352,354,354,356,354,351,350,350,350,350,349,351,358,365,368,365,356,356,357,351,340,334,336,339,334,326,317,315,313,311,308,306,304,302,302,302,302,302,304,304,303,305,309,312,315,318,321,318,315,312,310,312,310,305,303,302,301,300,300,301,302,302,304,306,308,309,312,315,314,303,298,295,296,301,314,320,306,291,291,294,298,301,304,305,305,305,309,311,311,312,311,310,308,306,305,303,302,301,301,302,303,302,302,306,305,304,304,303,302,302,304,305,305,305,304,304,303,302,302,301,301,300,300,299,298,298,297,296,296,296,295,295,295,294,294,294,293,293,293,292,292,290,289,288,287,286,285,284,284,283,283,283,283,283,282,282,283,282,282,282,281,282,282,283,284,283,283,283,280,282,282,282,281,281,281,281,280,280,280,286,294,299,306,318,318,310,297,288,282,283,282,283,286,288,291,297,305,308,313,312,312,312,316,325,337,352,361,370,385,394,398,398,394,390,376,365,358,359,353,338,323,309,295,287,279,278,287,294,289,277,265,254,250,254,253,430,429,427,424,427,433,438,444,450,450,455,462,469,477,490,505,520,536,552,565,580,596,606,613,629,645,655,643,636,630,621,605,584,579,594,608,618,624,626,629,636,644,652,659,658,653,649,640,629,616,610,599,585,582,586,581,571,558,557,556,557,559,572,574,561,555,548,550,547,549,550,551,551,544,528,516,504,492,473,460,450,439,438,445,452,459,463,464,468,469,460,449,437,426,424,420,416,411,404,393,384,372,359,350,341,340,336,332,328,328,327,329,330,331,334,337,339,337,333,328,324,321,320,326,332,342,342,350,355,356,357,357,354,352,349,348,349,349,348,343,343,346,354,356,356,357,351,346,350,347,340,328,328,333,332,326,316,314,313,310,308,306,305,302,302,301,300,299,301,301,303,306,312,316,319,318,318,314,311,305,304,306,305,303,302,301,300,295,297,299,300,302,305,305,306,307,305,305,306,302,295,293,295,298,308,314,302,290,292,296,301,305,307,309,309,309,310,311,311,311,310,309,307,305,304,303,302,302,303,304,304,303,304,305,305,305,304,303,303,302,304,305,305,304,304,303,302,301,301,300,299,299,299,298,298,297,296,296,295,295,295,294,294,294,294,293,293,292,292,292,291,290,288,288,287,287,286,285,284,284,284,284,284,284,284,284,284,284,283,283,283,283,284,284,285,286,286,285,282,282,282,282,282,282,281,281,280,280,281,293,302,312,320,324,319,308,294,285,280,281,280,282,285,289,292,299,305,308,310,306,305,307,314,325,336,349,360,372,384,396,403,393,378,369,360,352,339,343,344,341,330,316,298,288,280,276,270,275,273,260,254,250,250,251,252,426,424,424,426,431,436,437,440,447,450,456,462,469,476,488,505,523,533,545,555,571,592,610,628,642,655,663,657,649,640,628,611,589,590,604,614,622,632,636,637,643,649,659,663,661,655,646,638,631,624,617,600,585,573,577,574,563,554,550,548,552,564,573,568,557,548,548,545,548,549,550,553,550,538,522,511,502,488,470,457,445,437,434,439,446,453,456,453,458,462,457,448,437,423,414,412,410,405,398,389,381,370,361,352,350,345,342,337,332,329,325,326,327,328,332,334,334,329,328,324,319,316,321,326,334,343,347,351,355,356,354,352,348,344,344,346,349,350,347,342,340,344,347,349,350,351,344,339,340,344,338,327,324,328,330,324,316,315,312,310,309,308,306,305,305,299,297,296,297,299,302,308,316,318,318,317,314,309,307,305,305,307,309,308,306,303,303,293,295,296,296,303,304,305,306,306,305,301,303,296,294,294,296,300,305,307,295,292,296,302,304,307,309,311,311,311,310,310,310,311,310,307,305,303,303,303,303,303,304,305,305,305,305,306,306,305,305,304,302,303,304,305,305,304,303,302,302,300,299,298,298,298,298,298,297,296,295,295,294,294,294,294,294,293,293,292,292,291,291,290,290,289,288,288,287,287,287,286,285,285,284,284,284,284,284,284,285,285,285,285,284,284,285,285,286,286,286,286,284,283,283,282,282,282,282,281,280,280,284,295,307,319,328,329,319,308,292,285,279,279,279,280,284,288,292,298,303,306,307,304,303,307,315,322,331,336,349,368,381,392,395,382,365,351,343,333,326,326,330,330,328,319,306,294,286,278,267,256,255,253,250,250,250,251,251,431,430,427,430,435,441,444,445,449,453,461,467,475,479,485,497,508,522,541,560,574,588,609,634,650,662,663,657,648,636,622,608,607,612,610,621,631,637,643,645,651,656,664,669,664,655,645,638,631,626,613,598,584,571,564,559,557,549,547,544,550,564,568,559,551,540,541,545,548,549,551,550,548,532,516,504,493,481,469,456,442,433,427,433,439,445,448,444,447,452,448,442,432,423,414,408,402,400,392,385,379,370,363,357,355,350,345,342,333,330,325,324,324,327,331,332,332,330,325,322,316,317,322,326,331,339,343,344,348,349,347,344,341,340,342,346,350,353,356,347,338,340,342,342,342,345,340,335,334,335,332,327,322,320,321,318,316,314,311,310,308,308,306,306,304,299,294,294,294,296,299,303,310,315,314,308,306,304,303,304,306,316,321,323,318,309,298,292,295,294,293,304,305,303,305,307,300,296,294,293,292,294,306,301,303,303,293,295,300,305,309,311,311,312,311,312,311,311,310,309,308,306,305,305,305,305,304,304,305,305,306,306,306,306,306,305,304,304,304,304,305,305,305,304,303,303,301,300,300,299,298,298,297,297,296,295,294,294,294,293,293,294,293,293,292,292,291,290,290,289,289,288,287,287,287,287,287,286,286,285,285,285,285,285,285,285,285,286,286,286,285,286,286,286,286,286,285,286,284,283,283,283,282,282,281,280,280,280,286,297,309,320,326,328,316,305,292,285,280,279,279,280,284,288,292,296,301,304,304,302,303,307,313,316,315,321,344,362,372,381,392,385,369,351,337,326,317,314,312,315,317,319,315,302,290,283,269,255,253,251,250,250,250,250,250,442,437,432,434,438,442,451,454,454,458,462,469,475,484,493,500,510,530,554,571,586,601,620,638,651,662,660,651,641,632,621,619,625,630,628,632,640,645,652,655,659,667,672,672,666,660,652,643,632,632,618,604,590,575,568,563,558,553,547,542,554,562,562,554,544,536,548,554,554,550,550,545,532,518,507,500,488,474,466,454,442,433,424,426,432,439,440,432,431,439,439,435,428,421,416,408,401,396,389,384,378,370,365,358,355,349,344,340,333,327,321,320,322,326,329,329,332,333,331,332,317,317,322,328,331,334,335,337,340,340,340,338,338,340,343,345,350,352,353,350,339,337,337,336,339,342,334,328,329,328,329,320,318,316,315,316,314,313,310,307,308,307,306,304,301,298,294,292,293,294,296,300,306,307,304,303,303,303,302,303,305,316,327,331,316,305,295,293,294,293,294,298,300,300,305,306,297,293,292,292,292,294,295,300,305,298,300,300,302,307,311,313,313,313,313,312,312,311,311,310,309,307,306,306,306,305,304,305,305,306,306,306,306,306,306,305,304,304,304,305,305,305,305,304,304,302,300,299,299,298,298,297,297,296,295,294,294,293,293,292,293,293,292,292,291,291,290,290,289,288,288,287,287,287,287,287,287,287,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,286,285,284,284,284,283,283,282,282,281,281,280,280,280,286,297,309,323,326,322,312,300,290,285,280,279,279,281,284,288,292,295,298,300,299,301,303,307,306,304,305,316,326,339,355,372,389,387,371,352,340,327,320,311,303,304,309,305,303,299,295,284,265,255,252,250,250,250,251,250,250,449,440,436,437,443,447,454,461,460,462,468,473,480,486,493,507,523,538,554,571,589,608,626,643,659,663,657,647,638,636,636,635,638,643,645,649,651,658,660,664,668,672,672,666,660,655,649,643,637,636,623,612,600,588,580,570,563,562,550,536,550,555,553,549,539,540,552,560,556,550,540,526,513,505,497,491,483,470,457,448,442,433,424,417,424,428,426,424,426,433,433,431,424,420,414,410,404,396,388,380,376,371,364,361,353,347,339,333,327,324,317,318,320,323,326,329,334,338,336,333,315,318,322,328,330,329,327,331,333,334,333,334,340,342,344,347,350,352,353,351,340,337,332,330,334,331,328,326,326,324,325,321,318,316,315,315,315,312,310,306,307,307,305,304,300,297,294,292,292,293,294,298,305,306,305,299,301,303,303,303,304,312,324,330,315,304,297,292,293,292,295,295,292,294,302,299,294,292,292,293,293,293,293,297,301,303,304,304,306,310,312,313,314,314,313,313,312,312,311,310,310,308,307,307,306,305,305,305,306,306,306,307,306,306,305,305,304,304,305,305,305,305,305,305,304,301,300,298,299,298,298,297,296,295,294,294,292,292,292,292,291,292,292,291,290,290,289,289,288,287,286,286,286,286,286,287,287,287,286,286,286,287,286,286,286,287,287,286,286,287,287,286,287,286,286,285,284,283,283,283,282,282,281,281,280,280,279,280,284,296,308,319,323,318,308,296,288,284,280,279,280,281,284,288,291,294,296,296,294,297,300,300,298,300,304,314,330,348,366,378,390,390,375,358,343,334,329,318,306,298,293,293,285,284,280,276,260,254,250,248,250,251,251,251,250,451,448,443,442,447,452,457,465,465,469,475,481,489,497,500,505,520,534,550,563,582,606,628,646,659,660,656,648,647,644,645,646,646,652,659,662,664,670,669,670,672,671,666,661,655,649,645,645,644,644,633,623,611,597,589,576,570,564,553,537,537,547,542,537,534,544,554,558,559,545,522,508,500,495,487,477,473,466,451,437,433,428,422,412,415,415,414,417,428,429,428,427,424,420,418,416,411,402,391,385,379,377,368,367,360,353,342,335,333,334,326,315,317,320,326,331,336,342,337,326,316,317,320,324,324,322,323,327,329,329,329,335,341,344,351,350,350,352,353,349,340,334,331,327,331,335,335,334,333,331,330,328,322,318,316,316,316,316,309,305,306,306,305,303,301,297,294,292,292,292,293,295,297,297,303,297,301,304,305,305,301,307,318,324,316,304,298,294,291,291,293,293,291,295,294,291,290,292,294,296,296,295,295,298,303,305,307,310,312,312,313,314,314,314,314,313,312,311,311,310,310,309,308,307,306,306,306,306,306,306,307,307,307,305,305,305,304,305,305,305,305,305,306,305,303,301,299,298,298,297,297,296,295,294,294,293,293,292,292,291,291,291,291,290,290,289,288,288,287,286,285,285,284,284,285,286,287,286,286,286,287,287,287,287,287,287,288,287,286,286,287,287,286,286,285,284,283,283,282,282,282,282,281,281,281,280,280,279,280,292,305,312,308,310,305,292,286,282,279,279,279,281,284,287,289,293,293,293,292,297,295,292,291,294,301,308,319,336,353,376,391,393,381,363,352,346,340,327,314,300,288,278,274,269,270,269,256,255,250,255,253,252,252,251,250,457,455,453,454,458,463,462,471,471,474,484,493,499,511,519,520,521,535,546,558,583,607,628,643,654,659,655,654,655,654,650,652,655,658,666,672,674,674,671,669,666,662,660,658,656,655,653,651,648,646,636,630,622,608,598,586,575,565,554,539,530,537,530,534,538,549,556,562,563,550,525,505,487,484,476,468,461,456,446,434,424,416,416,408,403,403,406,420,427,423,421,421,419,416,414,412,407,402,392,384,379,376,369,367,360,353,349,345,344,344,337,322,315,318,323,329,333,336,330,318,318,316,318,318,319,317,318,326,326,328,331,337,343,347,353,356,353,353,349,343,338,331,329,327,332,340,346,347,346,345,340,332,326,324,322,320,320,317,310,304,306,305,304,303,301,298,296,292,291,292,292,292,293,294,294,293,296,305,304,305,300,304,314,318,313,303,298,294,292,289,287,290,291,290,289,288,289,290,292,294,298,298,300,302,304,308,310,311,312,313,314,314,314,314,314,313,312,312,311,310,310,308,308,307,307,306,306,306,306,307,307,308,307,306,306,305,305,305,305,305,305,305,304,304,301,299,298,298,297,297,296,296,295,294,293,292,292,292,291,291,290,290,290,289,289,288,288,287,286,285,284,284,282,281,283,285,286,286,286,286,287,287,287,287,287,287,287,287,287,287,287,287,286,285,284,283,282,282,282,282,281,281,281,281,281,281,280,280,279,281,292,299,296,300,295,288,282,279,278,279,279,280,282,284,289,290,291,291,295,295,292,290,288,292,298,303,312,329,346,365,386,383,379,370,360,351,345,336,321,308,295,286,276,269,264,255,255,253,256,256,255,253,252,251,251,463,462,466,463,471,474,474,477,476,481,492,504,517,525,539,542,533,536,558,575,588,606,621,634,651,660,659,659,661,658,655,656,662,668,670,675,676,672,662,657,652,650,650,649,659,661,659,648,639,629,618,614,617,616,602,589,576,564,550,538,523,524,525,537,545,552,558,563,564,545,526,505,484,472,465,458,449,444,438,430,420,413,407,404,395,394,406,416,420,416,413,416,416,412,408,405,402,395,387,379,371,367,363,359,354,349,346,342,342,340,337,331,316,316,319,324,329,328,321,315,317,318,317,316,317,317,318,320,325,328,330,336,342,346,350,351,351,348,342,338,333,328,324,324,332,342,352,356,357,355,347,338,330,330,329,326,324,317,316,313,306,304,303,302,301,299,296,293,291,291,291,291,291,291,293,292,296,301,302,302,298,301,304,314,309,299,295,292,290,288,287,290,288,288,288,289,291,292,295,297,300,303,303,304,307,311,312,312,312,314,314,315,314,314,313,313,312,312,311,310,308,308,307,307,306,306,306,306,307,307,308,308,308,307,306,306,305,305,305,305,305,305,305,302,299,299,298,297,297,297,296,295,295,294,293,292,292,292,291,290,290,289,289,288,288,287,287,286,285,284,283,282,282,281,282,282,285,286,287,287,287,287,287,287,287,287,286,286,286,287,287,287,287,285,283,281,280,281,281,281,281,281,280,282,283,281,282,281,280,276,277,284,288,286,284,280,278,274,278,280,280,280,280,281,286,289,291,292,293,293,290,288,287,291,295,301,308,318,341,366,371,362,358,359,358,352,344,342,331,316,303,294,283,278,272,263,255,255,256,255,255,253,252,251,251,477,473,477,478,480,490,486,487,486,487,503,515,532,545,556,562,556,554,564,580,593,609,626,641,656,663,663,659,656,655,656,659,668,674,675,676,670,662,652,645,636,637,636,648,660,656,642,636,624,611,601,594,599,604,601,589,575,561,547,538,524,522,528,536,546,552,558,562,563,546,531,509,488,468,451,448,442,435,427,421,416,408,400,396,391,390,402,408,414,412,409,411,410,406,400,396,395,388,379,376,365,363,355,350,345,340,339,336,332,332,331,327,317,314,316,318,320,319,316,316,316,317,317,317,318,319,320,322,325,328,329,333,338,341,341,341,341,342,338,334,330,324,316,322,329,340,343,346,350,352,351,345,342,338,332,331,330,328,326,320,315,308,305,303,301,299,296,294,292,291,291,291,291,290,289,291,294,298,303,297,297,300,305,310,307,298,293,289,288,287,287,287,287,287,289,292,294,296,298,302,303,304,306,306,310,312,313,313,314,315,315,315,314,314,313,312,312,312,311,310,309,306,306,306,306,306,306,306,306,307,308,308,308,307,306,306,306,305,305,305,305,304,304,303,298,298,298,298,297,297,296,295,294,294,293,292,292,291,290,290,289,288,288,287,287,286,286,285,284,284,282,282,281,280,280,281,283,286,287,287,287,287,287,287,287,286,286,286,287,288,288,289,288,286,282,277,276,275,280,281,282,283,285,286,290,291,282,282,280,278,273,273,275,276,276,274,277,275,280,282,283,283,282,284,287,291,293,292,292,291,287,284,283,286,293,297,304,317,342,350,354,346,340,340,340,342,341,329,326,320,310,303,296,290,280,269,256,256,256,255,254,253,252,252,251,492,483,483,492,495,501,500,496,498,500,510,526,545,562,572,580,578,568,585,598,611,616,627,644,659,665,660,652,646,648,656,662,669,672,672,668,658,650,643,634,618,615,628,646,648,633,619,615,606,597,585,572,581,588,588,585,575,565,555,545,530,522,530,541,549,555,559,562,563,551,531,511,486,467,450,438,433,429,422,414,404,399,392,386,387,391,398,402,405,407,402,400,403,401,392,388,391,391,390,380,371,367,356,341,339,336,330,329,326,325,320,315,319,320,313,315,316,315,314,315,316,316,317,317,319,320,322,323,325,326,328,330,335,338,338,339,342,342,338,330,326,320,315,319,324,329,332,335,338,342,342,342,339,338,336,336,335,335,330,324,318,312,307,304,302,299,297,295,294,292,291,292,291,290,286,289,291,294,296,295,296,303,306,307,302,296,289,287,286,286,286,286,287,289,290,292,293,298,302,306,307,304,308,309,310,312,314,315,315,316,316,315,314,314,313,312,311,311,311,309,307,305,304,305,305,305,305,305,306,307,307,307,307,307,306,306,306,305,305,305,304,304,304,305,299,298,298,298,298,297,296,295,294,293,293,292,291,290,290,289,288,288,287,287,286,286,285,284,284,283,282,281,280,279,280,281,282,284,285,285,286,286,286,286,286,286,286,287,288,289,291,291,291,289,284,277,275,275,280,281,283,286,288,290,292,295,290,282,281,280,281,274,272,273,273,274,274,277,281,282,284,285,284,287,289,291,293,292,294,292,284,282,280,282,287,291,301,316,327,332,339,333,325,323,320,323,333,323,315,312,306,301,295,291,280,267,261,256,255,255,254,253,252,252,251,504,491,491,500,510,512,512,508,511,519,522,527,548,573,590,602,599,584,596,617,627,633,638,647,659,665,658,646,644,650,657,660,665,666,664,653,638,632,627,620,607,601,619,632,629,614,600,597,587,586,576,560,562,572,574,578,576,565,551,536,525,522,531,541,550,558,562,564,554,540,526,505,484,466,448,433,424,421,416,408,401,393,383,377,388,393,394,394,401,403,396,389,393,393,385,382,388,391,393,386,377,373,361,356,350,339,329,328,325,327,328,332,332,333,326,314,313,312,313,314,316,318,318,319,319,321,323,324,326,327,329,332,337,339,341,348,353,350,340,329,323,317,315,317,314,318,322,326,330,334,339,339,341,341,340,339,340,338,330,322,317,312,307,304,302,300,298,296,295,293,292,291,290,290,286,287,288,290,292,292,294,300,301,304,300,295,289,286,285,285,286,286,288,289,291,294,298,303,307,311,311,310,308,311,311,313,314,316,317,317,317,316,314,313,312,312,311,310,310,308,306,305,304,304,304,304,304,305,306,307,307,307,307,307,306,306,306,305,305,305,303,302,304,304,299,299,299,299,298,297,296,294,294,293,293,292,291,290,289,289,288,287,286,286,285,285,284,283,283,282,281,280,279,280,280,281,282,283,282,282,282,284,285,285,286,287,287,288,290,291,292,292,292,290,284,275,275,275,280,282,283,288,292,292,291,291,289,282,280,281,282,277,273,273,273,274,275,278,281,282,282,281,288,289,290,292,292,293,295,290,280,278,277,278,280,285,293,303,308,312,318,323,314,305,301,302,316,316,313,302,295,291,286,278,270,264,259,256,255,254,254,253,252,252,251,512,507,498,508,518,525,521,520,522,535,543,540,552,576,599,616,621,605,608,628,639,644,648,655,660,658,650,641,639,646,649,656,660,660,656,646,624,612,610,602,589,587,608,612,609,600,586,575,572,569,564,554,557,564,569,572,574,563,547,536,522,520,528,539,549,556,557,555,543,527,512,496,478,463,446,430,417,413,409,403,397,391,378,376,387,392,389,391,397,393,388,380,386,384,378,378,386,392,393,388,381,377,371,362,357,345,337,331,329,337,343,346,342,344,336,323,314,310,310,311,317,318,320,320,320,322,323,325,326,328,331,336,339,344,352,357,357,352,342,331,321,315,315,317,325,331,330,329,329,332,335,337,337,339,340,343,340,334,327,322,317,313,305,304,302,300,298,297,296,295,292,291,290,288,287,286,285,286,288,289,292,295,297,300,299,297,291,287,285,285,285,286,288,289,292,296,303,307,310,311,312,312,310,310,312,314,315,317,317,317,317,316,314,312,312,311,311,310,309,306,305,304,304,302,303,303,305,305,305,306,307,307,307,306,306,306,305,305,305,305,304,303,302,302,301,300,299,298,297,296,295,294,293,293,293,292,291,290,289,288,287,286,286,285,284,284,283,282,282,282,281,280,280,280,281,282,282,282,281,280,280,282,284,285,287,288,289,290,292,292,293,296,297,293,290,277,275,275,277,280,280,286,291,292,292,287,284,280,281,280,281,276,273,273,274,274,275,278,280,280,280,282,288,290,292,292,293,293,295,288,279,276,275,277,277,279,282,286,286,290,300,312,307,294,281,282,293,300,305,300,287,280,277,269,265,261,257,256,255,254,254,253,252,252,251,503,518,512,513,527,539,533,533,535,547,560,564,560,581,601,624,634,621,625,637,647,650,648,650,649,643,638,633,633,636,644,651,655,652,647,640,623,602,585,582,573,569,590,595,589,582,575,561,557,553,552,547,555,563,565,566,564,560,548,535,518,516,532,541,550,550,551,547,535,513,498,483,468,455,441,429,419,410,401,397,393,387,380,376,380,385,381,390,392,388,377,374,380,379,373,375,382,388,392,392,389,382,378,368,360,350,344,332,336,350,359,359,353,348,337,325,315,305,306,312,317,319,320,320,320,321,323,326,327,329,334,340,347,353,354,350,346,346,341,330,319,314,315,321,332,332,331,330,334,336,340,342,341,344,347,350,344,339,332,328,321,315,306,303,302,300,298,296,295,294,294,294,292,286,285,286,281,284,285,286,288,291,292,300,299,296,291,287,285,285,285,286,288,291,295,301,307,311,313,312,312,312,311,310,311,313,315,316,317,317,316,314,312,311,311,311,310,309,308,307,305,305,301,300,300,304,304,304,304,306,306,306,306,306,306,305,305,305,305,305,304,302,301,300,299,298,298,297,296,294,294,293,292,292,292,292,290,289,288,287,286,286,285,284,284,283,282,281,281,281,281,279,279,280,280,281,282,282,281,280,281,283,286,287,288,289,291,292,294,295,302,306,306,301,293,277,276,275,275,278,281,285,288,291,291,289,286,282,280,280,277,274,273,273,274,274,276,278,279,279,282,286,291,292,291,290,291,292,293,289,281,276,273,273,273,274,275,276,276,279,292,291,284,279,272,266,271,271,288,289,282,275,271,266,262,258,256,255,255,254,253,253,252,252,251,511,524,525,523,540,551,550,549,553,559,578,585,582,594,612,631,636,631,635,645,647,642,637,636,636,634,627,625,630,635,643,648,648,645,640,633,620,602,580,562,554,558,577,581,570,566,562,553,542,542,539,537,551,557,558,561,553,550,545,528,509,521,533,539,539,538,536,537,528,507,487,470,455,444,435,427,419,412,400,390,389,382,379,373,377,377,378,382,382,377,368,365,376,373,366,368,378,384,390,392,388,382,377,368,360,352,342,338,342,357,368,369,360,348,337,326,316,304,306,312,317,318,319,320,319,320,322,324,328,331,338,346,354,358,350,339,334,336,336,327,316,312,317,325,337,332,330,329,330,332,338,342,346,350,354,356,355,350,341,332,322,313,304,303,302,300,298,294,294,294,293,293,292,290,289,288,286,285,285,285,286,288,293,298,295,292,289,287,285,285,285,286,288,292,297,305,310,312,312,312,312,312,312,312,311,314,315,316,316,315,314,313,312,311,310,310,310,308,307,306,306,305,302,298,300,301,303,303,305,305,306,306,305,305,305,304,304,304,304,303,302,300,300,299,298,298,296,296,294,293,293,293,292,292,291,290,290,288,287,286,285,284,284,283,283,282,281,280,280,280,280,280,279,279,280,280,281,280,280,280,280,283,287,288,289,290,292,296,300,302,305,312,310,304,296,278,280,277,275,278,280,281,280,288,292,290,291,284,281,279,277,274,271,274,275,276,275,278,279,280,285,291,294,290,290,289,287,290,293,288,280,272,269,270,269,270,268,267,266,274,278,272,258,256,256,251,260,256,267,269,269,269,266,262,259,256,255,255,254,254,253,252,252,251,251,523,534,542,535,552,565,564,565,575,580,592,607,607,616,627,636,637,634,640,640,636,628,624,623,622,623,618,622,627,632,640,644,642,638,635,624,607,588,573,556,541,548,556,560,556,552,549,544,534,533,533,534,544,547,549,551,539,538,539,528,517,516,525,527,527,525,524,528,527,510,492,473,454,438,428,416,412,406,400,390,385,378,375,368,368,369,373,375,374,369,364,365,369,367,363,364,373,382,388,391,388,381,374,364,356,350,342,340,347,360,371,366,355,345,336,327,316,304,304,308,315,317,317,318,318,318,318,324,328,333,341,350,357,356,351,341,328,328,329,322,314,314,319,328,335,332,330,325,325,328,329,334,343,349,357,360,356,356,346,334,325,319,314,306,302,299,296,294,293,293,293,293,292,293,294,296,293,288,285,285,286,288,293,295,291,289,287,286,285,285,285,286,289,294,300,304,307,311,310,311,312,312,312,313,314,315,315,315,315,314,313,312,311,310,309,309,308,307,306,305,305,305,299,299,300,299,301,304,305,306,306,306,305,305,304,303,302,302,301,301,299,299,299,298,297,296,296,294,293,292,292,292,292,291,290,290,289,288,286,285,284,282,282,282,282,282,281,280,280,280,279,279,278,278,278,279,280,279,279,279,280,282,285,287,289,291,294,301,306,314,318,318,312,302,294,276,281,276,275,277,278,279,280,287,292,294,292,284,282,280,277,273,269,270,275,276,276,277,280,282,287,292,293,293,289,285,284,283,290,283,277,270,266,266,265,264,261,259,257,265,262,256,256,256,252,254,255,258,260,256,264,262,260,257,256,255,255,254,254,253,253,252,252,251,251,532,549,561,551,560,579,582,585,601,601,607,618,629,638,647,643,637,632,634,634,629,622,613,610,608,609,610,618,627,635,644,648,641,632,623,620,608,587,565,552,537,528,532,542,541,537,535,533,524,526,522,526,536,535,542,541,529,527,532,528,509,508,504,511,515,514,517,522,525,517,499,478,465,446,432,418,408,398,394,390,384,379,372,365,363,363,366,367,367,366,359,359,362,358,360,368,378,383,385,384,382,378,375,368,362,355,353,349,355,363,367,360,352,343,334,326,316,306,304,305,310,314,316,316,316,316,318,324,328,334,342,352,355,349,342,340,331,321,318,316,310,313,319,329,331,333,329,322,317,314,318,328,334,340,353,355,343,348,346,344,337,331,319,313,307,300,295,293,294,293,293,294,292,292,293,294,296,291,286,285,286,287,291,292,290,287,286,285,284,285,285,286,289,293,300,302,305,307,307,310,312,312,312,313,315,316,316,316,313,312,311,311,310,309,308,308,307,306,306,304,303,300,299,298,299,299,301,304,306,305,305,305,305,304,303,302,301,300,299,299,298,298,298,297,296,295,294,294,293,292,292,291,291,291,290,289,288,287,286,284,282,281,282,282,282,282,282,281,280,279,279,278,278,277,278,278,278,278,278,279,280,280,283,286,288,292,297,306,312,318,320,315,305,296,288,280,281,275,275,275,275,279,284,290,294,293,290,283,281,279,274,271,267,269,273,274,275,275,278,281,285,288,291,292,290,282,281,281,280,278,274,268,264,262,258,257,256,257,258,258,256,256,255,254,253,253,255,256,257,259,258,256,256,256,255,255,254,254,253,253,252,252,251,251,250,551,562,576,578,573,596,603,612,621,622,619,629,635,639,633,626,621,614,620,622,616,614,610,599,596,596,609,622,628,635,639,634,629,622,611,609,608,595,577,560,543,522,515,528,528,525,527,526,517,516,510,518,524,531,537,530,522,516,522,525,515,502,497,499,499,507,512,520,517,515,496,473,457,441,427,414,413,402,389,388,381,380,374,368,365,360,359,360,361,358,354,354,353,354,363,372,380,381,379,376,374,371,368,369,367,367,362,358,364,367,361,353,346,340,332,324,315,308,304,305,308,312,314,314,315,316,319,322,327,332,341,352,357,349,339,331,332,324,314,307,306,313,319,326,326,328,328,319,313,313,316,320,318,334,343,344,332,335,344,355,350,342,330,324,315,301,295,292,292,292,292,293,292,291,293,294,293,291,286,285,286,287,290,292,290,288,286,285,284,284,285,286,289,293,298,300,302,304,304,307,310,311,310,311,316,317,317,314,312,311,310,310,308,307,306,306,306,305,305,304,301,300,298,297,299,300,300,301,303,304,305,305,305,304,302,301,300,299,298,299,297,297,297,296,294,294,293,293,293,292,291,291,291,290,290,289,288,287,286,282,281,280,280,281,281,281,281,282,281,279,278,277,276,276,276,277,277,277,278,278,279,280,283,286,290,293,301,310,319,321,314,306,297,290,285,280,280,275,275,275,275,280,287,293,298,292,284,283,281,278,273,270,267,269,271,273,273,273,273,276,280,281,282,281,279,278,274,270,268,270,269,268,261,257,257,256,256,256,256,256,256,255,255,254,253,253,254,255,256,256,255,255,255,255,254,254,254,253,253,252,252,251,251,250,250,572,574,592,598,600,608,618,634,636,629,629,634,633,625,616,612,607,606,609,612,606,600,600,595,585,596,610,615,621,625,626,624,617,609,598,591,593,590,577,559,542,523,509,514,516,518,522,517,509,513,499,508,516,527,525,516,513,508,507,510,504,495,492,495,501,513,518,523,519,506,488,467,451,442,433,426,417,406,395,387,377,376,373,365,363,357,353,353,355,354,350,349,351,355,364,370,376,378,375,368,366,364,363,363,368,376,375,367,367,361,353,346,341,335,329,321,314,307,304,305,307,310,312,313,314,316,316,320,327,334,342,351,358,354,341,328,322,320,315,307,307,312,321,323,320,318,320,315,311,310,313,315,316,327,332,328,323,328,339,357,356,347,333,322,309,299,293,294,292,290,290,293,291,290,293,294,293,288,285,285,286,287,290,293,293,288,286,285,284,284,284,286,290,294,294,297,298,301,304,306,306,308,309,311,315,317,316,312,311,310,309,308,307,305,305,305,304,304,305,304,300,300,297,293,297,300,298,298,300,302,303,304,304,303,301,300,299,298,298,298,298,296,296,294,293,292,292,292,293,292,292,291,291,291,290,290,289,288,286,284,282,281,280,280,281,280,281,281,280,278,277,276,275,274,275,275,276,277,278,278,279,280,284,288,291,296,305,318,330,321,314,304,296,291,284,282,280,275,275,278,278,283,289,294,296,293,288,283,281,280,276,271,267,269,271,272,272,272,268,270,273,275,274,271,268,268,267,267,266,266,263,264,257,256,256,256,256,256,256,256,256,255,254,254,253,253,254,255,255,255,254,255,254,254,254,253,253,253,252,252,251,251,250,250,250,592,599,610,619,621,621,635,644,642,641,636,630,616,609,606,600,596,600,602,603,601,591,584,580,582,592,601,602,607,612,614,613,607,598,587,574,575,574,568,554,536,518,502,501,505,514,515,507,496,504,494,500,510,516,512,506,504,500,493,491,489,486,487,494,503,509,514,518,524,511,488,474,461,450,440,430,420,411,403,396,388,382,378,369,366,359,353,351,350,350,346,342,343,351,357,364,369,371,370,366,360,355,354,357,366,371,368,363,358,352,345,339,335,330,324,318,312,306,304,305,308,309,311,312,314,315,319,324,331,337,344,349,351,353,345,334,320,312,308,304,307,312,320,320,316,311,311,310,309,309,311,313,313,316,318,316,324,329,344,363,362,344,327,315,304,303,303,298,295,291,289,288,288,288,289,293,289,286,285,285,285,286,288,289,288,290,286,284,284,283,284,286,286,287,292,294,297,300,302,304,304,306,308,310,312,311,311,310,309,309,308,307,306,305,304,304,304,304,305,305,300,299,296,292,293,297,297,297,299,300,301,301,301,300,300,299,298,298,298,297,296,296,295,294,293,292,292,292,292,292,292,292,291,291,291,291,290,289,287,285,284,282,281,282,281,281,280,279,279,278,276,274,273,273,273,274,275,276,277,279,280,280,286,288,291,300,311,323,329,321,316,307,298,292,283,281,280,275,275,280,280,286,292,291,290,285,282,284,282,280,280,272,267,268,270,272,273,272,270,267,269,270,268,267,266,266,266,265,264,263,261,260,256,256,256,256,256,256,256,256,255,255,254,254,253,253,254,254,254,254,254,254,254,253,253,253,252,252,252,251,251,250,250,250,250,594,600,607,614,626,640,644,645,641,636,628,617,607,599,600,594,588,593,597,594,592,587,574,571,574,583,588,591,595,599,602,602,595,581,573,551,551,554,553,542,527,513,500,495,504,512,513,499,488,494,487,496,504,507,500,497,498,489,482,480,483,484,485,492,500,506,513,517,524,516,496,482,469,455,444,432,422,413,405,404,394,388,380,373,366,360,354,351,348,344,342,341,341,346,352,356,363,365,368,366,362,353,348,352,362,367,362,355,351,345,339,330,330,325,320,315,309,304,304,305,307,308,310,312,314,316,319,323,328,333,340,343,338,341,344,337,322,311,304,304,306,310,318,317,305,308,308,310,311,313,313,313,313,316,320,328,333,337,345,360,363,347,330,321,312,307,304,307,302,294,290,288,288,287,288,289,288,286,285,285,285,285,285,285,285,285,285,284,283,282,283,284,286,287,291,294,296,299,300,301,304,306,307,309,309,309,309,308,308,307,307,306,306,305,304,305,305,305,305,304,301,298,295,292,293,294,293,296,297,298,299,298,299,299,300,299,298,297,296,294,295,295,295,293,293,292,292,292,292,292,292,292,292,292,292,292,292,290,288,286,285,284,282,282,282,281,280,279,277,276,274,273,271,270,271,272,274,275,277,279,280,281,287,289,291,300,311,316,319,313,305,302,296,288,280,280,280,275,280,281,282,286,290,286,280,282,282,280,282,280,279,270,267,268,270,273,274,275,274,270,268,267,267,266,264,264,263,263,262,261,258,256,256,256,256,256,256,256,256,256,255,255,255,254,254,253,253,253,253,253,253,253,253,253,252,252,252,251,251,251,250,250,250,250,250,579,585,595,603,614,625,634,637,630,621,612,606,598,591,592,590,584,584,589,589,582,578,567,557,562,571,578,581,583,587,586,585,584,568,552,541,529,531,534,529,518,506,493,486,501,510,508,495,479,483,488,495,500,499,491,488,487,479,475,478,484,487,487,491,498,511,510,515,522,518,498,477,461,449,439,426,416,411,400,391,386,381,379,373,367,362,355,351,349,344,342,340,339,341,345,350,354,360,365,368,366,356,341,346,356,362,358,351,346,342,334,326,321,319,316,313,306,304,303,304,306,307,309,310,313,309,317,320,320,324,331,334,329,331,335,337,329,311,304,303,305,307,309,308,308,308,310,314,318,320,320,316,311,315,317,322,329,329,340,352,358,354,341,329,320,308,306,305,304,296,292,290,289,287,287,290,287,286,285,284,284,283,283,283,283,283,283,283,282,281,281,283,286,287,290,293,295,297,298,299,303,306,306,306,307,307,307,307,306,306,306,305,305,305,304,305,305,305,305,304,300,297,294,292,292,292,292,294,296,296,296,295,295,298,299,298,297,295,293,293,294,295,295,294,293,292,292,292,293,294,294,294,293,294,294,293,292,290,289,288,286,285,284,283,282,281,280,278,276,275,273,271,269,269,270,271,273,274,276,278,278,280,285,288,290,292,305,316,313,306,298,291,292,284,280,279,280,275,280,281,283,284,287,283,282,280,279,280,280,280,274,268,267,268,270,274,275,274,272,269,267,266,265,264,262,262,262,261,260,257,256,256,256,256,256,256,256,256,256,256,256,255,255,254,254,253,253,253,253,252,252,252,252,252,252,252,251,251,250,250,250,250,250,250,250,565,572,580,594,605,614,623,627,620,607,597,599,593,584,583,584,579,580,585,586,578,571,560,547,549,562,570,571,574,576,570,566,566,558,539,529,520,511,517,520,511,500,488,475,493,501,500,488,472,476,487,494,500,490,482,481,477,472,473,482,489,496,493,493,499,503,508,514,517,518,498,477,463,448,436,428,426,417,407,397,387,379,376,371,364,361,356,354,350,346,344,342,339,338,339,342,348,354,361,367,366,355,341,343,352,356,354,348,342,337,331,323,317,315,314,311,307,304,303,303,305,307,308,308,308,304,311,316,314,317,324,330,326,325,326,328,324,309,303,303,304,306,306,306,307,308,310,316,328,331,329,322,315,315,316,314,315,323,328,345,362,358,346,332,322,313,305,304,300,300,295,292,291,289,287,287,286,286,285,284,284,282,282,281,281,281,281,281,282,280,280,282,284,286,289,292,293,293,295,298,301,304,304,305,305,306,306,305,305,304,304,304,305,305,305,305,305,305,305,305,302,299,294,292,291,292,292,294,294,295,294,294,293,293,296,297,296,295,294,292,293,294,295,295,295,293,292,293,295,296,297,296,297,297,297,295,292,291,289,289,288,286,285,283,282,281,279,277,276,274,272,270,268,268,269,270,272,274,275,276,278,280,284,286,291,293,301,308,311,305,298,292,285,283,281,279,279,275,280,280,281,282,285,283,281,280,277,279,280,278,270,268,267,268,268,271,272,272,270,268,266,265,264,262,261,261,260,259,259,258,257,256,256,256,256,256,256,256,256,256,256,256,256,255,254,254,253,253,253,252,252,252,252,252,251,251,251,250,250,250,250,250,250,250,250,552,569,577,585,595,606,620,633,618,597,584,588,589,582,573,575,571,574,576,579,576,562,550,544,548,558,561,561,565,566,553,542,545,545,536,523,511,503,501,508,506,498,484,473,483,491,491,481,472,477,483,488,492,485,474,479,471,468,476,484,492,500,497,497,498,499,505,511,512,517,507,485,465,450,441,438,432,423,412,404,397,391,388,379,369,364,356,352,349,346,344,342,339,335,336,338,343,349,357,366,363,352,340,341,345,349,349,344,338,333,327,319,315,314,312,311,309,305,303,303,304,306,307,307,306,304,307,312,313,315,319,323,320,317,320,322,314,307,303,303,304,305,305,305,305,306,309,314,322,335,339,330,323,323,326,322,318,325,335,348,364,359,343,332,325,318,311,305,299,294,294,296,291,289,287,288,287,286,285,285,284,282,281,280,279,279,279,280,281,280,280,282,286,287,288,289,291,292,293,298,300,301,301,303,304,304,304,304,303,303,304,304,305,305,305,305,306,306,306,306,303,299,294,292,291,292,292,293,294,294,293,293,292,293,295,295,294,294,293,292,292,294,295,296,296,295,293,296,297,299,299,300,300,300,299,297,294,292,291,290,290,287,285,284,282,281,279,277,275,273,271,269,267,267,268,270,271,273,275,276,278,280,285,289,294,301,309,318,309,300,291,285,279,280,279,279,279,275,280,279,280,280,282,282,280,279,279,275,270,270,268,267,267,267,267,268,270,269,268,266,265,264,263,262,261,260,258,258,257,257,257,256,256,256,256,256,256,256,256,256,256,256,257,255,255,254,254,254,254,253,252,252,251,251,251,250,250,250,250,250,250,250,250,250,250,565,579,590,597,600,607,616,625,616,596,582,580,583,577,572,566,562,562,564,569,573,563,547,536,542,548,548,551,556,554,541,524,528,532,530,520,507,497,491,497,498,492,483,471,468,480,480,466,466,478,486,487,487,481,473,470,462,464,475,482,488,495,501,500,500,500,503,508,514,521,513,486,471,461,453,446,437,428,417,408,400,394,392,384,374,365,357,352,349,344,342,340,337,334,331,333,338,344,351,355,357,356,342,338,340,342,343,340,336,332,327,317,317,316,313,314,311,305,303,303,304,306,306,306,306,304,305,312,317,317,317,318,317,315,320,318,312,304,303,303,303,306,306,305,305,305,307,310,315,329,341,337,332,335,336,332,326,327,331,342,347,351,343,330,322,320,315,306,299,294,294,294,292,288,286,286,286,286,285,286,285,286,282,280,278,278,278,279,279,280,280,280,280,285,285,288,292,294,295,298,299,298,299,301,302,302,301,301,301,302,303,304,305,304,305,305,306,306,306,307,303,298,294,292,291,291,292,292,293,293,293,293,293,293,294,294,293,293,292,292,292,293,295,296,297,297,297,299,300,302,302,304,304,304,302,300,297,294,293,292,291,288,285,284,282,281,280,277,274,271,270,269,267,267,268,269,271,272,274,276,279,283,289,292,303,315,319,313,306,300,292,285,280,282,281,278,278,275,278,280,280,281,281,280,280,279,277,272,268,267,267,267,266,266,267,267,268,267,266,265,264,263,262,260,259,258,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,255,255,255,255,255,254,253,251,250,250,250,250,250,250,250,250,250,250,250,250,250,574,587,600,609,606,609,614,619,611,592,575,571,574,574,569,559,554,547,551,557,560,559,549,536,536,536,541,545,550,541,528,516,511,518,520,516,504,491,483,484,488,486,479,469,460,464,462,459,465,476,488,491,482,476,471,465,461,450,466,476,483,492,499,500,502,502,502,505,515,524,518,494,476,461,449,439,426,420,415,405,398,392,389,382,374,365,358,352,348,343,340,338,334,333,330,328,332,338,342,346,350,350,340,336,336,338,339,340,342,345,344,338,328,318,319,318,308,305,303,303,304,306,306,306,306,306,308,313,317,317,315,316,317,318,326,322,312,306,303,303,303,303,306,307,307,307,309,312,317,329,333,330,336,343,344,340,333,327,325,331,334,339,342,329,320,313,315,304,300,296,290,291,289,287,285,285,285,286,285,285,286,286,288,282,279,278,278,278,279,280,281,280,282,283,285,288,292,294,293,293,297,298,299,301,300,300,299,299,299,301,302,303,304,305,305,306,306,306,306,307,304,298,295,292,291,291,291,292,292,292,292,293,293,293,293,292,292,292,292,292,292,293,294,296,298,300,301,303,303,304,306,309,314,313,305,302,300,297,296,293,291,289,284,282,282,280,279,277,275,272,268,268,267,267,267,269,271,271,274,278,283,289,293,302,314,322,316,304,299,297,295,292,290,290,289,282,275,275,275,278,280,280,280,280,280,278,277,270,267,267,266,265,265,265,265,265,265,265,264,264,263,262,260,259,258,258,256,257,256,256,256,256,257,256,256,256,256,256,256,256,256,256,256,256,256,255,255,256,256,256,252,250,250,250,250,250,250,250,250,250,250,250,250,250,250,582,595,605,608,603,603,605,606,600,586,573,563,564,562,562,557,546,538,542,546,546,545,541,536,532,533,541,539,538,537,522,510,497,502,505,512,503,489,476,475,478,478,475,462,456,457,452,453,460,470,479,486,480,467,464,462,458,448,459,471,481,490,496,502,504,505,505,505,515,527,525,499,476,462,449,434,417,409,404,397,391,384,379,373,370,365,359,352,351,345,341,338,340,339,333,328,328,329,332,336,340,339,334,331,336,339,341,346,350,354,354,344,333,327,330,322,307,304,303,303,303,306,306,306,305,306,308,314,317,315,314,316,317,323,330,324,314,307,301,300,300,302,306,309,310,310,310,316,319,323,323,319,324,329,335,343,336,325,317,316,322,328,337,328,316,309,305,303,300,298,292,286,286,286,285,284,284,283,281,284,287,287,290,284,280,279,278,278,279,280,281,280,282,284,286,288,291,292,292,292,295,297,299,300,300,298,297,298,299,300,302,303,304,305,305,306,306,306,306,306,304,299,295,292,291,291,291,291,292,292,293,292,292,292,292,292,292,291,291,292,292,292,294,296,299,302,304,304,306,309,312,318,326,323,314,306,302,300,296,292,291,288,285,281,280,278,278,278,277,270,269,267,267,267,267,268,270,268,275,280,288,298,310,318,320,318,310,299,290,292,297,296,295,293,293,286,279,270,270,274,277,277,277,277,278,280,273,268,267,266,265,264,264,264,264,264,264,263,263,262,261,260,258,257,257,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,255,255,256,256,256,256,252,250,250,249,250,250,250,250,250,250,250,250,250,250,250,590,599,596,589,590,591,593,595,589,580,571,559,552,551,548,548,541,531,532,532,534,533,531,528,528,535,538,533,524,528,520,504,491,485,488,502,501,490,478,464,463,465,462,455,449,453,447,450,456,464,471,476,472,462,458,456,452,445,457,470,478,489,497,502,505,511,509,510,514,526,527,507,481,464,448,437,428,415,402,389,384,376,377,369,364,362,366,363,358,351,341,343,349,347,343,336,326,324,326,327,329,331,328,330,334,338,344,352,356,360,358,352,344,341,335,325,308,305,303,303,303,304,305,306,304,305,306,311,313,310,313,316,321,326,329,322,313,306,299,297,297,300,307,311,311,312,306,308,310,312,311,309,314,316,329,342,334,320,312,314,320,324,330,323,312,301,299,297,305,296,290,286,286,286,284,283,282,281,281,282,284,288,288,287,282,279,279,278,279,280,280,280,281,284,287,288,289,291,292,293,295,297,299,300,299,296,295,297,299,300,301,302,304,305,305,305,305,305,305,305,304,298,294,292,290,290,290,290,290,291,292,291,292,292,292,291,291,291,291,291,292,292,294,296,298,302,304,307,310,312,316,330,336,333,323,314,306,302,298,294,291,290,286,282,281,281,281,280,276,270,268,266,267,266,266,267,268,267,279,285,294,304,317,315,312,308,301,294,284,288,294,296,296,294,292,285,279,271,268,271,272,272,273,273,274,273,268,267,266,265,264,263,263,263,263,262,262,262,261,260,258,257,256,256,255,255,255,255,255,255,256,256,256,256,256,256,256,256,256,256,256,256,255,255,255,255,256,256,256,254,252,250,250,250,250,251,251,251,251,251,251,251,250,250,250,586,584,578,575,576,576,583,587,583,569,561,554,546,542,539,537,536,527,521,522,523,523,523,524,529,534,537,527,515,515,517,506,489,474,481,492,495,490,480,468,457,454,454,450,446,447,442,439,453,459,463,464,461,456,452,449,446,448,459,469,476,488,500,499,509,511,515,514,511,520,529,515,489,468,449,434,424,411,401,392,378,377,375,374,370,367,375,373,365,353,345,352,359,358,351,343,331,329,326,327,327,328,328,333,338,342,349,356,360,366,364,355,346,343,336,328,313,304,303,303,303,303,305,305,304,304,305,305,304,308,313,317,320,321,323,316,313,305,298,294,296,301,307,318,309,303,304,304,305,306,307,308,311,318,333,338,332,318,308,305,313,315,321,317,308,301,292,294,293,292,288,285,285,285,284,283,282,280,280,280,285,289,288,287,282,280,279,278,278,280,280,280,280,283,285,287,288,290,292,292,294,297,299,299,297,294,292,294,298,299,300,302,303,305,305,305,305,304,304,303,300,298,294,293,291,289,289,289,289,290,291,290,290,290,290,290,290,290,291,291,291,292,294,296,298,300,304,307,312,316,320,338,345,343,330,320,309,303,299,294,292,290,286,283,282,282,280,279,276,270,269,268,266,266,266,266,266,267,278,286,295,304,308,305,302,303,294,285,278,284,293,296,296,293,290,280,277,270,267,269,271,268,268,270,270,270,267,266,264,263,262,262,262,262,262,262,261,261,260,257,256,255,255,255,255,255,255,255,255,255,255,255,255,256,256,256,256,256,256,256,256,256,255,255,255,255,256,256,256,255,253,252,252,252,252,251,252,252,252,251,251,251,251,251,250,572,574,562,563,560,566,571,573,572,562,548,544,538,534,532,528,525,520,514,515,520,524,529,526,526,528,531,521,509,500,510,506,492,473,472,483,490,483,477,470,459,450,448,443,438,442,440,438,452,451,452,450,448,448,441,440,442,448,457,466,475,484,489,498,507,512,517,519,516,518,524,521,495,471,452,437,429,421,409,394,392,385,388,387,383,382,390,378,362,353,349,356,365,366,355,346,343,342,332,326,324,326,328,334,342,348,354,357,357,359,363,352,341,336,331,322,313,304,303,303,303,303,303,304,303,303,303,303,304,307,313,310,310,317,320,315,305,302,297,293,296,304,311,312,306,303,302,302,303,305,309,309,312,318,323,323,329,325,312,302,306,310,314,316,319,312,301,290,289,288,287,286,286,286,285,285,284,281,279,279,287,288,286,285,281,281,280,280,278,280,278,278,280,280,281,286,288,290,291,292,294,297,298,297,294,292,292,293,296,299,300,301,303,304,305,305,304,303,302,300,299,295,292,291,291,289,287,288,288,289,289,288,289,289,290,290,290,290,291,291,292,292,294,296,298,301,304,308,314,320,335,346,354,351,338,323,313,305,300,295,292,289,287,285,281,280,278,275,273,270,269,268,267,266,265,265,264,267,275,284,293,302,299,295,295,297,292,283,278,282,293,298,296,294,293,287,280,274,268,268,270,268,267,267,268,267,266,264,263,262,262,262,262,262,261,261,261,260,258,256,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,256,256,256,256,256,256,255,255,255,255,255,256,256,256,256,255,255,254,254,253,253,253,252,252,252,252,251,251,251,250,559,563,552,548,548,552,557,559,556,552,544,535,530,526,525,523,516,510,510,516,524,530,528,527,521,522,526,521,507,496,498,500,492,474,463,476,488,482,468,463,456,453,445,438,437,437,430,436,446,441,439,436,435,435,435,437,439,447,456,469,479,484,491,502,509,515,518,523,525,522,525,525,500,475,461,452,442,430,416,404,402,399,402,399,392,392,386,374,360,351,354,361,368,365,355,351,354,348,338,328,324,325,328,334,342,349,355,353,350,351,355,355,345,337,326,316,311,304,305,303,303,302,302,302,302,302,302,303,306,309,313,308,307,312,314,309,301,296,294,294,296,301,307,304,303,302,303,305,308,310,317,311,314,318,320,311,316,319,314,303,301,304,305,315,320,314,303,292,288,288,288,289,289,287,285,288,288,284,280,279,287,286,285,284,281,282,282,281,278,278,278,281,280,280,283,286,288,290,291,292,294,295,295,294,292,292,292,292,295,298,299,300,302,303,304,303,302,302,301,300,299,293,291,290,288,287,287,287,288,288,288,287,287,287,289,289,290,290,291,292,292,293,294,297,299,302,305,310,315,325,341,354,362,357,344,328,315,304,300,296,292,290,288,285,282,281,278,275,270,268,267,267,266,265,264,264,264,267,273,282,293,294,287,284,292,293,284,279,277,283,292,297,296,295,295,293,285,279,271,269,268,268,267,266,266,265,264,262,262,262,261,261,261,261,261,260,259,258,257,256,256,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,256,256,257,258,258,257,257,257,255,255,254,254,253,253,252,252,251,251,251,251,547,552,552,540,537,542,540,541,545,544,537,530,522,517,516,516,513,502,508,522,530,536,528,521,511,515,518,515,506,492,488,490,485,473,459,469,480,479,468,455,448,445,441,435,428,428,426,437,436,432,427,427,428,431,435,440,452,455,454,462,474,481,490,501,512,510,524,524,527,526,527,524,510,487,471,458,445,432,420,414,412,411,403,396,393,388,379,369,356,353,361,367,374,365,362,358,358,351,339,330,324,324,326,331,338,345,345,340,340,343,353,359,354,345,332,321,316,314,308,306,303,301,301,300,301,302,303,305,308,312,312,307,305,305,304,301,296,293,292,293,294,298,299,298,299,301,303,314,315,320,318,310,306,316,316,304,306,313,314,303,293,294,305,312,309,302,296,291,290,290,291,292,291,289,292,290,290,287,281,280,279,284,286,287,286,285,283,282,278,278,281,282,284,284,285,287,288,290,291,292,294,293,293,292,292,291,292,292,295,297,299,300,301,301,302,301,301,300,299,299,297,293,293,289,287,287,287,287,288,287,287,286,287,287,288,289,290,291,291,292,292,294,295,297,298,302,306,311,316,328,344,359,368,364,348,331,318,305,301,297,294,291,288,285,283,281,279,277,272,269,269,268,266,265,264,264,264,266,271,278,282,280,285,288,286,284,279,277,278,283,292,296,296,295,293,289,285,280,273,269,267,266,266,265,264,264,263,262,262,261,261,261,261,260,260,259,259,257,257,256,256,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,256,255,256,256,257,259,260,261,260,260,259,256,255,255,254,254,253,252,251,251,251,251,250,538,542,545,537,531,532,532,534,537,540,532,524,516,510,511,509,503,492,504,523,537,536,528,520,509,499,503,504,499,487,483,482,476,464,452,461,473,476,461,453,443,436,434,428,424,423,426,435,429,425,421,423,428,435,441,449,456,464,467,468,472,478,488,497,502,512,522,526,529,528,523,518,514,501,479,460,445,435,428,424,423,413,401,389,382,376,370,364,360,362,371,379,379,375,369,367,364,354,342,331,322,320,323,328,333,340,338,338,342,347,352,354,354,348,340,330,318,316,314,307,303,300,298,297,299,301,304,305,308,313,317,310,305,303,305,299,294,292,292,292,293,293,295,294,295,298,302,305,314,315,310,304,304,311,312,303,297,303,307,305,293,291,300,298,293,295,293,291,291,293,295,293,292,293,293,292,291,287,281,279,279,282,286,287,287,286,285,281,278,280,281,286,287,288,287,287,287,289,292,293,293,291,290,290,290,291,292,293,295,297,299,299,299,300,300,300,299,298,298,296,294,293,291,288,286,286,287,287,287,287,286,286,287,288,289,289,290,291,292,292,293,294,296,297,299,302,307,312,316,331,347,362,372,366,350,332,319,307,301,297,293,291,288,286,284,282,281,279,274,271,270,268,268,265,264,263,263,265,268,268,276,278,282,285,281,278,276,277,278,284,292,295,294,290,286,282,281,278,274,272,269,267,265,264,264,263,263,262,261,261,261,260,260,260,260,259,258,257,256,256,256,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,256,256,256,256,257,259,261,262,264,264,264,262,259,256,255,255,254,252,252,251,251,250,250,250,529,534,538,531,523,523,524,526,531,536,526,517,510,507,500,497,488,484,506,524,527,521,516,513,513,500,489,490,489,481,474,477,471,462,449,452,464,466,453,440,437,428,425,423,419,417,425,429,424,420,420,425,431,441,446,449,456,462,467,476,484,487,487,492,500,508,513,521,524,519,516,512,509,502,484,467,451,441,438,434,423,412,402,386,375,366,367,371,374,374,381,389,387,379,374,369,363,354,343,331,319,318,320,325,329,328,330,338,347,355,351,341,339,334,333,330,322,316,315,306,301,296,294,293,297,299,303,304,306,311,316,314,305,302,305,297,294,293,292,292,292,292,292,292,293,295,296,299,305,307,303,296,299,307,308,302,292,294,305,301,293,290,291,294,294,293,293,293,293,294,295,295,293,294,294,294,290,286,281,278,279,281,283,286,287,286,282,280,278,280,281,284,288,286,287,287,286,287,292,292,290,288,287,288,290,292,292,294,295,297,298,299,299,299,299,299,297,295,295,294,293,291,289,287,286,286,286,287,287,286,286,286,287,288,289,290,290,291,292,293,294,295,296,297,299,302,307,312,316,331,346,361,371,368,354,337,322,308,301,295,292,290,288,284,283,282,281,278,273,269,268,266,266,266,263,262,263,264,268,280,279,281,282,282,275,268,272,275,278,283,290,293,290,286,284,280,276,274,272,270,267,265,263,263,262,262,262,262,261,261,260,260,260,260,259,258,258,257,256,256,256,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,256,256,257,259,261,263,265,266,267,268,266,263,258,256,255,254,252,251,250,250,250,250,250,520,525,526,524,515,512,515,521,527,527,521,512,508,507,503,485,474,478,497,512,516,513,500,500,508,502,487,478,474,474,466,466,461,454,443,446,455,453,448,440,427,423,414,415,416,414,431,431,420,418,422,429,440,445,451,453,454,460,465,477,485,495,501,494,498,501,506,510,512,505,503,502,499,499,489,476,459,452,452,437,422,410,401,384,374,376,380,383,386,386,388,392,386,379,370,362,354,345,335,326,316,315,317,322,326,329,334,340,351,358,354,343,332,327,321,320,320,314,307,303,300,296,293,292,295,294,297,301,304,308,311,308,304,301,297,295,296,297,296,294,292,292,292,292,292,292,292,289,298,300,295,291,297,302,305,299,291,294,296,293,287,288,291,293,294,294,295,296,294,296,297,297,296,295,294,292,290,284,280,278,279,279,280,284,285,285,282,280,278,280,281,283,287,287,284,284,285,289,288,289,288,286,286,288,289,291,293,294,295,297,298,299,299,298,298,298,295,294,293,292,292,289,287,286,286,286,286,286,287,287,286,287,287,288,289,290,290,292,292,293,294,294,295,297,298,301,304,309,315,329,344,358,368,369,356,338,322,309,301,294,291,290,287,283,280,279,278,278,275,272,272,271,271,268,265,263,262,266,274,284,286,285,283,278,269,268,271,275,278,281,283,283,279,283,280,277,273,270,266,267,264,263,262,262,262,262,262,261,261,260,260,260,260,259,259,258,258,257,256,256,256,256,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,256,258,260,262,264,265,267,270,275,276,271,267,262,257,255,252,252,250,250,249,250,250,250,510,514,512,511,506,509,517,524,528,524,512,512,510,513,507,490,467,473,485,500,504,504,499,489,496,498,489,479,467,462,458,457,451,446,437,440,448,444,438,436,427,420,412,414,412,414,421,418,418,419,425,433,439,446,452,454,456,459,463,472,479,485,490,493,495,498,501,501,502,494,491,488,484,488,486,478,470,465,451,433,418,405,395,383,374,380,391,392,394,390,396,396,386,376,363,353,346,340,332,324,316,314,317,321,325,329,335,341,352,363,364,354,345,337,327,316,314,310,305,302,304,296,292,292,294,293,295,299,304,305,306,305,305,300,296,294,297,306,305,297,294,292,292,293,290,289,287,287,292,294,290,292,292,292,292,294,287,286,287,288,290,292,294,295,296,297,297,297,298,299,302,299,297,295,294,295,288,284,280,278,278,278,279,281,281,281,280,279,278,279,281,282,286,279,279,282,288,287,286,286,286,285,287,288,290,292,293,294,294,297,298,299,298,297,297,296,295,293,292,291,290,288,287,286,285,285,285,286,286,286,286,287,288,288,289,290,290,291,292,292,293,294,294,297,299,301,304,308,313,317,331,354,365,368,356,338,322,308,301,296,292,290,288,282,281,280,278,278,278,278,279,279,278,272,266,264,261,261,274,281,282,286,282,272,267,268,270,271,274,278,278,274,273,273,271,271,269,265,263,262,262,262,262,262,261,261,261,261,260,260,260,260,259,259,259,258,258,257,256,256,256,256,256,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,256,258,260,262,264,266,268,273,280,283,283,279,272,267,261,256,254,250,250,249,249,249,250,250,496,498,498,498,501,513,519,525,525,519,512,510,511,507,500,484,464,465,479,492,492,494,491,486,484,490,486,474,464,455,449,448,443,439,429,432,440,438,431,425,422,416,410,407,401,401,407,413,416,420,426,438,437,441,446,450,453,455,461,466,473,479,484,488,492,493,490,491,491,488,478,476,474,478,482,478,478,466,449,430,411,399,390,378,377,386,398,402,401,402,404,398,387,374,362,354,346,342,336,330,319,314,317,320,324,328,336,348,358,366,370,368,358,349,340,326,316,304,307,309,307,299,294,292,293,293,293,296,300,302,302,305,305,300,296,294,299,305,307,301,296,293,292,292,290,286,284,284,284,285,285,285,287,287,287,286,285,284,284,289,294,297,301,299,299,298,298,298,299,302,304,303,298,294,292,290,286,286,283,280,279,278,278,278,278,279,279,278,278,279,280,281,279,279,280,284,287,286,284,284,284,285,287,289,290,292,293,293,294,296,297,298,297,296,295,295,295,294,292,290,288,286,286,286,285,284,285,285,286,286,287,287,288,288,289,289,290,291,292,292,292,292,292,296,298,300,303,307,311,317,333,349,359,365,354,337,321,306,300,296,292,290,286,284,281,279,280,281,282,284,285,287,286,282,275,267,261,261,270,278,282,282,278,268,266,268,268,272,272,272,270,267,268,265,266,266,264,262,262,262,261,262,261,261,261,261,261,260,260,260,260,260,259,259,259,258,258,257,257,256,256,256,256,256,256,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,256,258,260,262,265,267,271,276,281,283,283,283,279,274,270,265,258,252,251,250,249,249,249,249,249,486,489,495,498,504,511,520,520,516,510,502,509,512,508,489,475,457,461,472,478,482,482,478,474,480,483,482,478,469,456,442,437,436,432,427,426,434,431,421,414,416,410,402,401,400,403,406,411,416,422,430,442,447,447,446,448,449,451,457,463,469,472,475,482,489,490,486,482,481,478,467,464,472,480,476,473,471,464,449,430,414,399,387,390,393,399,403,407,408,408,402,394,384,372,362,353,355,349,344,335,322,314,316,320,325,329,338,348,355,362,364,360,358,357,352,340,326,319,317,315,307,301,295,292,292,292,292,295,297,299,305,307,305,298,295,294,297,302,305,301,297,293,293,292,290,288,285,284,283,283,283,282,283,283,284,284,283,282,286,291,304,305,301,303,301,300,298,299,300,302,305,303,300,296,293,291,294,293,287,284,283,282,281,281,278,278,278,278,278,279,279,279,279,280,282,280,280,282,282,281,281,286,288,289,291,292,293,292,294,295,295,296,295,295,294,293,293,293,292,290,288,286,286,287,284,284,284,285,286,287,287,287,287,288,288,289,290,291,292,291,292,292,293,295,298,300,302,304,309,315,329,342,354,359,353,335,320,304,299,295,292,290,287,284,281,282,282,283,284,287,291,294,293,287,282,270,261,261,268,277,281,277,273,266,264,264,267,270,270,269,266,261,268,261,262,262,262,262,262,261,261,261,261,261,261,260,260,260,260,260,259,259,259,259,259,258,258,257,257,256,256,256,256,256,256,256,256,255,255,255,255,255,255,255,255,255,255,255,255,255,257,260,262,265,267,271,278,282,283,283,282,282,278,272,268,265,260,252,251,250,249,249,249,249,249,495,500,500,505,511,515,514,512,504,499,506,513,512,496,479,465,451,454,464,468,472,471,468,470,474,474,477,474,468,461,446,430,427,427,419,421,427,420,417,408,406,404,397,396,399,403,406,410,417,425,434,444,451,449,449,450,452,452,455,463,467,466,471,478,488,491,485,475,473,470,463,464,476,478,477,472,465,464,450,433,415,399,398,404,406,411,412,413,413,415,400,388,380,373,367,363,359,353,348,335,320,314,316,320,325,331,336,340,346,357,351,340,341,352,356,344,336,334,330,321,312,304,297,292,292,292,292,293,293,303,307,302,300,296,294,293,296,300,303,300,293,292,292,292,291,293,290,286,287,284,283,283,282,282,282,282,282,283,286,295,299,302,304,304,304,298,297,298,299,302,303,304,300,299,297,297,296,294,291,289,288,287,286,286,285,281,279,278,278,279,279,279,279,279,279,279,280,280,280,280,282,286,288,290,291,292,293,293,294,294,294,294,294,294,293,292,292,292,291,289,288,287,285,283,282,282,283,284,286,287,287,287,287,287,287,289,290,291,292,292,292,293,294,296,297,298,300,303,307,312,320,333,346,353,345,330,318,304,298,293,291,289,287,285,283,282,282,283,285,287,292,295,291,284,278,270,262,261,266,272,274,268,272,268,262,262,265,268,265,262,261,260,263,261,261,260,261,260,260,260,260,260,260,260,260,260,260,260,260,259,259,259,259,259,258,258,258,257,257,257,256,256,256,256,256,256,256,256,255,255,255,255,255,255,255,255,255,255,255,256,258,261,264,266,270,276,281,282,282,282,282,281,275,270,267,264,261,253,252,251,250,249,249,249,249,498,504,508,510,514,514,507,499,496,501,510,514,500,485,468,457,447,448,455,459,463,461,461,467,465,467,468,465,454,453,444,431,423,420,414,415,422,417,409,400,395,392,393,395,400,405,408,410,414,426,435,442,447,451,451,451,453,455,458,462,464,462,466,474,482,488,483,470,463,462,458,465,473,475,478,475,461,454,443,429,410,399,406,413,417,422,424,423,424,422,406,394,385,382,376,370,365,359,349,336,321,314,316,318,322,328,329,335,344,349,349,343,335,340,346,337,329,330,332,328,317,309,300,293,292,292,292,293,294,293,301,301,293,294,293,292,293,297,300,304,295,292,292,291,292,292,291,289,288,286,285,283,282,281,281,282,282,283,287,292,293,299,299,296,294,293,295,296,297,298,299,299,300,300,304,302,300,298,292,294,295,294,291,290,288,283,279,278,278,279,279,279,278,278,278,279,279,279,280,282,285,287,289,292,292,293,293,292,293,293,293,293,293,293,292,291,291,291,289,288,287,286,284,282,280,281,282,284,285,285,286,286,287,287,287,288,290,291,292,293,293,294,295,296,297,298,299,302,305,309,315,324,337,347,342,325,316,302,294,292,291,290,287,285,284,282,282,282,284,286,290,290,285,280,273,266,261,261,264,269,269,267,268,270,265,262,261,258,258,258,258,258,258,258,258,259,259,259,260,260,260,260,260,260,260,260,260,259,259,259,259,259,259,258,258,258,258,257,257,257,256,256,256,256,256,256,256,256,255,255,255,255,255,255,255,255,255,255,255,256,259,262,265,268,271,278,282,282,281,281,281,279,271,267,266,264,262,254,253,252,251,250,249,249,249,504,508,512,514,510,504,498,494,498,506,515,515,502,488,471,456,440,438,443,449,450,450,458,464,461,459,457,454,447,446,440,425,415,416,415,413,416,406,399,393,387,390,392,396,405,408,415,416,418,425,432,436,442,449,447,449,452,454,456,459,459,462,463,472,476,484,479,468,458,450,451,462,470,476,475,468,457,444,435,421,402,404,417,422,424,430,434,436,436,426,410,398,392,391,381,373,368,358,350,339,326,314,315,316,319,323,327,333,337,337,338,335,327,333,339,334,327,319,319,316,311,306,301,295,292,292,292,293,293,294,294,295,295,292,292,292,292,293,294,297,294,292,291,291,292,292,291,291,290,289,286,284,282,281,281,281,282,284,287,291,294,296,295,293,292,292,293,295,296,296,297,298,299,302,304,305,303,301,298,296,295,295,296,294,290,281,279,278,279,279,279,278,278,277,277,278,279,280,282,284,287,289,291,292,293,293,291,293,292,292,293,292,292,292,292,291,290,289,288,287,286,285,283,281,280,280,281,283,286,286,287,287,286,286,287,288,291,292,293,293,294,295,296,297,298,298,298,300,303,306,311,317,330,342,330,321,312,300,292,290,290,289,287,285,283,281,280,280,281,282,286,283,278,277,272,268,261,261,263,266,266,262,263,267,263,262,261,260,258,258,258,258,258,258,258,258,258,258,259,259,259,259,260,260,260,259,259,259,259,259,258,258,258,258,258,258,258,257,257,256,256,256,256,256,256,256,256,256,256,255,255,255,255,255,255,255,255,255,255,256,260,264,267,270,273,279,280,281,281,279,277,278,272,269,267,266,263,261,254,252,251,250,249,248,248,514,513,512,505,497,493,489,497,503,510,516,507,490,478,466,454,439,427,432,436,437,443,452,454,450,447,441,439,436,441,437,427,410,408,404,405,404,394,389,387,387,390,396,402,401,410,413,419,425,432,439,441,441,445,447,449,449,450,452,458,460,462,465,471,476,480,477,470,465,454,450,460,472,478,477,464,451,439,428,416,412,418,429,432,434,438,442,443,440,425,416,407,399,393,383,373,365,356,347,336,325,312,314,314,316,320,324,327,329,327,328,327,333,339,342,343,336,320,313,308,305,304,301,296,292,292,292,292,293,292,292,292,292,292,291,291,292,292,292,292,292,292,291,291,292,292,293,293,293,291,288,286,282,281,281,281,282,284,286,290,293,294,293,292,291,291,292,293,294,295,296,296,296,299,301,303,304,305,302,298,295,294,296,294,287,280,279,278,278,278,279,278,277,276,276,278,279,280,282,285,287,289,292,293,292,292,292,293,293,293,292,291,291,291,290,290,289,288,287,286,286,284,282,281,280,280,282,286,287,288,286,286,286,286,288,290,291,292,293,294,295,296,297,298,299,299,298,298,301,303,307,313,319,328,317,315,307,298,292,290,289,288,287,285,283,281,279,277,278,271,272,268,266,266,265,261,261,261,262,263,262,261,261,261,261,261,261,260,258,258,258,258,258,258,258,258,258,258,258,259,259,259,259,259,259,259,259,259,259,258,258,258,258,258,258,258,257,257,256,256,256,256,256,256,256,256,256,256,256,256,256,256,255,255,255,255,255,255,256,258,262,265,268,272,275,277,279,279,279,278,278,277,276,272,270,268,264,261,254,252,250,249,248,248,248,509,502,497,492,485,483,487,501,510,518,509,501,491,474,458,448,433,421,418,423,427,437,441,443,438,433,425,424,423,433,430,418,402,400,393,393,388,382,383,383,386,393,407,409,401,410,417,421,427,437,442,446,443,446,447,448,449,451,453,461,462,465,468,473,476,478,474,468,463,454,448,458,467,474,477,466,451,438,426,424,424,432,439,444,446,448,449,448,439,426,424,414,402,393,380,368,360,351,340,335,329,316,313,311,314,318,319,318,322,327,330,336,343,344,337,340,342,335,329,321,314,306,303,294,293,292,292,292,292,292,291,291,291,291,291,291,291,291,291,291,290,290,290,290,291,293,293,293,292,292,290,287,283,281,281,281,281,283,285,286,290,289,291,290,290,290,292,293,293,293,293,292,294,297,299,302,304,305,301,293,292,293,292,289,285,280,279,279,279,279,278,277,276,275,275,277,280,282,284,285,287,289,289,286,288,292,293,293,293,292,290,290,289,289,289,289,288,287,286,285,285,283,282,280,280,281,283,285,288,287,286,286,286,287,290,291,292,293,294,294,295,296,298,298,299,298,298,297,299,301,304,309,312,314,313,311,305,296,291,289,288,288,287,286,282,280,279,280,281,274,267,268,266,264,262,261,261,261,261,261,261,261,261,261,261,261,261,261,258,258,258,258,258,258,258,258,258,258,258,258,259,259,259,259,259,259,259,259,258,258,258,258,258,258,258,257,257,257,256,256,256,256,256,256,256,256,256,256,256,256,256,256,255,255,255,255,255,256,257,259,264,266,268,273,276,277,277,278,279,278,278,278,279,275,272,269,265,262,253,251,249,248,248,248,248,496,492,485,478,474,481,494,506,517,520,500,486,483,475,462,452,439,427,412,413,422,427,429,431,427,422,413,411,417,428,422,409,396,386,379,382,381,381,381,382,386,394,407,407,403,410,416,424,431,440,445,452,451,448,448,449,450,451,456,460,464,466,469,472,475,477,470,463,456,447,438,452,459,466,471,467,454,441,437,434,435,440,448,453,456,458,458,451,442,433,427,414,403,393,380,370,362,353,346,342,332,319,307,307,310,314,315,319,325,334,341,346,354,348,329,328,334,337,335,328,321,317,308,300,294,292,292,293,293,293,292,292,291,291,291,291,291,291,291,291,290,289,288,288,289,292,292,293,294,293,293,288,284,281,280,280,281,282,283,284,284,285,284,287,288,290,291,292,293,292,292,292,294,295,297,299,302,304,303,296,291,289,282,281,280,281,280,280,280,280,279,277,276,275,275,276,280,282,282,284,286,286,285,286,290,293,293,292,290,289,288,288,288,288,288,288,287,287,286,285,284,282,281,280,280,282,283,286,293,286,285,286,286,288,290,291,293,294,294,294,294,295,297,298,299,299,297,292,296,298,302,305,308,309,310,308,302,293,291,289,288,288,288,289,290,283,283,284,282,281,279,280,272,268,268,261,262,261,261,261,261,261,261,261,261,262,261,261,259,258,258,258,258,258,258,258,258,258,258,258,259,259,259,259,259,258,258,258,258,258,258,258,258,258,257,257,257,257,256,256,256,256,256,256,256,256,256,256,256,256,256,256,255,255,255,255,256,257,259,263,268,270,272,276,278,280,280,280,280,279,280,281,281,278,274,270,266,264,253,250,248,248,248,248,248,484,479,474,464,472,489,502,514,516,505,490,476,470,468,463,452,439,427,411,405,412,416,419,422,417,412,406,401,406,412,411,398,390,380,378,380,382,382,381,384,387,392,402,407,408,414,420,426,433,440,446,451,451,450,450,449,450,453,457,461,464,466,468,469,470,469,466,458,451,441,436,444,451,460,465,464,458,452,449,441,441,443,451,456,461,465,464,455,443,434,424,413,403,391,380,372,366,357,347,338,328,318,308,306,307,310,314,318,327,338,346,354,359,352,336,324,325,330,334,332,330,318,308,300,297,295,295,296,296,296,295,294,293,292,292,292,292,292,292,291,290,288,287,286,287,289,289,291,293,292,291,289,284,281,280,280,280,281,282,282,282,282,281,283,286,288,290,290,289,290,292,293,294,294,293,296,299,302,299,297,293,290,285,281,281,281,280,281,282,282,280,279,277,275,274,275,278,280,281,282,283,285,286,288,290,291,290,288,287,287,286,286,286,287,288,288,288,287,287,286,284,282,280,280,281,282,282,284,284,284,284,285,287,288,290,292,293,293,293,293,293,294,295,295,297,298,295,291,293,294,300,302,303,305,307,306,298,292,290,288,288,288,290,292,291,288,287,285,285,283,284,283,281,277,274,269,266,264,264,264,265,266,265,264,264,263,262,262,259,258,258,258,258,258,258,258,258,258,258,258,258,259,259,258,258,258,258,258,258,258,258,258,258,257,257,257,257,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,255,255,255,255,256,257,258,266,270,273,277,280,281,281,281,282,282,282,282,283,282,281,277,271,267,269,254,250,248,248,248,248,248,472,462,455,456,472,489,506,506,499,491,484,472,460,456,459,452,438,424,408,399,402,405,410,412,407,404,401,394,395,396,390,383,378,376,378,382,389,389,386,387,388,394,402,413,412,417,423,430,436,442,448,452,452,452,451,450,451,455,459,464,465,466,468,465,462,461,460,454,446,434,428,439,446,454,457,457,455,452,449,443,443,445,449,454,460,462,458,449,440,431,421,412,403,391,382,375,369,366,353,341,329,319,310,306,304,308,312,317,326,336,344,351,356,356,341,327,316,323,332,331,325,315,305,302,302,301,301,302,302,300,297,297,295,294,293,294,293,292,292,292,291,288,286,283,283,286,287,288,290,291,290,288,285,282,280,280,280,280,281,281,282,282,281,283,284,286,286,287,287,291,294,293,293,293,292,293,296,298,298,296,296,298,294,289,283,280,281,283,283,283,282,281,278,276,274,274,275,278,279,280,281,285,286,287,287,286,286,284,282,283,285,286,286,286,287,288,288,287,287,286,284,282,280,279,280,280,280,281,280,280,283,285,287,288,290,291,293,293,292,292,292,292,293,293,294,295,294,291,291,294,297,298,298,300,302,302,294,290,288,286,287,289,292,294,296,294,290,286,281,280,281,282,284,281,279,277,270,269,268,268,270,270,268,266,264,262,262,261,259,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,257,257,257,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,255,255,255,255,256,257,258,260,265,269,275,279,282,282,282,283,282,283,283,283,283,282,282,277,271,268,271,269,252,249,248,247,247,247,455,450,446,457,475,491,495,485,480,475,477,471,458,441,443,453,444,430,416,401,392,393,401,402,398,395,393,386,383,382,377,374,373,376,379,390,391,392,389,392,393,400,407,415,413,419,426,433,440,444,450,452,453,453,451,451,453,456,460,464,466,467,465,461,457,455,455,451,442,428,423,432,441,446,450,450,449,447,444,441,439,448,451,453,456,456,453,442,434,426,418,410,402,393,383,375,366,358,350,339,327,318,312,306,304,306,311,316,325,332,338,342,349,356,346,331,320,316,322,326,322,315,308,309,306,307,305,307,306,303,301,297,295,294,294,293,293,293,293,294,293,289,286,283,282,283,284,284,287,288,288,287,285,282,281,280,280,280,280,280,281,281,281,283,284,284,283,284,285,290,293,293,294,292,292,293,295,296,296,296,296,296,294,292,291,284,285,286,285,284,283,282,281,276,274,274,274,276,278,280,282,282,283,283,283,282,280,280,280,282,283,284,286,287,287,287,287,287,287,286,284,282,281,280,279,278,278,277,277,279,282,286,287,288,288,288,290,291,291,290,290,290,291,291,292,293,292,290,290,293,295,295,295,295,296,296,292,288,285,282,287,290,294,301,304,303,294,288,281,277,280,281,280,277,274,272,270,269,269,270,271,270,268,266,263,262,262,260,259,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,257,257,257,257,256,256,256,256,256,256,256,256,256,256,255,255,255,255,255,255,255,255,255,255,256,256,261,263,269,272,274,276,277,278,280,283,282,283,283,283,283,282,281,279,274,276,276,275,255,251,248,247,247,247,442,438,442,458,476,478,476,470,466,462,465,465,455,441,431,439,444,437,424,408,389,386,390,390,388,386,385,379,374,371,369,370,372,379,388,392,393,394,394,398,401,403,411,417,418,423,428,436,441,446,451,453,454,453,453,454,456,460,461,464,466,464,460,454,450,449,449,445,437,427,416,424,432,438,442,444,443,441,437,436,440,446,452,454,452,451,446,437,429,422,416,407,399,391,381,373,364,355,346,337,327,322,317,307,303,305,309,316,320,324,323,331,337,346,345,336,325,316,314,314,315,313,308,306,305,310,315,314,309,304,300,295,293,293,293,293,293,293,293,293,292,290,287,285,284,284,283,283,284,286,286,286,284,283,282,281,280,280,280,280,280,280,283,283,283,283,282,282,283,286,291,293,292,290,291,293,294,296,295,295,293,294,293,292,291,289,290,289,286,285,283,282,280,276,274,274,275,277,279,280,280,279,278,278,278,278,279,280,281,283,284,285,286,287,287,287,287,287,286,285,284,282,281,279,278,277,276,276,276,279,282,285,287,287,287,287,288,289,288,289,288,288,288,289,290,290,288,288,286,290,292,292,292,292,292,291,289,286,284,281,289,292,298,305,304,306,297,288,280,275,275,274,271,268,268,268,268,269,269,269,270,270,268,264,262,262,261,260,259,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,257,257,257,256,256,256,256,256,256,256,256,256,256,256,255,255,255,255,255,255,255,255,255,256,256,265,270,268,267,271,272,273,274,276,278,279,281,282,282,283,283,282,280,278,280,282,282,281,260,255,251,248,247,247,436,431,442,454,462,459,458,455,451,447,454,456,448,436,425,427,436,436,429,415,394,379,380,380,377,377,377,372,368,366,368,370,379,384,391,393,395,396,397,402,405,409,413,415,424,428,432,438,442,447,451,453,454,454,454,454,456,459,462,464,465,459,454,450,446,443,443,439,432,422,412,416,420,428,434,436,436,432,430,437,440,446,451,453,449,443,439,432,425,418,411,403,394,388,381,373,365,355,346,338,330,322,318,306,303,304,307,315,316,320,324,329,334,334,336,331,322,316,313,310,306,305,307,306,305,306,316,319,313,304,300,295,293,292,292,293,292,292,293,293,292,291,291,288,288,287,284,282,282,283,284,283,283,283,283,282,281,280,280,280,280,280,281,281,281,282,281,281,281,283,287,289,290,287,290,292,292,294,294,294,293,292,292,292,292,293,292,290,287,285,283,282,280,276,274,274,276,277,277,277,277,276,274,274,274,276,279,281,282,285,286,287,287,287,287,287,287,286,285,284,283,282,281,280,280,278,277,275,276,278,282,284,286,286,285,285,286,286,286,285,285,285,285,286,286,286,285,281,283,286,288,288,288,288,288,288,286,284,282,280,284,290,299,305,306,305,296,286,274,268,271,268,268,268,267,267,267,268,268,268,269,269,266,263,262,261,261,260,259,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,257,257,257,257,256,256,256,256,256,256,256,256,256,256,255,255,255,255,255,255,255,255,255,256,256,265,283,281,278,273,269,273,274,275,276,277,279,280,281,282,283,283,281,279,280,288,292,289,286,282,259,254,250,248,248,431,424,430,442,445,443,442,444,437,435,444,446,443,427,414,415,427,429,423,416,403,385,376,373,370,369,367,365,364,367,374,379,380,385,386,389,392,392,398,402,404,412,417,420,426,431,439,440,444,447,450,452,454,454,453,452,453,457,460,463,460,451,446,443,441,436,436,433,429,421,409,404,409,419,425,426,426,424,431,438,442,446,449,452,448,440,427,425,421,415,407,398,388,382,379,371,362,352,345,336,327,320,311,305,303,303,305,308,315,322,327,332,336,339,327,321,316,319,319,316,310,306,310,309,306,303,308,316,310,304,302,294,291,292,291,292,292,292,292,291,293,292,292,294,290,289,285,282,281,282,282,282,282,282,282,281,281,280,280,280,280,280,280,280,281,281,281,280,281,281,283,285,282,282,285,288,290,290,293,292,293,290,290,291,291,293,292,290,287,284,282,280,278,277,274,274,274,276,276,276,274,274,274,274,276,279,281,283,285,287,287,287,287,287,287,287,287,286,285,284,283,282,281,281,281,279,277,275,276,278,280,282,285,284,284,282,283,283,284,282,281,282,283,283,282,282,283,281,281,284,284,284,284,285,285,284,282,281,280,279,281,289,297,304,302,301,292,282,273,268,268,268,268,267,267,267,267,267,268,268,268,267,264,262,262,261,260,259,259,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,258,257,257,257,257,257,257,257,256,256,256,256,256,256,256,256,255,255,255,255,255,255,255,255,255,256,264,278,292,286,284,282,278,281,282,282,278,278,279,280,281,282,283,283,282,279,286,295,300,294,289,284,263,258,254,251,249,426,422,419,428,430,430,432,434,427,425,436,433,431,429,414,402,415,420,417,404,398,392,379,368,367,363,360,361,365,369,379,387,389,389,388,388,389,393,401,401,404,411,416,420,426,433,439,442,445,449,451,453,453,453,452,449,450,454,453,454,451,444,437,438,434,430,426,424,424,417,409,393,401,411,415,418,419,426,432,439,443,448,450,451,447,438,426,422,418,412,403,393,382,374,370,366,356,349,342,332,326,322,309,303,303,303,304,306,313,325,332,338,343,344,337,327,323,325,332,328,318,316,319,313,308,301,300,306,303,301,305,300,292,288,291,291,291,291,292,291,291,293,293,293,293,289,285,283,281,280,280,281,281,281,281,281,281,280,280,280,280,280,280,280,280,281,281,280,280,281,283,284,282,281,283,284,284,285,288,292,291,290,289,289,291,291,291,290,286,284,282,279,278,275,273,273,274,275,274,274,273,273,274,276,278,281,282,284,286,287,287,287,287,287,287,287,287,286,286,284,284,283,283,283,282,281,277,275,275,278,280,282,283,283,283,281,280,282,283,282,280,280,281,282,281,280,282,280,280,282,282,281,280,282,283,281,279,278,278,278,280,286,293,299,291,293,285,278,271,268,268,267,267,267,267,266,266,266,267,266,266,264,263,262,261,260,260,259,259,258,258,258,259,259,259,259,259,258,258,258,258,258,258,258,258,258,258,258,258,257,257,257,256,256,256,256,256,256,256,256,256,256,256,256,256,255,255,255,255,255,255,255,256,260,270,283,286,287,288,287,283,285,288,289,285,280,279,280,281,282,284,285,281,283,294,302,306,302,293,288,282,262,258,254,251,438,431,416,411,409,416,417,427,419,415,427,424,415,417,406,397,406,410,407,400,388,386,382,378,369,360,357,360,372,381,384,386,389,393,393,392,392,399,404,406,410,410,412,414,424,433,439,443,445,450,452,453,453,452,450,446,446,449,448,445,442,438,432,426,425,419,419,418,414,408,401,388,394,402,410,413,420,426,433,439,444,451,452,454,446,433,424,419,415,406,398,388,376,368,363,359,352,344,342,334,327,318,309,303,303,303,304,305,311,316,319,330,339,342,342,331,329,333,342,338,327,318,319,312,309,301,296,297,296,297,304,297,292,289,287,287,287,288,289,289,289,290,291,291,291,290,286,283,281,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,280,281,281,281,281,281,283,282,280,283,284,284,288,286,287,288,289,288,287,285,283,281,280,280,274,273,273,273,273,273,273,273,274,276,277,280,281,283,285,287,287,287,287,287,287,287,287,286,286,285,285,284,284,285,285,284,281,277,276,275,279,282,282,282,282,281,280,279,281,282,281,280,279,280,282,281,279,280,279,280,280,279,278,278,280,280,279,278,276,276,277,279,281,285,289,282,283,278,272,268,268,267,267,267,267,266,266,266,266,265,264,263,263,262,262,261,260,260,259,259,258,258,258,259,259,259,259,259,258,258,258,258,258,258,258,258,258,258,258,258,257,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,255,255,255,255,255,255,257,259,263,272,281,280,288,290,293,291,291,294,294,291,281,280,280,281,283,284,289,290,290,300,308,314,310,302,291,286,281,262,258,255,452,438,424,413,404,396,402,414,412,402,414,416,407,404,399,387,394,400,398,390,382,375,378,379,367,359,356,362,375,388,391,390,394,397,399,402,402,401,407,413,415,412,414,421,428,434,439,439,446,450,452,454,456,452,444,438,441,441,440,436,436,434,429,422,414,411,411,410,404,393,385,379,395,405,411,416,423,429,435,440,445,450,450,449,441,430,422,416,409,400,391,380,372,364,361,357,352,340,330,330,326,318,308,302,303,303,304,306,307,318,322,328,333,329,330,333,335,341,342,339,328,313,302,313,307,303,296,293,291,295,293,292,289,286,284,284,284,284,285,286,287,288,289,289,288,288,286,283,281,280,280,280,280,280,280,280,280,280,280,280,280,280,280,279,280,280,279,279,279,280,280,280,280,280,280,280,281,280,281,281,280,281,282,282,285,286,286,285,283,282,280,278,276,273,273,273,273,273,273,273,273,274,274,277,279,281,283,286,287,287,287,287,287,287,287,286,286,286,285,285,285,285,286,287,285,282,276,276,275,279,282,282,282,281,280,280,279,279,280,282,280,278,279,280,280,279,278,278,279,278,277,277,277,278,278,278,276,274,274,270,272,275,278,279,275,274,270,268,267,267,267,267,267,267,267,267,266,266,265,264,262,262,262,261,260,260,259,259,258,258,258,259,259,259,259,259,259,259,258,258,258,258,258,258,258,258,258,258,258,257,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,255,255,255,256,257,259,262,263,267,275,282,289,294,296,296,294,298,298,297,284,281,280,282,284,288,291,290,292,303,313,319,317,306,293,288,284,281,277,259,459,449,437,423,410,396,388,398,402,395,401,404,400,392,392,381,378,387,390,382,374,365,369,367,363,358,360,366,379,389,394,397,399,402,405,408,411,408,412,417,419,416,418,427,432,437,441,444,448,451,452,454,452,447,441,433,428,434,434,431,427,428,425,418,408,399,402,399,392,379,373,382,399,407,414,420,425,431,436,441,447,451,448,443,437,425,419,412,406,395,384,376,370,364,359,355,349,341,331,325,318,317,307,302,302,303,304,306,317,324,331,332,324,317,321,327,329,331,334,329,330,317,300,302,308,302,297,292,291,291,290,288,286,284,282,282,282,282,283,284,284,285,286,286,286,285,284,282,281,280,280,280,280,280,280,280,280,280,280,280,280,280,279,279,279,279,279,279,279,279,279,279,279,280,280,280,280,280,280,280,281,282,282,282,283,284,284,283,282,281,279,276,275,274,273,273,273,273,273,273,274,274,275,276,279,281,283,286,286,286,287,286,286,286,286,286,286,285,285,285,285,286,286,287,286,282,277,277,275,279,280,282,282,280,279,280,280,278,279,281,280,278,278,278,277,276,276,275,275,275,275,275,275,275,275,275,274,272,270,269,268,270,271,271,269,268,267,267,267,267,268,268,268,267,267,267,267,267,266,263,262,262,261,261,260,260,259,259,258,258,258,259,259,259,259,259,259,259,259,258,258,258,258,258,258,258,258,258,257,257,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,255,256,257,258,261,264,266,268,277,285,293,296,299,300,300,301,304,303,285,281,280,282,285,288,290,291,292,302,312,319,321,313,299,291,287,284,281,277,467,455,440,426,413,400,386,386,388,386,391,393,392,383,381,374,368,376,376,372,365,359,360,360,358,359,367,377,385,393,399,403,404,407,410,414,418,416,412,416,422,424,427,430,435,440,446,450,451,452,453,450,443,437,432,425,423,428,426,426,422,421,419,415,404,390,386,384,375,364,368,384,399,410,416,423,428,433,438,445,450,453,447,438,431,422,415,408,402,392,381,378,371,365,357,351,345,338,332,323,315,312,305,302,302,302,303,306,314,324,331,324,316,317,325,329,333,328,325,320,330,320,302,300,299,299,296,292,291,290,289,287,284,283,282,282,282,282,282,282,282,283,283,283,283,282,282,282,281,280,280,280,280,280,280,280,280,280,280,280,280,280,279,279,279,279,279,279,280,279,279,278,279,279,280,280,280,280,280,280,280,278,280,281,282,282,282,282,281,280,279,280,275,274,273,273,273,273,274,274,274,275,275,274,277,280,281,284,286,286,286,285,285,286,286,287,286,285,286,286,285,286,286,287,286,283,279,277,275,278,279,281,281,280,280,279,279,277,278,280,277,275,276,275,275,274,273,271,272,272,272,272,271,272,272,272,271,270,269,268,268,268,268,267,267,267,267,267,267,268,268,268,268,268,268,268,267,267,267,263,262,262,261,261,260,260,259,259,258,258,258,259,259,259,259,259,259,259,259,259,258,258,258,258,258,258,258,258,258,257,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,256,257,259,260,261,265,271,274,278,284,293,298,303,305,304,306,310,307,286,282,281,283,285,287,289,291,291,298,308,318,322,317,305,293,290,286,284,281,473,457,444,431,416,404,389,379,377,377,381,382,382,376,369,364,363,366,363,360,358,355,354,355,357,360,374,383,391,398,403,404,408,411,414,418,424,423,420,420,426,429,431,435,438,445,451,454,454,454,450,442,435,427,425,419,419,420,419,419,417,413,413,410,402,390,374,364,359,363,379,390,404,414,421,427,431,436,443,450,453,453,445,436,425,419,410,402,395,388,382,379,370,363,354,346,341,335,331,324,316,305,304,301,300,300,298,303,314,321,322,313,310,319,325,327,330,321,318,315,321,312,305,298,295,296,293,292,291,291,290,288,286,284,284,284,284,283,282,281,281,281,281,281,281,281,281,281,280,280,280,280,280,280,280,280,280,280,281,281,280,280,279,279,279,279,279,280,280,279,278,278,278,280,280,280,280,280,281,280,276,274,276,280,280,280,280,279,280,280,280,280,280,275,273,273,273,273,273,274,274,275,274,276,278,280,281,282,284,284,284,284,284,285,286,287,287,286,286,286,285,286,286,287,286,284,280,276,275,276,278,279,280,281,279,277,276,276,275,275,274,274,273,273,272,271,271,269,270,270,269,269,269,269,271,271,269,269,269,267,267,267,267,267,267,267,267,268,268,268,268,269,268,268,268,268,267,267,266,262,262,262,261,261,260,260,259,259,258,258,258,259,259,259,259,259,259,259,259,259,259,259,258,258,258,258,258,258,258,257,257,257,257,257,256,256,256,256,256,256,256,257,257,257,257,257,258,258,259,260,261,262,267,276,282,282,284,291,297,306,309,313,317,316,308,288,284,283,284,285,286,288,291,294,294,304,314,320,321,309,298,292,289,286,283,470,457,445,434,419,408,396,387,377,376,374,372,370,367,363,359,357,356,353,352,352,352,353,356,358,368,380,388,393,402,405,406,410,414,417,424,429,430,428,427,428,431,434,438,443,450,451,456,454,450,443,436,429,424,421,414,412,411,414,412,408,402,401,394,393,384,373,356,352,365,381,393,406,417,424,430,434,439,449,449,455,452,441,432,426,416,404,390,384,380,377,373,367,358,346,338,333,331,330,323,318,309,303,299,296,296,298,303,312,317,312,303,312,320,316,315,320,318,310,304,305,304,300,296,294,293,293,291,291,291,291,293,289,287,287,287,287,285,283,282,282,281,281,281,281,281,281,281,281,281,282,281,281,281,281,281,282,282,282,282,281,280,280,280,280,280,280,280,280,279,278,278,278,279,280,280,280,280,281,283,279,275,276,276,276,276,277,278,280,280,280,282,282,277,273,273,273,273,273,274,274,275,275,276,278,281,282,282,283,283,282,282,283,284,285,286,287,286,286,286,285,284,285,286,287,285,280,276,275,275,276,278,279,281,275,274,274,273,273,273,272,272,271,271,270,270,269,268,268,268,268,268,268,268,270,270,268,268,267,267,267,267,267,267,267,268,268,269,269,270,269,268,268,268,268,268,267,267,264,262,262,261,261,261,260,260,259,259,258,258,258,259,259,259,259,260,259,259,259,259,259,259,259,259,259,258,258,258,258,258,257,257,257,257,256,256,257,258,258,257,257,258,258,258,258,259,259,260,261,262,263,266,272,279,286,293,293,290,293,302,313,320,323,320,309,292,290,293,294,287,287,288,291,295,299,302,309,318,322,317,303,295,291,289,286,467,459,448,438,430,419,409,398,386,376,367,370,366,362,358,354,351,348,347,348,350,352,355,361,371,379,382,390,394,401,402,404,409,414,418,422,427,429,429,430,431,434,436,439,447,450,451,453,450,444,437,430,426,423,419,418,415,401,402,404,404,389,386,375,377,365,359,351,352,365,382,394,409,419,426,431,436,440,450,444,452,444,434,426,418,408,400,392,377,367,365,364,361,353,341,332,326,322,319,316,314,306,300,295,294,296,300,306,305,306,304,306,312,317,312,305,317,315,306,299,297,293,296,294,293,292,291,291,291,291,291,291,290,290,291,291,289,287,285,284,284,283,283,283,283,283,283,283,283,284,284,284,284,284,284,284,284,284,284,283,282,282,282,282,282,281,280,281,280,279,278,278,278,278,277,276,278,280,281,282,275,274,274,274,274,274,274,275,278,280,281,282,280,277,274,273,273,273,273,273,274,274,275,275,275,280,281,281,282,282,281,280,282,282,284,285,286,286,285,286,283,283,283,285,287,286,281,278,275,275,275,275,275,274,273,273,273,273,273,271,270,270,269,269,268,268,267,267,267,267,267,267,267,267,267,267,267,267,267,267,267,267,268,268,267,269,270,271,271,269,269,268,268,268,267,268,267,265,262,262,261,261,261,261,260,260,259,259,259,258,259,259,259,259,260,260,260,260,260,259,259,259,259,259,259,259,259,258,258,258,258,258,258,258,258,260,262,262,261,260,260,259,259,260,260,261,261,261,262,264,266,268,275,283,291,296,300,300,299,302,308,320,329,324,317,306,294,292,301,296,290,290,292,295,298,300,304,314,320,320,311,301,295,292,289,455,456,453,444,436,428,419,406,395,388,380,375,368,361,357,351,347,344,345,348,350,352,357,365,375,380,384,391,394,396,398,401,404,407,414,415,419,426,428,430,433,436,438,442,448,449,445,446,442,437,430,426,424,421,420,417,415,400,388,396,397,383,373,363,362,356,348,343,351,362,378,394,411,422,429,432,437,440,445,440,436,432,427,415,408,400,394,388,381,363,352,354,356,345,336,327,321,318,315,313,310,306,301,295,293,297,302,308,304,303,302,306,309,309,306,306,306,305,301,296,293,293,293,292,292,291,291,291,291,291,291,291,291,291,292,292,290,289,287,287,287,287,286,286,286,286,286,286,287,288,289,289,289,290,289,289,288,288,286,285,284,284,285,285,285,284,282,281,280,280,279,278,278,277,275,274,276,277,281,274,273,273,273,273,273,273,274,274,275,279,282,283,280,278,273,273,273,273,273,273,274,274,274,274,275,276,278,280,282,282,279,280,280,282,284,285,285,285,284,284,283,281,281,283,285,285,282,280,278,274,273,273,273,273,273,273,273,273,273,270,268,268,268,268,268,269,269,267,267,267,267,267,267,267,267,267,267,267,267,267,267,267,269,270,269,270,272,272,271,268,268,268,269,270,271,271,269,264,262,262,261,261,261,261,260,260,260,259,259,259,259,259,259,259,260,260,260,260,260,260,260,260,260,259,259,259,259,259,259,259,259,259,260,261,262,262,263,263,262,262,261,261,260,261,262,262,262,262,264,267,270,275,280,287,296,303,309,314,313,308,312,322,330,330,323,311,295,292,307,303,294,290,293,294,297,300,302,310,319,323,318,307,301,296,293,443,445,447,443,439,429,422,414,405,400,392,384,379,370,361,349,343,343,347,351,353,353,354,363,364,374,378,388,390,394,399,401,401,404,412,413,416,423,428,430,433,437,440,443,441,440,441,438,436,430,426,423,421,420,418,416,410,390,378,384,389,378,361,352,350,344,339,342,351,357,370,392,407,420,427,428,436,437,436,426,423,418,414,404,398,393,387,377,375,366,344,339,345,338,328,320,319,316,316,317,315,312,305,296,293,297,303,306,306,302,302,307,305,303,302,303,301,299,296,293,292,292,292,291,291,291,291,291,291,291,291,291,291,291,291,291,290,290,290,290,291,291,291,292,291,289,290,291,291,291,293,293,293,294,292,292,292,289,288,288,287,288,288,288,288,288,286,284,284,281,280,279,278,277,276,275,275,276,274,273,273,273,273,273,273,273,273,274,274,274,277,280,280,274,273,273,273,273,273,273,274,273,273,273,272,273,277,281,281,277,276,276,280,285,285,285,284,283,282,282,282,280,280,280,282,283,282,281,280,278,276,274,274,274,273,274,274,274,274,274,273,271,270,271,271,271,271,273,274,271,270,267,267,267,267,267,267,268,270,270,271,270,271,272,272,273,274,274,272,271,271,272,276,277,277,271,269,262,262,262,262,261,261,261,261,260,260,260,259,259,259,259,259,260,260,260,260,260,260,260,260,260,260,260,260,260,260,260,260,260,260,260,261,261,262,262,263,263,263,263,262,262,262,262,263,264,264,264,266,269,275,278,282,290,298,307,317,321,324,325,324,330,336,333,326,314,298,297,307,307,301,296,290,294,296,299,301,308,316,321,322,316,306,301,298,431,431,435,434,430,422,413,404,403,399,388,380,373,366,354,346,342,343,348,356,357,360,365,368,375,379,384,389,390,395,402,404,409,414,418,419,419,424,427,430,434,437,440,440,439,437,434,434,431,427,423,420,418,418,418,415,399,383,368,368,372,364,354,341,341,339,338,346,356,363,372,387,402,417,428,433,432,427,422,417,416,410,404,402,389,382,378,375,369,359,345,330,331,334,329,323,320,316,315,318,331,319,310,298,293,296,300,303,303,301,303,305,301,299,297,295,295,294,293,292,291,291,291,291,291,291,291,291,291,291,291,291,291,291,291,292,292,293,293,293,293,293,293,293,290,292,293,294,294,294,295,296,298,298,297,296,293,291,291,290,289,290,291,291,290,290,292,291,282,280,280,279,278,277,277,276,276,275,273,273,273,273,273,273,273,273,273,273,274,275,275,275,274,273,273,273,273,273,273,273,274,273,272,272,273,273,278,282,276,275,275,277,281,285,286,285,283,282,281,280,279,278,279,280,282,282,282,282,282,280,279,276,275,275,275,275,275,275,275,275,275,275,274,274,274,273,272,274,274,274,273,270,270,270,271,272,271,273,275,276,277,276,276,276,276,277,276,275,276,276,277,279,281,280,276,271,263,263,262,262,262,262,261,261,261,261,260,260,260,260,259,260,260,260,260,260,260,260,260,260,260,260,260,260,260,260,260,260,260,260,260,261,261,262,262,263,264,264,264,263,263,263,263,264,265,265,265,266,267,268,276,280,285,292,298,308,318,323,330,334,337,341,340,336,326,312,302,301,308,312,308,304,292,287,296,299,302,305,312,319,327,325,316,307,302,418,419,424,424,424,414,404,394,388,390,386,376,363,356,348,343,342,344,350,352,357,363,370,377,382,386,392,399,401,400,405,410,416,419,425,426,426,424,427,430,433,436,438,438,436,434,431,429,430,427,419,416,412,415,416,414,396,381,365,354,355,352,346,339,339,337,342,351,362,372,379,392,407,420,430,429,423,413,407,407,408,403,395,393,389,378,373,368,364,352,342,327,322,326,332,332,325,317,312,315,321,333,311,299,293,295,297,298,298,295,297,298,298,294,294,293,292,292,292,291,291,291,291,291,291,291,291,291,291,291,291,291,291,291,291,293,294,294,294,294,294,294,294,293,291,294,295,296,297,297,298,300,303,305,302,300,296,294,293,292,291,292,294,294,293,293,294,291,287,282,280,279,278,278,278,278,276,275,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,272,272,273,274,276,280,275,274,274,279,282,283,284,283,282,282,281,279,278,277,278,280,281,282,282,282,282,281,280,279,277,276,276,275,276,276,276,276,276,276,275,275,275,274,274,274,274,275,274,273,274,274,275,276,276,278,280,281,281,279,279,279,279,279,278,278,278,278,278,279,279,276,271,267,264,262,262,262,262,262,262,262,261,261,261,261,260,260,260,260,260,260,260,260,260,260,260,261,261,261,261,261,261,261,261,261,261,261,261,262,262,262,263,264,264,264,264,264,264,265,265,266,266,266,266,267,268,269,274,280,285,292,298,304,311,318,328,337,344,342,345,338,327,317,304,302,309,315,318,309,295,288,290,298,300,302,310,318,327,332,329,319,310,404,407,411,412,417,412,398,386,377,377,379,371,359,350,343,341,343,346,350,353,359,365,370,377,383,389,396,403,407,408,411,414,419,423,426,426,428,426,428,431,432,434,435,435,432,430,427,418,424,427,412,403,395,395,398,400,390,375,361,348,340,342,338,331,330,337,345,355,367,381,391,401,413,423,424,417,414,405,390,396,398,394,388,386,384,379,368,362,356,348,337,325,317,317,326,332,333,322,314,309,311,312,307,297,292,294,294,294,294,293,293,294,293,293,292,292,291,291,291,291,291,291,291,291,291,291,291,291,291,291,291,292,293,293,292,293,295,295,295,296,296,295,295,293,293,295,297,300,302,302,301,302,305,306,305,304,301,299,296,292,292,294,295,295,294,294,294,293,290,284,280,279,279,279,279,279,278,276,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,273,272,272,272,273,274,275,274,274,274,275,279,280,281,281,280,281,281,281,281,278,277,276,277,281,281,281,281,281,281,280,279,278,277,277,277,276,276,277,277,276,276,276,276,275,275,275,275,274,274,277,277,277,277,278,280,279,280,283,285,284,283,282,282,282,281,280,279,280,281,281,279,276,273,269,262,262,262,262,262,262,262,262,262,262,262,262,261,261,260,260,260,260,260,260,260,261,261,261,261,261,261,261,261,261,261,261,261,261,261,261,262,263,263,264,264,265,265,265,266,266,266,267,267,268,267,268,268,268,271,274,278,282,288,294,298,303,305,318,330,341,350,353,342,331,318,304,302,315,321,323,313,301,293,289,297,300,304,307,316,324,331,333,331,322,394,396,398,402,407,406,396,382,367,364,369,358,350,345,341,341,343,348,350,354,364,368,374,379,383,389,395,403,408,414,416,419,423,426,433,437,425,426,432,432,432,432,432,432,430,427,418,404,408,418,417,402,380,372,376,380,377,367,358,348,340,332,331,328,332,338,344,353,366,383,400,412,419,416,414,406,401,396,385,384,388,389,380,376,378,378,368,355,349,342,333,324,316,312,317,324,330,330,313,305,301,301,299,294,292,292,292,292,292,292,292,292,292,292,291,291,292,291,291,291,291,291,291,291,291,291,291,291,292,292,292,293,294,293,293,294,295,296,297,297,297,296,295,293,294,298,301,303,305,304,302,304,306,306,307,305,303,305,304,295,296,296,296,296,295,295,294,293,287,282,279,279,281,280,280,281,280,279,276,273,273,274,275,277,281,281,277,274,274,274,273,273,273,273,273,273,273,273,273,273,273,272,272,272,272,273,273,273,273,274,274,278,280,279,275,276,281,282,281,282,279,277,276,277,281,281,281,282,281,280,279,279,278,278,278,278,277,277,277,277,277,277,277,276,276,276,276,276,276,277,279,280,280,280,281,284,284,286,288,289,288,288,287,287,286,286,285,284,284,284,284,280,276,270,265,262,262,262,262,262,262,262,262,262,262,262,262,262,261,261,260,260,260,261,261,261,261,261,261,261,261,261,261,261,261,261,262,262,262,262,262,262,263,264,264,265,266,266,267,267,267,267,268,268,270,269,270,272,273,273,276,278,279,284,290,293,300,302,305,316,331,344,353,342,330,319,314,306,316,329,329,321,309,300,294,292,299,302,305,314,321,327,331,335,331,384,387,387,394,397,398,391,381,366,351,354,348,343,341,342,343,346,350,353,354,363,365,370,377,380,386,391,398,404,413,418,421,427,425,432,436,426,429,437,433,433,428,428,428,424,413,408,396,395,403,404,393,377,359,363,366,366,361,355,346,338,328,328,329,334,338,344,355,368,386,399,413,415,406,401,393,384,385,374,370,377,378,373,367,371,371,367,356,346,335,326,319,315,310,306,313,317,319,314,299,295,294,294,293,292,292,293,292,292,292,292,292,292,292,292,292,292,292,291,291,291,291,291,291,291,291,292,292,293,294,295,294,295,293,294,295,296,297,298,298,298,298,296,296,300,300,303,305,306,305,302,304,307,307,306,305,306,305,303,298,298,298,297,296,295,294,293,288,282,279,279,281,281,280,281,283,283,281,279,277,277,277,278,281,282,281,280,278,275,274,274,273,273,273,273,273,273,273,273,273,273,272,272,272,272,272,272,273,274,275,276,277,279,275,275,276,280,281,282,281,278,276,275,277,279,281,280,282,280,280,279,279,279,278,278,278,278,278,278,278,278,278,277,277,277,277,277,279,280,279,280,282,283,284,285,289,291,292,293,294,292,292,292,291,291,290,290,289,289,288,283,280,272,266,264,263,262,262,262,262,262,262,262,262,262,262,262,262,262,261,261,261,261,261,261,261,261,261,261,262,262,262,262,262,261,262,262,263,262,262,262,262,263,264,265,266,266,267,268,268,268,268,268,271,272,275,278,280,280,278,277,280,283,290,295,302,306,308,309,313,321,334,348,345,339,333,322,312,314,330,335,330,316,306,301,294,300,303,309,313,318,319,324,330,335,375,376,377,386,388,388,384,373,361,347,342,339,339,340,343,347,352,353,358,364,365,364,365,371,376,382,390,396,402,409,415,418,422,426,419,417,421,421,439,430,429,424,421,422,414,402,398,389,385,391,390,387,372,354,352,355,351,351,343,338,331,326,326,329,334,338,344,354,364,374,386,402,406,393,386,381,373,374,367,360,367,366,362,362,366,364,360,356,347,334,326,317,314,308,304,301,302,297,298,295,293,292,293,295,295,296,297,301,297,294,293,292,292,293,293,293,293,293,293,292,291,291,291,291,292,292,293,294,294,295,295,296,296,295,296,297,298,299,300,300,300,300,300,300,302,303,304,305,307,305,305,304,307,307,307,307,307,304,302,301,300,299,298,296,295,293,290,285,280,281,282,283,282,281,282,285,286,283,282,279,279,280,280,282,282,280,279,278,275,274,274,274,274,273,273,273,273,273,273,273,273,273,272,272,272,272,272,273,274,275,275,276,275,274,274,275,280,280,282,282,277,276,274,275,276,278,280,280,281,279,279,279,278,278,278,279,279,279,279,278,278,278,278,278,278,278,278,279,281,282,282,285,287,288,290,293,296,297,298,299,298,298,298,298,298,297,296,294,293,292,287,278,266,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,261,261,261,261,261,261,261,261,261,262,262,262,262,262,262,262,262,262,263,263,263,263,263,264,265,266,266,267,268,268,269,270,270,270,272,276,279,281,282,282,281,280,282,288,294,301,308,314,318,323,327,325,333,345,356,352,342,329,316,314,330,340,332,323,314,305,302,302,303,305,306,309,314,319,327,334,366,366,371,379,376,382,378,363,350,341,335,335,339,342,345,350,355,358,365,369,375,374,373,375,378,381,388,393,399,403,409,409,411,413,414,409,415,424,429,426,421,412,410,415,406,395,388,381,375,379,376,372,365,352,341,344,340,342,338,328,326,325,325,329,334,340,346,353,361,370,381,391,394,385,376,369,365,361,360,352,354,356,354,353,356,353,352,351,343,331,325,323,328,318,311,304,296,294,293,293,293,294,296,299,303,300,305,301,303,298,295,293,293,294,293,294,294,293,293,293,292,291,292,292,293,294,295,295,295,296,296,297,297,298,298,299,300,301,302,302,302,302,302,302,303,304,304,304,305,305,306,305,306,309,309,309,306,305,303,302,302,300,298,297,295,293,292,286,283,285,286,287,287,284,283,287,292,291,286,282,283,283,283,283,283,281,280,278,275,274,274,274,274,275,273,273,273,273,273,273,273,273,272,272,272,272,272,273,273,274,274,274,274,274,276,280,282,281,280,281,280,276,273,273,273,274,274,278,279,278,278,278,278,278,278,279,279,279,279,279,279,279,279,279,279,279,279,280,280,284,286,288,290,292,295,297,301,303,304,305,304,304,305,305,305,305,304,299,296,293,279,266,262,263,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,261,261,261,262,262,262,262,262,261,262,262,262,263,263,263,263,263,263,264,264,264,264,264,265,266,266,267,268,268,270,271,271,272,273,275,278,280,282,283,284,283,283,285,290,294,303,310,319,325,333,342,344,351,359,365,357,345,330,317,317,336,346,339,329,318,317,304,302,300,300,302,308,318,330,336,342,356,358,367,368,365,374,368,355,343,335,333,335,340,345,350,352,354,360,366,373,380,384,384,384,385,388,392,397,402,406,401,399,399,400,400,401,410,416,418,417,411,402,399,408,405,394,382,373,365,368,365,361,355,348,340,338,335,332,329,326,321,319,325,329,334,338,343,352,355,366,376,384,388,380,378,371,366,360,352,347,345,345,342,348,350,345,342,344,338,325,320,321,323,317,306,296,295,293,294,295,296,298,300,305,303,303,305,302,301,299,296,295,295,295,295,293,293,293,293,293,293,292,292,292,293,294,295,296,296,297,298,298,299,299,300,301,302,303,303,304,304,303,303,303,304,306,306,305,303,305,307,307,309,311,312,311,310,306,305,304,303,300,298,297,295,294,289,289,287,289,292,291,290,287,287,289,294,294,291,289,287,286,286,285,284,282,280,278,276,275,275,274,275,275,274,274,274,273,273,273,273,273,272,272,272,272,272,272,272,272,272,272,272,273,274,276,277,274,278,282,281,276,273,271,270,270,271,273,274,275,277,277,278,278,278,279,279,280,280,280,280,280,280,279,279,279,280,281,282,286,289,291,293,297,301,305,305,311,312,314,312,308,308,309,308,306,300,293,285,273,265,262,263,263,263,262,262,262,262,262,262,262,262,262,262,262,262,262,261,261,261,261,262,262,262,262,262,262,262,262,263,263,263,263,264,264,264,264,264,265,265,265,266,266,267,268,268,270,271,272,273,274,275,276,277,279,281,283,285,286,287,290,291,293,301,304,315,320,331,344,356,364,370,368,357,344,330,315,327,342,352,345,333,328,322,306,302,300,302,308,313,323,334,344,352,350,351,356,358,364,368,358,343,336,333,335,338,342,347,354,356,361,362,364,374,382,390,392,391,392,394,398,402,404,404,396,389,388,388,387,395,403,407,408,405,401,389,390,400,404,394,383,372,364,362,358,355,349,342,338,332,330,328,326,322,318,318,323,328,331,338,343,352,358,369,373,378,387,386,385,381,379,369,360,351,341,339,339,344,346,340,332,333,331,317,312,318,312,298,298,296,296,296,297,299,300,302,305,304,304,305,302,302,301,300,298,296,296,294,293,293,293,293,294,293,293,292,292,292,293,294,295,296,297,298,299,299,300,301,302,302,304,304,305,306,305,304,304,305,306,307,307,307,306,304,307,310,312,313,314,314,316,313,308,305,303,301,298,296,295,293,291,290,291,293,294,293,292,292,290,290,291,292,294,292,290,289,288,287,284,282,280,279,278,277,276,275,274,275,274,274,274,273,273,273,273,273,272,272,272,272,272,272,271,271,271,271,271,271,272,272,273,274,278,282,282,277,274,270,268,268,268,271,274,275,276,277,278,278,278,279,279,280,281,281,281,280,280,280,280,280,280,284,288,290,292,295,299,303,308,312,315,317,318,318,313,309,305,296,294,288,278,273,268,264,262,263,263,263,263,263,262,262,262,262,262,262,262,262,262,262,262,262,261,261,261,262,262,262,263,263,262,263,263,263,264,264,264,264,264,264,265,265,265,265,266,266,268,268,268,269,270,271,272,274,275,276,277,278,278,278,279,281,284,287,289,292,295,299,305,308,314,321,332,345,358,367,370,363,352,340,329,320,331,348,358,354,344,337,328,308,305,304,306,311,315,323,333,345,354,344,345,348,351,356,356,345,337,333,333,338,341,344,349,355,362,367,368,370,376,383,391,396,399,399,400,404,409,409,402,391,384,378,376,380,385,388,396,396,394,393,382,382,394,401,390,381,373,367,358,352,351,345,340,333,329,327,325,323,320,317,318,322,326,330,336,343,352,358,362,365,374,385,392,392,390,385,377,368,358,347,339,339,343,345,338,327,323,322,313,303,306,304,301,297,297,298,299,301,302,305,305,304,304,304,304,302,301,301,300,299,298,296,294,293,293,294,294,295,293,293,293,292,292,292,294,295,296,298,299,300,300,301,302,303,304,305,306,307,308,307,306,306,306,307,308,309,308,307,306,308,311,315,315,316,316,317,315,308,305,303,301,298,295,294,291,292,293,292,293,294,293,293,293,291,291,291,292,292,292,292,292,290,289,286,284,281,280,279,278,277,275,274,274,274,274,273,273,273,273,273,273,272,272,272,272,271,271,270,270,270,270,270,270,269,270,271,274,277,280,281,277,275,272,268,267,267,271,275,275,276,277,277,278,279,279,280,280,282,283,283,283,283,283,282,282,285,290,293,294,296,301,304,310,316,318,316,317,317,307,296,289,284,279,273,268,268,267,267,267,264,264,263,263,263,263,263,263,262,262,262,262,262,262,262,262,262,262,261,261,262,262,262,263,263,264,264,264,264,264,264,265,265,265,265,265,266,266,266,266,266,267,268,268,269,270,271,272,274,275,277,278,280,280,280,280,280,283,286,288,290,292,296,302,306,310,318,326,336,346,354,366,370,368,361,350,338,326,333,350,361,363,354,342,331,321,308,307,308,311,315,323,331,344,356,341,340,341,345,346,344,337,332,331,338,339,343,346,351,355,362,368,374,379,381,386,392,399,404,404,406,412,416,408,400,391,382,373,366,370,371,373,383,385,383,380,375,379,390,395,390,376,367,362,355,350,348,344,339,331,328,325,322,321,318,316,318,320,324,328,330,339,341,347,352,359,370,382,391,392,392,389,379,368,361,348,335,332,341,342,336,325,314,313,306,303,302,302,300,303,300,299,303,305,301,305,306,304,304,305,303,302,301,301,301,300,299,298,296,294,294,294,294,294,293,293,293,292,292,292,294,295,297,298,299,301,302,303,303,303,305,306,307,309,309,309,308,309,309,309,310,310,310,310,310,314,314,316,317,317,318,317,314,308,304,303,301,298,294,293,292,293,293,294,294,294,294,293,292,291,291,292,292,292,292,292,294,292,290,288,285,283,282,280,279,278,275,274,274,274,274,274,274,273,273,273,273,272,272,272,272,271,271,270,270,270,270,270,269,268,268,269,272,274,278,279,277,274,274,270,267,268,271,274,275,276,277,277,278,279,280,281,282,283,284,285,286,286,287,288,290,292,295,298,301,302,306,313,318,321,322,317,307,294,286,279,268,268,267,267,267,267,266,266,265,265,264,264,263,263,263,263,263,262,262,262,262,262,262,262,262,262,262,262,262,262,263,263,264,264,264,265,265,265,265,265,266,266,266,266,266,266,266,267,267,267,267,268,269,270,272,273,274,275,277,279,280,282,282,284,284,285,288,289,290,291,292,295,301,302,304,312,318,325,332,339,352,366,373,368,360,347,333,333,349,363,370,362,348,336,325,311,309,310,313,319,323,331,343,357,340,338,339,340,338,334,332,329,330,337,341,344,348,351,356,362,366,374,380,383,388,393,399,404,408,412,416,416,405,394,387,379,372,363,362,360,361,367,369,369,368,374,378,387,393,387,375,365,360,355,351,345,343,337,330,326,322,318,318,317,315,317,318,322,326,330,335,341,344,352,360,369,377,380,380,380,380,375,370,361,348,333,330,337,337,332,326,314,306,303,302,302,302,304,305,302,301,304,304,304,305,306,305,305,305,304,304,302,302,302,301,300,299,298,297,296,296,295,293,293,293,294,293,292,292,293,295,297,299,300,302,303,304,304,305,305,307,309,311,313,312,312,312,312,311,311,312,312,313,314,316,316,317,316,318,317,315,312,309,305,304,303,300,297,293,294,294,294,294,294,294,294,293,292,291,292,293,292,292,291,292,293,293,291,289,287,285,283,282,280,279,277,276,274,274,274,274,274,273,273,273,273,273,272,272,271,271,270,270,270,270,270,270,269,269,268,268,269,273,276,277,275,274,271,268,267,268,270,274,275,276,276,277,278,279,280,281,283,285,286,287,288,289,290,292,294,297,301,304,306,308,314,319,318,313,303,294,286,272,268,267,267,266,266,266,266,265,265,265,264,264,264,263,263,263,263,262,262,262,262,262,262,262,262,262,262,262,262,262,262,263,264,264,265,266,266,266,266,266,266,267,267,266,266,267,267,267,268,267,267,268,268,269,270,271,273,274,275,277,278,280,281,283,284,287,289,290,292,292,292,293,296,302,304,305,308,310,314,313,316,326,340,351,366,373,367,356,342,338,340,357,367,362,351,338,326,320,313,312,314,317,321,331,343,356,339,337,336,335,333,329,329,330,334,339,344,347,350,352,355,362,364,371,379,382,386,390,396,402,408,414,418,416,407,397,388,379,370,363,361,355,351,352,354,358,365,373,382,390,389,381,375,366,357,355,349,343,336,331,327,322,319,316,315,316,315,315,317,320,324,331,334,342,346,352,358,365,370,371,371,371,370,365,366,357,345,329,328,333,331,327,320,309,304,302,303,304,304,305,306,302,302,306,307,306,306,307,306,305,305,304,303,303,302,303,302,302,301,300,299,298,297,296,294,293,294,294,293,292,292,293,296,297,299,301,302,303,305,306,306,308,310,312,314,316,315,314,314,314,314,313,314,314,315,316,317,316,316,317,317,315,313,310,308,307,304,304,301,298,296,295,295,295,295,294,294,294,293,292,292,292,292,293,292,292,292,292,293,292,290,287,285,283,282,282,281,280,277,275,275,274,274,274,273,273,273,273,273,272,272,271,271,271,271,271,271,271,271,269,269,268,268,268,272,274,274,273,274,271,267,267,268,270,272,275,276,276,276,277,278,279,280,283,287,288,289,290,292,294,297,300,303,306,307,308,310,310,309,300,289,281,270,268,267,266,266,265,265,264,264,264,264,264,264,263,264,263,263,263,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,264,265,266,267,267,267,266,266,267,267,268,268,267,267,268,268,268,268,268,268,268,269,270,271,273,274,275,277,278,279,281,283,284,288,290,292,294,294,294,295,298,304,307,313,316,317,319,321,322,323,325,327,338,349,365,374,368,358,350,342,347,356,357,351,342,338,332,317,314,313,314,317,327,339,351,336,335,334,332,330,329,328,331,336,344,346,354,355,355,361,366,366,366,373,376,382,388,393,401,410,415,419,418,410,400,392,381,371,365,369,367,357,349,351,354,364,372,380,380,374,366,366,356,351,349,344,338,329,329,328,318,316,315,316,315,315,314,315,316,317,323,331,338,344,349,354,358,362,363,362,361,360,352,352,351,339,325,326,330,326,319,311,303,303,304,304,305,305,304,306,303,304,307,310,309,308,309,307,306,304,304,305,304,304,304,303,303,302,302,301,299,298,297,296,295,295,295,294,293,292,293,296,298,300,302,303,304,306,307,308,310,312,314,316,317,316,316,317,316,315,315,315,316,316,314,317,315,314,313,315,313,310,308,306,304,304,303,301,299,297,296,296,295,295,294,294,294,294,292,292,291,291,291,291,290,291,291,291,290,288,286,285,283,282,281,281,279,275,275,274,274,274,274,272,272,272,272,272,273,272,271,271,271,272,272,273,275,270,269,269,268,268,268,269,270,272,272,273,268,267,267,268,268,271,273,274,275,275,275,276,279,280,284,289,291,292,293,296,299,302,305,306,308,309,308,302,294,285,276,268,267,267,266,265,264,264,263,263,263,263,263,263,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,264,266,267,268,268,268,268,267,267,267,268,269,268,268,268,269,269,269,268,268,269,270,271,272,273,274,276,277,278,279,280,282,284,286,290,293,294,295,296,297,298,301,304,309,316,322,328,332,335,337,337,340,337,342,346,360,376,379,373,366,361,352,350,360,357,357,352,342,330,316,314,315,320,324,335,346,331,330,332,330,329,328,329,329,337,341,346,353,356,364,366,370,375,376,378,382,388,392,399,406,414,418,416,416,412,401,391,378,375,375,378,374,366,353,348,349,356,366,369,371,364,352,355,350,342,340,339,333,329,325,326,318,317,317,319,318,313,314,314,314,318,325,331,335,340,342,346,350,352,352,354,357,354,346,341,339,330,321,323,322,320,312,304,304,304,304,304,305,305,305,306,305,309,310,310,310,310,310,308,305,306,307,307,307,306,305,305,304,304,304,303,301,299,298,297,296,296,295,294,293,292,293,296,299,301,303,304,305,307,308,309,312,314,316,318,319,316,316,318,317,317,316,316,317,316,315,316,316,315,314,312,311,308,305,304,303,303,302,300,299,298,297,296,295,294,294,294,293,292,292,290,290,289,290,288,289,288,289,289,288,287,285,284,283,281,281,280,278,275,275,275,274,274,272,272,272,272,272,272,272,272,272,272,272,273,274,274,274,272,268,268,268,268,268,268,269,270,271,274,274,267,267,268,268,269,270,272,273,273,273,275,280,284,289,292,295,298,300,302,304,306,307,308,301,290,282,273,268,268,267,267,266,266,265,264,263,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,264,267,269,270,270,270,270,269,268,268,268,268,269,270,269,269,270,271,271,270,270,272,272,274,275,276,276,277,278,279,280,282,284,285,287,289,291,293,294,297,296,300,302,304,307,311,319,326,334,343,348,350,352,353,360,361,369,384,389,386,382,377,368,362,361,366,366,361,350,338,324,316,316,317,321,331,342,329,331,332,329,328,328,331,334,342,343,347,351,354,363,369,374,379,381,384,390,394,399,404,410,415,415,410,410,408,402,393,387,385,386,381,375,366,355,343,341,348,354,358,363,358,351,344,342,340,336,331,329,328,327,321,320,318,320,320,330,330,321,316,314,316,319,326,330,331,334,341,344,345,343,352,353,348,340,332,330,323,315,325,330,320,305,304,305,305,304,305,306,306,306,307,308,310,311,311,310,311,312,311,309,309,309,309,310,308,307,306,305,305,305,303,302,300,300,299,298,297,296,294,293,293,294,298,300,302,304,305,306,308,309,311,314,315,317,318,318,318,317,319,318,317,316,317,318,317,317,317,316,316,313,311,309,307,305,304,302,302,301,300,299,297,296,295,294,294,294,292,292,292,291,289,288,287,287,286,286,286,286,287,286,285,284,283,282,282,281,280,279,277,276,276,274,272,272,272,272,272,273,274,274,273,272,272,273,275,278,278,277,274,271,270,269,270,268,268,268,269,271,273,268,267,267,267,268,267,268,270,270,271,274,278,282,288,294,298,302,304,303,302,307,306,300,291,279,268,267,267,266,266,266,265,265,264,264,263,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,263,267,270,272,273,274,274,274,272,271,272,270,268,269,270,270,271,272,272,272,273,273,274,276,277,278,278,278,278,278,279,280,282,283,286,288,289,290,290,291,292,295,300,301,301,302,304,313,324,335,345,351,357,363,368,372,377,383,392,395,394,392,387,381,372,366,365,366,358,351,338,327,318,316,317,322,326,337,330,331,329,328,328,329,329,336,343,349,355,355,353,359,364,370,375,379,388,394,401,404,406,406,408,406,400,400,404,405,400,395,394,389,381,372,363,354,344,339,341,346,351,355,353,347,342,338,335,332,330,329,328,326,325,322,321,325,325,333,332,328,320,314,313,315,315,318,318,329,334,339,340,340,343,344,345,342,331,323,317,317,324,325,318,304,305,305,305,305,306,307,307,308,310,310,311,311,311,312,312,312,311,311,312,312,311,310,309,308,307,306,306,305,304,302,301,300,300,299,298,297,296,296,294,297,299,301,303,304,306,307,309,310,312,313,315,317,317,318,320,319,319,319,318,316,319,317,317,317,317,317,316,312,310,308,307,305,304,302,301,300,299,298,297,296,294,294,294,293,292,291,290,290,287,286,284,284,284,284,283,284,284,284,283,283,283,283,281,281,281,280,276,274,274,272,272,272,273,275,274,274,274,275,275,274,273,273,277,281,282,281,276,273,272,270,270,269,268,267,267,268,271,267,267,267,267,267,267,268,268,268,273,277,281,287,293,299,305,300,297,294,300,293,285,274,268,266,266,265,265,264,264,264,264,264,263,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,267,271,273,275,276,278,279,278,278,277,276,268,268,269,270,271,272,273,274,275,275,276,277,278,280,280,279,279,279,279,279,280,281,284,286,288,290,291,292,293,295,300,303,304,303,302,304,309,320,331,340,341,352,364,370,375,379,384,389,392,394,397,394,390,381,374,371,367,361,350,342,336,327,317,316,317,318,331,328,328,328,328,328,330,333,338,340,350,357,366,361,362,366,371,377,382,389,396,403,407,404,401,399,397,390,389,397,404,403,399,394,388,378,370,362,352,344,340,337,340,346,350,344,343,340,336,331,331,330,329,327,327,326,326,327,334,338,337,336,334,324,315,313,312,313,312,317,323,330,334,330,330,336,341,339,336,330,322,316,314,317,308,304,304,305,306,306,306,307,308,309,310,310,310,311,310,311,313,313,313,313,313,313,313,312,311,310,309,308,307,306,305,304,303,302,301,301,300,299,298,298,298,298,299,300,302,304,305,307,308,310,311,313,314,315,316,317,318,320,321,320,320,318,319,319,317,315,315,315,314,313,309,308,306,305,303,302,301,300,299,298,297,296,295,294,293,292,292,291,289,288,287,284,284,283,280,282,282,281,284,283,281,281,282,280,280,280,280,280,278,274,273,272,273,274,276,274,274,278,278,278,277,275,278,277,274,277,281,284,284,279,274,273,271,270,267,267,267,267,268,267,267,267,267,267,267,267,267,267,271,276,280,286,292,297,304,300,293,288,285,278,268,267,266,264,264,263,262,263,263,263,263,263,263,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,263,266,271,274,277,278,280,281,281,281,280,281,278,269,268,269,271,273,274,275,276,277,278,279,280,281,282,282,280,281,281,281,280,280,282,287,288,290,292,294,295,298,303,306,310,311,310,306,302,304,314,322,326,334,347,357,366,371,375,379,382,385,389,392,394,395,390,377,375,374,368,359,352,342,331,317,317,321,329,336,327,327,328,328,329,332,336,343,346,352,353,364,368,365,370,376,381,388,394,401,409,411,406,399,389,388,383,381,391,396,398,396,392,388,380,373,366,356,346,336,331,336,341,341,339,339,337,334,332,330,329,329,330,330,328,328,331,332,339,341,342,332,321,313,312,312,313,313,316,324,327,334,327,325,330,331,332,330,327,317,317,316,313,306,305,305,306,307,307,308,308,309,310,311,311,311,311,312,313,314,315,316,315,315,315,314,314,312,311,310,308,307,305,304,303,303,303,302,302,301,300,300,300,299,300,300,302,303,304,306,308,309,311,312,314,315,316,317,318,320,322,322,322,322,321,320,319,318,316,314,313,310,308,307,306,305,303,302,300,299,298,297,296,295,295,295,294,294,293,292,290,289,286,284,282,283,282,281,280,282,280,281,282,281,281,281,280,279,279,276,274,274,273,273,275,274,277,281,280,279,283,285,285,283,280,283,284,278,277,283,283,287,281,277,274,272,269,267,267,267,267,267,267,267,267,267,267,267,267,267,268,273,280,284,291,293,296,299,297,283,276,267,266,265,264,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,265,271,275,278,280,281,281,282,283,282,282,284,282,274,268,270,272,274,276,277,278,279,280,282,283,284,286,286,285,285,285,284,281,283,288,290,290,291,292,294,296,303,310,316,320,323,324,319,312,306,308,314,319,331,339,346,356,367,370,373,374,375,376,380,387,394,392,384,380,380,377,364,354,341,330,316,321,325,330,333,327,327,328,329,330,334,338,342,352,358,365,368,371,373,373,377,383,390,398,404,413,410,403,398,391,382,376,374,381,385,389,391,388,386,380,374,367,356,346,335,328,332,336,339,342,341,334,330,331,331,330,330,329,330,326,330,330,331,339,342,340,331,318,314,311,312,314,318,326,332,332,330,327,321,323,325,326,326,324,322,318,318,317,306,310,306,306,308,308,309,309,310,311,312,312,312,313,314,315,316,317,318,318,317,316,316,315,313,312,310,308,305,304,304,304,304,303,303,303,302,302,301,301,300,301,302,303,304,305,307,309,310,312,314,315,317,318,320,320,323,324,324,324,323,322,321,320,317,315,312,310,308,305,304,303,302,300,298,297,296,296,295,293,293,294,294,294,294,293,293,290,287,284,281,280,282,281,281,280,280,280,280,282,280,281,283,281,280,278,276,275,274,274,277,280,279,278,282,285,286,289,292,293,292,292,293,293,283,279,282,288,290,287,282,276,271,268,267,267,267,267,267,267,267,267,267,267,267,267,267,267,274,282,282,286,288,285,279,268,267,266,264,263,263,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,271,276,280,281,282,283,283,283,283,284,284,283,282,275,269,269,274,277,279,279,279,280,281,283,285,289,290,291,291,290,289,287,285,288,291,293,292,292,292,292,294,302,310,320,327,333,337,335,328,317,311,310,316,324,332,337,346,357,364,366,365,363,361,364,374,391,392,388,386,383,379,369,355,344,333,316,316,327,331,337,327,328,329,332,337,339,340,343,352,359,367,373,377,380,380,378,384,390,399,404,402,398,394,392,388,378,369,365,372,375,381,381,378,380,379,373,366,356,345,333,327,329,333,337,341,340,333,328,329,332,330,330,329,328,326,329,328,330,331,336,336,325,315,314,309,310,316,323,330,332,332,330,324,317,317,319,321,322,321,320,319,317,315,314,311,310,310,309,309,309,310,311,312,313,314,314,315,316,318,319,322,323,324,322,319,318,316,314,313,311,309,306,306,306,306,305,304,303,304,304,304,302,302,301,302,302,304,304,305,307,309,311,313,314,316,318,320,322,324,326,326,326,326,325,323,321,318,316,313,310,309,307,304,302,300,299,297,295,295,294,293,293,293,294,294,293,293,293,292,291,289,286,284,282,280,280,280,282,281,281,280,280,280,280,282,283,283,282,281,279,279,276,276,280,282,281,278,281,291,294,296,300,301,302,301,299,294,286,280,282,288,293,292,285,279,273,268,267,267,267,267,267,267,267,267,267,267,267,267,267,269,268,268,270,270,268,268,267,267,266,263,262,263,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,267,274,280,281,282,283,284,284,284,285,285,284,283,281,276,275,274,275,278,280,281,281,280,282,284,287,290,292,294,294,294,293,291,288,290,293,298,298,296,296,295,293,299,304,318,330,339,346,347,342,332,320,314,313,318,323,329,335,342,348,350,348,346,352,360,376,386,392,390,391,384,380,369,357,346,333,325,325,321,334,340,327,335,332,338,342,348,348,351,352,356,364,372,380,388,385,385,387,388,391,395,393,388,386,386,385,380,372,363,364,369,371,372,373,376,376,365,361,352,341,330,327,326,330,335,336,334,331,327,328,330,329,330,330,326,320,324,326,328,330,340,331,328,319,307,305,311,317,322,326,328,327,324,320,316,316,316,317,318,318,317,316,315,315,314,314,315,312,310,310,310,311,312,313,314,315,316,317,318,321,324,327,329,330,327,324,321,318,316,314,312,310,309,308,306,305,304,303,304,305,305,304,303,302,302,302,303,304,305,306,308,310,312,314,316,317,319,321,324,326,329,329,328,328,326,325,320,318,315,310,308,307,305,302,299,296,295,293,292,293,292,293,293,292,293,292,292,291,290,290,289,287,285,283,282,281,280,281,280,282,281,280,280,280,280,283,284,285,285,284,280,283,280,278,280,283,284,280,280,289,297,304,308,310,309,306,302,295,290,282,282,289,294,293,287,280,279,275,268,267,267,267,267,267,267,267,267,267,267,267,267,267,267,268,268,268,267,267,267,266,266,263,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,264,267,278,282,283,284,284,285,285,286,286,286,286,284,281,279,278,280,278,279,280,282,284,283,284,287,290,292,294,296,297,298,297,294,290,290,294,304,304,304,306,304,300,297,303,313,326,338,347,355,353,344,335,326,317,314,316,321,326,328,332,334,337,340,350,356,366,374,387,393,389,380,373,364,354,346,334,330,335,333,326,340,329,335,339,342,349,355,358,359,360,360,364,368,380,392,393,390,390,387,384,386,385,378,377,388,384,376,370,363,355,360,362,366,373,374,372,367,366,356,345,335,323,319,330,332,332,330,328,324,324,325,324,325,326,316,316,318,325,330,334,340,342,330,320,309,304,308,313,315,317,319,318,317,316,315,314,313,313,312,311,312,312,314,317,316,317,316,313,311,310,311,312,313,315,316,316,317,319,320,324,328,330,331,330,330,327,324,320,316,315,313,312,311,307,306,304,304,304,305,306,305,304,303,303,302,303,304,304,305,308,309,311,314,316,317,319,321,323,326,328,330,330,331,329,328,323,320,317,311,306,306,305,301,298,296,292,292,291,291,291,291,292,292,292,291,291,290,290,289,288,287,285,284,282,282,281,279,279,279,279,279,279,280,280,280,281,284,287,288,286,284,286,282,280,280,281,285,282,284,291,296,306,315,317,312,307,302,297,293,287,288,292,294,294,290,287,284,280,275,268,267,267,267,267,267,267,267,267,267,267,267,267,267,267,268,268,267,266,266,266,265,264,263,263,262,262,263,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,265,267,271,282,286,284,284,285,286,287,288,288,288,286,282,279,282,281,279,279,279,282,283,284,286,288,289,290,292,297,301,303,302,299,294,291,292,307,310,313,316,312,303,301,300,305,319,334,344,352,358,356,349,341,332,323,323,324,324,327,324,328,336,341,348,352,358,368,387,394,389,378,370,363,353,344,338,342,344,343,335,327,329,335,339,344,353,362,366,368,368,368,365,370,380,392,398,396,392,384,377,376,379,368,368,377,373,363,357,356,350,352,363,371,376,377,374,369,366,355,346,335,322,316,322,327,329,326,322,318,317,320,317,319,317,315,317,324,330,330,333,339,337,327,319,308,305,305,309,310,313,314,314,315,315,314,314,311,310,311,311,311,312,315,316,316,317,316,313,311,311,311,312,313,315,316,317,318,319,322,324,326,329,330,331,331,331,328,322,317,315,313,311,309,307,304,305,305,305,305,305,304,304,303,303,303,303,304,305,306,308,310,312,314,317,319,320,323,325,327,329,332,330,337,343,328,320,314,309,305,303,302,301,298,294,292,291,289,289,289,289,290,290,291,291,291,290,289,288,287,286,284,283,282,279,279,279,279,279,279,279,279,279,279,280,280,280,282,289,292,291,291,292,291,289,288,287,288,289,290,292,299,309,318,316,309,304,300,296,293,291,291,294,296,296,294,293,288,281,280,282,270,269,268,267,267,267,267,267,267,267,267,267,267,267,267,267,266,266,265,265,264,264,263,263,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,265,271,271,266,273,283,283,284,286,287,288,289,290,290,289,284,282,283,283,281,279,278,279,281,284,287,289,290,292,294,296,302,306,306,304,299,292,295,305,315,321,325,318,308,302,302,304,314,324,330,338,349,357,356,354,346,340,341,338,331,326,326,333,342,352,359,366,374,377,388,395,392,384,375,366,356,347,340,350,354,350,342,332,328,335,339,342,352,360,367,371,376,379,376,378,389,396,402,401,389,380,369,365,370,363,360,367,362,351,341,341,340,351,366,378,381,380,377,369,363,352,342,332,319,315,317,320,321,322,316,316,317,317,314,313,316,316,319,327,329,329,330,331,330,323,314,306,304,304,306,308,309,310,312,313,313,313,313,312,310,311,312,311,312,311,313,314,315,314,313,311,311,311,312,314,316,318,319,319,319,322,326,327,329,330,331,331,331,330,324,317,315,312,311,310,308,308,308,307,306,305,305,304,303,303,303,303,303,305,306,307,310,312,314,316,317,320,322,324,327,329,330,329,326,325,325,322,317,310,303,303,301,300,297,294,292,290,289,288,287,287,288,289,289,290,290,289,289,288,286,285,284,283,282,281,279,279,279,279,279,279,279,279,279,279,279,280,281,283,289,295,298,300,301,299,298,296,293,292,293,294,295,297,310,312,308,304,301,296,294,292,291,291,294,298,301,298,296,289,290,291,294,293,286,269,267,267,266,266,266,266,266,266,266,266,266,266,266,266,265,264,264,263,263,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,267,273,287,281,273,267,276,280,283,285,288,290,291,291,292,292,290,287,286,285,283,280,280,281,282,287,289,291,293,295,297,300,302,304,308,309,305,298,300,312,321,329,333,324,309,306,308,309,312,316,323,327,332,345,359,363,360,358,357,351,341,328,328,337,350,360,370,380,390,395,397,395,391,383,374,364,354,349,352,358,359,355,346,336,329,335,341,343,351,352,360,365,376,382,385,386,392,399,400,394,386,373,363,355,361,356,349,354,349,338,329,334,345,358,371,381,381,382,377,368,361,350,341,332,322,315,312,314,319,318,315,315,314,312,312,313,315,318,321,322,324,327,325,320,318,316,308,304,303,304,306,305,305,308,310,311,311,312,311,311,313,315,312,311,310,311,312,311,312,312,312,311,311,311,311,311,314,317,317,318,319,323,328,329,327,330,330,331,331,331,326,317,315,313,311,310,311,311,310,309,308,306,304,304,303,303,303,303,304,306,307,309,311,313,315,317,319,321,323,325,328,331,331,329,323,317,318,316,314,312,305,301,300,297,294,292,290,289,288,287,286,287,287,288,288,289,288,288,287,286,285,284,283,281,281,279,279,282,284,284,284,285,286,286,285,279,279,279,280,280,288,296,304,307,308,307,306,302,299,298,299,297,298,303,302,306,305,302,298,294,291,290,289,291,293,299,303,304,297,291,294,298,302,300,289,279,278,272,266,266,266,266,266,266,266,266,266,266,265,265,264,264,263,263,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,263,264,266,266,272,276,284,284,284,284,276,273,279,280,284,289,290,292,293,294,294,292,290,290,288,284,281,280,283,285,287,290,292,295,297,300,304,304,304,309,310,308,303,304,313,326,337,340,328,313,313,318,317,319,327,333,339,344,348,356,367,370,368,367,358,345,330,326,338,352,362,374,387,398,407,404,394,387,377,370,363,354,353,359,365,365,358,349,339,329,337,342,352,356,356,361,364,370,380,391,392,394,395,392,386,379,371,359,349,348,347,338,341,336,329,327,338,351,364,373,381,382,379,372,366,356,347,340,332,323,316,314,314,316,315,315,312,311,310,311,312,315,316,317,321,326,327,326,320,316,309,305,303,303,304,306,305,305,306,308,309,309,310,312,314,317,315,311,310,309,310,310,311,313,311,311,311,310,310,311,311,311,312,315,316,317,318,324,326,326,328,329,330,331,332,327,318,315,313,312,310,310,311,312,310,309,305,304,304,303,303,303,304,305,307,308,310,312,314,316,318,320,322,324,326,328,330,331,328,322,316,314,314,311,307,304,301,297,295,292,291,290,288,287,287,286,286,286,287,287,288,288,287,286,285,283,282,281,280,280,280,281,284,286,289,288,290,291,291,291,288,284,279,278,278,284,294,299,304,305,306,306,305,302,301,304,304,303,302,304,305,302,299,295,292,289,288,289,290,293,300,305,307,302,299,302,307,309,300,288,279,276,273,271,269,265,265,265,265,265,265,265,265,264,264,264,263,263,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,263,263,264,265,267,269,277,280,288,286,286,283,280,278,279,282,286,291,292,293,295,295,295,294,293,294,291,287,282,284,286,289,291,291,292,296,300,304,306,308,308,310,315,315,310,313,322,332,344,344,330,315,320,328,328,329,337,344,349,354,357,363,371,375,375,367,356,345,332,336,348,353,360,365,382,399,411,408,401,391,382,375,366,358,361,367,371,367,359,350,340,335,341,348,352,354,362,369,374,377,379,388,392,392,389,383,377,369,366,356,343,338,338,329,328,324,325,328,338,347,356,368,379,378,372,362,356,349,341,335,330,323,316,312,313,314,312,310,310,309,309,310,312,314,314,317,323,328,330,330,326,317,306,303,303,303,303,305,305,305,305,306,306,309,311,312,315,315,314,311,307,307,308,310,311,311,311,310,308,307,308,308,309,310,311,312,313,315,318,320,322,323,326,329,330,331,332,327,318,315,313,311,310,311,311,312,311,309,306,304,304,303,303,304,304,306,308,309,311,313,315,317,319,321,323,325,327,329,333,330,325,320,314,309,304,305,304,302,299,297,294,292,291,289,288,287,286,286,286,286,286,287,287,287,287,286,285,281,281,280,281,282,283,285,288,290,292,296,296,297,296,295,293,289,284,280,278,278,286,294,297,299,301,302,303,302,301,304,305,304,304,303,300,296,295,292,289,287,287,288,291,295,301,306,309,309,306,313,318,308,297,284,278,274,271,269,268,268,268,265,265,265,265,265,264,264,263,263,263,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,262,263,262,263,263,264,265,266,268,268,279,282,284,286,286,283,280,280,282,285,290,293,294,295,296,296,296,296,296,296,290,289,290,290,290,293,293,293,292,295,300,305,308,312,314,314,317,320,315,317,330,339,346,348,336,320,325,332,336,338,344,351,356,360,364,369,378,383,381,368,355,346,339,343,359,367,374,377,381,397,408,411,407,398,388,377,365,365,370,375,375,366,358,350,343
};*/ // ATASCADERO ENDS HERE
















































































/************************
 INITIAL SET UP FUNCTIONS
 ************************/
/* Function name: setUpView()
   Description:   Sets up the initial view when executing the program      
*/
void setUpView() {
   // this code initializes the viewing transform
   glLoadIdentity();

   camera.apply();

   // moves viewer along coordinate axes
   gluLookAt(0.0, 0.0, 5.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

   // move the view back some relative to viewer[] position
   glTranslatef(0.0f,0.0f,5.0f);

   // rotates view
   glRotatef(0, 1.0, 0.0, 0.0);
   glRotatef(0, 0.0, 1.0, 0.0);
   glRotatef(0, 0.0, 0.0, 1.0);
    
   return;
}

/* Function name: setUpModelTransform()
   Description:   Sets up the initial model transform.
*/
void setUpModelTransform() {
   // moves model along coordinate axes
   glTranslatef(modelTrans[0], modelTrans[1], modelTrans[2]);

   // rotates model
   glRotatef(theta[0], 1.0, 0.0, 0.0);
   glRotatef(theta[1], 0.0, 1.0, 0.0);
   glRotatef(theta[2], 0.0, 0.0, 1.0);
}


/* Function name: setUpLight()
   Description:   Sets up the light source.
*/
void setUpLight() {
   GLfloat lightDir1[] = {0.0, 0.0, 0.0, 1.0};
   GLfloat diffuseComp[] = {1.0, 1.0, 1.0, 1.0};

   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);

   glLightfv(GL_LIGHT0, GL_POSITION, lightDir1);
   glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseComp);
   return;
}


/******************************
 RENDERING LANDSCAPE FUNCTIONS
 ******************************/

/* Function name: initializeCoord()
   Description:   Renders the initial landscape and calls the
                  recursive function to build the coordinates
*/
void initializeCoord(){
   for(int i=0; i < MATRIX_MAX; i++){
      for(int j=0; j < MATRIX_MAX; j++){
         coord[i][j][X] = i*30;
         coord[i][j][Y] = j*30;
         coord[i][j][Z] = zValues[i*MATRIX_MAX+j];
      }
   }
}

/* Function name: displayLandscape()
   Description:   After calculating the coordinates of the terrain,
                  the function snitches the coordinates together
                  creating grids and forming the landscape.
*/
void displayLandscape(){
    //printf("displayLandscape\n");
    int index = 1;
	GLfloat tLeft[3];
	GLfloat tRight[3];
	GLfloat bRight[3];
	GLfloat bLeft[3];
	GLfloat out[3];
	GLfloat vec1[3];
	GLfloat vec2[3];

	glPushMatrix();
	glTranslatef(-0.5, 0, 0);
	glMatrixMode(GL_MODELVIEW);
	glBegin(GL_QUADS);

	for(int i = 0; i < MATRIX_MAX-1; i++){
		for(int j = 0; j < MATRIX_MAX-1; j++){
         // Take in the top left coordinates
         tLeft[X] = (coord[i][j][X]*1.0/60.0)/50.0;
         tLeft[Y] = (coord[i][j][Y]*1.0/60.0)/50.0;
         tLeft[Z] = coord[i][j][Z]/ELEV_DIVISOR;

         // Take in the top right coordinates
         tRight[X] = (coord[i][j+index][X]*1.0/60.0)/50.0;
         tRight[Y] = (coord[i][j+index][Y]*1.0/60.0)/50.0;
         tRight[Z] = coord[i][j+index][Z]/ELEV_DIVISOR;

         // Take in the bottom right coordinates
         bRight[X] = (coord[i+index][j+index][X]*1.0/60.0)/50.0;
         bRight[Y] = (coord[i+index][j+index][Y]*1.0/60.0)/50.0;
         bRight[Z] = coord[i+index][j+index][Z]/ELEV_DIVISOR;

         // Take in the bottom left coordinates
         bLeft[X] = (coord[i+index][j][X]*1.0/60.0)/50.0;
         bLeft[Y] = (coord[i+index][j][Y]*1.0/60.0)/50.0;
         bLeft[Z] = coord[i+index][j][Z]/ELEV_DIVISOR;

         // Get the top left shading
         getShading(tLeft, out);
         glMaterialfv(GL_FRONT, GL_DIFFUSE, out);

         // Otherwise, calculate the cross product
         for(int k=0; k<3; k++) {
	         vec1[k] = tLeft[k]-tRight[k];
	         vec2[k] = tLeft[k]-bLeft[k];
             normCrossProd(vec1, vec2, out);
         }
         
         // Apply the normal to all four coordinates
         glNormal3fv(out);
         glVertex3fv(tLeft);

//         getShading(tRight, out);
//         glMaterialfv(GL_FRONT, GL_DIFFUSE, out);
         glVertex3fv(tRight);

//         getShading(bRight, out);
//         glMaterialfv(GL_FRONT, GL_DIFFUSE, out);
         glVertex3fv(bRight);

//         getShading(bLeft, out);
//         glMaterialfv(GL_FRONT, GL_DIFFUSE, out);
         glVertex3fv(bLeft);
      }
   }
   glEnd();
   glPopMatrix();
}

void displayDistancePath(){
	if(dist[end_x][end_y] != myINFINITY){
		int index = 1;
		GLfloat tLeft[3];
		GLfloat tRight[3];
		GLfloat bRight[3];
		GLfloat bLeft[3];
		GLfloat out[3];
		GLfloat vec1[3];
		GLfloat vec2[3];

		glPushMatrix();
		glTranslatef(-0.5, 0, 0);
		glMatrixMode(GL_MODELVIEW);
		glBegin(GL_QUADS);

	   int currentX = end_x;
	   int currentY = end_y;
	   int newX = end_x;
	   int newY = end_y;

	   while (currentX != start_x || currentY != start_y){
		  if(currentX < MATRIX_MAX-1 && currentY < MATRIX_MAX-1){

			 // Take in the top left coordinates
			 tLeft[X] = (coord[currentX][currentY][X]*1.0/60.0)/50.0+PATH_WIDTH;
			 tLeft[Y] = (coord[currentX][currentY][Y]*1.0/60.0)/50.0+PATH_WIDTH;
			 tLeft[Z] = coord[currentX][currentY][Z]/ELEV_DIVISOR+PATH_HEIGHT_UP;

			 // Take in the top right coordinates
			 tRight[X] = (coord[currentX][currentY+index][X]*1.0/60.0)/50.0+PATH_WIDTH;
			 tRight[Y] = (coord[currentX][currentY+index][Y]*1.0/60.0)/50.0-PATH_WIDTH;
			 tRight[Z] = coord[currentX][currentY+index][Z]/ELEV_DIVISOR+PATH_HEIGHT_UP;

			 // Take in the bottom right coordinates
			 bRight[X] = (coord[currentX+index][currentY+index][X]*1.0/60.0)/50.0-PATH_WIDTH;
			 bRight[Y] = (coord[currentX+index][currentY+index][Y]*1.0/60.0)/50.0-PATH_WIDTH;
			 bRight[Z] = coord[currentX+index][currentY+index][Z]/ELEV_DIVISOR+PATH_HEIGHT_UP;

			 // Take in the bottom left coordinates
			 bLeft[X] = (coord[currentX+index][currentY][X]*1.0/60.0)/50.0-PATH_WIDTH;
			 bLeft[Y] = (coord[currentX+index][currentY][Y]*1.0/60.0)/50.0+PATH_WIDTH;
			 bLeft[Z] = coord[currentX+index][currentY][Z]/ELEV_DIVISOR+PATH_HEIGHT_UP;

			 // Get the top left shading
			 getDistancePathShading(tLeft, out);
			 glMaterialfv(GL_FRONT, GL_DIFFUSE, out);

				// Otherwise, calculate the cross product
				for(int k=0; k<3; k++) {
					 vec1[k] = tLeft[k]-tRight[k];
					 vec2[k] = tLeft[k]-bLeft[k];
				   normCrossProd(vec1, vec2, out);
				}

			 // Apply the normal to all four coordinates
			 glNormal3fv(out);
			 glVertex3fv(tLeft);

			 getDistancePathShading(tRight, out);
//			 glMaterialfv(GL_FRONT, GL_DIFFUSE, out);
			 glVertex3fv(tRight);

			 getDistancePathShading(bRight, out);
//			 glMaterialfv(GL_FRONT, GL_DIFFUSE, out);
			 glVertex3fv(bRight);

			 getDistancePathShading(bLeft, out);
//			 glMaterialfv(GL_FRONT, GL_DIFFUSE, out);
			 glVertex3fv(bLeft);
		  }
		  newX = dist_info[currentX][currentY][PREV_X];
		  newY = dist_info[currentX][currentY][PREV_Y];
		  currentX = newX;
		  currentY = newY;
	   }

	   // Meters to miles
	   float k = dist[end_x][end_y]/1609.344;
	   float k1 = 0.075283 - 0.0015892*speed + 0.000015066*speed*speed;
	   printf("\nDISTANCE DATA\nTotal Distance Path:\t%f miles\nTotal Fuel Consumption:\t%f gallons\n", k, k*k1);

	   glEnd();
	   glPopMatrix();
	} else {
		printf("\tUNREACHABLE DISTANCE PATH\n");
	}
}

void displayEnergyPath(){
	if(energy[end_x][end_y] != myINFINITY){
		int index = 1;
		GLfloat tLeft[3];
		GLfloat tRight[3];
		GLfloat bRight[3];
		GLfloat bLeft[3];
		GLfloat out[3];
		GLfloat vec1[3];
		GLfloat vec2[3];

		glPushMatrix();
		glTranslatef(-0.5, 0, 0);
		glMatrixMode(GL_MODELVIEW);
		glBegin(GL_QUADS);

	   int currentX = end_x;
	   int currentY = end_y;
	   int newX = end_x;
	   int newY = end_y;

	   while (currentX != start_x || currentY != start_y){
		  if(currentX < MATRIX_MAX-1 && currentY < MATRIX_MAX-1){

			 // Take in the top left coordinates
			 tLeft[X] = (coord[currentX][currentY][X]*1.0/60.0)/50.0+PATH_WIDTH;
			 tLeft[Y] = (coord[currentX][currentY][Y]*1.0/60.0)/50.0+PATH_WIDTH;
			 tLeft[Z] = coord[currentX][currentY][Z]/ELEV_DIVISOR+PATH_HEIGHT_UP;

			 // Take in the top right coordinates
			 tRight[X] = (coord[currentX][currentY+index][X]*1.0/60.0)/50.0+PATH_WIDTH;
			 tRight[Y] = (coord[currentX][currentY+index][Y]*1.0/60.0)/50.0-PATH_WIDTH;
			 tRight[Z] = coord[currentX][currentY+index][Z]/ELEV_DIVISOR+PATH_HEIGHT_UP;

			 // Take in the bottom right coordinates
			 bRight[X] = (coord[currentX+index][currentY+index][X]*1.0/60.0)/50.0-PATH_WIDTH;
			 bRight[Y] = (coord[currentX+index][currentY+index][Y]*1.0/60.0)/50.0-PATH_WIDTH;
			 bRight[Z] = coord[currentX+index][currentY+index][Z]/ELEV_DIVISOR+PATH_HEIGHT_UP;

			 // Take in the bottom left coordinates
			 bLeft[X] = (coord[currentX+index][currentY][X]*1.0/60.0)/50.0-PATH_WIDTH;
			 bLeft[Y] = (coord[currentX+index][currentY][Y]*1.0/60.0)/50.0+PATH_WIDTH;
			 bLeft[Z] = coord[currentX+index][currentY][Z]/ELEV_DIVISOR+PATH_HEIGHT_UP;

			 // Get the top left shading
			 getEnergyPathShading(tLeft, out);
			 glMaterialfv(GL_FRONT, GL_DIFFUSE, out);

				// Otherwise, calculate the cross product
				for(int k=0; k<3; k++) {
					 vec1[k] = tLeft[k]-tRight[k];
					 vec2[k] = tLeft[k]-bLeft[k];
				   normCrossProd(vec1, vec2, out);
				}

			 // Apply the normal to all four coordinates
			 glNormal3fv(out);
			 glVertex3fv(tLeft);

			 getEnergyPathShading(tRight, out);
//			 glMaterialfv(GL_FRONT, GL_DIFFUSE, out);
			 glVertex3fv(tRight);

			 getEnergyPathShading(bRight, out);
//			 glMaterialfv(GL_FRONT, GL_DIFFUSE, out);
			 glVertex3fv(bRight);

			 getEnergyPathShading(bLeft, out);
//			 glMaterialfv(GL_FRONT, GL_DIFFUSE, out);
			 glVertex3fv(bLeft);
		  }
		  newX = energy_info[currentX][currentY][PREV_X];
		  newY = energy_info[currentX][currentY][PREV_Y];
		  currentX = newX;
		  currentY = newY;
	   }

	   float k = energy[end_x][end_y];
	   printf("\nENERGY DATA\nTotal Energy Path:\t%f joules\n", k);

	   glEnd();
	   glPopMatrix();
	} else {
		printf("\tUNREACHABLE ENERGY PATH\n");
	}
}

void displayDirectPath(){
		int index = 1;
		GLfloat tLeft[3];
		GLfloat tRight[3];
		GLfloat bRight[3];
		GLfloat bLeft[3];
		GLfloat out[3];
		GLfloat vec1[3];
		GLfloat vec2[3];

		glPushMatrix();
		glTranslatef(-0.5, 0, 0);
		glMatrixMode(GL_MODELVIEW);
		glBegin(GL_QUADS);

		float slope = ((end_y-start_y)*1.0/(end_x-start_x)*1.0);

	   int currentX = end_x;
	   int currentY = end_y;

	   for(int i = start_x; i <= end_x; i++){
		   currentX = i;
		   currentY = (int)(slope*(i-start_x)+start_y);

			 // Take in the top left coordinates
			 tLeft[X] = (coord[currentX][currentY][X]*1.0/60.0)/50.0+PATH_WIDTH;
			 tLeft[Y] = (coord[currentX][currentY][Y]*1.0/60.0)/50.0+PATH_WIDTH;
			 tLeft[Z] = coord[currentX][currentY][Z]/ELEV_DIVISOR+PATH_HEIGHT_UP;

			 // Take in the top right coordinates
			 tRight[X] = (coord[currentX][currentY+index][X]*1.0/60.0)/50.0+PATH_WIDTH;
			 tRight[Y] = (coord[currentX][currentY+index][Y]*1.0/60.0)/50.0-PATH_WIDTH;
			 tRight[Z] = coord[currentX][currentY+index][Z]/ELEV_DIVISOR+PATH_HEIGHT_UP;

			 // Take in the bottom right coordinates
			 bRight[X] = (coord[currentX+index][currentY+index][X]*1.0/60.0)/50.0-PATH_WIDTH;
			 bRight[Y] = (coord[currentX+index][currentY+index][Y]*1.0/60.0)/50.0-PATH_WIDTH;
			 bRight[Z] = coord[currentX+index][currentY+index][Z]/ELEV_DIVISOR+PATH_HEIGHT_UP;

			 // Take in the bottom left coordinates
			 bLeft[X] = (coord[currentX+index][currentY][X]*1.0/60.0)/50.0-PATH_WIDTH;
			 bLeft[Y] = (coord[currentX+index][currentY][Y]*1.0/60.0)/50.0+PATH_WIDTH;
			 bLeft[Z] = coord[currentX+index][currentY][Z]/ELEV_DIVISOR+PATH_HEIGHT_UP;

			 // Get the top left shading
			 getDirectPathShading(tLeft, out);
			 glMaterialfv(GL_FRONT, GL_DIFFUSE, out);

				// Otherwise, calculate the cross product
				for(int k=0; k<3; k++) {
					 vec1[k] = tLeft[k]-tRight[k];
					 vec2[k] = tLeft[k]-bLeft[k];
				   normCrossProd(vec1, vec2, out);
				}

			 // Apply the normal to all four coordinates
			 glNormal3fv(out);
			 glVertex3fv(tLeft);

			 getDirectPathShading(tRight, out);
//			 glMaterialfv(GL_FRONT, GL_DIFFUSE, out);
			 glVertex3fv(tRight);

			 getDirectPathShading(bRight, out);
//			 glMaterialfv(GL_FRONT, GL_DIFFUSE, out);
			 glVertex3fv(bRight);

			 getDirectPathShading(bLeft, out);
//			 glMaterialfv(GL_FRONT, GL_DIFFUSE, out);
			 glVertex3fv(bLeft);
	   }

	   float k = sqrt(float((start_x-end_x)*(start_x-end_x) + (start_y-end_y)*(start_y-end_y)));
	   printf("Total Direct Path:\t%f meters\n", k);

	   glEnd();
	   glPopMatrix();
}
/* Function name: getShading()
   Description:   Takes in the given coordinates and analyze the
                  y-value to determine what type of shading it
                  will receive. It will return the color values
                  in the out. 
   Parameters:    coord - the current coordinates being passed
                  out - the color array cooresponding the altitude
*/
void getShading(GLfloat coord[3], GLfloat out[3]){
   int x = (int)(coord[X]*2.0*50.0);
   int y = (int)(coord[Y]*2.0*50.0);

   GLfloat distance = dist[x][y];
   if(distance == myINFINITY){
      out[0] = 1.0;
      out[1] = 0.0;
      out[2] = 0.0;
      return;
   }

   if(mode == DISTANCE_MODE){
	   for(int i=0; i < distRings; i++){
		  out[0] = (255.0-i*(255.0/distRings))/255.0;
		  out[1] = (255.0-i*(255.0/distRings))/255.0;
		  out[2] = (255.0-i*(255.0/distRings))/255.0;

		  if(dist[x][y] < (i+1)*distInc){
			 return;
		  }
	   }
   }

   if (mode == ENERGY_MODE){
	   for(int i=0; i < energyRings; i++){
		  out[0] = (255.0-i*(255.0/distRings))/255.0;
		  out[1] = (255.0-i*(255.0/distRings))/255.0;
		  out[2] = (255.0-i*(255.0/distRings))/255.0;

		  if(energy[x][y] < (i+1)*energyInc){
			 return;
		  }
	   }
   }
  out[0] = 1.0;
  out[1] = 1.0;
  out[2] = 1.0;
}

void getDistancePathShading(GLfloat coord[3], GLfloat out[3]){
   out[0] = 0.0; out[1] = 0.0; out[2] = 1.0;
}

void getEnergyPathShading(GLfloat coord[3], GLfloat out[3]){
   out[0] = 0.0; out[1] = 1.0; out[2] = 0.0;
}

void getDirectPathShading(GLfloat coord[3], GLfloat out[3]){
   out[0] = 0.0; out[1] = 1.0; out[2] = 1.0;
}


void runDijkstra(){
   for(int i=0; i < MATRIX_MAX; i++){
      for(int j=0; j < MATRIX_MAX; j++){
         dist[i][j] = myINFINITY;
		 dist_info[i][j][PREV_X] = myUNDEFINED;
         dist_info[i][j][PREV_Y] = myUNDEFINED;
         dist_info[i][j][VISITED] = NO;

		 energy[i][j] = myINFINITY;
         energy_info[i][j][PREV_X] = myUNDEFINED;
         energy_info[i][j][PREV_Y] = myUNDEFINED;
         energy_info[i][j][VISITED] = NO;
      }
   }

   // STARTING VERTEX
   dist[start_x][start_y] = 0;
   energy[start_x][start_y] = 0;
   int startX = start_x;
   int startY = start_y;         
   
   processVertex(startX, startY);
   for(int i=1; i < MATRIX_MAX*2; i = i+2){

      // RIGHT
      for(int j=0; j < i; j++){
         if(startX+1 < MATRIX_MAX && startX+1 >= 0 && startY < MATRIX_MAX && startY >= 0){
            processVertex(startX+1, startY);
         }
         startX++;
      }

      // UP
      for(int j=0; j < i; j++){
         if(startX < MATRIX_MAX && startX >= 0 && startY+1 < MATRIX_MAX && startY+1 >= 0){
            processVertex(startX, startY+1);
         }
         startY++;
      }

      // LEFT
      for(int j=0; j < i+1; j++){
         if(startX-1 < MATRIX_MAX && startX-1 >= 0 && startY < MATRIX_MAX && startY >= 0){
            processVertex(startX-1, startY);
         }
         startX--;
      }

      // DOWN
      for(int j=0; j < i+1; j++){
         if(startX < MATRIX_MAX && startX >= 0 && startY-1 < MATRIX_MAX && startY-1 >= 0){
            processVertex(startX, startY-1);
         }
         startY--;
      }
   }
}




void processVertex(int x, int y){
   GLfloat distance = 0;
   GLfloat joules = 0;

      if(x+1 < MATRIX_MAX){
         //////////////
         // X+1, Y+1
         //////////////
         if(y+1 < MATRIX_MAX){
            distance = calcDistance(   coord[x][y][X], coord[x+1][y+1][X],
                                       coord[x][y][Y], coord[x+1][y+1][Y],
                                       coord[x][y][Z], coord[x+1][y+1][Z]);
            if(checkGrade(sqrt(2.0), distance)){
               if(dist[x+1][y+1] > dist[x][y] + distance){
                  dist[x+1][y+1] = dist[x][y] + distance;
                  dist_info[x+1][y+1][PREV_X] = x;
                  dist_info[x+1][y+1][PREV_Y] = y;
               }
			   joules = carWeight*GRAVITY*fabs(coord[x+1][y+1][Z]-coord[x][y][Z]);
			   if(energy[x+1][y+1] > energy[x][y] + joules){
                  energy[x+1][y+1] = energy[x][y] + joules;
                  energy_info[x+1][y+1][PREV_X] = x;
                  energy_info[x+1][y+1][PREV_Y] = y;
			   }
            }
         }
         //////////////
         // X+1, Y
         //////////////
         distance = calcDistance(   coord[x][y][X], coord[x+1][y][X],
                                    coord[x][y][Y], coord[x+1][y][Y],
                                    coord[x][y][Z], coord[x+1][y][Z]);
         if(checkGrade(1.0, distance)){
            if(dist[x+1][y] > dist[x][y] + distance){
               dist[x+1][y] = dist[x][y] + distance;
               dist_info[x+1][y][PREV_X] = x;
               dist_info[x+1][y][PREV_Y] = y;
            }
			   joules = carWeight*GRAVITY*fabs(coord[x+1][y][Z]-coord[x][y][Z]);
			   if(energy[x+1][y] > energy[x][y] + joules){
				  energy[x+1][y] = energy[x][y] + joules;
				  energy_info[x+1][y][PREV_X] = x;
				  energy_info[x+1][y][PREV_Y] = y;
			   }
         }
         //////////////
         // X+1, Y-1
         //////////////
         if(y-1 >= 0){
            distance = calcDistance(   coord[x][y][X], coord[x+1][y-1][X],
                                       coord[x][y][Y], coord[x+1][y-1][Y],
                                       coord[x][y][Z], coord[x+1][y-1][Z]);
            if(checkGrade(sqrt(2.0), distance)){
               if(dist[x+1][y-1] > dist[x][y] + distance){
                  dist[x+1][y-1] = dist[x][y] + distance;
                  dist_info[x+1][y-1][PREV_X] = x;
                  dist_info[x+1][y-1][PREV_Y] = y;
               }
			   joules = carWeight*GRAVITY*fabs(coord[x+1][y-1][Z]-coord[x][y][Z]);
			   if(energy[x+1][y-1] > energy[x][y] + joules){
                  energy[x+1][y-1] = energy[x][y] + joules;
                  energy_info[x+1][y-1][PREV_X] = x;
                  energy_info[x+1][y-1][PREV_Y] = y;
			   }
            }
         }
      }
      //////////////
      // X, Y+1
      //////////////
      if(y+1 < MATRIX_MAX){
         distance = calcDistance(   coord[x][y][X], coord[x][y+1][X],
                                    coord[x][y][Y], coord[x][y+1][Y],
                                    coord[x][y][Z], coord[x][y+1][Z]);
         if(checkGrade(1.0, distance)){
            if(dist[x][y+1] > dist[x][y] + distance){
               dist[x][y+1] = dist[x][y] + distance;
               dist_info[x][y+1][PREV_X] = x;
               dist_info[x][y+1][PREV_Y] = y;
            }
			   joules = carWeight*GRAVITY*fabs(coord[x][y+1][Z]-coord[x][y][Z]);
			   if(energy[x][y+1] > energy[x][y] + joules){
                  energy[x][y+1] = energy[x][y] + joules;
                  energy_info[x][y+1][PREV_X] = x;
                  energy_info[x][y+1][PREV_Y] = y;
			   }
         }
      }
      //////////////
      // X, Y-1
      //////////////
      if(y-1 >= 0){
         distance = calcDistance(   coord[x][y][X], coord[x][y-1][X],
                                    coord[x][y][Y], coord[x][y-1][Y],
                                    coord[x][y][Z], coord[x][y-1][Z]);
         if(checkGrade(1.0, distance)){
            if(dist[x][y-1] > dist[x][y] + distance){
               dist[x][y-1] = dist[x][y] + distance;
               dist_info[x][y-1][PREV_X] = x;
               dist_info[x][y-1][PREV_Y] = y;
            }
			   joules = carWeight*GRAVITY*fabs(coord[x][y-1][Z]-coord[x][y][Z]);
			   if(energy[x][y-1] > energy[x][y] + joules){
                  energy[x][y-1] = energy[x][y] + joules;
                  energy_info[x][y-1][PREV_X] = x;
                  energy_info[x][y-1][PREV_Y] = y;
			   }
         }
      }

      if(x-1 >= 0){
         //////////////
         // X-1, Y+1
         //////////////
         if(y+1 < MATRIX_MAX){
            distance = calcDistance(   coord[x][y][X], coord[x-1][y+1][X],
                                       coord[x][y][Y], coord[x-1][y+1][Y],
                                       coord[x][y][Z], coord[x-1][y+1][Z]);
            if(checkGrade(sqrt(2.0), distance)){
               if(dist[x-1][y+1] > dist[x][y] + distance){
                  dist[x-1][y+1] = dist[x][y] + distance;
                  dist_info[x-1][y+1][PREV_X] = x;
                  dist_info[x-1][y+1][PREV_Y] = y;
               }
            }
			joules = carWeight*GRAVITY*fabs(coord[x-1][y+1][Z]-coord[x][y][Z]);
			   if(energy[x-1][y+1] > energy[x][y] + joules){
                  energy[x-1][y+1] = energy[x][y] + joules;
                  energy_info[x-1][y+1][PREV_X] = x;
                  energy_info[x-1][y+1][PREV_Y] = y;
			   }
         }
         //////////////
         // X-1, Y
         //////////////
         distance = calcDistance(   coord[x][y][X], coord[x-1][y][X],
                                    coord[x][y][Y], coord[x-1][y][Y],
                                    coord[x][y][Z], coord[x-1][y][Z]);
         if(checkGrade(1.0, distance)){
            if(dist[x-1][y] > dist[x][y] + distance){
               dist[x-1][y] = dist[x][y] + distance;
               dist_info[x-1][y][PREV_X] = x;
               dist_info[x-1][y][PREV_Y] = y;
            }
			   joules = carWeight*GRAVITY*fabs(coord[x-1][y][Z]-coord[x][y][Z]);
			   if(energy[x-1][y] > energy[x][y] + joules){
                  energy[x-1][y] = energy[x][y] + joules;
                  energy_info[x-1][y][PREV_X] = x;
                  energy_info[x-1][y][PREV_Y] = y;
			   }
         }
         //////////////
         // X-1, Y-1
         //////////////
         if(y-1 >= 0){
            distance = calcDistance(   coord[x][y][X], coord[x-1][y-1][X],
                                       coord[x][y][Y], coord[x-1][y-1][Y],
                                       coord[x][y][Z], coord[x-1][y-1][Z]);
            if(checkGrade(sqrt(2.0), distance)){
               if(dist[x-1][y-1] > dist[x][y] + distance){
                  dist[x-1][y-1] = dist[x][y] + distance;
                  dist_info[x-1][y-1][PREV_X] = x;
                  dist_info[x-1][y-1][PREV_Y] = y;
               }
			   joules = carWeight*GRAVITY*fabs(coord[x-1][y-1][Z]-coord[x][y][Z]);
			   if(energy[x-1][y-1] > energy[x][y] + joules){
                  energy[x-1][y-1] = energy[x][y] + joules;
                  energy_info[x-1][y-1][PREV_X] = x;
                  energy_info[x-1][y-1][PREV_Y] = y;
			   }
            }
         }
      }
}

int checkGrade(GLfloat base, GLfloat hyp){
   if(acos(base*30/hyp) <= atan(GRADE_PERCENT_MAX/100.0)) return 1;
   return 0;
}

// Calcuate between two 3-D points
GLfloat calcDistance(GLfloat startX, GLfloat endX, GLfloat startY, GLfloat endY, GLfloat startZ, GLfloat endZ){
   GLfloat zDelta = fabs(endZ - startZ);
   GLfloat yDelta = fabs(endY - startY);
   GLfloat xDelta = fabs(endX - startX);
   return( sqrt(zDelta*zDelta + xDelta*xDelta + yDelta*yDelta) );
}

void printData(){
   // Print the distance table after Dijkstra's Algorithm
   ofstream myfile;
   myfile.open("distanceTable.txt");
   myfile << "X\tY\tZ\tDIST\n";
   for(int i=0; i < MATRIX_MAX; i++){
      for(int j=0; j < MATRIX_MAX; j++){
         myfile << coord[i][j][X] << "\t" << coord[i][j][Y] << "\t" << coord[i][j][Z] << "\t" << dist[i][j] << "\n";
      }
   }
   myfile.close();

   // Print the path of previous distance vertices after Dijkstra's Algorithm
   ofstream prevfile;
   prevfile.open("previousDistanceVertices.txt");
   if(dist[end_x][end_y] != myINFINITY){
	   int currentX = end_x;
	   int currentY = end_y;
	   int newX = end_x;
	   int newY = end_y;
	   while ( currentX != start_x || currentY != start_y ){
		  prevfile << currentX << "\t" << currentY << "\t" << dist[currentX][currentY] << "\n";
		  newX = dist_info[currentX][currentY][PREV_X];
		  newY = dist_info[currentX][currentY][PREV_Y];
		  currentX = newX;
		  currentY = newY;
	   }
   }
   prevfile.close();

   // Print the path of previous energy vertices after Dijkstra's Algorithm
   ofstream energyfile;
   energyfile.open("previousEnergyVertices.txt");
   if(energy[end_x][end_y] != myINFINITY){
	   int currentX = end_x;
	   int currentY = end_y;
	   int newX = end_x;
	   int newY = end_y;
	   while ( currentX != start_x || currentY != start_y ){
		  energyfile << currentX << "\t" << currentY << "\t" << energy[currentX][currentY] << "\n";
		  newX = energy_info[currentX][currentY][PREV_X];
		  newY = energy_info[currentX][currentY][PREV_Y];
		  currentX = newX;
		  currentY = newY;
	   }
   }
   energyfile.close();
}









static bool dirty;
static int mList;
int main(int argc, char* argv[]) {
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
   glutInitWindowSize(500, 500);
   glutInitWindowPosition(100, 100);
   glutCreateWindow("Landscape (John Vu)");

   dirty = true;
   mList = glGenLists(1);
   initializeCoord();
   runDijkstra();

   glEnable(GL_DEPTH_TEST);
   glutTimerFunc(10, timer, 0);
   glutDisplayFunc(display);
   glutReshapeFunc(reshapeCallback);
   glutKeyboardFunc(keyCallback);
   glutKeyboardUpFunc(keyUpCallback);
   glutMouseFunc(mouseCallback);
   glutMotionFunc(mouseMotionCallback);
   glutMainLoop();
   return 0;
}


void display(void) {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	setUpView();
	setUpLight();
	setUpModelTransform();


	if(dirty){
        //printf("dirty\n");
        glDeleteLists(mList, 1);
        mList = glGenLists(1);
		glNewList(mList, GL_COMPILE_AND_EXECUTE);

		displayLandscape();
		if(mode == DISTANCE_MODE) displayDistancePath();
		if(mode == ENERGY_MODE) displayEnergyPath();
		displayDirectPath();

		glEndList();
		dirty = false;
	} else {
         glCallList(mList);
    }

	glutSwapBuffers();
}

void normalize(GLfloat v[3]) {
   GLfloat d = sqrt(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]);
   if (d == 0.0) {
      printf("Error: Zero Length Vector\n");
      return;
   }
   v[0] = v[0]/d; v[1] = v[1]/d; v[2] = v[2]/d;
}

void normCrossProd(GLfloat v1[3], GLfloat v2[3], GLfloat out[3]) {
   out[0] = v1[1]*v2[2] - v1[2]*v2[1];
   out[1] = v1[2]*v2[0] - v1[0]*v2[2];
   out[2] = v1[0]*v2[1] - v1[1]*v2[0];
   normalize(out);
}


void reshapeCallback(int w, int h) {
   // from Angel, p.562
   glViewport(0,0,w,h);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   /*
   if (w < h) {
      glFrustum(-1.0, 1.0, -1.0*(GLfloat) h / (GLfloat) w,
                1.0*(GLfloat) h / (GLfloat) w, 1.0, 100.0);
   }
   else {
      glFrustum(-1.0, 1.0, -1.0*(GLfloat) w / (GLfloat) h,
                1.0*(GLfloat) w / (GLfloat) h, 1.0, 100.0);
   }
   */

   gluPerspective(45, ((double)w)/h, .01, 1000);

   glMatrixMode(GL_MODELVIEW);
}

void mouseMotionCallback(int x, int y) {
    int dx = lastx-x;
    int dy = lasty-y;
    double inverted = 1;
    double camspeed = .005;
    switch(clickLocation) {
        case MIDDLE:
            camera.rotateAboutRight(camspeed*inverted*dy);
            camera.rotateAboutUp(camspeed*dx);
            break;
        case TOP_RIGHT:
            camera.rotateAboutForward(-camspeed*dx-camspeed*dy);
            break;
        case BOTTOM_RIGHT:
            camera.rotateAboutForward(camspeed*dx-camspeed*dy);
            break;
        case BOTTOM_LEFT:
            camera.rotateAboutForward(camspeed*dx+camspeed*dy);
            break;
        case TOP_LEFT:
            camera.rotateAboutForward(-camspeed*dx+camspeed*dy);
            break;
    }
    lastx = x;
    lasty = y;
    glutPostRedisplay();
}


void mouseCallback(int button, int state, int x, int y) {
    lastx = x;
    lasty = y;

    if(state == GLUT_DOWN) {
        double ux = ((double)lastx/width-.5);
        double uy = ((double)lasty/height-.5);

        if((uy*uy+ux*ux < .5*.5)) {
            clickLocation = MIDDLE;
        } else {
            if(ux > 0 && uy > 0) {
                clickLocation = BOTTOM_RIGHT;
            } else if(ux > 0 && uy < 0) {
                clickLocation = TOP_RIGHT;
            } else if(ux < 0 && uy > 0) {
                clickLocation = BOTTOM_LEFT;
            } else if(ux < 0 && uy < 0) {
                clickLocation = TOP_LEFT;
            }
        }
    }
    /*
   // rotate camera
   GLint axis = 3;
   if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN) axis = 0;
   if (button == GLUT_MIDDLE_BUTTON && state == GLUT_DOWN) axis = 1;
   if (button == GLUT_RIGHT_BUTTON && state == GLUT_DOWN) axis = 2;
   if (axis < 3) {  // button ups won't change axis value from 3
      theta[axis] += thetaIncr;
      if (theta[axis] > 360.0) theta[axis] -= 360.0;
      //printf("thetaX: %f, thetaY: %f, Ztheta: %f\n", theta[0], theta[1], theta[2]);
       glutPostRedisplay();
   }
   */
}   

void keyUpCallback(unsigned char key, int x, int y) {
    switch(key) {
        case 'w':
            camera.endForward();
            break;
        case 's':
            camera.endBackward();
            break;
        case 'a':
            camera.endLeft();
            break;
        case 'd':
            camera.endRight();
            break;
        case 'r':
            camera.endUp();
            break;
        case 'f':
            camera.endDown();
            break;
    }
    glutPostRedisplay();
}

void timer(int v) {
    camera.update(.01);
    glutTimerFunc(20, timer, 0);
}

void keyCallback(unsigned char key, int x, int y) {
    bool cameraKey = false;
    switch(key) {
        case 'w':
            camera.endBackward();
            camera.beginForward();
            cameraKey = true;
            break;
        case 's':
            camera.endForward();
            camera.beginBackward();
            cameraKey = true;
            break;
        case 'a':
            camera.endRight();
            camera.beginLeft();
            cameraKey = true;
            break;
        case 'd':
            camera.endLeft();
            camera.beginRight();
            cameraKey = true;
            break;
        case 'r':
            camera.endDown();
            camera.beginUp();
            cameraKey = true;
            break;
        case 'f':
            camera.endUp();
            camera.beginDown();
            cameraKey = true;
            break;
    }
    if(cameraKey) {
        glutPostRedisplay();
        return;
    }
   // move viewer with x, y, and z keys
   // capital moves in + direction, lower-case - direction
   if (key == 'x') modelTrans[0] -= 0.25;
   if (key == 'X') modelTrans[0] += 0.25;
   if (key == 'y') modelTrans[1] -= 0.25;
   if (key == 'Y') modelTrans[1] += 0.25;
   if (key == 'z') modelTrans[2] -= 0.25;
   if (key == 'Z') modelTrans[2] += 0.25;

   if (key == '-') thetaIncr = -thetaIncr;

   if (key == '+') {
      if (thetaIncr < 0) thetaIncr = thetaIncr - 0.25;
      else               thetaIncr = thetaIncr + 0.25;
   }

   if (key == ']'){
	   if(mode == DISTANCE_MODE){
			if(distRings < 100.0) distRings++;
	   } else {
			if(energyRings < 100.0) energyRings++;
	   }
   }
   if (key == '['){
	   if(mode == DISTANCE_MODE){
			if(distRings > 0.0) distRings--;
	   } else {
			if(energyRings > 0.0) energyRings--;
	   }
   }

   if (key == '}'){
	   if(mode == DISTANCE_MODE) distInc = distInc + 100.0;
	   else energyInc = energyInc + 50000.0;
   }
   if (key == '{'){
	   if(mode == DISTANCE_MODE){
		  if(distInc > 0.0) distInc = distInc - 100.0;
	   } else {
		  if(energyInc > 0.0) energyInc = energyInc - 50000.0;
	   }
   }

   if (key == 'm'){
	   if(mode == ENERGY_MODE) mode = DISTANCE_MODE;
	   else mode = ENERGY_MODE;
   }

   if (key == 'p'){
	    int tempStartX;
		int tempStartY;
		int tempEndX;
		int tempEndY;
   		printf("\nPlease enter in start_x, start_y, end_x, end_y: ");
		scanf("%i %i %i %i", &tempStartX, &tempStartY, &tempEndX, &tempEndY);
		if(tempStartX < MATRIX_MAX && tempStartY < MATRIX_MAX && tempEndX < MATRIX_MAX && tempEndY < MATRIX_MAX &&
			tempStartX > 0 && tempStartY > 0 && tempEndX > 0 && tempEndY > 0){
			start_x = tempStartX;
			start_y = tempStartY;
			end_x = tempEndX;
			end_y = tempEndY;
			runDijkstra();
		} else {
			printf("Some of the input values exceed the range.\n");
		}
   }

   if (key == 'S') speed = speed + 5.0;
   if (key == 's'){
      if(speed > 0.0) speed = speed - 5.0;
   }

   if (key == 'P'){
	   printf("\nPrinting distance table...\n");
	   printData();
   }
   printf("\nUPDATED DATA\n");
   printf("Speed (mph):                   %f\n", speed);
   printf("Number of Distance Rings:      %f\n", distRings);
   printf("Number of Distance Increment:  %f\n", distInc);
   printf("Number of Energy Rings:        %f\n", energyRings);
   printf("Number of Energy Increment:    %f\n", energyInc);
   //printf("X: %f, Y: %f, Z: %f\n", modelTrans[0], modelTrans[1], modelTrans[2]);
   dirty = true;
   glutPostRedisplay();
}


// Camera interactive view
// Color terrain based on cost
// Picking the points
